/**
* @file CConferenceBoardDevice.cpp
* @brief Conference Board Device class implementations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "LogUtil.h"
#include "CConferenceBoardDevice.h"
#include "CConferenceBoardDeviceState.h"
#include "CConference.h"
#include "CGcManager.h"

//*****************************************************************************
// Function: CConferenceBoardDevice::CConferenceBoardDevice(const std::string &a_deviceName)
// Description: Initializing constructor
// Return:  
// Parameters: const std::string &a_deviceName 
//*****************************************************************************
CConferenceBoardDevice::CConferenceBoardDevice(const std::string& a_deviceName)
   :m_deviceState(AConferenceBoardDeviceState::DeviceState(ConferenceBoardDeviceClosedState))
   ,m_deviceName(a_deviceName)
{
}

//*****************************************************************************
// Function: void CConferenceBoardDevice::SetDeviceStateTo(AConferenceBoardDeviceState *state)
// Description: Set the device state
// Return: void 
// Parameters: AConferenceBoardDeviceState *state 
//*****************************************************************************
void CConferenceBoardDevice::SetDeviceStateTo(AConferenceBoardDeviceState *state)
{
   m_deviceState = state;
   LogMsg(DeviceName(), AsString("CNF Board Device State:") + m_deviceState->Desc());
}

//*****************************************************************************
// All device and call operations and event handling are delegated to the state
// object with the following functions
//*****************************************************************************

int CConferenceBoardDevice::Open() {return DeviceState().CnfOpenBoard(*this);}
int CConferenceBoardDevice::OnCnfOpenBoardComplete(METAEVENT& a_metaEvent) {return DeviceState().OnCnfOpenBoardComplete(*this, a_metaEvent);}
int CConferenceBoardDevice::CnfResetDevices() {return DeviceState().CnfResetDevices(*this);}
int CConferenceBoardDevice::OnCnfResetDevicesComplete(METAEVENT& a_metaEvent) {return DeviceState().OnCnfResetDevicesComplete(*this, a_metaEvent);}
int CConferenceBoardDevice::Close() {return DeviceState().CnfCloseBoard(*this);}

//*****************************************************************************
// Function: int CConferenceBoardDevice::PrimCnfOpenBoard()
// Description: Process a CNF conference board open command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConferenceBoardDevice::PrimCnfOpenBoard()
{
   SRL_DEVICE_HANDLE res = cnf_Open( m_deviceName.c_str(), NULL, (void *) this);
   if (res == CNF_ERROR)
   {
      LogMsg(DeviceName(), "ERROR cnf_Open failed for " + DeviceName());
   }
   else
   {
      Device(res);
      CGcManager::RegisterDevice(Device(), this);
      LogMsg(DeviceName(), "cnf_Open");
   }
   return res;
}

//*****************************************************************************
// Function: int CConferenceBoardDevice::PrimOnCnfOpenBoardComplete(METAEVENT &a_metaEvent)
// Description: Process a CNF conference board open completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferenceBoardDevice::PrimOnCnfOpenBoardComplete(METAEVENT& a_metaEvent)
{
   CNF_RETURN res(0); 

   CNF_ATTR l_attribute;
   memset(&l_attribute, 0, sizeof(CNF_ATTR));
   l_attribute.unAttribute = ECNF_BRD_ATTR_TONE_CLAMPING;
   if (CConference::SystemConfiguration()->ConferenceToneClamping())
   {
      l_attribute.unValue = ECNF_ATTR_STATE_ENABLED;
   }
   else
   {
      l_attribute.unValue = ECNF_ATTR_STATE_DISABLED;
   }

   CNF_ATTR_INFO l_attributeInfo;
   memset(&l_attributeInfo, 0, sizeof(CNF_ATTR_INFO));
   l_attributeInfo.unAttrCount = 1;
   l_attributeInfo.pAttrList = &l_attribute;

   res = cnf_SetAttributes(Device(), &l_attributeInfo, 0);
   if (res == CNF_ERROR)
   {
      ErrMsg(Device(), "cnf_SetAttributes");
      return res;
   }
   else
   {
      LogMsg(DeviceName(), "cnf_SetAttributes-ECNF_BRD_ATTR_TONE_CLAMPING:"+AsString(l_attribute.unValue));
   }

   CGcManager::AddOpenDevice(this);
   return res;
}

//*****************************************************************************
// Function: int CConferenceBoardDevice::OnSetAttributesComplete(METAEVENT &a_metaEvent)
// Description: Process a CNF set attributes completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferenceBoardDevice::OnSetAttributesComplete(METAEVENT& a_metaEvent)
{
   return 0;
}

//*****************************************************************************
// Function: int CConferenceBoardDevice::PrimCnfResetDevices()
// Description: Process a CNF conference board reset devices command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConferenceBoardDevice::PrimCnfResetDevices()
{
   LogMsg(DeviceName(), "PrimCnfResetDevices");
   SRL_DEVICE_HANDLE res = cnf_ResetDevices(Device(), NULL, (void *) this);
   if (res == CNF_ERROR)
   {
      ErrMsg(Device(), "cnf_ResetDevices");
   }
   else
   {
      LogMsg(Device(), "cnf_ResetDevices");
   }
   return res;
}

//*****************************************************************************
// Function: int CConferenceBoardDevice::PrimOnCnfResetDevicesComplete(METAEVENT &a_metaEvent)
// Description: Process a CNF conference board reset devices completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferenceBoardDevice::PrimOnCnfResetDevicesComplete(METAEVENT& a_metaEvent)
{
   CConference::OnConferenceBoardDeviceOpenComplete();
   return 0;
}

//*****************************************************************************
// Function: int CConferenceBoardDevice::PrimCnfCloseBoard()
// Description: Process a CNF conference board close command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConferenceBoardDevice::PrimCnfCloseBoard()
{
   CNF_RETURN res;
   if (Device() != -1)
   {
      res = cnf_Close(Device(), 0);
      if (res == CNF_ERROR)
      {
         ErrMsg(Device(), DeviceName() + " cnf_Close");
      }
      else
      {
         LogMsg(DeviceName(), "cnf_Close");
         CGcManager::RemoveOpenDevice(this);      
      }
      Device(-1);
   }
   else
   {
      LogMsg(DeviceName(), "device already closed");
   }
   return res;
}

//*****************************************************************************
// Function: void CConferenceBoardDevice::ProcessEvent(METAEVENT &a_metaEvent)
// Description: Process an event
// Return: void 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
void CConferenceBoardDevice::ProcessEvent(METAEVENT& a_metaEvent)
{
   switch (a_metaEvent.evttype)
   {
      case 0x00:
         break;

      case CNFEV_OPEN:
         LogMsg(Device(), "CNFEV_OPEN cnfDevice=" + AsString(":") + AsString(Device()));
         OnCnfOpenBoardComplete(a_metaEvent);
         break;

      case CNFEV_OPEN_FAIL:
         LogMsg(DeviceName(), "CNFEV_OPEN_FAIL");
         break;

      case CNFEV_RESET_DEVICES:
         LogMsg(DeviceName(), "CNFEV_RESET_DEVICES");
         OnCnfResetDevicesComplete(a_metaEvent);
         break;

      case CNFEV_GET_DEVICE_COUNT:
         LogMsg(Device(), "CNFEV_GET_DEVICE_COUNT");
         break;

      case CNFEV_GET_DEVICE_COUNT_FAIL:
         LogMsg(Device(), "CNFEV_GET_DEVICE_COUNT_FAIL");
         break;

      case CNFEV_SET_ATTRIBUTE:
         LogMsg(Device(), "CNFEV_SET_ATTRIBUTE");
         OnSetAttributesComplete(a_metaEvent);
         break;

      case CNFEV_ERROR:
         ErrMsg(Device(), "CNFEV_ERROR");
         break;

      case APPEVENT_CLOSE_COMMAND:
         LogMsg(Device(), "APPEVENT_CLOSE_COMMAND");
         Close();
         break;

      default:
         LogMsg(Device(), "unhandled event:" + AsHexString(a_metaEvent.evttype));
   }
}

