/**
* @file CConferenceBoardDeviceState.cpp
* @brief Conference Board Device States - class implementations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "CConferenceBoardDeviceState.h"
#include "LogUtil.h"

CConferenceBoardDeviceStates AConferenceBoardDeviceState::m_deviceStates;

//*****************************************************************************
// Function: int AConferenceBoardDeviceState::LogStateError(CConferenceBoardDevice &a_device, const string &msg)
// Description: Log an invalid state for operation/event message
// Return: int - 0 for success, otherwise error 
// Parameters: CConferenceBoardDevice &a_device 
//             const string &msg 
//*****************************************************************************
int AConferenceBoardDeviceState::LogStateError(CConferenceBoardDevice& a_device, const string& msg)
{
   std::cout << a_device.DeviceName()
   << " " << Desc() << " is invalid device state for "
   << msg
   << std::endl;
   return -1;
}

//*****************************************************************************
// Function: int AConferenceBoardDeviceState::CnfCloseBoard(CConferenceBoardDevice &a_device)
// Description: Close a Conference Board device
// Return: int - 0 for success, otherwise error 
// Parameters: CConferenceBoardDevice &a_device 
//*****************************************************************************
int AConferenceBoardDeviceState::CnfCloseBoard(CConferenceBoardDevice& a_device) 
{
   int res = a_device.PrimCnfCloseBoard();
   a_device.SetDeviceStateTo(DeviceState(ConferenceBoardDeviceClosedState)); 
   return res;
}

//*****************************************************************************
// Function: CConferenceBoardDeviceStates::CConferenceBoardDeviceStates()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CConferenceBoardDeviceStates::CConferenceBoardDeviceStates()
{
   m_deviceStates[ConferenceBoardDeviceClosedState] = new CConferenceBoardDeviceClosedState();
   m_deviceStates[ConferenceBoardDeviceOpeningState] = new CConferenceBoardDeviceOpeningState();
   m_deviceStates[ConferenceBoardDeviceOpenedState] = new CConferenceBoardDeviceOpenedState();
}

//*****************************************************************************
// Function: int CConferenceBoardDeviceClosedState::CnfOpenBoard(CConferenceBoardDevice &a_device)
// Description: Handle Conference board open command
// Return: int - 0 for success, otherwise error 
// Parameters: CConferenceBoardDevice &a_device 
//*****************************************************************************
int CConferenceBoardDeviceClosedState::CnfOpenBoard(CConferenceBoardDevice& a_device) 
{
   a_device.SetDeviceStateTo(DeviceState(ConferenceBoardDeviceOpeningState));
   return a_device.PrimCnfOpenBoard();
}

//*****************************************************************************
// Function: int CConferenceBoardDeviceOpeningState::OnCnfOpenBoardComplete(CConferenceBoardDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle open conference board completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CConferenceBoardDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferenceBoardDeviceOpeningState::OnCnfOpenBoardComplete(CConferenceBoardDevice& a_device, METAEVENT& a_metaEvent) 
{
   int res = a_device.PrimOnCnfOpenBoardComplete(a_metaEvent);
   if (res == 0)
   {
      a_device.CnfResetDevices();
   }
   return res;
}

//*****************************************************************************
// Function: int CConferenceBoardDeviceOpeningState::CnfResetDevices(CConferenceBoardDevice &a_device)
// Description: Reset conference board devices command
// Return: int - 0 for success, otherwise error 
// Parameters: CConferenceBoardDevice &a_device 
//*****************************************************************************
int CConferenceBoardDeviceOpeningState::CnfResetDevices(CConferenceBoardDevice& a_device) 
{
   return a_device.PrimCnfResetDevices();
}

//*****************************************************************************
// Function: int CConferenceBoardDeviceOpeningState::OnCnfResetDevicesComplete(CConferenceBoardDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle reset conference board devices completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CConferenceBoardDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferenceBoardDeviceOpeningState::OnCnfResetDevicesComplete(CConferenceBoardDevice& a_device, METAEVENT& a_metaEvent) 
{
   int res = a_device.PrimOnCnfResetDevicesComplete(a_metaEvent);
   if (res != -1)
   {
      a_device.SetDeviceStateTo(DeviceState(ConferenceBoardDeviceOpenedState));
   }
   return res;
}


