/**
* @file CConferenceBoardDeviceState.h
* @brief Conference Board Device States - class declarations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CConferenceBoardDeviceState_h
#define CConferenceBoardDeviceState_h

#include <map>
#include <string>

#include "CConferenceBoardDevice.h"

typedef enum 
{
   ConferenceBoardDeviceClosedState = 0,
   ConferenceBoardDeviceOpeningState,
   ConferenceBoardDeviceOpenedState,
} ConferenceBoardDeviceState;

class AConferenceBoardDeviceState;    // forward decleration

//*****************************************************************************
// Class: ::CConferenceBoardDeviceStates
// Description: Map of conference board device state objects
//*****************************************************************************
class CConferenceBoardDeviceStates
{
public:
   CConferenceBoardDeviceStates();
   ~CConferenceBoardDeviceStates() {}
   AConferenceBoardDeviceState* DeviceState(ConferenceBoardDeviceState deviceState) {return m_deviceStates[ deviceState];}
private:
   std::map< ConferenceBoardDeviceState, AConferenceBoardDeviceState*> m_deviceStates;
};

//*****************************************************************************
// Class: ::AConferenceBoardDeviceState
// Description: Abstract CNF board device state
// Notes:
//    This abstract class declares all functions for all device states as 
//    virtual and implements them as an error.
// 
//    Concrete subclasses implement only the functions that are valid for that
//    state.
// 
//    State classes are responsible only for calling primative functions on the 
//    context device object and changing the state of the context device object.
// 
//    As state classes have no non-static member variables, one set of device 
//    state objects are shared by all devices.
//*****************************************************************************
class AConferenceBoardDeviceState
{
public:
   virtual const char* Desc() = 0;

   virtual int CnfOpenBoard(CConferenceBoardDevice& a_device) {return LogStateError(a_device, "CnfOpenBoard");}
   virtual int OnCnfOpenBoardComplete(CConferenceBoardDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnCnfOpenBoardComplete");}
   virtual int CnfResetDevices(CConferenceBoardDevice& a_device) {return LogStateError(a_device, "CnfResetDevices");}
   virtual int OnCnfResetDevicesComplete(CConferenceBoardDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnCnfResetDevicesComplete");}
   virtual int CnfCloseBoard(CConferenceBoardDevice& a_device); 
   static AConferenceBoardDeviceState* DeviceState(ConferenceBoardDeviceState deviceState) {return m_deviceStates.DeviceState(deviceState);}

private:
   int LogStateError(CConferenceBoardDevice& a_device, const std::string& msg);
   static CConferenceBoardDeviceStates m_deviceStates;
};

//*****************************************************************************
// Class: ::CConferenceBoardDeviceClosedState
// Description: CNF board device closed state
//*****************************************************************************
class CConferenceBoardDeviceClosedState : public AConferenceBoardDeviceState
{
public:
   const char* Desc() {return "closed";}
   int CnfOpenBoard(CConferenceBoardDevice& a_device);
};

//*****************************************************************************
// Class: ::CConferenceBoardDeviceOpeningState
// Description: CNF board device opening state
//*****************************************************************************
class CConferenceBoardDeviceOpeningState : public AConferenceBoardDeviceState
{
public:
   const char* Desc() {return "opening";}
   int OnCnfOpenBoardComplete(CConferenceBoardDevice& a_device, METAEVENT& a_metaEvent);
   int CnfResetDevices(CConferenceBoardDevice& a_device);
   int OnCnfResetDevicesComplete(CConferenceBoardDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CConferenceBoardDeviceOpenedState
// Description: CNF board device opened and available state
//*****************************************************************************
class CConferenceBoardDeviceOpenedState : public AConferenceBoardDeviceState
{
public:
   const char* Desc() {return "opened";}
};

#endif

