/**
* @file CConferenceDevice.cpp
* @brief Conference Device class implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "lb_mtklib.h"

#include "config.h"

#include "LogUtil.h"
#include "CConferenceDevice.h"
#include "CConferenceDeviceState.h"
#include "CConferenceBoardDevice.h"
#include "CConferencePartyDevice.h"
#include "CConference.h"
#include "CGcManager.h"
#include "CMediaDevice.h"
#include "CVirtualCaller.h"

//*****************************************************************************
// Function: CConferenceDevice::CConferenceDevice(const std::string &a_deviceName, CConferenceBoardDevice *a_cnfBoardDevice)
// Description: Default constructor
// Return:  
// Parameters: const std::string &a_deviceName 
//             CConferenceBoardDevice *a_cnfBoardDevice 
//*****************************************************************************
CConferenceDevice::CConferenceDevice(const std::string& a_deviceName, CConferenceBoardDevice* a_cnfBoardDevice)
   :m_deviceState(AConferenceDeviceState::DeviceState(ConferenceDeviceClosedState))
   ,m_deviceName(a_deviceName)
	,m_confID(a_deviceName)
   ,m_cnfBoardDevice(a_cnfBoardDevice)
   ,m_observer(0)
   ,m_layoutTileCount(0)
   ,m_featureState1(0)
   ,m_featureState2(0)
{
}

//*****************************************************************************
// All device and call operations and event handling are delegated to the state
// object with the following functions
//*****************************************************************************
 
int CConferenceDevice::Open() {return DeviceState().CnfOpenConference(*this);}
int CConferenceDevice::Close() {return DeviceState().CnfCloseConference(*this);}
int CConferenceDevice::OnCnfOpenConferenceComplete(METAEVENT& a_metaEvent) {return DeviceState().OnCnfOpenConferenceComplete(*this, a_metaEvent);}

//*****************************************************************************
// Function: void CConferenceDevice::SetDeviceStateTo(AConferenceDeviceState *state)
// Description: Set the device state
// Return: void 
// Parameters: AConferenceDeviceState *state 
//*****************************************************************************
void CConferenceDevice::SetDeviceStateTo(AConferenceDeviceState *state)
{
   m_deviceState = state;
   LogMsg(DeviceName(), AsString("CNF Conf Device State:") + m_deviceState->Desc());
}

//*****************************************************************************
// Function: int CConferenceDevice::PrimCnfOpenConference()
// Description: Process the CNF open conference command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConferenceDevice::PrimCnfOpenConference()
{
   SRL_DEVICE_HANDLE res = cnf_OpenConference( m_cnfBoardDevice->Device(), 0, 0, (void *) this);
   if (res == CNF_ERROR)
   {
      ErrMsg(0, DeviceName() + " cnf_OpenConference");
   }
   else
   {
      Device(res);
      CGcManager::RegisterDevice(Device(), this);
      LogMsg(DeviceName(), "cnf_OpenConference");
   }

   return res;
}

//*****************************************************************************
// Function: int CConferenceDevice::PrimCnfCloseConference()
// Description: Process the CNF close conference command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConferenceDevice::PrimCnfCloseConference()
{
   CNF_RETURN res(0);
   if (Device() != -1)
   {
      res = cnf_CloseConference(Device(), 0);
      if (res == CNF_ERROR)
      {
         ErrMsg(Device(), DeviceName() + " cnf_CloseConference");
      }
      else
      {
         LogMsg(DeviceName(), "cnf_CloseConference");
         CGcManager::RemoveOpenDevice(this);
      }
      Device(-1);
   }
   else
   {
      LogMsg(DeviceName(), "device already closed");
   }
   return res;
}

//*****************************************************************************
// Function: int CConferenceDevice::PrimOnCnfOpenConferenceComplete(METAEVENT &a_metaEvent)
// Description: Process the CNF open conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferenceDevice::PrimOnCnfOpenConferenceComplete(METAEVENT& a_metaEvent)
{
   CNF_RETURN res;
   CNF_ATTR l_attribute;

   memset(&l_attribute, 0, sizeof(CNF_ATTR));
   l_attribute.unAttribute = ECNF_CONF_ATTR_TONE_CLAMPING;
   if (CConference::SystemConfiguration()->ConferenceToneClamping())
   {
      l_attribute.unValue = ECNF_ATTR_STATE_ENABLED;
   }
   else
   {
      l_attribute.unValue = ECNF_ATTR_STATE_DISABLED;
   }

   CNF_ATTR_INFO l_attributeInfo;
   memset(&l_attributeInfo, 0, sizeof(CNF_ATTR_INFO));
   l_attributeInfo.unAttrCount = 1;
   l_attributeInfo.pAttrList = &l_attribute;

   res = cnf_SetAttributes(Device(), &l_attributeInfo, 0);
   if (res == CNF_ERROR)
   {
      ErrMsg(Device(), "cnf_SetAttributes");
      return res;
   }
   else
   {
      LogMsg(DeviceName(), "cnf_SetAttributes-ECNF_CONF_ATTR_TONE_CLAMPING:"+AsString(l_attribute.unValue));
   }

#if 0
// SKS Enable event for RJK
// SKS looks like some issue here after enabling events

  char str[128];
  unsigned int CnfEventList[10];
  CnfEventList[0] = ECNF_CONF_EVT_PARTY_ADDED;
  CnfEventList[1] = ECNF_CONF_EVT_PARTY_REMOVED;
  CnfEventList[2] = ECNF_CONF_EVT_DTMF_DETECTION;
  CnfEventList[3] = ECNF_CONF_EVT_ACTIVE_TALKER;
  CNF_EVENT_INFO CnfEventInfo;
  CnfEventInfo.unEventCount = 4;
  CnfEventInfo.punEventList = CnfEventList;
  if (cnf_EnableEvents(Device(), &CnfEventInfo, NULL) == CNF_ERROR) {
      sprintf(str,"cnf_EnableEvents(cnfH) failed");
  } else {
      sprintf(str,"cnf_EnableEvents(cnfH) Success");
  }
  LogMsg(DeviceName(),str);
#endif

   CGcManager::AddOpenDevice(this);

   res = SetConferenceLayout(CConference::InitialLayout());
   if (res == 0)
   {
      return CConference::OnConferenceDeviceOpenComplete();
   }
   return res;
}

//*****************************************************************************
// Function: int CConferenceDevice::SetConferenceLayout(int a_tiles)
// Description: Set the conference video layput
// Return: int - 0 for success, otherwise error 
// Parameters: int a_tiles 
//*****************************************************************************
int CConferenceDevice::SetConferenceLayout(int a_tiles)
{
   m_layoutTileCount = a_tiles;
   m_layoutHandle = 0;

   eLB_LAYOUT_TYPE l_layout(eLB_LAYOUT_TYPE_4_1);
   switch (a_tiles)
   {
      case 1:
         l_layout = eLB_LAYOUT_TYPE_1_1;
         break;
      case 2:
         l_layout = eLB_LAYOUT_TYPE_CUSTOM;
         break;
      case 4:
         l_layout = eLB_LAYOUT_TYPE_4_1;
         break;
      case 6:
         l_layout = eLB_LAYOUT_TYPE_6_1;
         break;
      case 9:
         l_layout = eLB_LAYOUT_TYPE_9_1;
         break;
      default:
         LogMsg(Device(), "error: invalid layout:"+AsString(a_tiles));
         return -1;
   }

   LB_FRAME_HANDLE l_layoutTemplateHandle = lb_CreateLayoutTemplate(l_layout);
   if (l_layoutTemplateHandle == CNF_ERROR)
   {
      ErrMsg(Device(), "lb_CreateLayoutTemplate");
      return -1;
   }

   if (l_layout == eLB_LAYOUT_TYPE_CUSTOM)
   {
      MTK_RECT l_regionRect;
      INIT_MTK_RECT(&l_regionRect);

      l_regionRect.x = 0;
      l_regionRect.y = 25;
      l_regionRect.height = 50.00;
      l_regionRect.width = 50.00;

      LB_FRAME_HANDLE l_regionHandle = lb_AddRegion(l_layoutTemplateHandle, &l_regionRect);
      if (l_regionHandle == MTK_ERROR)
      {
         ErrMsg(Device(), "lb_AddRegion - 1");
         return -1;
      }

      l_regionRect.x = 50.00;
      l_regionRect.y = 25.00;
      l_regionRect.height = 50.00;
      l_regionRect.width = 50.00;

      l_regionHandle = lb_AddRegion(l_layoutTemplateHandle, &l_regionRect);
      if (l_regionHandle == MTK_ERROR)
      {
         ErrMsg(Device(), "lb_AddRegion  - 2");
         return -1;
      }
   }

   CNF_RETURN res(CNF_ERROR);

   res = SetLayout(l_layoutTemplateHandle);

   if ( lb_DestroyLayoutTemplate(l_layoutTemplateHandle) == MTK_ERROR)
   {
      ErrMsg(Device(), "lb_DestroyLayoutTemplate");
   }
   return res;
}

//*****************************************************************************
// Function: int CConferenceDevice::OnCnfSetVideoLayoutComplete(METAEVENT &a_metaEvent)
// Description: Handle a set conference video layout completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferenceDevice::OnCnfSetVideoLayoutComplete(METAEVENT& a_metaEvent)
{
   if (!m_layoutHandle)
   {
      PCNF_VIDEO_LAYOUT_INFO l_layoutInfo = (PCNF_VIDEO_LAYOUT_INFO) a_metaEvent.evtdatap;
      m_layoutHandle = l_layoutInfo->LayoutHandle;
   
      m_regionListCount = MAX_REGION_COUNT;
      int res = lb_GetRegionList(m_layoutHandle, m_regionList, &m_regionListCount);
      if (res == MTK_ERROR)
      {
         ErrMsg(Device(), "lb_GetRegionList");
      }
      else
      {
         LogMsg(DeviceName(),"lb_GetRegionList:"+AsString(m_regionListCount));
         m_observer->PostEvent(APPEVENT_CNF_SET_VIDEO_LAYOUT_COMPLETE);
      }
      return res;
   }
   return 0;
}

//*****************************************************************************
// Function: int CConferenceDevice::AddConferenceParty(CConferencePartyDevice *a_conferenceParty)
// Description: Add a party to the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice *a_conferenceParty 
//*****************************************************************************
int CConferenceDevice::AddConferenceParty( CConferencePartyDevice* a_conferenceParty)
{
	 LogMsg("SKS::CONF_DEV", "CConferenceDevice::AddConferenceParty()");	
   long l_party = a_conferenceParty->Device();
   CNF_PARTY_INFO l_partyInfo;
   l_partyInfo.unPartyCount = 1;
   l_partyInfo.pPartyList = &l_party;

   CNF_RETURN res = cnf_AddParty(Device(), &l_partyInfo, a_conferenceParty);
   if (res == CNF_ERROR)
   {
      ErrMsg(Device(), a_conferenceParty->DeviceName() + " cnf_AddParty");
   }
   else
   {
      LogMsg(a_conferenceParty->DeviceName(), "cnf_AddParty");
   }
   return res;
}

//*****************************************************************************
// Function: int CConferenceDevice::SetVisibleParties(CNF_VISIBLE_PARTY_LIST &a_visiblePartyList)
// Description: Set the parties visible in the conference output
// Return: int - 0 for success, otherwise error 
// Parameters: CNF_VISIBLE_PARTY_LIST &a_visiblePartyList 
//*****************************************************************************
int CConferenceDevice::SetVisibleParties(CNF_VISIBLE_PARTY_LIST& a_visiblePartyList)
{
   int res(0);

   for (int i=0; i<a_visiblePartyList.unCount; i++)
   {
      LogMsg(DeviceName(), "SetVisibleParties region: " + AsString(a_visiblePartyList.pPartyList[i].RegionHandle) 
                            + " party: " + AsString (a_visiblePartyList.pPartyList[i].PartyHandle));
      
      eLB_DISPLAY_MODE l_displayMode(eLB_DISPLAY_MODE_LIVE);
 
      if (a_visiblePartyList.pPartyList[i].PartyHandle == -1)
      {
         l_displayMode = eLB_DISPLAY_MODE_BLANK;
      }

      res = lb_SetDisplayMode(a_visiblePartyList.pPartyList[i].RegionHandle, l_displayMode);
      if (res == MTK_ERROR)
      {
         ErrMsg(Device(), "lb_SetDisplayMode");
         break;
      }
      else
      {
         LogMsg(DeviceName(),"lb_SetDisplayMode:"
                              +AsHexString(a_visiblePartyList.pPartyList[i].RegionHandle)
                              +"-"+AsHexString(l_displayMode));
      }
   }
/*
   res = cnf_SetVideoLayout(Device(), &m_layoutInfo, 0);
   if (res == CNF_ERROR)
   {
      ErrMsg(Device(), "cnf_SetVideoLayout");
   }
   else
   {
      LogMsg(Device(), "cnf_SetVideoLayout");
   }
*/
   if (a_visiblePartyList.unCount)
   {
      res = cnf_SetVisiblePartyList(Device(), &a_visiblePartyList, 0);
      if (res == CNF_ERROR)
      {
         ErrMsg(Device(),"cnf_SetVisiblePartyList");
      }
      else
      {
         LogMsg(Device(),"cnf_SetVisiblePartyList:"+AsString(a_visiblePartyList.unCount));
      }
   }

   return res;
}

//*****************************************************************************
// Function: int CConferenceDevice::OnConferencePartyAdded(CConferencePartyDevice *a_conferenceParty)
// Description: Handle an add conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice *a_conferenceParty 
//*****************************************************************************
int CConferenceDevice::OnConferencePartyAdded( CConferencePartyDevice* a_conferenceParty)
{
   return 0;
}

//*****************************************************************************
// Function: int CConferenceDevice::RemoveConferenceParty(CConferencePartyDevice *a_conferenceParty)
// Description: Remove a party from the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice *a_conferenceParty 
//*****************************************************************************
int CConferenceDevice::RemoveConferenceParty( CConferencePartyDevice* a_conferenceParty)
{
   long l_party = a_conferenceParty->Device();
   CNF_PARTY_INFO l_partyInfo;
   l_partyInfo.unPartyCount = 1;
   l_partyInfo.pPartyList = &l_party;

   CNF_RETURN res = cnf_RemoveParty(Device(), &l_partyInfo, a_conferenceParty);
   if (res == CNF_ERROR)
   {
      ErrMsg(Device(), a_conferenceParty->DeviceName() + " cnf_RemoveParty");
   }
   else
   {
      LogMsg(a_conferenceParty->DeviceName(), "cnf_RemoveParty");
   }
   return res;
}

//*****************************************************************************
// Function: int CConferenceDevice::OnConferencePartyRemoved(CConferencePartyDevice *a_conferenceParty)
// Description: Handle a remove conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice *a_conferenceParty 
//*****************************************************************************
int CConferenceDevice::OnConferencePartyRemoved( CConferencePartyDevice* a_conferenceParty)
{
   return 0;
}

int CConferenceDevice::OnCnfSetVisiblePartyList(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_CNF_SET_VISIBLE_PARTY_LIST_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: void CConferenceDevice::ProcessEvent(METAEVENT &a_metaEvent)
// Description: Process an event
// Return: void 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
void CConferenceDevice::ProcessEvent(METAEVENT& a_metaEvent)
{
   switch (a_metaEvent.evttype)
   {
      case CNFEV_OPEN_CONF:
         LogMsg(Device(), "CNFEV_OPEN_CONF cnfDevice=" + AsString(":") + AsString(Device()));
//			SetDeviceName(ATDV_NAMEP((LINEDEV) Device()));
			m_deviceName=ATDV_NAMEP((LINEDEV) Device());
         OnCnfOpenConferenceComplete(a_metaEvent);
         break;

      case CNFEV_OPEN_CONF_FAIL:
         LogMsg(DeviceName(), "CNFEV_OPEN_CONF_FAIL");
         break;

      case CNFEV_SET_ATTRIBUTE:
         LogMsg(DeviceName(), "CNFEV_SET_ATTRIBUTE");
         break;

      case CNFEV_SET_ATTRIBUTE_FAIL:
         LogMsg(DeviceName(), "CNFEV_SET_ATTRIBUTE_FAIL");
         break;

      case CNFEV_SET_VIDEO_LAYOUT:
         LogMsg(DeviceName(), "CNFEV_SET_VIDEO_LAYOUT");
         OnCnfSetVideoLayoutComplete(a_metaEvent);
         break;

      case CNFEV_SET_VIDEO_LAYOUT_FAIL:
         LogMsg(DeviceName(), "CNFEV_SET_VIDEO_LAYOUT_FAIL");
         break;

      case CNFEV_SET_VISIBLE_PARTY_LIST:
         LogMsg(DeviceName(), "CNFEV_SET_VISIBLE_PARTY_LIST");
         OnCnfSetVisiblePartyList(a_metaEvent);
         break;

      case CNFEV_SET_VISIBLE_PARTY_LIST_FAIL:
         LogMsg(DeviceName(), "CNFEV_SET_VISIBLE_PARTY_LIST_FAIL");
         break;

      case CNFEV_ERROR:
         ErrMsg(Device(), "CNFEV_ERROR");
         break;

      case APPEVENT_CLOSE_COMMAND:
         LogMsg(Device(), "APPEVENT_CLOSE_COMMAND");
         Close();
         break;

      default:
         LogMsg(Device(), "unhandled event:" + AsHexString(a_metaEvent.evttype));
   }
}

//*****************************************************************************
// Function: int CConferenceDevice::SetPipDemoFeature1Layout()
// Description: Update layout for PIP demo feature #1
// Return: int 
// Parameters: 
//*****************************************************************************
int CConferenceDevice::SetPipDemoFeature1Layout()
{
   m_layoutHandle = 0;

   LB_FRAME_HANDLE l_layoutTemplateHandle = lb_CreateLayoutTemplate(eLB_LAYOUT_TYPE_CUSTOM);
   if (l_layoutTemplateHandle == CNF_ERROR)
   {
      ErrMsg(Device(), "lb_CreateLayoutTemplate");
   }
   LogMsg(Device(), "lb_CreateLayoutTemplate");

   MTK_RECT l_regionRect;
   INIT_MTK_RECT(&l_regionRect);

   l_regionRect.x = 0;
   l_regionRect.y = 0;
   l_regionRect.height = 100.00;
   l_regionRect.width = 100.00;

   LB_FRAME_HANDLE l_regionHandle = lb_AddRegion(l_layoutTemplateHandle, &l_regionRect);
   if (l_regionHandle == MTK_ERROR)
   {
      ErrMsg(Device(), "lb_AddRegion - 1");
      return -1;
   }

   float l_loc;
   float l_dim;
   if ((m_featureState1 % 16) < 8)
   {
      l_loc = (m_featureState1 % 16) * 3.125;
   }
   else
   {
      l_loc = ((m_featureState1 % 16) - 2*(m_featureState1 % 8)) * 3.125;
   }
   l_dim = l_loc * 2;

   switch(m_featureState1 / 16)
   {
      case 0:
         l_regionRect.x = 25.00-l_loc;
         l_regionRect.y = 25.00-l_loc;
         break;

      case 1:
         l_regionRect.x = 75.00-l_loc;
         l_regionRect.y = 25.00-l_loc;
         break;

      case 2:
         l_regionRect.x = 75.00-l_loc;
         l_regionRect.y = 75.00-l_loc;
         break;

      default:
         l_regionRect.x = 25.00-l_loc;
         l_regionRect.y = 75.00-l_loc;
         break;
   }
   l_regionRect.height = l_dim;
   l_regionRect.width = l_dim;

   l_regionHandle = lb_AddRegion(l_layoutTemplateHandle, &l_regionRect);
   if (l_regionHandle == MTK_ERROR)
   {
      ErrMsg(Device(), "lb_AddRegion  - 2");
      return -1;
   }

   CNF_RETURN res(CNF_ERROR);
   res = SetLayout(l_layoutTemplateHandle);
   if ( lb_DestroyLayoutTemplate(l_layoutTemplateHandle) == MTK_ERROR)
   {
      ErrMsg(Device(), "lb_DestroyLayoutTemplate");
   }

   m_featureState1 = ++m_featureState1 % 64;
   return res;
}

//*****************************************************************************
// Function: int CConferenceDevice::SetPipDemoFeature2Layout()
// Description: Update layout for PIP demo feature #2
// Return: int 
// Parameters: none 
//*****************************************************************************
int CConferenceDevice::SetPipDemoFeature2Layout()
{
   m_layoutHandle = 0;

   LB_FRAME_HANDLE l_layoutTemplateHandle = lb_CreateLayoutTemplate(eLB_LAYOUT_TYPE_CUSTOM);
   if (l_layoutTemplateHandle == CNF_ERROR)
   {
      ErrMsg(Device(), "lb_CreateLayoutTemplate");
   }

   MTK_RECT l_regionRect;
   INIT_MTK_RECT(&l_regionRect);

   l_regionRect.x = 0;
   l_regionRect.y = 0;
   l_regionRect.height = 100.00;
   l_regionRect.width = 100.00;

   LB_FRAME_HANDLE l_regionHandle = lb_AddRegion(l_layoutTemplateHandle, &l_regionRect);
   if (l_regionHandle == MTK_ERROR)
   {
      ErrMsg(Device(), "lb_AddRegion - 1");
      return -1;
   }
   unsigned char l_pipMap(0x00);
   char l_pipActionMap(0x00);
   switch(m_featureState1)
   {
      case 0:
         l_pipMap = 0x00;
         l_pipActionMap = 0x01;
         break;
      case 1:
         l_pipMap = 0x01;
         l_pipActionMap = 0x02;
         break;
      case 2:
         l_pipMap = 0x03;
         l_pipActionMap = 0x04;
         break;
      case 3:
         l_pipMap = 0x07;
         l_pipActionMap = 0x08;
         break;
      case 4:
         l_pipMap = 0x07;
         l_pipActionMap = 0x08;
         break;
      case 5:
         l_pipMap = 0x03;
         l_pipActionMap = 0x04;
         break;
      case 6:
         l_pipMap = 0x01;
         l_pipActionMap = 0x02;
         break;
      default:
         l_pipMap = 0x00;
         l_pipActionMap = 0x01;
         break;
   }

   float l_loc;
   float l_dim;
   if (m_featureState1 < 4)
   {
      l_loc = m_featureState2 * 3.125;
   }
   else
   {
      l_loc = 25 - (m_featureState2 * 3.125);
   }
   l_dim = l_loc * 2;

   for (int i=0; i<4; i++)
   {
      switch(l_pipActionMap & (1 << i))
      {
         case 0x01:
            l_regionRect.x = 25.00-l_loc;
            l_regionRect.y = 25.00-l_loc;
            l_regionRect.height = l_dim;
            l_regionRect.width = l_dim;

            break;

         case 0x02:
            l_regionRect.x = 75.00-l_loc;
            l_regionRect.y = 25.00-l_loc;
            l_regionRect.height = l_dim;
            l_regionRect.width = l_dim;

            break;

         case 0x04:
            l_regionRect.x = 75.00-l_loc;
            l_regionRect.y = 75.00-l_loc;
            l_regionRect.height = l_dim;
            l_regionRect.width = l_dim;
            break;

         case 0x08:
            l_regionRect.x = 25.00-l_loc;
            l_regionRect.y = 75.00-l_loc;
            l_regionRect.height = l_dim;
            l_regionRect.width = l_dim;
            break;
      }

      switch(l_pipMap & (1 << i))
      {
         case 0x01:
            l_regionRect.x = 0;
            l_regionRect.y = 0;
            l_regionRect.height = 50;
            l_regionRect.width = 50;
            break;

         case 0x02:
            l_regionRect.x = 50;
            l_regionRect.y = 0;
            l_regionRect.height = 50;
            l_regionRect.width = 50;
            break;

         case 0x04:
            l_regionRect.x = 50;
            l_regionRect.y = 50;
            l_regionRect.height = 50;
            l_regionRect.width = 50;
            break;

         case 0x08:
            l_regionRect.x = 0;
            l_regionRect.y = 50;
            l_regionRect.height = 50;
            l_regionRect.width = 50;
            break;
      }

      l_regionHandle = lb_AddRegion(l_layoutTemplateHandle, &l_regionRect);
      if (l_regionHandle == MTK_ERROR)
      {
         ErrMsg(Device(), "lb_AddRegion  - 2");
         return -1;
      }
   }

   CNF_RETURN res(CNF_ERROR);
   res = SetLayout(l_layoutTemplateHandle);
   if ( lb_DestroyLayoutTemplate(l_layoutTemplateHandle) == MTK_ERROR)
   {
      ErrMsg(Device(), "lb_DestroyLayoutTemplate");
   }

   m_featureState2++;
   if ( (m_featureState2 / 8) && !(m_featureState2 % 8) )
   {
      m_featureState1++;
   }

   m_featureState1 = m_featureState1 % 8;
   m_featureState2 = m_featureState2 % 8;

   return res;
}

//*****************************************************************************
// Function: int CConferenceDevice::SetPipDemoFeature3Layout()
// Description: Update layout for PIP demo feature #2
// Return: int 
// Parameters: none 
//*****************************************************************************
int CConferenceDevice::SetPipDemoFeature3Layout()
{
   m_layoutHandle = 0;

   LB_FRAME_HANDLE l_layoutTemplateHandle = lb_CreateLayoutTemplate(eLB_LAYOUT_TYPE_CUSTOM);
   if (l_layoutTemplateHandle == CNF_ERROR)
   {
      ErrMsg(Device(), "lb_CreateLayoutTemplate");
   }
   LogMsg(Device(), "lb_CreateLayoutTemplate");

   MTK_RECT l_regionRect;
   INIT_MTK_RECT(&l_regionRect);

   l_regionRect.x = 0;
   l_regionRect.y = 0;
   l_regionRect.height = 100.00;
   l_regionRect.width = 100.00;

   LB_FRAME_HANDLE l_regionHandle = lb_AddRegion(l_layoutTemplateHandle, &l_regionRect);
   if (l_regionHandle == MTK_ERROR)
   {
      ErrMsg(Device(), "lb_AddRegion - 1");
      return -1;
   }

// l_regionRect.height = l_regionRect.width = 10 + (80 * (rand() / RAND_MAX + 1.0));

   l_regionRect.height = l_regionRect.width = 10 + (80 * (rand() / ((float)RAND_MAX)));


//   l_regionRect.x = (100 - l_regionRect.height) * (rand() / RAND_MAX + 1.0 );
//   l_regionRect.y = (100 - l_regionRect.height) * (rand() / RAND_MAX + 1.0);
   l_regionRect.x = (100 - l_regionRect.height) * (rand() / ((float)RAND_MAX));
   l_regionRect.y = (100 - l_regionRect.height) * (rand() / ((float)RAND_MAX));

   l_regionHandle = lb_AddRegion(l_layoutTemplateHandle, &l_regionRect);
   if (l_regionHandle == MTK_ERROR)
   {
      ErrMsg(Device(), "lb_AddRegion  - 2");
      return -1;
   }

   CNF_RETURN res(CNF_ERROR);
   res = SetLayout(l_layoutTemplateHandle);
   if ( lb_DestroyLayoutTemplate(l_layoutTemplateHandle) == MTK_ERROR)
   {
      ErrMsg(Device(), "lb_DestroyLayoutTemplate");
   }

   m_featureState1 = ++m_featureState1 % 4;
   return res;
}

//*****************************************************************************
// Function: int CConferenceDevice::SetLayout(LB_FRAME_HANDLE a_layoutTemplateHandle)
// Description: Set the conference layout
// Return: int 
// Parameters: LB_FRAME_HANDLE a_layoutTemplateHandle 
//*****************************************************************************
int CConferenceDevice::SetLayout(LB_FRAME_HANDLE a_layoutTemplateHandle)
{
   LB_FRAME_HANDLE l_regionList[MAX_REGION_COUNT];
   unsigned int l_regionListCount = MAX_REGION_COUNT;

   CNF_RETURN res(CNF_ERROR);
   res = lb_GetRegionList(a_layoutTemplateHandle, l_regionList, &l_regionListCount);
   if (res == MTK_ERROR)
   {
      ErrMsg(Device(), "lb_GetRegionList");
      return -1;
   }
   LogMsg(DeviceName(),"lb_GetRegionList:"+AsString(l_regionListCount));

   for (int i=0; i<l_regionListCount; i++)
   {
      eLB_SELECTION_MODE l_mode(eLB_SELECTION_MODE_USER_SELECT);
      if (CConference::SystemConfiguration()->ActiveTalkerMode() && (i==0))
      {
         switch (m_layoutTileCount)
         {
            case 1:
            case 6:
               l_mode = eLB_SELECTION_MODE_VOICE_ACTIVATED;
         }
      }
      LogMsg(DeviceName(),"lb_SetSelectionMode "+AsString(i)+":"+AsString(l_mode));
      res = lb_SetSelectionMode(l_regionList[i], l_mode);
      if (res == MTK_ERROR)
      {
         ErrMsg(Device(), "lb_SetSelectionMode");
         break;
      }
   }

   memset(&m_layoutInfo, 0, sizeof(m_layoutInfo));
   m_layoutInfo.eLayoutSize = g_confLayoutSize;
   m_layoutInfo.LayoutHandle = a_layoutTemplateHandle;
   LogMsg(DeviceName(),"cnf_SetVideoLayout");
   res = cnf_SetVideoLayout(Device(), &m_layoutInfo, 0);
   if (res == CNF_ERROR)
   {
      ErrMsg(Device(), "cnf_SetVideoLayout");
      return -1;
   }

   return 0;
}


