/**
* @file CConferenceDeviceState.h
* @brief Conference Device States -  class declarations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CConferenceDeviceState_h
#define CConferenceDeviceState_h

#include <map>
#include <string>

#include "CConferenceDevice.h"

typedef enum 
{
   ConferenceDeviceClosedState = 0,
   ConferenceDeviceOpeningState,
   ConferenceDeviceOpenedState,
} ConferenceDeviceState;

class AConferenceDeviceState;    // forward decleration

//*****************************************************************************
// Class: ::CConferenceDeviceStates
// Description: Map of the confernce device state objects
//*****************************************************************************
class CConferenceDeviceStates
{
public:
   CConferenceDeviceStates();
   ~CConferenceDeviceStates() {}
   AConferenceDeviceState* DeviceState(ConferenceDeviceState deviceState) {return m_deviceStates[ deviceState];}

private:
   std::map< ConferenceDeviceState, AConferenceDeviceState*> m_deviceStates;
};

//*****************************************************************************
// Class: ::AConferenceDeviceState
// Description: Abstract CNF conference device state
// Notes:      
//    This abstract class declares all functions for all device states as 
//    virtual and implements them as an error.
// 
//    Concrete subclasses implement only the functions that are valid for that
//    state.
// 
//    State classes are responsible only for calling primative functions on the 
//    context device object and changing the state of the context device object.
// 
//    As state classes have no non-static member variables, one set of device 
//    state objects are shared by all devices.
//*****************************************************************************
class AConferenceDeviceState
{
public:
   virtual const char* Desc() = 0;

   virtual int CnfOpenConference(CConferenceDevice& a_device) {return LogStateError(a_device, "CnfOpenConference");}
   virtual int OnCnfOpenConferenceComplete(CConferenceDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnCnfOpenConferenceComplete");}
   virtual int CnfCloseConference(CConferenceDevice& a_device);
   static AConferenceDeviceState* DeviceState(ConferenceDeviceState deviceState) {return m_deviceStates.DeviceState(deviceState);}

private:
   int LogStateError(CConferenceDevice& a_device, const std::string& msg);
   static CConferenceDeviceStates m_deviceStates;
};

//*****************************************************************************
// Class: ::CConferenceDeviceClosedState
// Description: CNF conference device closed state
//*****************************************************************************
class CConferenceDeviceClosedState : public AConferenceDeviceState
{
public:
   const char* Desc() {return "closed";}
   int CnfOpenConference(CConferenceDevice& a_device);
};

//*****************************************************************************
// Class: ::CConferenceDeviceOpeningState
// Description: CNF board device opening state
//*****************************************************************************
class CConferenceDeviceOpeningState : public AConferenceDeviceState
{
public:
   const char* Desc() {return "opening";}
   int OnCnfOpenConferenceComplete(CConferenceDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CConferenceDeviceOpenedState
// Description: CNF board device opened state
//*****************************************************************************
class CConferenceDeviceOpenedState : public AConferenceDeviceState
{
public:
   const char* Desc() {return "opened";}
};

#endif

