/**
* @file CConferencePartyDevice.cpp
* @brief Conference Party Device class implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "cnfevts.h"

#include "LogUtil.h"
#include "CConferencePartyDevice.h"
#include "CConferencePartyDeviceState.h"
#include "CConferenceBoardDevice.h"
#include "CConference.h"
#include "CSipSession.h"
#include "CGcManager.h"

#include "devmgmt.h"

//*****************************************************************************
// Function: CConferencePartyDevice::CConferencePartyDevice(const std::string &a_deviceName, CConferenceBoardDevice *a_cnfBoardDevice, AEventHandler *a_observer)
// Description: Initializing constructor
// Return:  
// Parameters: const std::string &a_deviceName 
//             CConferenceBoardDevice *a_cnfBoardDevice 
//             AEventHandler *a_observer 
//*****************************************************************************
CConferencePartyDevice::CConferencePartyDevice(const std::string& a_deviceName, CConferenceBoardDevice* a_cnfBoardDevice, AEventHandler* a_observer)
   :m_deviceState(AConferencePartyDeviceState::DeviceState(ConferencePartyDeviceClosedState))
   ,m_deviceName(a_deviceName)
   ,m_cnfBoardDevice(a_cnfBoardDevice)
   ,m_observer(a_observer)
   ,m_virtualParty(0)
	,m_recorderParty(0)
{
}

//*****************************************************************************
// All device and call operations and event handling are delegated to the state
// object with the following functions
//*****************************************************************************
int CConferencePartyDevice::Open() {return DeviceState().CnfOpenConferenceParty(*this);}
int CConferencePartyDevice::Close() {return DeviceState().CnfCloseConferenceParty(*this);}
int CConferencePartyDevice::OnCnfOpenConferencePartyComplete(METAEVENT& a_metaEvent) {return DeviceState().OnCnfOpenConferencePartyComplete(*this, a_metaEvent);}
int CConferencePartyDevice::OnDevConnected(METAEVENT& a_metaEvent) {return DeviceState().OnDevConnected(*this, a_metaEvent);}
int CConferencePartyDevice::OnDevDisconnected(METAEVENT& a_metaEvent) {return DeviceState().OnDevDisconnected(*this, a_metaEvent);}
int CConferencePartyDevice::OnCnfPartyAdded(METAEVENT& a_metaEvent) {return DeviceState().OnCnfPartyAdded(*this, a_metaEvent);}
int CConferencePartyDevice::OnCnfPartyRemoved(METAEVENT& a_metaEvent) {return DeviceState().OnCnfPartyRemoved(*this, a_metaEvent);}
int CConferencePartyDevice::OnCnfRemovePartyFailed(METAEVENT& a_metaEvent) {return DeviceState().OnCnfRemovePartyFailed(*this, a_metaEvent);}

//*****************************************************************************
// Function: void CConferencePartyDevice::SetDeviceStateTo(AConferencePartyDeviceState *state)
// Description: Set the device state
// Return: void 
// Parameters: AConferencePartyDeviceState *state 
//*****************************************************************************
void CConferencePartyDevice::SetDeviceStateTo(AConferencePartyDeviceState *state)
{
   m_deviceState = state;
   LogMsg(DeviceName(), AsString("CNF Party Device State:") + m_deviceState->Desc());
}

//*****************************************************************************
// Function: int CConferencePartyDevice::PrimCnfOpenConferenceParty()
// Description: Handle an open conference party device command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConferencePartyDevice::PrimCnfOpenConferenceParty()
{
   SRL_DEVICE_HANDLE res = cnf_OpenParty( m_cnfBoardDevice->Device(), 0, 0, (void *) this);
   if (res == CNF_ERROR)
   {
      ErrMsg(0, DeviceName() + " cnf_OpenParty");
   }
   else
   {
      Device(res);
      LogMsg(DeviceName(), "cnf_OpenParty:"+AsHexString(Device()));
   }

   return res;
}

//*****************************************************************************
// Function: int CConferencePartyDevice::PrimCnfCloseConferenceParty()
// Description: Handle a close the conference party device command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConferencePartyDevice::PrimCnfCloseConferenceParty()
{
   CNF_RETURN res(0);
   if (Device() != -1)
   {
      res = cnf_CloseParty(Device(), 0);
      if (res == CNF_ERROR)
      {
         ErrMsg(Device(), DeviceName() + " cnf_CloseParty");
      }
      else
      {
         LogMsg(DeviceName(), "cnf_CloseParty");
         if (m_observer)
         {
            m_observer->PostEvent(APPEVENT_ON_CLOSE_CNF_PARTY_COMPLETE);
         }
         CGcManager::RemoveOpenDevice(this);
      }
      Device(-1);
   }
   else
   {
      LogMsg(DeviceName(), "device already closed");
   }
   return res;
}

//*****************************************************************************
// Function: int CConferencePartyDevice::PrimOnCnfOpenConferencePartyComplete(METAEVENT &a_metaEvent)
// Description: Process an open conference party device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDevice::PrimOnCnfOpenConferencePartyComplete(METAEVENT& a_metaEvent)
{
   CNF_ATTR l_attributes[2];
   memset(&l_attributes[0], 0, sizeof(CNF_ATTR));
   memset(&l_attributes[1], 0, sizeof(CNF_ATTR));
   l_attributes[0].unAttribute = ECNF_PARTY_ATTR_ECHO_CANCEL;
   if (CConference::SystemConfiguration()->ConferencePartyEchoCancelation())
   {
      l_attributes[0].unValue = ECNF_ATTR_STATE_ENABLED;
   }
   else
   {
      l_attributes[0].unValue = ECNF_ATTR_STATE_DISABLED;
   }

   l_attributes[1].unAttribute = ECNF_PARTY_ATTR_TONE_CLAMPING;
   if (CConference::SystemConfiguration()->ConferenceToneClamping())
   {
      l_attributes[1].unValue = ECNF_ATTR_STATE_ENABLED;
   }
   else
   {
      l_attributes[1].unValue = ECNF_ATTR_STATE_DISABLED;
   }

   CNF_ATTR_INFO l_attributeInfo;
   memset(&l_attributeInfo, 0, sizeof(CNF_ATTR_INFO));
   l_attributeInfo.unAttrCount = 2;
   l_attributeInfo.pAttrList = l_attributes;

   CNF_RETURN res = cnf_SetAttributes(Device(), &l_attributeInfo, 0);
   if (res == CNF_ERROR)
   {
      ErrMsg(Device(), "cnf_SetAttributes");
      return res;
   }
   else
   {
      LogMsg(DeviceName(), "cnf_SetAttributes-ECNF_PARTY_ATTR_ECHO_CANCEL:"+AsString(l_attributes[0].unValue));
      LogMsg(DeviceName(), "cnf_SetAttributes-ECNF_PARTY_ATTR_TONE_CLAMPING:"+AsString(l_attributes[1].unValue));
   }

   CGcManager::AddOpenDevice(this);
   m_deviceName = ATDV_NAMEP(Device());
   m_observer->PostEvent(APPEVENT_ON_OPEN_CNF_PARTY_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CConferencePartyDevice::PrimOnDevConnected(METAEVENT &a_metaEvent)
// Description: Handle a conference party device connected event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDevice::PrimOnDevConnected(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_CNF_CONNECTED);
   return 0;
}

//*****************************************************************************
// Function: int CConferencePartyDevice::PrimOnDevDisconnected(METAEVENT &a_metaEvent)
// Description: Handle a conference party device disconnected event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDevice::PrimOnDevDisconnected(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_CNF_DISCONNECTED);
   return 0;
}

//*****************************************************************************
// Function: int CConferencePartyDevice::PrimOnCnfPartyAdded(METAEVENT &a_metaEvent)
// Description: Handle an add conference party device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDevice::PrimOnCnfPartyAdded(METAEVENT& a_metaEvent)
{
//   LogMsg(DeviceName(),"PrimOnCnfPartyAdded");
   m_observer->PostEvent(APPEVENT_ON_JOIN_CNF_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CConferencePartyDevice::PrimOnCnfPartyRemoved(METAEVENT &a_metaEvent)
// Description: Handle a remove conference party device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDevice::PrimOnCnfPartyRemoved(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_ON_LEAVE_CNF_COMPLETE);
   return 0;
}
//*****************************************************************************
// Function: int CConferencePartyDevice::PrimOnCnfRemovePartyFailed(METAEVENT &a_metaEvent)
// Description: Handle a remove conference party device completion event
// Return: int - 0 for success, otherwise error
// Parameters: METAEVENT &a_metaEvent
//*****************************************************************************
int CConferencePartyDevice::PrimOnCnfRemovePartyFailed(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_ON_LEAVE_CNF_FAILED);
   return 0;
}


//*****************************************************************************
// Function: int CConferencePartyDevice::OnCnfGetVisiblePartyList(METAEVENT &a_metaEvent)
// Description: Handle a get visible party list completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDevice::OnCnfGetVisiblePartyList(METAEVENT& a_metaEvent)
{
   return 0;
}

//*****************************************************************************
// Function: int CConferencePartyDevice::GetReceivePortInfo()
// Description: Get the conference party device receive port information
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConferencePartyDevice::GetReceivePortInfo()
{
   int res = dev_GetReceivePortInfo(Device(), (void*)(ID()));
   if (res == -1)
   {
      ErrMsg(Device(), "dev_GetReceivePortInfo");
   }
   else
   {
      LogMsg(Device(), "dev_GetReceivePortInfo");
   }
   return res;
}

//*****************************************************************************
// Function: int CConferencePartyDevice::OnGetReceivePortInfoComplete(METAEVENT &a_metaEvent)
// Description: Handle a get device receive port information completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDevice::OnGetReceivePortInfoComplete(METAEVENT& a_metaEvent)
{
   memcpy(&(m_rxPortInfoList),a_metaEvent.evtdatap,sizeof(DM_PORT_INFO_LIST));
//   LogMsg(Device(), "Number of RX ports: " + AsString(m_rxPortInfoList.unCount));
//   for (int i = 0; i <m_rxPortInfoList.unCount; i++ )
//   {
//      LogMsg(Device(), "port list[" + AsString(i)+"] "
//             + " device_ID:" + AsHexString((long)m_rxPortInfoList.port_info[i].device_ID)
//             + " port_ID:" + AsHexString((long)m_rxPortInfoList.port_info[i].port_ID)
//             + " port__media_type:" + AsString(m_rxPortInfoList.port_info[i].port_media_type));
//   }
   m_observer->PostEvent(APPEVENT_CNF_GET_RX_PORT_INFO_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CConferencePartyDevice::GetTransmitPortInfo()
// Description: Get the conference party device transmit port information
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConferencePartyDevice::GetTransmitPortInfo()
{
   int res = dev_GetTransmitPortInfo(Device(), (void*)(ID()));
   if (res == -1)
   {
      ErrMsg(Device(), "dev_GetTransmitPortInfo");
   }
   else
   {
      LogMsg(Device(), "dev_GetTransmitPortInfo");
   }
   return res;
}

//*****************************************************************************
// Function: int CConferencePartyDevice::OnGetTransmitPortInfoComplete(METAEVENT &a_metaEvent)
// Description: Handle a get device transmit port information completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDevice::OnGetTransmitPortInfoComplete(METAEVENT& a_metaEvent)
{
   memcpy(&(m_txPortInfoList),a_metaEvent.evtdatap,sizeof(DM_PORT_INFO_LIST));
//   LogMsg(Device(), "Number of TX ports: " + AsString(m_txPortInfoList.unCount));
//   for (int i = 0; i <m_txPortInfoList.unCount; i++ )
//   {
//      LogMsg(Device(), "port list[" + AsString(i)+"] "
//             + " device_ID:" + AsHexString((long)m_txPortInfoList.port_info[i].device_ID)
//             + " port_ID:" + AsHexString((long)m_txPortInfoList.port_info[i].port_ID)
//             + " port__media_type:" + AsString(m_txPortInfoList.port_info[i].port_media_type));
//   }
   m_observer->PostEvent(APPEVENT_CNF_GET_TX_PORT_INFO_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CConferencePartyDevice::OnPortConnected(METAEVENT &a_metaEvent)
// Description: Handle a conference party device port connect completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDevice::OnPortConnected(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_CNF_PORT_CONNECTED);
   return 0;
}

//*****************************************************************************
// Function: int CConferencePartyDevice::OnPortDisconnected(METAEVENT &a_metaEvent)
// Description: Handle a conference party device port disconnect completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDevice::OnPortDisconnected(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_CNF_PORT_DISCONNECTED);
   return 0;
}

//*****************************************************************************
// Function: void CConferencePartyDevice::ProcessEvent(METAEVENT &a_metaEvent)
// Description: Process an event
// Return: void 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
void CConferencePartyDevice::ProcessEvent(METAEVENT& a_metaEvent)
{
   switch (a_metaEvent.evttype)
   {
      case CNFEV_OPEN_PARTY:
         LogMsg(Device(), "CNFEV_OPEN_PARTY cnfDevice=" + AsString(":") + AsString(Device()));
         OnCnfOpenConferencePartyComplete(a_metaEvent);
         break;

      case CNFEV_OPEN_PARTY_FAIL:
         LogMsg(Device(), "CNFEV_OPEN_PARTY_FAIL");
         break;

      case CNFEV_SET_ATTRIBUTE:
         LogMsg(Device(), "CNFEV_SET_ATTRIBUTE");
         break;

      case CNFEV_SET_ATTRIBUTE_FAIL:
         LogMsg(Device(), "CNFEV_SET_ATTRIBUTE_FAIL");
         break;

      case CNFEV_ERROR:
         ErrMsg(Device(), "CNFEV_ERROR");
         break;

      case DMEV_CONNECT:
         LogMsg(Device(), "DMEV_CONNECT");
         OnDevConnected(a_metaEvent);
         break;

      case DMEV_DISCONNECT:
         LogMsg(Device(), "DMEV_DISCONNECT");
         OnDevDisconnected(a_metaEvent);
         break;

      case DMEV_GET_RX_PORT_INFO:
         LogMsg(Device(), "DMEV_GET_RX_PORT_INFO");
         OnGetReceivePortInfoComplete(a_metaEvent);
         break;

      case DMEV_GET_RX_PORT_INFO_FAIL:
         LogMsg(Device(), "DMEV_GET_RX_PORT_INFO_FAIL");
         break;

      case DMEV_GET_TX_PORT_INFO:
         LogMsg( Device(), "DMEV_GET_TX_PORT_INFO");
         OnGetTransmitPortInfoComplete(a_metaEvent);
         break;

      case DMEV_GET_TX_PORT_INFO_FAIL:
         LogMsg(Device(), "DMEV_GET_TX_PORT_INFO_FAIL");
         break;

      case DMEV_PORT_CONNECT:
         LogMsg(Device(), "DMEV_PORT_CONNECT");
         OnPortConnected(a_metaEvent);
         break;

      case DMEV_PORT_CONNECT_FAIL:
         LogMsg(Device(), "DMEV_PORT_CONNECT_FAIL");
         break;

      case DMEV_PORT_DISCONNECT:
         LogMsg(Device(), "DMEV_PORT_DISCONNECT");
         OnPortDisconnected(a_metaEvent);
         break;

      case DMEV_PORT_DISCONNECT_FAIL:
         LogMsg(Device(), "DMEV_PORT_DISCONNECT_FAIL");
         break;

      case CNFEV_ADD_PARTY:
         LogMsg(Device(), "CNFEV_ADD_PARTY");
         OnCnfPartyAdded(a_metaEvent);
         break;

      case CNFEV_REMOVE_PARTY:
         LogMsg(Device(), "CNFEV_REMOVE_PARTY");
         OnCnfPartyRemoved(a_metaEvent);
         break;

		case CNFEV_REMOVE_PARTY_FAIL:
         LogMsg(Device(), "CNFEV_REMOVE_PARTY_FAIL");
         OnCnfRemovePartyFailed(a_metaEvent);
         break;


      case CNFEV_GET_VISIBLE_PARTY_LIST:
         LogMsg(Device(), "CNFEV_GET_VISIBLE_PARTY_LIST");
         OnCnfGetVisiblePartyList(a_metaEvent);
         break;

      case CNFEV_GET_VISIBLE_PARTY_LIST_FAIL:
         LogMsg(Device(), "CNFEV_GET_VISIBLE_PARTY_LIST_FAIL");
         break;

      case APPEVENT_CLOSE_COMMAND:
         LogMsg(Device(), "APPEVENT_CLOSE_COMMAND");
         Close();
         break;

      default:
         LogMsg(Device(), "unhandled event:" + AsHexString(a_metaEvent.evttype));
   }
}


int CConferencePartyDevice::DisplayCallerId()
{
	// SKS not for recorder
	if(!IsPartyRecorder()) {
   	m_observer->PostEvent(APPEVENT_DISPLAY_CALLERID_COMMAND);
	}
   return 0;
}

int CConferencePartyDevice::DisplayConferenceLayout(int a_layout)
{
   m_observer->PostEvent(APPEVENT_DISPLAY_LAYOUT_COMMAND, a_layout);
   return 0;
}

void CConferencePartyDevice::SetCallerId(int a_callerId)
{
   CSipSession* l_sipSession = dynamic_cast<CSipSession*>(m_observer);
   if (l_sipSession)
   {
      l_sipSession->SetCallerId(a_callerId);
   }
}
