/**
* @file CConferencePartyDeviceState.cpp
* @brief Conference Party Device States - class implementations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "LogUtil.h"
#include "CConferencePartyDeviceState.h"

CConferencePartyDeviceStates AConferencePartyDeviceState::m_deviceStates;

//*****************************************************************************
// Function: int AConferencePartyDeviceState::LogStateError(CConferencePartyDevice &a_device, const string &msg)
// Description: Log an invalid state for operation/event message
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice &a_device 
//             const string &msg 
//*****************************************************************************
int AConferencePartyDeviceState::LogStateError(CConferencePartyDevice& a_device, const string& msg)
{
   std::cout << a_device.DeviceName()
   << " " << Desc() << " is invalid device state for "
   << msg
   << std::endl;
   return -1;
}

//*****************************************************************************
// Function: int AConferencePartyDeviceState::CnfCloseConferenceParty(CConferencePartyDevice &a_device)
// Description: Close a conference party device
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice &a_device 
//*****************************************************************************
int AConferencePartyDeviceState::CnfCloseConferenceParty(CConferencePartyDevice& a_device) 
{
   a_device.SetDeviceStateTo(DeviceState(ConferencePartyDeviceClosedState));
   return a_device.PrimCnfCloseConferenceParty();
}

//*****************************************************************************
// Function: CConferencePartyDeviceStates::CConferencePartyDeviceStates()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CConferencePartyDeviceStates::CConferencePartyDeviceStates()
{
   m_deviceStates[ConferencePartyDeviceClosedState] = new CConferencePartyDeviceClosedState();
   m_deviceStates[ConferencePartyDeviceOpeningState] = new CConferencePartyDeviceOpeningState();
   m_deviceStates[ConferencePartyDeviceOpenedState] = new CConferencePartyDeviceOpenedState();
}

//*****************************************************************************
// Function: int CConferencePartyDeviceClosedState::CnfOpenConferenceParty(CConferencePartyDevice &a_device)
// Description: Open a conference party device
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice &a_device 
//*****************************************************************************
int CConferencePartyDeviceClosedState::CnfOpenConferenceParty(CConferencePartyDevice& a_device) 
{
   a_device.SetDeviceStateTo(DeviceState(ConferencePartyDeviceOpeningState));
   return a_device.PrimCnfOpenConferenceParty();
}

//*****************************************************************************
// Function: int CConferencePartyDeviceOpeningState::OnCnfOpenConferencePartyComplete(CConferencePartyDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an open conference party device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDeviceOpeningState::OnCnfOpenConferencePartyComplete(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent) 
{
   a_device.SetDeviceStateTo(DeviceState(ConferencePartyDeviceOpenedState));
   return a_device.PrimOnCnfOpenConferencePartyComplete(a_metaEvent);
}

//*****************************************************************************
// Function: int CConferencePartyDeviceOpenedState::OnDevConnected(CConferencePartyDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle Conference party device connected event
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDeviceOpenedState::OnDevConnected(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent) 
{
   return a_device.PrimOnDevConnected(a_metaEvent);
}

//*****************************************************************************
// Function: int CConferencePartyDeviceOpenedState::OnDevDisconnected(CConferencePartyDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle conference party device disconnected event
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDeviceOpenedState::OnDevDisconnected(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent) 
{
   return a_device.PrimOnDevDisconnected(a_metaEvent);
}

//*****************************************************************************
// Function: int CConferencePartyDeviceOpenedState::OnCnfPartyAdded(CConferencePartyDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle add conference party device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDeviceOpenedState::OnCnfPartyAdded(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent) 
{
   return a_device.PrimOnCnfPartyAdded(a_metaEvent);
}

//*****************************************************************************
// Function: int CConferencePartyDeviceOpenedState::OnCnfPartyRemoved(CConferencePartyDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle remove conference party device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CConferencePartyDeviceOpenedState::OnCnfPartyRemoved(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent) 
{
   return a_device.PrimOnCnfPartyRemoved(a_metaEvent);
}

//*****************************************************************************
// Function: int CConferencePartyDeviceOpenedState::OnCnfPartyRemoved(CConferencePartyDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle remove conference party device completion event
// Return: int - 0 for success, otherwise error
// Parameters: CConferencePartyDevice &a_device
//             METAEVENT &a_metaEvent
//*****************************************************************************
int CConferencePartyDeviceOpenedState::OnCnfRemovePartyFailed(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent)
{
   return a_device.PrimOnCnfRemovePartyFailed(a_metaEvent);
}

