/**
* @file CConferencePartyDeviceState.h
* @brief Conference Party Device States -  class declarations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CConferencePartyDeviceState_h
#define CConferencePartyDeviceState_h

#include <map>
#include <string>

#include "CConferencePartyDevice.h"

typedef enum 
{
   ConferencePartyDeviceClosedState = 0,
   ConferencePartyDeviceOpeningState,
   ConferencePartyDeviceOpenedState,
} ConferencePartyDeviceState;

class AConferencePartyDeviceState;

//*****************************************************************************
// Class: ::CConferencePartyDeviceStates
// Description: Map of the conference party device state objects
//*****************************************************************************
class CConferencePartyDeviceStates
{
public:
   CConferencePartyDeviceStates();
   ~CConferencePartyDeviceStates()
   {
      ;
   }
   AConferencePartyDeviceState* DeviceState(ConferencePartyDeviceState deviceState)
   {
      return m_deviceStates[ deviceState];
   }
private:
   std::map< ConferencePartyDeviceState, AConferencePartyDeviceState*> m_deviceStates;
};

//*****************************************************************************
// Class: ::AConferencePartyDeviceState
// Description: Abstract CNF conference party device state class
// Notes:      
//    This abstract class declares all functions for all device states as 
//    virtual and implements them as an error.
// 
//    Concrete subclasses implement only the functions that are valid for that
//    state.
// 
//    State classes are responsible only for calling primative functions on the 
//    context device object and changing the state of the context device object.
// 
//    As state classes have no non-static member variables, one set of device 
//    state objects are shared by all devices.
//*****************************************************************************
class AConferencePartyDeviceState
{
public:
   virtual const char* Desc() = 0;

   virtual int CnfOpenConferenceParty(CConferencePartyDevice& a_device) {return LogStateError(a_device, "CnfOpenConferenceParty");}
   virtual int OnCnfOpenConferencePartyComplete(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnCnfOpenConferencePartyComplete");}
   virtual int CnfCloseConferenceParty(CConferencePartyDevice& a_device); 
   virtual int OnDevConnected(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnDevConnectedComplete");}
   virtual int OnDevDisconnected(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnDevDisconnectedComplete");}
   virtual int OnCnfPartyAdded(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnCnfPartyAdded");}
   virtual int OnCnfPartyRemoved(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnCnfPartyRemoved");}
   virtual int OnCnfRemovePartyFailed(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnCnfRemovePartyFailed");}

   static AConferencePartyDeviceState* DeviceState(ConferencePartyDeviceState deviceState) {return m_deviceStates.DeviceState(deviceState);}

private:
   int LogStateError(CConferencePartyDevice& a_device, const std::string& msg);
   static CConferencePartyDeviceStates m_deviceStates;
};

//*****************************************************************************
// Class: ::CConferencePartyDeviceClosedState
// Description: CNF conference party device closed state
//*****************************************************************************
class CConferencePartyDeviceClosedState : public AConferencePartyDeviceState
{
public:
   const char* Desc() {return "closed";}
   int CnfOpenConferenceParty(CConferencePartyDevice& a_device);
};

//*****************************************************************************
// Class: ::CConferencePartyDeviceOpeningState
// Description: CNF conference party device opening state
//*****************************************************************************
class CConferencePartyDeviceOpeningState : public AConferencePartyDeviceState
{
public:
   const char* Desc() {return "opening";}
   int OnCnfOpenConferencePartyComplete(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CConferencePartyDeviceOpenedState
// Description: CNF conference party device opened state
//*****************************************************************************
class CConferencePartyDeviceOpenedState : public AConferencePartyDeviceState
{
public:
   const char* Desc() {return "opened";}
   int OnDevConnected(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent);
   int OnDevDisconnected(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent);
   int OnCnfPartyAdded(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent);
   int OnCnfPartyRemoved(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent);
   int OnCnfRemovePartyFailed(CConferencePartyDevice& a_device, METAEVENT& a_metaEvent);
};

#endif

