/**
* @file CConnection.cpp
* @brief Connection class implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "config.h"

#include "LogUtil.h"
#include "CConnection.h"
#include "CIpmDevice.h"

CIpmDevice* CConnection::m_ipmDevice1 = 0;
CIpmDevice* CConnection::m_ipmDevice2 = 0;

DM_PORT_CONNECT_INFO_LIST CConnection::m_portConnectInfoListIpm1TxIpm2Rx;
DM_PORT_CONNECT_INFO_LIST CConnection::m_portConnectInfoListIpm2TxIpm1Rx;

//*****************************************************************************
// Function: CConnection::CConnection()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CConnection::CConnection()
{
}

//*****************************************************************************
// Function: int CConnection::Connect(CIpmDevice *a_ipmDevice)
// Description: Connect a device
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice *a_ipmDevice 
//*****************************************************************************
int CConnection::Connect(CIpmDevice* a_ipmDevice)
{
   LogMsg("TEST IPM-IPM Connect:", a_ipmDevice->DeviceName());
   if (!m_ipmDevice1 && !m_ipmDevice2)
   {
      m_ipmDevice1 = a_ipmDevice;
      return 0;
   }
   if (m_ipmDevice1 && !m_ipmDevice2)
   {
      m_ipmDevice2 = a_ipmDevice;
      return MakePortConnections();
   }
   LogMsg("Connect: devices invalid");
   return -1;
}

//*****************************************************************************
// Function: int CConnection::MakePortConnections()
// Description: Connect the ports between two devices
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConnection::MakePortConnections()
{
   int num_matched_ports = CreatePortConnectInfoList( m_portConnectInfoListIpm1TxIpm2Rx,
                                                      m_ipmDevice1->TxPortInfoList(),
                                                      m_ipmDevice2->RxPortInfoList());
   if (0 == num_matched_ports)
   {
      LogMsg("No matched ports found - Ipm1TxIpm2Rx");
      return -1;
   }

   int res = dev_PortConnect(m_ipmDevice1->Device(), &m_portConnectInfoListIpm1TxIpm2Rx, NULL);
   if (-1 == res)
   {
      ErrMsg(m_ipmDevice1->Device(), "dev_PortConnect-Ipm1TxIpm2Rx failed");
      return res;
   }
   else
   {
      LogMsg(m_ipmDevice1->Device(), "TEST IPM-IPM dev_PortConnect-Ipm1TxIpm2Rx");
   }

   num_matched_ports = CreatePortConnectInfoList( m_portConnectInfoListIpm2TxIpm1Rx,
                                                  m_ipmDevice2->TxPortInfoList(),
                                                  m_ipmDevice1->RxPortInfoList());
   if (0 == num_matched_ports)
   {
      LogMsg("No matched ports found-Ipm2TxIpm1Rx");
      return -1;
   }

   res = dev_PortConnect(m_ipmDevice2->Device(), &m_portConnectInfoListIpm2TxIpm1Rx, NULL);
   if (-1 == res)
   {
      ErrMsg(m_ipmDevice2->Device(), "dev_PortConnect-Ipm2TxIpm1Rx Failed");
      return res;
   }
   else
   {
      LogMsg(m_ipmDevice2->Device(), "TEST IPM-IPM dev_PortConnect-Ipm2TxIpm1Rx");
   }
   return res;
}

//*****************************************************************************
// Function: int CConnection::CreatePortConnectInfoList(DM_PORT_CONNECT_INFO_LIST &a_portConnectionInfoList, const DM_PORT_INFO_LIST &a_txPortInfoList, const DM_PORT_INFO_LIST &a_rxPortInfoList)
// Description: Create a port connection info list
// Return: int - 0 for success, otherwise error 
// Parameters: DM_PORT_CONNECT_INFO_LIST &a_portConnectionInfoList 
//             const DM_PORT_INFO_LIST &a_txPortInfoList 
//             const DM_PORT_INFO_LIST &a_rxPortInfoList 
//*****************************************************************************
int CConnection::CreatePortConnectInfoList(  DM_PORT_CONNECT_INFO_LIST& a_portConnectionInfoList,
                                             const DM_PORT_INFO_LIST& a_txPortInfoList,
                                             const DM_PORT_INFO_LIST& a_rxPortInfoList)
{
   INIT_DM_PORT_CONNECT_INFO_LIST(&a_portConnectionInfoList);

   // Loop through all transmit ports of 1st device
   int k = 0;
   int i;
   LogMsg("TEST IPM-IPM CreatePortConnectInfoList: count=", AsString(a_txPortInfoList.unCount));
   for (i = 0; i < a_txPortInfoList.unCount; ++i)
   {
      LogMsg("txPortInfoList[" + AsString(i)+"] "
             + " device_ID:" + AsHexString((long)a_txPortInfoList.port_info[i].device_ID)
             + " port_ID:" + AsHexString((long)a_txPortInfoList.port_info[i].port_ID)
             + " port__media_type:" + AsString(a_txPortInfoList.port_info[i].port_media_type));
      DM_PORT_MEDIA_TYPE type_tx = a_txPortInfoList.port_info[i].port_media_type;
      // find appropriate RX port on 2nd device
      if (type_tx == DM_PORT_MEDIA_TYPE_NBUP)
      {
         continue;
      }

      bool bFound = false;
      int j;
      for (j = 0; j < a_rxPortInfoList.unCount; ++j)
      {
         DM_PORT_MEDIA_TYPE type_rx = a_rxPortInfoList.port_info[j].port_media_type;
         if (type_tx == type_rx)
         {
            bFound = true;
            break;
         }
      }
      if (!bFound)
      {
         LogMsg("no match for txPortInfoList[" + AsString(i)+"] "
                + " device_ID:" + AsHexString((long)a_txPortInfoList.port_info[i].device_ID)
                + " port_ID:" + AsHexString((long)a_txPortInfoList.port_info[i].port_ID)
                + " port__media_type:" + AsString(a_txPortInfoList.port_info[i].port_media_type));
         continue;
      }

      LogMsg("match for rxPortInfoList[" + AsString(i)+"] "
             + " device_ID:" + AsHexString((long)a_rxPortInfoList.port_info[i].device_ID)
             + " port_ID:" + AsHexString((long)a_rxPortInfoList.port_info[i].port_ID)
             + " port__media_type:" + AsString(a_rxPortInfoList.port_info[i].port_media_type));
      // create element of connect list
      DM_PORT_CONNECT_INFO& info = a_portConnectionInfoList.port_connect_info[k];
      INIT_DM_PORT_CONNECT_INFO(&info);
      info.unFlags = DMFL_TRANSCODE_ON;
      info.port_info_tx = a_txPortInfoList.port_info[i];
      info.port_info_rx = a_rxPortInfoList.port_info[j];
      ++k;
   }
   a_portConnectionInfoList.unCount = k;
   return k;
}

//*****************************************************************************
// Function: int CConnection::Disconnect(CIpmDevice *a_ipmDevice)
// Description: Disconnect the port connections between the two devies
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice *a_ipmDevice 
//*****************************************************************************
int CConnection::Disconnect(CIpmDevice* a_ipmDevice)
{
   if (m_ipmDevice1 && m_ipmDevice2)
   {
      int res = dev_PortDisconnect(m_ipmDevice1->Device(), &CConnection::m_portConnectInfoListIpm1TxIpm2Rx, NULL);
      if (-1 == res)
      {
         ErrMsg( m_ipmDevice1->Device(), "dev_PortDisconnect");
         return res;
      }
      else
      {
         LogMsg( m_ipmDevice1->Device(), "dev_PortDisconnect");
      }

      res = dev_PortDisconnect(m_ipmDevice2->Device(), &m_portConnectInfoListIpm2TxIpm1Rx, NULL);
      if (-1 == res)
      {
         ErrMsg( m_ipmDevice2->Device(), "dev_PortDisconnect");
         return res;
      }
      else
      {
         LogMsg( m_ipmDevice2->Device(), "dev_PortDisconnect");
      }
      m_ipmDevice1 = 0;
      m_ipmDevice2 = 0;
   }
   return 0;
}

