/**
* @file CGcManager.cpp
* @brief GlobalCall Manager class implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "CGcManager.h"
#include "LogUtil.h"
#include "CGcSipDevice.h"
#include "CIpmDevice.h"
#include "CMediaDevice.h"
#include "CSipSession.h"
#include "CConference.h"
#include "CConferenceBoardDevice.h"
#include "CConferenceDevice.h"
#include "CVirtualCaller.h"
#include "CScheduler.h"

SystemDeviceMap CGcManager::m_systemDevices;
DeviceSet CGcManager::m_openDevices;
SipSessionSet CGcManager::m_activeSipSessions;

//*****************************************************************************
// Function: CGcManager::CGcManager()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CGcManager::CGcManager(CSystemConfiguration& m_systemConfiguration)
   :m_mmScheduler(*CScheduler<CMediaDevice>::Instance())
   ,m_featureScheduler(*CScheduler<CConference>::Instance())
   ,m_systemConfiguration(m_systemConfiguration)
{
}

//*****************************************************************************
// Function: void CGcManager::RegisterDevice(ADevice *a_device)
// Description: Register a device for event dispatch
// Return: void 
// Parameters: ADevice *a_device 
//*****************************************************************************
void CGcManager::RegisterDevice(ADevice* a_device)
{
   m_systemDevices[a_device->ID()] = a_device;
}

//*****************************************************************************
// Function: void CGcManager::RegisterDevice(long a_id, ADevice *a_device)
// Description: Register a device for event dispatch
// Return: void 
// Parameters: long a_id 
//             ADevice *a_device 
//*****************************************************************************
void CGcManager::RegisterDevice(long a_id, ADevice* a_device)
{
   m_systemDevices[a_id] = a_device;
}

//*****************************************************************************
// Function: void CGcManager::UnregisterDevice(long a_id)
// Description: Unregister a device for event dispatch
// Return: void 
// Parameters: long a_id 
//*****************************************************************************
void CGcManager::UnregisterDevice(long a_id)
{
   SystemDeviceMap::iterator pos;

   if ((pos = m_systemDevices.find(a_id)) == m_systemDevices.end())
   {
      ErrMsg(0, AsString("Could find/unregister device with ID=") + AsString(a_id));
   }
   else
   {
      m_systemDevices.erase(pos);
   }
}


//*****************************************************************************
// Function: int CGcManager::Start()
// Description: Initialize and start GlobalCall, and open configured devices
// Return: int - 0 for success, otherwise error 
// Parameters: none
//*****************************************************************************
int CGcManager::Start()
{
   LogMsg("starting GlobalCall...");

   memset(&m_libData,0,sizeof(IPCCLIB_START_DATA));
   memset(m_virtBoard,0,sizeof(IP_VIRTBOARD) * 1); 
   INIT_IPCCLIB_START_DATA(&m_libData, 1, m_virtBoard);
   INIT_IP_VIRTBOARD(&m_virtBoard[0]);

   m_libData.num_boards = 1; 
   m_libData.max_parm_data_size = 4096;
   m_libData.media_operational_mode = MEDIA_OPERATIONAL_MODE_3PCC;

   m_ccLibStart[0].cclib_name = "GC_H3R_LIB";
   m_ccLibStart[0].cclib_data = &m_libData;
   m_ccLibStart[1].cclib_name = "GC_IPM_LIB";
   m_ccLibStart[1].cclib_data = 0;

   m_virtBoard[0].total_max_calls = m_systemConfiguration.getMaxCalls();
   m_virtBoard[0].h323_max_calls = IP_CFG_NO_CALLS;
   m_virtBoard[0].sip_max_calls = m_systemConfiguration.getMaxCalls();
   m_virtBoard[0].sip_signaling_port = m_systemConfiguration.SipPort();
   m_virtBoard[0].localIP.ip_ver = IPVER4;
   m_virtBoard[0].localIP.u_ipaddr.ipv4 = m_systemConfiguration.LocalIPinByteOrder();
	m_virtBoard[0].sip_msginfo_mask = IP_SIP_MIME_ENABLE | IP_SIP_MSGINFO_ENABLE;

   m_gcLibStart.cclib_list = m_ccLibStart;
   m_gcLibStart.num_cclibs = 1;

   if (gc_Start(&m_gcLibStart) != GC_SUCCESS)
   {
      LogMsg("ERROR: gc_Start error.  Check that another gcLib process is not running");
      return -1;
   }
   LogMsg("GlobalCall started");

   CConference::AllowedLayouts(m_systemConfiguration.AllowedLayoutsCount(), m_systemConfiguration.AllowedLayouts());
   CConference::InitialLayout(m_systemConfiguration.InitialLayout());
   CConferenceBoardDevice* l_conferenceBoard = new CConferenceBoardDevice("mcxB1");
   CConference::SetConferenceBoard(l_conferenceBoard);
   CConference::SystemConfiguration(&m_systemConfiguration);

   RegisterDevice(l_conferenceBoard);
   if (l_conferenceBoard->Open() != 1 ){
      LogMsg("ERROR: Conference Board Open error.  Check that license includes VidCnf(mcx) or HDVoiceCnf(mcx)");
      return -1;
   }

   std::string l_sipAddress = m_systemConfiguration.PhoneNumber();
   l_sipAddress  = l_sipAddress + AsString("@") + m_systemConfiguration.LocalIpAddress();

   char l_tmp[64];

   sprintf(l_tmp, "%s@%s", m_systemConfiguration.PhoneNumber().c_str(), m_systemConfiguration.LocalIpAddress().c_str());
   LogMsg(AsString(l_tmp));

   CGcSipDevice* gcBoardDevice = new CGcSipDevice("iptB1", m_systemConfiguration.SipRegistrar(), l_sipAddress);
   RegisterDevice(gcBoardDevice);
   if (gcBoardDevice->Open() != GC_SUCCESS){
      LogMsg("ERROR: GC Board Open error.  Check that license includes IPCC");
      return -1;
   }

   // TODO: need to know that the cnf board reset has 
   // sleep(2);

   CGcSipDevice* l_gcDevice(0);
   CSipSession* l_sipSession(0);
   CIpmDevice* l_ipmDevice(0);
   CMediaDevice* l_mmDevice(0);
   char l_devName[30];
   std::string l_name;

   for (int i = 1; i <= m_systemConfiguration.getMaxCalls(); i++)
   {
      sprintf( l_devName, "iptB1T%d", i);
      l_name = l_devName;
      l_gcDevice = new CGcSipDevice(l_name, m_systemConfiguration.SipRegistrar(), l_sipAddress);
      RegisterDevice(l_gcDevice);

      l_ipmDevice = new CIpmDevice(CIpmDevice::GetNextIpmDeviceName());
      //JM
      l_ipmDevice->SetDtmfMode( m_systemConfiguration.DtmfMode()); 
      RegisterDevice(l_ipmDevice);

      l_mmDevice = new CMediaDevice(CMediaDevice::GetNextMediaDeviceName());
      RegisterDevice(l_mmDevice);
      l_mmDevice->SetClip( m_systemConfiguration.MediaFilePath(), m_systemConfiguration.GreetingClip());

      sprintf( l_devName, "caller%d", i);
      l_name = l_devName;
      l_sipSession = new CSipSession(l_name, l_gcDevice, l_ipmDevice, l_mmDevice, m_systemConfiguration);
      l_sipSession->SetDefaultVideoBandwidth( m_systemConfiguration.DefaultVideoRate());
      l_gcDevice->Open();
      l_ipmDevice->Open();
      l_mmDevice->Open();
   }

   return 0;
}

//*****************************************************************************
// Function: void CGcManager::Stop()
// Description: Stop and close all devices and stop GlobalCall
// Return: void 
// Parameters: none 
//*****************************************************************************
void CGcManager::Stop()
{
   SipSessionSet l_activeSipSessions(m_activeSipSessions);
   SipSessionSet::iterator l_sipSessionPos;

   LogMsg("Active SIP Sessions:");
   for (l_sipSessionPos = m_activeSipSessions.begin();
       l_sipSessionPos != m_activeSipSessions.end();
       l_sipSessionPos++)
   {
      const std::string& l_deviceName = (*l_sipSessionPos)->SessionName();
      LogMsg(l_deviceName);
   }

   LogMsg("Stopping active SIP sessions...");
   for (l_sipSessionPos = m_activeSipSessions.begin();
       l_sipSessionPos != m_activeSipSessions.end();
       l_sipSessionPos++)
   {
      const std::string& l_deviceName = (*l_sipSessionPos)->SessionName();
      (*l_sipSessionPos)->StopSession();
   }

   while (ActiveSipSessionsExist())
   {
      LogMsg(AsString(m_activeSipSessions.size()) + " SIP sessions active, Ctrl-C to abort");
#ifdef WIN32
	Sleep(2000);
#else
      sleep(2);
#endif
   }

   DeviceSet l_openDevices(m_openDevices);
   DeviceSet::iterator l_devicePos;

   LogMsg("devices to close:");
   for (l_devicePos = l_openDevices.begin();
       l_devicePos != l_openDevices.end();
       l_devicePos++)
   {
      const std::string& l_deviceName = (*l_devicePos)->DeviceName();
      LogMsg(l_deviceName);
   }

   LogMsg("closing devices...");
   for (l_devicePos = l_openDevices.begin();
       l_devicePos != l_openDevices.end();
       l_devicePos++)
   {
      const std::string& l_deviceName = (*l_devicePos)->DeviceName();
      (*l_devicePos)->PutEvent(APPEVENT_CLOSE_COMMAND,0,0);
   }

   while (OpenDevicesExist())
   {
      LogMsg(AsString(m_openDevices.size()) + " devices not closed, Ctrl-C to abort");
#ifdef WIN32
	Sleep(2000);
#else
      sleep(2);
#endif
   }

   LogMsg("stopping GlobalCall...");
   if (gc_Stop() != GC_SUCCESS)
   {
      GcErrMsg(0, "gc_Stop");
   }
   LogMsg("GlobalCall stopped");
}

//*****************************************************************************
// Function: void CGcManager::ProcessEvents()
// Description: Process SRL events
// Return: void 
// Parameters: none 
//*****************************************************************************
void CGcManager::ProcessEvents()
{

   if (sr_waitevt(100) != -1)
   {
      METAEVENT l_metaEvent; 

      if (gc_GetMetaEvent(&l_metaEvent) != GC_SUCCESS)
      {
         GcErrMsg(0,"gc_GetMetaEvent");
         return;
      }

      long deviceLocator;
      if (l_metaEvent.flags & GCME_GC_EVENT)
      {
         deviceLocator = (long) l_metaEvent.usrattr;
      }
      else
      {
         deviceLocator = reinterpret_cast<long>(sr_getUserContext());
      }

      if (deviceLocator == 0)
      {
         if (sr_getparm(l_metaEvent.evtdev, SR_USERCONTEXT, (void*) &deviceLocator) == -1)
         {
            ErrMsg(l_metaEvent.evtdev, "sr_getparm");
            return;
         }
      }

      if (deviceLocator == 0)
      {
         SystemDeviceMap::iterator pos;
         if ((pos = m_systemDevices.find(l_metaEvent.evtdev)) != m_systemDevices.end())
         {
            deviceLocator = l_metaEvent.evtdev;
         }
      }

      if (deviceLocator!=0)
      {
         // Make sure the context has been determined
         SystemDeviceMap::iterator devPos;
         if ((devPos = m_systemDevices.find(deviceLocator)) == m_systemDevices.end())
         {
            if ((devPos = m_systemDevices.find(l_metaEvent.evtdev)) == m_systemDevices.end())
            {
               ErrMsg(l_metaEvent.evtdev, "bad contect - no context found for event: " + AsString(l_metaEvent.evttype) 
                      + " dev:" + AsString(l_metaEvent.evtdev) 
                      + "locator:" + AsHexString(deviceLocator));
               return;
            }
         }

         ADevice* device = ( dynamic_cast<ADevice*>(devPos->second));

         if (device)
         {
            device->ProcessEvent(l_metaEvent);
         }
         else
         {
            ErrMsg(l_metaEvent.evtdev, "invalid device");
         }
      }
      else
      {
         LogMsg(l_metaEvent.evtdev, "deviceLocator not registered");
      }
   }
   m_mmScheduler.Run();
   m_featureScheduler.Run();
}

//*****************************************************************************
// Function: void CGcManager::AddOpenDevice(ADevice *a_device)
// Description: Add a device to the open devices list
// Return: void 
// Parameters: ADevice *a_device 
//*****************************************************************************
void CGcManager::AddOpenDevice(ADevice* a_device)
{
   m_openDevices.insert(a_device);
}

//*****************************************************************************
// Function: void CGcManager::RemoveOpenDevice(ADevice *a_device)
// Description: Remove a device from the open devices list
// Return: void 
// Parameters: ADevice *a_device 
//*****************************************************************************
void CGcManager::RemoveOpenDevice(ADevice* a_device)
{
   m_openDevices.erase(a_device);
}

//*****************************************************************************
// Function: bool CGcManager::OpenDevicesExist()
// Description: Answer if open devices exist
// Return: bool 
// Parameters: none 
//*****************************************************************************
bool CGcManager::OpenDevicesExist()
{
   return !m_openDevices.empty();
}

//*****************************************************************************
// Function: void CGcManager::AddActiveSipSession(CSipSession *a_sipSession)
// Description: Add a SIP session to the active SIP session list
// Return: void 
// Parameters: CSipSession *a_sipSession 
//*****************************************************************************
void CGcManager::AddActiveSipSession(CSipSession* a_sipSession)
{
   m_activeSipSessions.insert(a_sipSession);
}

//*****************************************************************************
// Function: void CGcManager::RemoveActiveSipSession(CSipSession *a_sipSession)
// Description: Remove a SIP session from the active SIP session list
// Return: void 
// Parameters: CSipSession *a_device 
//*****************************************************************************
void CGcManager::RemoveActiveSipSession(CSipSession* a_sipSession)
{
   m_activeSipSessions.erase(a_sipSession);
}

//*****************************************************************************
// Function: bool CGcManager::ActiveSipSessionsExist()
// Description: Answer if active SIP sessions exist
// Return: bool 
// Parameters: none 
//*****************************************************************************
bool CGcManager::ActiveSipSessionsExist()
{
   return !m_activeSipSessions.empty();
}

