/**
* @file CGcSipCallState.cpp
* @brief SIP Call States - class implementations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include <string>
#include <iostream>
#include <map>

using std::string;
using std::cout;
using std::endl;

#include "CGcSipCallState.h"

CGcSipCallStates AGcSipCallState::m_callStates;

//*****************************************************************************
// Function: int AGcSipCallState::LogStateError(CGcSipDevice &a_device, const string &msg)
// Description: Log an invalid state for operation/event message
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             const string &msg 
//*****************************************************************************
int AGcSipCallState::LogStateError(CGcSipDevice& a_device, const string& msg)
{
   cout << a_device.DeviceName()
   << " " << Desc() << " is invalid call state for "
   << msg
   << endl;
   return -1;
}

//*****************************************************************************
// Function: CGcSipCallStates::CGcSipCallStates()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CGcSipCallStates::CGcSipCallStates()
{
   m_callStates[GcSipCallNullState] = new CGcSipCallNullState();
   m_callStates[GcSipCallOfferedState] = new CGcSipCallOfferedState();
   m_callStates[GcSipCallSendingProceedingState] = new CGcSipCallSendingProceedingState();
   m_callStates[GcSipCallOfferedProceedingState] = new CGcSipCallOfferedProceedingState();
   m_callStates[GcSipCallAcceptingCallState] = new CGcSipCallAcceptingCallState();
   m_callStates[GcSipCallAcceptedState] = new CGcSipCallAcceptedState();
   m_callStates[GcSipCallAnsweringCallState] = new CGcSipCallAnsweringCallState();
   m_callStates[GcSipCallWaitingFor200OkState] = new CGcSipCallWaitingFor200OkState();
   m_callStates[GcSipCallReceived200OkState] = new CGcSipCallReceived200OkState();
   m_callStates[GcSipCallConnectedState] = new CGcSipCallConnectedState();
   m_callStates[GcSipCallDisconnectedState] = new CGcSipCallDisconnectedState();
   m_callStates[GcSipCallDroppingCallState] = new CGcSipCallDroppingCallState();
   m_callStates[GcSipCallDroppedState] = new CGcSipCallDroppedState();
   m_callStates[GcSipCallReleasingCallState] = new CGcSipCallReleasingCallState();
}

//*****************************************************************************
// Function: int CGcSipCallNullState::GcWaitForCall(CGcSipDevice &a_device)
// Description: Wait for an inbound call
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipCallNullState::GcWaitForCall(CGcSipDevice& a_device) 
{
   return a_device.PrimGcWaitForCall();
}

//*****************************************************************************
// Function: int CGcSipCallNullState::ProcessNextCall(CGcSipDevice &a_device)
// Description: Handle the next call
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipCallNullState::ProcessNextCall(CGcSipDevice& a_device) 
{
   return a_device.PrimProcessNextCall();
}

//*****************************************************************************
// Function: int CGcSipCallNullState::OnGcCallOffered(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle GC call offered event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipCallNullState::OnGcCallOffered(CGcSipDevice& a_device, METAEVENT& a_metaEvent) 
{
   a_device.SetCallStateTo(CallState(GcSipCallOfferedState)); 
   return a_device.PrimOnGcCallOffered(a_metaEvent);
}

//*****************************************************************************
// Function: int AGcSipCallActiveState::OnGcCallDisconnected(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle GC call disconnected event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int AGcSipCallActiveState::OnGcCallDisconnected(CGcSipDevice& a_device, METAEVENT& a_metaEvent)  
{
   a_device.SetCallStateTo(CallState(GcSipCallDisconnectedState)); 
   return a_device.PrimOnGcCallDisconnected(a_metaEvent);
}

//*****************************************************************************
// Function: int AGcSipCallActiveState::GcDropCall(CGcSipDevice &a_device)
// Description: Drop the call
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int AGcSipCallActiveState::GcDropCall(CGcSipDevice& a_device) 
{
   a_device.SetCallStateTo(CallState(GcSipCallDroppingCallState)); 
   return a_device.PrimGcDropCall();
}

//*****************************************************************************
// Function: int CGcSipCallOfferedState::GcSendProceeding(CGcSipDevice &a_device)
// Description: Send call proceeding 
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipCallOfferedState::GcSendProceeding(CGcSipDevice& a_device) 
{
   a_device.SetCallStateTo(CallState(GcSipCallSendingProceedingState)); 
   return a_device.PrimGcSendProceeding();
}

//*****************************************************************************
// Function: int CGcSipCallSendingProceedingState::OnGcCallProceedingComplete(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle send call proceeding completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipCallSendingProceedingState::OnGcCallProceedingComplete(CGcSipDevice& a_device, METAEVENT& a_metaEvent)
{
   a_device.SetCallStateTo(CallState(GcSipCallOfferedProceedingState)); 
   return a_device.PrimOnGcCallProceedingComplete(a_metaEvent);
}

//*****************************************************************************
// Function: int CGcSipCallOfferedState::GcAcceptCall(CGcSipDevice &a_device)
// Description: Accept the call
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipCallOfferedState::GcAcceptCall(CGcSipDevice& a_device) 
{
   a_device.SetCallStateTo(CallState(GcSipCallAcceptingCallState)); 
   return a_device.PrimGcAcceptCall();
}

//*****************************************************************************
// Function: int CGcSipCallOfferedState::GcAnswerCall(CGcSipDevice &a_device)
// Description: Answer an offered call
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipCallOfferedState::GcAnswerCall(CGcSipDevice& a_device) 
{
   a_device.SetCallStateTo(CallState(GcSipCallAnsweringCallState)); 
   return a_device.PrimGcAnswerCall();
}

//*****************************************************************************
// Function: int CGcSipCallOfferedProceedingState::GcAcceptCall(CGcSipDevice &a_device)
// Description: Accept an offered call
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipCallOfferedProceedingState::GcAcceptCall(CGcSipDevice& a_device) 
{
   a_device.SetCallStateTo(CallState(GcSipCallAcceptingCallState)); 
   return a_device.PrimGcAcceptCall();
}

//*****************************************************************************
// Function: int CGcSipCallOfferedProceedingState::GcAnswerCall(CGcSipDevice &a_device)
// Description: Answer an offered and proceeding call
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipCallOfferedProceedingState::GcAnswerCall(CGcSipDevice& a_device) 
{
   a_device.SetCallStateTo(CallState(GcSipCallAnsweringCallState)); 
   return a_device.PrimGcAnswerCall();
}

//*****************************************************************************
// Function: int CGcSipCallAcceptingCallState::OnGcCallAccepted(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an accept call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipCallAcceptingCallState::OnGcCallAccepted(CGcSipDevice& a_device, METAEVENT& a_metaEvent)  
{
   a_device.SetCallStateTo(CallState(GcSipCallAcceptedState)); 
   return a_device.PrimOnGcCallAccepted(a_metaEvent);
}

//*****************************************************************************
// Function: int CGcSipCallAcceptedState::GcAnswerCall(CGcSipDevice &a_device)
// Description: Answer an accepted call
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipCallAcceptedState::GcAnswerCall(CGcSipDevice& a_device)  
{
   a_device.SetCallStateTo(CallState(GcSipCallAnsweringCallState)); 
   return a_device.PrimGcAnswerCall();
}

//*****************************************************************************
// Function: int CGcSipCallAnsweringCallState::OnGcCallAnswered(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an answer call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipCallAnsweringCallState::OnGcCallAnswered(CGcSipDevice& a_device, METAEVENT& a_metaEvent)  
{
   return a_device.PrimOnGcCallAnswered(a_metaEvent);
}

//*****************************************************************************
// Function: int CGcSipCallAnsweringCallState::OnGcSipAck(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a SIP ACK event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipCallAnsweringCallState::OnGcSipAck(CGcSipDevice& a_device, METAEVENT& a_metaEvent)
{
   a_device.SetCallStateTo(CallState(GcSipCallConnectedState)); 
   return a_device.PrimOnGcSipAckFromAnswer(a_metaEvent);
}

//*****************************************************************************
// Function: int CGcSipCallConnectedState::GcSendIFrameRequest(CGcSipDevice& a_device)
// Description: Handle a send IFrame request command
// Return: int 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipCallConnectedState::GcSendIFrameRequest(CGcSipDevice& a_device)
{
   return a_device.PrimGcSendIFrameRequest();
}

//*****************************************************************************
// Function: int CGcSipCallWaitingFor200OkState::OnGcSip200Ok(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a SIP 200 OK event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipCallWaitingFor200OkState::OnGcSip200Ok(CGcSipDevice& a_device, METAEVENT& a_metaEvent)  
{
   a_device.SetCallStateTo(CallState(GcSipCallReceived200OkState)); 
   int res = a_device.PrimOnGcSip200Ok(a_metaEvent);
   if (res == GC_SUCCESS)
   {
      res = a_device.GcSipAck();
   }
   return res;
}

//*****************************************************************************
// Function: int CGcSipCallReceived200OkState::GcSipAck(CGcSipDevice &a_device)
// Description: Handle a SIP ACK event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipCallReceived200OkState::GcSipAck(CGcSipDevice& a_device)  
{
   a_device.SetCallStateTo(CallState(GcSipCallSendingAckState)); 
   return a_device.PrimGcSipAck();
}

//*****************************************************************************
// Function: int CGcSipCallDisconnectedState::OnGcCallDisconnected(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle GC call disconnected event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipCallDisconnectedState::OnGcCallDisconnected(CGcSipDevice& a_device, METAEVENT& a_metaEvent)  
{
   return a_device.PrimOnGcCallDisconnected(a_metaEvent);
}

//*****************************************************************************
// Function: int CGcSipCallDisconnectedState::GcDropCall(CGcSipDevice &a_device)
// Description: Drop the call
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipCallDisconnectedState::GcDropCall(CGcSipDevice& a_device) 
{
   a_device.SetCallStateTo(CallState(GcSipCallDroppingCallState)); 
   return a_device.PrimGcDropCall();
}

//*****************************************************************************
// Function: int CGcSipCallDroppingCallState::OnGcCallDropped(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a drop call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipCallDroppingCallState::OnGcCallDropped(CGcSipDevice& a_device, METAEVENT& a_metaEvent)  
{
   a_device.SetCallStateTo(CallState(GcSipCallDroppedState)); 
   int res = a_device.PrimOnGcCallDropped(a_metaEvent);
   if (res == GC_SUCCESS)
   {
      res = a_device.GcReleaseCall();
   }
   return res;
}

//*****************************************************************************
// Function: int CGcSipCallDroppedState::GcReleaseCall(CGcSipDevice &a_device)
// Description: Release a dropped call
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipCallDroppedState::GcReleaseCall(CGcSipDevice& a_device) 
{
   a_device.SetCallStateTo(CallState(GcSipCallReleasingCallState)); 
   return a_device.PrimGcReleaseCall();
}

//*****************************************************************************
// Function: int CGcSipCallReleasingCallState::OnGcCallReleased(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a release call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipCallReleasingCallState::OnGcCallReleased(CGcSipDevice& a_device, METAEVENT& a_metaEvent)  
{
   a_device.SetCallStateTo(CallState(GcSipCallNullState)); 
   int res = a_device.PrimOnGcCallReleased(a_metaEvent);
   if (res == GC_SUCCESS)
   {
      res = a_device.ProcessNextCall();
   }
   return res;
}

