/**
* @file CGcSipCallState.h
* @brief SIP Call States -  class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CGcSipCallState_h
#define CGcSipCallState_h

#include "CGcSipDevice.h"

#include <map>

typedef enum 
{
   GcSipCallNullState = 0,
   GcSipCallOfferedState,
   GcSipCallSendingProceedingState,
   GcSipCallOfferedProceedingState,
   GcSipCallAcceptingCallState,
   GcSipCallAcceptedState,
   GcSipCallAnsweringCallState,
   GcSipCallDialingState,
   GcSipCallProceedingState,
   GcSipCallAlertingState,
   GcSipCallWaitingFor200OkState,
   GcSipCallReceived200OkState,
   GcSipCallSendingAckState,
   GcSipCallConnectedState,
   GcSipCallDisconnectedState,
   GcSipCallDroppingCallState,
   GcSipCallDroppedState,
   GcSipCallReleasingCallState
} GcSipCallState;

class AGcSipCallState;

//*****************************************************************************
// Class: ::CGcSipCallStates
// Description: Map of SIP call state objects
//*****************************************************************************
class CGcSipCallStates
{
public:
   CGcSipCallStates();
   ~CGcSipCallStates() {}
   AGcSipCallState* CallState(GcSipCallState callState) {return m_callStates[callState];}
private:
   std::map< GcSipCallState, AGcSipCallState*> m_callStates;
};

//*****************************************************************************
// Class: ::AGcSipCallState
// Description: Abstract GlobalCall SIP call state class
// Notes:      
//    This abstract class declares all functions for all call states as 
//    virtual and implements them as an error.
// 
//    Concrete subclasses implement only the functions that are valid for that
//    state.
// 
//    State classes are responsible only for calling primative functions on the 
//    context device object and changing the state of the context device object.
// 
//    As state classes have no non-static member variables, one set of call 
//    state objects are shared by all devices.
//*****************************************************************************
class AGcSipCallState
{
public:
   virtual const char* Desc() = 0;

   virtual int ProcessNextCall(CGcSipDevice& a_device) {return LogStateError(a_device, "ProcessNextCall");}

   virtual int GcWaitForCall(CGcSipDevice& a_device) {return LogStateError(a_device, "GcWaitForCall");}
   virtual int GcSipAck(CGcSipDevice& a_device) {return LogStateError(a_device, "GcSipAck");}
   virtual int GcSendProceeding(CGcSipDevice& a_device) {return LogStateError(a_device, "GcAcceptCall");}
   virtual int GcAcceptCall(CGcSipDevice& a_device) {return LogStateError(a_device, "GcAcceptCall");}
   virtual int GcAnswerCall(CGcSipDevice& a_device) {return LogStateError(a_device, "GcAnswerCall");}
   virtual int GcDropCall(CGcSipDevice& a_device) {return LogStateError(a_device, "GcDropCall");}
   virtual int GcReleaseCall(CGcSipDevice& a_device) {return LogStateError(a_device, "GcReleaseCall");}

   virtual int GcSendIFrameRequest(CGcSipDevice& a_device) {return LogStateError(a_device, "GcSendIFrameRequest");}

   virtual int OnGcCallDisconnected(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnGcCallDisconnected");}
   virtual int OnGcCallDropped(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnGcCallDropped");}
   virtual int OnGcCallReleased(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnGcCallReleased");}
   virtual int OnGcCallProceeding(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnGcCallProceeding");}
   virtual int OnGcCallProceedingComplete(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnGcCallProceedingComplete");}
   virtual int OnGcCallAlerting(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnGcCallAlerting");}
   virtual int OnGcSip200Ok(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnGcSip200Ok");}
   virtual int OnGcCallOffered(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnGcCallOffered");}
   virtual int OnGcCallAccepted(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnGcCallAccepted");}
   virtual int OnGcCallAnswered(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnGcCallAnswered");}
   virtual int OnGcSipAck(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnGcCallAnswered");}

   static AGcSipCallState* CallState(GcSipCallState callState) {return m_callStates.CallState(callState);}

private:
   int LogStateError(CGcSipDevice& a_device, const std::string& msg);
   static CGcSipCallStates m_callStates;
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallNullState
// Description: GlobalCall SIP call NULL state
//*****************************************************************************
class CGcSipCallNullState : public AGcSipCallState
{
public:
   const char* Desc() {return "null";}
   int ProcessNextCall(CGcSipDevice& a_device);
   int GcWaitForCall(CGcSipDevice& a_device);
   int OnGcCallOffered(CGcSipDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: AGcSipCallState::AGcSipCallActiveState
// Description: Abstract GlobalCall SIP call active state class to encapsulate
//    functiuonality for initiating call teardown
//*****************************************************************************
class AGcSipCallActiveState : public AGcSipCallState
{
public:
   virtual const char* Desc() = 0;
   int OnGcCallDisconnected(CGcSipDevice& a_device, METAEVENT& a_metaEvent);
   int GcDropCall(CGcSipDevice& a_device);
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallOfferedState
// Description: GlobalCall SIP call offered state
//*****************************************************************************
class CGcSipCallOfferedState : public AGcSipCallActiveState
{
public:
   const char* Desc() {return "call offered";}
   int GcSendProceeding(CGcSipDevice& a_device);
   int GcAcceptCall(CGcSipDevice& a_device);
   int GcAnswerCall(CGcSipDevice& a_device);
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallSendingProceedingState
// Description: GlobalCall SIP call proceeding state
//*****************************************************************************
class CGcSipCallSendingProceedingState : public AGcSipCallActiveState
{
public:
   const char* Desc() {return "sending proceeding";}
   int OnGcCallProceedingComplete(CGcSipDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallOfferedProceedingState
// Description: GlobalCall SIP call offered and proceeding state
//*****************************************************************************
class CGcSipCallOfferedProceedingState : public AGcSipCallActiveState
{
public:
   const char* Desc() {return "offered proceeding";}
   int GcAcceptCall(CGcSipDevice& a_device);
   int GcAnswerCall(CGcSipDevice& a_device);
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallAcceptingCallState
// Description: GlobalCall SIP call accepting state
//*****************************************************************************
class CGcSipCallAcceptingCallState : public AGcSipCallActiveState
{
public:
   const char* Desc() {return "accepting call";}
   int OnGcCallAccepted(CGcSipDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallAcceptedState
// Description: GlobalCall SIP call accepted state
//*****************************************************************************
class CGcSipCallAcceptedState : public AGcSipCallActiveState
{
public:
   const char* Desc() {return "call accepted";}
   int GcAnswerCall(CGcSipDevice& a_device);
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallAnsweringCallState
// Description: GlobalCall SIP call answering state
//*****************************************************************************
class CGcSipCallAnsweringCallState : public AGcSipCallActiveState
{
public:
   const char* Desc() {return "answering call";}
   int OnGcCallAnswered(CGcSipDevice& a_device, METAEVENT& a_metaEvent);
   int OnGcSipAck(CGcSipDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallWaitingFor200OkState
// Description: GlobalCall SIP call waiting for 200 OK state
//*****************************************************************************
class CGcSipCallWaitingFor200OkState : public AGcSipCallActiveState
{
public:
   virtual const char* Desc() {return "waiting for 200OK";}
   int OnGcSip200Ok(CGcSipDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallReceived200OkState
// Description: GlobalCall SIP call received 200 OK state
//*****************************************************************************
class CGcSipCallReceived200OkState : public AGcSipCallActiveState
{
public:
   virtual const char* Desc() {return "received 200OK";}
   int GcSipAck(CGcSipDevice& a_device);
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallConnectedState
// Description: GlobalCall SIP call connected state
//*****************************************************************************
class CGcSipCallConnectedState : public AGcSipCallActiveState
{
public:
   const char* Desc() {return "call connected";}
   int GcSendIFrameRequest(CGcSipDevice& a_device);
};

//*****************************************************************************
// Class: AGcSipCallState::AGcSipCallTeardownState
// Description: Abstract GlobalCall SIP call teardown state class
//*****************************************************************************
class AGcSipCallTeardownState : public AGcSipCallState
{
public:
   virtual const char* Desc() = 0;
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallDisconnectedState
// Description: GlobalCall SIP call disconnected state
//*****************************************************************************
class CGcSipCallDisconnectedState : public AGcSipCallTeardownState
{
public:
   const char* Desc() {return "call disconnected";}
   int OnGcCallDisconnected(CGcSipDevice& a_device, METAEVENT& a_metaEvent);
   int GcDropCall(CGcSipDevice& a_device);
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallDroppingCallState
// Description: GlobalCall SIP call dropping call state
//*****************************************************************************
class CGcSipCallDroppingCallState : public AGcSipCallTeardownState
{
public:
   const char* Desc() {return "dropping call";}
   int OnGcCallDropped(CGcSipDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallDroppedState
// Description: GlobalCall SIP call dropped state
//*****************************************************************************
class CGcSipCallDroppedState : public AGcSipCallTeardownState
{
public:
   const char* Desc() {return "call dropped";}
   int GcReleaseCall(CGcSipDevice& a_device);
};

//*****************************************************************************
// Class: AGcSipCallState::CGcSipCallReleasingCallState
// Description: GlobalCall SIP call releasing call state
//*****************************************************************************
class CGcSipCallReleasingCallState : public AGcSipCallTeardownState
{
public:
   const char* Desc()
   {
      return "releasing call";
   }
   int OnGcCallReleased(CGcSipDevice& a_device, METAEVENT& a_metaEvent);
};

#endif

