/**
* @file CGcSipDevice.cpp
* @brief GlobalCall SIP Device class implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#ifdef WIN32

   #include <fcntl.h>

#else

   #include <fcntl.h>
   #include <unistd.h>
   #include <errno.h>
   #include <gcpdkrt.h>
#endif

#include <dm3cc_parm.h>
#include <gcip.h>

#include "CGcSipDevice.h"
#include "CSipSession.h"
#include "CGcSipDeviceState.h"
#include "CGcSipCallState.h"
#include "LogUtil.h"
#include "CGcManager.h"
#include "CConference.h"

const char* CGcSipDevice::m_iFrameRequest = "<?xml version=\"1.0\" encoding=\"utf-8\" ?><media_control><vc_primitive><to_encoder><picture_fast_update></picture_fast_update></to_encoder></vc_primitive></media_control>";

// JM: TODO Verify SIP Authorization, Add parameters to config file.
// Information for digest authorization
#define REALM "johnsnet.com"
// If an identity is given, the username/password pair is associated with the given identity
// URI.  Identity must appear EXACTLY the same as in the From: field in the Header in the
// "401 Unauthorized" message rceived as a challenge.  This includes "sip" prefix and port number,
// if any.
#define IDENTITY1               "sip:1031@192.168.1.103:5060"
// A null identity string will authenticate the given username and password for any URI in the
// realm that registers or calls
#define NULLIDENTITY    ""
#define USERNAME1               "1031"
#define PASSWORD1               "password1031"



//*****************************************************************************
// Function: CGcSipDevice::CGcSipDevice(const std::string &a_deviceName, const std::string &a_sipRegistrar, const std::string &a_sipAddress)
// Description: Initializing constructor
// Return:  
// Parameters: const std::string &a_deviceName 
//             const std::string &a_sipRegistrar 
//             const std::string &a_sipAddress 
//*****************************************************************************
CGcSipDevice::CGcSipDevice(const std::string& a_deviceName, const std::string& a_sipRegistrar, const std::string& a_sipAddress)
   :m_deviceState(AGcSipDeviceState::DeviceState(GcSipDeviceClosedState))
   ,m_callState(AGcSipCallState::CallState(GcSipCallNullState))
   ,m_deviceName(a_deviceName)
   ,m_sipRegistrar(a_sipRegistrar)
   ,m_sipAddress(a_sipAddress)
   ,m_crn(0)
   ,m_dropCallCause(GC_NORMAL_CLEARING)
   ,m_isBlocked(true)
   ,m_waitForCallIssued(false)
   ,m_earlyGcUnblockedEvent(false)
   ,m_sipInfoType(0)
   ,m_sipInfoData(0)

{
}

//*****************************************************************************
// Function: void CGcSipDevice::SetDeviceStateTo(AGcSipDeviceState *a_state)
// Description: et the device state
// Return: void 
// Parameters: AGcSipDeviceState *a_state 
//*****************************************************************************
void CGcSipDevice::SetDeviceStateTo(AGcSipDeviceState* a_state)
{
   m_deviceState = a_state;
   LogMsg(DeviceName(), AsString("SIP Device State:") + m_deviceState->Desc());
}

//*****************************************************************************
// Function: void CGcSipDevice::SetCallStateTo(AGcSipCallState *a_state)
// Description: Set the call state
// Return: void 
// Parameters: AGcSipCallState *a_state 
//*****************************************************************************
void CGcSipDevice::SetCallStateTo(AGcSipCallState* a_state)
{
   m_callState = a_state;
   LogMsg(DeviceName(), AsString("CallState:") + m_callState->Desc());
}

//*****************************************************************************
// All device and call operations and event handling are delegated to the state
// object with the following functions
//*****************************************************************************

int CGcSipDevice::Open() {return DeviceState().GcOpen(*this);}
int CGcSipDevice::GcSipRegister() {return DeviceState().GcSipRegister(*this);}
int CGcSipDevice::GcSipDeregister() {return DeviceState().GcSipDeregister(*this);}
int CGcSipDevice::GcResetLineDevice() {return DeviceState().GcResetLineDevice(*this);}
int CGcSipDevice::Close() {return DeviceState().GcClose(*this);}

int CGcSipDevice::OnGcOpened(METAEVENT& a_metaEvent) {return DeviceState().OnGcOpened(*this, a_metaEvent);}
int CGcSipDevice::OnGcUnblocked(METAEVENT& a_metaEvent) {return DeviceState().OnGcUnblocked(*this, a_metaEvent);}
int CGcSipDevice::OnGcBlocked(METAEVENT& a_metaEvent) {return DeviceState().OnGcBlocked(*this, a_metaEvent);}
int CGcSipDevice::OnGcSipRegisterComplete(METAEVENT& a_metaEvent) {return DeviceState().OnGcSipRegisterComplete(*this, a_metaEvent);}
int CGcSipDevice::OnGcResetLineDevice(METAEVENT& a_metaEvent) {return DeviceState().OnGcResetLineDevice(*this, a_metaEvent);}
int CGcSipDevice::OnGcSetConfigData(METAEVENT& a_metaEvent) {return DeviceState().OnGcSetConfigData(*this, a_metaEvent);}

int CGcSipDevice::ProcessNextCall() {return DeviceState().ProcessNextCall(*this);}

int CGcSipDevice::GcWaitForCall() {return DeviceState().GcWaitForCall(*this);}
int CGcSipDevice::GcSendProceeding() {return DeviceState().GcSendProceeding(*this);}
int CGcSipDevice::GcAcceptCall() {return DeviceState().GcAcceptCall(*this);}
int CGcSipDevice::GcAnswerCall() {return DeviceState().GcAnswerCall(*this);}
int CGcSipDevice::GcSipAck() {return DeviceState().GcSipAck(*this);}
int CGcSipDevice::GcDropCall() {return DeviceState().GcDropCall(*this);}
int CGcSipDevice::GcReleaseCall() {return DeviceState().GcReleaseCall(*this);}

int CGcSipDevice::GcSendIFrameRequest() {return DeviceState().GcSendIFrameRequest(*this);}

int CGcSipDevice::OnGcCallDisconnected(METAEVENT& a_metaEvent) {return DeviceState().OnGcCallDisconnected(*this, a_metaEvent);}
int CGcSipDevice::OnGcCallDropped(METAEVENT& a_metaEvent) {return DeviceState().OnGcCallDropped(*this, a_metaEvent);}
int CGcSipDevice::OnGcCallReleased(METAEVENT& a_metaEvent) {return DeviceState().OnGcCallReleased(*this, a_metaEvent);}

int CGcSipDevice::OnGcCallProceeding(METAEVENT& a_metaEvent) {return DeviceState().OnGcCallProceeding(*this, a_metaEvent);}
int CGcSipDevice::OnGcCallProceedingComplete(METAEVENT& a_metaEvent) {return DeviceState().OnGcCallProceedingComplete(*this, a_metaEvent);}
int CGcSipDevice::OnGcCallAlerting(METAEVENT& a_metaEvent) {return DeviceState().OnGcCallAlerting(*this, a_metaEvent);}
int CGcSipDevice::OnGcSip200Ok(METAEVENT& a_metaEvent) {return DeviceState().OnGcSip200Ok(*this, a_metaEvent);}

int CGcSipDevice::OnGcCallOffered(METAEVENT& a_metaEvent) {return DeviceState().OnGcCallOffered(*this, a_metaEvent);}
int CGcSipDevice::OnGcCallAccepted(METAEVENT& a_metaEvent) {return DeviceState().OnGcCallAccepted(*this, a_metaEvent);}
int CGcSipDevice::OnGcCallAnswered(METAEVENT& a_metaEvent) {return DeviceState().OnGcCallAnswered(*this, a_metaEvent);}
int CGcSipDevice::OnGcSipAck(METAEVENT& a_metaEvent) {return DeviceState().OnGcSipAck(*this, a_metaEvent);}

//*****************************************************************************
// Function: int CGcSipDevice::PrimGcOpen()
// Description: Handle open GC device command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimGcOpen()
{
   int res = GC_SUCCESS;

   std::string gcOpenDeviceName;
   if (IsHMPBoardDevice())
   {
      gcOpenDeviceName = ":P_IP:N_" + DeviceName();
   }
   else
   {
      gcOpenDeviceName = ":P_SIP:N_" + DeviceName();
   }

   long int l_deviceHandle;
   if ((res = gc_OpenEx(  &l_deviceHandle,
                          const_cast<char*>(gcOpenDeviceName.c_str()),
                          EV_ASYNC,
                          (void*) ID())) != GC_SUCCESS)
   {
      GcErrMsg(0, DeviceName() + " gc_OpenEx " + AsString(gcOpenDeviceName));
   }
   else
   {
      Device(l_deviceHandle);
      CGcManager::RegisterDevice(Device(), this);
      LogMsg(Device(), "gc_OpenEx " + DeviceName());
   }

   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcOpened(METAEVENT &a_metaEvent)
// Description: Handle open GC device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcOpened(METAEVENT& a_metaEvent)
{
   GC_PARM_BLKP l_parmblk = 0;
   long l_requestId = 0;

   CGcManager::AddOpenDevice(this);

   if (IsHMPBoardDevice())
   {
      if (gc_util_insert_parm_ref(&l_parmblk,
                                  IPSET_CONFIG,
                                  IPPARM_REGISTER_SIP_HEADER,
                                  (unsigned char)strlen("Call-ID") + 1,
                                  (void*)"Call-ID") != GC_SUCCESS)
      {
         GcErrMsg(Device(), "gc_util_insert_parm_ref");
         return -1;
      }

      if (gc_util_insert_parm_ref(&l_parmblk,
                                  IPSET_CONFIG,
                                  IPPARM_REGISTER_SIP_HEADER,
                                  (unsigned char)strlen("To") + 1,
                                  (void*)"To") != GC_SUCCESS)
      {
         GcErrMsg(Device(), "gc_util_insert_parm_ref");
         return -1;
      }

      if (gc_util_insert_parm_ref(&l_parmblk,
                                  IPSET_CONFIG,
                                  IPPARM_REGISTER_SIP_HEADER,
                                  (unsigned char)strlen("From") + 1,
                                  (void*)"From") != GC_SUCCESS)
      {
         GcErrMsg(Device(), "gc_util_insert_parm_ref");
         return -1;
      }

      if (gc_util_insert_parm_ref(&l_parmblk,
                                  IPSET_CONFIG,
                                  IPPARM_REGISTER_SIP_HEADER,
                                  (unsigned char)strlen("Contact") + 1,
                                  (void*)"From") != GC_SUCCESS)
      {
         GcErrMsg(Device(), "gc_util_insert_parm_ref");
         return -1;
      }

      if (gc_util_insert_parm_ref(&l_parmblk,
                                  IPSET_CONFIG,
                                  IPPARM_REGISTER_SIP_HEADER,
                                  (unsigned char)strlen("Request-URI") + 1,
                                  (void*)"From") != GC_SUCCESS)
      {
         GcErrMsg(Device(), "gc_util_insert_parm_ref");
         return -1;
      }

      if (gc_SetConfigData(  GCTGT_CCLIB_NETIF,
                             Device(),
                             l_parmblk,
                             0,
                             GCUPDATE_IMMEDIATE,
                             &l_requestId,
                             EV_ASYNC) != GC_SUCCESS)
      {
         GcErrMsg(Device(), "gc_SetConfigData");
         return -1;
      }
      else
      {
         LogMsg(Device(), "gc_SetConfigData " + DeviceName());
      }
   }
   else
   {
      if (gc_util_insert_parm_val(&l_parmblk,
                                  GCSET_CALLEVENT_MSK,
                                  GCACT_ADDMSK,
                                  sizeof(unsigned long),
                                  GCEV_DIALING) != GC_SUCCESS)
      {
         GcErrMsg(Device(), "gc_util_insert_parm_val");
         return -1;
      }

      if (gc_util_insert_parm_val(&l_parmblk,
                                  GCSET_CALLEVENT_MSK,
                                  GCACT_ADDMSK,
                                  sizeof(unsigned long),
                                  GCEV_EXTENSION) != GC_SUCCESS)
      {
         GcErrMsg(Device(), "gc_util_insert_parm_val");
         return -1;
      }

      if (gc_util_insert_parm_val(&l_parmblk,
                                  GCSET_CALLEVENT_MSK,
                                  GCACT_ADDMSK,
                                  sizeof(unsigned long),
                                  GCEV_EXTENSIONCMPLT) < 0)
      {
         GcErrMsg(Device(), "gc_util_insert_parm_val");\
         return -1;
      }

      if (gc_util_insert_parm_val(&l_parmblk,
                                  GCSET_CALLEVENT_MSK,
                                  GCACT_ADDMSK,
                                  sizeof(unsigned long),
                                  GCMSK_SIP_ACK) != GC_SUCCESS)
      {
         GcErrMsg(Device(), "gc_util_insert_parm_val");
         return -1;
      }

      if (gc_util_insert_parm_val(&l_parmblk,
                                  GCSET_CALLEVENT_MSK,
                                  GCACT_ADDMSK,
                                  sizeof(unsigned long),
                                  GCMSK_200_OK) != GC_SUCCESS)
      {
         GcErrMsg(Device(), "gc_util_insert_parm_val");
         return -1;
      }

      GC_PARM_BLKP l_retblk = 0;
      if (gc_SetConfigData( GCTGT_GCLIB_CHAN,
                            Device(),
                            l_parmblk,
                            0,
                            GCUPDATE_IMMEDIATE,
                            &l_requestId,
                            EV_ASYNC) != GC_SUCCESS)
      {
         GcErrMsg(Device(), "gc_SetConfigData");
         return -1;
      }
      else
      {
         LogMsg(Device(), "gc_SetConfigData " + DeviceName());
      }
   }

   gc_util_delete_parm_blk(l_parmblk);

   return GC_SUCCESS;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcUnblocked(METAEVENT &a_metaEvent)
// Description: Handle unblocked event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcUnblocked(METAEVENT& a_metaEvent)
{
   GotEarlyGcUnblockedEvent(false);
   return GC_SUCCESS;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcBlocked(METAEVENT &a_metaEvent)
// Description: Handle blocked event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcBlocked(METAEVENT& a_metaEvent)
{
   return GC_SUCCESS;
}

//JM: TODO Verify SIP Auth
//=========================================================================
// NAME : int CGcSipDevice::GcSipSetAuthentication ()
// DESCRIPTION : Set up digest authentication before a SIP request is issued
// INPUT : (Various defines set at the top of the file)
// OUTPUT : None
// RETURNS : None
//=========================================================================
int CGcSipDevice::GcSipSetAuthentication()
{
        // Digest Authentication setup
        GC_PARM_BLKP pAuthParmBlock = 0;
        IP_AUTHENTICATION authentication;
        INIT_IP_AUTHENTICATION (&authentication);

        LogMsg(Device(), AsString("GcSipSetAuthentication()"));

        // Use 3 sets of values set as #defines
        authentication.realm = REALM;
        //authentication.identity = IDENTITY1;
        authentication.identity = NULLIDENTITY;
        authentication.username = USERNAME1;
        authentication.password = PASSWORD1;
        gc_util_insert_parm_ref(&pAuthParmBlock, IPSET_CONFIG,
                                                        IPPARM_AUTHENTICATION_CONFIGURE,
                                                        (unsigned char)(sizeof(IP_AUTHENTICATION)),
                                                        &authentication);

        // gc_SetAuthenticationInfo requires the board-level device handle from
        // gc_OpenEx()
        if (gc_SetAuthenticationInfo(GCTGT_CCLIB_NETIF, Device(), pAuthParmBlock))
        {
            GcErrMsg(Device(), "gc_SetAuthenticationInfo");
        }
        // Done with parm block - clean up
        gc_util_delete_parm_blk(pAuthParmBlock);

        return GC_SUCCESS;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimGcSipRegister()
// Description: Handle SIP register command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimGcSipRegister()
{
   GC_PARM_BLKP l_parmBlock = 0;
   int resc = GC_SUCCESS;

// SKS
// if(m_sipRegistrar.c_str()==NULL && m_sipRegistrar.c_str()=="0.0.0.0")
   if(AsString(m_sipRegistrar)=="0.0.0.0")
   {
      LogMsg(Device(), "PrimGcSipRegister() No SIP registrar server to register");	
      return resc;
   } else {
      LogMsg(Device(), AsString("PrimGcSipRegister() SIP registrar server to register ") + AsString(m_sipRegistrar));	
   }

   // JM: TODO Verify SIP Authentication
   // Get ready for Digest Authentication before registering
   //GcSipSetAuthentication();

   if (gc_util_insert_parm_val(&l_parmBlock,
                               GCSET_SERVREQ,
                               PARM_REQTYPE,
                               sizeof(char),
                               IP_REQTYPE_REGISTRATION) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(GCSET_SERVREQ-PARM_REQTYPE)");
      return -1;
   }

   if (gc_util_insert_parm_val(&l_parmBlock,
                               GCSET_SERVREQ,
                               PARM_ACK,
                               sizeof(char),
                               1) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(GCSET_SERVREQ-PARM_ACK)");
      return -1;
   }

   if (gc_util_insert_parm_val(&l_parmBlock,
                               IPSET_PROTOCOL,
                               IPPARM_PROTOCOL_BITMASK,
                               sizeof(char),
                               IP_PROTOCOL_SIP) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(IPSET_PROTOCOL-IPPARM_PROTOCOL_BITMASK)");
      return -1;
   }

   if (gc_util_insert_parm_val(&l_parmBlock,
                               IPSET_REG_INFO,
                               IPPARM_OPERATION_REGISTER,
                               sizeof(char),
                               IP_REG_SET_INFO) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(IPSET_REG_INFO-IPPARM_OPERATION_REGISTER)");
      return -1;
   }

   IP_REGISTER_ADDRESS l_registerAddress;
   strcpy(l_registerAddress.reg_server, m_sipRegistrar.c_str());        // set server address
   strcpy(l_registerAddress.reg_client,m_sipAddress.c_str());     // set alias for SIP
   l_registerAddress.max_hops = 70;
   l_registerAddress.time_to_live = 3600;

   LogMsg(AsString("SIP registrar:") + AsString(l_registerAddress.reg_server));
   LogMsg(AsString("SIP address:") + AsString(l_registerAddress.reg_client));

   if (gc_util_insert_parm_ref(&l_parmBlock,
                               IPSET_REG_INFO,
                               IPPARM_REG_ADDRESS,
                               (unsigned char)sizeof(IP_REGISTER_ADDRESS),
                               &l_registerAddress) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_util_insert_parm_ref(IPSET_REG_INFO-IPPARM_REG_ADDRESS)");
      return -1;
   }

   if (gc_util_insert_parm_ref(&l_parmBlock,
                               IPSET_LOCAL_ALIAS,
                               (unsigned short)IPPARM_ADDRESS_EMAIL,
                               (unsigned char)(m_sipAddress.size()+1),
                               (void *) m_sipAddress.c_str()) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_util_insert_parm_ref(IPSET_LOCAL_ALIAS-IPPARM_ADDRESS_EMAIL)");
      return -1;
   }

   unsigned long serviceID = 1;
   if (gc_ReqService(GCTGT_CCLIB_NETIF,
                     Device(),
                     &serviceID,
                     l_parmBlock,
                     0,
                     EV_ASYNC) != GC_SUCCESS)
   {
      gc_util_delete_parm_blk(l_parmBlock);
      GcErrMsg(Device(), "gc_ReqService");
      return -1;
   }

   gc_util_delete_parm_blk(l_parmBlock);
   LogMsg(Device(), "gc_ReqService");
   return 0;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimGcSipDeregister()
// Description: Handle SIP deregister command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimGcSipDeregister()
{
   GC_PARM_BLKP l_parmBlock = 0;
   int resc = GC_SUCCESS;

   if (gc_util_insert_parm_val(&l_parmBlock,
                               GCSET_SERVREQ,
                               PARM_REQTYPE,
                               sizeof(char),
                               IP_REQTYPE_REGISTRATION) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(GCSET_SERVREQ-PARM_REQTYPE)");
      return -1;
   }

   if (gc_util_insert_parm_val(&l_parmBlock,
                               GCSET_SERVREQ,
                               PARM_ACK,
                               sizeof(char),
                               1) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(GCSET_SERVREQ-PARM_ACK)");
      return -1;
   }

   if (gc_util_insert_parm_val(&l_parmBlock,
                               IPSET_PROTOCOL,
                               IPPARM_PROTOCOL_BITMASK,
                               sizeof(char),
                               IP_PROTOCOL_SIP) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(IPSET_PROTOCOL-IPPARM_PROTOCOL_BITMASK)");
      return -1;
   }

   if (gc_util_insert_parm_val(&l_parmBlock,
                               IPSET_REG_INFO,
                               IPPARM_OPERATION_DEREGISTER,
                               sizeof(char),
                               IP_REG_DELETE_ALL) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(IPSET_REG_INFO-IPPARM_OPERATION_REGISTER)");
      return -1;
   }

   unsigned long serviceID = 2;
   if (gc_ReqService(GCTGT_CCLIB_NETIF,
                     Device(),
                     &serviceID,
                     l_parmBlock,
                     0,
                     EV_ASYNC) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_ReqService");
      return -1;
   }

   LogMsg(Device(), "gc_ReqService");

   gc_util_delete_parm_blk(l_parmBlock);

   return 0;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcSipRegisterComplete(METAEVENT &a_metaEvent)
// Description: Handle SIP register completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcSipRegisterComplete(METAEVENT& a_metaEvent)
{
   GetRegisterationInfo(a_metaEvent);
   return GC_SUCCESS;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcSipDeregisterComplete(METAEVENT &a_metaEvent)
// Description: Handle SIP deregister completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcSipDeregisterComplete(METAEVENT& a_metaEvent)
{
   return GC_SUCCESS;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimGcResetLineDevice()
// Description: Handle reset GC line device command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimGcResetLineDevice()
{
   WaitForCallIssued(false);
   int res = gc_ResetLineDev(Device(), EV_ASYNC);
   if (res != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_ResetLineDev");
   }
   else
   {
      LogMsg(DeviceName(), "gc_ResetLineDev");
   }
   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcResetLineDevice(METAEVENT &a_metaEvent)
// Description: Handle reset GC line device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcResetLineDevice(METAEVENT& a_metaEvent)
{
   return GC_SUCCESS;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimGcClose()
// Description: Handle close GC device command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimGcClose()
{
   int res(0);
   if (Device() != -1)
   {
      res = gc_Close(Device());
      if (res != GC_SUCCESS)
      {
         GcErrMsg(Device(),"gc_Close");
      }
      else
      {
         LogMsg(DeviceName(), "gc_Close");
         CGcManager::RemoveOpenDevice(this);
      }
      Device(-1);
   }
   else
   {
      LogMsg(DeviceName(), "device already closed");
   }
   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimGcWaitForCall()
// Description: Handle GC wait for call command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimGcWaitForCall()
{
   int res = GC_SUCCESS;

   if (!WaitForCallIssued())
   {
      WaitForCallIssued(true);
      res = gc_WaitCall(  Device(),
                          0,
                          0,
                          -1,
                          EV_ASYNC);
      if (res != GC_SUCCESS)
      {
         GcErrMsg(Device(),"gc_WaitCall");
      }
      else
      {
         LogMsg(Device(),"gc_WaitCall");
      }
   }

   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcCallDisconnected(METAEVENT &a_metaEvent)
// Description: Handle GC call disconnected event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcCallDisconnected(METAEVENT& a_metaEvent)
{
   int res = GC_SUCCESS;
   m_sipSession->OnCallDisconnected();
   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimGcDropCall()
// Description: Handle GC drop call command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimGcDropCall()
{
   int res = gc_DropCall(CallReference(), DropCallCause(), EV_ASYNC);

   if (res != GC_SUCCESS)
   {
      GcErrMsg(Device(),"gc_DropCall");
   }
   else
   {
      LogMsg(Device(),"gc_DropCall");
   }

   return res;
}
//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcDropCall(METAEVENT &a_metaEvent)
// Description: Handle GC drop call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcDropCall(METAEVENT& a_metaEvent)
{
   int res = GC_SUCCESS;
   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimGcReleaseCall()
// Description: HGandle GC release call command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimGcReleaseCall()
{
   int res = gc_ReleaseCallEx(CallReference(), EV_ASYNC);

   if (res != GC_SUCCESS)
   {
      GcErrMsg(Device(),"gc_ReleaseCall");
   }
   else
   {
      LogMsg(Device(),"gc_ReleaseCall");
   }

   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcReleaseCall(METAEVENT &a_metaEvent)
// Description: Handle GC release call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcReleaseCall(METAEVENT& a_metaEvent)
{
   return GC_SUCCESS;
}


//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcCallProceedingComplete(METAEVENT &a_metaEvent)
// Description: Handle GC call proceeding event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcCallProceedingComplete(METAEVENT& a_metaEvent)
{
   m_sipSession->OnInboundCallSendProceedingComplete();
   return GC_SUCCESS;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcSip200Ok(METAEVENT &a_metaEvent)
// Description: Handle GC SIP call 200 OK event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcSip200Ok(METAEVENT& a_metaEvent)
{
   GetSipCallInfo(a_metaEvent);
   return GC_SUCCESS;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimGcSipAck()
// Description: Handle GC SIP call ACK
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimGcSipAck()
{
   int res = gc_SipAck(CallReference(), 0, EV_ASYNC);
   if (res != GC_SUCCESS)
   {
      GcErrMsg(Device(),"gc_SipAck");
   }
   else
   {
      LogMsg(Device(),"gc_SipAck");
   }

   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcCallDropped(METAEVENT &a_metaEvent)
// Description: Handle GC drop call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcCallDropped(METAEVENT& a_metaEvent)
{
   return GC_SUCCESS;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcCallReleased(METAEVENT &a_metaEvent)
// Description: Handle GC release call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcCallReleased(METAEVENT& a_metaEvent)
{
   int res = GC_SUCCESS;
   m_sipSession->OnCallReleased();
   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimGcSendProceeding()
// Description: Handle GC send call proceeding completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimGcSendProceeding()
{
   GC_CALLACK_BLK gcCallAckBlk;
   memset(&gcCallAckBlk, 0, sizeof(GC_CALLACK_BLK));
   gcCallAckBlk.type = GCACK_SERVICE_PROC;

   int res = gc_CallAck(CallReference(), &gcCallAckBlk, EV_ASYNC);
   if (res != GC_SUCCESS)
   {
      GcErrMsg(Device(),"gc_CallAck");
   }
   else
   {
      LogMsg(Device(),"gc_CallAck");
   }
   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimGcAcceptCall()
// Description: Handle GC accept call command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimGcAcceptCall()
{
   int res = gc_AcceptCall(CallReference() , 10, EV_ASYNC);
   if (res != GC_SUCCESS)
   {
      GcErrMsg(Device(),"gc_AcceptCall");
   }
   else
   {
      LogMsg(Device(),"gc_AcceptCall");
   }
   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcCallOffered(METAEVENT &a_metaEvent)
// Description: Handle GC call offered event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcCallOffered(METAEVENT& a_metaEvent)
{
   int res = GC_SUCCESS;
   CallReference(a_metaEvent.crn);
   GetSipCallInfo(a_metaEvent);
// SKS	
   GetConferenceIDFromURI();
   m_sipSession->OnInboundCallOffered();
   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimGcAnswerCall()
// Description: Handle GC answer call command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimGcAnswerCall()
{

   char sdpMsg[1024];
   int err = m_answerSdp.exportSDP(sdpMsg, 1024, false);
   if (err != 0)
   {
      ErrMsg(Device(), "exportSDP: " + AsString(err));
   }

   LogMsg( Device(), AsString("answer SDP\n") + sdpMsg);
   m_sipCallAnswerSdp = sdpMsg;

   GC_PARM_BLKP l_gcParmBlk = 0;

   if (gc_util_insert_parm_ref_ex(&l_gcParmBlk,
                                  IPSET_SDP,
                                  IPPARM_SDP_ANSWER,
                                  strlen(m_sipCallAnswerSdp.c_str()),
                                  const_cast<char*>(m_sipCallAnswerSdp.c_str())) != GC_SUCCESS)
   {
      GcErrMsg(Device(),"gc_util_insert_parm_ref_ex");
      return -1;
   }

   if (gc_SetUserInfo(GCTGT_GCLIB_CRN,
                      CallReference(),
                      l_gcParmBlk,
                      GC_NEXT_OUTBOUND_MSG) != GC_SUCCESS)
   {
      GcErrMsg(Device(),"gc_SetUserInfo");
      return -1;
   }

   gc_util_delete_parm_blk(l_gcParmBlk);

   int res = gc_AnswerCall(CallReference(), 0, EV_ASYNC);

   if (res != GC_SUCCESS)
   {
      GcErrMsg(Device(),"gc_AnswerCall");
   }
   else
   {
      LogMsg(Device(),"gc_AnswerCall");
   }
   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcCallAccepted(METAEVENT &a_metaEvent)
// Description: Handle GC accept call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcCallAccepted(METAEVENT& a_metaEvent)
{
   int res = GC_SUCCESS;
   GetSipCallInfo(a_metaEvent);
   m_sipSession->OnAcceptInboundCallComplete();
   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcCallAnswered(METAEVENT &a_metaEvent)
// Description: Handle GC answer call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcCallAnswered(METAEVENT& a_metaEvent)
{
   int res = GC_SUCCESS;
   GetSipCallInfo(a_metaEvent);
   return res;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimOnGcSipAckFromAnswer(METAEVENT &a_metaEvent)
// Description: Handle GC SIP call ACK from answer call event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::PrimOnGcSipAckFromAnswer(METAEVENT& a_metaEvent)
{
   int res = GC_SUCCESS;
   GetSipCallInfo(a_metaEvent);
   m_sipSession->OnAnswerInboundCallComplete();
   return res;
}

//*****************************************************************************
// Function: void CGcSipDevice::GcCallInfoMsg()
// Description: Send a SIP INFO message
// Return: void 
// Parameters: none 
//*****************************************************************************
void CGcSipDevice::GcCallInfoMsg()
{
}

//*****************************************************************************
// Function: int CGcSipDevice::OnGcCallInfo(METAEVENT &a_metaEvent)
// Description: Handle a GC SIP INFO message event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDevice::OnGcCallInfo(METAEVENT& a_metaEvent)
{

   GetSipCallInfo(a_metaEvent);
   if (strstr(m_sipInfoData,"<picture_fast_update></picture_fast_update>"))
   {
      LogMsg( Device(), "SIP INFO: FAST UPDATE");
      m_sipSession->OnIFrameRequest();
   }

   GC_PARM_BLKP l_parmBlock = 0;
   GC_PARM_BLKP l_returnParmBlock = 0;

   if (gc_util_insert_parm_val(&l_parmBlock,
                               IPSET_MSG_SIP,
                               IPPARM_MSGTYPE,
                               sizeof(int) ,
                               IP_MSGTYPE_SIP_INFO_OK) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(IPSET_MSG_SIP,IPPARM_MSGTYPE)");
      return -1;
   }

   if (gc_SetUserInfo(GCTGT_GCLIB_CRN, CallReference(), l_parmBlock, GC_SINGLECALL) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_SetUserInfo");
      return -1;
   }

   if (gc_Extension(   GCTGT_GCLIB_CRN,
                       CallReference(),
                       IPEXTID_SENDMSG,
                       l_parmBlock,
                       &l_returnParmBlock,
                       EV_ASYNC) != GC_SUCCESS)
   {
      GcErrMsg(Device(), "gc_Extension(IPEXTID_SENDMSG)");
      return -1;
   }
   gc_util_delete_parm_blk( l_parmBlock);

   return 0;
}

//*****************************************************************************
// Function: int CGcSipDevice::PrimProcessNextCall()
// Description: Handle a process next call command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CGcSipDevice::PrimProcessNextCall()
{
   GcWaitForCall();
   return 0;
}

//*****************************************************************************
// Function: void CGcSipDevice::GetSipCallInfo(METAEVENT &a_metaEvent)
// Description: Get the SIP call header and SDP data
// Return: void 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
void CGcSipDevice::GetSipCallInfo(METAEVENT& a_metaEvent)
{
   GC_PARM_BLK* l_parmBlock = static_cast<GC_PARM_BLK*>(a_metaEvent.extevtdatap);
   GC_PARM_DATA_EXT l_parm;
   INIT_GC_PARM_DATA_EXT(&l_parm);

   while ((gc_util_next_parm_ex(l_parmBlock, &l_parm)) == GC_SUCCESS)
   {
      switch (l_parm.set_ID)
      {
         case IPSET_SIP_MSGINFO:
#if 0 
// SKS enable when required
            switch (l_parm.parm_ID)
            {
               case IPPARM_SIP_HDR:
                  {
                     char* l_parmData = new char[l_parm.data_size+1];
                     strncpy(l_parmData,(char*)l_parm.pData, l_parm.data_size);
                     l_parmData[l_parm.data_size]='\0';
							if (strstr(l_parmData, "To:")) {
                        m_sipCallTo = l_parmData;
                        LogMsg(Device(), " - sip: " + m_sipCallTo);
                     }
							else if (strstr(l_parmData, "Call-ID:")) {
                        m_sipCallId = l_parmData;
                        LogMsg(Device(), " - sip: " + m_sipCallId);
                     } else if (strstr(l_parmData, "From:")) {
                        m_sipCallFrom = l_parmData;
                        LogMsg(Device(), " - sip: " + m_sipCallFrom);
                     } else if (strstr(l_parmData,"Contact:")) {
                        m_sipCallFrom = l_parmData;
                        LogMsg(Device(), " - sip: " + m_sipCallContact);
                     } else if (strstr(l_parmData,"Request-URI:")) {
                        m_sipCallRequest = l_parmData;
                        LogMsg(Device(), " - sip: " + m_sipCallRequest);
                     } else {
                        LogMsg(Device(), " - unhandled SIP header:" + std::string (l_parmData));
                     }
                     delete[] l_parmData;
                  }
                  break;
            }
#endif
            break;

         case IPSET_SDP:
            switch (l_parm.parm_ID)
            {
               case IPPARM_SDP_OFFER:
                  {
                     char* l_parmData = new char[l_parm.data_size+1];
                     strncpy(l_parmData,(char*)l_parm.pData, l_parm.data_size);
                     l_parmData[l_parm.data_size]='\0';
                     m_sipCallOfferSdp = l_parmData;
                     LogMsg(Device(), " - offer SDP:\n" + m_sipCallOfferSdp);
                     ParseSdp(m_offerSdp, l_parmData);
                     delete[] l_parmData;
                  }
                  break;

               case IPPARM_SDP_ANSWER:
                  {
                     char* l_parmData = new char[l_parm.data_size+1];
                     strncpy(l_parmData,(char*)l_parm.pData, l_parm.data_size);
                     l_parmData[l_parm.data_size]='\0';
                     m_sipCallAnswerSdp = l_parmData;
                     LogMsg(Device(), " - answer SDP:\n" + m_sipCallAnswerSdp);
                     ParseSdp(m_answerSdp, l_parmData);
                     delete[] l_parmData;
                  }
                  break;
            }
            break;

         case IPSET_MIME:
            switch (l_parm.parm_ID)
   			{
   			case IPPARM_MIME_PART:
   				{
                  GC_PARM_BLK* l_mimeParmBlock = (GC_PARM_BLK*)(*(unsigned int*)( l_parm.pData));
                  GC_PARM_DATA_EXT l_mimeParm;
                  INIT_GC_PARM_DATA_EXT(&l_mimeParm);
   					unsigned long l_mimeSize = 0;
   					unsigned long l_mimeTypeSize = 0;
   					while ( gc_util_next_parm_ex(l_mimeParmBlock, &l_mimeParm) == GC_SUCCESS)  
   					{
   						switch (l_mimeParm.parm_ID)
   						{
   						case IPPARM_MIME_PART_TYPE:
   							l_mimeTypeSize = l_mimeParm.data_size;

   							if (m_sipInfoType)
   							{
   								delete[] m_sipInfoType;
   							}
   							m_sipInfoType = new char[l_mimeTypeSize + 1];
   							memcpy(m_sipInfoType, (char*)(l_mimeParm.pData), l_mimeSize);
   							m_sipInfoType[l_mimeSize] = '\0';
   							break;
   
   						case IPPARM_MIME_PART_BODY_SIZE:
   							l_mimeSize = *(unsigned int*)(l_mimeParm.pData);
   							break;
   
   						case IPPARM_MIME_PART_BODY:
   							if (m_sipInfoData)
   							{
   								delete[] m_sipInfoData;
   							}
   							m_sipInfoData = new char[l_mimeSize+1];
   							if (m_sipInfoData != 0)
   							{
   								memcpy(m_sipInfoData, (char*)(*(unsigned int*)(l_mimeParm.pData)),l_mimeSize);
   								m_sipInfoData[l_mimeSize] = '\0';
   							}
   							break;
   						}
   					}
   				}
   				break;
   			}
   			break;
      }
   }
	
}

//*****************************************************************************
// Function: void CGcSipDevice::GetConferenceIDFromURI()
// Description: Get the Conference ID from the SIP Invite URI
// Return: void 
// Parameters: 
//*****************************************************************************
void CGcSipDevice::GetConferenceIDFromURI()
{
   LogMsg( Device(), "GetConferenceIDFromURI()");
   char *callTo=new char[255];
   gc_GetCallInfo(CallReference(),DESTINATION_ADDRESS,callTo);

   m_sipCallTo=callTo;
   std::string tempCnfID(m_sipCallTo,0,m_sipCallTo.find("@")); 

// SKS
   if( CConference::IsConfRoomExist(tempCnfID)==false ) {
      LogMsg( Device(), "WARNING Invalid ConfID: " + tempCnfID + " will be using default:" + CConference::GetDefaultConferenceID());
      m_sipSession->SetConferenceID(CConference::GetDefaultConferenceID());
   } else {
      m_sipSession->SetConferenceID(tempCnfID);
   }
   delete[] callTo;
}

//*****************************************************************************
// Function: void CGcSipDevice::ParseSdp(sdpSessionDescription &a_sdp, char *a_dataBuffer)
// Description: Parse SDP data
// Return: void 
// Parameters: sdpSessionDescription &a_sdp 
//             char *a_dataBuffer 
//*****************************************************************************
void CGcSipDevice::ParseSdp(sdpSessionDescription& a_sdp, char* a_dataBuffer)
{
   a_sdp.clear();

   // 2nd param is for debugging (true/false) 
   a_sdp.importSDP(a_dataBuffer, false);

   int l_mediaDescriptorCount = a_sdp.mediaDescriptionList()->numItem();
   for (int i=0; i<l_mediaDescriptorCount;i++)
   {
      sdpMediaDescription* l_mediaDescriptor = a_sdp.mediaDescriptionList()->getItem(i);
      sdpMedia* l_media = l_mediaDescriptor->media();
      int l_mediaFormatCount = l_media->getNumFormat();
//        LogMsg(Device(), "Media descriptor " + AsString(i+1) + " = " + l_media->getValue());
//        LogMsg(Device(), "Media descriptor " + AsString(i+1) + " contains " + AsString(l_mediaFormatCount) + " media format(s)");
      for (int j=0;j<l_mediaFormatCount;j++)
      {
//            LogMsg(Device(), "Media format for " + AsString(j+1) + " = " + l_media->getFormat(j));
//            LogMsg(Device(), "Media for " + AsString(j+1) + " = " + AsString(l_media->getMedia()));
//            LogMsg(Device(), "Media port for " + AsString(j+1) + " = " + AsString(l_media->getPort()));
         // Media format 34 is H.263, format 0 is G.711 PCMU
         if (!strcmp(l_media->getFormat(j), "34"))
         {
            //                                 m_remoteVideoRTPPort = pMedia->getPort();
            //                                 strcpy(m_remoteVideoIPAddress, pMD->connection()->getAddress());
//                LogMsg(Device(), "Video media IP address for " + AsString(i+1) + " = " + l_mediaDescriptor->connection()->getAddress());
         }
         else if (!strcmp(l_media->getFormat(j), "96"))
         {
            //                                 m_remoteAudioRTPPort = pMedia->getPort();
            //                                 strcpy(m_remoteAudioIPAddress, pMD->connection()->getAddress());
//                LogMsg(Device(), "Audio media IP address for " + AsString(i+1) + " = " + l_mediaDescriptor->connection()->getAddress());
         }
      }

      //Leave this here - may become interested in attributes
      sdpAttributeList* l_attributeList = l_mediaDescriptor->attributeList();
      int l_attributeCount = l_attributeList->numItem();
//        LogMsg(Device(), "Media descriptor " + AsString(i+1) + " contains " + AsString(l_attributeCount) + " attribute(s)");
      //                     LOG_ENTRY(m_GCSIPHandle, "Media descriptor #%d contains %d attribute(s)\n",i+1,NumAttribute);
      for (int l=0;l<l_attributeCount;l++)
      {
         sdpAttribute* l_attribute = l_attributeList->getItem(l);
//            LogMsg(Device(), "Attribute  " + AsString(l+1) + " = " + l_attribute->getValue());
         //LOG_ENTRY(m_GCSIPHandle, "Attribute #%d = %s\n",j+1,pAttribute->getValue());
      }
   }
}

//*****************************************************************************
// Function: void CGcSipDevice::GetRegisterationInfo(METAEVENT &a_metaEvent)
// Description: Get SIP registration info
// Return: void 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
void CGcSipDevice::GetRegisterationInfo(METAEVENT &a_metaEvent)
{
   GC_PARM_BLK *parmBlock = (GC_PARM_BLK*)(a_metaEvent.extevtdatap);;
   GC_PARM_DATA  *parm = 0;
   while ((parm = gc_util_next_parm(parmBlock, parm)) != 0)
   {
      switch (parm->set_ID)
      {
         case IPSET_PROTOCOL:
            switch (parm->parm_ID)
            {
               case IPPARM_PROTOCOL_BITMASK:
                  switch (parm->value_buf[0])
                  {
                     case IP_PROTOCOL_H323:
                        LogMsg(Device(), "IP_PROTOCOL_H323");
                        break;
                     case IP_PROTOCOL_SIP:
                        LogMsg(Device(), "IP_PROTOCOL_SIP");
                        break;
                     default:
                        LogMsg(Device(), "unknown IPPARM_PROTOCOL_BITMASK value:" + AsHexString(parm->value_buf[0]));
                  }
                  break;
               default:
                  LogMsg(Device(), "unknown IPSET_PROTOCOL parm_ID:" + AsHexString(parm->parm_ID));
            }
            break;

         case IPSET_REG_INFO:
            switch (parm->parm_ID)
            {
               case IPPARM_REG_STATUS:
                  switch (parm->value_buf[0])
                  {
                     case IP_REG_CONFIRMED:
                        LogMsg(Device(), "IP_REG_CONFIRMED");
                        break;
                     case IP_REG_REJECTED:
                        LogMsg(Device(), "IP_REG_REJECTED");
                        break;
                     default:
                        LogMsg(Device(), "unknown IPPARM_REG_STATUS value:" + AsHexString(parm->value_buf[0]));
                  }
                  break;
               case IPPARM_REG_SERVICEID:
                  {
                     int l_servId = parm->value_buf[0];
                     LogMsg(Device(), "IPPARM_REG_SERVICEID value:" + AsString(l_servId));
                  }
                  break;
               default:
                  LogMsg(Device(), "unknown IPSET_REG_INFO parm_ID:" + AsHexString(parm->parm_ID));
            }
            break;

         case IPSET_LOCAL_ALIAS:
            {
               std::string localAlias( (char*) &parm->value_buf, parm->value_size);
               LogMsg(Device(), "IPSET_LOCAL_ALIAS value:" + localAlias);
            }
            break;

         case IPSET_SIP_MSGINFO:
            {
               std::string msgInfo( (char*) &parm->value_buf, parm->value_size);
               LogMsg(Device(), "IPSET_SIP_MSGINFO value:" + msgInfo);
            }
            break;

         default:
            LogMsg(Device(), "\tGetRegistrationInfo: unknown set_ID: " + AsHexString(parm->set_ID));
      }
   }
}

//*****************************************************************************
// Function: void CGcSipDevice::ProcessEvent(METAEVENT &a_metaEvent)
// Description: Process an event
// Return: void 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
void CGcSipDevice::ProcessEvent(METAEVENT& a_metaEvent)
{
   switch (a_metaEvent.evttype)
   {
      case GCEV_OPENEX:
         LogMsg(DeviceName(), "GCEV_OPENEX gcDevice=" + AsString(":") + AsString(Device()));
         OnGcOpened(a_metaEvent);
         break;

      case GCEV_OPENEX_FAIL:
         LogMsg(Device(), "GCEV_OPENEX_FAIL");
         break; 

      case GCEV_BLOCKED: 
         LogMsg(Device(), "GCEV_BLOCKED");
         OnGcBlocked(a_metaEvent);
         break;

      case GCEV_UNBLOCKED: 
         LogMsg(DeviceName(), "GCEV_UNBLOCKED");
         OnGcUnblocked(a_metaEvent);
         break;

      case GCEV_SETCONFIGDATA:
         LogMsg(Device(), "GCEV_SETCONFIGDATA");
         OnGcSetConfigData(a_metaEvent);
         break;

      case GCEV_RESETLINEDEV:
         LogMsg(Device(), "GCEV_RESETLINEDEV");
         OnGcResetLineDevice(a_metaEvent);
         break;

      case GCEV_DISCONNECTED:
         LogMsg(Device(), "GCEV_DISCONNECTED");
         GcResultMsg(Device(), a_metaEvent);
         OnGcCallDisconnected(a_metaEvent);
         break; 

      case GCEV_DROPCALL:
         LogMsg(Device(), "GCEV_DROPCALL");
         OnGcCallDropped(a_metaEvent);
         break; 

      case GCEV_RELEASECALL:
         LogMsg(Device(), "GCEV_RELEASECALL");
         OnGcCallReleased(a_metaEvent);
         break; 

      case GCEV_DIALING:
         LogMsg(Device(), "GCEV_DIALING");
         break; 

      case GCEV_ALERTING:
         LogMsg(Device(), "GCEV_ALERTING");
         OnGcCallAlerting(a_metaEvent);
         break; 

      case GCEV_PROCEEDING:
         LogMsg(Device(), "GCEV_PROCEEDING");
         OnGcCallProceeding(a_metaEvent);
         break; 

      case GCEV_CALLPROC:
         LogMsg(Device(), "GCEV_CALLPROC");
         OnGcCallProceedingComplete(a_metaEvent);
         break; 

      case GCEV_CONNECTED:
         LogMsg(Device(), "GCEV_CONNECTED");
         break; 

      case GCEV_SIP_200OK:
         LogMsg(Device(), "GCEV_SIP_200OK");
         OnGcSip200Ok(a_metaEvent);
         break; 

      case GCEV_SIP_ACK_OK:
         LogMsg(Device(), "GCEV_SIP_ACK_OK");
         break; 

      case GCEV_OFFERED:
         LogMsg(Device(), "GCEV_OFFERED");
         OnGcCallOffered(a_metaEvent);
         break;

      case GCEV_ACCEPT:
         LogMsg(Device(), "GCEV_ACCEPT");
         OnGcCallAccepted(a_metaEvent);
         break;

      case GCEV_ANSWERED:
         LogMsg(Device(), "GCEV_ANSWERED");
         OnGcCallAnswered(a_metaEvent);
         break;

      case GCEV_SIP_ACK:
         LogMsg(Device(), "GCEV_SIP_ACK");
         OnGcSipAck(a_metaEvent);
         break;

      case GCEV_CALLSTATUS:
         LogMsg(Device(), "GCEV_CALLSTATUS");
         break;

      case GCEV_SETUP_ACK:
         LogMsg(Device(), "GCEV_SETUP_ACK");
         break;

      case GCEV_SERVICERESP:
         LogMsg(DeviceName(), "GCEV_SERVICERESP");
         OnGcSipRegisterComplete(a_metaEvent);
         break;

      case GCEV_EXTENSION:
//      LogMsg(DeviceName(), "GCEV_EXTENSION");
         break;

      case GCEV_EXTENSIONCMPLT:
//	  LogMsg(DeviceName(), "GCEV_EXTENSIONCMPLT");
         break;

      case GCEV_CALLINFO:
//     LogMsg(DeviceName(), "GCEV_CALLINFO");
         OnGcCallInfo(a_metaEvent);
         break;

      case GCEV_TASKFAIL:
         LogMsg(Device(), "GCEV_TASKFAIL");
         GcResultMsg(Device(), a_metaEvent);
         break;

      case GCEV_ERROR:
         LogMsg(Device(), "GCEV_ERROR");
         break;

      case GCEV_FATALERROR:
         LogMsg(Device(), "GCEV_FATALERROR");
         break;

      case APPEVENT_DROPCALL_COMMAND:
         LogMsg(Device(), "APPEVENT_DROPCALL_COMMAND");
         OnGcCallDisconnected(a_metaEvent);
         break;

      case APPEVENT_CLOSE_COMMAND:
         LogMsg(Device(), "APPEVENT_CLOSE_COMMAND");
         Close();
         break;

	  case GCEV_REQ_MODIFY_CALL:
         LogMsg(Device(), "GCEV_REQ_MODIFY_CALL - Not Handled !!!");
		 break;

      default:
         LogMsg(Device(), "unhandled GC event:" + AsHexString(a_metaEvent.evttype));
   }
}

int CGcSipDevice::PrimGcSendIFrameRequest()
{
   GC_PARM_BLKP l_gcParmBlk_mime = 0;
   GC_PARM_BLKP l_gcParmBlk_mime1 = 0;
   GC_PARM_BLKP l_gcParmBlk_info = 0;
   char* l_bodyType = "Content-Type:application/media_control+xml"; // specify the body type

   LogMsg(DeviceName(), "GcSendIFrameRequest");
   if ( gc_util_insert_parm_ref(&l_gcParmBlk_mime,
                                IPSET_MIME,
                                IPPARM_MIME_PART_TYPE,
                                (unsigned char)(strlen(l_bodyType) + 1),
                                l_bodyType) < 0 )
   {
      GcErrMsg(Device(), "gc_util_insert_parm_ref(IPSET_MIME,IPPARM_MIME_PART_TYPE)");
      return -1;
   }

   // insert the body size
   if ( gc_util_insert_parm_val(&l_gcParmBlk_mime,
                                IPSET_MIME,
                                IPPARM_MIME_PART_BODY_SIZE,
                                sizeof(unsigned long),
                                strlen(m_iFrameRequest)) < 0 )
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(IPSET_MIME,IPPARM_MIME_PART_BODY_SIZE)");
      return -1;
   }

   // insert the body
   if ( gc_util_insert_parm_val(&l_gcParmBlk_mime,
                                IPSET_MIME,
                                IPPARM_MIME_PART_BODY,
                                sizeof(unsigned long),
                                (unsigned long)(m_iFrameRequest)) < 0 )
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(IPSET_MIME,IPPARM_MIME_PART_BODY)");
      return -1;
   }

   // insert the list of parmBlks into the top level parmBlk
   if ( gc_util_insert_parm_val(&l_gcParmBlk_mime1,
                                IPSET_MIME,
                                IPPARM_MIME_PART,
                                sizeof(unsigned long),
                                (unsigned long)l_gcParmBlk_mime) < 0 )
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(IPSET_MIME,IPPARM_MIME_PART)");
      return -1;
   }

   // now set it on the device
   if ( gc_SetUserInfo(GCTGT_GCLIB_CRN,
                       CallReference(),
                       l_gcParmBlk_mime1,
                       GC_SINGLECALL) < 0 ) // for this call only
   {
      GcErrMsg(Device(), "gc_SetUserInfo()");
      return -1;
   }

   // insert the message type
   if ( gc_util_insert_parm_val(&l_gcParmBlk_info,
                                IPSET_MSG_SIP,
                                IPPARM_MSGTYPE,
                                sizeof(int),
                                IP_MSGTYPE_SIP_INFO) < 0 )
   {
      GcErrMsg(Device(), "gc_util_insert_parm_val(IPSET_MSG_SIP,IPPARM_MSGTYPE)");
      return -1;
   }

   if ( gc_Extension(GCTGT_GCLIB_CRN, CallReference(), IPEXTID_SENDMSG, 
                     l_gcParmBlk_info, NULL, EV_ASYNC) < 0 )
   {
      GcErrMsg(Device(), "gc_Extension()");
      return -1;
   }
   gc_util_delete_parm_blk(l_gcParmBlk_info);
   gc_util_delete_parm_blk(l_gcParmBlk_mime);

   return 0;

}
