/**
* @file CGcSipDeviceState.cpp
* @brief GlobalCall SIP Device States - class implementations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include <string>
#include <iostream>
#include <map>

#include "CGcSipDeviceState.h"
#include "LogUtil.h"

CGcSipDeviceStates AGcSipDeviceState::m_deviceStates;

//*****************************************************************************
// Function: int AGcSipDeviceState::LogStateError(CGcSipDevice &a_device, const string &msg)
// Description: Log an invalid state for operation/event message
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             const string &msg 
//*****************************************************************************
int AGcSipDeviceState::LogStateError(CGcSipDevice& a_device, const string& msg)
{
   std::cout << a_device.DeviceName()
   << " " << Desc() << " is invalid a_device state for "
   << msg
   << std::endl;
   return -1;
}

//*****************************************************************************
// Function: int AGcSipDeviceState::GcClose(CGcSipDevice &a_device)
// Description: Close a GC SIP device
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int AGcSipDeviceState::GcClose(CGcSipDevice& a_device) 
{
   int res = a_device.PrimGcClose();
   if (res == GC_SUCCESS)
   {
      a_device.SetDeviceStateTo(DeviceState(GcSipDeviceClosedState));
   }
   return res;
}

//*****************************************************************************
// Function: int AGcSipDeviceState::GcSipDeregister(CGcSipDevice &a_device)
// Description: Deregister from a SIP registrar
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int AGcSipDeviceState::GcSipDeregister(CGcSipDevice& a_device) 
{
   int res = a_device.PrimGcSipDeregister();
   if (res == GC_SUCCESS)
   {
      a_device.SetDeviceStateTo(DeviceState(GcSipDeviceSipDeregisteringState));
   }
   return res;
}

//*****************************************************************************
// Function: CGcSipDeviceStates::CGcSipDeviceStates()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CGcSipDeviceStates::CGcSipDeviceStates()
{
   m_deviceStates[GcSipDeviceClosedState] = new CGcSipDeviceClosedState();
   m_deviceStates[GcSipDeviceOpeningState] = new CGcSipDeviceOpeningState();
   m_deviceStates[GcSipDeviceSettingConfigDataState] = new CGcSipDeviceSettingConfigDataState();
   m_deviceStates[GcSipDeviceOpenedWaitingForUnblockedState] = new CGcSipDeviceOpenedWaitingForUnblockedState();
   m_deviceStates[GcSipDeviceSipRegisteringState] = new CGcSipDeviceSipRegisteringState();
   m_deviceStates[GcSipDeviceSipDeregisteringState] = new CGcSipDeviceSipDeregisteringState();
   m_deviceStates[GcSipDeviceResettingState] = new CGcSipDeviceResettingState();
   m_deviceStates[GcSipDeviceActiveState] = new CGcSipDeviceActiveState();
   m_deviceStates[GcSipDeviceBlockedState] = new CGcSipDeviceBlockedState();
}

//*****************************************************************************
// Function: int CGcSipDeviceBlockedState::OnGcUnblocked(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle GC unblocked event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceBlockedState::OnGcUnblocked(CGcSipDevice& a_device, METAEVENT& a_metaEvent) 
{
   a_device.SetDeviceStateTo(DeviceState(GcSipDeviceActiveState));
   int res = a_device.PrimOnGcUnblocked(a_metaEvent);
   if (res == GC_SUCCESS)
   {
      res = a_device.ProcessNextCall();
   }
   return res;
}

//*****************************************************************************
// Function: int CGcSipDeviceBlockedState::OnGcCallDisconnected(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle GC call disconnected event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceBlockedState::OnGcCallDisconnected(CGcSipDevice& a_device, METAEVENT& a_metaEvent)  
{
   return a_device.CallState().OnGcCallDisconnected(a_device, a_metaEvent);
}

//*****************************************************************************
// Function: int CGcSipDeviceBlockedState::GcWaitForCall(CGcSipDevice &a_device)
// Description: Wait for inbound calls
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipDeviceBlockedState::GcWaitForCall(CGcSipDevice& a_device)
{
   LogMsg(a_device.Device(), "GcWaitForCall ignored");
   return 0;
}

//*****************************************************************************
// Function: int CGcSipDeviceBlockedState::GcDropCall(CGcSipDevice &a_device)
// Description: Drop a call
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipDeviceBlockedState::GcDropCall(CGcSipDevice& a_device)
{
   return a_device.CallState().GcDropCall(a_device);
}

//*****************************************************************************
// Function: int CGcSipDeviceBlockedState::GcReleaseCall(CGcSipDevice &a_device)
// Description: Release a call
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipDeviceBlockedState::GcReleaseCall(CGcSipDevice& a_device)
{
   return a_device.CallState().GcReleaseCall(a_device);
}

//*****************************************************************************
// Function: int CGcSipDeviceBlockedState::OnGcCallDropped(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a drop call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceBlockedState::OnGcCallDropped(CGcSipDevice& a_device, METAEVENT& a_metaEvent)
{
   return a_device.CallState().OnGcCallDropped(a_device, a_metaEvent);
}

//*****************************************************************************
// Function: int CGcSipDeviceBlockedState::OnGcCallReleased(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a release call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceBlockedState::OnGcCallReleased(CGcSipDevice& a_device, METAEVENT& a_metaEvent)
{
   return a_device.CallState().OnGcCallReleased(a_device, a_metaEvent);
}

//*****************************************************************************
// Function: int AGcSipDeviceUnblockedState::OnGcBlocked(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a GlobalCall blocked event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int AGcSipDeviceUnblockedState::OnGcBlocked(CGcSipDevice& a_device, METAEVENT& a_metaEvent) 
{
   a_device.SetDeviceStateTo(DeviceState(GcSipDeviceBlockedState));
   return a_device.PrimOnGcBlocked(a_metaEvent);
}

//*****************************************************************************
// Function: int CGcSipDeviceClosedState::GcOpen(CGcSipDevice &a_device)
// Description: Open a GlobalCall SIP device
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipDeviceClosedState::GcOpen(CGcSipDevice& a_device) 
{
   a_device.SetDeviceStateTo(DeviceState(GcSipDeviceOpeningState));
   return a_device.PrimGcOpen();
}

//*****************************************************************************
// Function: int CGcSipDeviceOpeningState::OnGcOpened(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a GC open completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceOpeningState::OnGcOpened(CGcSipDevice& a_device, METAEVENT& a_metaEvent) 
{
   a_device.SetDeviceStateTo(DeviceState(GcSipDeviceSettingConfigDataState));
   int res = a_device.PrimOnGcOpened(a_metaEvent);
   if (res == GC_SUCCESS)
   {
      if (a_device.GotEarlyGcUnblockedEvent())
      {
         res = a_device.OnGcUnblocked(a_metaEvent);
      }
   }
   return res;
}

//*****************************************************************************
// Function: int CGcSipDeviceOpeningState::OnGcUnblocked(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a GC unblocked event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceOpeningState::OnGcUnblocked(CGcSipDevice& a_device, METAEVENT& a_metaEvent) 
{
   a_device.GotEarlyGcUnblockedEvent(true);
   return 0;
}

//*****************************************************************************
// Function: int CGcSipDeviceSettingConfigDataState::OnGcSetConfigData(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a set configuration data completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceSettingConfigDataState::OnGcSetConfigData(CGcSipDevice& a_device, METAEVENT& a_metaEvent) 
{
   a_device.SetDeviceStateTo(DeviceState(GcSipDeviceOpenedWaitingForUnblockedState));
   if (a_device.GotEarlyGcUnblockedEvent())
   {
      a_device.OnGcUnblocked(a_metaEvent);
   }
   return 0;
}

//*****************************************************************************
// Function: int CGcSipDeviceSettingConfigDataState::OnGcUnblocked(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a get configuration data completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceSettingConfigDataState::OnGcUnblocked(CGcSipDevice& a_device, METAEVENT& a_metaEvent) 
{
   a_device.GotEarlyGcUnblockedEvent(true);
   return 0;
}

//*****************************************************************************
// Function: int CGcSipDeviceOpenedWaitingForUnblockedState::OnGcSetConfigData(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a set configuration data completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceOpenedWaitingForUnblockedState::OnGcSetConfigData(CGcSipDevice& a_device, METAEVENT& a_metaEvent) 
{
   return 0; 
}

//*****************************************************************************
// Function: int CGcSipDeviceOpenedWaitingForUnblockedState::OnGcUnblocked(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a GlobalCall unblocked event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceOpenedWaitingForUnblockedState::OnGcUnblocked(CGcSipDevice& a_device, METAEVENT& a_metaEvent) 
{
   a_device.SetDeviceStateTo(DeviceState(GcSipDeviceActiveState));
   int res = a_device.PrimOnGcUnblocked(a_metaEvent);
   if (res == GC_SUCCESS)
   {
      if (a_device.IsHMPBoardDevice())
      {
         res = a_device.GcSipRegister();
      }
      else
      {
         res = a_device.GcResetLineDevice();
      }
   }
   return res;
}

//*****************************************************************************
// Function: int CGcSipDeviceActiveState::GcSipRegister(CGcSipDevice &a_device)
// Description: Register SIP device with the confugured SIP registrar
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipDeviceActiveState::GcSipRegister(CGcSipDevice& a_device)
{
   a_device.SetDeviceStateTo(DeviceState(GcSipDeviceSipRegisteringState));
   return a_device.PrimGcSipRegister();
}

//*****************************************************************************
// Function: int CGcSipDeviceSipRegisteringState::OnGcSipRegisterComplete(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle registrer completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceSipRegisteringState::OnGcSipRegisterComplete(CGcSipDevice& a_device, METAEVENT& a_metaEvent) 
{
   int res = a_device.PrimOnGcSipRegisterComplete(a_metaEvent);
   if (res == 0)
   {
      a_device.SetDeviceStateTo(DeviceState(GcSipDeviceActiveState)); 
   }
   return res;
}

//*****************************************************************************
// Function: int CGcSipDeviceSipDeregisteringState::OnGcSipDeregisterComplete(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a SIP registrar deregister completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceSipDeregisteringState::OnGcSipDeregisterComplete(CGcSipDevice& a_device, METAEVENT& a_metaEvent) 
{
   int res = a_device.PrimOnGcSipDeregisterComplete(a_metaEvent);
   if (res == 0)
   {
      GcClose(a_device); 
   }
   return res;
}

//*****************************************************************************
// Function: int CGcSipDeviceActiveState::GcResetLineDevice(CGcSipDevice &a_device)
// Description: Reset a GlobalCall line device
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//*****************************************************************************
int CGcSipDeviceActiveState::GcResetLineDevice(CGcSipDevice& a_device)
{
   a_device.SetDeviceStateTo(DeviceState(GcSipDeviceResettingState));
   return a_device.PrimGcResetLineDevice();
}

//*****************************************************************************
// Function: int CGcSipDeviceResettingState::OnGcResetLineDevice(CGcSipDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a reset line device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CGcSipDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CGcSipDeviceResettingState::OnGcResetLineDevice(CGcSipDevice& a_device, METAEVENT& a_metaEvent) 
{
   a_device.SetDeviceStateTo(DeviceState(GcSipDeviceActiveState)); 
   int res = a_device.PrimOnGcResetLineDevice(a_metaEvent);
   if (res == GC_SUCCESS)
   {
      res = a_device.ProcessNextCall();
   }
   return res;
}

//*****************************************************************************
//
// The following call related functions and events are delegated to the call state
// 
//*****************************************************************************

int CGcSipDeviceActiveState::ProcessNextCall(CGcSipDevice& a_device) {return a_device.CallState().ProcessNextCall(a_device);}
int CGcSipDeviceActiveState::GcWaitForCall(CGcSipDevice& a_device) {return a_device.CallState().GcWaitForCall(a_device);}
int CGcSipDeviceActiveState::GcSendProceeding(CGcSipDevice& a_device) {return a_device.CallState().GcSendProceeding(a_device);}
int CGcSipDeviceActiveState::GcAcceptCall(CGcSipDevice& a_device) {return a_device.CallState().GcAcceptCall(a_device);}
int CGcSipDeviceActiveState::GcAnswerCall(CGcSipDevice& a_device) {return a_device.CallState().GcAnswerCall(a_device);}
int CGcSipDeviceActiveState::GcDropCall(CGcSipDevice& a_device) {return a_device.CallState().GcDropCall(a_device);}
int CGcSipDeviceActiveState::GcReleaseCall(CGcSipDevice& a_device) {return a_device.CallState().GcReleaseCall(a_device);}
int CGcSipDeviceActiveState::GcSendIFrameRequest(CGcSipDevice& a_device) {return a_device.CallState().GcSendIFrameRequest(a_device);}
int CGcSipDeviceActiveState::OnGcCallDisconnected(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return a_device.CallState().OnGcCallDisconnected(a_device, a_metaEvent);}
int CGcSipDeviceActiveState::OnGcCallDropped(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return a_device.CallState().OnGcCallDropped(a_device, a_metaEvent);}
int CGcSipDeviceActiveState::OnGcCallReleased(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return a_device.CallState().OnGcCallReleased(a_device, a_metaEvent);}
int CGcSipDeviceActiveState::OnGcCallProceeding(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return a_device.CallState().OnGcCallProceeding(a_device, a_metaEvent);}
int CGcSipDeviceActiveState::OnGcCallProceedingComplete(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return a_device.CallState().OnGcCallProceedingComplete(a_device, a_metaEvent);}
int CGcSipDeviceActiveState::OnGcCallAlerting(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return a_device.CallState().OnGcCallAlerting(a_device, a_metaEvent);}
int CGcSipDeviceActiveState::OnGcCallOffered(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return a_device.CallState().OnGcCallOffered(a_device, a_metaEvent);}
int CGcSipDeviceActiveState::OnGcCallAccepted(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return a_device.CallState().OnGcCallAccepted(a_device, a_metaEvent);}
int CGcSipDeviceActiveState::OnGcCallAnswered(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return a_device.CallState().OnGcCallAnswered(a_device, a_metaEvent);}
int CGcSipDeviceActiveState::OnGcSipAck(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return a_device.CallState().OnGcSipAck(a_device, a_metaEvent);}
int CGcSipDeviceActiveState::OnGcSip200Ok(CGcSipDevice& a_device, METAEVENT& a_metaEvent) {return a_device.CallState().OnGcSip200Ok(a_device, a_metaEvent);}
int CGcSipDeviceActiveState::GcSipAck(CGcSipDevice& a_device) {return a_device.CallState().GcSipAck(a_device);}

