/**
* @file CGcSipDeviceState.h
* @brief GlobalCall SIP Device States - class declarations 
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CGcSipDeviceState_h
#define CGcSipDeviceState_h

#include <map>
#include <string>

#include "CGcSipDevice.h"
#include "CGcSipCallState.h"

typedef enum 
{
   GcSipDeviceClosedState = 0,
   GcSipDeviceOpeningState,
   GcSipDeviceSettingConfigDataState,
   GcSipDeviceOpenedWaitingForUnblockedState,
   GcSipDeviceSipRegisteringState,
   GcSipDeviceSipDeregisteringState,
   GcSipDeviceResettingState,
   GcSipDeviceActiveState,
   GcSipDeviceBlockedState
} GcSipDeviceState;

class AGcSipDeviceState;

//*****************************************************************************
// Class: ::CGcSipDeviceStates
// Description: Map of SIP device state objects
//*****************************************************************************
class CGcSipDeviceStates
{
public:
   CGcSipDeviceStates();
   ~CGcSipDeviceStates() {}
   AGcSipDeviceState* DeviceState(GcSipDeviceState deviceState) {return m_deviceStates[ deviceState];}

private:
   std::map< GcSipDeviceState, AGcSipDeviceState*> m_deviceStates;
};

//*****************************************************************************
// Class: ::AGcSipDeviceState
// Description: Abstract GlobalCall SIP device state class
// 
// Notes:      
// 
//    This abstract class declares all functions for all device states as 
//    virtual and implements them as an error.
// 
//    Concrete subclasses implement only the functions that are valid for that 
//    state.
// 
//    State classes are responsible only for calling primative functions on the 
//    context device object and changing the state of the context device object.
// 
//    As state classes have no non-static member variables, one set of device 
//    state objects are shared by all devices.
//*****************************************************************************
class AGcSipDeviceState
{
public:
   virtual const char* Desc() = 0;

   virtual int GcOpen(CGcSipDevice& device) {return LogStateError(device, "GcOpen");}
   virtual int GcSipRegister(CGcSipDevice& device) {return LogStateError(device, "GcSipRegister");}
   virtual int GcSipDeregister(CGcSipDevice& device); 
   virtual int GcResetLineDevice(CGcSipDevice& device) {return LogStateError(device, "GcResetLineDevice");}
   virtual int GcClose(CGcSipDevice& device);
   virtual int ProcessNextCall(CGcSipDevice& device) {return LogStateError(device, "ProcessNextCall");}

   virtual int GcWaitForCall(CGcSipDevice& device) {return LogStateError(device, "GcWaitForCall");}
   virtual int GcSipAck(CGcSipDevice& device) {return LogStateError(device, "GcSipAck");}
   virtual int GcSendProceeding(CGcSipDevice& device) {return LogStateError(device, "GcSendProceeding");}
   virtual int GcAcceptCall(CGcSipDevice& device) {return LogStateError(device, "GcAcceptCall");}
   virtual int GcAnswerCall(CGcSipDevice& device) {return LogStateError(device, "GcAnswerCall");}
   virtual int GcDropCall(CGcSipDevice& device) {return LogStateError(device, "GcDropCall");}
   virtual int GcReleaseCall(CGcSipDevice& device) {return LogStateError(device, "GcReleaseCall");}

   virtual int GcSendIFrameRequest(CGcSipDevice& device) {return LogStateError(device, "GcSendIFrameRequest");}

   virtual int OnGcOpened(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcOpened");}
   virtual int OnGcUnblocked(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcUnblocked");}
   virtual int OnGcSipRegisterComplete(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcSipRegisterComplete");}
   virtual int OnGcSipDeregisterComplete(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcSipDeregisterComplete");}
   virtual int OnGcResetLineDevice(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcResetLineDevice");}
   virtual int OnGcBlocked(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcBlocked");}
   virtual int OnGcSetConfigData(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcSetConfigData");}

   virtual int OnGcCallDisconnected(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcCallDisconnected");}
   virtual int OnGcCallDropped(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcCallDropped");}
   virtual int OnGcCallReleased(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcCallReleased");}
   virtual int OnGcCallProceeding(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcCallProceeding");}
   virtual int OnGcCallProceedingComplete(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcCallProceedingComplete");}
   virtual int OnGcCallAlerting(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcCallAlerting");}    
   virtual int OnGcSip200Ok(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcSip200Ok");}    
   virtual int OnGcCallOffered(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcCallOffered");}  
   virtual int OnGcCallAccepted(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcCallAccepted");}    
   virtual int OnGcCallAnswered(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcCallAnswered");}
   virtual int OnGcSipAck(CGcSipDevice& device, METAEVENT& a_metaEvent) {return LogStateError(device, "OnGcSipAck");
   }

   static AGcSipDeviceState* DeviceState(GcSipDeviceState deviceState) {return m_deviceStates.DeviceState(deviceState);}

private:
   int LogStateError(CGcSipDevice& device, const std::string& msg);
   static CGcSipDeviceStates m_deviceStates;
};

//*****************************************************************************
// Class: ::CGcSipDeviceBlockedState
// Description: GlobalCall SIP device blocked state
//*****************************************************************************
class CGcSipDeviceBlockedState : public AGcSipDeviceState
{
public:
   virtual const char* Desc() {return "blocked";}
   int OnGcUnblocked(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcCallDisconnected(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int GcWaitForCall(CGcSipDevice& device);
   int GcDropCall(CGcSipDevice& device);
   int GcReleaseCall(CGcSipDevice& device);
   int OnGcCallDropped(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcCallReleased(CGcSipDevice& device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::AGcSipDeviceUnblockedState
// Description: GlobalCall SIP device unblocked state
//*****************************************************************************
class AGcSipDeviceUnblockedState : public AGcSipDeviceState
{
public:
   const char* Desc() {return "unblocked";}
   int OnGcBlocked(CGcSipDevice& device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CGcSipDeviceClosedState
// Description: GlobalCall SIP device closed state
//*****************************************************************************
class CGcSipDeviceClosedState : public AGcSipDeviceState
{
public:
   const char* Desc() {return "closed";}
   int GcOpen(CGcSipDevice& device);
};

//*****************************************************************************
// Class: ::CGcSipDeviceOpeningState
// Description: GlobalCall SIP device opening state
//*****************************************************************************
class CGcSipDeviceOpeningState : public AGcSipDeviceState
{
public:
   const char* Desc() {return "opening";}
   int OnGcOpened(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcUnblocked(CGcSipDevice& device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CGcSipDeviceSettingConfigDataState
// Description: GlobalCall SIP device setting configuration data state
//*****************************************************************************
class CGcSipDeviceSettingConfigDataState : public AGcSipDeviceState
{
public:
   const char* Desc() {return "config";}
   int OnGcSetConfigData(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcUnblocked(CGcSipDevice& device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CGcSipDeviceOpenedWaitingForUnblockedState
// Description: GlobalCall SIP device waiting for unblocked state
//*****************************************************************************
class CGcSipDeviceOpenedWaitingForUnblockedState : public AGcSipDeviceState
{
public:
   const char* Desc()
   {
      return "opened waiting for unblocked";
   }
   int OnGcSetConfigData(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcUnblocked(CGcSipDevice& device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CGcSipDeviceSipRegisteringState
// Description: GlobalCall SIP device registering state
//*****************************************************************************
class CGcSipDeviceSipRegisteringState : public AGcSipDeviceUnblockedState
{
public:
   const char* Desc()
   {
      return "registering with SIP server";
   }
   int OnGcSipRegisterComplete(CGcSipDevice& device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CGcSipDeviceSipDeregisteringState
// Description: GlobalCall SIP device deregistering state
//*****************************************************************************
class CGcSipDeviceSipDeregisteringState : public AGcSipDeviceUnblockedState
{
public:
   const char* Desc() {return "deregistering with SIP server";}
   int OnGcSipDeregisterComplete(CGcSipDevice& device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CGcSipDeviceResettingState
// Description: GlobalCall SIP device resetting state
//*****************************************************************************
class CGcSipDeviceResettingState : public AGcSipDeviceUnblockedState
{
public:
   const char* Desc() {return "resetting line device";}
   int OnGcResetLineDevice(CGcSipDevice& device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CGcSipDeviceActiveState
// Description: GlobalCall SIP device active state
//*****************************************************************************
class CGcSipDeviceActiveState : public AGcSipDeviceUnblockedState
{
public:
   const char* Desc() {return "active";}
   int ProcessNextCall(CGcSipDevice& device);
   int GcSipRegister(CGcSipDevice& device);
   int GcResetLineDevice(CGcSipDevice& device);
   int GcWaitForCall(CGcSipDevice& device);
   int GcSendProceeding(CGcSipDevice& device);
   int GcAcceptCall(CGcSipDevice& device);
   int GcAnswerCall(CGcSipDevice& device);
   int GcSipAck(CGcSipDevice& device);
   int GcDropCall(CGcSipDevice& device);
   int GcReleaseCall(CGcSipDevice& device);
   int GcSendIFrameRequest(CGcSipDevice& device);
   int OnGcCallDisconnected(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcCallDropped(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcCallReleased(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcCallProceeding(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcCallProceedingComplete(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcCallAlerting(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcSip200Ok(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcCallOffered(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcCallAccepted(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcCallAnswered(CGcSipDevice& device, METAEVENT& a_metaEvent);
   int OnGcSipAck(CGcSipDevice& device, METAEVENT& a_metaEvent);
};

#endif


