/**
* @file CIpmDevice.cpp
* @brief IP Media Device class implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "errno.h"
#include "devmgmt.h"
#include "port_connect.h"

#include "LogUtil.h"
#include "CIpmDevice.h"
#include "CSipSession.h"
#include "CIpmDeviceState.h"
#include "AEventHandler.h"
#include "config.h"
#include "CGcManager.h"

int g_signalIdEvents[] = {
	'0',
   '1',
   '2',
   '3',
   '4',
   '5',
   '6',
   '7',
   '8',
   '9',
   '*',
   '#',
   'A',
   'B',
   'C',
   'D'
};

int CIpmDevice::m_nextIpmDevice(1);

//*****************************************************************************
// Function: std::string CIpmDevice::GetNextIpmDeviceName()
// Description: Get the next available ipm device name
// Return: std::string 
// Parameters: none 
//*****************************************************************************
std::string CIpmDevice::GetNextIpmDeviceName()
{
   char l_devName[30];
   sprintf( l_devName, "ipmB1C%d", m_nextIpmDevice++);
   return l_devName;
}

//*****************************************************************************
// Function: CIpmDevice::CIpmDevice(const std::string &a_deviceName)
// Description: Initializing constructor
// Return:  
// Parameters: const std::string &a_deviceName 
//*****************************************************************************
CIpmDevice::CIpmDevice(const std::string& a_deviceName)
   :m_deviceState(AIpmDeviceState::DeviceState(IpmDeviceClosedState))
   ,m_deviceName(a_deviceName)
   ,m_timeslot(-1)
   ,m_voxDevH(-1)
   ,m_voxDevName("")
   ,m_observer(0)
   ,m_dtmfMode(DTMFMODE_INBAND)
{
   if (a_deviceName != "ipmB1")
   {
      int l_chan(0);
      if (a_deviceName.size() > 6)
      {
         sscanf(a_deviceName.c_str(), "ipmB1C%d", &l_chan);
         //JM
         if (m_dtmfMode == DTMFMODE_INBAND) {
            SetDxDeviceName(l_chan);
         }
      }
   }
}

//*****************************************************************************
// Function: void CIpmDevice::SetDxDeviceName()
// Description: Set the next available Dx device name to detect Digit
// Return: none
// Parameters: int ipm channel number
//*****************************************************************************
void CIpmDevice::SetDxDeviceName(int ipmCh)
{
   char l_devName[30];
   int brd, ch;

   brd=((ipmCh%4)==0?(ipmCh/4):(ipmCh/4)+1);
   ch=((ipmCh%4)==0?4:(ipmCh%4));

   sprintf( l_devName, "dxxxB%dC%d", brd,ch);
   m_voxDevName=l_devName;
   
   LogMsg(DeviceName(), "SetDxDeviceName()=" + AsString(m_voxDevName));

}
//*****************************************************************************
// Function: void CIpmDevice::SetDeviceStateTo(AIpmDeviceState *state)
// Description: Set the device state
// Return: void 
// Parameters: AIpmDeviceState *state 
//*****************************************************************************
void CIpmDevice::SetDeviceStateTo(AIpmDeviceState* state)
{
   m_deviceState = state;
   LogMsg(DeviceName(), AsString("IPM Device State:") + m_deviceState->Desc());
}

//*****************************************************************************
// All device and call operations and event handling are delegated to the state
// object with the following functions
//*****************************************************************************

int CIpmDevice::Open() {return DeviceState().IpmOpen(*this);}
int CIpmDevice::Close() {return DeviceState().IpmClose(*this);}
int CIpmDevice::IpmGetXmitSlot() {return DeviceState().IpmGetXmitSlot(*this);}
int CIpmDevice::IpmReserveResource(eDEV_RESOURCE_TYPE a_resourceType) {return DeviceState().IpmReserveResource(*this,a_resourceType);}
int CIpmDevice::IpmReleaseResource() {return DeviceState().IpmReleaseResource(*this);}
int CIpmDevice::IpmGetLocalMediaInfo() {return DeviceState().IpmGetLocalMediaInfo(*this);}
int CIpmDevice::IpmStartMedia( eIPM_DATA_DIRECTION a_dir) {return DeviceState().IpmStartMedia(*this, a_dir);}
int CIpmDevice::IpmStopMedia() {return DeviceState().IpmStopMedia(*this);}
int CIpmDevice::IpmGenerateIFrame() {return DeviceState().IpmGenerateIFrame(*this);}
int CIpmDevice::OnIpmOpened(METAEVENT& a_metaEvent) {return DeviceState().OnIpmOpened(*this, a_metaEvent);}
int CIpmDevice::OnIpmGetXmitSlotComplete(METAEVENT& a_metaEvent) {return DeviceState().OnIpmGetXmitSlotComplete(*this, a_metaEvent);}
int CIpmDevice::OnIpmReserveResourceComplete(METAEVENT& a_metaEvent) {return DeviceState().OnIpmReserveResourceComplete(*this, a_metaEvent);}
int CIpmDevice::OnIpmReleaseResourceComplete(METAEVENT& a_metaEvent) {return DeviceState().OnIpmReleaseResourceComplete(*this, a_metaEvent);}
int CIpmDevice::OnIpmGetLocalMediaInfoComplete(METAEVENT& a_metaEvent) {return DeviceState().OnIpmGetLocalMediaInfoComplete(*this, a_metaEvent);}
int CIpmDevice::OnIpmStartMediaComplete(METAEVENT& a_metaEvent) {return DeviceState().OnIpmStartMediaComplete(*this, a_metaEvent);}
int CIpmDevice::OnIpmStopMediaComplete(METAEVENT& a_metaEvent) {return DeviceState().OnIpmStopMediaComplete(*this, a_metaEvent);}
int CIpmDevice::OnIpmEvError(METAEVENT& a_metaEvent) {return DeviceState().OnIpmEvError(*this, a_metaEvent);}

//*****************************************************************************
// Function: int CIpmDevice::PrimIpmOpen()
// Description: Handle an open IPM device command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CIpmDevice::PrimIpmOpen()
{
   int res = ipm_Open(DeviceName().c_str(), 0, EV_ASYNC);

   if (res == -1)
   {
      //ErrMsg(0, DeviceName() + AsString(" ipm_Open"));
      LogMsg(DeviceName(), AsString("ERROR: ipm_Open failed"));
   }
   else
   {
      CGcManager::RegisterDevice(res, this);
      Device(res);
      long deviceLocator = ID();
      LogMsg(DeviceName(), "ipm_Open(" + AsString(DeviceName().c_str()) +  "):" + AsString(res));
      res = sr_setparm(res, SR_USERCONTEXT, &deviceLocator);
      if (res == -1)
      {
         ErrMsg(0, DeviceName() + " sr_setparm(SR_USERCONTEXT)");
      }
      else
      {
// SKS
         LogMsg(DeviceName(), "sr_setparm(SR_USERCONTEXT)");
    
         if (m_dtmfMode == DTMFMODE_INBAND)
         {
            LogMsg(DeviceName(), "Dtmfmode = DTMFMODE_INBAND");
            LogMsg(DeviceName(), "Opening dx device " + AsString(m_voxDevName.c_str()) + "...");
            m_voxDevH = dx_open(m_voxDevName.c_str(), 0);
            if (m_voxDevH == -1)
            {
               res = -1;
               //ErrMsg(Device(), "dx_open");
               LogMsg(DeviceName(), "Error: dx_open Failed for " + AsString(m_voxDevName.c_str()));
            }
            else
            {
               LogMsg(m_voxDevName, "dx_open("+ AsString(m_voxDevName.c_str()) + "):" + AsString(m_voxDevH));
               res = sr_setparm(m_voxDevH, SR_USERCONTEXT, &deviceLocator);
               if (res == -1)
               {
                  ErrMsg(0, m_voxDevName + " sr_setparm(SR_USERCONTEXT)");
               }
               else
               {
#if 0
                  LogMsg(m_voxDevH, "sr_setparm(SR_USERCONTEXT)");
                  res = dx_setevtmsk(m_voxDevH, DM_DIGITS);
                  if (res == -1)
                  {
                     ErrMsg(m_voxDevH, "dx_setevtmsk");
                  }
                  else
                  {
                     LogMsg(m_voxDevH, "dx_setevtmsk");
                  }
#endif
               }
            }
         } //m_dtmfMode==INBAND
         else if (m_dtmfMode == DTMFMODE_RFC2833){ 
            LogMsg(DeviceName(), "Dtmfmode = DTMFMODE_RFC2833");
         }
         else if (m_dtmfMode == DTMFMODE_SIPINFO) {
            LogMsg(DeviceName(), "Dtmfmode = DTMFMODE_SIPINFO");
         }
      }
   }
   return res;
}

//*****************************************************************************
// Function: int CIpmDevice::GetDigits(int numDigits)
// Description: Handle an open IPM device completion event
// Return: int - 0 for success, otherwise error
// Parameters: number of digits
//*****************************************************************************
void CIpmDevice::GetDigits(int numDigits)
{

   if (m_dtmfMode == DTMFMODE_INBAND) {
      LogMsg(DeviceName(), "GetDigits(INBAND) numDigits=" + AsString(numDigits));
      
      dx_clrtpt(m_tpt, 1);
      m_tpt[0].tp_type    = IO_EOT;
      m_tpt[0].tp_termno  = DX_MAXDTMF;
      m_tpt[0].tp_length  = numDigits; 
      m_tpt[0].tp_flags   = TF_MAXDTMF;

      m_digits.dg_value[0] = 0;
 
      if (dx_clrdigbuf(m_voxDevH) == -1) {
         ErrMsg(m_voxDevH, "dx_clrdigbuf()");
      }

      dx_setdigtyp(m_voxDevH, D_DTMF);

      if ( dx_getdig(m_voxDevH, m_tpt, &m_digits, EV_ASYNC) < 0 ) {
         ErrMsg(m_voxDevH, "dx_getdig()");
      } else {
         LogMsg(m_voxDevH, "dx_getdig()");
      }
   }
   else if (m_dtmfMode == DTMFMODE_RFC2833){
      LogMsg(DeviceName(), "GetDigits(RFC2833) numDigits=" + AsString(numDigits));
   }
   else if (m_dtmfMode == DTMFMODE_SIPINFO) {
      LogMsg(DeviceName(), "GetDigits(SIPINFO) numDigits=" + AsString(numDigits));
   }
   else {
      ErrMsg(Device(), "GetDigits Error:unknown dmtfMode");
   }
}


//*****************************************************************************
// Function: int CIpmDevice::PrimOnIpmOpened(METAEVENT &a_metaEvent)
// Description: Handle an open IPM device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDevice::PrimOnIpmOpened(METAEVENT& a_metaEvent)
{
   CGcManager::AddOpenDevice(this);

   SetRfc2833PayloadTypes(DEFAULT_RFC2833_PAYLOAD_TYPE, DEFAULT_RFC2833_PAYLOAD_TYPE);

   if (m_dtmfMode == DTMFMODE_RFC2833 || m_dtmfMode == DTMFMODE_INBAND)
   {
      eIPM_EVENT l_events[1] ={EVT_TELEPHONY};

      int res = ipm_EnableEvents(Device(), l_events, 1, EV_ASYNC);
      if (res != 0)
      {
         ErrMsg(Device(),"ipm_EnableEvents(EVT_TELEPHONY)");
         return 0;
      }
      else
      {
         LogMsg(DeviceName(), "ipm_EnableEvents(EVT_TELEPHONY)");
      }
   }

#if 1
   IPM_QOS_THRESHOLD_INFO rtpQosThresholdInfo;

    rtpQosThresholdInfo.unCount = 1;
    rtpQosThresholdInfo.QoSThresholdData[0].eQoSType = QOSTYPE_RTPTIMEOUT;
    rtpQosThresholdInfo.QoSThresholdData[0].unTimeInterval = 1000;//1sec
    rtpQosThresholdInfo.QoSThresholdData[0].unDebounceOn = 0;
    rtpQosThresholdInfo.QoSThresholdData[0].unDebounceOff = 0;
    rtpQosThresholdInfo.QoSThresholdData[0].unFaultThreshold = 150; //15sec timeout
    rtpQosThresholdInfo.QoSThresholdData[0].unPercentSuccessThreshold = 0;
    rtpQosThresholdInfo.QoSThresholdData[0].unPercentFailThreshold = 0;

    if(ipm_SetQoSThreshold(Device(), &rtpQosThresholdInfo, EV_ASYNC) == -1)
    {
         ErrMsg(Device(),"ipm_SetQoSThreshold()");
    }
#endif
   return 0;
}

//*****************************************************************************
// Function: int CIpmDevice::SetRfc2833PayloadTypes(unsigned long a_txPayloadType, unsigned long a_rxPayloadType)
// Description: Set the RFC 2833 mode and payload types
// Return: int - 0 for success, otherwise error 
// Parameters: unsigned long a_txPayloadType 
//             unsigned long a_rxPayloadType 
//*****************************************************************************
int CIpmDevice::SetRfc2833PayloadTypes(unsigned long a_txPayloadType, unsigned long a_rxPayloadType)
{
   IPM_PARM_INFO parmInfo;
   unsigned long ulParmValue = DTMFXFERMODE_RFC2833;
   parmInfo.eParm = PARMCH_DTMFXFERMODE;
   parmInfo.pvParmValue = &ulParmValue;
   int res = ipm_SetParm(Device(), &parmInfo, EV_SYNC);
   if (res != 0)
   {
      ErrMsg(Device(),"ipm_SetParm(PARMCH_DTMFXFERMODE)");
      return -1;
   }
   else
   {
      LogMsg(DeviceName(), "ipm_SetParm(PARMCH_DTMFXFERMODE)");
   }

   ulParmValue = a_txPayloadType;
   parmInfo.eParm = PARMCH_RFC2833EVT_TX_PLT;
   parmInfo.pvParmValue = &ulParmValue;
   res = ipm_SetParm(Device(), &parmInfo, EV_SYNC);
   if (res != 0)
   {
      ErrMsg(Device(),"ipm_SetParm(PARMCH_RFC2833EVT_TX_PLT)");
      return -1;
   }
   else
   {
      LogMsg(DeviceName(), "ipm_SetParm(PARMCH_RFC2833EVT_TX_PLT):" + AsString(a_txPayloadType));
   }

   ulParmValue = a_rxPayloadType;
   parmInfo.eParm = PARMCH_RFC2833EVT_RX_PLT;
   parmInfo.pvParmValue = &ulParmValue;
   res = ipm_SetParm(Device(), &parmInfo, EV_SYNC);
   if (res != 0)
   {
      ErrMsg(Device(),"ipm_SetParm(PARMCH_RFC2833EVT_RX_PLT)");
      return -1;
   }
   else
   {
      LogMsg(DeviceName(), "ipm_SetParm(PARMCH_RFC2833EVT_RX_PLT):" + AsString(a_rxPayloadType));
   }
   return 0;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimIpmClose()
// Description: Handle an IPM close device command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CIpmDevice::PrimIpmClose()
{
	LogMsg(DeviceName(), "CIpmDevice::PrimIpmClose()");
   int res(0);
// SKS
   if (m_voxDevH != -1)
   {
		if (dx_unlistenEx(m_voxDevH, EV_ASYNC) == -1) {
         ErrMsg(m_voxDevH,"dx_unlistenEx");
		}
      if (dx_close(m_voxDevH) == -1)
      {
         ErrMsg(m_voxDevH,"dx_close");
      }
      else
      {
         LogMsg(m_voxDevName, "dx_close");
      }
      m_voxDevH = -1;
   }

   if (Device() != -1)
   {
      if (ipm_Stop(Device(), STOP_ALL, EV_SYNC)==-1)
      {
         ErrMsg(Device(),"ipm_Stop");
      }
      else
      {
         LogMsg(DeviceName(), "ipm_Stop");
      }
#if 1
      eIPM_EVENT qosEvents[2] ={ EVT_RTPTIMEOUT, EVT_TELEPHONY};
      ipm_DisableEvents(Device(), qosEvents, 2, EV_SYNC);
#endif

      INIT_DEV_RESOURCE_LIST(&m_devResourceList);
      //m_devResourceList.rsList[0] = g_ipmAudioCoderResource;
      m_devResourceList.rsList[0] = RESOURCE_IPM_ALL_AUDIO_CODERS;
      m_devResourceList.count = 1;

      if (dev_ReleaseResourceEx(Device(), &m_devResourceList, EV_SYNC) == -1)
      {
         ErrMsg(Device(),"dev_ReleaseResourceEx");
      }
      else
      {
         LogMsg(DeviceName(), "dev_ReleaseResourceEx");
      }

      res = ipm_Close(Device(), 0);
      if (res != GC_SUCCESS)
      {
         ErrMsg(Device(),"ipm_Close");
      }
      else
      {
         LogMsg(DeviceName(), "ipm_Close");
         CGcManager::RemoveOpenDevice(this);
      }
      Device(-1);
   }
   else
   {
      LogMsg(DeviceName(), "device already closed");
   }

   return res;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimIpmGetXmitSlot()
// Description: Handle an IPM device get xmit timeslot command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CIpmDevice::PrimIpmGetXmitSlot()
{
   int res = ipm_GetXmitSlot(Device(), 0, EV_ASYNC);

   if (res == -1)
   {
      ErrMsg(0, DeviceName() + " ipm_GetXmitSlot");
   }
   else
   {
      LogMsg(DeviceName(), "ipm_GetXmitSlot");
   }
   return res;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimOnIpmGetXmitSlotComplete(METAEVENT &a_metaEvent)
// Description: Handle an IPM device get xmit timeslot completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDevice::PrimOnIpmGetXmitSlotComplete(METAEVENT& a_metaEvent)
{
   SC_TSINFO* pTimeSlotInfo = static_cast<SC_TSINFO*>(a_metaEvent.evtdatap);
   m_timeslot = *(pTimeSlotInfo->sc_tsarrayp);
   LogMsg(DeviceName(), "timeslot=" + AsString(m_timeslot));

   SC_TSINFO l_tsInfo;
   l_tsInfo.sc_numts = 1;
   l_tsInfo.sc_tsarrayp = &m_timeslot;

   if (m_dtmfMode == DTMFMODE_INBAND)
   {
      int res = dx_listenEx(m_voxDevH, &l_tsInfo, EV_ASYNC);

      if (res == -1)
      {
         ErrMsg(Device(), "dx_listenEx Fail, m_voxDevH=" + AsString(m_voxDevH));
      }
      else
      {
         LogMsg(DeviceName(), "dx_listenEx; m_voxDevH=" + AsString(m_voxDevH));
      }
   }
   else
   {
      LogMsg(DeviceName(), "Skip dx_listen()");
   }
   return 0;

}

//*****************************************************************************
// Function: int CIpmDevice::PrimIpmReserveResource(eDEV_RESOURCE_TYPE a_resourceType)
// Description: Handle an IPM device reserve resource command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CIpmDevice::PrimIpmReserveResource(eDEV_RESOURCE_TYPE a_resourceType)
{
   INIT_DEV_RESOURCE_LIST(&m_devResourceList);
   m_devResourceList.rsList[0] = a_resourceType;
   m_devResourceList.count = 1;

   int res = dev_ReserveResourceEx( Device(), &m_devResourceList, EV_ASYNC);
   if (res == -1)
   {
      ErrMsg(Device(), "error-dev_ReserveResourceEx");
   }
   else
   {
      LogMsg(DeviceName(), "dev_ReserveResourceEx");
   }

   return res;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimOnIpmReserveResourceComplete(METAEVENT &a_metaEvent)
// Description: Handle an IPM device reserve resource completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDevice::PrimOnIpmReserveResourceComplete(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_RESERVE_RESOURCE_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimIpmReleaseResource()
// Description: Handle an IPM device release resource command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CIpmDevice::PrimIpmReleaseResource()
{
   int res = dev_ReleaseResourceEx( Device(), &m_devResourceList, EV_ASYNC);
   if (res == -1)
   {
      ErrMsg(Device(), "dev_ReleaseResourceEx");
   }
   else
   {
      LogMsg(DeviceName(), "dev_ReleaseResourceEx");
   }

   return res;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimOnIpmReleaseResourceComplete(METAEVENT &a_metaEvent)
// Description: Handle an IPM device release resource completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDevice::PrimOnIpmReleaseResourceComplete(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_RELEASE_RESOURCE_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimIpmGetLocalMediaInfo()
// Description: Handle an IPM device get local media command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CIpmDevice::PrimIpmGetLocalMediaInfo()
{
   m_localMediaInfo.unCount = 2;
   m_localMediaInfo.MediaData[0].eMediaType = MEDIATYPE_AUDIO_LOCAL_RTP_INFO;
   m_localMediaInfo.MediaData[1].eMediaType = MEDIATYPE_VIDEO_LOCAL_RTP_INFO;
   int res = ipm_GetLocalMediaInfo( Device(), &m_localMediaInfo, EV_ASYNC);
   if (res == -1)
   {
      ErrMsg(Device(), "ipm_GetLocalMediaInfo");
   }
   else
   {
      LogMsg(DeviceName(), "ipm_GetLocalMediaInfo");
   }

   return res;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimOnIpmGetLocalMediaInfoComplete(METAEVENT &a_metaEvent)
// Description: Handle an IPM device get local media completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDevice::PrimOnIpmGetLocalMediaInfoComplete(METAEVENT& a_metaEvent)
{
   IPM_MEDIA_INFO* l_mediaInfo;
   l_mediaInfo = static_cast<IPM_MEDIA_INFO*>(a_metaEvent.evtdatap);
   for (int i=0; i<l_mediaInfo ->unCount; i++)
   {
      switch (l_mediaInfo->MediaData[i].eMediaType)
      {
         case MEDIATYPE_AUDIO_LOCAL_RTP_INFO:
            memcpy( &m_localAudioPortInfo, &l_mediaInfo->MediaData[i].mediaInfo.PortInfo, sizeof( IPM_PORT_INFO));
//         LogMsg(Device(), "local RTP port = MediaType=MEDIATYPE_AUDIO_LOCAL_RTP_INFO");
            LogMsg(DeviceName(), AsString(m_localAudioPortInfo.cIPAddress) + ":" + AsString(m_localAudioPortInfo.unPortId));
            break;
         case MEDIATYPE_AUDIO_LOCAL_RTCP_INFO:
//         LogMsg(Device(), "local RTPC port = MediaType=MEDIATYPE_AUDIO_LOCAL_RTCP_INFO");
            break;
         case MEDIATYPE_VIDEO_LOCAL_RTP_INFO:
            memcpy( &m_localVideoPortInfo, &l_mediaInfo->MediaData[i].mediaInfo.PortInfo, sizeof( IPM_PORT_INFO));
//         LogMsg(Device(), "local RTP port = MediaType=MEDIATYPE_VIDEO_LOCAL_RTP_INFO");
            LogMsg(DeviceName(), AsString(m_localVideoPortInfo.cIPAddress) + ":" + AsString(m_localVideoPortInfo.unPortId));
            break;
         case MEDIATYPE_VIDEO_LOCAL_RTCP_INFO:
//         LogMsg(Device(), "local RTP port = MediaType=MEDIATYPE_VIDEO_LOCAL_RTCP_INFO");
            break;
         default:
            LogMsg(DeviceName(), "??? eMediaType:" + AsString(l_mediaInfo->MediaData[i].eMediaType));
      }
   }
   m_observer->PostEvent(APPEVENT_GET_LOCAL_MEDIA_INFO_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimIpmStartMedia()
// Description: Handle an IPM device get start media command
// Return: int - 0 for success, otherwise error 
// Parameters:  eIPM_DATA_DIRECTION a_dir = DATA_IP_TDM_BIDIRECTIONAL, DATA_IP_RECEIVEONLY, DATA_IP_SENDONLY   
//*****************************************************************************
int CIpmDevice::PrimIpmStartMedia(eIPM_DATA_DIRECTION a_dir)
{
   int res = ipm_StartMedia(Device(), &m_remoteMediaInfo , a_dir, EV_ASYNC);
   if (res == -1)
   {
      ErrMsg(Device(), "ipm_StartMedia failed");
   }
   else
   {
      LogMsg(DeviceName(), "ipm_StartMedia");
   }
   return res;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimOnIpmStartMediaComplete(METAEVENT &a_metaEvent)
// Description: Handle an IPM device get start media completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDevice::PrimOnIpmStartMediaComplete(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_START_MEDIA_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimIpmStopMedia()
// Description: Handle an IPM device get stop media command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CIpmDevice::PrimIpmStopMedia()
{
   LogMsg(DeviceName(), "CIpmDevice::PrimIpmStopMedia()");
   if(m_dtmfMode==DTMFMODE_INBAND){
      if(dx_stopch(m_voxDevH,EV_ASYNC)==-1) {
         ErrMsg(Device(), "dx_stopch()");
      } else {
         LogMsg(DeviceName(), "dx_stopch()");
	  }
	}
   int res = ipm_Stop(Device(), STOP_ALL, EV_ASYNC);
   if (res == -1)
   {
      ErrMsg(Device(), "ipm_Stop");
   }
   else
   {
      LogMsg(DeviceName(), "ipm_Stop");
   }
   return res;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimOnIpmStopMediaComplete(METAEVENT &a_metaEvent)
// Description: Handle an IPM device get stop media completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDevice::PrimOnIpmStopMediaComplete(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_STOP_MEDIA_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CIpmDevice::PrimGenerateIFrame()
// Description: Handle an IPM device generate IFrame command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CIpmDevice::PrimIpmGenerateIFrame()
{
   IFRAME_INFO l_iframeInfo;
   INIT_IFRAME_INFO(&l_iframeInfo);

   int res = ipm_GenerateIFrame(Device(), &l_iframeInfo, EV_ASYNC);
   if (res == -1)
   {
      ErrMsg(Device(), "ipm_GenerateIFrame");
   }
   else
   {
      LogMsg(DeviceName(), "ipm_GenerateIFrame");
   }
   return res;
}

//*****************************************************************************
// Function: int CIpmDevice::ListenTo(CIpmDevice *a_ipmDevice)
// Description: Handle an IPM device listen to a timeslot command
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice *a_ipmDevice 
//*****************************************************************************
int CIpmDevice::ListenTo(CIpmDevice* a_ipmDevice)
{
   SC_TSINFO l_tsInfo;
   long l_timeslot = a_ipmDevice->XmitSlot();
   l_tsInfo.sc_tsarrayp = &l_timeslot;
   l_tsInfo.sc_numts = 1;
   int res = ipm_Listen(Device(), &l_tsInfo, EV_ASYNC);
   if (res == -1)
   {
      ErrMsg(Device(), "ipm_Listen");
   }
   else
   {
      LogMsg(DeviceName(), "ipm_Listen");
   }
   return res;
}

//*****************************************************************************
// Function: int CIpmDevice::OnListenToComplete(METAEVENT &a_metaEvent)
// Description: Handle an IPM device listen to a timeslot completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDevice::OnListenToComplete(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_ON_JOIN_CNF_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CIpmDevice::GetReceivePortInfo()
// Description: Handle an IPM device get receive port info command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CIpmDevice::GetReceivePortInfo()
{
   int res = dev_GetReceivePortInfo(Device(), (void*)(ID()));
   if (res == -1)
   {
      ErrMsg(Device(), "dev_GetReceivePortInfo");
   }
   else
   {
      LogMsg(DeviceName(), "dev_GetReceivePortInfo");
   }
   return res;
}

//*****************************************************************************
// Function: int CIpmDevice::OnGetReceivePortInfoComplete(METAEVENT &a_metaEvent)
// Description: Handle an IPM device get receive port info completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDevice::OnGetReceivePortInfoComplete(METAEVENT& a_metaEvent)
{
   memcpy(&(m_rxPortInfoList),a_metaEvent.evtdatap,sizeof(DM_PORT_INFO_LIST));
   m_observer->PostEvent(APPEVENT_IPM_GET_RX_PORT_INFO_COMPLETE);
   return 0;
}                       

//*****************************************************************************
// Function: int CIpmDevice::GetTransmitPortInfo()
// Description: Handle an IPM device get transmit port info command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CIpmDevice::GetTransmitPortInfo()
{
   int res = dev_GetTransmitPortInfo(Device(), 0);
   if (res == -1)
   {
      ErrMsg(Device(), "dev_GetTransmitPortInfo");
   }
   else
   {
      LogMsg(DeviceName(), "dev_GetTransmitPortInfo");
   }
   return res;
}

//*****************************************************************************
// Function: int CIpmDevice::OnGetTransmitPortInfoComplete(METAEVENT &a_metaEvent)
// Description: Handle an IPM device get transmit port info completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDevice::OnGetTransmitPortInfoComplete(METAEVENT& a_metaEvent)
{
   memcpy(&(m_txPortInfoList),a_metaEvent.evtdatap,sizeof(DM_PORT_INFO_LIST));
   m_observer->PostEvent(APPEVENT_IPM_GET_TX_PORT_INFO_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CIpmDevice::OnPortConnected(METAEVENT &a_metaEvent)
// Description: Handle an IPM device port connected event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDevice::OnPortConnected(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_IPM_PORT_CONNECTED);
   return 0;
}

//*****************************************************************************
// Function: int CIpmDevice::OnPortDisconnected(METAEVENT &a_metaEvent)
// Description: Handle an IPM device port disconnected event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDevice::OnPortDisconnected(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_IPM_PORT_DISCONNECTED);
   return 0;
}

//*****************************************************************************
// 		  NAME : int CIpmDevice::MPIToFPS   
// DESCRIPTION : Converts MPI to Frames Per Sec
// 		 INPUT : FPS
// 	    OUTPUT : MPI
// 	   RETURNS : None
// 	  CAUTIONS : None
//*****************************************************************************
int CIpmDevice::MPIToFPS(int i) {
   int c;

   switch (i) {
    case 1: c =  VIDEO_FRAMESPERSEC_2997;   	break;         /* 29.97 fps */
    case 2: c =  VIDEO_FRAMESPERSEC_15;      	break;        /* 15 fps */
    case 3: c =  VIDEO_FRAMESPERSEC_10;     	break;        /* 10 fps */
    case 5: c =  VIDEO_FRAMESPERSEC_6;       	break;       /* 6 fps  */
    default: c =  VIDEO_FRAMESPERSEC_DEFAULT;	break;
    }

   return c;
}

//*****************************************************************************
// 		  NAME : int CIpmDevice::FPSToMPI   
// DESCRIPTION : Converts Frames Per Sec to MPI
// 		 INPUT : FPS
// 	    OUTPUT : MPI
// 	   RETURNS : None
// 	  CAUTIONS : None
//*****************************************************************************
int CIpmDevice::FPSToMPI(eVIDEO_FRAMESPERSEC fps) 
{
   int nMPI;
  
   if(fps==VIDEO_FRAMESPERSEC_30) {
      nMPI=1;
   }else if(fps==VIDEO_FRAMESPERSEC_2997) {
      nMPI=1;
   }else if(fps==VIDEO_FRAMESPERSEC_15) {
      nMPI=2;
   }else if(fps==VIDEO_FRAMESPERSEC_10) {
      nMPI=3;
   }else if(fps==VIDEO_FRAMESPERSEC_6) {
      nMPI=5;
   } else {
      nMPI=0;
   }
   return nMPI;
}



//*****************************************************************************
// 		  NAME : void CIpmDevice::PrintRemoteLocalInfo   
// DESCRIPTION : Prints Remote and Local Ipm Session Info
// 		 INPUT : None
// 	    OUTPUT : None
// 	   RETURNS : None
// 	  CAUTIONS : None
//*****************************************************************************
void CIpmDevice::PrintRemoteLocalInfo()
{
   LogMsg(DeviceName(), "PrintRemoteLocalInfo()");
   unsigned int ctr;
   char *tempStr;
   tempStr=(char *)malloc(1024);
   for(ctr=0;ctr<m_remoteMediaInfo.unCount;ctr++) {
      switch (m_remoteMediaInfo.MediaData[ctr].eMediaType) 
      {
         case MEDIATYPE_AUDIO_REMOTE_RTP_INFO:
         {
            sprintf(tempStr,"Remote Audio RTP Info: IpAddress = %s | PortID = %u",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.cIPAddress,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.unPortId);
            break;
         }
         case MEDIATYPE_AUDIO_LOCAL_RTP_INFO:
         {
            sprintf(tempStr,"Local Audio RTP Info: IpAddress = %s | PortID = %u",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.cIPAddress,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.unPortId);
            break;
         }
         case MEDIATYPE_AUDIO_REMOTE_RTCP_INFO:
         {
            sprintf(tempStr,"Remote Audio RTCP Info: IpAddress = %s | PortID = %u",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.cIPAddress,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.unPortId);
            break;
         }
         case MEDIATYPE_AUDIO_LOCAL_RTCP_INFO:
         {
            sprintf(tempStr,"Local Audio RTCP Info: IpAddress = %s | PortID = %u",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.cIPAddress,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.unPortId);
            break;
         }
         case MEDIATYPE_AUDIO_REMOTE_CODER_INFO:
         {
            sprintf(tempStr,"Remote Audio Info: CoderType=%d | Pt=%u | FrmSz=%d | FPP=%u | Vad=%d",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.AudioCoderInfo.eCoderType,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.AudioCoderInfo.unCoderPayloadType,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.AudioCoderInfo.eFrameSize,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.AudioCoderInfo.unFramesPerPkt,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.AudioCoderInfo.eVadEnable);
            break;
         }
         case MEDIATYPE_AUDIO_LOCAL_CODER_INFO:
         {
            sprintf(tempStr,"Local Audio Info: CoderType=%d | Pt=%u | FrmSz=%d | FPP=%u | Vad=%d",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.AudioCoderInfo.eCoderType,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.AudioCoderInfo.unCoderPayloadType,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.AudioCoderInfo.eFrameSize,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.AudioCoderInfo.unFramesPerPkt,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.AudioCoderInfo.eVadEnable);
            break;
         }
         case MEDIATYPE_AUDIO_REMOTE_CODER_OPTIONS_INFO:
         {
            sprintf(tempStr,"Remote Audio Coder Option Info: Coder Option =%u",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.AudioCoderOptionsInfo.unCoderOptions);
            break;
         }
         case MEDIATYPE_AUDIO_LOCAL_CODER_OPTIONS_INFO:
         {
            sprintf(tempStr,"Local Audio Coder Option Info: Coder Option =%u",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.AudioCoderOptionsInfo.unCoderOptions);
            break;
         }
         case MEDIATYPE_VIDEO_REMOTE_RTP_INFO:
         {
            sprintf(tempStr,"Remote Video RTP Info: IpAddress = %s | PortID = %u",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.cIPAddress,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.unPortId);
            break;
         }
         case MEDIATYPE_VIDEO_LOCAL_RTP_INFO:
         {
            sprintf(tempStr,"Local Video RTP Info: IpAddress = %s | PortID = %u",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.cIPAddress,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.unPortId);
            break;
         }
         case MEDIATYPE_VIDEO_REMOTE_RTCP_INFO:
         {
            sprintf(tempStr,"Remote Video RTCP Info: IpAddress = %s | PortID = %u",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.cIPAddress,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.unPortId);
            break;
         }
         case MEDIATYPE_VIDEO_LOCAL_RTCP_INFO:
         {
            sprintf(tempStr,"Local Video RTCP Info: IpAddress = %s | PortID = %u",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.cIPAddress,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.PortInfo.unPortId);
            break;
         }
         case MEDIATYPE_VIDEO_REMOTE_CODER_INFO:
         {
            sprintf(tempStr,"Remote Video Info: CoderT=%d | Pt=%u | Prof=%d | Lvl=%d | Resln=(%dx%d) | SamplingRate=%d | Rate=%uK | FPS=%d | DciSz=%u",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.eCoderType,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.unCoderPayloadType,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->eProfile,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->eLevel,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->eImageWidth,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->eImageHeight,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->eSamplingRate,
                            (m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->unBitRate/1000),
                            (30/FPSToMPI(m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->eFramesPerSec)),
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->unVisualConfigSize);
            break;
         }
         case MEDIATYPE_VIDEO_LOCAL_CODER_INFO:
         {
            sprintf(tempStr,"Local Video Info: CoderT=%d | Pt=%u | Prof=%d | Lvl=%d | Resln=(%dx%d) | SamplingRate=%d | Rate=%uK | FPS=%d | DciSz=%u",
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.eCoderType,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.unCoderPayloadType,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->eProfile,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->eLevel,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->eImageWidth,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->eImageHeight,
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->eSamplingRate,
                            (m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->unBitRate/1000),
                            (30/FPSToMPI(m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->eFramesPerSec)),
                             m_remoteMediaInfo.MediaData[ctr].mediaInfo.VideoCoderInfo.pExtraCoderInfo->unVisualConfigSize);
            break;
         }
         default:
         break;
      } // end of switch case statement
      LogMsg(DeviceName(), AsString(tempStr));
      tempStr[0]='\0';
   }	// end of for loop
   free(tempStr);
}

//*****************************************************************************
// Function: void CIpmDevice::ProcessEvent(METAEVENT &a_metaEvent)
// Description: Process an event
// Return: void 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
void CIpmDevice::ProcessEvent(METAEVENT& a_metaEvent)
{
   switch (a_metaEvent.evttype)
   {
      case IPMEV_OPEN:
         LogMsg(DeviceName(), "IPMEVEV_OPEN ipmDevice=" + AsString(":") + AsString(Device()));
         OnIpmOpened(a_metaEvent);
         break;

      case IPMEV_EVENT_ENABLED:
         LogMsg(DeviceName(), "IPMEV_EVENT_ENABLED ipmDevice=" + AsString(":") + AsString(Device()));
         break;

	  case IPMEV_SET_QOS_THRESHOLD_INFO:
         LogMsg(DeviceName(), "IPMEV_SET_QOS_THRESHOLD_INFO ipmDevice=" + AsString(":") + AsString(Device()));
		 {
			eIPM_EVENT qosEvents[1] ={ EVT_RTPTIMEOUT};
			if(ipm_EnableEvents(Device(), qosEvents, 1, EV_ASYNC) == -1)
			{
		         LogMsg(DeviceName(), "ipm_EnableEvents failed for ipmDevice=" + AsString(":") + AsString(Device()));
    		}
		 }
         break;

    case IPMEV_QOS_ALARM:
         LogMsg(DeviceName(), "IPMEV_QOS_ALARM ipmDevice=" + AsString(":") + AsString(Device()));
         {
            IPM_QOS_ALARM_DATA * l_pAlarm = (IPM_QOS_ALARM_DATA*)a_metaEvent.evtdatap;
            if (l_pAlarm->eQoSType == QOSTYPE_RTPTIMEOUT)
			{
                LogMsg(DeviceName(), "Alarm Type = RTP Timeout\n");
				m_observer->PostEvent(APPEVENT_DROPCALL_COMMAND);
            }
            LogMsg(DeviceName(), "Alarm state = " + AsString(l_pAlarm->eAlarmState? "On": "Off"));
         }
         break;
      case IPMEV_TELEPHONY_EVENT:
         LogMsg(DeviceName(), "IPMEV_TELEPHONY_EVENT ipmDevice=" + AsString(":") + AsString(Device()));
         {
            IPM_TELEPHONY_INFO* l_telephonyInfo = (IPM_TELEPHONY_INFO*)a_metaEvent.evtdatap;
            if (l_telephonyInfo->eTelInfoType == TEL_INFOTYPE_EVENT)
            {
               LogMsg(DeviceName(), "RFC2833 NAMED EVENT ID:" + AsString(l_telephonyInfo->TelephonyInfo.TelEvtInfo.eTelephonyEventID));
               LogMsg(DeviceName(), "                Volume:" + AsString(l_telephonyInfo->TelephonyInfo.TelEvtInfo.sVolume));
               LogMsg(DeviceName(), "              Duration:" + AsString(l_telephonyInfo->TelephonyInfo.TelEvtInfo.usDuration));
               
              if (m_dtmfMode == DTMFMODE_RFC2833){
                 m_observer->PostEvent(APPEVENT_DTMF_DIGIT, g_signalIdEvents[l_telephonyInfo->TelephonyInfo.TelEvtInfo.eTelephonyEventID]);
              }
            }
         }
         break;
         
      case IPMEV_GET_XMITTS_INFO:
         LogMsg(DeviceName(), "IPMEV_GET_XMITTS_INFO");
         OnIpmGetXmitSlotComplete(a_metaEvent);
         break;

      case IPMEV_GET_LOCAL_MEDIA_INFO:
         LogMsg(DeviceName(), "IPMEV_GET_LOCAL_MEDIA_INFO");
         OnIpmGetLocalMediaInfoComplete(a_metaEvent);
         break;

      case IPMEV_STARTMEDIA:
         LogMsg(DeviceName(), "IPMEV_STARTMEDIA");
         OnIpmStartMediaComplete(a_metaEvent);
         break;

      case IPMEV_LISTEN:
         LogMsg(DeviceName(), "IPMEV_LISTEN");
         OnListenToComplete(a_metaEvent);
         break;

      case IPMEV_UNLISTEN:
         LogMsg(DeviceName(), "IPMEV_UNLISTEN");
         break;

      case IPMEV_STOPPED:
         LogMsg(DeviceName(), "IPMEV_STOPPED");
         OnIpmStopMediaComplete(a_metaEvent);
         break;

      case IPMEV_GENERATEIFRAME:
         LogMsg(DeviceName(), "IPMEV_GENERATEIFRAME");
         break;

      case IPMEV_ERROR:
         ErrMsg(Device(), "IPMEV_ERROR");
         OnIpmEvError(a_metaEvent);
         break;

      case DMEV_CONNECT:
         LogMsg(DeviceName(), "DMEV_CONNECT");
         m_observer->PostEvent(APPEVENT_IPM_CONNECTED);
         break;

      case DMEV_DISCONNECT:
         LogMsg(DeviceName(), "DMEV_DISCONNECT");
         m_observer->PostEvent(APPEVENT_IPM_DISCONNECTED);
         break;

      case DMEV_RESERVE_RESOURCE:
         LogMsg(DeviceName(), "DMEV_RESERVE_RESOURCE");
         OnIpmReserveResourceComplete(a_metaEvent);
         break;

      case DMEV_RESERVE_RESOURCE_FAIL:
         LogMsg(DeviceName(), "DMEV_RESERVE_RESOURCE_FAIL");
         OnIpmReserveResourceComplete(a_metaEvent);
         break;

      case DMEV_RELEASE_RESOURCE:
         LogMsg(DeviceName(), "DMEV_RELEASE_RESOURCE");
         OnIpmReleaseResourceComplete(a_metaEvent);
         break;

      case DMEV_RELEASE_RESOURCE_FAIL:
         LogMsg(DeviceName(), "DMEV_RELEASE_RESOURCE_FAIL");
         break;

      case DMEV_GET_RX_PORT_INFO:
         LogMsg(DeviceName(), "DMEV_GET_RX_PORT_INFO");
         OnGetReceivePortInfoComplete(a_metaEvent);
         break;

      case DMEV_GET_RX_PORT_INFO_FAIL:
         LogMsg(DeviceName(), "DMEV_GET_RX_PORT_INFO_FAIL");
         break;

      case DMEV_GET_TX_PORT_INFO:
         LogMsg(DeviceName(), "DMEV_GET_TX_PORT_INFO");
         OnGetTransmitPortInfoComplete(a_metaEvent);
         break;

      case DMEV_GET_TX_PORT_INFO_FAIL:
         LogMsg(DeviceName(), "DMEV_GET_TX_PORT_INFO_FAIL");
         break;

      case DMEV_PORT_CONNECT:
         LogMsg(DeviceName(), "DMEV_PORT_CONNECT");
         OnPortConnected(a_metaEvent);
         break;

      case DMEV_PORT_CONNECT_FAIL:
         LogMsg(DeviceName(), "DMEV_PORT_CONNECT_FAIL");
         break;

      case DMEV_PORT_DISCONNECT:
         LogMsg(DeviceName(), "DMEV_PORT_DISCONNECT");
         OnPortDisconnected(a_metaEvent);
         break;

      case DMEV_PORT_DISCONNECT_FAIL:
         LogMsg(DeviceName(), "DMEV_PORT_DISCONNECT_FAIL");
         break;

      case TDX_LISTEN:
         LogMsg(DeviceName(), "TDX_LISTEN");
         break;
      case TDX_GETDIG:
      {
         // SKS
         long termno = 0;
         int digit;
         char tmpdig[16];
         termno = ATDX_TERMMSK(m_voxDevH);
         if ( termno & TM_MAXDTMF )
         {
            if(strlen(m_digits.dg_value)==1) {
               digit=m_digits.dg_value[0];
               //sscanf(m_digits.dg_value,"%d",&digit);
            } else {
               sscanf(m_digits.dg_value,"%d",&digit);
            }
            sprintf(tmpdig, "%c", digit);
            LogMsg(DeviceName(), "TDX_GETDIG: " + AsString(tmpdig));
            m_observer->PostEvent(APPEVENT_DTMF_DIGIT,digit);
         }
         break;
      }
      case TDX_CST:
         {
/*
            DX_CST* l_cst = (DX_CST*) a_metaEvent.evtdatap;
            switch (l_cst->cst_event)
            {
               case DE_DIGITS:
                  LogMsg(m_voxDevice, "DE_DIGITS:" + AsHexString(l_cst->cst_data));
                  m_observer->PostEvent(APPEVENT_DTMF_DIGIT, l_cst->cst_data);
                  break;

               default:
                  LogMsg(m_voxDevice, "unknown TDX_CST event" + AsHexString(l_cst->cst_event));
            }
*/
         }
         break;

      case SMEV_ADD_OVERLAY:
         LogMsg(DeviceName(), "SMEV_ADD_OVERLAY");
         m_observer->PostEvent(a_metaEvent);
         break;

      case SMEV_REMOVE_OVERLAY:
         LogMsg(DeviceName(), "SMEV_REMOVE_OVERLAY");
         m_observer->PostEvent(a_metaEvent);
         break;

      case SMEV_REMOVE_ALL_OVERLAYS:
         LogMsg(DeviceName(), "SMEV_REMOVE_ALL_OVERLAYS");
         break;

      case SMEV_DISABLE_EVENT:
         LogMsg(DeviceName(), "SMEV_DISABLE_EVENT");
         break;

      case SMEV_ENABLE_EVENT:
         LogMsg(DeviceName(), "SMEV_ENABLE_EVENT");
         break;

      case SMEV_OVERLAY_COMPLETED:
         LogMsg(DeviceName(), "SMEV_OVERLAY_COMPLETED");
         break;

      case SMEV_ERROR:
         LogMsg(DeviceName(), "SMEV_ERROR");
         break;

      case SMEV_LAST_EVENT_ERROR:
         LogMsg(DeviceName(), "SMEV_LAST_EVENT_ERROR");
         break;

      case SMEV_ADD_OVERLAY_FAIL:
         LogMsg(DeviceName(), "SMEV_ADD_OVERLAY_FAIL");
         m_observer->PostEvent(a_metaEvent);
         break;

      case SMEV_REMOVE_OVERLAY_FAIL:
         LogMsg(DeviceName(), "SMEV_REMOVE_OVERLAY_FAIL");
         m_observer->PostEvent(a_metaEvent);
         break;

      case SMEV_REMOVE_ALL_OVERLAYS_FAIL:
         LogMsg(DeviceName(), "SMEV_REMOVE_ALL_OVERLAYS_FAIL");
         break;

      case SMEV_DISABLE_EVENT_FAIL:
         LogMsg(DeviceName(), "SMEV_DISABLE_EVENT_FAIL");
         break;

      case SMEV_ENABLE_EVENT_FAIL:
         LogMsg(DeviceName(), "SMEV_ENABLE_EVENT_FAIL");
         break;

      case APPEVENT_CLOSE_COMMAND:
         LogMsg(DeviceName(), "APPEVENT_CLOSE_COMMAND");
         Close();
         break;

      default:
         LogMsg(DeviceName(), "unhandled event:" + AsHexString(a_metaEvent.evttype));
   }
}


