/**
* @file CIpmDevice.h
* @brief IP Media Device class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CImpDevice_h
#define CImpDevice_h

#include "srllib.h"
#include "gclib.h"
#include "ipmlib.h"
#include "port_connect.h"

#include "ADevice.h"

#include <string>

#define DEFAULT_RFC2833_PAYLOAD_TYPE  101
#define MAX_CNF_PIN_DIGITS	6
#define DTMFMODE_INBAND 0
#define DTMFMODE_RFC2833 1
#define DTMFMODE_SIPINFO 2

class AIpmDeviceState;
class CSipSession;

//*****************************************************************************
// Class: ::CIpmDevice
// Description: Wrapper class for an IPM device
//*****************************************************************************
class CIpmDevice : public ADevice
{
   friend class AIpmDeviceState;
   friend class CIpmDeviceClosedState;
   friend class CIpmDeviceOpeningState;
   friend class CIpmDeviceIdleState;
   friend class CIpmDeviceReservingResourceState;
   friend class CIpmDeviceReleasingResourceState;
   friend class CIpmDeviceGettingLocalMediaInfoState;
   friend class CIpmDeviceStartingMediaState;
   friend class CIpmDeviceMediaStreamingState;
   friend class CIpmDeviceStoppingMediaState;

public:
   CIpmDevice(const std::string& a_deviceName);

   const std::string& DeviceName() {return m_deviceName;}

   long ID() {return(long) this;}

   AIpmDeviceState& DeviceState() {return *m_deviceState;}
   void SetDeviceStateTo(AIpmDeviceState* state);
   void Observer(AEventHandler* a_observer) {m_observer = a_observer;}

   int Stop() {return IpmStopMedia();}

   void ProcessEvent(METAEVENT& a_metaEvent);

   // SKS
   void SetDxDeviceName(int ipmCh);
   //JM
   void SetDtmfMode(int a_dtmfMode) {m_dtmfMode = a_dtmfMode;}

   int Open();
   int Close();
   int IpmGetXmitSlot();
   int IpmReserveResource(eDEV_RESOURCE_TYPE a_resourceType);
   int IpmReleaseResource();
   int IpmGetLocalMediaInfo();
   int IpmStartMedia(eIPM_DATA_DIRECTION a_dir);
   int IpmStopMedia();
   int IpmGenerateIFrame();

   int OnIpmOpened(METAEVENT& a_metaEvent);
   int OnIpmGetXmitSlotComplete(METAEVENT& a_metaEvent);
   int OnIpmReserveResourceComplete(METAEVENT& a_metaEvent);
   int OnIpmReleaseResourceComplete(METAEVENT& a_metaEvent);
   int OnIpmGetLocalMediaInfoComplete(METAEVENT& a_metaEvent);
   int OnIpmStartMediaComplete(METAEVENT& a_metaEvent);
   int OnIpmStopMediaComplete(METAEVENT& a_metaEvent);
   int OnIpmEvError(METAEVENT& a_metaEvent);

   IPM_PORT_INFO& LocalAudioPortInfo() {return m_localAudioPortInfo;}
   IPM_PORT_INFO& LocalVideoPortInfo() {return m_localVideoPortInfo;}
   IPM_MEDIA_INFO& RemoteMediaInfo() {return m_remoteMediaInfo;}

   IPM_VIDEO_CODER_INFO_EX& LocalVideoCoderInfoEx() {return m_localVideoCoderInfoEx;}
   IPM_VIDEO_CODER_INFO_EX& RemoteVideoCoderInfoEx() {return m_remotelVideoCoderInfoEx;}

   int GetReceivePortInfo();
   int OnGetReceivePortInfoComplete(METAEVENT& a_metaEvent);
   int GetTransmitPortInfo();
   int OnGetTransmitPortInfoComplete(METAEVENT& a_metaEvent);

   const DM_PORT_INFO_LIST& RxPortInfoList() {return m_rxPortInfoList;}
   const DM_PORT_INFO_LIST& TxPortInfoList() {return m_txPortInfoList;}

   int OnPortConnected(METAEVENT& a_metaEvent);
   int OnPortDisconnected(METAEVENT& a_metaEvent);

   int ListenTo(CIpmDevice* a_ipmDevice);
   int OnListenToComplete(METAEVENT& a_metaEvent);

   long XmitSlot() {return m_timeslot;}

   bool ShouldClose() {return true;}

   int SetRfc2833PayloadTypes(unsigned long a_txPayloadType, unsigned long a_rxPayloadType);
// SKS
   void GetDigits(int numDigits);

   static std::string GetNextIpmDeviceName();
   int MPIToFPS(int i);
   int FPSToMPI(eVIDEO_FRAMESPERSEC fps);
   void PrintRemoteLocalInfo();

private:
   // force initialization
   CIpmDevice();

   // make non-copyable
   CIpmDevice(const CIpmDevice&);
   const CIpmDevice& operator=(const CIpmDevice&);

   // primative functions
   int PrimIpmOpen();
   int PrimIpmClose();
   int PrimIpmGetXmitSlot();
   int PrimIpmReserveResource(eDEV_RESOURCE_TYPE a_resourceType);
   int PrimIpmReleaseResource();
   int PrimIpmGetLocalMediaInfo();
   int PrimIpmStartMedia(eIPM_DATA_DIRECTION a_dir);
   int PrimIpmStopMedia();
   int PrimIpmGenerateIFrame();
   int PrimOnIpmOpened(METAEVENT& a_metaEvent);
   int PrimOnIpmGetXmitSlotComplete(METAEVENT& a_metaEvent);
   int PrimOnIpmReserveResourceComplete(METAEVENT& a_metaEvent);
   int PrimOnIpmReleaseResourceComplete(METAEVENT& a_metaEvent);
   int PrimOnIpmGetLocalMediaInfoComplete(METAEVENT& a_metaEvent);
   int PrimOnIpmStartMediaComplete(METAEVENT& a_metaEvent);
   int PrimOnIpmStopMediaComplete(METAEVENT& a_metaEvent);

   // member variables
   AIpmDeviceState* m_deviceState;
   std::string m_deviceName;
 
   //JM
   int m_dtmfMode;

   long m_timeslot;
// SKS
    DV_TPT	m_tpt[2];
    DV_DIGIT	m_digits;


   int m_voxDevH;
   std::string m_voxDevName;

   AEventHandler* m_observer;

   IPM_MEDIA_INFO m_localMediaInfo;
   IPM_PORT_INFO m_localAudioPortInfo;
   IPM_PORT_INFO m_localVideoPortInfo;

   IPM_MEDIA_INFO m_remoteMediaInfo;
   IPM_PORT_INFO m_remotePortInfo;

   IPM_VIDEO_CODER_INFO_EX m_localVideoCoderInfoEx;
   IPM_VIDEO_CODER_INFO_EX m_remotelVideoCoderInfoEx;

   DM_PORT_INFO_LIST m_rxPortInfoList;
   DM_PORT_INFO_LIST m_txPortInfoList;

   DEV_RESOURCE_LIST m_devResourceList;

   static int m_nextIpmDevice;

};

#endif


