/**
* @file CIpmDeviceState.cpp
* @brief IP Media Device States - class implementations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "CIpmDeviceState.h"
#include "LogUtil.h"

CIpmDeviceStates AIpmDeviceState::m_deviceStates;

//*****************************************************************************
// Function: int AIpmDeviceState::LogStateError(CIpmDevice &device, const string &msg)
// Description: Log an invalid state for operation/event message
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//             const string &msg 
//*****************************************************************************
int AIpmDeviceState::LogStateError(CIpmDevice& device, const string& msg)
{
   std::cout << device.DeviceName()
   << " " << Desc() << " is invalid device state for "
   << msg
   << std::endl;
   return -1;
}

//*****************************************************************************
// Function: int AIpmDeviceState::IpmClose(CIpmDevice &device)
// Description: Close an IPM device
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//*****************************************************************************
int AIpmDeviceState::IpmClose(CIpmDevice& device) 
{
   int res = device.PrimIpmClose();
   if (res != -1)
   {
      device.SetDeviceStateTo(DeviceState(IpmDeviceClosedState));
   }
   return res;
}

//*****************************************************************************
// Function: CIpmDeviceStates::CIpmDeviceStates()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CIpmDeviceStates::CIpmDeviceStates()
{
   m_deviceStates[IpmDeviceClosedState] = new CIpmDeviceClosedState();
   m_deviceStates[IpmDeviceOpeningState] = new CIpmDeviceOpeningState();
   m_deviceStates[IpmDeviceIdleState] = new CIpmDeviceIdleState();
   m_deviceStates[IpmDeviceReservingResourceState] = new CIpmDeviceReservingResourceState();
   m_deviceStates[IpmDeviceReleasingResourceState] = new CIpmDeviceReleasingResourceState();
   m_deviceStates[IpmDeviceGettingLocalMediaInfoState] = new CIpmDeviceGettingLocalMediaInfoState();
   m_deviceStates[IpmDeviceStartingMediaState] = new CIpmDeviceStartingMediaState();
   m_deviceStates[IpmDeviceStoppingMediaState] = new CIpmDeviceStoppingMediaState();
   m_deviceStates[IpmDeviceMediaStreamingState] = new CIpmDeviceMediaStreamingState();
}

//*****************************************************************************
// Function: int CIpmDeviceClosedState::IpmOpen(CIpmDevice &device)
// Description: Open an IPM device
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//*****************************************************************************
int CIpmDeviceClosedState::IpmOpen(CIpmDevice& device) 
{
   device.SetDeviceStateTo(DeviceState(IpmDeviceOpeningState));
   return device.PrimIpmOpen();
}

//*****************************************************************************
// Function: int CIpmDeviceOpeningState::OnIpmOpened(CIpmDevice &device, METAEVENT &a_metaEvent)
// Description: Handle an IPM device open completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDeviceOpeningState::OnIpmOpened(CIpmDevice& device, METAEVENT& a_metaEvent) 
{
   device.PrimOnIpmOpened(a_metaEvent);
   return device.IpmGetXmitSlot();;
}

//*****************************************************************************
// Function: int CIpmDeviceOpeningState::IpmGetXmitSlot(CIpmDevice &device)
// Description: Get an IPM device xmit timeslot 
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//*****************************************************************************
int CIpmDeviceOpeningState::IpmGetXmitSlot(CIpmDevice &device)
{
   return device.PrimIpmGetXmitSlot();
}

//*****************************************************************************
// Function: int CIpmDeviceOpeningState::OnIpmGetXmitSlotComplete(CIpmDevice &device, METAEVENT &a_metaEvent)
// Description: Handle a get IPM device get xmit timeslot completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDeviceOpeningState::OnIpmGetXmitSlotComplete(CIpmDevice& device, METAEVENT& a_metaEvent) 
{
   device.SetDeviceStateTo(DeviceState(IpmDeviceIdleState));
   return device.PrimOnIpmGetXmitSlotComplete(a_metaEvent);
}

//*****************************************************************************
// Function: int CIpmDeviceIdleState::IpmReserveResource(CIpmDevice &device,eDEV_RESOURCE_TYPE a_resourceType)
// Description: Reserve an IPM device resource
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//             eDEV_RESOURCE_TYPE a_resourceType
//*****************************************************************************
int CIpmDeviceIdleState::IpmReserveResource(CIpmDevice& device,eDEV_RESOURCE_TYPE a_resourceType) 
{
   device.SetDeviceStateTo(DeviceState(IpmDeviceReservingResourceState));
   return device.PrimIpmReserveResource(a_resourceType);
}

//*****************************************************************************
// Function: int CIpmDeviceReservingResourceState::OnIpmReserveResourceComplete(CIpmDevice &device, METAEVENT &a_metaEvent)
// Description: Reserve an IPM device resource completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDeviceReservingResourceState::OnIpmReserveResourceComplete(CIpmDevice& device, METAEVENT& a_metaEvent) 
{
   device.SetDeviceStateTo(DeviceState(IpmDeviceIdleState));
   return device.PrimOnIpmReserveResourceComplete(a_metaEvent);
}

//*****************************************************************************
// Function: int CIpmDeviceIdleState::IpmGetLocalMediaInfo(CIpmDevice &device)
// Description: Get IPM device local media info
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//*****************************************************************************
int CIpmDeviceIdleState::IpmGetLocalMediaInfo(CIpmDevice& device) 
{
   device.SetDeviceStateTo(DeviceState(IpmDeviceGettingLocalMediaInfoState));
   return device.PrimIpmGetLocalMediaInfo();
}

//*****************************************************************************
// Function: int CIpmDeviceGettingLocalMediaInfoState::OnIpmGetLocalMediaInfoComplete(CIpmDevice &device, METAEVENT &a_metaEvent)
// Description: Get IPM device local media info completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDeviceGettingLocalMediaInfoState::OnIpmGetLocalMediaInfoComplete(CIpmDevice& device, METAEVENT& a_metaEvent) 
{
   device.SetDeviceStateTo(DeviceState(IpmDeviceIdleState));
   return device.PrimOnIpmGetLocalMediaInfoComplete(a_metaEvent);
}

//*****************************************************************************
// Function: int CIpmDeviceIdleState::IpmStartMedia(CIpmDevice &device, eIPM_DATA_DIRECTION a_dir)
// Description: Start media streaming on an IPM device
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//*****************************************************************************
int CIpmDeviceIdleState::IpmStartMedia(CIpmDevice& device, eIPM_DATA_DIRECTION a_dir) 
{
   device.SetDeviceStateTo(DeviceState(IpmDeviceStartingMediaState));
   return device.PrimIpmStartMedia(a_dir);
}

//*****************************************************************************
// Function: int CIpmDeviceStartingMediaState::OnIpmStartMediaComplete(CIpmDevice &device, METAEVENT &a_metaEvent)
// Description: Start media streaming on an IPM device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDeviceStartingMediaState::OnIpmStartMediaComplete(CIpmDevice& device, METAEVENT& a_metaEvent) 
{
   device.SetDeviceStateTo(DeviceState(IpmDeviceMediaStreamingState));
   return device.PrimOnIpmStartMediaComplete(a_metaEvent);
}

//*****************************************************************************
// Function: int CIpmDeviceStartingMediaState::OnIpmEvError(CIpmDevice &device, METAEVENT &a_metaEvent)
// Description: Start media streaming on an IPM device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDeviceStartingMediaState::OnIpmEvError(CIpmDevice& device, METAEVENT& a_metaEvent) 
{
   LogMsg(device.DeviceName(), "CIpmDeviceStartingState::OnIpmEvError() initiating IpmStopMedia on channel.");
   //JM - better to clean up here by stoping device...
   device.SetDeviceStateTo(DeviceState(IpmDeviceStoppingMediaState));
   return device.PrimIpmStopMedia();
}

//*****************************************************************************
// Function: int CIpmDeviceMediaStreamingState::IpmStopMedia(CIpmDevice &device)
// Description: Stop media streaming on an IPM device
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//*****************************************************************************
int CIpmDeviceMediaStreamingState::IpmStopMedia(CIpmDevice& device) 
{
   device.SetDeviceStateTo(DeviceState(IpmDeviceStoppingMediaState));
   return device.PrimIpmStopMedia();
}

//*****************************************************************************
// Function: int CIpmDeviceMediaStreamingState::IpmGenerateIFrame(CIpmDevice &device)
// Description: Generate an IFrame
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//*****************************************************************************
int CIpmDeviceMediaStreamingState::IpmGenerateIFrame(CIpmDevice& device) 
{
   return device.PrimIpmGenerateIFrame();
}

//*****************************************************************************
// Function: int CIpmDeviceStoppingMediaState::OnIpmStopMediaComplete(CIpmDevice &device, METAEVENT &a_metaEvent)
// Description: Start media streaming on an IPM device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDeviceStoppingMediaState::OnIpmStopMediaComplete(CIpmDevice& device, METAEVENT& a_metaEvent) 
{
   device.SetDeviceStateTo(DeviceState(IpmDeviceIdleState));
   return device.PrimOnIpmStopMediaComplete(a_metaEvent);
}

//*****************************************************************************
// Function: int CIpmDeviceIdleState::IpmReleaseResource(CIpmDevice &device)
// Description: Release an IPM device resource
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//*****************************************************************************
int CIpmDeviceIdleState::IpmReleaseResource(CIpmDevice& device) 
{
   device.SetDeviceStateTo(DeviceState(IpmDeviceReleasingResourceState));
   return device.PrimIpmReleaseResource();
}

//*****************************************************************************
// Function: int CIpmDeviceReleasingResourceState::OnIpmReleaseResourceComplete(CIpmDevice &device, METAEVENT &a_metaEvent)
// Description: Release an IPM device resource completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CIpmDevice &device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CIpmDeviceReleasingResourceState::OnIpmReleaseResourceComplete(CIpmDevice& device, METAEVENT& a_metaEvent) 
{
   device.SetDeviceStateTo(DeviceState(IpmDeviceIdleState));
   return device.PrimOnIpmReleaseResourceComplete(a_metaEvent);
}

