/**
* @file CIpmDeviceState.h
* @brief IP Media Device States - class declarations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CIpmDeviceState_h
#define CIpmDeviceState_h

#include <map>
#include <string>

#include "LogUtil.h"
#include "CIpmDevice.h"

typedef enum 
{
   IpmDeviceClosedState = 0,
   IpmDeviceOpeningState,
   IpmDeviceIdleState,
   IpmDeviceReservingResourceState,
   IpmDeviceGettingLocalMediaInfoState,
   IpmDeviceStartingMediaState,
   IpmDeviceMediaStreamingState,
   IpmDeviceStoppingMediaState,
   IpmDeviceReleasingResourceState
} IpmDeviceState;

class AIpmDeviceState;

//*****************************************************************************
// Class: ::CIpmDeviceStates
// Description: Map of IPM device state objects
//*****************************************************************************
class CIpmDeviceStates
{
public:
   CIpmDeviceStates();
   ~CIpmDeviceStates() {}
   AIpmDeviceState* DeviceState(IpmDeviceState deviceState) {return m_deviceStates[ deviceState];}

private:
   std::map< IpmDeviceState, AIpmDeviceState*> m_deviceStates;
};

//*****************************************************************************
// Class: ::AIpmDeviceState
// Description: Abstract IPM device state class
// Notes:
// 
//    This abstract class declares all functions for all device states as 
//    virtual and implements them as an error.
// 
//    Concrete subclasses implement only the functions that are valid for that 
//    state.
// 
//    State classes are responsible only for calling primative functions on the 
//    context device object and changing the state of the context device object.
// 
//    As state classes have no non-static member variables, one set of device 
//    state objects are shared by all devices.
//*****************************************************************************
class AIpmDeviceState
{
public:
   virtual const char* Desc() = 0;

   virtual int IpmOpen(CIpmDevice& a_device) {return LogStateError(a_device, "IpmOpen");}
   virtual int IpmClose(CIpmDevice& a_device);
   virtual int IpmGetXmitSlot(CIpmDevice& a_device) {return LogStateError(a_device, "IpmGetXmitSlot");}
   virtual int IpmReserveResource(CIpmDevice& a_device, eDEV_RESOURCE_TYPE a_resourceType) {return LogStateError(a_device, "IpmReserveResource");}
   virtual int IpmGetLocalMediaInfo(CIpmDevice& a_device) {return LogStateError(a_device, "IpmGetLocalMediaInfo");}
   virtual int IpmStartMedia(CIpmDevice& a_device,eIPM_DATA_DIRECTION a_dir) {return LogStateError(a_device, "IpmStartMedia");}
   virtual int IpmStopMedia(CIpmDevice& a_device) {LogMsg(a_device.DeviceName(), "IpmStopMedia ignored"); return 0;}
   virtual int IpmReleaseResource(CIpmDevice& a_device) {return LogStateError(a_device, "IpmReleaseResource");}
   virtual int IpmGenerateIFrame(CIpmDevice& a_device) {LogMsg(a_device.DeviceName(), "IpmGenerateIFrame ignored"); return 0;}
   virtual int OnIpmOpened(CIpmDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnIpmOpened");}
   virtual int OnIpmGetXmitSlotComplete(CIpmDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnIpmGetXmitSlotComplete");}
   virtual int OnIpmReserveResourceComplete(CIpmDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "IpmReserveResourceComplete");}
   virtual int OnIpmGetLocalMediaInfoComplete(CIpmDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "IpmOnGetLocalMediaInfoComplete");}
   virtual int OnIpmStartMediaComplete(CIpmDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnIpmStartMediaComplete");}
   virtual int OnIpmStopMediaComplete(CIpmDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnIpmStopMediaComplete");}
   virtual int OnIpmReleaseResourceComplete(CIpmDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "IpmReleaseResourceComplete");}
   virtual int OnIpmEvError(CIpmDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnIpmEvError");}
 
   static AIpmDeviceState* DeviceState(IpmDeviceState deviceState) {return m_deviceStates.DeviceState(deviceState);}

private:
   int LogStateError(CIpmDevice& a_device, const std::string& msg);
   static CIpmDeviceStates m_deviceStates;
};

//*****************************************************************************
// Class: ::CIpmDeviceClosedState
// Description: IPM device closed state
//*****************************************************************************
class CIpmDeviceClosedState : public AIpmDeviceState
{
public:
   const char* Desc() {return "closed";}
   int IpmOpen(CIpmDevice& a_device);
};

//*****************************************************************************
// Class: ::CIpmDeviceOpeningState
// Description: IPM device opening state
//*****************************************************************************
class CIpmDeviceOpeningState : public AIpmDeviceState
{
public:
   const char* Desc() {return "opening";}
   int OnIpmOpened(CIpmDevice& a_device, METAEVENT& a_metaEvent);
   int IpmGetXmitSlot(CIpmDevice& a_device);
   int OnIpmGetXmitSlotComplete(CIpmDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CIpmDeviceIdleState
// Description: IPM device opened and available state
//*****************************************************************************
class CIpmDeviceIdleState : public AIpmDeviceState
{
public:
   const char* Desc() {return "idle";}
   int IpmReserveResource(CIpmDevice& a_device,eDEV_RESOURCE_TYPE a_resourceType);
   int IpmGetLocalMediaInfo(CIpmDevice& a_device);
   int IpmStartMedia(CIpmDevice& a_devicee, eIPM_DATA_DIRECTION a_dir);
   int IpmReleaseResource(CIpmDevice& a_device);
};

//*****************************************************************************
// Class: ::CIpmDeviceReservingResourceState
// Description: IPM device reserving resource state
//*****************************************************************************
class CIpmDeviceReservingResourceState : public AIpmDeviceState
{
public:
   const char* Desc() {return "reserve resource";}
   int OnIpmReserveResourceComplete(CIpmDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CIpmDeviceGettingLocalMediaInfoState
// Description: IPM device getting local media info state
//*****************************************************************************
class CIpmDeviceGettingLocalMediaInfoState : public AIpmDeviceState
{
public:
   const char* Desc() {return "get local media info";}
   int OnIpmGetLocalMediaInfoComplete(CIpmDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CIpmDeviceStartingMediaState
// Description: IPM device starting media state
//*****************************************************************************
class CIpmDeviceStartingMediaState : public AIpmDeviceState
{
public:
   const char* Desc() {return "starting";}
   int OnIpmStartMediaComplete(CIpmDevice& a_device, METAEVENT& a_metaEvent);
   int OnIpmEvError(CIpmDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CIpmDeviceMediaStreamingState
// Description: IPM device medis atreaming state
//*****************************************************************************
class CIpmDeviceMediaStreamingState : public AIpmDeviceState
{
public:
   const char* Desc() {return "streaming";}
   int IpmStopMedia(CIpmDevice& a_device);
   int IpmGenerateIFrame(CIpmDevice& a_device);
};

//*****************************************************************************
// Class: ::CIpmDeviceStoppingMediaState
// Description: IPM device stopping media state
//*****************************************************************************
class CIpmDeviceStoppingMediaState : public AIpmDeviceState
{
public:
   const char* Desc() {return "stopping";}
   int OnIpmStopMediaComplete(CIpmDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CIpmDeviceReleasingResourceState
// Description: IPM device releasing resource state
//*****************************************************************************
class CIpmDeviceReleasingResourceState : public AIpmDeviceState
{
public:
   const char* Desc() {return "release resource";}
   int OnIpmReleaseResourceComplete(CIpmDevice& a_device, METAEVENT& a_metaEvent);
};

#endif

