/**
* @file CMediaDevice.cpp
* @brief Multimedia Device class implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif


#include "LogUtil.h"
#include "CMediaDevice.h"
#include "CSystemConfiguration.h"
#include "CSipSession.h"
#include "CMediaDeviceState.h"
#include "CGcManager.h"
#include "CScheduler.h"

int CMediaDevice::m_nextMediaDevice(1);

//*****************************************************************************
// Function: std::string CMediaDevice::GetNextMediaDeviceName()
// Description: Get the next available media device name
// Return: std::string 
// Parameters: none 
//*****************************************************************************
std::string CMediaDevice::GetNextMediaDeviceName()
{
   char l_devName[30];
   sprintf( l_devName, "mmB1C%d", m_nextMediaDevice++);
   return l_devName;
}


//*****************************************************************************
// Function: CMediaDevice::CMediaDevice(const std::string &a_deviceName)
// Description: Initializing constructor
// Return:  
// Parameters: const std::string &a_deviceName 
//*****************************************************************************
CMediaDevice::CMediaDevice(const std::string& a_deviceName)
   :m_deviceState(AMediaDeviceState::DeviceState(MediaDeviceClosedState))
   ,m_closingSubState(0)
   ,m_deviceName(a_deviceName)
   ,m_audioFile("")
   ,m_videoFile("")
   ,m_haveClip(false)
   ,m_recVideoCoding(VIDEO_CODING_H263)
   ,m_is3gp (false)
{
}

//*****************************************************************************
// Function: void CMediaDevice::SetDeviceStateTo(AMediaDeviceState *state)
// Description: Set the device state
// Return: void 
// Parameters: AMediaDeviceState *state 
//*****************************************************************************
void CMediaDevice::SetDeviceStateTo(AMediaDeviceState* state)
{
   m_deviceState = state;
   LogMsg(DeviceName(), AsString("MM Device State:") + m_deviceState->Desc());
}


#ifdef USE_MTK_MP
//*****************************************************************************
// Function: void CMediaDevice::SetClip(const std::string &a_clip)
// Description: Set the base name of the multimedia clip to play
// Return: Error (-1) or Success (0)
// Parameters: const std::string& a_path,
//             const std::string &a_clip
//*****************************************************************************
int CMediaDevice::SetClip(const std::string& a_path, const std::string& a_clip)
{
   unsigned long       ret;            //Variable to store the return value of a call
   MP_MMFILE           mmFile;         //MP_MMFILE structure of the multimedia file opened
   MP_FILE_INFO_DESC   fileInfoDesc;   //File Information Descriptor structure
   unsigned long       minFileInfoSize;//Minimum File Information size
   string              fileToParse;        //multimedia file name from which we're gonna get info



   INIT_MM_VIDEO_CODEC(&m_videoCodec);
   INIT_MM_AUDIO_CODEC(&m_audioCodec);

   if (a_clip.size()>0)
   {

      // is it a 3GP or MP4 file ?
      if ((a_clip.find(".3gp") != string::npos)||(a_clip.find(".mp4") != string::npos))
      {
         LogMsg(DeviceName(), AsString("The file is a 3GP file") );
         m_audioFile = a_path + a_clip; // used by the MM device
         m_videoFile = a_path + a_clip; // used by the MM device
         m_is3gp = true;
         m_audioCodec.unCoding         = 0; 
      }
      else // assume a DMF file
      {
         LogMsg(DeviceName(), AsString("The file is a DMF file") );
         m_audioFile = a_path + a_clip + ".pcm";
         m_videoFile = a_path + a_clip + ".vid";

         // Only raw PCM is supported so far for Audio
         m_audioCodec.unCoding         = MM_DATA_FORMAT_PCM;
         m_audioCodec.unSampleRate     = 8000;
         m_audioCodec.unBitsPerSample  = 16;
         m_is3gp = false;
      }

      LogMsg(DeviceName(), AsString("SetClip:") + m_audioFile + AsString(":") + m_videoFile);
      fileToParse = m_videoFile; 
   }
   else
   {
      LogMsg(DeviceName(), AsString("SetClip: Clip size is 0") );
      return (-1);
   }


   //Initialize version field in MP_MMFILE structure
   mmFile.unVersion    = MP_MMFILE_VERSION;

   LogMsg(DeviceName(), AsString("Calling mp_OpenFile for file: ") + fileToParse);

   //Open the  file
   if (m_is3gp == true)
      ret = mp_OpenFile(const_cast<char*>(fileToParse.c_str()), MP_FILE_TYPE_ISO, &mmFile);
   else
      ret = mp_OpenFile(const_cast<char*>(fileToParse.c_str()), MP_FILE_TYPE_DMF, &mmFile);

   if (ret!= 0)
   {
      LogMsg(DeviceName(), AsString("ERROR in mp_OpenFile ") + AsString(ret));
      return -1;
   }

   //Get file info size 
   if ((ret = mp_GetFileInfoSize(&mmFile, &minFileInfoSize)) != 0)
   {
      LogMsg(DeviceName(), AsString("ERROR in mp_GetFileInfoSize ") + AsString(ret));
      return -1;
   }

   //Initialize necessary fields in File Info Desc structure
   fileInfoDesc.unVersion    = MP_FILE_INFO_DESC_VERSION;
   fileInfoDesc.fileInfoSize = minFileInfoSize;
   //allocate the memory that will hold the info retrieved by mp_GetFileInfo
   fileInfoDesc.pFileInfo    = new unsigned char[minFileInfoSize];

   //Get file info
   if ((ret = mp_GetFileInfo(&mmFile, &fileInfoDesc)) != 0)
   {
      LogMsg(DeviceName(), AsString("ERROR in mp_GetFileInfo ") + AsString(ret));
      return -1;
   }

   //Parse File Info and fill MM Codec structures
   if ((ret = ParseFileInfo(&fileInfoDesc, &m_videoCodec, &m_audioCodec)) != 0)
   {
      LogMsg(DeviceName(), AsString("ERROR in ParseFileInfo ") );
      return -1;
   }
   
   //Display the Audio and Video MM codec structures parameters
   PrintMMCodecParameters(&m_videoCodec, &m_audioCodec);


   //Close the file
   if ((ret = mp_CloseFile(&mmFile)) != 0)
   {
      LogMsg(DeviceName(), AsString("ERROR in mp_CloseFile ") + AsString(ret));
      return -1;
   }

   delete[] fileInfoDesc.pFileInfo;

   m_haveClip = true;

   return (0);
}

#endif




#ifdef USE_MTK_MP
//*****************************************************************************
// Function: void CMediaDevice::ParseFileInfo ()
// Description: verify file and stream level information retrieved
// Return: Error (-1) or Success (0)
// Parameters: MP_FILE_INFO_DESC *pFileInfoDesc
//             MM_VIDEO_CODEC *pVideoCodec
//             MM_AUDIO_CODEC *pAudioCodec
//*****************************************************************************
int CMediaDevice::ParseFileInfo (MP_FILE_INFO_DESC *pFileInfoDesc, MM_VIDEO_CODEC *pVideoCodec, MM_AUDIO_CODEC *pAudioCodec)
{

   unsigned char   *pFileInfo;         //pointer on the file info
   unsigned short  infoSizeLeft;       //size left in file info buffer
   unsigned int    blockSize;          //size of current block
   unsigned int    blockType;          //size of current block

   //Block structure pointers
   MP_BLK_HEADER               *pHdr;              //pointer on Header block
   MP_BLK_PRESENTATION         *pPresentation;     //pointer on Presentation block
   MP_BLK_STREAM_AUDIO         *pStreamAudio;      //pointer on Audio Stream block
   MP_BLK_STREAM_VIDEO         *pStreamVideo;      //pointer on Video Stream block
   MP_BLK_CODEC_AUDIO_AMR      *pAmr;              //pointer on AMR codec block
   MP_BLK_CODEC_VIDEO_H263     *pH263;             //pointer on H.263 block
   MP_BLK_CODEC_VIDEO_MPEG4    *pMpeg4;            //pointer on MPEG-4 block
   MP_BLK_CODEC_VIDEO_H264     *pH264;             //pointer on H.264 block



   if ((pFileInfoDesc->fileType != MP_FILE_TYPE_3GP) && (pFileInfoDesc->fileType != MP_FILE_TYPE_DMF) && (pFileInfoDesc->fileType != MP_FILE_TYPE_MP4)  && (pFileInfoDesc->fileType != MP_FILE_TYPE_ISO))
   {
      LogMsg(DeviceName(), AsString("ERROR in ParseFileInfo - File is not an ISO based or a DMF file - type = ") + AsString(pFileInfoDesc->fileType));
      return -1;
   }
   if (pFileInfoDesc->flags & MP_DESC_FLAG_INSUFF_SPACE)
   {
      LogMsg(DeviceName(), AsString("ERROR in ParseFileInfo - insuff space ") );
      return -1;
   }
   if (pFileInfoDesc->NbVideoTracks == 0)
   {
      LogMsg(DeviceName(), AsString("WARNING in ParseFileInfo - Nb Video Track = 0") );
      //return -1;
   }
   if (pFileInfoDesc->NbAudioTracks == 0)
   {
      LogMsg(DeviceName(), AsString("WARNING in ParseFileInfo - Nb Audio Track = 0") );
      //return -1;
   }


   //Initialize pointer to file info and info size
   pFileInfo = pFileInfoDesc -> pFileInfo;
   infoSizeLeft = pFileInfoDesc -> fileInfoSize;

   while (infoSizeLeft > 0)
   {
      //Determine blockSize and blockType
      pHdr = (MP_BLK_HEADER *)pFileInfo;
      blockSize = pHdr->blkSize;
      blockType = pHdr->blkType;

      switch (blockType)
      {
         case MP_BLK_TYPE_PRESENTATION:
            //LogMsg(DeviceName(), AsString("Presentation Block Info ") );
            pPresentation = (MP_BLK_PRESENTATION *)pFileInfo;

            break; 


         case MP_BLK_TYPE_STREAM_AUDIO:
            LogMsg(DeviceName(), AsString("Audio Stream Info ") );
            pStreamAudio = (MP_BLK_STREAM_AUDIO *)pFileInfo;

            LogMsg(DeviceName(), AsString("   MP Audio bitrate ") + AsString(pStreamAudio -> bitRate) + AsString("   MP Audio frameRate ") + AsString(pStreamAudio -> frameRate) );
            
            LogMsg(DeviceName(), AsString("   MP Audio Stream Duration ") + AsString(pStreamAudio -> duration) );
            if (pStreamAudio -> duration == 0)
            {
               LogMsg(DeviceName(), AsString("      WARNING: Audio Stream Duration = 0 ") );
            }

            break;


         case MP_BLK_TYPE_STREAM_VIDEO:
            LogMsg(DeviceName(), AsString("Video Stream Info ") );
            pStreamVideo = (MP_BLK_STREAM_VIDEO *)pFileInfo;

            LogMsg(DeviceName(), AsString("   MP Video bitrate ") + AsString(pStreamVideo -> bitRate) + AsString("   MP Video frameRate ") + AsString(pStreamVideo -> frameRate) );
            LogMsg(DeviceName(), AsString("   MP Video width ") + AsString(pStreamVideo -> width) + AsString("   MP Video height ") + AsString(pStreamVideo -> height) );
            LogMsg(DeviceName(), AsString("   MP Video sample Rate ") + AsString( pStreamVideo -> sampleRate ) );
            LogMsg(DeviceName(), AsString("   MP Video Stream Duration ") + AsString(pStreamVideo -> duration) );

            if (pStreamVideo -> duration == 0)
            {
               LogMsg(DeviceName(), AsString("      WARNING: Video Stream Duration = 0 ") );
            }

            if (pStreamVideo -> bitRate > 0) {
               pVideoCodec->BitRate      = (eVIDEO_BITRATE)(pStreamVideo -> bitRate * 1000);
               LogMsg(DeviceName(), AsString("   Setting Video bitrate in bps ") + AsString( (eVIDEO_BITRATE)(pStreamVideo -> bitRate * 1000)) );
            }
            else {
               LogMsg(DeviceName(), AsString("      WARNING: Video Stream bitRate = 0 ") );
            }


            if (pStreamVideo -> frameRate > 0) {
               int a = (int)pStreamVideo -> frameRate;
               int b = (int)((pStreamVideo -> frameRate-(double)a)*100.0);
               int fps = a << 16 | b;
               
               LogMsg(DeviceName(), AsString("   Setting Video Frame Rate ") + AsHexString(fps) );
               pVideoCodec->FramesPerSec     = (eVIDEO_FRAMESPERSEC)fps;
            }
            else {
               LogMsg(DeviceName(), AsString("      WARNING: Video Stream frameRate = 0 ") );
            }


            if (pStreamVideo -> sampleRate == 90000) {
               LogMsg(DeviceName(), AsString("   Setting sampleRate = 90KHz ") );
            }
            else {
               LogMsg(DeviceName(), AsString("      WARNING: Video Stream sampleRate != 90000  - got :") + AsString( pStreamVideo -> sampleRate ) );
            }
            pVideoCodec->SamplingRate      = VIDEO_SAMPLING_RATE_90000;


            if ((pStreamVideo -> width == 128) && (pStreamVideo -> height == 96))
            {
               LogMsg(DeviceName(), AsString("      This is a SQCIF Video ") );
               pVideoCodec->ImageWidth       = VIDEO_IMAGE_WIDTH_SQCIF;
               pVideoCodec->ImageHeight      = VIDEO_IMAGE_HEIGHT_SQCIF;
            }
            else if ((pStreamVideo -> width == 176) && (pStreamVideo -> height == 144))
            {
               LogMsg(DeviceName(), AsString("      This is a QCIF Video ") );
               pVideoCodec->ImageWidth       = VIDEO_IMAGE_WIDTH_QCIF;
               pVideoCodec->ImageHeight      = VIDEO_IMAGE_HEIGHT_QCIF;
            }
            else if ((pStreamVideo -> width == 352) && (pStreamVideo -> height == 288))
            {
               LogMsg(DeviceName(), AsString("      This is a CIF Video ") );
               pVideoCodec->ImageWidth       = VIDEO_IMAGE_WIDTH_CIF;
               pVideoCodec->ImageHeight      = VIDEO_IMAGE_HEIGHT_CIF;
            }
#ifdef ENABLE_VGA
            else if ((pStreamVideo -> width == 640) && (pStreamVideo -> height == 480))
            {
               LogMsg(DeviceName(), AsString("      This is a VGA Video ") );
               pVideoCodec->ImageWidth       = VIDEO_IMAGE_WIDTH_VGA;
               pVideoCodec->ImageHeight      = VIDEO_IMAGE_HEIGHT_VGA;
            }
            else if ((pStreamVideo -> width == 704) && (pStreamVideo -> height == 576))
            {
               LogMsg(DeviceName(), AsString("      This is a 4CIF Video ") );
               pVideoCodec->ImageWidth       = VIDEO_IMAGE_WIDTH_4CIF;
               pVideoCodec->ImageHeight      = VIDEO_IMAGE_HEIGHT_4CIF;
            }
#endif
            else 
            {
               LogMsg(DeviceName(), AsString("      WARNING: unsupported video resolution : ") + AsString(pStreamVideo -> width) + AsString(" x ") + AsString(pStreamVideo -> height)  );
               if (pStreamVideo -> width <= 128)
                  pVideoCodec->ImageWidth       = VIDEO_IMAGE_WIDTH_SQCIF;
               else if (pStreamVideo -> width <= 176)
                  pVideoCodec->ImageWidth       = VIDEO_IMAGE_WIDTH_QCIF;
#ifdef ENABLE_VGA
               else if (pStreamVideo -> width <= 352)
                  pVideoCodec->ImageWidth       = VIDEO_IMAGE_WIDTH_CIF;
               else if (pStreamVideo -> width <= 640)
                  pVideoCodec->ImageWidth       = VIDEO_IMAGE_WIDTH_VGA;
               else 
                  pVideoCodec->ImageWidth       = VIDEO_IMAGE_WIDTH_4CIF;
#else
               else 
                  pVideoCodec->ImageWidth       = VIDEO_IMAGE_WIDTH_CIF;
#endif
               if (pStreamVideo -> height <= 96)
                  pVideoCodec->ImageHeight       = VIDEO_IMAGE_HEIGHT_SQCIF;
               else if (pStreamVideo -> height <= 144)
                  pVideoCodec->ImageHeight       = VIDEO_IMAGE_HEIGHT_QCIF;
#ifdef ENABLE_VGA
               else if (pStreamVideo -> height <= 288)
                  pVideoCodec->ImageHeight       = VIDEO_IMAGE_HEIGHT_CIF;
               else if (pStreamVideo -> height <= 480)
                  pVideoCodec->ImageHeight       = VIDEO_IMAGE_HEIGHT_VGA;
               else 
                  pVideoCodec->ImageHeight       = VIDEO_IMAGE_HEIGHT_4CIF;
#else
               else 
                  pVideoCodec->ImageHeight       = VIDEO_IMAGE_HEIGHT_CIF;
#endif
               LogMsg(DeviceName(), AsString("      Selecting the closest higher resolution : ") + AsString(pVideoCodec->ImageWidth) + AsString (" x ") + AsString(pVideoCodec->ImageHeight));

            }

            break;


         case MP_BLK_TYPE_AUDIO_CODEC_AMR:
            if (pStreamAudio->codec == MP_ISO_AUD_CODEC_AMR_NB )
            {
               LogMsg(DeviceName(), AsString("   AMR NB Codec Info: ") );
               pAmr = (MP_BLK_CODEC_AUDIO_AMR *)pFileInfo;

               LogMsg(DeviceName(), AsString("      MP sampleRate ") + AsString(pStreamAudio -> sampleRate) + AsString("      MP bitsPerSample ") + AsString(pStreamAudio -> bitsPerSample)  );
               LogMsg(DeviceName(), AsString("      MP Packetization Mode ") + AsString(pAmr -> packetizationMode) );

               // Audio bitrate does not matter for MM Play
               pAudioCodec->unCoding         = MM_DATA_FORMAT_AMR_NB_12_20K;

               if (pStreamAudio -> sampleRate == 8000) {
                  LogMsg(DeviceName(), AsString("   Setting 8KHz Sampling Rate ") );
               }
               else {
                  LogMsg(DeviceName(), AsString("   WARNING - Sampling rate shall be 8KHz instead of ") + AsString(pStreamAudio -> sampleRate) );
                  return -1;
               }
               pAudioCodec->unSampleRate     = MM_DRT_8KHZ;
               
            }
            else if (pStreamAudio->codec == MP_ISO_AUD_CODEC_AMR_WB)
            {
               LogMsg(DeviceName(), AsString("   AMR WB Codec Info: ") );
               pAmr = (MP_BLK_CODEC_AUDIO_AMR *)pFileInfo;

               LogMsg(DeviceName(), AsString("      MP sampleRate ") + AsString(pStreamAudio -> sampleRate) + AsString("      MP bitsPerSample ") + AsString(pStreamAudio -> bitsPerSample)  );
               LogMsg(DeviceName(), AsString("      MP Packetization Mode ") + AsString(pAmr -> packetizationMode) );

               // Audio bitrate does not matter for MM Play
               pAudioCodec->unCoding         = MM_DATA_FORMAT_AMR_WB_23_85K;

               if (pStreamAudio -> sampleRate == 16000) {
                  LogMsg(DeviceName(), AsString("   Setting 16KHz Sampling Rate ") );
               }
               else {
                  LogMsg(DeviceName(), AsString("   WARNING - Sampling rate shall be 8KHz instead of ") + AsString(pStreamAudio -> sampleRate) );
               }
               pAudioCodec->unSampleRate     = MM_DRT_16KHZ;

            }
            else
            {
               LogMsg(DeviceName(), AsString("ERROR - INCORRECT CODEC (%d) DURING AMR CODEC BLOCK  ") );
               return -1;
            }



            if ( (pStreamAudio->codec == MP_ISO_AUD_CODEC_AMR_NB ) || (pStreamAudio->codec == MP_ISO_AUD_CODEC_AMR_WB))
            {
               PMM_AUDIO_CODEC_OPTION_LIST audioOption = new(MM_AUDIO_CODEC_OPTION_LIST);
               INIT_MM_AUDIO_CODEC_OPTION_LIST(audioOption);
               pAudioCodec->options=audioOption;

               audioOption->eParm=EMM_AUDIO_CODEC_OPTION_PAYLOAD_FORMAT;

               if (  (pAmr -> packetizationMode == MP_S_PACK_AMR_OCTET_ALIGN) || (pAmr -> packetizationMode == MP_S_PACK_NONE) || (pAmr -> packetizationMode == MP_S_PACK_UNKNWN) )
               {
                  LogMsg(DeviceName(), AsString("      AMR is packetized in Octet Aligned, or is not packetized") );
                  audioOption->nValue = EMM_AUDIO_PAYLOAD_FORMAT_OCTET_ALIGNED;
               }
               else if (pAmr -> packetizationMode == MP_S_PACK_AMR_BW_EFF)
               {
                  LogMsg(DeviceName(), AsString("      AMR is packetized in BW efficient mode") );
                  audioOption->nValue = EMM_AUDIO_PAYLOAD_FORMAT_BANDWIDTH_EFFICIENT;
               }
               else
               {
                  LogMsg(DeviceName(), AsString("      ERROR: AMR Packetization Mode is not supported ") + AsString(pAmr -> packetizationMode) );
                  return -1;
               }


               if (pStreamAudio -> bitsPerSample == 16) {
                  LogMsg(DeviceName(), AsString("   Setting 16 bits per sample ") );
                  pAudioCodec->unBitsPerSample  = (unsigned int)pStreamAudio -> bitsPerSample;
               }
               else {
                  LogMsg(DeviceName(), AsString("   WARNING - Audio bits per sample shall be 16 instead of ") + AsString(pStreamAudio -> bitsPerSample) );
                  pAudioCodec->unBitsPerSample  = 16;
               }
            }

            break;

         case MP_BLK_TYPE_VIDEO_CODEC_H263:
            if (pStreamVideo->codec == MP_VID_CODEC_H263)
            {
               LogMsg(DeviceName(), AsString("   H.263 Codec Info ") );
               pH263 = (MP_BLK_CODEC_VIDEO_H263 *)pFileInfo;

               LogMsg(DeviceName(), AsString("      MP Packetization Mode ") + AsString(pH263 -> packetizationMode) + AsString(" MP profile ") + AsString((eVIDEO_PROFILE)(pH263 -> profile)) 
                  + AsString(" MP level ") + AsString((eVIDEO_LEVEL)(pH263 -> level)));

               if (pH263 -> packetizationMode == MP_S_PACK_H263_2190)
               {
                  LogMsg(DeviceName(), AsString("      H263 is packetized with RFC2190") );
                  pVideoCodec->Coding            = VIDEO_CODING_H263;

               }
               else if ((pH263 -> packetizationMode == MP_S_PACK_H263_2429) || (pH263 -> packetizationMode == MP_S_PACK_NONE) || (pH263 -> packetizationMode == MP_S_PACK_UNKNWN) )
               {
                  LogMsg(DeviceName(), AsString("      H263 is packetized with RFC2429, or is not packetized") );
                  pVideoCodec->Coding            = VIDEO_CODING_H263_1998;
               }
               else
               {
                  LogMsg(DeviceName(), AsString("      ERROR: H263 Packetization Mode is not supported ") + AsString(pH263 -> packetizationMode) );
                  return -1;
               }

               pVideoCodec -> Profile = (eVIDEO_PROFILE)pH263 -> profile;
               pVideoCodec -> Level = (eVIDEO_LEVEL)(pH263 -> level);
            }
            else
            {
               LogMsg(DeviceName(), AsString("ERROR - INCORRECT CODEC (%d) DURING H263 CODEC BLOCK  ") );
               return -1;
            }

            break;

         case MP_BLK_TYPE_VIDEO_CODEC_MPEG4:
            if (pStreamVideo->codec == MP_VID_CODEC_MPEG4)
            {
               LogMsg(DeviceName(), AsString("   MPEG-4 Codec Info ") );
               pMpeg4 = (MP_BLK_CODEC_VIDEO_MPEG4 *)pFileInfo;

               LogMsg(DeviceName(), AsString("      MP Packetization Mode ") + AsString(pMpeg4 -> packetizationMode) 
                  + AsString(" MP profile_and_level_ind ") + AsString((eVIDEO_PROFILE)(pMpeg4 -> profile_and_level_ind)) );
               LogMsg(DeviceName(), AsString("      DCI size ") + AsString(pMpeg4 -> dcSpecInfoSize) );

               pVideoCodec->Coding            = VIDEO_CODING_MP4V_ES;
               
               pVideoCodec -> Profile = (eVIDEO_PROFILE)(pMpeg4 -> profile_and_level_ind);

               if ( pMpeg4 -> dcSpecInfoSize > 0 )
               {
                  unsigned char *dci = new unsigned char[pMpeg4 -> dcSpecInfoSize & MMRSC_DCI_SIZE];
                  // truncate it, as MP supports 512 bytes DCI max size, while MMRSC supports 255 bytes DCI max size
                  pVideoCodec->VisualConfigSize = (unsigned int )pMpeg4 -> dcSpecInfoSize & MMRSC_DCI_SIZE;
                  LogMsg(DeviceName(), AsString("      Setting MPEG4 DCI - size = ") + AsString(pVideoCodec->VisualConfigSize) );
                  memcpy(dci, pMpeg4 -> pDcSpecInfo, pVideoCodec->VisualConfigSize );
                  pVideoCodec -> VisualConfiguration = dci;
               }
               else {
                  LogMsg(DeviceName(), AsString("      ERROR: MPEG4  DCI - size = ") + AsString(pMpeg4 -> dcSpecInfoSize));
                  return -1;
               }

            }
            else
            {
               LogMsg(DeviceName(), AsString("ERROR - INCORRECT CODEC (%d) DURING MPEG4 CODEC BLOCK  ") );
               return -1;
            }

            break;


         case MP_BLK_TYPE_VIDEO_CODEC_H264:
            if (pStreamVideo->codec == MP_VID_CODEC_H264)
            {
               LogMsg(DeviceName(), AsString("   H264 Codec Info ") );
               pH264 = (MP_BLK_CODEC_VIDEO_H264 *)pFileInfo;

               LogMsg(DeviceName(), AsString("      MP Packetization Mode ") + AsString(pH264 -> packetizationMode) + AsString(" MP profile ") + AsHexString((eVIDEO_PROFILE)(pH264 -> profile)) + 
                  AsString("  MP profile_iop ") + AsHexString((eH264_PROFILE_IOP)(pH264 -> profile_iop)) + AsString("  MP level ") + AsHexString((eVIDEO_LEVEL)(pH264 -> level)));
               LogMsg(DeviceName(), AsString("      DCI size ") + AsString(pH264 -> dcSpecInfoSize) );

               pVideoCodec->Coding            = VIDEO_CODING_H264;

               pVideoCodec -> Profile = (eVIDEO_PROFILE)pH264 -> profile;
               pVideoCodec -> H264_ProfileIOP = (unsigned int)(pH264 -> profile_iop);
               pVideoCodec -> Level = (eVIDEO_LEVEL)(pH264 -> level);

               if ( pH264 -> dcSpecInfoSize > 0 )
               {
                  unsigned char *dci = new unsigned char[pH264 -> dcSpecInfoSize & MMRSC_DCI_SIZE];
                  // truncate it, as MP supports 512 bytes DCI max size, while MMRSC supports 255 bytes DCI max size
                  pVideoCodec->VisualConfigSize = (unsigned int )pH264 -> dcSpecInfoSize & MMRSC_DCI_SIZE;
                  LogMsg(DeviceName(), AsString("      Setting H264 DCI - size = ") + AsString(pVideoCodec->VisualConfigSize) );
                  memcpy(dci, pH264 -> pDcSpecInfo, pVideoCodec->VisualConfigSize );
                  pVideoCodec -> VisualConfiguration = dci;
               }
               else {
                  LogMsg(DeviceName(), AsString("      ERROR: H264  DCI - size = ") + AsString(pH264 -> dcSpecInfoSize) );
                  return -1;
               }


               if (pH264 -> packetizationMode == MP_S_PACK_H264_MODE0) {
                  pVideoCodec -> H264_PacketizationMode = H264_PACKETIZATION_MODE_SINGLE_NAL;
                  LogMsg(DeviceName(), AsString("      H264 Packetization Mode is Single NAL ") );
               }
               else if ( (pH264 -> packetizationMode == MP_S_PACK_H264_MODE1) || (pH264 -> packetizationMode == MP_S_PACK_NONE) || (pH264 -> packetizationMode == MP_S_PACK_UNKNWN) ) {
                  pVideoCodec -> H264_PacketizationMode = H264_PACKETIZATION_MODE_NON_INTERLEAVED;
                  LogMsg(DeviceName(), AsString("      H264 Packetization Mode is Non Interleaved or is not packetized") );
               }
               else {
                  LogMsg(DeviceName(), AsString("      ERROR: H264 Packetization Mode is not supported ") + AsString(pH264 -> packetizationMode) );
                  return -1;
               }

            }
            else
            {
               LogMsg(DeviceName(), AsString("ERROR - INCORRECT CODEC (%d) DURING H264 CODEC BLOCK ") );
               return -1;
            }

            break;

         case MP_BLK_TYPE_AUDIO_CODEC_AAC:
            LogMsg(DeviceName(), AsString("WARNING: AAC is not supported ") );
            break;

         default:
            //LogMsg(DeviceName(), AsString("Unknown block type - Skipping ") );

            break;
      }   // end of switch (blkType)

      // go to next block
      pFileInfo += blockSize;
      infoSizeLeft -= blockSize;

   }   // end of while (infoSizeLeft >0)

   if (pAudioCodec->unCoding == 0)
   {
      LogMsg(DeviceName(), AsString("No Audio Codec in the file ") );
   }

   return (0);
}
#endif



//*****************************************************************************
// Function: void CMediaDevice::PrintMMCodecParameters ()
// Description: verify file and stream level information retrieved
// Return: void
// Parameters: 
//             MM_VIDEO_CODEC *pVideoCodec
//             MM_AUDIO_CODEC *pAudioCodec
//*****************************************************************************
void CMediaDevice::PrintMMCodecParameters (MM_VIDEO_CODEC *pVideoCodec, MM_AUDIO_CODEC *pAudioCodec)
{

   LogMsg(DeviceName(), AsString("MM Video Codec Parameters: ") );
   
   LogMsg(DeviceName(), AsString("   unVersion ") + AsHexString(pVideoCodec -> unVersion) );
   LogMsg(DeviceName(), AsString("   Coding ") + AsString(pVideoCodec -> Coding) );
   LogMsg(DeviceName(), AsString("   Profile ") + AsString(pVideoCodec -> Profile) );
   LogMsg(DeviceName(), AsString("   Level ") + AsString(pVideoCodec -> Level) );
   LogMsg(DeviceName(), AsString("   ImageWidth ") + AsString(pVideoCodec -> ImageWidth) );
   LogMsg(DeviceName(), AsString("   ImageHeight ") + AsString(pVideoCodec -> ImageHeight) );
   LogMsg(DeviceName(), AsString("   BitRate ") + AsString(pVideoCodec -> BitRate) );
   LogMsg(DeviceName(), AsString("   FramesPerSec ") + AsHexString(pVideoCodec -> FramesPerSec) );
   LogMsg(DeviceName(), AsString("   VisualConfigSize ") + AsString(pVideoCodec -> VisualConfigSize) );

   if (pVideoCodec -> VisualConfigSize > 0)
   {
      string dci;
      char tmp[10];
      int i;
      for (i=0; i<pVideoCodec -> VisualConfigSize; i++)
      {
         sprintf(tmp,"0x%02x ",pVideoCodec -> VisualConfiguration[i]);
         dci += tmp;
      }
      LogMsg(DeviceName(), AsString("   VisualConfiguration ") +  dci);
   }

   LogMsg(DeviceName(), AsString("   H264_AcceptRedundantSlices ") + AsString(pVideoCodec -> H264_AcceptRedundantSlices) );
   LogMsg(DeviceName(), AsString("   H264_ProfileIOP ") + AsHexString(pVideoCodec -> H264_ProfileIOP) );
   LogMsg(DeviceName(), AsString("   H264_PacketizationMode ") + AsString(pVideoCodec -> H264_PacketizationMode) );
   LogMsg(DeviceName(), AsString("   H264_MaxNalUnitSize ") + AsString(pVideoCodec -> H264_MaxNalUnitSize) );
    


   LogMsg(DeviceName(), AsString("MM Audio Codec Parameters: ") );
   
   LogMsg(DeviceName(), AsString("   unVersion ") + AsHexString(pAudioCodec -> unVersion) );
   LogMsg(DeviceName(), AsString("   unCoding ") + AsString(pAudioCodec -> unCoding) );  
   LogMsg(DeviceName(), AsString("   unSampleRate ") + AsString(pAudioCodec -> unSampleRate) );
   LogMsg(DeviceName(), AsString("   unBitsPerSample ") + AsString(pAudioCodec -> unBitsPerSample) );

   if (pAudioCodec->options != (CPMM_AUDIO_CODEC_OPTION_LIST)0)
   {
      LogMsg(DeviceName(), AsString("   options eParm ") + AsHexString(pAudioCodec->options->eParm) );
      LogMsg(DeviceName(), AsString("   options value ") + AsHexString(pAudioCodec->options->nValue) );
   }
}


#ifndef USE_MTK_MP
//*****************************************************************************
// Function: void CMediaDevice::SetClip(const std::string &a_clip)
// Description: Set the base name of the multimedia DMF clip to play
// Return: void
// Parameters: const std::string& a_path,
//             const std::string &a_clip
//*****************************************************************************
int CMediaDevice::SetClip(const std::string& a_path, const std::string& a_clip)
{

   if (a_clip.size()>0)
   {
      m_audioFile = a_path + a_clip + ".pcm";
      m_videoFile = a_path + a_clip + ".vid";
      m_haveClip = true;
      m_is3gp = false;
      LogMsg(DeviceName(), AsString("SetDmfClip:") + m_audioFile + AsString(":") + m_videoFile);

      INIT_MM_VIDEO_CODEC(&m_videoCodec);
      m_videoCodec.FramesPerSec     = g_mmFramesPerSec;
      m_videoCodec.ImageHeight      = g_mmImageHeight;
      m_videoCodec.ImageWidth       = g_mmImageWidth;

#if 0
#ifdef ENABLE_VGA
      // Phil: that should be configurable
      m_videoCodec.Coding            = VIDEO_CODING_H264;
      //m_videoCodec.Level            = VIDEO_LEVEL_2_2_H264;
      m_videoCodec.Level            = VIDEO_LEVEL_3_1_H264;
      m_videoCodec.Profile          = VIDEO_PROFILE_BASELINE_H264;
      m_videoCodec.BitRate          = VIDEO_BITRATE_768K;  //req: use 768K to allocate mem buf
#else
      m_videoCodec.Level            = EMM_VIDEO_LEVEL_UNDEFINED;
      m_videoCodec.Profile          = EMM_VIDEO_PROFILE_UNDEFINED;
      m_videoCodec.BitRate          = VIDEO_BITRATE_384K;  //req: use 384K to allocate mem buf
#endif
#endif
      // Get the codec type from the file name
      if (a_clip.find("h264") != string::npos)
      {
         LogMsg(DeviceName(), AsString("h264 found in file name") );
         m_videoCodec.Coding            = VIDEO_CODING_H264;
      }
      else if (a_clip.find("mpeg4") != string::npos)
      {
         LogMsg(DeviceName(), AsString("mpeg4 found in file name") );
         m_videoCodec.Coding            = VIDEO_CODING_MP4V_ES;
      }
      else if (a_clip.find("h263") != string::npos)
      {
         LogMsg(DeviceName(), AsString("h263 found in file name") );
         m_videoCodec.Coding            = VIDEO_CODING_H263;
      }
      else if (a_clip.find("h263_1998") != string::npos)
      {
         LogMsg(DeviceName(), AsString("h263_1998 found in file name") );
         m_videoCodec.Coding            = VIDEO_CODING_H263_1998; 
      }
      else
      {

         LogMsg(DeviceName(), AsString("CODEC NOT found in file name. Using H263") );
         m_videoCodec.Coding            = VIDEO_CODING_H263;  // Default for DMF
      }

      // Get the picture resolution from the file name
      if (a_clip.find("qcif") != string::npos)
      {
         LogMsg(DeviceName(), AsString("qcif found in file name") );
         m_videoCodec.ImageHeight      = VIDEO_IMAGE_HEIGHT_QCIF;
         m_videoCodec.ImageWidth       = VIDEO_IMAGE_WIDTH_QCIF;
         m_videoCodec.BitRate          = VIDEO_BITRATE_384K;  //req: use 384K to allocate mem buf
         if (m_videoCodec.Coding==VIDEO_CODING_H264) { 
            m_videoCodec.Level            = VIDEO_LEVEL_1_2_H264;
            m_videoCodec.Profile          = VIDEO_PROFILE_BASELINE_H264;
         }
         else if (m_videoCodec.Coding==VIDEO_CODING_MP4V_ES){
            m_videoCodec.Level            = VIDEO_LEVEL_DEFAULT;
            m_videoCodec.Profile          = VIDEO_PROFILE_LEVEL_SP0_MPEG4;
         }
         else if (m_videoCodec.Coding==VIDEO_CODING_H263_1998){
            m_videoCodec.Level            = VIDEO_LEVEL_20_H263;
            m_videoCodec.Profile          = VIDEO_PROFILE_0_H263;
         }
         else if (m_videoCodec.Coding==VIDEO_CODING_H263){
            m_videoCodec.Level            = VIDEO_LEVEL_20_H263;
            m_videoCodec.Profile          = VIDEO_PROFILE_0_H263;
         }
      
      }
      else if (a_clip.find("sqcif") != string::npos)
      {
         LogMsg(DeviceName(), AsString("sqcif found in file name") );
         m_videoCodec.ImageHeight      = VIDEO_IMAGE_HEIGHT_SQCIF;
         m_videoCodec.ImageWidth       = VIDEO_IMAGE_WIDTH_SQCIF;
         m_videoCodec.BitRate          = VIDEO_BITRATE_384K;  //req: use 384K to allocate mem buf
         if (m_videoCodec.Coding==VIDEO_CODING_H264) { 
            m_videoCodec.Level            = VIDEO_LEVEL_1_H264;
            m_videoCodec.Profile          = VIDEO_PROFILE_BASELINE_H264;
         }
         else if (m_videoCodec.Coding==VIDEO_CODING_MP4V_ES){
            m_videoCodec.Level            = VIDEO_LEVEL_DEFAULT;
            m_videoCodec.Profile          = VIDEO_PROFILE_LEVEL_SP0_MPEG4;
         }
         else if (m_videoCodec.Coding==VIDEO_CODING_H263_1998){
            m_videoCodec.Level            = VIDEO_LEVEL_10_H263;
            m_videoCodec.Profile          = VIDEO_PROFILE_0_H263;
         }
         else if (m_videoCodec.Coding==VIDEO_CODING_H263){
            m_videoCodec.Level            = VIDEO_LEVEL_10_H263;
            m_videoCodec.Profile          = VIDEO_PROFILE_0_H263;
         }
      }
#ifdef ENABLE_VGA
      else if (a_clip.find("vga") != string::npos)
      {
         LogMsg(DeviceName(), AsString("vga found in file name") );
         m_videoCodec.ImageHeight      = VIDEO_IMAGE_HEIGHT_VGA;
         m_videoCodec.ImageWidth       = VIDEO_IMAGE_WIDTH_VGA;
         m_videoCodec.BitRate          = VIDEO_BITRATE_768K;  //req: use 768K to allocate mem buf
         if (m_videoCodec.Coding==VIDEO_CODING_H264) { 
            m_videoCodec.Level            = VIDEO_LEVEL_3_1_H264;
            m_videoCodec.Profile          = VIDEO_PROFILE_BASELINE_H264;
         }
         else if (m_videoCodec.Coding==VIDEO_CODING_MP4V_ES){
            m_videoCodec.Level            = VIDEO_LEVEL_DEFAULT;
            m_videoCodec.Profile          = VIDEO_PROFILE_LEVEL_SP0_MPEG4;
         }
         else if (m_videoCodec.Coding==VIDEO_CODING_H263_1998){
            m_videoCodec.Level            = VIDEO_LEVEL_20_H263;
            m_videoCodec.Profile          = VIDEO_PROFILE_0_H263;
         }
         else if (m_videoCodec.Coding==VIDEO_CODING_H263){
            m_videoCodec.Level            = VIDEO_LEVEL_20_H263;
            m_videoCodec.Profile          = VIDEO_PROFILE_0_H263;
         }
      }
#endif
      else {
         if (a_clip.find("cif") != string::npos){
            LogMsg(DeviceName(), AsString("cif found in file name") );
         }
         else {
            LogMsg(DeviceName(), AsString("PICTURE RESOLUTION NOT found in file name. Using CIF") );
         }
         m_videoCodec.ImageHeight      = VIDEO_IMAGE_HEIGHT_CIF;
         m_videoCodec.ImageWidth       = VIDEO_IMAGE_WIDTH_CIF;
         m_videoCodec.BitRate          = VIDEO_BITRATE_384K;  //req: use 384K to allocate mem buf
         if (m_videoCodec.Coding==VIDEO_CODING_H264) { 
            m_videoCodec.Level            = VIDEO_LEVEL_1_3_H264;
            m_videoCodec.Profile          = VIDEO_PROFILE_BASELINE_H264;
         }
         else if (m_videoCodec.Coding==VIDEO_CODING_MP4V_ES){
            m_videoCodec.Level            = VIDEO_LEVEL_DEFAULT;
            m_videoCodec.Profile          = VIDEO_PROFILE_LEVEL_SP3_MPEG4;
         }
         else if (m_videoCodec.Coding==VIDEO_CODING_H263_1998){
            m_videoCodec.Level            = VIDEO_LEVEL_30_H263;
            m_videoCodec.Profile          = VIDEO_PROFILE_0_H263;
         }
         else if (m_videoCodec.Coding==VIDEO_CODING_H263){
            m_videoCodec.Level            = VIDEO_LEVEL_30_H263;
            m_videoCodec.Profile          = VIDEO_PROFILE_0_H263;
         }
      }

      INIT_MM_AUDIO_CODEC(&m_audioCodec);
      m_audioCodec.unCoding         = MM_DATA_FORMAT_PCM;
      m_audioCodec.unSampleRate     = 8000;
      m_audioCodec.unBitsPerSample  = 16;


   }

   return (0);
}

#endif

//*****************************************************************************
// Function: void CMediaDevice::SetRecordClip(const std::string &a_clip)
// Description: Set the base name of the multimedia clip to play
// Return: void
// Parameters: const std::string& a_path,
//             const std::string &a_clip
//*****************************************************************************
void CMediaDevice::SetRecordClip(const std::string& a_path, const std::string& a_clip)
{
   if (a_clip.size()>0)
   {
      m_recAudioFile = a_path + a_clip + ".pcm";
      m_recVideoFile = a_path + a_clip + ".vid";
      m_haveClip = true;
      LogMsg(DeviceName(), AsString("SetRecordClip:") + m_recAudioFile + AsString(":") + m_recVideoFile);
   }
}


//*****************************************************************************
// Function: void CMediaDevice::SetRecordCodec(eMM_VIDEO_CODING a_recVideoCoding)
// Description: Set the record Coding
// Return: void
// Parameters: eMM_VIDEO_CODING a_recVideoCoding
//*****************************************************************************
void CMediaDevice::SetRecordCoding(eMM_VIDEO_CODING a_recVideoCoding)
{
   if (a_recVideoCoding == VIDEO_CODING_H264)
   {
      m_recVideoCoding = a_recVideoCoding;
      LogMsg(DeviceName(), AsString("SetRecordCoding:H264:") + AsString( m_recVideoCoding));
   }
   else if (a_recVideoCoding == VIDEO_CODING_H263)
   {
      m_recVideoCoding = a_recVideoCoding;
      LogMsg(DeviceName(), AsString("SetRecordCoding:H263:") + AsString( m_recVideoCoding));
   }
   else
   {
      LogMsg(DeviceName(), AsString("WARNING: SetRecordCoding:UnSupported:") + AsString( m_recVideoCoding));
   }
}

//*****************************************************************************
// All device and call operations and event handling are delegated to the state
// object with the following functions
//*****************************************************************************

int CMediaDevice::Open() {return DeviceState().MmOpen(*this);}
int CMediaDevice::Close() {return DeviceState().MmClose(*this);}
int CMediaDevice::OnMmOpened(METAEVENT& a_metaEvent) {return DeviceState().OnMmOpened(*this, a_metaEvent);}
int CMediaDevice::MmPlay() {return DeviceState().MmPlay(*this);}
int CMediaDevice::OnMmPlayComplete(METAEVENT& a_metaEvent) {return DeviceState().OnMmPlayComplete(*this, a_metaEvent);}
int CMediaDevice::OnMmPlayTimeoutComplete() {return DeviceState().OnMmPlayTimeoutComplete(*this);}
int CMediaDevice::OnMmPlayFail(METAEVENT& a_metaEvent) {return DeviceState().OnMmPlayFail(*this, a_metaEvent);}
int CMediaDevice::MmRecord() {return DeviceState().MmRecord(*this);}
int CMediaDevice::OnMmRecordComplete(METAEVENT& a_metaEvent) {return DeviceState().OnMmRecordComplete(*this, a_metaEvent);}
int CMediaDevice::OnMmRecordTimeoutComplete() {return DeviceState().OnMmRecordTimeoutComplete(*this);}
int CMediaDevice::OnMmRecordFail(METAEVENT& a_metaEvent) {return DeviceState().OnMmRecordFail(*this, a_metaEvent);}
int CMediaDevice::MmStop() {return DeviceState().MmStop(*this);}
int CMediaDevice::OnMmStopAck(METAEVENT& a_metaEvent) {return DeviceState().OnMmStopAck(*this, a_metaEvent);}

//*****************************************************************************
// Function: int CMediaDevice::PrimMmOpen()
// Description: Handle open MM device command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CMediaDevice::PrimMmOpen()
{
   int res = mm_Open(DeviceName().c_str(), 0, NULL);
   if (res == -1)
   {
      LogMsg(DeviceName(), AsString(" mm_Open failed"));
   }
   else
   {
      Device(res);
      CGcManager::RegisterDevice(Device(), this);
      LogMsg(DeviceName(), "mm_Open");
   }
   return res;
}

//*****************************************************************************
// Function: int CMediaDevice::PrimOnMmOpened(METAEVENT &a_metaEvent)
// Description: Handle open MM device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDevice::PrimOnMmOpened(METAEVENT& a_metaEvent)
{
   CGcManager::AddOpenDevice(this);
// SKS to shift this after conference started
   m_observer->PostEvent(APPEVENT_MM_OPEN_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CMediaDevice::PrimMmClose()
// Description: Handle close MM device command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CMediaDevice::PrimMmClose()
{
   int res(0);
   if (Device() != -1)
   {
      // delete MM dynanic buffers
      if (m_audioCodec.options != (CPMM_AUDIO_CODEC_OPTION_LIST)0)
      {
         delete (m_audioCodec.options);
      }
      if (m_videoCodec.VisualConfigSize > 0)
      {
         delete[] m_videoCodec.VisualConfiguration;
      }
      
      res = mm_Close(Device(), 0);
      if (res != GC_SUCCESS)
      {
         ErrMsg(Device(),"mm_Close");
      }
      else
      {
         LogMsg(DeviceName(), "mm_Close");
         CGcManager::RemoveOpenDevice(this);
      }
      Device(-1);
   }
   else
   {
      LogMsg(DeviceName(), "device already closed");
   }

   return res;
}

//*****************************************************************************
// Function: int CMediaDevice::GetReceivePortInfo()
// Description: Get receive port info
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CMediaDevice::GetReceivePortInfo()
{
   int res = dev_GetReceivePortInfo(Device(), (void*)(ID()));
   if (res == -1)
   {
      ErrMsg(Device(), "dev_GetReceivePortInfo");
   }
   else
   {
      LogMsg(Device(), "dev_GetReceivePortInfo");
   }
   return res;
}

//*****************************************************************************
// Function: int CMediaDevice::OnGetReceivePortInfoComplete(METAEVENT &a_metaEvent)
// Description: Handle get receive port info completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDevice::OnGetReceivePortInfoComplete(METAEVENT& a_metaEvent)
{
   DM_PORT_INFO_LIST* l_portInfoList = reinterpret_cast<DM_PORT_INFO_LIST*>(a_metaEvent.evtdatap);
   m_rxPortInfoList = *l_portInfoList;
   m_observer->PostEvent(APPEVENT_MM_GET_RX_PORT_INFO_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CMediaDevice::GetTransmitPortInfo()
// Description: Get transmit port info
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CMediaDevice::GetTransmitPortInfo()
{
   int res = dev_GetTransmitPortInfo(Device(), 0);
   if (res == -1)
   {
      ErrMsg(Device(), "dev_GetTransmitPortInfo");
   }
   else
   {
      LogMsg(Device(), "dev_GetTransmitPortInfo");
   }
   return res;
}

//*****************************************************************************
// Function: int CMediaDevice::OnGetTransmitPortInfoComplete(METAEVENT &a_metaEvent)
// Description: Handle get transmit port info completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDevice::OnGetTransmitPortInfoComplete(METAEVENT& a_metaEvent)
{
   DM_PORT_INFO_LIST* l_portInfoList = reinterpret_cast<DM_PORT_INFO_LIST*>(a_metaEvent.evtdatap);
   m_txPortInfoList = *l_portInfoList;
   m_observer->PostEvent(APPEVENT_MM_GET_TX_PORT_INFO_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CMediaDevice::OnPortConnected(METAEVENT &a_metaEvent)
// Description: Handle port conmnected event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDevice::OnPortConnected(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_MM_PORT_CONNECTED);
   return 0;
}

//*****************************************************************************
// Function: int CMediaDevice::OnPortDisconnected(METAEVENT &a_metaEvent)
// Description: Handle port disconnected event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDevice::OnPortDisconnected(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_MM_PORT_DISCONNECTED);
   return 0;
}

//*****************************************************************************
// Function: int CMediaDevice::PrimMmRecord()
// Description: Handle MM device play command
// Return: int - 0 for success, otherwise error
// Parameters: none
//*****************************************************************************
int CMediaDevice::PrimMmRecord()
{
   
   INIT_MM_VIDEO_CODEC(&m_recVideoCodec);
   if(g_confLayoutSize == eLB_LAYOUT_SIZE_QCIF)
   { 
      m_recVideoCodec.FramesPerSec     = VIDEO_FRAMESPERSEC_15;
      m_recVideoCodec.ImageHeight      = VIDEO_IMAGE_HEIGHT_QCIF;
      m_recVideoCodec.ImageWidth       = VIDEO_IMAGE_WIDTH_QCIF;
      m_recVideoCodec.BitRate          = VIDEO_BITRATE_128K;
      if(m_recVideoCoding == VIDEO_CODING_H263) {
         m_recVideoCodec.Coding            = VIDEO_CODING_H263;
         m_recVideoCodec.Level            = VIDEO_LEVEL_20_H263;
         m_recVideoCodec.Profile          = VIDEO_PROFILE_0_H263;
      }
      else if(m_recVideoCoding == VIDEO_CODING_H264) {
         m_recVideoCodec.Coding            = VIDEO_CODING_H264;
         m_recVideoCodec.Level            = VIDEO_LEVEL_1_2_H264;
         m_recVideoCodec.Profile          = VIDEO_PROFILE_BASELINE_H264;
      }      
   }
#ifdef ENABLE_VGA
   else if(g_confLayoutSize == eLB_LAYOUT_SIZE_VGA)
   { 
      m_recVideoCodec.FramesPerSec     = VIDEO_FRAMESPERSEC_30;
      m_recVideoCodec.ImageHeight      = VIDEO_IMAGE_HEIGHT_VGA;
      m_recVideoCodec.ImageWidth       = VIDEO_IMAGE_WIDTH_VGA;      
      m_recVideoCodec.BitRate          = VIDEO_BITRATE_768K;
      m_recVideoCodec.Coding           = VIDEO_CODING_H264;
      m_recVideoCodec.Level            = VIDEO_LEVEL_3_0_H264;
      m_recVideoCodec.Profile          = VIDEO_PROFILE_BASELINE_H264;
   }
#endif
   else // (g_confLayoutSize == eLB_LAYOUT_SIZE_CIF)
   { 
      m_recVideoCodec.FramesPerSec     = VIDEO_FRAMESPERSEC_30;
      m_recVideoCodec.ImageHeight      = VIDEO_IMAGE_HEIGHT_CIF;
      m_recVideoCodec.ImageWidth       = VIDEO_IMAGE_WIDTH_CIF;
      m_recVideoCodec.BitRate          = VIDEO_BITRATE_384K;
      if(m_recVideoCoding == VIDEO_CODING_H263) {
         m_recVideoCodec.Coding            = VIDEO_CODING_H263;
         m_recVideoCodec.Level            = VIDEO_LEVEL_30_H263;
         m_recVideoCodec.Profile          = VIDEO_PROFILE_0_H263;
      }
      else if(m_recVideoCoding == VIDEO_CODING_H264) {
         m_recVideoCodec.Coding            = VIDEO_CODING_H264;
         m_recVideoCodec.Level            = VIDEO_LEVEL_1_3_H264;
         m_recVideoCodec.Profile          = VIDEO_PROFILE_BASELINE_H264;
      }      
   }
  

   INIT_MM_AUDIO_CODEC(&m_recAudioCodec);
   m_recAudioCodec.unCoding         = MM_DATA_FORMAT_PCM;
   m_recAudioCodec.unSampleRate     = MM_DRT_8KHZ;
   m_recAudioCodec.unBitsPerSample  = 16;

   INIT_MM_MEDIA_ITEM_LIST(&m_recAudioMediaItemList);
   INIT_MM_MEDIA_AUDIO(&m_recAudioMediaItemList.item.audio);
   m_recAudioMediaItemList.ItemChain                = EMM_ITEM_EOT;
   m_recAudioMediaItemList.item.audio.eFileFormat   = EMM_AUD_FILEFORMAT_VOX; //pcm
   m_recAudioMediaItemList.item.audio.codec         = m_recAudioCodec;
   m_recAudioMediaItemList.item.audio.unMode        = MM_MODE_AUD_FILE_TYPE_VOX;
   m_recAudioMediaItemList.item.audio.unOffset      = 0;
   m_recAudioMediaItemList.item.audio.szFileName    = m_recAudioFile.c_str();
   m_recAudioMediaItemList.item.audio.unAccessMode  = MM_MEDIA_ACCESS_MODE_FILE;
   
   INIT_MM_MEDIA_ITEM_LIST(&m_recVideoMediaItemList);
   INIT_MM_MEDIA_VIDEO(&m_recVideoMediaItemList.item.video);
   m_recVideoMediaItemList.ItemChain                = EMM_ITEM_EOT;
   m_recVideoMediaItemList.item.video.eFileFormat   = EMM_FILE_FORMAT_PROPRIETARY;
   m_recVideoMediaItemList.item.video.codec         = m_recVideoCodec;
   m_recVideoMediaItemList.item.video.szFileName    = m_recVideoFile.c_str();
   m_recVideoMediaItemList.item.video.unAccessMode  = MM_MEDIA_ACCESS_MODE_FILE;

   INIT_MM_PLAY_RECORD_LIST(&m_recordList[0]);
   m_recordList[0].ItemChain                   = EMM_ITEM_CONT;
   m_recordList[0].ItemType                    = EMM_MEDIA_TYPE_AUDIO;
   m_recordList[0].list                        = &m_recAudioMediaItemList;

   INIT_MM_PLAY_RECORD_LIST(&m_recordList[1]);
   m_recordList[1].ItemChain                   = EMM_ITEM_EOT;
   m_recordList[1].ItemType                    = EMM_MEDIA_TYPE_VIDEO;
   m_recordList[1].list                        = &m_recVideoMediaItemList;

   INIT_MM_PLAY_RECORD_INFO(&m_recordInfo);
   m_recordInfo.list             = m_recordList;
   m_recordInfo.eFileFormat      = EMM_FILE_FORMAT_PROPRIETARY;
   //m_recordInfo.eFileFormat      = EMM_FILE_FORMAT_UNDEFINED;

   LogMsg(Device(),m_recAudioFile + ":" + m_recVideoFile);
   PrintMMCodecParameters(&m_recVideoCodec, &m_recAudioCodec);
   int res = mm_Record(Device(), &m_recordInfo, NULL, NULL);
   if (res == -1) {
      ErrMsg(Device(), "mm_Record");
   } else {
      LogMsg(Device(), "mm_Record");
   }
   return res;
}

//*****************************************************************************
// Function: int CMediaDevice::PrimMmPlay()
// Description: Handle MM device play command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CMediaDevice::PrimMmPlay()
{

   INIT_MM_MEDIA_ITEM_LIST(&m_videoMediaItemList);
   INIT_MM_MEDIA_VIDEO(&m_videoMediaItemList.item.video);
   m_videoMediaItemList.ItemChain                = EMM_ITEM_EOT;

   if (m_is3gp == true)
      m_videoMediaItemList.item.video.eFileFormat   = EMM_FILE_FORMAT_3GP;
   else
      m_videoMediaItemList.item.video.eFileFormat   = EMM_FILE_FORMAT_PROPRIETARY;


   m_videoMediaItemList.item.video.codec         = m_videoCodec;
   m_videoMediaItemList.item.video.unMode        = 0;
   m_videoMediaItemList.item.video.unOffset      = 0;
   m_videoMediaItemList.item.video.szFileName    = m_videoFile.c_str();
   m_videoMediaItemList.item.video.unAccessMode  = MM_MEDIA_ACCESS_MODE_FILE;

   INIT_MM_PLAY_RECORD_LIST(&m_playList[0]);

   if (m_audioCodec.unCoding != 0) 
      m_playList[0].ItemChain                   = EMM_ITEM_CONT;
   else
      m_playList[1].ItemChain                   = EMM_ITEM_EOT;
   m_playList[0].ItemType                    = EMM_MEDIA_TYPE_VIDEO;
   m_playList[0].list                        = &m_videoMediaItemList;  

   if (m_audioCodec.unCoding != 0) 
   {
      INIT_MM_MEDIA_ITEM_LIST(&m_audioMediaItemList);
      INIT_MM_MEDIA_AUDIO(&m_audioMediaItemList.item.audio);
      m_audioMediaItemList.ItemChain                = EMM_ITEM_EOT;

      if (m_is3gp == true)
         m_audioMediaItemList.item.audio.eFileFormat   = EMM_FILE_FORMAT_3GP;
      else
         m_audioMediaItemList.item.audio.eFileFormat   = EMM_AUD_FILEFORMAT_VOX;


      m_audioMediaItemList.item.audio.codec         = m_audioCodec;

      if (m_is3gp == true)
         m_audioMediaItemList.item.audio.unMode        = 0;
      else {
         m_audioMediaItemList.item.audio.unMode        = MM_MODE_AUD_FILE_TYPE_VOX;
         m_audioMediaItemList.item.audio.unMode        = 0;
      }

      m_audioMediaItemList.item.audio.unOffset      = 0;
      m_audioMediaItemList.item.audio.szFileName    = m_audioFile.c_str();
      m_audioMediaItemList.item.audio.unAccessMode  = MM_MEDIA_ACCESS_MODE_FILE;

      INIT_MM_PLAY_RECORD_LIST(&m_playList[1]);
      m_playList[1].ItemChain                   = EMM_ITEM_EOT;
      m_playList[1].ItemType                    = EMM_MEDIA_TYPE_AUDIO;
      m_playList[1].list                        = &m_audioMediaItemList; 
   }
   else
      LogMsg(Device(), "No Audio Codec - Playing Video only ");

   INIT_MM_PLAY_INFO(&m_playInfo);
   m_playInfo.list             = m_playList;
//   m_playInfo.eFileFormat      = EMM_FILE_FORMAT_PROPRIETARY;
   m_playInfo.eFileFormat      = EMM_FILE_FORMAT_UNDEFINED;
PrintMMCodecParameters(&m_videoMediaItemList.item.video.codec, &m_audioMediaItemList.item.audio.codec);

//   LogMsg(Device(),m_audioFile+":"+m_videoFile);
   int res = mm_Play(Device(), &m_playInfo, NULL, NULL);
   if (res == -1) {
      ErrMsg(Device(), "mm_Play() Failed");
      if(fopen(m_audioMediaItemList.item.audio.szFileName,"r")==NULL){
         ErrMsg(Device(), "\t !ERROR:->File does not exist:" + AsString(m_audioMediaItemList.item.audio.szFileName));
      }
      if(fopen(m_videoMediaItemList.item.video.szFileName,"r")==NULL){
         ErrMsg(Device(), "\t !ERROR:->File does not exist:" + AsString(m_videoMediaItemList.item.video.szFileName));
      }
   } else {
      LogMsg(Device(), "mm_Play() Success ");
   }
   return res;
}

typedef int ( CMediaDevice::*mmMethod)();

//*****************************************************************************
// Function: int CMediaDevice::PrimOnMmPlayComplete(METAEVENT &a_metaEvent)
// Description: Handle MM device play completion event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDevice::PrimOnMmPlayComplete(METAEVENT& a_metaEvent)
{
   CScheduler<CMediaDevice>::Instance()->ScheduleTask( this, &CMediaDevice::OnMmPlayTimeoutComplete, 1L);
   return 0;
}

//*****************************************************************************
// Function: int CMediaDevice::PrimOnMmPlayTimeoutComplete()
// Description: Handle MM device play completion timeout
// Return: int - 0 for success, otherwise error 
// Parameters: none
//*****************************************************************************
int CMediaDevice::PrimOnMmPlayTimeoutComplete()
{
   m_closingSubState |= MM_CLOSE_PLAY;
   m_observer->PostEvent(APPEVENT_PLAY_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CMediaDevice::PrimOnMmPlayFail(METAEVENT &a_metaEvent)
// Description: Handle MM device play failure event
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDevice::PrimOnMmPlayFail(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_PLAY_FAIL);
   return 0;
}

//*****************************************************************************
// Function: int CMediaDevice::PrimOnMmRecordComplete(METAEVENT &a_metaEvent)
// Description: Handle MM device play completion event
// Return: int - 0 for success, otherwise error
// Parameters: METAEVENT &a_metaEvent
//*****************************************************************************
int CMediaDevice::PrimOnMmRecordComplete(METAEVENT& a_metaEvent)
{
   CScheduler<CMediaDevice>::Instance()->ScheduleTask( this, &CMediaDevice::OnMmRecordTimeoutComplete, 1L);
   return 0;
}

//*****************************************************************************
// Function: int CMediaDevice::PrimOnMmRecordTimeoutComplete()
// Description: Handle MM device play completion timeout
// Return: int - 0 for success, otherwise error
// Parameters: none
//*****************************************************************************
int CMediaDevice::PrimOnMmRecordTimeoutComplete()
{
   m_closingSubState |= MM_CLOSE_RECORD;
   m_observer->PostEvent(APPEVENT_RECORD_COMPLETE);
   return 0;
}

//*****************************************************************************
// Function: int CMediaDevice::PrimOnMmRecordFail(METAEVENT &a_metaEvent)
// Description: Handle MM device play failure event
// Return: int - 0 for success, otherwise error
// Parameters: METAEVENT &a_metaEvent
//*****************************************************************************
int CMediaDevice::PrimOnMmRecordFail(METAEVENT& a_metaEvent)
{
   m_observer->PostEvent(APPEVENT_RECORD_FAIL);
   return 0;
}

//*****************************************************************************
// Function: int CMediaDevice::PrimMmStop()
// Description: HYandle stop MM device operation command
// Return: int - 0 for success, otherwise error 
// Parameters: operation
//*****************************************************************************
int CMediaDevice::PrimMmStop(int operation)
{
   m_closingSubState = 0;
   MM_STOP l_stopInfo[2];

   INIT_MM_STOP(&l_stopInfo[0]);
   l_stopInfo[0].ItemType = (operation==STOP_PLAY?EMM_STOP_VIDEO_PLAY:EMM_STOP_VIDEO_RECORD);
   l_stopInfo[0].ItemChain = EMM_ITEM_CONT;
   l_stopInfo[0].next = &l_stopInfo[1];

   INIT_MM_STOP(&l_stopInfo[1]);
   l_stopInfo[1].ItemType = (operation==STOP_PLAY?EMM_STOP_AUDIO_PLAY:EMM_STOP_AUDIO_RECORD);

   int res = mm_Stop(Device(), l_stopInfo, NULL);
   if (res == EMM_ERROR)
   {
      ErrMsg(Device(), "mm_Stop");
   }
   else
   {
      LogMsg(Device(), "mm_Stop");
   }
   return res;
}

//*****************************************************************************
// Function: int CMediaDevice::PrimOnMmStopAck(METAEVENT &a_metaEvent)
// Description: Handle stop MM device operation command ACK
// Return: int - 0 for success, otherwise error 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDevice::PrimOnMmStopAck(METAEVENT& a_metaEvent)
{
   m_closingSubState |= MM_CLOSE_STOP_ACK;
   return 0;
}

//*****************************************************************************
// Function: void CMediaDevice::ProcessEvent(METAEVENT &a_metaEvent)
// Description: Process event
// Return: void 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
void CMediaDevice::ProcessEvent(METAEVENT& a_metaEvent)
{
   switch (a_metaEvent.evttype)
   {
      case MMEV_OPEN:
         LogMsg(Device(), "MMEV_OPEN mmDevice=" + AsString(":") + AsString(Device()));
         OnMmOpened(a_metaEvent);
         break;
      case MMEV_OPEN_FAIL:
         LogMsg(Device(), "MMEV_OPEN_FAIL mmDevice=" + AsString(":") + AsString(Device()));
         break;

		case MMEV_VIDEO_RECORD_STARTED:
         LogMsg(Device(), "MMEV_VIDEO_RECORD_STARTED");
         break;
      case MMEV_RECORD_ACK:
         LogMsg(Device(), "MMEV_RECORD_ACK");
         break;
      case MMEV_RECORD_ACK_FAIL:
         LogMsg(Device(), "MMEV_RECORD_ACK_FAIL");
         OnMmRecordFail(a_metaEvent);
         break;
      case MMEV_RECORD:
         LogMsg(Device(), "MMEV_RECORD");
         OnMmRecordComplete(a_metaEvent);
         break;
      case MMEV_RECORD_FAIL:
         LogMsg(Device(), "MMEV_RECORD_FAIL");
         OnMmRecordFail(a_metaEvent);
         break;

      case MMEV_PLAY_ACK:
         LogMsg(Device(), "MMEV_PLAY_ACK");
         break;

      case MMEV_PLAY_ACK_FAIL:
         LogMsg(Device(), "MMEV_PLAY_ACK_FAIL");
         OnMmPlayFail(a_metaEvent);
         break;

      case MMEV_PLAY:
         LogMsg(Device(), "MMEV_PLAY");
         OnMmPlayComplete(a_metaEvent);
         break;

      case MMEV_PLAY_FAIL:
         LogMsg(Device(), "MMEV_PLAY_FAIL");
         OnMmPlayFail(a_metaEvent);
         break;

      case MMEV_STOP_ACK:
         LogMsg(Device(), "MMEV_STOP_ACK");
         OnMmStopAck(a_metaEvent);
         break;

      case MMEV_ERROR:
         ErrMsg(Device(), "MMEV_ERROR");
         break;

      case DMEV_CONNECT:
         LogMsg(Device(), "DMEV_CONNECT");
         m_observer->PostEvent(APPEVENT_ON_MEDIA_CONNECTED);
         break;

      case DMEV_DISCONNECT:
         LogMsg(Device(), "DMEV_DISCONNECT");
         m_observer->PostEvent(APPEVENT_ON_MEDIA_DISCONNECTED);
         break;

      case DMEV_GET_RX_PORT_INFO:
         LogMsg(Device(), "DMEV_GET_RX_PORT_INFO");
         OnGetReceivePortInfoComplete(a_metaEvent);
         break;

      case DMEV_GET_RX_PORT_INFO_FAIL:
         LogMsg(Device(), "DMEV_GET_RX_PORT_INFO_FAIL");
         break;

      case DMEV_GET_TX_PORT_INFO:
         LogMsg(Device(), "DMEV_GET_TX_PORT_INFO");
         OnGetTransmitPortInfoComplete(a_metaEvent);
         break;

      case DMEV_GET_TX_PORT_INFO_FAIL:
         LogMsg(Device(), "DMEV_GET_TX_PORT_INFO_FAIL");
         break;

      case DMEV_PORT_CONNECT:
         LogMsg(Device(), "DMEV_PORT_CONNECT");
         OnPortConnected(a_metaEvent);
         break;

      case DMEV_PORT_CONNECT_FAIL:
         LogMsg(Device(), "DMEV_PORT_CONNECT_FAIL");
         break;

      case DMEV_PORT_DISCONNECT:
         LogMsg(Device(), "DMEV_PORT_DISCONNECT");
         OnPortDisconnected(a_metaEvent);
         break;

      case DMEV_PORT_DISCONNECT_FAIL:
         LogMsg(Device(), "DMEV_PORT_DISCONNECT_FAIL");
         break;

      case APPEVENT_CLOSE_COMMAND:
         LogMsg(Device(), "APPEVENT_CLOSE_COMMAND");
// SKS do nothing since mm_Stop already done when conference closed
         Close();
         break;

      default:
         LogMsg(Device(), "unhandled event:" + AsHexString(a_metaEvent.evttype));
   }
}


