/**
* @file CMediaDeviceState.cpp
* @brief Multimedia Device States - class implementations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "CMediaDeviceState.h"
#include "LogUtil.h"

CMediaDeviceStates AMediaDeviceState::m_deviceStates;

//*****************************************************************************
// Function: int AMediaDeviceState::LogStateError(CMediaDevice &a_device, const string &a_msg)
// Description: Log an invalid state for operation/event message
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//             const string &a_msg 
//*****************************************************************************
int AMediaDeviceState::LogStateError(CMediaDevice& a_device, const string& a_msg)
{
   std::cout << a_device.DeviceName()
   << " " << Desc() << " is invalid device state for "
   << a_msg
   << std::endl;
   return -1;
}

//*****************************************************************************
// Function: int AMediaDeviceState::MmClose(CMediaDevice &a_device)
// Description: Close an MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//*****************************************************************************
int AMediaDeviceState::MmClose(CMediaDevice& a_device) 
{
   a_device.SetDeviceStateTo(DeviceState(MediaDeviceClosedState));
   return a_device.PrimMmClose();
}

//*****************************************************************************
// Function: CMediaDeviceStates::CMediaDeviceStates()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CMediaDeviceStates::CMediaDeviceStates()
{
   m_deviceStates[MediaDeviceClosedState] = new CMediaDeviceClosedState();
   m_deviceStates[MediaDeviceOpeningState] = new CMediaDeviceOpeningState();
   m_deviceStates[MediaDeviceIdleState] = new CMediaDeviceIdleState();
   m_deviceStates[MediaDevicePlayingState] = new CMediaDevicePlayingState();
   m_deviceStates[MediaDeviceRecordingState] = new CMediaDeviceRecordingState();
   m_deviceStates[MediaDeviceClosingState] = new CMediaDeviceClosingState();
}

//*****************************************************************************
// Function: int CMediaDeviceClosedState::MmOpen(CMediaDevice &a_device)
// Description: Open an MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//*****************************************************************************
int CMediaDeviceClosedState::MmOpen(CMediaDevice& a_device) 
{
   a_device.SetDeviceStateTo(DeviceState(MediaDeviceOpeningState));
   return a_device.PrimMmOpen();
}

//*****************************************************************************
// Function: int CMediaDeviceOpeningState::OnMmOpened(CMediaDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an opne MM device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDeviceOpeningState::OnMmOpened(CMediaDevice& a_device, METAEVENT& a_metaEvent) 
{
   int res = a_device.PrimOnMmOpened(a_metaEvent);
   if (res != -1)
   {
      a_device.SetDeviceStateTo(DeviceState(MediaDeviceIdleState));
   }
   return 0;
}

//*****************************************************************************
// Function: int CMediaDeviceIdleState::MmPlay(CMediaDevice &a_device)
// Description: Play a clip on an MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//*****************************************************************************
int CMediaDeviceIdleState::MmPlay(CMediaDevice& a_device) 
{
   a_device.SetDeviceStateTo(DeviceState(MediaDevicePlayingState));
   int res = a_device.PrimMmPlay();
   if (res == -1)
   {
      a_device.SetDeviceStateTo(DeviceState(MediaDeviceIdleState));
   }
   return res;
}

//*****************************************************************************
// Function: int CMediaDeviceIdleState::MmRecord(CMediaDevice &a_device)
// Description: Record conference on an MM device
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//*****************************************************************************
int CMediaDeviceIdleState::MmRecord(CMediaDevice& a_device)
{
   a_device.SetDeviceStateTo(DeviceState(MediaDeviceRecordingState));
   int res = a_device.PrimMmRecord();
   if (res == -1)
   {
      a_device.SetDeviceStateTo(DeviceState(MediaDeviceIdleState));
   }
   return res;
}


//*****************************************************************************
// Function: int CMediaDevicePlayingState::MmClose(CMediaDevice &a_device)
// Description: Close an MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//*****************************************************************************
int CMediaDevicePlayingState::MmClose(CMediaDevice& a_device) 
{
   a_device.SetDeviceStateTo(DeviceState(MediaDeviceClosingState));
   return a_device.PrimMmStop(STOP_PLAY);
}

//*****************************************************************************
// Function: int CMediaDevicePlayingState::OnMmPlayComplete(CMediaDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an MM device play completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDevicePlayingState::OnMmPlayComplete(CMediaDevice& a_device, METAEVENT& a_metaEvent) 
{
   return a_device.PrimOnMmPlayComplete(a_metaEvent);
}

//*****************************************************************************
// Function: int CMediaDevicePlayingState::OnMmPlayTimeoutComplete(CMediaDevice &a_device)
// Description: Handle an MM device play completion timeout event
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//*****************************************************************************
int CMediaDevicePlayingState::OnMmPlayTimeoutComplete(CMediaDevice& a_device) 
{
   a_device.SetDeviceStateTo(DeviceState(MediaDeviceIdleState));
   return a_device.PrimOnMmPlayTimeoutComplete();
}

//*****************************************************************************
// Function: int CMediaDevicePlayingState::OnMmPlayFail(CMediaDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an MM device play failure event
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDevicePlayingState::OnMmPlayFail(CMediaDevice& a_device, METAEVENT& a_metaEvent) 
{
   a_device.SetDeviceStateTo(DeviceState(MediaDeviceIdleState));
   return a_device.PrimOnMmPlayFail(a_metaEvent);
}

//*****************************************************************************
// Function: int CMediaDevicePlayingState::MmStop(CMediaDevice &a_device)
// Description: Stop an MM device operation
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//*****************************************************************************
int CMediaDevicePlayingState::MmStop(CMediaDevice& a_device) 
{
   return a_device.PrimMmStop(STOP_PLAY);
}

//*****************************************************************************
// Function: int CMediaDevicePlayingState::OnMmStopAck(CMediaDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a stop MM device operation ACK event
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDevicePlayingState::OnMmStopAck(CMediaDevice& a_device, METAEVENT& a_metaEvent)
{
   return a_device.PrimOnMmStopAck(a_metaEvent);
}

//*****************************************************************************
// Function: int CMediaDeviceClosingState::OnMmPlayComplete(CMediaDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an MM device play completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDeviceClosingState::OnMmPlayComplete(CMediaDevice& a_device, METAEVENT& a_metaEvent)
{
   return a_device.PrimOnMmPlayComplete(a_metaEvent);
}

//*****************************************************************************
// Function: int CMediaDeviceClosingState::OnMmPlayTimeoutComplete(CMediaDevice &a_device)
// Description: Handle an MM device play completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//*****************************************************************************
int CMediaDeviceClosingState::OnMmPlayTimeoutComplete(CMediaDevice& a_device)
{
   int res = a_device.PrimOnMmPlayTimeoutComplete();
   if (res != -1)
   {
      if (a_device.ClosingSubState() == MM_CLOSE_COMPLETE)
      {
         a_device.SetDeviceStateTo(DeviceState(MediaDeviceClosedState));
         res = a_device.PrimMmClose();
      }
   }
   return res;
}

//*****************************************************************************
// Function: int CMediaDeviceClosingState::OnMmStopAck(CMediaDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an MM device stop operation ACK event
// Return: int - 0 for success, otherwise error 
// Parameters: CMediaDevice &a_device 
//             METAEVENT &a_metaEvent 
//*****************************************************************************
int CMediaDeviceClosingState::OnMmStopAck(CMediaDevice& a_device, METAEVENT& a_metaEvent)
{
   int res = a_device.PrimOnMmStopAck(a_metaEvent);
   if (res != -1)
   {
      if (a_device.ClosingSubState() == MM_CLOSE_COMPLETE)
      {
         a_device.SetDeviceStateTo(DeviceState(MediaDeviceClosedState));
         res = a_device.PrimMmClose();
      }
   }
   return res;
}

//*****************************************************************************
// Function: int CMediaDeviceRecordingState::MmPlay(CMediaDevice &a_device)
// Description: Play a clip on an MM device
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//*****************************************************************************
int CMediaDeviceRecordingState::MmPlay(CMediaDevice& a_device)
{
   int res = a_device.PrimMmPlay();
   if (res == -1)
   {
		res=0;
		// SKS DO nothing
   }
   return res;
}

//*****************************************************************************
// Function: int CMediaDeviceRecordingState::OnMmPlayComplete(CMediaDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an MM device play completion event
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//             METAEVENT &a_metaEvent
//*****************************************************************************
int CMediaDeviceRecordingState::OnMmPlayComplete(CMediaDevice& a_device, METAEVENT& a_metaEvent)
{
	// SKS DO NOTHING
	return 0;
   //return a_device.PrimOnMmPlayComplete(a_metaEvent);
}

//*****************************************************************************
// Function: int CMediaDeviceRecordingState::OnMmPlayTimeoutComplete(CMediaDevice &a_device)
// Description: Handle an MM device play completion timeout event
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//*****************************************************************************
int CMediaDeviceRecordingState::OnMmPlayTimeoutComplete(CMediaDevice& a_device)
{
	// SKS DO NOTHING
	return 0;
 //  return a_device.PrimOnMmPlayTimeoutComplete();
}

//*****************************************************************************
// Function: int CMediaDeviceRecordingState::OnMmPlayFail(CMediaDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an MM device play failure event
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//             METAEVENT &a_metaEvent
//*****************************************************************************
int CMediaDeviceRecordingState::OnMmPlayFail(CMediaDevice& a_device, METAEVENT& a_metaEvent)
{
	// SKS DO NOTHING
	return 0;
//   return a_device.PrimOnMmPlayFail(a_metaEvent);
}


//*****************************************************************************
// Function: int CMediaDeviceRecordingState::MmClose(CMediaDevice &a_device)
// Description: Close an MM device
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//*****************************************************************************
int CMediaDeviceRecordingState::MmClose(CMediaDevice& a_device)
{
   a_device.SetDeviceStateTo(DeviceState(MediaDeviceClosingState));
   return a_device.PrimMmStop(STOP_RECORD);
}

//*****************************************************************************
// Function: int CMediaDeviceRecordingState::OnMmRecordComplete(CMediaDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an MM device play completion event
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//             METAEVENT &a_metaEvent
//*****************************************************************************
int CMediaDeviceRecordingState::OnMmRecordComplete(CMediaDevice& a_device, METAEVENT& a_metaEvent)
{
   return a_device.PrimOnMmRecordComplete(a_metaEvent);
}

//*****************************************************************************
// Function: int CMediaDeviceRecordingState::OnMmRecordTimeoutComplete(CMediaDevice &a_device)
// Description: Handle an MM device play completion timeout event
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//*****************************************************************************
int CMediaDeviceRecordingState::OnMmRecordTimeoutComplete(CMediaDevice& a_device)
{
   a_device.SetDeviceStateTo(DeviceState(MediaDeviceIdleState));
   return a_device.PrimOnMmRecordTimeoutComplete();
}

//*****************************************************************************
// Function: int CMediaDeviceRecordingState::OnMmRecordFail(CMediaDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an MM device play failure event
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//             METAEVENT &a_metaEvent
//*****************************************************************************
int CMediaDeviceRecordingState::OnMmRecordFail(CMediaDevice& a_device, METAEVENT& a_metaEvent)
{
   a_device.SetDeviceStateTo(DeviceState(MediaDeviceIdleState));
   return a_device.PrimOnMmRecordFail(a_metaEvent);
}

//*****************************************************************************
// Function: int CMediaDeviceRecordingState::MmStop(CMediaDevice &a_device)
// Description: Stop an MM device operation
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//*****************************************************************************
int CMediaDeviceRecordingState::MmStop(CMediaDevice& a_device)
{
   return a_device.PrimMmStop(STOP_RECORD);
}

//*****************************************************************************
// Function: int CMediaDeviceRecordingState::OnMmStopAck(CMediaDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle a stop MM device operation ACK event
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//             METAEVENT &a_metaEvent
//*****************************************************************************
int CMediaDeviceRecordingState::OnMmStopAck(CMediaDevice& a_device, METAEVENT& a_metaEvent)
{
   return a_device.PrimOnMmStopAck(a_metaEvent);
}

//*****************************************************************************
// Function: int CMediaDeviceClosingState::OnMmRecordComplete(CMediaDevice &a_device, METAEVENT &a_metaEvent)
// Description: Handle an MM device play completion event
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//             METAEVENT &a_metaEvent
//*****************************************************************************
int CMediaDeviceClosingState::OnMmRecordComplete(CMediaDevice& a_device, METAEVENT& a_metaEvent)
{
   return a_device.PrimOnMmRecordComplete(a_metaEvent);
}

//*****************************************************************************
// Function: int CMediaDeviceClosingState::OnMmRecordTimeoutComplete(CMediaDevice &a_device)
// Description: Handle an MM device play completion event
// Return: int - 0 for success, otherwise error
// Parameters: CMediaDevice &a_device
//*****************************************************************************
int CMediaDeviceClosingState::OnMmRecordTimeoutComplete(CMediaDevice& a_device)
{
   int res = a_device.PrimOnMmRecordTimeoutComplete();
   if (res != -1)
   {
      if (a_device.ClosingSubState() == MM_CLOSE_COMPLETE)
      {
         a_device.SetDeviceStateTo(DeviceState(MediaDeviceClosedState));
         res = a_device.PrimMmClose();
      }
   }
   return res;
}
