/**
* @file CMediaDeviceState.h
* @brief Multimedia Device States - class declarations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CMmDeviceState_h
#define CMmDeviceState_h

#include <map>
#include <string>

#include "LogUtil.h"
#include "CMediaDevice.h"

#define STOP_PLAY 	0
#define STOP_RECORD 	1

typedef enum 
{
   MediaDeviceClosedState = 0,
   MediaDeviceOpeningState,
   MediaDeviceIdleState,
   MediaDevicePlayingState,
   MediaDeviceRecordingState,
   MediaDeviceClosingState
} MediaDeviceState;

class AMediaDeviceState;

//*****************************************************************************
// Class: ::CMediaDeviceStates
// Description: Map of MM device state objects
//*****************************************************************************
class CMediaDeviceStates
{
public:
   CMediaDeviceStates();
   ~CMediaDeviceStates() {;}
   AMediaDeviceState* DeviceState(MediaDeviceState a_deviceState) {return m_deviceStates[a_deviceState];}
private:
   std::map< MediaDeviceState, AMediaDeviceState*> m_deviceStates;
};

//*****************************************************************************
// Class: ::AMediaDeviceState
// Description: Abstract MM device state class
// Notes:      
//    This abstract class declares all functions for all device states as 
//    virtual and implements them as an error.
// 
//    Concrete subclasses implement only the functions that are valid for that
//    state.
// 
//    State classes are responsible only for calling primative functions on the 
//    context device object and changing the state of the context device object.
// 
//    As state classes have no non-static member variables, one set of device 
//    state objects are shared by all devices.
//*****************************************************************************
class AMediaDeviceState
{
public:
   virtual const char* Desc() = 0;

   virtual int MmOpen(CMediaDevice& a_device) {return LogStateError(a_device, "MmOpen");}
   virtual int MmClose(CMediaDevice& a_device); 
   virtual int OnMmOpened(CMediaDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnMmOpened");}
   virtual int MmPlay(CMediaDevice& a_device) {return LogStateError(a_device, "MmPlay");}
   virtual int OnMmPlayComplete(CMediaDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnMmPlayComplete");}
   virtual int OnMmPlayTimeoutComplete(CMediaDevice& a_device) {return LogStateError(a_device, "OnMmPlayTimeoutComplete");}
   virtual int OnMmPlayFail(CMediaDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnMmPlayFail");}
   virtual int MmStop(CMediaDevice& a_device) {LogMsg(a_device.DeviceName(), "MmStop ignored"); return 0;}
   virtual int OnMmStopAck(CMediaDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnMmStopAck");}

   virtual int MmRecord(CMediaDevice& a_device) {return LogStateError(a_device, "MmRecord");}
   virtual int OnMmRecordComplete(CMediaDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnMmRecordComplete");}
   virtual int OnMmRecordTimeoutComplete(CMediaDevice& a_device) {return LogStateError(a_device, "OnMmRecordTimeoutComplete");}
   virtual int OnMmRecordFail(CMediaDevice& a_device, METAEVENT& a_metaEvent) {return LogStateError(a_device, "OnMmRecordFail");}

   static AMediaDeviceState* DeviceState(MediaDeviceState a_deviceState) {return m_deviceStates.DeviceState(a_deviceState);}

private:
   int LogStateError(CMediaDevice& a_device, const std::string& msg);
   static CMediaDeviceStates m_deviceStates;
};

//*****************************************************************************
// Class: ::CMediaDeviceClosedState
// Description: MM device closed state
//*****************************************************************************
class CMediaDeviceClosedState : public AMediaDeviceState
{
public:
   const char* Desc() {return "closed";}
   int MmOpen(CMediaDevice& a_device);
};

//*****************************************************************************
// Class: ::CMediaDeviceOpeningState
// Description: MM device opening state
//*****************************************************************************
class CMediaDeviceOpeningState : public AMediaDeviceState
{
public:
   const char* Desc() {return "opening";}
   int OnMmOpened(CMediaDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CMediaDeviceIdleState
// Description: MM device idle state
//*****************************************************************************
class CMediaDeviceIdleState : public AMediaDeviceState
{
public:
   const char* Desc() {return "idle";}
   int MmPlay(CMediaDevice& a_device);
   int MmRecord(CMediaDevice& a_device);
};

//*****************************************************************************
// Class: ::CMediaDevicePlayingState
// Description: MM device playing state
//*****************************************************************************
class CMediaDevicePlayingState : public AMediaDeviceState
{
public:
   const char* Desc() {return "playing";}
   int MmClose(CMediaDevice& a_device);
   int OnMmPlayComplete(CMediaDevice& a_device, METAEVENT& a_metaEvent);
   int OnMmPlayTimeoutComplete(CMediaDevice& a_device);
   int OnMmPlayFail(CMediaDevice& a_device, METAEVENT& a_metaEvent);
   int MmStop(CMediaDevice& a_device);
   int OnMmStopAck(CMediaDevice& a_device, METAEVENT& a_metaEvent);
};

//*****************************************************************************
// Class: ::CMediaDeviceRecordingState
// Description: MM device Recording state
//*****************************************************************************
class CMediaDeviceRecordingState : public AMediaDeviceState
{
public:
   const char* Desc() {return "recording";}
   int MmPlay(CMediaDevice& a_device);
   int OnMmPlayComplete(CMediaDevice& a_device, METAEVENT& a_metaEvent);
   int OnMmPlayTimeoutComplete(CMediaDevice& a_device);
   int OnMmPlayFail(CMediaDevice& a_device, METAEVENT& a_metaEvent);
   int MmClose(CMediaDevice& a_device);
   int OnMmRecordComplete(CMediaDevice& a_device, METAEVENT& a_metaEvent);
   int OnMmRecordTimeoutComplete(CMediaDevice& a_device);
   int OnMmRecordFail(CMediaDevice& a_device, METAEVENT& a_metaEvent);
   int MmStop(CMediaDevice& a_device);
   int OnMmStopAck(CMediaDevice& a_device, METAEVENT& a_metaEvent);
};


//*****************************************************************************
// Class: ::CMediaDeviceClosingState
// Description: MM device closing state
//*****************************************************************************
class CMediaDeviceClosingState : public AMediaDeviceState
{
public:
   const char* Desc() {return "closing";}
	int MmRecord(CMediaDevice& a_device) {return -1;}
   int MmPlay(CMediaDevice& a_device) {return -1;}
   int OnMmPlayComplete(CMediaDevice& a_device, METAEVENT& a_metaEvent);
   int OnMmPlayTimeoutComplete(CMediaDevice& a_device);
   int OnMmStopAck(CMediaDevice& a_device, METAEVENT& a_metaEvent);
   int OnMmRecordComplete(CMediaDevice& a_device, METAEVENT& a_metaEvent);
   int OnMmRecordTimeoutComplete(CMediaDevice& a_device);
};

#endif

