/**
* @file CRecorder.cpp
* @brief Recorder class implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#include <sys/timeb.h>
#endif

#include "config.h"

#include "devmgmt.h"
#include <time.h>

#include "LogUtil.h"
#include "CRecorder.h"
#include "CRecorderState.h"
#include "CMediaDevice.h"
#include "CConference.h"
#include "CConferencePartyDevice.h"
#include "CGcManager.h"


#include "CConnection.h"

int CRecorder::MM_RX_PORT_INFO = 0x0001;
int CRecorder::MM_TX_PORT_INFO = 0x0002;
int CRecorder::CNF_RX_PORT_INFO = 0x0004;
int CRecorder::CNF_TX_PORT_INFO = 0x0008;
int CRecorder::PORT_INFO_COMPLETE = 0x000F;

int CRecorder::MM_TX_CNF_RX_CONNECT = 0x0011;
int CRecorder::CNF_TX_MM_RX_CONNECT = 0x0012;
int CRecorder::PORT_CONNECT_COMPLETE = 0x0013;

int CRecorder::MM_TX_CNF_RX_DISCONNECT = 0x0021;
int CRecorder::CNF_TX_MM_RX_DISCONNECT = 0x0022;
int CRecorder::PORT_DISCONNECT_COMPLETE = 0x0023;

//*****************************************************************************
// Function: CRecorder::CRecorder(const std::string &a_name,std::string confID)
// Description: Initializing constructor
// Return:  
// Parameters: const std::string &a_name 
//             CMediaDevice *a_mmDevice 
//*****************************************************************************
CRecorder::CRecorder(std::string confID,bool bRecordConf)
   : m_state(ARecorderState::State(RecorderIdleState))
	, m_confID(confID)
	, m_bRecordConf(bRecordConf)
   , m_conferenceParty(0)
   , m_portInfoSubState(0)
   , m_portConnectSubState(0)
   , m_portDisconnectSubState(0)
   , m_stopRecord(false)
   , m_stopPlay(false)
{
	OpenMediaDevice();
}

void CRecorder::OpenMediaDevice()
{
   time_t currentTime;
   time(&currentTime);

		m_name=CMediaDevice::GetNextMediaDeviceName();
      m_mmDevice = new CMediaDevice(m_name);
      CGcManager::RegisterDevice(m_mmDevice);
   	m_mmDevice->Observer(this);
      m_mmDevice->Open();
      CSystemConfiguration* m_systemConfiguration=CConference::SystemConfiguration();
      m_mmDevice->SetClip( m_systemConfiguration->MediaFilePath(), m_systemConfiguration->TimeOutClip());
      m_mmDevice->SetRecordClip(m_systemConfiguration->MediaFilePath(),AsString("ConfRecording_" + AsString(currentTime) + "_" + m_confID));
      m_mmDevice->SetRecordCoding(m_systemConfiguration->getRecordVideoCodec());
}

//*****************************************************************************
// Function: void CRecorder::SetStateTo(ARecorderState *state)
// Description: Set the virtual caller state
// Return: void 
// Parameters: ARecorderState *state 
//*****************************************************************************
void CRecorder::SetStateTo(ARecorderState* state)
{
   m_state = state;
   LogMsg(Name(), AsString("Recorder Caller State:") + m_state->Desc());
}

//*****************************************************************************
// All session operations and event handling are delegated to the state
// object with the following functions
//*****************************************************************************

int CRecorder::StartRecorder() {return State().StartRecorder(*this);}
int CRecorder::OpenConferenceParty() {return State().OpenConferenceParty(*this);}
int CRecorder::OnOpenConferencePartyComplete() {return State().OnOpenConferencePartyComplete(*this);}
int CRecorder::ConnectToConferenceParty() {return State().ConnectToConferenceParty(*this);}
int CRecorder::OnConnectToConferencePartyComplete() {return State().OnConnectToConferencePartyComplete(*this);}
int CRecorder::JoinConference() {return State().JoinConference(*this);}
int CRecorder::OnJoinConferenceComplete() {return State().OnJoinConferenceComplete(*this);}
int CRecorder::LeaveConference() {return State().LeaveConference(*this);}
int CRecorder::OnLeaveConferenceComplete() {return State().OnLeaveConferenceComplete(*this);}
int CRecorder::OnLeaveConferenceFailed() {return State().OnLeaveConferenceFailed(*this);}
int CRecorder::DisconnectFromConferenceParty() {return State().DisconnectFromConferenceParty(*this);}
int CRecorder::OnDisconnectFromConferencePartyComplete() {return State().OnDisconnectFromConferencePartyComplete(*this);}
int CRecorder::CloseConferenceParty() {return State().CloseConferenceParty(*this);}
int CRecorder::OnCloseConferencePartyComplete() {return State().OnCloseConferencePartyComplete(*this);}
int CRecorder::OnGetMmRxPortInfoComplete() {return State().OnGetMmRxPortInfoComplete(*this);}
int CRecorder::OnGetMmTxPortInfoComplete() {return State().OnGetMmTxPortInfoComplete(*this);}
int CRecorder::OnGetCnfRxPortInfoComplete() {return State().OnGetCnfRxPortInfoComplete(*this);}
int CRecorder::OnGetCnfTxPortInfoComplete() {return State().OnGetCnfTxPortInfoComplete(*this);}
int CRecorder::OnCnfPortConnected() {return State().OnCnfPortConnected(*this);}
int CRecorder::OnMmPortConnected() {return State().OnMmPortConnected(*this);}
int CRecorder::OnCnfPortDisconnected() {return State().OnCnfPortDisconnected(*this);}
int CRecorder::OnMmPortDisconnected() {return State().OnMmPortDisconnected(*this);}

int CRecorder::Play() {return State().Play(*this);}
int CRecorder::OnPlayComplete() {return State().OnPlayComplete(*this);}

int CRecorder::Record() {return State().Record(*this);}
int CRecorder::OnRecordComplete() {return State().OnRecordComplete(*this);}
int CRecorder::Stop() {return State().Stop(*this);}

//*****************************************************************************
// Function: int CRecorder::PrimStartRecorder()
// Description: Handle a start virtual caller command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimStartRecorder()
{
   int res(0);
   if (!m_mmDevice)
   {
      res = -1;
   }
   return res;
}

//*****************************************************************************
// Function: int CRecorder::PrimOpenConferenceParty()
// Description: Handle an open conference party command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOpenConferenceParty()
{
   m_conferenceParty = new CConferencePartyDevice(Name(), CConference::GetConferenceBoard(), this);
   CGcManager::RegisterDevice(m_conferenceParty);
   m_conferenceParty->SetRecorderParty(this);
   return m_conferenceParty->Open();
}

//*****************************************************************************
// Function: int CRecorder::PrimOnOpenConferencePartyComplete()
// Description: Handle an open conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnOpenConferencePartyComplete()
{
   CGcManager::RegisterDevice(m_conferenceParty->Device(), m_conferenceParty);
   return 0;
}

//*****************************************************************************
// Function: int CRecorder::PrimConnectToConferenceParty()
// Description: Handle a connect to conference party command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimConnectToConferenceParty()
{
   m_portInfoSubState = 0;
   m_mmDevice->GetReceivePortInfo();
   m_mmDevice->GetTransmitPortInfo();
   m_conferenceParty->GetReceivePortInfo();
   m_conferenceParty->GetTransmitPortInfo();
   return 0;
}

//*****************************************************************************
// Function: int CRecorder::PrimOnConnectToConferencePartyComplete()
// Description: Handle a connect to conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnConnectToConferencePartyComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CRecorder::PrimJoinConference()
// Description: Handle join conference command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimJoinConference()
{
// SKS
   CConference* l_conference = CConference::GetConference(m_confID);
   return l_conference->AddConferenceParty(m_conferenceParty);
}

//*****************************************************************************
// Function: int CRecorder::PrimOnJoinConferenceComplete()
// Description: Handle join conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnJoinConferenceComplete()
{
   LogMsg(Name(),"PrimOnJoinConferenceComplete");
// SKS
   return CConference::GetConference(m_confID)->OnConferencePartyAdded(m_conferenceParty);
}

//*****************************************************************************
// Function: int CRecorder::PrimLeaveConference()
// Description: Handle a leave conference command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimLeaveConference()
{
   CConference* l_conference = CConference::GetConference(m_confID);
   return l_conference->RemoveConferenceParty(m_conferenceParty);
}

//*****************************************************************************
// Function: int CRecorder::PrimOnLeaveConferenceComplete()
// Description: Handle a leave conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnLeaveConferenceComplete()
{
   return CConference::GetConference(m_confID)->OnConferencePartyRemoved(m_conferenceParty);
}

//*****************************************************************************
// Function: int CRecorder::PrimDisconnectFromConferenceParty()
// Description: Handle a disconnect from conference party command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimDisconnectFromConferenceParty()
{
   m_portDisconnectSubState = 0;
   int res = dev_PortDisconnect(m_mmDevice->Device(), &m_portConnectInfoListMmTxCnfRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_mmDevice->Device(), "dev_PortDisconnect");
      return res;
   }
   else
   {
      LogMsg( Name(), "dev_PortDisconnect");
   }

   res = dev_PortDisconnect(m_conferenceParty->Device(), &m_portConnectInfoListCnfTxMmRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_conferenceParty->Device(), "dev_PortDisconnect");
      return res;
   }
   else
   {
      LogMsg( Name(), "dev_PortDisconnect");
   }
   return res;
}

//*****************************************************************************
// Function: int CRecorder::PrimOnDisconnectFromConferencePartyComplete()
// Description: Handle a disconnect from conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnDisconnectFromConferencePartyComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CRecorder::PrimCloseConferenceParty()
// Description: Handle a close conference party command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimCloseConferenceParty()
{
   return m_conferenceParty->Close();
}

//*****************************************************************************
// Function: int CRecorder::PrimOnCloseConferencePartyComplete()
// Description: Handle a close conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnCloseConferencePartyComplete()
{
	// SKS 
  // CGcManager::UnregisterDevice(m_conferenceParty->ID());
   return 0;
}

//*****************************************************************************
// Function: int CRecorder::CreatePortConnectInfoList(DM_PORT_CONNECT_INFO_LIST &a_portConnectionInfoList, const DM_PORT_INFO_LIST &a_txPortInfoList, const DM_PORT_INFO_LIST &a_rxPortInfoList)
// Description: Create a port connedct info list
// Return: int - number of items in the port connection info list
// Parameters: DM_PORT_CONNECT_INFO_LIST &a_portConnectionInfoList 
//             const DM_PORT_INFO_LIST &a_txPortInfoList 
//             const DM_PORT_INFO_LIST &a_rxPortInfoList 
//*****************************************************************************
int CRecorder::CreatePortConnectInfoList(  DM_PORT_CONNECT_INFO_LIST& a_portConnectionInfoList,
                                                const DM_PORT_INFO_LIST& a_txPortInfoList,
                                                const DM_PORT_INFO_LIST& a_rxPortInfoList)
{
   INIT_DM_PORT_CONNECT_INFO_LIST(&a_portConnectionInfoList);

   // Loop through all transmit ports of 1st device
   int k = 0;
   int i;
   for (i = 0; i < a_txPortInfoList.unCount; ++i)
   {
      DM_PORT_MEDIA_TYPE type_tx = a_txPortInfoList.port_info[i].port_media_type;
      if (type_tx == DM_PORT_MEDIA_TYPE_NBUP)
      {
         continue;
      }

      // find appropriate RX port on 2nd device
      bool bFound = false;
      int j;
      for (j = 0; j < a_rxPortInfoList.unCount; ++j)
      {
         DM_PORT_MEDIA_TYPE type_rx = a_rxPortInfoList.port_info[j].port_media_type;
         if (type_tx == type_rx)
         {
            bFound = true;
            break;
         }
      }
      if (!bFound)
      {
         continue;
      }
      // create element of connect list
      DM_PORT_CONNECT_INFO& info = a_portConnectionInfoList.port_connect_info[k];
      INIT_DM_PORT_CONNECT_INFO(&info);

      info.unFlags = DMFL_TRANSCODE_ON;
      info.port_info_tx = a_txPortInfoList.port_info[i];
      info.port_info_rx = a_rxPortInfoList.port_info[j];
      ++k;
   }
   a_portConnectionInfoList.unCount = k;
   return k;
}

//*****************************************************************************
// Function: int CRecorder::PrimOnGetMmRxPortInfoComplete()
// Description: Handle a get RX port info from the MM device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnGetMmRxPortInfoComplete()
{
   m_portInfoSubState |= MM_RX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CRecorder::PrimOnGetMmTxPortInfoComplete()
// Description: Handle a get TX port info from the MM device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnGetMmTxPortInfoComplete()
{
   m_portInfoSubState |= MM_TX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CRecorder::PrimOnGetCnfRxPortInfoComplete()
// Description: Handle a get RX port info from the CNF party device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnGetCnfRxPortInfoComplete()
{
   m_portInfoSubState |= CNF_RX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CRecorder::PrimOnGetCnfTxPortInfoComplete()
// Description: Handle a get TX port info from the CNF party device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnGetCnfTxPortInfoComplete()
{
   m_portInfoSubState |= CNF_TX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CRecorder::PrimPortConnect()
// Description: Handle a port connect command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimPortConnect()
{
   m_portConnectSubState = 0;
   int num_matched_ports = CreatePortConnectInfoList( m_portConnectInfoListMmTxCnfRx,
                                                      m_mmDevice->TxPortInfoList(),
                                                      m_conferenceParty->RxPortInfoList());
   if (0 == num_matched_ports)
   {
      LogMsg( Name(), "No matched ports found");
      return -1;
   }

   int res = dev_PortConnect(m_mmDevice->Device(), &m_portConnectInfoListMmTxCnfRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_mmDevice->Device(), "dev_PortConnect");
      return res;
   }
   else
   {
      LogMsg( Name(), "dev_PortConnect");
   }

   num_matched_ports = CreatePortConnectInfoList( m_portConnectInfoListCnfTxMmRx,
                                                  m_conferenceParty->TxPortInfoList(),
                                                  m_mmDevice->RxPortInfoList());
   if (0 == num_matched_ports)
   {
      LogMsg( Name(), "No matched ports found");
      return -1;
   }

   res = dev_PortConnect(m_conferenceParty->Device(), &m_portConnectInfoListCnfTxMmRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_conferenceParty->Device(), "dev_PortConnect");
      return res;
   }
   else
   {
      LogMsg( Name(), "dev_PortConnect");
   }

   return res;
}


//*****************************************************************************
// Function: int CRecorder::PrimOnCnfPortConnected()
// Description: Handle a port connected event from the CNF device
// Return: int - value of port connect substate
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnCnfPortConnected()
{
   m_portConnectSubState |= CNF_TX_MM_RX_CONNECT;
   return m_portConnectSubState;
}

//*****************************************************************************
// Function: int CRecorder::PrimOnMmPortConnected()
// Description: Handle a port connected event from the MM device
// Return: int - value of port connect substate 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnMmPortConnected()
{
   m_portConnectSubState |= MM_TX_CNF_RX_CONNECT;
   return m_portConnectSubState;
}

//*****************************************************************************
// Function: int CRecorder::PrimOnCnfPortDisconnected()
// Description: Handle a port disconnected event from the CNF device
// Return: int - value of port disconnect substate 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnCnfPortDisconnected()
{
   m_portDisconnectSubState |= CNF_TX_MM_RX_DISCONNECT;
   return m_portDisconnectSubState;
}

//*****************************************************************************
// Function: int CRecorder::PrimOnMmPortDisconnected()
// Description: Handle a port disconnected event from the MM device
// Return: int - value of port disconnect substate 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnMmPortDisconnected()
{
   m_portDisconnectSubState |= MM_TX_CNF_RX_DISCONNECT;
   return m_portDisconnectSubState;
}

//*****************************************************************************
// Function: int CRecorder::PrimRecord()
// Description: Handle a play command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimRecord()
{
	if(m_bRecordConf==true) {
   	m_stopRecord = false;
   	return m_mmDevice->MmRecord();
	}
	return 0;
}

//*****************************************************************************
// Function: int CRecorder::PrimOnRecordComplete()
// Description: Handle a play completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimOnRecordComplete()
{
   return(m_stopRecord!=true);
}

//*****************************************************************************
// Function: int CRecorder::PrimPlay()
// Description: Handle a play command
// Return: int - 0 for success, otherwise error
// Parameters: none
//*****************************************************************************
int CRecorder::PrimPlay()
{
   m_stopPlay = false;
   return m_mmDevice->MmPlay();
}

//*****************************************************************************
// Function: int CRecorder::PrimOnPlayComplete()
// Description: Handle a play completion event
// Return: int - 0 for success, otherwise error
// Parameters: none
//*****************************************************************************
int CRecorder::PrimOnPlayComplete()
{
   return(m_stopPlay!=true);
}


//*****************************************************************************
// Function: int CRecorder::PrimStop()
// Description: Handle a stop media operation command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CRecorder::PrimStop()
{
   m_stopRecord = true;
   return m_mmDevice->MmStop();
}


//*****************************************************************************
// Function: void CRecorder::ProcessEvent(METAEVENT &a_metaEvent)
// Description: Process and event
// Return: void 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
void CRecorder::ProcessEvent(METAEVENT& a_metaEvent)
{
   switch (a_metaEvent.evttype)
   {
      case APPEVENT_MM_OPEN_COMPLETE:
// SKS JACK need to move when conference starting
//         StartRecorder();
         break;
		case APPEVENT_ON_CNF_STARTED:
         StartRecorder();
			break;

      case APPEVENT_ON_OPEN_CNF_PARTY_COMPLETE:
         OnOpenConferencePartyComplete();
         break;

      case APPEVENT_ON_CLOSE_CNF_PARTY_COMPLETE:
         OnCloseConferencePartyComplete();
         break;

      case APPEVENT_ON_JOIN_CNF_COMPLETE:
         OnJoinConferenceComplete();
         break;

      case APPEVENT_ON_LEAVE_CNF_COMPLETE:
         OnLeaveConferenceComplete();
         break;
      case APPEVENT_ON_LEAVE_CNF_FAILED:
         OnLeaveConferenceFailed();
         break;

      case APPEVENT_CNF_GET_RX_PORT_INFO_COMPLETE:
         OnGetCnfRxPortInfoComplete();
         break;

      case APPEVENT_CNF_GET_TX_PORT_INFO_COMPLETE:
         OnGetCnfTxPortInfoComplete();
         break;

      case APPEVENT_CNF_PORT_CONNECTED:
         OnCnfPortConnected();
         break;

      case APPEVENT_CNF_PORT_DISCONNECTED:
         OnCnfPortDisconnected();
         break;

      case APPEVENT_MM_GET_RX_PORT_INFO_COMPLETE:
         OnGetMmRxPortInfoComplete();
         break;

      case APPEVENT_MM_GET_TX_PORT_INFO_COMPLETE:
         OnGetMmTxPortInfoComplete();
         break;

      case APPEVENT_MM_PORT_CONNECTED:
         OnMmPortConnected();
         break;

      case APPEVENT_MM_PORT_DISCONNECTED:
         OnMmPortDisconnected();
         break;

     case APPEVENT_PLAY_COMPLETE:
         OnPlayComplete();
         break;

      case APPEVENT_PLAY_FAIL:
         break;

      case APPEVENT_RECORD_COMPLETE:
         OnRecordComplete();
         break;

      case APPEVENT_RECORD_FAIL:
         break;

      default:
         LogMsg(Name(), "unhandled event:" + AsHexString(a_metaEvent.evttype));
   }
}

