/**
* @file CRecorderState.cpp
* @brief Recorder States - class implementations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "config.h"

#include "LogUtil.h"
#include "CRecorderState.h"

#include "globals.h"

CRecorderStates ARecorderState::m_recorderStates;

//*****************************************************************************
// Function: CRecorderStates::CRecorderStates()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CRecorderStates::CRecorderStates()
{
   m_recorderStates[RecorderIdleState] = new CRecorderIdleState();
   m_recorderStates[RecorderSetupState] = new CRecorderSetupState();
   m_recorderStates[RecorderAvailableState] = new CRecorderAvailableState();
   m_recorderStates[RecorderRecordingState] = new CRecorderRecordingState();
   m_recorderStates[RecorderTeardownState] = new CRecorderTeardownState();
}

//*****************************************************************************
// Function: int ARecorderState::LogStateError(CRecorder &a_recorder, const std::string &msg)
// Description: Log an invalid state for operation/event message
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//             const std::string &msg 
//*****************************************************************************
int ARecorderState::LogStateError(CRecorder& a_recorder, const std::string& msg)
{
   std::cout << "a_recorder"
   << " " << Desc() << " is invalid device state for "
   << msg
   << std::endl;
   return -1;
}


//*****************************************************************************
// Function: int CRecorderIdleState::StartRecorder(CRecorder &a_recorder)
// Description: Start the recorder
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderIdleState::StartRecorder( CRecorder& a_recorder)
{
   int res = a_recorder.PrimStartRecorder();
   if (!res)
   {
      a_recorder.SetStateTo(State(RecorderSetupState));
      a_recorder.OpenConferenceParty();
   }
   return res;
}
//*****************************************************************************
// Function: int CRecorderIdleState::OnRecordComplete(CRecorder &a_recorder)
// Description: Handle a record completion event
// Return: int - 0 for success, otherwise error
// Parameters: CRecorder &a_recorder
//*****************************************************************************
int CRecorderIdleState::OnRecordComplete(CRecorder &a_recorder)
{
   return 0;
}

//*****************************************************************************
// Function: int CRecorderIdleState::OnPlayComplete(CRecorder &a_recorder)
// Description: Handle a record completion event
// Return: int - 0 for success, otherwise error
// Parameters: CRecorder &a_recorder
//*****************************************************************************
int CRecorderIdleState::OnPlayComplete(CRecorder &a_recorder)
{
   return 0;
}


//*****************************************************************************
// Function: int CRecorderAvailableState::Record(CRecorder &a_recorder)
// Description: P{lay the recorder media clip
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderAvailableState::Record(CRecorder &a_recorder)
{
   a_recorder.SetStateTo(State(RecorderRecordingState));
   return a_recorder.PrimRecord();
}

//*****************************************************************************
// Function: int CRecorderAvailableState::LeaveConference(CRecorder &a_recorder)
// Description: Leave the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderAvailableState::LeaveConference(CRecorder &a_recorder)
{
   a_recorder.SetStateTo(State(RecorderTeardownState));
   return a_recorder.PrimLeaveConference();
}



//*****************************************************************************
// Function: int CRecorderRecordingState::OnRecordComplete(CRecorder &a_recorder)
// Description: Handle a record completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderRecordingState::OnRecordComplete(CRecorder &a_recorder)
{
   if ((a_recorder.PrimOnRecordComplete() != 0) && (g_applicationState == ApplicationStateStarted))
   {
      return a_recorder.PrimRecord();
   }
   else
   {
      a_recorder.SetStateTo(State(RecorderAvailableState));
   }
   return 0;
}

//*****************************************************************************
// Function: int CRecorderRecordingState::OnPlayComplete(CRecorder &a_recorder)
// Description: Handle a record completion event
// Return: int - 0 for success, otherwise error
// Parameters: CRecorder &a_recorder
//*****************************************************************************
int CRecorderRecordingState::OnPlayComplete(CRecorder &a_recorder)
{
   return 0;
}


//*****************************************************************************
// Function: int CRecorderRecordingState::Stop(CRecorder &a_recorder)
// Description: Stop recording the recorder media clip
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderRecordingState::Stop(CRecorder &a_recorder)
{
   return a_recorder.PrimStop();
}

//*****************************************************************************
// Function: int CRecorderSetupState::OpenConferenceParty(CRecorder &a_recorder)
// Description: Open the conference party
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderSetupState::OpenConferenceParty(CRecorder &a_recorder)
{
   return a_recorder.PrimOpenConferenceParty();
}

//*****************************************************************************
// Function: int CRecorderSetupState::OnOpenConferencePartyComplete(CRecorder &a_recorder)
// Description: Handle an open conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderSetupState::OnOpenConferencePartyComplete(CRecorder &a_recorder)
{
   int res = a_recorder.PrimOnOpenConferencePartyComplete();
   if (res == 0)
   {
      res = a_recorder.ConnectToConferenceParty();
   }
   return res;
}

//*****************************************************************************
// Function: int CRecorderSetupState::ConnectToConferenceParty(CRecorder &a_recorder)
// Description: Connect to the conference party
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderSetupState::ConnectToConferenceParty(CRecorder &a_recorder)
{
   return a_recorder.PrimConnectToConferenceParty();
}

//*****************************************************************************
// Function: int CRecorderSetupState::OnGetMmRxPortInfoComplete(CRecorder &a_recorder)
// Description: Handle a get RX port info completion event from the MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderSetupState::OnGetMmRxPortInfoComplete(CRecorder& a_recorder)
{
   if (a_recorder.PrimOnGetMmRxPortInfoComplete() == CRecorder::PORT_INFO_COMPLETE)
   {
      return a_recorder.PrimPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CRecorderSetupState::OnGetMmTxPortInfoComplete(CRecorder &a_recorder)
// Description: Handle a get TX port info completion event from the MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderSetupState::OnGetMmTxPortInfoComplete(CRecorder& a_recorder)
{
   if (a_recorder.PrimOnGetMmTxPortInfoComplete() == CRecorder::PORT_INFO_COMPLETE)
   {
      return a_recorder.PrimPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CRecorderSetupState::OnGetCnfRxPortInfoComplete(CRecorder &a_recorder)
// Description: Handle a get RX port info completion event from the CNF party device
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderSetupState::OnGetCnfRxPortInfoComplete(CRecorder& a_recorder)
{
   if (a_recorder.PrimOnGetCnfRxPortInfoComplete() == CRecorder::PORT_INFO_COMPLETE)
   {
      return a_recorder.PrimPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CRecorderSetupState::OnGetCnfTxPortInfoComplete(CRecorder &a_recorder)
// Description: Handle a get TX port info completion event from the CNF party device
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderSetupState::OnGetCnfTxPortInfoComplete(CRecorder& a_recorder)
{
   if (a_recorder.PrimOnGetCnfTxPortInfoComplete() == CRecorder::PORT_INFO_COMPLETE)
   {
      return a_recorder.PrimPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CRecorderSetupState::OnCnfPortConnected(CRecorder &a_recorder)
// Description: Handle a port connected event from the CNF party device
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderSetupState::OnCnfPortConnected(CRecorder& a_recorder)
{
   if (a_recorder.PrimOnCnfPortConnected() == CRecorder::PORT_CONNECT_COMPLETE)
   {
      return a_recorder.OnConnectToConferencePartyComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CRecorderSetupState::OnMmPortConnected(CRecorder &a_recorder)
// Description: Handle a port connected event from the MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderSetupState::OnMmPortConnected(CRecorder& a_recorder)
{
   if (a_recorder.PrimOnMmPortConnected() == CRecorder::PORT_CONNECT_COMPLETE)
   {
      return a_recorder.OnConnectToConferencePartyComplete();
   }
   return 0;

}

//*****************************************************************************
// Function: int CRecorderSetupState::OnConnectToConferencePartyComplete(CRecorder &a_recorder)
// Description: Handle a connect to conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderSetupState::OnConnectToConferencePartyComplete(CRecorder &a_recorder)
{
   LogMsg(a_recorder.Name(), "OnConnectToConferencePartyComplete");

   int res = a_recorder.PrimOnConnectToConferencePartyComplete();
   if (res == 0)
   {
      res = a_recorder.JoinConference();
   }
   return res;
}

//*****************************************************************************
// Function: int CRecorderSetupState::JoinConference(CRecorder &a_recorder)
// Description: Join the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderSetupState::JoinConference(CRecorder &a_recorder)
{
   return a_recorder.PrimJoinConference();
}

//*****************************************************************************
// Function: int CRecorderSetupState::OnJoinConferenceComplete(CRecorder &a_recorder)
// Description: Handle a join conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderSetupState::OnJoinConferenceComplete(CRecorder& a_recorder)
{
   a_recorder.SetStateTo(State(RecorderAvailableState));
   a_recorder.PrimOnJoinConferenceComplete();
   return a_recorder.Record();
}

//*****************************************************************************
// Function: int CRecorderRecordingState::Record(CRecorder &a_recorder)
// Description: P{lay the recorder media clip
// Return: int - 0 for success, otherwise error
// Parameters: CRecorder &a_recorder
//*****************************************************************************
int CRecorderRecordingState::Play(CRecorder &a_recorder)
{
   return a_recorder.PrimPlay();
}


//*****************************************************************************
// Function: int CRecorderRecordingState::LeaveConference(CRecorder &a_recorder)
// Description: Leave the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderRecordingState::LeaveConference(CRecorder &a_recorder)
{
   a_recorder.Stop();
   a_recorder.SetStateTo(State(RecorderTeardownState));
   return a_recorder.PrimLeaveConference();
}

//*****************************************************************************
// Function: int CRecorderTeardownState::OnLeaveConferenceComplete(CRecorder &a_recorder)
// Description: Handle a leave conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderTeardownState::OnLeaveConferenceComplete(CRecorder &a_recorder)
{
   int res = a_recorder.PrimOnLeaveConferenceComplete();
   if (res == 0)
   {
      res = a_recorder.DisconnectFromConferenceParty();
   }
   return res;
}
//*****************************************************************************
// Function: int CRecorderTeardownState::OnLeaveConferenceFailed(CRecorder &a_recorder)
// Description: Handle a leave conference completion event
// Return: int - 0 for success, otherwise error
// Parameters: CRecorder &a_recorder
//*****************************************************************************
int CRecorderTeardownState::OnLeaveConferenceFailed(CRecorder &a_recorder)
{
   int res=0;
// SKS TODO when fail this moment focibly disconnect and close conference
	res = a_recorder.PrimOnLeaveConferenceComplete();
   if (res == 0)
   {
      res = a_recorder.DisconnectFromConferenceParty();
   }
   return res;

}


//*****************************************************************************
// Function: int CRecorderTeardownState::DisconnectFromConferenceParty(CRecorder &a_recorder)
// Description: Disconnect from the conferenmce party
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderTeardownState::DisconnectFromConferenceParty(CRecorder &a_recorder)
{
   return a_recorder.PrimDisconnectFromConferenceParty();
}

//*****************************************************************************
// Function: int CRecorderTeardownState::OnCnfPortDisconnected(CRecorder &a_recorder)
// Description: Handle a port disconnectred event from the CNF party device
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderTeardownState::OnCnfPortDisconnected(CRecorder& a_recorder)
{
   if (a_recorder.PrimOnCnfPortDisconnected() == CRecorder::PORT_DISCONNECT_COMPLETE)
   {
      return a_recorder.OnDisconnectFromConferencePartyComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CRecorderTeardownState::OnMmPortDisconnected(CRecorder &a_recorder)
// Description: Handle a port disconnectred event from the MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderTeardownState::OnMmPortDisconnected(CRecorder& a_recorder)
{
   if (a_recorder.PrimOnMmPortDisconnected() == CRecorder::PORT_DISCONNECT_COMPLETE)
   {
      return a_recorder.OnDisconnectFromConferencePartyComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CRecorderTeardownState::OnDisconnectFromConferencePartyComplete(CRecorder &a_recorder)
// Description: Handle a disconnect from conferenmce party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderTeardownState::OnDisconnectFromConferencePartyComplete(CRecorder &a_recorder)
{
   int res = a_recorder.PrimOnDisconnectFromConferencePartyComplete();
   if (res == 0)
   {
      res = a_recorder.CloseConferenceParty();
   }
   return res;
}

//*****************************************************************************
// Function: int CRecorderTeardownState::CloseConferenceParty(CRecorder &a_recorder)
// Description: Close the conference party
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int CRecorderTeardownState::CloseConferenceParty(CRecorder &a_recorder)
{
   return a_recorder.PrimCloseConferenceParty();
}

//*****************************************************************************
// Function: int ARecorderState::OnCloseConferencePartyComplete(CRecorder &a_recorder)
// Description: Handle a close conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CRecorder &a_recorder 
//*****************************************************************************
int ARecorderState::OnCloseConferencePartyComplete(CRecorder &a_recorder)
{
// SKS
	int res=a_recorder.PrimOnCloseConferencePartyComplete();
	if(res==0)
	{
		a_recorder.SetStateTo(State(RecorderIdleState));
	}
	return res;
}


