/**
* @file CRecorderState.h
* @brief Recorder Class States - class declarations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CRecorderState_h
#define CRecorderState_h

#include <map>

#include "CRecorder.h"

typedef enum 
{
   RecorderIdleState = 0,
   RecorderSetupState,
   RecorderAvailableState,
   RecorderRecordingState,
   RecorderTeardownState
} RecorderState;

class ARecorderState;

class CRecorderStates
{
public:
   CRecorderStates();
   virtual ~CRecorderStates() {}
   ARecorderState* State(RecorderState a_recorderState) {return m_recorderStates[a_recorderState];}
private:
   std::map< RecorderState, ARecorderState*> m_recorderStates;
};

//*****************************************************************************
// Class: ::ARecorderState
// Description: Abstract recorder state
// Notes:      
//    This abstract class declares all functions for all Recorder states 
//    as virtual and implements them as an error.
// 
//    Concrete subclasses implement only the functions that are valid for that
//    state.
// 
//    State classes are responsible only for calling primative functions on the
//    context recorder object and changing the state of the context 
//    recorder object.
// 
//    As state classes have no non-static member variables, one set of recorder
//    state objects are shared by all recorder objects
//*****************************************************************************
class ARecorderState
{
public:
   ARecorderState() {}
   virtual ~ARecorderState() {}

   virtual const char* Desc() = 0;

   virtual int StartRecorder(CRecorder& a_recorder) {return LogStateError(a_recorder, "StartRecorder");}
   virtual int OpenConferenceParty(CRecorder& a_recorder) {return LogStateError(a_recorder, "OpenConferenceParty");}
   virtual int OnOpenConferencePartyComplete(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnOpenConferencePartyComplete");}
   virtual int ConnectToConferenceParty(CRecorder& a_recorder) {return LogStateError(a_recorder, "ConnectToConferenceParty");}
   virtual int OnConnectToConferencePartyComplete(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnConnectToConferencePartyComplete");}
   virtual int JoinConference(CRecorder& a_recorder) {return LogStateError(a_recorder, "JoinConference");}
   virtual int OnJoinConferenceComplete(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnJoinConferenceComplete");}

   virtual int OnGetMmRxPortInfoComplete(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnGetMmRxPortInfoComplete");}
   virtual int OnGetMmTxPortInfoComplete(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnGetMmTxPortInfoComplete");}
   virtual int OnGetCnfRxPortInfoComplete(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnGetCnfRxPortInfoComplete");}
   virtual int OnGetCnfTxPortInfoComplete(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnGetCnfTxPortInfoComplete");}
 
   virtual int OnCnfPortConnected(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnCnfPortConnected");}
   virtual int OnMmPortConnected(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnMmConnected");}

   virtual int Play(CRecorder& a_recorder) {return LogStateError(a_recorder, "Play");}
   virtual int OnPlayComplete(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnPlayComplete");}
   virtual int Record(CRecorder& a_recorder) {return LogStateError(a_recorder, "Record");}
   virtual int OnRecordComplete(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnRecordComplete");}
   virtual int Stop(CRecorder& a_recorder) {return LogStateError(a_recorder, "Stop");}

   virtual int OnCnfPortDisconnected(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnCnfPortDisconnected");}
   virtual int OnMmPortDisconnected(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnMmPortDisconnected");}

   virtual int LeaveConference(CRecorder& a_recorder) {return LogStateError(a_recorder, "LeaveConference");}
   virtual int OnLeaveConferenceComplete(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnLeaveConferenceComplete");}
   virtual int OnLeaveConferenceFailed(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnLeaveConferenceFailed");}
   virtual int DisconnectFromConferenceParty(CRecorder& a_recorder) {return LogStateError(a_recorder, "DisconnectFromConferenceParty");}
   virtual int OnDisconnectFromConferencePartyComplete(CRecorder& a_recorder) {return LogStateError(a_recorder, "OnDisconnectFromConferencePartyComplete");}
   virtual int CloseConferenceParty(CRecorder& a_recorder) {return LogStateError(a_recorder, "CloseConferenceParty");}
   virtual int OnCloseConferencePartyComplete(CRecorder& a_recorder);

   static ARecorderState* State(RecorderState a_recorderState) {return m_recorderStates.State(a_recorderState);}

private:
   int LogStateError(CRecorder& a_recorder, const std::string& msg);
   static CRecorderStates m_recorderStates;
};

//*****************************************************************************
// Class: ::CRecorderIdleState
// Description: Recorder idle state
//*****************************************************************************
class CRecorderIdleState : public ARecorderState
{
public:
   CRecorderIdleState() {}
   int OnRecordComplete(CRecorder& a_recorder);
   int OnPlayComplete(CRecorder& a_recorder);
   virtual ~CRecorderIdleState() {}
   const char* Desc() {return "idle";}
   int StartRecorder(CRecorder& a_recorder);
};

//*****************************************************************************
// Class: ::CRecorderSetupState
// Description: Recorder setup state
//*****************************************************************************
class CRecorderSetupState : public ARecorderState
{
public:
   CRecorderSetupState() {}
   virtual ~CRecorderSetupState() {}
   const char* Desc() {return "setup";}
   int OpenConferenceParty(CRecorder& a_recorder);
   int OnOpenConferencePartyComplete(CRecorder& a_recorder);
   int ConnectToConferenceParty(CRecorder& a_recorder);
   int OnConnectToConferencePartyComplete(CRecorder& a_recorder);
   int JoinConference(CRecorder& a_recorder);
   int OnJoinConferenceComplete(CRecorder& a_recorder);
   int OnGetMmRxPortInfoComplete(CRecorder& a_recorder);
   int OnGetMmTxPortInfoComplete(CRecorder& a_recorder);
   int OnGetCnfRxPortInfoComplete(CRecorder& a_recorder);
   int OnGetCnfTxPortInfoComplete(CRecorder& a_recorder);
   int OnCnfPortConnected(CRecorder& a_recorder);
   int OnMmPortConnected(CRecorder& a_recorder);
};

//*****************************************************************************
// Class: ::CRecorderAvailableState
// Description: Recorder available state
//*****************************************************************************
class CRecorderAvailableState : public ARecorderState
{
public:
   CRecorderAvailableState() {}
   virtual ~CRecorderAvailableState() {}
   const char* Desc() {return "Available";}
   int Record(CRecorder& a_recorder);
   int LeaveConference(CRecorder& a_recorder);
};

//*****************************************************************************
// Class: ::CRecorderRecordingState
// Description: Recorder recording state
//*****************************************************************************
class CRecorderRecordingState : public ARecorderState
{
public:
   CRecorderRecordingState() {}
   virtual ~CRecorderRecordingState() {}
   const char* Desc() {return "recording";}
   int OnRecordComplete(CRecorder& a_recorder);
   int Play(CRecorder& a_recorder);
   int OnPlayComplete(CRecorder& a_recorder);
   int Stop(CRecorder& a_recorder);
   int LeaveConference(CRecorder& a_recorder);
};

//*****************************************************************************
// Class: ::CRecorderTeardownState
// Description: Recorder teardown state
//*****************************************************************************
class CRecorderTeardownState : public ARecorderState
{
public:
   CRecorderTeardownState() {}
   virtual ~CRecorderTeardownState() {}
   const char* Desc() {return "teardown";}
   int OnCnfPortDisconnected(CRecorder& a_recorder);
   int OnMmPortDisconnected(CRecorder& a_recorder);
   int OnLeaveConferenceComplete(CRecorder& a_recorder);
   int OnLeaveConferenceFailed(CRecorder& a_recorder);
   int DisconnectFromConferenceParty(CRecorder& a_recorder);
   int OnDisconnectFromConferencePartyComplete(CRecorder& a_recorder);
   int CloseConferenceParty(CRecorder& a_recorder);
};

#endif

