/*
 * This file is part of Project DiaStar Server.
 *
 * More information about this project can be found at:
 * http://www.projectdiastar.org.
 *
 * Copyright (C) 2010 Dialogic Corp.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 * Alternatively see <http://www.gnu.org/licenses/>.
 * Or see the LICENSE file included within the source tree.
 */

/*!
 * \file   rtspconnector.cxx
 * \brief  RTSP client based on the live555 RTSP library.
 * \author John Tarlton
 */


#include <iostream>
#include <iomanip>

#include "CRtspConnector.h"
//#include "logger.h"
#include "LogUtil.h"

#include <BasicUsageEnvironment.hh>
#include <GroupsockHelper.hh>
#include <liveMedia.hh>
static const unsigned int RTSP_KEEPALIVE_TIMEOUT_SECONDS = 30;

/*!
 *  Overload << for printing MediaSubsession objects.
 */
inline std::ostream& operator << ( std::ostream &os, MediaSubsession& subsession )
{
	os << "mediumName: " << subsession.mediumName() << std::endl;
	os << "clientPortNum: " << subsession.clientPortNum() << std::endl;
	os << "rtpPayloadFormat: " << static_cast<int>(subsession.rtpPayloadFormat()) << std::endl;
	os << "rtpSource: " << subsession.rtpSource() << std::endl;
	os << "mediumName: " << subsession.mediumName() << std::endl;
	os << "codecName: " << subsession.codecName() << std::endl;
	os << "protocolName: " << subsession.protocolName() << std::endl;
	os << "controlPath: " << subsession.controlPath() << std::endl;

	if ( strcasecmp(subsession.mediumName(), "audio") == 0 )
	{
		os << "fmtp_objecttype: " << subsession.fmtp_objecttype() << std::endl;
		os << "fmtp_octetalign: " << subsession.fmtp_octetalign() << std::endl;
		os << "fmtp_robustsorting: " << subsession.fmtp_robustsorting() << std::endl;
	}

	if ( strcasecmp(subsession.mediumName(), "video") == 0 )
	{
		os << "videoWidth: " << subsession.videoWidth() << std::endl;
		os << "videoHeight: " << subsession.videoHeight() << std::endl;
		os << "videoFPS: " << subsession.videoFPS() << std::endl;
		os << "fmtp_profile_level_id: " << std::hex << subsession.fmtp_profile_level_id() << std::dec << std::endl;
		os << "fmtp_sizelength: " << subsession.fmtp_sizelength() << std::endl;
		os << "fmtp_streamtype: " << subsession.fmtp_streamtype() << std::endl;
		os << "fmtp_config: " << subsession.fmtp_config() << std::endl;
		//os << "fmtp_mode: " << subsession.fmtp_mode() << std::endl;
		//os << "fmtp_spropparametersets: " << subsession.fmtp_spropparametersets() << std::endl;
	}

	os << "playStartTime: " << subsession.playStartTime() << std::endl;
	os << "playEndTime: " << subsession.playEndTime() << std::endl;
	return os;
}


/*!
 * \struct RtspCallbackInfo
 * Container used for passing data to task scheduler callback functions.
 */
struct RtspCallbackInfo
{
	RtspConnector* rtsp;
	int            session_id;
};


/*
 * ctor.
 */
RtspConnector::RtspConnector( const std::string& appname,
                              RtspConnector::callback_t callback )
 	: user_agent_ (appname),
	  callback_ (callback),
	  session_id_ (0)
{
	scheduler_ = BasicTaskScheduler::createNew();
	env_ = BasicUsageEnvironment::createNew(*scheduler_);

	LogMsg(Name(), "CRtspConnector Constructor");
}


/*
 * dtor, base class will cleanup the thread.
 */
RtspConnector::~RtspConnector()
{
	// destroy sessions
	watch_ = 1;       /* cancel task scheduler */
}


/*
 * Thread entry point, delegate the real work to the rtsp task scheduler.
 */
int RtspConnector::run()
{
	//LOGDEBUG("RtspConnector::run()");
	LogMsg(Name(), "CRtspConnector::run()");

	RtspCallbackInfo info;
	info.rtsp = this;
	env_->taskScheduler().scheduleDelayedTask(0, monitor, &info);

	watch_ = 0;
	env_->taskScheduler().doEventLoop(&watch_);

	//LOGDEBUG("RtspConnector::run() exit");
	LogMsg(Name(), "CRtspConnector::run() exit");
	
	return 0;
}


/*
 * Lookup a session object using its public id.
 */
RtspConnector::Session* RtspConnector::findSession( int session_id ) const
{
	session_map_t::const_iterator i = sessions_.find(session_id);
	if ( i == sessions_.end() )
	{
		return 0;
	}
	return (*i).second;
}


/*
 * Create a request object and append it to the work queue.
 */
void RtspConnector::open( const std::string& url,
                          const std::string& username,
                          const std::string& password,
                          int audio_port,
                          int video_port,
                          void* userdata )
{
	RtspRequest* request = new RtspRequest(RTSP_CMD_OPEN);
	request->url = url;
	request->username = username;
	request->password = password;
	request->audio_port = audio_port;
	request->video_port = video_port;
	request->userdata = userdata;
	put(request);
}


/*
 * Create a request object and append it to the work queue.
 */
void RtspConnector::play( int session_id )
{
	RtspRequest* request = new RtspRequest(RTSP_CMD_PLAY);
	request->session_id = session_id;
	put(request);
}


/*
 * Create a request object and append it to the work queue.
 */
void RtspConnector::stop( int session_id )
{
	LogMsg(Name(), "CRtspConnector::stop() id="+ AsString(session_id));
	RtspRequest* request = new RtspRequest(RTSP_CMD_STOP);
	request->session_id = session_id;
	put(request);
}


/*
 * Task scheduler callback. Poll the workqueue and process pending requests.
 */
/*static*/void RtspConnector::monitor( void* client_data )
{
	RtspCallbackInfo* info = static_cast<RtspCallbackInfo*>(client_data);
	RtspConnector* rtsp = info->rtsp;

	RtspRequest* request = rtsp->get(0);
	while ( request )
	{
		RtspEvent event;
		event.userdata = request->userdata;

		if ( request->cmd == RTSP_CMD_OPEN )
		{
			RtspConnector::Session* session = rtsp->createSession(request->url,
			                                                      request->username,
			                                                      request->password,
			                                                      request->audio_port,
			                                                      request->video_port);

			if ( session )
			{
				session->userdata = request->userdata;

				event.type = RTSP_EVT_OPEN_OK;
				if ( session->audioSubsession )
				{
					event.audio_payload_format = session->audioSubsession->rtpPayloadFormat();
					event.audio_rtsp_port = session->audioSubsession->serverPortNum;
					event.audio_rtsp_addr = session->audioSubsession->connectionEndpointAddress();

					if ( strcasecmp(session->audioSubsession->codecName(), "PCMA") == 0 )
					{
						event.audio_codec = RTSP_CODEC_PCMA;
					}
					else if ( strcasecmp(session->audioSubsession->codecName(), "PCMU") == 0 )
					{
						event.audio_codec = RTSP_CODEC_PCMU;
					}
					else if ( strcasecmp(session->audioSubsession->codecName(), "L16") == 0 )
					{
						event.audio_codec = RTSP_CODEC_L16;
					}
					else if ( strcasecmp(session->audioSubsession->codecName(), "AMR") == 0 )
					{
						event.audio_codec = RTSP_CODEC_AMR;
					}
					else if ( strcasecmp(session->audioSubsession->codecName(), "G726-32") == 0 )
					{
						event.audio_codec = RTSP_CODEC_G726_32;
					}
					else if ( strcasecmp(session->audioSubsession->codecName(), "G722") == 0 )
					{
						event.audio_codec = RTSP_CODEC_G722;
					}
					else if ( strcasecmp(session->audioSubsession->codecName(), "G729") == 0 )
					{
						event.audio_codec = RTSP_CODEC_G729;
					}
					else
					{
						//LOGERROR( "RtspConnector::monitor() unsupported audio codec: " << session->audioSubsession->codecName());
						LogMsg("RtspConn", "CRtspConnector::monitor() unsupported audio codec: " + AsString(session->audioSubsession->codecName()));
						event.type = RTSP_EVT_OPEN_FAILED;
					}
				}
				if ( session->videoSubsession )
				{
					event.video_payload_format = session->videoSubsession->rtpPayloadFormat();
					event.video_rtsp_port = session->videoSubsession->serverPortNum;
					event.video_rtsp_addr = session->videoSubsession->connectionEndpointAddress();

					if ( strcasecmp(session->videoSubsession->codecName(), "H263") == 0 )
					{
						event.video_codec = RTSP_CODEC_H263;
					}
					else if ( strcasecmp(session->videoSubsession->codecName(), "H263-1998") == 0 )
					{
						event.video_codec = RTSP_CODEC_H263_1998;
					}
					else if ( strcasecmp(session->videoSubsession->codecName(), "H264") == 0 )
					{
						event.video_codec = RTSP_CODEC_H264;
					}
					else if ( strcasecmp(session->videoSubsession->codecName(), "MP4V-ES") == 0 )
					{
						event.video_codec = RTSP_CODEC_MP4VES;
					}
					else
					{
						//LOGERROR("RtspConnector::monitor() unsupported video codec: " << session->videoSubsession->codecName());
						LogMsg("RtspConn", "CRtspConnector::monitor() unsupported video codec: " + AsString(session->videoSubsession->codecName()));
						event.type = RTSP_EVT_OPEN_FAILED;
					}
					event.height = session->videoSubsession->videoHeight();
					event.width = session->videoSubsession->videoWidth();
					event.fps = session->videoSubsession->videoFPS();
					event.profile_level_id = session->videoSubsession->fmtp_profile_level_id();

					const char* sprops = session->videoSubsession->fmtp_spropparametersets();
					if ( sprops )
					{
						if ( strlen(sprops) < sizeof(event.sprop_parameter_sets) )
						{
							strcpy(event.sprop_parameter_sets, sprops);
						}
						else
						{
							LogMsg("RtspConn", "CRtspConnector::monitor() sprop-parameter-sets exceeds buffer size");
							event.type = RTSP_EVT_OPEN_FAILED;
						}
					}
					else
					{
						event.sprop_parameter_sets[0] = '\0';
					}
				}

				if ( event.type == RTSP_EVT_OPEN_OK )
				{
					event.session_id = session->id;
				}
				else /* RTSP_EVT_OPEN_FAILED */
				{
					rtsp->destroySession(session);
				}
			}
			else /* No session. */
			{
				event.type = RTSP_EVT_OPEN_FAILED;
			}
			(*rtsp->callback_)(&event);
		}
		else if ( request->cmd == RTSP_CMD_PLAY )
		{
			Session* session = rtsp->findSession(request->session_id);
			if ( !session )
			{
				event.type = RTSP_EVT_PLAY_FAILED;
				event.session_id = request->session_id;
			}
			else
			{
				if ( rtsp->playSession(session) )
				{
					event.type = RTSP_EVT_PLAY_OK;
				}
				else
				{
					event.type = RTSP_EVT_PLAY_FAILED;
				}
				event.session_id = session->id;
				event.userdata = session->userdata;
			}
			(*rtsp->callback_)(&event);
		}
		else if ( request->cmd == RTSP_CMD_STOP )
		{
			Session* session = rtsp->findSession(request->session_id);
			if ( !session )
			{
				event.type = RTSP_EVT_STOP_FAILED;
				event.session_id = request->session_id;
			}
			else
			{
				event.type = RTSP_EVT_PLAY_COMPLETED;
				event.session_id = session->id;
				event.userdata = session->userdata;
				rtsp->destroySession(session);
			}
			(*rtsp->callback_)(&event);
		}
		else
		{
			event.type == RTSP_EVT_ERROR;
			(*rtsp->callback_)(&event);
		}
		delete request;

		/* next */
		request = rtsp->get(0);
	}

	/* Call this again, after a 10ms.
	 * client_data is always valid so can be reused.
	 */
	int64_t useconds = 10000LL;
	rtsp->env_->taskScheduler().scheduleDelayedTask(useconds, monitor, client_data);
}


/*
 * Task scheduler callback for play timeouts.
 */
/*static*/void RtspConnector::onEndPlay( void* client_data )
{
	//LOGDEBUG("RtspConnector::onEndPlay()");
	LogMsg("RtspConn", "CRtspConnector::onEndPlay()");

	RtspCallbackInfo* info = static_cast<RtspCallbackInfo*>(client_data);
	Session* session = info->rtsp->findSession(info->session_id);
	if ( session )
	{
		RtspEvent event;
		event.type = RTSP_EVT_PLAY_COMPLETED;
		event.session_id = session->id;
		event.userdata = session->userdata;
		(*session->owner.callback_)(&event);

		session->owner.destroySession(session);
	}
	delete info;
}


/*
 * Task scheduler callback for keeping a session alive.
 */
/*static*/void RtspConnector::onKeepAlive( void* client_data )
{
	RtspCallbackInfo* info = static_cast<RtspCallbackInfo*>(client_data);
	Session* session = info->rtsp->findSession(info->session_id);
	if ( session )
	{

		bool is_alive = false;
/*  JM - Something wrong here, force GET_PARAMETER for Axis Camera: 
		if ( strstr(session->options.c_str(), "GET_PARAMETER") == NULL )
		{
			std::string dummy_options;
			//LOGDEBUG("RtspConnector::onKeepAlive() id: " << info->session_id);
			LogMsg("RtspConn", "CRtspConnector::onKeepAlive() OPTIONS id: " + AsString(info->session_id) );
			is_alive = info->rtsp->sendOptions(session->client,
		                             session->url,
		                             session->username,
		                             session->password,
		                             3,
									 dummy_options); 
		}
		else
*/
		{
			char* dummy;
			LogMsg("RtspConn", "CRtspConnector::onKeepAlive() GET_PARAMETER id: " + AsString(info->session_id) );
			is_alive = session->client->getMediaSessionParameter(*(session->mediaSession), NULL, dummy);
		}
		if ( is_alive )
		{
			/* Re-use info */
			int64_t useconds = RTSP_KEEPALIVE_TIMEOUT_SECONDS * 1000000LL;
			info->rtsp->env_->taskScheduler().scheduleDelayedTask(useconds, onKeepAlive, info);
		}
		else
		{
			//LOGDEBUG("RtspConnector::onKeepAlive() no response. Ending session, id: " << info->session_id);
			LogMsg("RtspConn", "CRtspConnector::onKeepAlive() no response. Ending session, id: " + AsString(info->session_id) );
			RtspEvent event;
			event.type = RTSP_EVT_PLAY_COMPLETED;
			event.session_id = session->id;
			event.userdata = session->userdata;
			(*session->owner.callback_)(&event);

			session->owner.destroySession(session);
			delete info;
		}
	}
	else
	{
		//LOGDEBUG("RtspConnector::onKeepAlive() session has ended, id: " << info->session_id);
		LogMsg("RtspConn", "CRtspConnector::onKeepAlive() session has ended, id: " + AsString(info->session_id) );
		delete info;
	}
}


/*
 * Generates a unique session id.
 */
int RtspConnector::generateSessionId()
{
	++session_id_;
	return session_id_;
}


/*
 * Connect to the rstp server and create a session for the media.
 */
RtspConnector::Session* RtspConnector::createSession( const std::string& url,
                                                      const std::string& username,
                                                      const std::string& password,
                                                      int audio_port,
                                                      int video_port )
{
	int timeout_seconds = 5;

	LogMsg(Name(), "CRtspConnector::createSession(): url=" + AsString(url) );
	LogMsg(Name(), "CRtspConnector::createSession(): audio_port=" + AsString(audio_port) );
	LogMsg(Name(), "CRtspConnector::createSession(): video_port=" + AsString(video_port) );
	
	RTSPClient* client = RTSPClient::createNew(*env_, RTSP_VERBOSITY_LEVEL, user_agent_.c_str(), 0);
	if ( !client )
	{
		//LOGERROR("RtspConnector::createSession() failed to create client");
		LogMsg(Name(), "ERROR: CRtspConnector::createSession() failed to create client" );
		return 0;
	}

	/* Check that the url looks sane.
	 */
	NetAddress address;
	portNumBits port;
	char* uname = NULL;
    char* passwd = NULL;
	if ( !RTSPClient::parseRTSPURL(*env_, url.c_str(), uname, passwd, address, port, NULL) )
	//JM - Enable for old live555 stack
        //if ( !RTSPClient::parseRTSPURL(*env_, url.c_str(), address, port, 0) )
	
	{
		//LOGERROR("RtspConnector::createSession() Invalid URL: " << url);
		LogMsg(Name(), "ERROR: CRtspConnector::createSession(): Invalid URL=" + AsString(url) );
		Medium::close(client);
		return 0;
	}

	//LOGDEBUG("RtspConnector::createSession() url: " << url);
	LogMsg(Name(), "CRtspConnector::createSession(OK): url=" + AsString(url) );

	/* Send an OPTIONS request.
	 */
	std::string options;
	if ( !sendOptions(client, url, username, password, timeout_seconds, options) )
	{
		LogMsg(Name(), "ERROR: CRtspConnector::createSession(): sendOptions() exiting create..." );
		Medium::close(client);
		return 0;
	}

	/* Open the URL to get a SDP description.
	 */
	char* sdp_description = 0;
	if ( !username.empty() )
	{
		sdp_description = client->describeWithPassword(url.c_str(),
		                                               username.c_str(),
		                                               password.c_str(),
		                                               False,
		                                               timeout_seconds);
	}
	else
	{
		sdp_description = client->describeURL(url.c_str(),
		                                      NULL,
		                                      False,
		                                      timeout_seconds);
	}
	if ( !sdp_description )
	{
		//LOGERROR("RtspConnector::createSession() Failed to get a SDP description from URL \""  << url << "\" " << env_->getResultMsg());
		LogMsg(Name(), "ERROR: CRtspConnector::createSession(): Failed to get a SDP description from URL=" + AsString(url) );
		Medium::close(client);
		return 0;
	}
	//LOGDEBUG("RtspConnector::createSession() sdp:" << std::endl << sdp_description);
	LogMsg(Name(), "CRtspConnector::createSession() sdp: \n" + AsString(sdp_description) );

	/* Create a media session object.
	 */
	MediaSession* media_session = MediaSession::createNew(*env_, sdp_description);
	delete[] sdp_description;
	if ( !media_session )
	{
		//LOGERROR("RtspConnector::createSession() Failed to create a MediaSession: " << env_->getResultMsg());
		LogMsg(Name(), "ERROR: CRtspConnector::createSession(): Failed to create a MediaSession:" + AsString(env_->getResultMsg()) );
		Medium::close(client);
		return 0;
	}

	if ( !media_session->hasSubsessions() )
	{
		//LOGERROR("RtspConnector::createSession() Session has no media subsessions.");
		LogMsg(Name(), "ERROR: CRtspConnector::createSession(): Session has no media subsessions:" );
		Medium::close(media_session);
		Medium::close(client);
		return 0;
	}

	/* Find subsessions.
	 */
	MediaSubsession* audio_subsession = 0;
	MediaSubsession* video_subsession = 0;

	MediaSubsessionIterator iter(*media_session);
	MediaSubsession* subsession = iter.next();
	while ( subsession )
	{
		if ( audio_port != 0 )
		{
			if ( strcasecmp(subsession->mediumName(), "audio") == 0 )
			{
				//LOGDEBUG("RtspConnector::createSession()" << std::endl << *subsession);
				LogMsg(Name(), "CRtspConnector::createSession(): " + AsString(subsession->mediumName()) );
				audio_subsession = subsession;
				subsession->setClientPortNum(audio_port);
			}
		}
		if ( video_port != 0 )
		{
			if ( strcasecmp(subsession->mediumName(), "video") == 0 )
			{
				//LOGDEBUG("RtspConnector::createSession()" << std::endl << *subsession);
				LogMsg(Name(), "CRtspConnector::createSession(): " + AsString(subsession->mediumName()) );
				video_subsession = subsession;
				subsession->setClientPortNum(video_port);
			}
		}
		subsession = iter.next();
	}

	if ( !audio_subsession && !video_subsession )
	{
		Medium::close(media_session);
		Medium::close(client);
		return 0;
	}

	/* Save session info.
	 */
	Session* session = new Session(*this, generateSessionId());
	session->client = client;
	session->mediaSession = media_session;
	session->audioSubsession = audio_subsession;
	session->videoSubsession = video_subsession;
	session->url = url;
	session->username = username;
	session->password = password;
	session->clientAudioPort = audio_port;
	session->clientVideoPort = video_port;

	sessions_.insert(session_map_t::value_type(session->id, session));
	return session;
}


/*
 *  Tear down the session and release its resources.
 */
void RtspConnector::destroySession( RtspConnector::Session* session )
{
	//LOGDEBUG("RtspConnector::destroySession() id: " << session->id);
	LogMsg(Name(), "CRtspConnector::destroySession() id:" + AsString(session->id) );
	session->client->teardownMediaSession(*(session->mediaSession));
	Medium::close(session->mediaSession);
	Medium::close(session->client);

	sessions_.erase(session->id);
	delete session;
}


/*
 * Send an OPTIONS request to the rstp server.
 */
bool RtspConnector::sendOptions( RTSPClient* client,
                                 const std::string& url,
                                 const std::string& username,
                                 const std::string& password,
                                 int timeout_seconds,
                                 std::string& options )
{
	char* options_response = 0;
	LogMsg(Name(), "CRtspConnector::sendOptions()" );

	if ( !username.empty() )
	{
		/* sendOptionsCmd() takes non-const args. */
		char* uname = strdup(username.c_str());
		char* passwd = strdup(password.c_str());
		options_response = client->sendOptionsCmd(url.c_str(), uname, passwd, NULL, timeout_seconds);
		free(uname);
		free(passwd);
	}
	else
	{
		options_response = client->sendOptionsCmd(url.c_str(), NULL, NULL, NULL, timeout_seconds);
	}
	if ( !options_response )
	{
		return false;
	}
	options = options_response;
	delete[] options_response;
	return true;
}


/*
 * Send a GET_PARAMETER keep alive request to the rstp server.
 */
void RtspConnector::sendGetParameter( RtspConnector::Session* session )
{
	/* Send and GET_PARM request.
	 */

	char* ParmName="GET_PARAMETER";
	char* ParmVal;
	LogMsg(Name(), "CRtspConnector::sendGetParm() id:" + AsString(session->id) );
	session->client->getMediaSessionParameter(*(session->mediaSession), ParmName, ParmVal);
	return;
}


/*
 * Initiate media playback.
 */
bool RtspConnector::playSession( RtspConnector::Session* session )
{
	double start_time = 0.0;
	double end_time = -1.0;          /* Video takes precedence for timeouts. */
	double scale = 1.0;

	LogMsg(Name(), "ERROR CRtspConnector::playSession()" );

	if ( session->audioSubsession )
	{
		session->client->setupMediaSubsession(*(session->audioSubsession), 0, 0);

		if ( session->audioSubsession->playEndTime() > 0.0 )
		{
		  end_time = session->audioSubsession->playEndTime();
	    }

		if ( !session->client->playMediaSession(*(session->mediaSession),
		                                        static_cast<float>(start_time),
		                                        static_cast<float>(end_time),
		                                        static_cast<float>(scale)) )
		{
			//LOGERROR("RtspConnector::playSession() ERROR: " << env_->getResultMsg());
			LogMsg(Name(), "ERROR CRtspConnector::playSession(audio) err=" + AsString(env_->getResultMsg()) );
			return false;
		}
		//LOGDEBUG("RtspConnector::playSession() audio, session_id: " << session->id);
		LogMsg(Name(), "CRtspConnector::playSession() audio, session_id:" + AsString(session->id) );

	}
	if ( session->videoSubsession )
	{
		session->client->setupMediaSubsession(*(session->videoSubsession), 0, 0);

		if ( session->videoSubsession->playEndTime() > 0.0 )
		{
		  end_time = session->videoSubsession->playEndTime();
	    }
		
		if ( !session->client->playMediaSession(*(session->mediaSession),
		                                        static_cast<float>(start_time),
		                                        static_cast<float>(end_time),
		                                        static_cast<float>(scale)) )
		{
			//LOGERROR("RtspConnector::playSession() ERROR: " << env_->getResultMsg());
			LogMsg(Name(), "ERROR CRtspConnector::playSession(video) err=" + AsString(env_->getResultMsg()) );
			return false;
		}
		//LOGDEBUG("RtspConnector::playSession() video, session_id: " << session->id);
		LogMsg(Name(), "CRtspConnector::playSession() video, session_id:" + AsString(session->id) );
	}

	if ( end_time > 0.0 )
	{
		int64_t timeout_useconds = static_cast<int64_t>(end_time * 1000000LL);

		RtspCallbackInfo* info = new RtspCallbackInfo; /* Deleted by onEndPlay. */
		info->rtsp = this;
		info->session_id = session->id;
		env_->taskScheduler().scheduleDelayedTask(timeout_useconds,
		                                          onEndPlay,
		                                          info);
	}
	if ( (end_time <= 0.0) || (end_time > 60.0) )
	{
		RtspCallbackInfo* info = new RtspCallbackInfo; /* Deleted by onKeepAlive. */
		info->rtsp = this;
		info->session_id = session->id;
		int64_t keepalive_useconds = RTSP_KEEPALIVE_TIMEOUT_SECONDS * 1000000LL;
		info->rtsp->env_->taskScheduler().scheduleDelayedTask(keepalive_useconds, onKeepAlive, info);
	}
	return true;
}


/*
 * Initiate a pause.
 */
bool RtspConnector::pauseSession( RtspConnector::Session* session )
{
	LogMsg(Name(), "ERROR CRtspConnector::pauseSession()");
	if ( !session->client->pauseMediaSession(*(session->mediaSession)) )
	{
		//LOGERROR("RtspConnector::pauseSession() ERROR: " << env_->getResultMsg());
		LogMsg(Name(), "ERROR CRtspConnector::pauseSession() err=" + AsString(env_->getResultMsg()) );

		return false;
	}
	return true;
}

/* vim:ts=4:set nu:
 * EOF
 */

