/*
 * This file is part of Project DiaStar Server.
 *
 * More information about this project can be found at:
 * http://www.projectdiastar.org.
 *
 * Copyright (C) 2010 Dialogic Corp.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 * Alternatively see <http://www.gnu.org/licenses/>.
 * Or see the LICENSE file included within the source tree.
 */

/*!
 * \file   rtspconnector.h
 * \brief  RTSP client.
 * \author John Tarlton
 */


#ifndef _RTSPCONNECTOR_H
#define _RTSPCONNECTOR_H

#include <string.h>
#include <string>
#include <map>
#include <ostream>

#include "workqueuethread.h"

/*
 * Forward declarations
 */
class UsageEnvironment;
class TaskScheduler;
class RTSPClient;
class MediaSession;
class MediaSubsession;

#define RTSP_VERBOSITY_LEVEL     1

/*
 * Supported CODECs.
 */
enum RtspCodec
{
	RTSP_CODEC_NONE = 0,

	/* audio */
	RTSP_CODEC_PCMU,
	RTSP_CODEC_PCMA,
	RTSP_CODEC_L16,
	RTSP_CODEC_AMR,
	RTSP_CODEC_G726_32,
	RTSP_CODEC_G722,
	RTSP_CODEC_G729,

	/* video */
	RTSP_CODEC_H263,
	RTSP_CODEC_H263_1998,
	RTSP_CODEC_H264,
	RTSP_CODEC_MP4VES
};

/*
 * Event identifiers for RtspEvent.
 */
enum RtspEventType
{
	RTSP_EVT_NONE,
	RTSP_EVT_ERROR,
	RTSP_EVT_OPEN_OK,
	RTSP_EVT_OPEN_FAILED,
	RTSP_EVT_PLAY_OK,
	RTSP_EVT_PLAY_FAILED,
	RTSP_EVT_PLAY_COMPLETED,
	RTSP_EVT_STOP_OK,
	RTSP_EVT_STOP_FAILED
};

/*!
 * Container used to pass event data such as completion events to the
 * application's callback handler.
 */
struct RtspEvent
{
	RtspEvent()
		: type (RTSP_EVT_NONE),
		  userdata (0),
		  session_id (0),
		  audio_codec (RTSP_CODEC_NONE),
		  video_codec (RTSP_CODEC_NONE),
		  audio_rtsp_port (0),
		  video_rtsp_port (0),
		  audio_rtsp_addr (0),
		  video_rtsp_addr (0),
		  height(0),
		  width (0),
		  fps (0),
		  profile_level_id (0)
	{
		memset(sprop_parameter_sets, sizeof(sprop_parameter_sets), 0);
	}


	RtspEventType type;

	void*         userdata;          /*!< app data passed into the request */
	int           session_id;        /*!< uniquely identifies the media session */

	RtspCodec     audio_codec;
	unsigned int  audio_payload_format;

	RtspCodec     video_codec;
	unsigned int  video_payload_format;

	unsigned int  audio_rtsp_port;
	unsigned int  audio_rtsp_addr; 
	unsigned int  video_rtsp_port;
    unsigned int  video_rtsp_addr;

	unsigned int  height;            /*!< height of image in pixels */
	unsigned int  width;             /*!< width of image in pixels */
	unsigned int  fps;               /*!< framerate */
	unsigned int  profile_level_id;  /*!< MP4V-ES and H264 */
	char          sprop_parameter_sets[100]; /*!< Base64 string */

};

/*!
 *  Overload << for printing RtspEvent objects.
 */
inline std::ostream& operator << ( std::ostream &os, const RtspEvent& event )
{
	os << "event: ";
	if ( event.type == RTSP_EVT_ERROR )
	{
		os << "ERROR";
	}
	else if ( event.type == RTSP_EVT_OPEN_OK )
	{
		os << "OPEN_OK";
	}
	else if ( event.type == RTSP_EVT_OPEN_FAILED )
	{
		os << "OPEN_FAILED";
	}
	else if ( event.type == RTSP_EVT_PLAY_OK )
	{
		os << "PLAY_OK";
	}
	else if ( event.type == RTSP_EVT_PLAY_FAILED )
	{
		os << "PLAY_FAILED";
	}
	else if ( event.type == RTSP_EVT_PLAY_COMPLETED )
	{
		os << "PLAY_COMPLETED";
	}
	else if ( event.type == RTSP_EVT_STOP_OK )
	{
		os << "STOP_OK";
	}
	else if ( event.type == RTSP_EVT_STOP_FAILED )
	{
		os << "STOP_FAILED";
	}
	os << std::endl;

	os << "session_id: " << event.session_id << std::endl;

	if ( event.audio_codec == RTSP_CODEC_PCMU )
	{
		os << "audio_codec: " << "PCMU" << std::endl;
	}
	else if ( event.audio_codec == RTSP_CODEC_PCMA )
	{
		os << "audio_codec: " << "PCMA" << std::endl;
	}
	else if ( event.audio_codec == RTSP_CODEC_L16 )
	{
		os << "audio_codec: " << "L16" << std::endl;
	}
	else if ( event.audio_codec == RTSP_CODEC_AMR )
	{
		os << "audio_codec: " << "AMR" << std::endl;
	}
	else if ( event.audio_codec == RTSP_CODEC_G726_32 )
	{
		os << "audio_codec: " << "G726-32" << std::endl;
	}
	else if ( event.audio_codec == RTSP_CODEC_G722 )
	{
		os << "audio_codec: " << "G722" << std::endl;
	}
	else if ( event.audio_codec == RTSP_CODEC_G729 )
	{
		os << "audio_codec: " << "G729" << std::endl;
	}

	if ( event.video_codec == RTSP_CODEC_H263 )
	{
		os << "video_codec: " << "H263" << std::endl;
	}
	else if ( event.video_codec == RTSP_CODEC_H264 )
	{
		os << "video_codec: " << "H264" << std::endl;
	}
	else if ( event.video_codec == RTSP_CODEC_MP4VES )
	{
		os << "video_codec: " << "MP4V-ES" << std::endl;
	}
	if ( event.video_codec != RTSP_CODEC_NONE )
	{
		os << "height: " << event.height << std::endl;
		os << "width: " << event.width << std::endl;
		os << "fps: " << event.fps << std::endl;
		os << "profile_level_id: " << std::hex << event.profile_level_id << std::dec<< std::endl;
	}
	return os;
}


/*
 * Commands for use with RtspRequest objects.
 */
enum RtspCommand
{
	RTSP_CMD_OPEN,
	RTSP_CMD_PLAY,
	RTSP_CMD_STOP,
	RTSP_CMD_PAUSE,
};

/*!
 * Container for request parameters.
 * Used internally only.
 */
struct RtspRequest
{
	RtspRequest( RtspCommand command )
		: cmd (command),
		  userdata (0),
		  session_id (0),
		  audio_port (0),
		  video_port (0)
	{ ; }

	RtspCommand   cmd;
	void*         userdata;

	int           session_id;

	std::string   url;
	std::string   username;
	std::string   password;
	int           audio_port;
	int           video_port;
};


/*!
 * \class RtspConnector
 * Application interface and work thread..
 */
class RtspConnector : public WorkQueueThread<RtspRequest>
{
public:

	/*!
	 * This defines the signature of the callback function that will receive
	 * response events when a request has been completed.
	 * The contents of the response structure are valid until the callback
	 * function returns; the application must make copies of any parts of the
	 * response structure that it requires access to after this time.
	 */
	typedef void (*callback_t)(RtspEvent*);

	/*!
	 * ctor
	 * \param app_name - application identifier passed in User-Agent header.
	 * \param callback - application's event handler callback function.
	 */
	RtspConnector( const std::string& app_name,
	               RtspConnector::callback_t callback );

	/*!
	 * dtor
	 */
	~RtspConnector();

	/*!
	 * Open a media session.
	 * \param url - media
	 * \param username - user name for the rstp server.
	 * \param password - user password for the rstp server.
	 * \param audio_port - local rtp port for audio.
	 * \param video_port - local rtp port for video.
	 * \param userdata - opaque application data, returned in responses.
	 */
	void open( const std::string& url,
	           const std::string& username,
	           const std::string& password,
	           int audio_port,
	           int video_port,
	           void* userdata );

	/*!
	 * Initiate a media playback.
	 * \param session_id - identifies the session .
	 */
	void play( int session_id );

	/*
	 * Cancel an active session.
	 * \param session_id - identifies the session to terminate .
	 */
	void stop( int session_id );


	const std::string Name() {return "RTSPConn";}

private:

	struct Session
	{
		Session( RtspConnector& rtsp_connector, int session_id )
		 : owner (rtsp_connector),
		   id (session_id)
		{ ; }

		RtspConnector&   owner;
		int              id;

		void*            userdata;

		RTSPClient*      client;
		MediaSession*    mediaSession;
		MediaSubsession* audioSubsession;
		MediaSubsession* videoSubsession;

		std::string url;
		std::string username;
		std::string password;
		int         clientAudioPort;
		int         clientVideoPort;
		std::string options;

	private:

		Session();
	};

	int generateSessionId();

	RtspConnector::Session* createSession( const std::string& url,
	                                       const std::string& username,
	                                       const std::string& password,
	                                       int audio_port,
	                                       int video_port );

	void destroySession( RtspConnector::Session* session );

	RtspConnector::Session* findSession( int session_id ) const;

	bool sendOptions( RTSPClient* client,
	                  const std::string& url,
	                  const std::string& username,
	                  const std::string& password,
	                  int timeout_seconds,
	                  std::string& options );

	void sendGetParameter( RtspConnector::Session* session );

	bool playSession( RtspConnector::Session* session );

	bool pauseSession( RtspConnector::Session* session );

	/*
	 * Worker thread's entry point.
	 */
	virtual int run();

	/*
	 * TaskScheduler callback used to process requests from the work queue.
	 * \param client_data - pointer to the RtspConnector object.
	 */
	static void monitor( void* client_data );

	/*
	 * TaskScheduler callback used to keep sessions alive.
	 * \param client_data - pointer to the RtspConnector::Session object.
	 */
	static void onKeepAlive( void* client_data );

	/*
	 * TaskScheduler callback used to signal that a play operation has reached
	 * its advertised end time.
	 * \param client_data - pointer to an RtspConnector::Session object
	 */
	static void onEndPlay( void* client_data );

private:

	std::string       user_agent_;  /* User-Agent header value */
	callback_t        callback_;    /* application completion-event handler */

	UsageEnvironment* env_;
	TaskScheduler*    scheduler_;
	char              watch_;       /* set non-zero to cancel scheduler. */

	int session_id_;                /* counter - provides unique session ids */

	typedef std::map<int, Session*> session_map_t;
	session_map_t sessions_;

};

#endif // _RTSPCONNECTOR_H

/* vim:ts=4:set nu:
 * EOF
 */

