/**
* @file CSipSession.h
* @brief SIP Session class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CSipSession_h
#define CSipSession_h

#include <string>

#include "ipmlib.h"
#include "port_connect.h"

#include "config.h"
#include "AEventHandler.h"
#include "CSystemConfiguration.h"
#include "MediaCoders.h"

#include "COverlay.h"

class CGcSipDevice;
class CIpmDevice;
class CMediaDevice;
class CConferencePartyDevice;
class ASipSessionState;

static int IPM_RX_PORT_INFO = 0x0001;
static int IPM_TX_PORT_INFO = 0x0002;
static int CNF_RX_PORT_INFO = 0x0004;
static int CNF_TX_PORT_INFO = 0x0008;
static int MM_RX_PORT_INFO  = 0x00010;
static int MM_TX_PORT_INFO  = 0x00020;
static int IPM_CNF_PORT_INFO_COMPLETE = 0x000F;
static int IPM_MM_PORT_INFO_COMPLETE  = 0x0033;


static int IPM_TX_CONNECT = 0x0011;
static int CNF_TX_CONNECT = 0x0012;
static int MM_TX_CONNECT  = 0x0014;
static int IPM_CNF_PORT_CONNECT_COMPLETE = 0x0013;
static int IPM_MM_PORT_CONNECT_COMPLETE  = 0x0015;

static int IPM_TX_DISCONNECT = 0x0021;
static int CNF_TX_DISCONNECT = 0x0022;
static int MM_TX_DISCONNECT  = 0x0024;
static int IPM_CNF_PORT_DISCONNECT_COMPLETE = 0x0023;
static int IPM_MM_PORT_DISCONNECT_COMPLETE  = 0x0025;

static int IPM_DEV_CONNECT       = 0x0041;
static int MM_DEV_CONNECT        = 0x0042;
static int DEV_CONNECT_COMPLETE  = 0x0043;

static int IPM_DEV_DISCONNECT       = 0x0081;
static int MM_DEV_DISCONNECT        = 0x0082;
static int DEV_DISCONNECT_COMPLETE  = 0x0083;

//*****************************************************************************
// Class: ::CSipSession
// Description: SIP session call flow
//*****************************************************************************
class CSipSession : public AEventHandler
{
   friend class ASipSessionState;
   friend class CSipSessionIdleState;
   friend class ASipCallActiveState;
   friend class CSipSessionInboundCallSetupState;
   friend class CSipSessionInboundCallAcceptedState;
   friend class CSipSessionInboundCallAnswerState;
   friend class CSipSessionStartingMediaState;
   friend class CSipSessionMediaStreamingState;
   friend class CSipSessionConferenceSetupState;
   friend class CSipSessionGreetingSetupState;
   friend class CSipSessionPlayingGreetingState;
   friend class CSipSessionGreetingTeardownState;
   friend class CSipSessionPlayingInvalidPinState;
   friend class CSipSessionJoiningConferenceState;
   friend class CSipSessionInConferenceState;
   friend class CSipSessionLeavingConferenceState;
   friend class CSipSessionStoppingMediaState;
   friend class CSipSessionCallTeardownState;
   friend class CSipSessionAddingCallerIdOverlayState;
   friend class CSipSessionRemovingCallerIdOverlayState;
   friend class CSipSessionRefreshingCallerIdOverlayState;
public:
   CSipSession(const std::string& a_sessionName, CGcSipDevice* a_sipDevice, CIpmDevice* a_ipmDevice, CMediaDevice* a_mmDevice, CSystemConfiguration& a_systemConfiguration);
   virtual ~CSipSession();
   int Stop() {return 0;}

   void ProcessEvent(METAEVENT& a_metaEvent);

   ASipSessionState& SessionState() {return *m_sessionState;}
   void SetSessionStateTo(ASipSessionState* state);

   const std::string& SessionName() {return m_sessionName;}

   int OnInboundCallOffered();
   int InboundCallSendProceeding();
   int OnInboundCallSendProceedingComplete();
   int ReserveResource();
   int OnReserveResourceComplete();
   int GetLocalMediaInfo();
   int OnGetLocalMediaInfoComplete();
   int AcceptInboundCall();
   int OnAcceptInboundCallComplete();
   int AnswerInboundCall();
   int OnAnswerInboundCallComplete();
   int StartMedia();
   int OnStartMediaComplete();
   int OpenConferenceParty();
   int OnOpenConferencePartyComplete();
   int ConnectToConferenceParty();
   int OnConnectToConferencePartyComplete();
   int JoinConference();
   int OnJoinConferenceComplete();
   int LeaveConference();
   int OnLeaveConferenceComplete();
   int DisconnectFromConferenceParty();
   int OnDisconnectFromConferencePartyComplete();
   int CloseConferenceParty();
   int OnCloseConferencePartyComplete();
   int StopMedia();
   int OnStopMediaComplete();
   int OnCallDisconnected();
   int ReleaseResource();
   int OnReleaseResourceComplete();
   int ReleaseCall();
   int OnCallReleased();

   int Device();

   int ConnectToMediaPlayer();
   int OnConnectToMediaPlayerComplete();
   int DisconnectFromMediaPlayer();
   int OnDisconnectFromMediaPlayerComplete();
	
   bool isMediaStarted(){return m_bMediaStarted;}
   bool isMediaPlayerConnected(){return m_bMediaPlayerConnected;}
   bool isConfPartyConnected(){return m_bConfPartyConnected;}

   int PlayGreeting();
   int OnPlayComplete();
   int OnRecordComplete();

   int OnGetIpmRxPortInfoComplete();
   int OnGetIpmTxPortInfoComplete();
   int OnGetCnfRxPortInfoComplete();
   int OnGetCnfTxPortInfoComplete();
   int OnGetMmRxPortInfoComplete();
   int OnGetMmTxPortInfoComplete();

   int PortInfoSubState() {return m_portInfoSubState;}
   // SKS needed for multiple conference 
   void SetConferenceID(std::string cnfID) {m_cnfID=cnfID;}
   std::string GetConferenceID() {return m_cnfID;}
   bool isPinAccepted(){return m_bPinAccepted;}
   int GetConfPINLen();
   //JM
   bool VerifyConfPIN();
   int collectDigit(std::string digit);
   void clearDigits();
   int GetDigitCnt(){return m_nDigitCnt;}

   int OnCnfPortConnected();
   int OnIpmPortConnected();
   int OnMmPortConnected();

   int PortConnectSubState() {return m_portConnectSubState;}

   int OnCnfPortDisconnected();
   int OnIpmPortDisconnected();
   int OnMmPortDisconnected();

   int PortDisconnectSubState() {return m_portDisconnectSubState;}

   int ListenTo(CSipSession* a_sipSession);
   CIpmDevice* IpmDevice() {return m_ipmDevice;}

   int OnDtmf(int a_dtmf);

   int OnIFrameRequest();

   bool HaveGreeting();

   int AddCallerIdOverlay();
   int AddConferenceLayoutOverlay(int a_layout);
   int RemoveCallerIdOverlay();
   int RemoveConferenceLayoutOverlay();
   int OnAddImageOverlay(METAEVENT& a_metaEvent);
   int OnRemoveImageOverlay(METAEVENT& a_metaEvent);

   void SetCallerId(int a_callerId) { m_callerId=a_callerId;}
   void SetDefaultVideoBandwidth(int a_vidbandwidth) {m_defaultBandwidth = a_vidbandwidth;}

   void StopSession();

private:
   // force initialization
   CSipSession();

   // make non-copyable
   CSipSession(const CSipSession&);
   const CSipSession& operator=(const CSipSession&);

   ASipSessionState* m_sessionState;
   std::string m_sessionName;

   CGcSipDevice* m_sipDevice;
   CIpmDevice* m_ipmDevice;
   CMediaDevice* m_mmDevice;

   CSystemConfiguration& m_systemConfiguration;

   CConferencePartyDevice* m_conferenceParty;

   // SKS required when multiple confernce 
   std::string m_cnfID;
   bool m_bPinAccepted;

   std::string m_digitBuf;
   int m_nDigitCnt;

   bool m_bMediaStarted;
   bool m_bMediaPlayerConnected;
   bool m_bConfPartyConnected;
   
   int m_sessionBandwidth;
   int m_audioBandwidth;
   int m_videoBandwidth;
   int m_defaultBandwidth;

   AAudioCoder* m_matchedAudioCoder;
   AAudioCoder* m_matchedRfc2833Coder;
   AVideoCoder* m_matchedVideoCoder;

   COverlay* m_callerIdOverlay;
   COverlay* m_layoutDescriptionOverlay;
   int m_callerId;

   int m_devConnectSubState;
   int m_devDisconnectSubState;
   int m_portInfoSubState;
   int m_portConnectSubState;
   int m_portDisconnectSubState;

   DM_PORT_CONNECT_INFO_LIST m_portConnectInfoListIpmTxCnfRx;
   DM_PORT_CONNECT_INFO_LIST m_portConnectInfoListCnfTxIpmRx;

   int PrimOnInboundCallOffered();
   int PrimInboundCallSendProceeding();
   int PrimOnInboundCallSendProceedingComplete();
   int PrimReserveResource();
   int PrimOnReserveResourceComplete();
   int PrimGetLocalMediaInfo();
   int PrimOnGetLocalMediaInfoComplete();
   int PrimAcceptInboundCall();
   int PrimOnAcceptInboundCallComplete();
   int PrimAnswerInboundCall();
   int PrimOnAnswerInboundCallComplete();
   int PrimStartMedia();
   int PrimOnStartMediaComplete();
   int PrimOpenConferenceParty();
   int PrimOnOpenConferencePartyComplete();
   int PrimConnectToConferenceParty();
   int PrimOnConnectToConferencePartyComplete();
   int PrimJoinConference();
   int PrimOnJoinConferenceComplete();
   int PrimLeaveConference();
   int PrimOnLeaveConferenceComplete();
   int PrimDisconnectFromConferenceParty();
   int PrimOnDisconnectFromConferencePartyComplete();
   int PrimCloseConferenceParty();
   int PrimOnCloseConferencePartyComplete();
   int PrimStopMedia();
   int PrimOnStopMediaComplete();
   int PrimOnCallDisconnected();
   int PrimReleaseResource();
   int PrimOnReleaseResourceComplete();
   int PrimReleaseCall();
   int PrimOnCallReleased();

   int PrimOnGetIpmRxPortInfoComplete();
   int PrimOnGetIpmTxPortInfoComplete();
   int PrimOnGetCnfRxPortInfoComplete();
   int PrimOnGetCnfTxPortInfoComplete();
   int PrimOnGetMmRxPortInfoComplete();
   int PrimOnGetMmTxPortInfoComplete();
   int PrimPortConnect();
   int PrimIpmMmPortConnect();

   int PrimOnCnfPortConnected();
   int PrimOnIpmPortConnected();
   int PrimOnMmPortConnected();

   int PrimOnCnfPortDisconnected();
   int PrimOnIpmPortDisconnected();
   int PrimOnMmPortDisconnected();


   int CreatePortConnectInfoList(  DM_PORT_CONNECT_INFO_LIST& a_portConnectionInfoList,
                                   const DM_PORT_INFO_LIST& a_portInfoList1,
                                   const DM_PORT_INFO_LIST& a_portInfoList2);

   int PrimConnectToMediaPlayer();
   int PrimOnConnectToMediaPlayerComplete();
   int PrimDisconnectFromMediaPlayer();
   int PrimOnDisconnectFromMediaPlayerComplete();

   int PrimPlayInvalidPinClip();
   int PrimPlayGreeting();
   int PrimOnPlayComplete();

   int PrimStopPlay(int a_dtmf);
   int PrimCycleConferenceLayout();
   int PrimRotateConferenceParties();

   int PrimOnIFrameRequest();

   int PrimAddCallerIdOverlay();
   int PrimAddConferenceLayoutOverlay(int a_layout);
   int PrimRemoveCallerIdOverlay();
   int PrimRemoveConferenceLayoutOverlay();
   int PrimOnAddCallerIdOverlay(METAEVENT& a_metaEvent);
   int PrimOnRemoveCallerIdOverlay(METAEVENT& a_metaEvent);
   int PrimOnAddConfernceLayoutOverlay(METAEVENT& a_metaEvent);
   int PrimOnRemoveConferenceLayoutOverlay(METAEVENT& a_metaEvent);

   int GetHostInformation();

   const char* StaticPayloadName(unsigned int a_payloadType);
   int BuildRemoteCapList(sdpSessionDescription& a_remoteSdp);
   int ValidateMediaConnection(sdpMediaDescription& a_sdpRemoteMediaDescription,
                               sdpConnection& a_sdpRemoteMediaConnection,
                               eIPM_DATA_DIRECTION& a_localMediaDirection);

   int ImportRtpmap(std::string& a_strAttribute, unsigned int a_payloadType);
   int ImportSdpFmtp(const std::string& a_attribute);
   int CreateRemoteMediaCoder(std::string& a_payloadCoderName, unsigned int a_payloadType);

   int BuildLocalCapList();
   int CreateLocalMediaCoder(IPM_AUDIO_CODER_INFO& l_ipmAudioCoderInfo);
   int CreateLocalMediaCoder(IPM_VIDEO_CODER_INFO& l_ipmVideoCoderInfo);

   int InitializeAnswerSdp(sdpSessionDescription& a_sdp);
   int BuildAnswerSDP(sdpSessionDescription& a_remoteSdp, sdpSessionDescription& a_localSdp);

   const static unsigned int  DYNAMIC_PAYLOAD_MIN     = 96;
   const static unsigned int  DYNAMIC_PAYLOAD_MAX     = 128;
   const static unsigned int  IP_NAME                 = 1024;

   char m_hostName[IP_NAME];
   char m_hostIpAddress[IP_NAME];

   std::vector<AMediaCoder*> m_remoteMediaCoders;
   std::vector<AMediaCoder*> m_localMediaCoders;
   int m_remotePayloads[DYNAMIC_PAYLOAD_MAX]; 

   TestOverlay m_callerIdOverlayInfo;
   TestOverlay m_layoutDescriptionOverlayInfo;

/*
   int DisplayCallerIdOverlay();
   int OnAddImageOverlay(METAEVENT& a_metaEvent);

   int RemoveCallerIdOverlay();
   int OnRemoveImageOverlay(METAEVENT& a_metaEvent);

   int DisplayConferenceLayout(int a_layout);
*/
};

#endif

