/**
* @file CSipSessionState.cpp
* @brief SIP Session States - class implementations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "config.h"

#include "LogUtil.h"
#include "CSipSessionState.h"
#include "CIpmDevice.h"
#include <string>
#include <iostream>
#include <map>

CSipSessionStates ASipSessionState::m_sipSessionStates;

//*****************************************************************************
// Function: CSipSessionStates::CSipSessionStates()
// Description: Map of SIP session state objects
// Return:  
// Parameters: none 
//*****************************************************************************
CSipSessionStates::CSipSessionStates()
{
   m_sipSessionStates[SipSessionIdleState] = new CSipSessionIdleState();
   m_sipSessionStates[SipSessionInboundCallSetupState] = new CSipSessionInboundCallSetupState();
   m_sipSessionStates[SipSessionInboundCallAcceptedState] = new CSipSessionInboundCallAcceptedState();
   m_sipSessionStates[SipSessionInboundCallAnswerState] = new CSipSessionInboundCallAnswerState();
   m_sipSessionStates[SipSessionStartingMediaState] = new CSipSessionStartingMediaState();
   m_sipSessionStates[SipSessionMediaStreamingState] = new CSipSessionMediaStreamingState();
   m_sipSessionStates[SipSessionConferenceSetupState] = new CSipSessionConferenceSetupState();
   m_sipSessionStates[SipSessionGreetingSetupState] = new CSipSessionGreetingSetupState();
   m_sipSessionStates[SipSessionPlayingGreetingState] = new CSipSessionPlayingGreetingState();
   m_sipSessionStates[SipSessionGreetingTeardownState] = new CSipSessionGreetingTeardownState();
   m_sipSessionStates[SipSessionPlayingInvalidPinState] = new CSipSessionPlayingInvalidPinState();
   m_sipSessionStates[SipSessionJoiningConferenceState] = new CSipSessionJoiningConferenceState();
   m_sipSessionStates[SipSessionInConferenceState] = new CSipSessionInConferenceState();
   m_sipSessionStates[SipSessionLeavingConferenceState] = new CSipSessionLeavingConferenceState();
   m_sipSessionStates[SipSessionStoppingMediaState] = new CSipSessionStoppingMediaState();
   m_sipSessionStates[SipSessionCallTeardownState] = new CSipSessionCallTeardownState();
   m_sipSessionStates[SipSessionAddingCallerIdOverlayState] = new CSipSessionAddingCallerIdOverlayState();
   m_sipSessionStates[SipSessionRemovingCallerIdOverlayState] = new CSipSessionRemovingCallerIdOverlayState();
   m_sipSessionStates[SipSessionRefreshingCallerIdOverlayState] = new CSipSessionRefreshingCallerIdOverlayState();
}

//*****************************************************************************
// Function: int ASipSessionState::LogStateError(CSipSession &a_sipSession, const std::string &msg)
// Description: Log an invalid state for operation/event message
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//             const std::string &msg 
//*****************************************************************************
int ASipSessionState::LogStateError(CSipSession& a_sipSession, const std::string& msg)
{
   LogMsg(a_sipSession.SessionName(), AsString(Desc()) + " is invalid device state for " + msg);
   std::cout << "WARNING:" << a_sipSession.SessionName()
   << " " << Desc() << " is invalid device state for "
   << msg
   << std::endl;
   return -1;
}

//*****************************************************************************
// Function: int CSipSessionIdleState::OnInboundCallOffered(CSipSession &a_sipSession)
// Description: Handle an inbound call offered event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionIdleState::OnInboundCallOffered( CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnInboundCallOffered();
   if (res == 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionInboundCallSetupState));
      a_sipSession.clearDigits();
      a_sipSession.InboundCallSendProceeding();
   }
   else
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
      a_sipSession.ReleaseCall();      
   }
   return res;
}
//*****************************************************************************
// Function: int CSipSessionIdleState::OnPlayComplete(CSipSession &a_sipSession)
// Description: Handle a play complete event
// Return: int - 0 for success, otherwise error
// Parameters: CSipSession &a_sipSession
//*****************************************************************************
int CSipSessionIdleState::OnPlayComplete(CSipSession &a_sipSession)
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionIdleState::OnRecordComplete(CSipSession &a_sipSession)
// Description: Handle a play complete event
// Return: int - 0 for success, otherwise error
// Parameters: CSipSession &a_sipSession
//*****************************************************************************
int CSipSessionIdleState::OnRecordComplete(CSipSession &a_sipSession)
{
   return 0;
}



//*****************************************************************************
// Function: int ASipCallActiveState::OnCallDisconnected(CSipSession &a_sipSession)
// Description: Handle a call disconnected event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int ASipCallActiveState::OnCallDisconnected(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnCallDisconnected();
   if (res == 0)
   {
      LogMsg(a_sipSession.SessionName(), "ASipCallActiveState::OnCallDisconnected() Calling LeaveConference()");	
      a_sipSession.LeaveConference();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionInboundCallSetupState::InboundCallSendProceeding(CSipSession &a_sipSession)
// Description: Send a proceeding message for an inbound call
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallSetupState::InboundCallSendProceeding(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimInboundCallSendProceeding();
   if (res != 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
      a_sipSession.ReleaseCall();      
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionInboundCallSetupState::OnInboundCallSendProceedingComplete(CSipSession &a_sipSession)
// Description: Handle an inbound call send proceeding completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallSetupState::OnInboundCallSendProceedingComplete(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnInboundCallSendProceedingComplete();
   if (res == 0)
   {
      a_sipSession.ReserveResource();
   }
   else
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
      a_sipSession.ReleaseCall();      
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionInboundCallSetupState::ReserveResource(CSipSession &a_sipSession)
// Description: Reserve codec resources
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallSetupState::ReserveResource(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimReserveResource();
   if (res != 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
      a_sipSession.ReleaseCall();      
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionInboundCallSetupState::OnReserveResourceComplete(CSipSession &a_sipSession)
// Description: Handle a reserve codec resources completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallSetupState::OnReserveResourceComplete(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnReserveResourceComplete();
   if (res == 0)
   {
      a_sipSession.GetLocalMediaInfo();
   }
   else
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
      a_sipSession.ReleaseResource();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionInboundCallSetupState::GetLocalMediaInfo(CSipSession &a_sipSession)
// Description: Get the local media info
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallSetupState::GetLocalMediaInfo(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimGetLocalMediaInfo();
   if (res != 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
      a_sipSession.ReleaseResource();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionInboundCallSetupState::OnGetLocalMediaInfoComplete(CSipSession &a_sipSession)
// Description: Handle a get local media info complete event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallSetupState::OnGetLocalMediaInfoComplete(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnGetLocalMediaInfoComplete();
   if (res == 0)
   {
      a_sipSession.AcceptInboundCall();
   }
   else
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
      a_sipSession.ReleaseResource();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionInboundCallSetupState::AcceptInboundCall(CSipSession &a_sipSession)
// Description: Accept an inbound call
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallSetupState::AcceptInboundCall(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimAcceptInboundCall();
   if (res != 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
      a_sipSession.ReleaseResource();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionInboundCallSetupState::OnAcceptInboundCallComplete(CSipSession &a_sipSession)
// Description: Handle an accept inbound call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallSetupState::OnAcceptInboundCallComplete(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnAcceptInboundCallComplete();
   if (res == 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionInboundCallAcceptedState));
      if (a_sipSession.HaveGreeting())
      {
         return a_sipSession.ConnectToMediaPlayer();
      }
      else
      {
         return a_sipSession.OpenConferenceParty();
      }
   }
   else
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
      a_sipSession.ReleaseResource();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionInboundCallAcceptedState::ConnectToMediaPlayer(CSipSession &a_sipSession)
// Description: Connect to the media player
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallAcceptedState::ConnectToMediaPlayer(CSipSession& a_sipSession)
{
   a_sipSession.SetSessionStateTo(SessionState(SipSessionGreetingSetupState));
   return a_sipSession.PrimConnectToMediaPlayer();
}

//*****************************************************************************
// Function: int CSipSessionInboundCallAccepted::OpenConferenceParty(CSipSession &a_sipSession)
// Description: Open a conference party
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallAcceptedState::OpenConferenceParty(CSipSession &a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OpenConferenceParty");
   a_sipSession.SetSessionStateTo(SessionState(SipSessionConferenceSetupState));
   return a_sipSession.PrimOpenConferenceParty();
}

//*****************************************************************************
// Function: int CSipSessionInboundCallAcceptedState::OnCallDisconnected(CSipSession &a_sipSession)
// Description: Handle a LeaveConference Request
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallAcceptedState::OnCallDisconnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "CSipSessionInboundCallAcceptedState::OnCallDisconnected()");	
   //JM - disconnect as call being established
   a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
   a_sipSession.ReleaseResource();
   return 0;
}


//*****************************************************************************
// Function: int CSipSessionStartingMediaState::OnStopMediaComplete(CSipSession &a_sipSession)
// Description: Handle an OnStopMediaComplete
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionStartingMediaState::OnStopMediaComplete(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "CSipSessionStartingMediaState::OnStopMediaComplete()");	
   //JM - Someone called stop before channel started - maybe ipm_Start failed?
   a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
   a_sipSession.ReleaseResource();
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionStartingMediaState::OnStartMediaComplete(CSipSession &a_sipSession)
// Description: Handle a start media streaming completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionStartingMediaState::OnStartMediaComplete(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnStartMediaComplete();
   if (res == 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionInboundCallAnswerState));
      return a_sipSession.AnswerInboundCall();
   }
   else
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
      a_sipSession.ReleaseResource();  
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionStartingMediaState::OnCallDisconnected(CSipSession &a_sipSession)
// Description: Handle a OnCalldisconnected Request
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionStartingMediaState::OnCallDisconnected(CSipSession& a_sipSession)
{
   int res = 0;
   LogMsg(a_sipSession.SessionName(), "CSipSessionStartingMediaState::OnCallDisconnected()");	
   //JM - Party connected but media not started - maybe got disconnected event before ipm_Start completed?
   a_sipSession.SetSessionStateTo(SessionState(SipSessionLeavingConferenceState));
   res = a_sipSession.PrimLeaveConference();
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionInboundCallAnswerState::AnswerInboundCall(CSipSession &a_sipSession)
// Description: Answer an inbound call
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallAnswerState::AnswerInboundCall(CSipSession& a_sipSession)
{
   int res =a_sipSession.PrimAnswerInboundCall();
   if (res != 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
      a_sipSession.ReleaseResource();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionInboundCallAnswerState::OnAnswerInboundCallComplete(CSipSession &a_sipSession)
// Description: Handle an answer inbound call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallAnswerState::OnAnswerInboundCallComplete(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnAnswerInboundCallComplete();
   if (res == 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionMediaStreamingState));
      if (a_sipSession.HaveGreeting())
      {
         return a_sipSession.PlayGreeting();
      }
      else
      {
         return a_sipSession.JoinConference();
      }
   }
   else
   {
      //JM - stop media in media streaming state to initiate call teardown
      a_sipSession.SetSessionStateTo(SessionState(SipSessionMediaStreamingState));
      a_sipSession.StopMedia();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionInboundCallAnswerState::OnCallDisconnected(CSipSession &a_sipSession)
// Description: Handle a OnCallDisconnected Request
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInboundCallAnswerState::OnCallDisconnected(CSipSession& a_sipSession)
{
   int res = 0;
   LogMsg(a_sipSession.SessionName(), "CSipSessionInboundCallAnswerState::OnCallDisconnected()");	
   //JM - disconnect as call being established - 
   a_sipSession.SetSessionStateTo(SessionState(SipSessionLeavingConferenceState));
   res = a_sipSession.PrimLeaveConference();
   return 0;
}


//*****************************************************************************
// Function: int CSipSessionMediaStreamingState::PlayGreeting(CSipSession &a_sipSession)
// Description: Play the greeting clip
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionMediaStreamingState::PlayGreeting(CSipSession &a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "PlayGreeting");
   a_sipSession.SetSessionStateTo(SessionState(SipSessionPlayingGreetingState));
   
   // Start digit detection for PIN code
   if (a_sipSession.m_systemConfiguration.isPinEnabled() == true) {
      //(a_sipSession.IpmDevice())->GetDigits(a_sipSession.GetConfPINLen());
      (a_sipSession.IpmDevice())->GetDigits(1);
   } else {
      (a_sipSession.IpmDevice())->GetDigits(1);
   }

   return a_sipSession.PrimPlayGreeting();
}

//*****************************************************************************
// Function: int CSipSessionMediaStreamingState::JoinConference(CSipSession &a_sipSession)
// Description: Join the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionMediaStreamingState::JoinConference(CSipSession &a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "JoinConference");
   a_sipSession.SetSessionStateTo(SessionState(SipSessionJoiningConferenceState));
   return a_sipSession.PrimJoinConference();
}


//*****************************************************************************
// Function: int CSipSessionMediaStreamingState::OnIFrameRequest(CSipSession &a_sipSession)
// Description: Send an IFrame Update
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionMediaStreamingState::OnIFrameRequest(CSipSession& a_sipSession)
{
   return a_sipSession.PrimOnIFrameRequest();
}

//*****************************************************************************
// Function: int CSipSessionMediaStreamingState::StopMedia(CSipSession &a_sipSession)
// Description: Stop the media streaming
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionMediaStreamingState::StopMedia(CSipSession& a_sipSession)
{
   a_sipSession.SetSessionStateTo(SessionState(SipSessionStoppingMediaState));
   return a_sipSession.PrimStopMedia();
}

//*****************************************************************************
// Function: int CSipSessionMediaStreamingState::OnCallDisconnected(CSipSession &a_sipSession)
// Description: Handle a OnCallDisconnected Request
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionMediaStreamingState::OnCallDisconnected(CSipSession& a_sipSession)
{
   int res = 0;
   LogMsg(a_sipSession.SessionName(), "CSipSessionMediaStreamingState::OnCallDisconnected()");	
   //JM - disconnect as call being established
   a_sipSession.SetSessionStateTo(SessionState(SipSessionLeavingConferenceState));
   res = a_sipSession.PrimLeaveConference();
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionGreetingSetupState::OnGetIpmRxPortInfoComplete(CSipSession &a_sipSession)
// Description: Handle a get RX port info complete event from the IPM device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingSetupState::OnGetIpmRxPortInfoComplete(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnGetIpmRxPortInfoComplete");
   if (a_sipSession.PrimOnGetIpmRxPortInfoComplete() == IPM_MM_PORT_INFO_COMPLETE)
   {
      return a_sipSession.PrimIpmMmPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionGreetingSetupState::OnGetIpmTxPortInfoComplete(CSipSession &a_sipSession)
// Description: Handle a get TX port info complete event from the IPM device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingSetupState::OnGetIpmTxPortInfoComplete(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnGetIpmTxPortInfoComplete");
   if (a_sipSession.PrimOnGetIpmTxPortInfoComplete() == IPM_MM_PORT_INFO_COMPLETE)
   {
      return a_sipSession.PrimIpmMmPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionGreetingSetupState::OnGetMmRxPortInfoComplete(CSipSession &a_sipSession)
// Description: Handle a get RX port info complete event from the MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingSetupState::OnGetMmRxPortInfoComplete(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnGetMmRxPortInfoComplete");
   if (a_sipSession.PrimOnGetMmRxPortInfoComplete() == IPM_MM_PORT_INFO_COMPLETE)
   {
      return a_sipSession.PrimIpmMmPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionGreetingSetupState::OnGetMmTxPortInfoComplete(CSipSession &a_sipSession)
// Description: Handle a get TX port info complete event from the MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingSetupState::OnGetMmTxPortInfoComplete(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnGetMmTxPortInfoComplete");
   if (a_sipSession.PrimOnGetMmTxPortInfoComplete() == IPM_MM_PORT_INFO_COMPLETE)
   {
      return a_sipSession.PrimIpmMmPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionGreetingSetupState::OnIpmPortConnected(CSipSession &a_sipSession)
// Description: Handle a port connected event from the IPM device
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingSetupState::OnIpmPortConnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnIpmPortConnected");
   if (a_sipSession.PrimOnIpmPortConnected() == IPM_MM_PORT_CONNECT_COMPLETE)
   {
      if (a_sipSession.HaveGreeting())
      {
         return a_sipSession.OnConnectToMediaPlayerComplete();
      }
      else
      {
         return a_sipSession.OpenConferenceParty();
      }
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionGreetingSetupState::OnMmPortConnected(CSipSession &a_sipSession)
// Description: Handle a port connected event from the MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingSetupState::OnMmPortConnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnMmPortConnected");
   if (a_sipSession.PrimOnMmPortConnected() == IPM_MM_PORT_CONNECT_COMPLETE)
   {
      return a_sipSession.OnConnectToMediaPlayerComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionGreetingSetupState::OnConnectToMediaPlayerComplete(CSipSession &a_sipSession)
// Description: Handle a connect to the media player complete event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingSetupState::OnConnectToMediaPlayerComplete(CSipSession &a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnConnectToMediaPlayerComplete");

   int res = a_sipSession.PrimOnConnectToMediaPlayerComplete();
   if (res == 0)
   {
      return a_sipSession.StartMedia();
   }

   return res;
}

//*****************************************************************************
// Function: int CSipSessionGreetingSetupState::StartMedia(CSipSession &a_sipSession)
// Description: Start media streaming
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingSetupState::StartMedia(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "StartMedia");
   a_sipSession.SetSessionStateTo(SessionState(SipSessionStartingMediaState));
   return a_sipSession.PrimStartMedia();
}

//*****************************************************************************
// Function: int CSipSessionGreetingSetupState::OnCallDisconnected(CSipSession &a_sipSession)
// Description: Handle a OnCallDisconnected Request
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingSetupState::OnCallDisconnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnCallDisconnected()");	
   a_sipSession.SetSessionStateTo(SessionState(SipSessionStoppingMediaState));
   a_sipSession.DisconnectFromMediaPlayer();
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionPlayingGreetingState::OnPlayComplete(CSipSession &a_sipSession)
// Description: Handle a play complete event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionPlayingGreetingState::OnPlayComplete(CSipSession &a_sipSession)
{
// SKS Play greeting again if PIN is enabled and not entered else put into conference
   if( ((a_sipSession.m_systemConfiguration.isPinEnabled()==true) && (a_sipSession.isPinAccepted()==true)) || 
       (a_sipSession.m_systemConfiguration.isPinEnabled()!=true) ) {
      return a_sipSession.DisconnectFromMediaPlayer();
   } else {
      // SKS Play Invalid PIN message
      LogMsg(a_sipSession.SessionName(), "PlayInvalidPin");
      a_sipSession.SetSessionStateTo(SessionState(SipSessionPlayingInvalidPinState));
      return a_sipSession.PrimPlayInvalidPinClip();
   }
}

//*****************************************************************************
// Function: int CSipSessionPlayingGreetingState::OnDtmf(CSipSession &a_sipSession, int a_dtmf)
// Description: Handle a DTMF detected event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//             int a_dtmf 
//*****************************************************************************
int CSipSessionPlayingGreetingState::OnDtmf(CSipSession &a_sipSession, int a_dtmf)
{
  char tmpstr[16];
  sprintf(tmpstr, "%c", a_dtmf);
  string dtmfstr(AsString(tmpstr));
  LogMsg(a_sipSession.SessionName(), "PlayingGreeting::OnDtmf =" + dtmfstr);

   a_sipSession.collectDigit(dtmfstr);
   if (a_sipSession.m_systemConfiguration.isPinEnabled()==true) {
      // SKS check pin code here before stopping play
      LogMsg(a_sipSession.SessionName(),"DigitCnt=" + AsString(a_sipSession.GetDigitCnt())+ ", PINLen=" + AsString(a_sipSession.GetConfPINLen()));
      if(a_sipSession.GetDigitCnt() == a_sipSession.GetConfPINLen())
      {
         a_sipSession.VerifyConfPIN(); 
         a_sipSession.clearDigits();
         (a_sipSession.IpmDevice())->GetDigits(1);
         return a_sipSession.PrimStopPlay(a_dtmf);
      }
      else {
         (a_sipSession.IpmDevice())->GetDigits(1);
         return 0;
      }
   }
   a_sipSession.clearDigits();
   (a_sipSession.IpmDevice())->GetDigits(1);	
   return a_sipSession.PrimStopPlay(a_dtmf);
}

//*****************************************************************************
// Function: int CSipSessionPlayingGreetingState::OnIFrameRequest(CSipSession &a_sipSession)
// Description: Send an IFrame Update
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionPlayingGreetingState::OnIFrameRequest(CSipSession& a_sipSession)
{
   return a_sipSession.PrimOnIFrameRequest();
}

//*****************************************************************************
// Function: int CSipSessionPlayingGreetingState::DisconnectFromMediaPlayer(CSipSession &a_sipSession)
// Description: Disconnect from the media player
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionPlayingGreetingState::DisconnectFromMediaPlayer(CSipSession &a_sipSession)
{
   a_sipSession.SetSessionStateTo(SessionState(SipSessionGreetingTeardownState));
   return a_sipSession.PrimDisconnectFromMediaPlayer();
}

//*****************************************************************************
// Function: int CSipSessionPlayingGreetingState::OnCallDisconnected(CSipSession &a_sipSession)
// Description: Handle a call disconnected event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionPlayingGreetingState::OnCallDisconnected(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnCallDisconnected();
   if (res == 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionStoppingMediaState));
      a_sipSession.DisconnectFromMediaPlayer();
   }
   return res;
}

// SKS

//*****************************************************************************
// Function: int CSipSessionPlayingInvalidPinState::OnPlayComplete(CSipSession &a_sipSession)
// Description: Handle a play complete event
// Return: int - 0 for success, otherwise error
// Parameters: CSipSession &a_sipSession
//*****************************************************************************
int CSipSessionPlayingInvalidPinState::OnPlayComplete(CSipSession &a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "InvalidPlayCmplt");
   a_sipSession.SetSessionStateTo(SessionState(SipSessionPlayingGreetingState));
   return a_sipSession.PrimPlayGreeting();
}

//*****************************************************************************
// Function: int CSipSessionPlayingInvalidPinState::OnDtmf(CSipSession &a_sipSession, int a_dtmf)
// Description: Handle a DTMF detected event
// Return: int - 0 for success, otherwise error
// Parameters: CSipSession &a_sipSession
//             int a_dtmf
//*****************************************************************************
int CSipSessionPlayingInvalidPinState::OnDtmf(CSipSession &a_sipSession, int a_dtmf)
{

  char tmpstr[16];
  sprintf(tmpstr, "%c", a_dtmf);
  string dtmfstr(AsString(tmpstr));
  LogMsg(a_sipSession.SessionName(), "PlayingInvalidPinState::OnDtmf =" + dtmfstr);

   a_sipSession.collectDigit(dtmfstr);
   LogMsg(a_sipSession.SessionName(), "->PlayingGreeting");
   a_sipSession.SetSessionStateTo(SessionState(SipSessionPlayingGreetingState));
   
   if(a_sipSession.GetDigitCnt() == a_sipSession.GetConfPINLen())
   {
      LogMsg(a_sipSession.SessionName(), "OnDtmf len=" + AsString(a_sipSession.GetConfPINLen()));
      a_sipSession.VerifyConfPIN(); 
   }
   a_sipSession.clearDigits();
   (a_sipSession.IpmDevice())->GetDigits(1);	
   return a_sipSession.PrimStopPlay(a_dtmf);

}
//*****************************************************************************
// Function: int CSipSessionPlayingInvalidPinState::OnCallDisconnected(CSipSession &a_sipSession)
// Description: Handle a call disconnected event
// Return: int - 0 for success, otherwise error
// Parameters: CSipSession &a_sipSession
//*****************************************************************************
int CSipSessionPlayingInvalidPinState::OnCallDisconnected(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnCallDisconnected();
   if (res == 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionStoppingMediaState));
      a_sipSession.DisconnectFromMediaPlayer();
   }
   return res;
}


// SKS
//*****************************************************************************
// Function: int CSipSessionStoppingMediaState::StopMedia(CSipSession &a_sipSession)
// Description: Stop the media streaming
// Return: int - 0 for success, otherwise error
// Parameters: CSipSession &a_sipSession
//*****************************************************************************
int CSipSessionStoppingMediaState::StopMedia(CSipSession& a_sipSession)
{
   a_sipSession.SetSessionStateTo(SessionState(SipSessionStoppingMediaState));
   return a_sipSession.PrimStopMedia();
}

//*****************************************************************************
// Function: int CSipSessionStoppingMediaState::DisconnectFromMediaPlayer(CSipSession &a_sipSession)
// Description: Disconnect from the media player
// Return: int - 0 for success, otherwise error
// Parameters: CSipSession &a_sipSession
//*****************************************************************************
int CSipSessionStoppingMediaState::DisconnectFromMediaPlayer(CSipSession &a_sipSession)
{
   return a_sipSession.PrimDisconnectFromMediaPlayer();
}
//*****************************************************************************
// Function: int CSipSessionStoppingMediaState::OnIpmPortDisconnected(CSipSession &a_sipSession)
// Description: Handle a port disconnected event from the IPM device
// Return: int - 0 for success, otherwise error
// Parameters: CSipSession &a_sipSession
//*****************************************************************************
int CSipSessionStoppingMediaState::OnIpmPortDisconnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnIpmPortDisconnected");
   a_sipSession.StopMedia();
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionStoppingMediaState::OnMmPortDisconnected(CSipSession &a_sipSession)
// Description: Handle a port disconnected event from the MM device
// Return: int - 0 for success, otherwise error
// Parameters: CSipSession &a_sipSession
//*****************************************************************************
int CSipSessionStoppingMediaState::OnMmPortDisconnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnMmPortDisconnected");
	a_sipSession.PrimStopPlay(0);
   return 0;
}
//*****************************************************************************
// Function: int CSipSessionStoppingMediaState::OnPlayComplete(CSipSession &a_sipSession)
// Description: Handle a play complete event
// Return: int - 0 for success, otherwise error
// Parameters: CSipSession &a_sipSession
//*****************************************************************************
int CSipSessionStoppingMediaState::OnPlayComplete(CSipSession &a_sipSession)
{
	a_sipSession.DisconnectFromMediaPlayer();
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionStoppingMediaState::OnCallDisconnected(CSipSession &a_sipSession)
// Description: Handle a OnCallDisconnected Request
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionStoppingMediaState::OnCallDisconnected(CSipSession& a_sipSession)
{
   int res = 0;
   LogMsg(a_sipSession.SessionName(), "CSipSessionStoppingMediaState::OnCallDisconnected()");	
   //JM - disconnect as media is stopping, ??? 
   a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
   a_sipSession.ReleaseResource();
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionGreetingTeardownState::OnIpmPortDisconnected(CSipSession &a_sipSession)
// Description: Handle a port disconnected event from the IPM device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingTeardownState::OnIpmPortDisconnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnIpmPortDisconnected");
   if (a_sipSession.PrimOnIpmPortDisconnected() == IPM_MM_PORT_DISCONNECT_COMPLETE)
   {
      return a_sipSession.OnDisconnectFromMediaPlayerComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionGreetingTeardownState::OnMmPortDisconnected(CSipSession &a_sipSession)
// Description: Handle a port disconnected event from the MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingTeardownState::OnMmPortDisconnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnMmPortDisconnected");
   if (a_sipSession.PrimOnMmPortDisconnected() == IPM_MM_PORT_DISCONNECT_COMPLETE)
   {
      return a_sipSession.OnDisconnectFromMediaPlayerComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionGreetingTeardownState::OnDisconnectFromMediaPlayerComplete(CSipSession &a_sipSession)
// Description: Handle a disconnect from the media player completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingTeardownState::OnDisconnectFromMediaPlayerComplete(CSipSession &a_sipSession)
{
   int res = a_sipSession.PrimOnDisconnectFromMediaPlayerComplete();
   if (res == 0)
   {
      res = a_sipSession.OpenConferenceParty();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionGreetingTeardownState::OpenConferenceParty(CSipSession &a_sipSession)
// Description: Open a conference party
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingTeardownState::OpenConferenceParty(CSipSession &a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OpenConferenceParty");
   a_sipSession.SetSessionStateTo(SessionState(SipSessionConferenceSetupState));
   return a_sipSession.PrimOpenConferenceParty();
}

//*****************************************************************************
// Function: int CSipSessionGreetingTeardownState::OnIFrameRequest(CSipSession &a_sipSession)
// Description: Send an IFrame Update
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionGreetingTeardownState::OnIFrameRequest(CSipSession& a_sipSession)
{
   return a_sipSession.PrimOnIFrameRequest();
}

//*****************************************************************************
// Function: int CSipSessionConferenceSetupState::OnOpenConferencePartyComplete(CSipSession &a_sipSession)
// Description: Handle an open conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionConferenceSetupState::OnOpenConferencePartyComplete(CSipSession &a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnOpenConferencePartyComplete");
   if (!a_sipSession.HaveGreeting() && a_sipSession.m_systemConfiguration.isPinEnabled() == false) {
      (a_sipSession.IpmDevice())->GetDigits(1);
   }
   int res = a_sipSession.PrimOnOpenConferencePartyComplete();
   if (res == 0)
   {
      res = a_sipSession.ConnectToConferenceParty();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionConferenceSetupState::ConnectToConferenceParty(CSipSession &a_sipSession)
// Description: Connect to the conference party
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionConferenceSetupState::ConnectToConferenceParty(CSipSession &a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "ConnectToConferenceParty");
   return a_sipSession.PrimConnectToConferenceParty();
}

//*****************************************************************************
// Function: int CSipSessionConferenceSetupState::OnGetIpmRxPortInfoComplete(CSipSession &a_sipSession)
// Description: Handle a get RX port info completion event from the IPM device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionConferenceSetupState::OnGetIpmRxPortInfoComplete(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnGetIpmRxPortInfoComplete");
   if (a_sipSession.PrimOnGetIpmRxPortInfoComplete() == IPM_CNF_PORT_INFO_COMPLETE)
   {
      return a_sipSession.PrimPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionConferenceSetupState::OnGetIpmTxPortInfoComplete(CSipSession &a_sipSession)
// Description: Handle a get TX port info completion event from the IPM device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionConferenceSetupState::OnGetIpmTxPortInfoComplete(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnGetIpmTxPortInfoComplete");
   if (a_sipSession.PrimOnGetIpmTxPortInfoComplete() == IPM_CNF_PORT_INFO_COMPLETE)
   {
      return a_sipSession.PrimPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionConferenceSetupState::OnGetCnfRxPortInfoComplete(CSipSession &a_sipSession)
// Description: Handle a get RX port info completion event from the CNF device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionConferenceSetupState::OnGetCnfRxPortInfoComplete(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnGetCnfRxPortInfoComplete");
   if (a_sipSession.PrimOnGetCnfRxPortInfoComplete() == IPM_CNF_PORT_INFO_COMPLETE)
   {
      return a_sipSession.PrimPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionConferenceSetupState::OnGetCnfTxPortInfoComplete(CSipSession &a_sipSession)
// Description: Handle a get TX port info completion event from the CNF device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionConferenceSetupState::OnGetCnfTxPortInfoComplete(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnGetCnfTxPortInfoComplete");
   if (a_sipSession.PrimOnGetCnfTxPortInfoComplete() == IPM_CNF_PORT_INFO_COMPLETE)
   {
      return a_sipSession.PrimPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionConferenceSetupState::OnCnfPortConnected(CSipSession &a_sipSession)
// Description: Handle a port connected event from the CNF device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionConferenceSetupState::OnCnfPortConnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnCnfPortConnected");
   if (a_sipSession.PrimOnCnfPortConnected() == IPM_CNF_PORT_CONNECT_COMPLETE)
   {
      return a_sipSession.OnConnectToConferencePartyComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionConferenceSetupState::OnIpmPortConnected(CSipSession &a_sipSession)
// Description: Handle a port connected event from the IMP device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionConferenceSetupState::OnIpmPortConnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnIpmPortConnected");
   if (a_sipSession.PrimOnIpmPortConnected() == IPM_CNF_PORT_CONNECT_COMPLETE)
   {
      return a_sipSession.OnConnectToConferencePartyComplete();
   }
   return 0;

}

//*****************************************************************************
// Function: int CSipSessionConferenceSetupState::OnConnectToConferencePartyComplete(CSipSession &a_sipSession)
// Description: Handle a connect to conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionConferenceSetupState::OnConnectToConferencePartyComplete(CSipSession &a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnConnectToConferencePartyComplete");

   int res = a_sipSession.PrimOnConnectToConferencePartyComplete();
   if (res == 0)
   {
      if(a_sipSession.isMediaStarted())
      {
        return a_sipSession.JoinConference();
      }
      else
      {
        return a_sipSession.StartMedia();
      }
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionConferenceSetupState::JoinConference(CSipSession &a_sipSession)
// Description: Join the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionConferenceSetupState::JoinConference(CSipSession &a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "JoinConference");
   a_sipSession.SetSessionStateTo(SessionState(SipSessionJoiningConferenceState));
   return a_sipSession.PrimJoinConference();
}

//*****************************************************************************
// Function: int CSipSessionConferenceSetupState::OnCallDisconnected(CSipSession &a_sipSession)
// Description: Handle a OnCallDisconnected Request
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionConferenceSetupState::OnCallDisconnected(CSipSession& a_sipSession)
{
   int res = 0;
   LogMsg(a_sipSession.SessionName(), "CSipSessionConferenceSetupState::OnCallDisconnected()");	
   //JM - disconnect as call being established
   a_sipSession.SetSessionStateTo(SessionState(SipSessionLeavingConferenceState));
   res = a_sipSession.PrimLeaveConference();
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionConferenceSetupState::StartMedia(CSipSession &a_sipSession)
// Description: Start media streaming
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionConferenceSetupState::StartMedia(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "StartMedia");
   a_sipSession.SetSessionStateTo(SessionState(SipSessionStartingMediaState));
   return a_sipSession.PrimStartMedia();
}

//*****************************************************************************
// Function: int CSipSessionJoiningConferenceState::OnJoinConferenceComplete(CSipSession &a_sipSession)
// Description: Handle a join conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionJoiningConferenceState::OnJoinConferenceComplete(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnJoinConferenceComplete");
   int res = a_sipSession.PrimOnJoinConferenceComplete();
   if (res == 0)
   {
        a_sipSession.SetSessionStateTo(SessionState(SipSessionAddingCallerIdOverlayState));
        a_sipSession.PrimAddCallerIdOverlay();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionJoiningConferenceState::OnIFrameRequest(CSipSession &a_sipSession)
// Description: Send an IFrame Update
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionJoiningConferenceState::OnIFrameRequest(CSipSession& a_sipSession)
{
   return a_sipSession.PrimOnIFrameRequest();
}

//*****************************************************************************
// Function: int CSipSessionAddingCallerIdOverlayState::OnAddImageOverlay(CSipSession &a_sipSession, METAEVENT& a_metaEvent)
// Description: Handle an add image overlay completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//             METAEVENT& a_metaEvent
//*****************************************************************************
int CSipSessionAddingCallerIdOverlayState::OnAddImageOverlay(CSipSession &a_sipSession, METAEVENT& a_metaEvent)
{
   LogMsg(a_sipSession.SessionName(), "OnAddImageOverlay");
   int res = a_sipSession.PrimOnAddCallerIdOverlay(a_metaEvent);
   if (res == 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionInConferenceState));
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionAddingCallerIdOverlayState::LeaveConference(CSipSession &a_sipSession)
// Description: Handle LeaveConference command
// Return: int 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionAddingCallerIdOverlayState::LeaveConference(CSipSession& a_sipSession)
{
	LogMsg(a_sipSession.SessionName(), "CSipSessionAddingCallerIdOverlayState::LeaveConference()");	
   a_sipSession.SetSessionStateTo(SessionState(SipSessionRemovingCallerIdOverlayState));
   return a_sipSession.PrimRemoveCallerIdOverlay();
}
//*****************************************************************************
// Function: int CSipSessionAddingCallerIdOverlayState::OnIFrameRequest(CSipSession &a_sipSession)
// Description: Send an IFrame Update
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionAddingCallerIdOverlayState::OnIFrameRequest(CSipSession& a_sipSession)
{
   return a_sipSession.PrimOnIFrameRequest();
}

//*****************************************************************************
// Function: int CSipSessionInConferenceState::LeaveConference(CSipSession &a_sipSession)
// Description: Leave the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInConferenceState::LeaveConference(CSipSession &a_sipSession)
{
	LogMsg(a_sipSession.SessionName(), "CSipSessionInConferenceState::LeaveConference()");	
   a_sipSession.SetSessionStateTo(SessionState(SipSessionRemovingCallerIdOverlayState));
   return a_sipSession.PrimRemoveCallerIdOverlay();
}

//*****************************************************************************
// Function: int CSipSessionInConferenceState::OnDtmf(CSipSession &a_sipSession, int a_dtmf)
// Description: Handle a DTMF event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//             int a_dtmf 
//*****************************************************************************
int CSipSessionInConferenceState::OnDtmf(CSipSession& a_sipSession, int a_dtmf)
{
   LogMsg(a_sipSession.SessionName(), "CSipSessionInConferenceState::OnDtmf()");
   int res(0);

   if (a_dtmf == '*')
   {
      res = a_sipSession.PrimCycleConferenceLayout();
   }
   else if (a_dtmf == '#')
   {
      res = a_sipSession.PrimRotateConferenceParties();
   }
   a_sipSession.clearDigits();
   (a_sipSession.IpmDevice())->GetDigits(1);
   return res;
}

//*****************************************************************************
// Function: int CSipSessionInConferenceState::OnIFrameRequest(CSipSession &a_sipSession)
// Description: Send an IFrame Update
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInConferenceState::OnIFrameRequest(CSipSession& a_sipSession)
{
   return a_sipSession.PrimOnIFrameRequest();
}

//*****************************************************************************
// Function: int CSipSessionInConferenceState::AddCallerIdOverlay(CSipSession &a_sipSession)
// Description: Display the caller ID overlay
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionInConferenceState::AddCallerIdOverlay(CSipSession& a_sipSession)
{
   a_sipSession.SetSessionStateTo(SessionState(SipSessionRefreshingCallerIdOverlayState));
   return a_sipSession.PrimRemoveCallerIdOverlay();
}

//*****************************************************************************
// Function: int CSipSessionRefreshingCallerIdOverlayState::AddCallerIdOverlay(CSipSession &a_sipSession)
// Description: Display the caller ID overlay
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionRefreshingCallerIdOverlayState::AddCallerIdOverlay(CSipSession& a_sipSession)
{
   //JM - Ignore request to add image overlay already refresh overlay - multiple parties join before refresh completes
   LogMsg(a_sipSession.SessionName(), "Ignore AddCallerIdOverlay in state:" + AsString(Desc()));
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionRefreshingCallerIdOverlayState::OnRemoveImageOverlay(CSipSession &a_sipSession, METAEVENT& a_metaEvent)
// Description: Handle a remove image overlay completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//             METAEVENT& a_metaEvent
//*****************************************************************************
int CSipSessionRefreshingCallerIdOverlayState::OnRemoveImageOverlay(CSipSession &a_sipSession, METAEVENT& a_metaEvent)
{
   LogMsg(a_sipSession.SessionName(), "OnRemoveImageOverlay");
   int res = a_sipSession.PrimOnRemoveCallerIdOverlay(a_metaEvent);
   if (res == 0)
   {
      res = a_sipSession.PrimAddCallerIdOverlay();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionRefreshingCallerIdOverlayState::LeaveConference(CSipSession &a_sipSession)
// Description: Handle a LeaveConference COmmand
// Return: int 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionRefreshingCallerIdOverlayState::LeaveConference(CSipSession& a_sipSession)
{
	LogMsg(a_sipSession.SessionName(), "CSipSessionRefreshingCallerIdOverlayState::LeaveConference()");	
   a_sipSession.SetSessionStateTo(SessionState(SipSessionRemovingCallerIdOverlayState));
   return a_sipSession.PrimRemoveCallerIdOverlay();
}

//*****************************************************************************
// Function: int CSipSessionRefreshingCallerIdOverlayState::OnAddImageOverlay(CSipSession &a_sipSession, METAEVENT& a_metaEvent)
// Description: Handle an add image overlay completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//             METAEVENT& a_metaEvent
//*****************************************************************************
int CSipSessionRefreshingCallerIdOverlayState::OnAddImageOverlay(CSipSession &a_sipSession, METAEVENT& a_metaEvent)
{
   LogMsg(a_sipSession.SessionName(), "OnAddImageOverlay");
   a_sipSession.SetSessionStateTo(SessionState(SipSessionInConferenceState));
   int res = a_sipSession.PrimOnAddCallerIdOverlay(a_metaEvent);
   if (res == 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionInConferenceState));
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionRefreshingCallerIdOverlayState::OnIFrameRequest(CSipSession &a_sipSession)
// Description: Send an IFrame Update
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionRefreshingCallerIdOverlayState::OnIFrameRequest(CSipSession& a_sipSession)
{
   return a_sipSession.PrimOnIFrameRequest();
}

//*****************************************************************************
// Function: int CSipSessionRemovingCallerIdOverlayState::OnRemoveImageOverlay(CSipSession &a_sipSession, METAEVENT& a_metaEvent)
// Description: Handle a remove image overlay completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//             METAEVENT& a_metaEvent
//*****************************************************************************
int CSipSessionRemovingCallerIdOverlayState::OnRemoveImageOverlay(CSipSession &a_sipSession, METAEVENT& a_metaEvent)
{
   LogMsg(a_sipSession.SessionName(), "OnRemoveImageOverlay");
   int res = a_sipSession.PrimOnRemoveCallerIdOverlay(a_metaEvent);
   if (res == 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionLeavingConferenceState));
      res = a_sipSession.PrimLeaveConference();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionRemovingCallerIdOverlayState::OnIFrameRequest(CSipSession &a_sipSession)
// Description: Send an IFrame Update
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionRemovingCallerIdOverlayState::OnIFrameRequest(CSipSession& a_sipSession)
{
   return a_sipSession.PrimOnIFrameRequest();
}

//*****************************************************************************
// Function: int CSipSessionLeavingConferenceState::OnLeaveConferenceComplete(CSipSession &a_sipSession)
// Description: Handle a leave conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionLeavingConferenceState::OnLeaveConferenceComplete(CSipSession &a_sipSession)
{
   int res = a_sipSession.PrimOnLeaveConferenceComplete();
   if (res == 0)
   {
      if(a_sipSession.isConfPartyConnected())
      {
         res = a_sipSession.DisconnectFromConferenceParty();
      } 
      else
      {
         res = a_sipSession.OnDisconnectFromConferencePartyComplete();
      }
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionLeavingConferenceState::DisconnectFromConferenceParty(CSipSession &a_sipSession)
// Description: Disconnect from the conferenmce party
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionLeavingConferenceState::DisconnectFromConferenceParty(CSipSession &a_sipSession)
{
   return a_sipSession.PrimDisconnectFromConferenceParty();
}

//*****************************************************************************
// Function: int CSipSessionLeavingConferenceState::OnCnfPortDisconnected(CSipSession &a_sipSession)
// Description: Handle a port disconnected event from the CNF device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionLeavingConferenceState::OnCnfPortDisconnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnCnfPortDisconnected");
   if (a_sipSession.PrimOnCnfPortDisconnected() == IPM_CNF_PORT_DISCONNECT_COMPLETE)
   {
      return a_sipSession.OnDisconnectFromConferencePartyComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionLeavingConferenceState::OnIpmPortDisconnected(CSipSession &a_sipSession)
// Description: Handle a port disconnected event from the IPM device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionLeavingConferenceState::OnIpmPortDisconnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnIpmPortDisconnected");
   if (a_sipSession.PrimOnIpmPortDisconnected() == IPM_CNF_PORT_DISCONNECT_COMPLETE)
   {
      return a_sipSession.OnDisconnectFromConferencePartyComplete();
   }
   return 0;
}


//*****************************************************************************
// Function: int CSipSessionLeavingConferenceState::OnDisconnectFromConferencePartyComplete(CSipSession &a_sipSession)
// Description: Handle a disconnect from conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionLeavingConferenceState::OnDisconnectFromConferencePartyComplete(CSipSession &a_sipSession)
{
   int res = a_sipSession.PrimOnDisconnectFromConferencePartyComplete();
   if (res == 0)
   {
      res = a_sipSession.CloseConferenceParty();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionLeavingConferenceState::CloseConferenceParty(CSipSession &a_sipSession)
// Description: Close the conference party device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionLeavingConferenceState::CloseConferenceParty(CSipSession &a_sipSession)
{
   return a_sipSession.PrimCloseConferenceParty();
}

//*****************************************************************************
// Function: int CSipSessionLeavingConferenceState::OnCloseConferencePartyComplete(CSipSession &a_sipSession)
// Description: Handle a close conferenmce party device copmpletion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionLeavingConferenceState::OnCloseConferencePartyComplete(CSipSession &a_sipSession)
{
   int res = a_sipSession.PrimOnCloseConferencePartyComplete();
   if (res == 0)
   {
      if(a_sipSession.isMediaStarted())
      {
        a_sipSession.SetSessionStateTo(SessionState(SipSessionMediaStreamingState));
        a_sipSession.StopMedia();
      }
      else
      {
        a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
        a_sipSession.ReleaseResource();
      }
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionLeavingConferenceState::OnIFrameRequest(CSipSession &a_sipSession)
// Description: Send an IFrame Update
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionLeavingConferenceState::OnIFrameRequest(CSipSession& a_sipSession)
{
   return a_sipSession.PrimOnIFrameRequest();
}


//*****************************************************************************
// Function: int CSipSessionStoppingMediaState::OnStopMediaComplete(CSipSession &a_sipSession)
// Description: Handle a stop media streaming completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionStoppingMediaState::OnStopMediaComplete(CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnStopMediaComplete();
   if (res == 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionCallTeardownState));
      a_sipSession.ReleaseResource();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionCallTeardownState::ReleaseResource(CSipSession &a_sipSession)
// Description: Release the codec resources
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionCallTeardownState::ReleaseResource( CSipSession& a_sipSession)
{
   return a_sipSession.PrimReleaseResource();
}

//*****************************************************************************
// Function: int CSipSessionCallTeardownState::OnReleaseResourceComplete(CSipSession &a_sipSession)
// Description: Handle a release codec resources completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionCallTeardownState::OnReleaseResourceComplete( CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnReleaseResourceComplete();
   if (res == 0)
   {
      //JM - Last check to make sure MediaPlayer is not connected
      if(a_sipSession.isMediaPlayerConnected())
      {
         res = a_sipSession.DisconnectFromMediaPlayer();
      } 
      else
      {
         res = a_sipSession.ReleaseCall();
      }
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionCallTeardownState::DisconnectFromMediaPlayer(CSipSession &a_sipSession)
// Description: Disconnect from the media player
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionCallTeardownState::DisconnectFromMediaPlayer(CSipSession &a_sipSession)
{
   return a_sipSession.PrimDisconnectFromMediaPlayer();
}

//*****************************************************************************
// Function: int CSipSessionCallTeardownState::OnIpmPortDisconnected(CSipSession &a_sipSession)
// Description: Handle a port disconnected event from the IPM device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionCallTeardownState::OnIpmPortDisconnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnIpmPortDisconnected");
   if (a_sipSession.PrimOnIpmPortDisconnected() == IPM_MM_PORT_DISCONNECT_COMPLETE)
   {
      return a_sipSession.OnDisconnectFromMediaPlayerComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionCallTeardownState::OnMmPortDisconnected(CSipSession &a_sipSession)
// Description: Handle a port disconnected event from the MM device
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionCallTeardownState::OnMmPortDisconnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "OnMmPortDisconnected");
   if (a_sipSession.PrimOnMmPortDisconnected() == IPM_MM_PORT_DISCONNECT_COMPLETE)
   {
      return a_sipSession.OnDisconnectFromMediaPlayerComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionCallTeardownState::OnDisconnectFromMediaPlayerComplete(CSipSession &a_sipSession)
// Description: Handle a disconnect from the media player completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionCallTeardownState::OnDisconnectFromMediaPlayerComplete(CSipSession &a_sipSession)
{
   int res = a_sipSession.PrimOnDisconnectFromMediaPlayerComplete();
   if (res == 0)
   {
      res = a_sipSession.ReleaseCall();
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionCallTeardownState::OnPlayComplete(CSipSession &a_sipSession)
// Description: Handle a play complete event
// Return: int - 0 for success, otherwise error
// Parameters: CSipSession &a_sipSession
//*****************************************************************************
int CSipSessionCallTeardownState::OnPlayComplete(CSipSession &a_sipSession)
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSessionCallTeardownState::ReleaseCall(CSipSession &a_sipSession)
// Description: Release a call
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionCallTeardownState::ReleaseCall( CSipSession& a_sipSession)
{
   return a_sipSession.PrimReleaseCall();
}

//*****************************************************************************
// Function: int CSipSessionCallTeardownState::OnCallReleased(CSipSession &a_sipSession)
// Description: Handle a release call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionCallTeardownState::OnCallReleased( CSipSession& a_sipSession)
{
   int res = a_sipSession.PrimOnCallReleased();
   if (res == 0)
   {
      a_sipSession.SetSessionStateTo(SessionState(SipSessionIdleState));
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSessionCallTeardownState::OnCallDisconnected(CSipSession &a_sipSession)
// Description: Handle a OnCallDisconnected
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession &a_sipSession 
//*****************************************************************************
int CSipSessionCallTeardownState::OnCallDisconnected(CSipSession& a_sipSession)
{
   LogMsg(a_sipSession.SessionName(), "CSipSessionCallTeardownState::OnCallDisconnected() ignoring...");	
   return 0;
}

