/**
* @file CSipSessionState.h
* @brief SIP Session States - class declarations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CSipSessionState_h
#define CSipSessionState_h

#include <map>

#include "CSipSession.h"

typedef enum 
{
   SipSessionIdleState = 0,
   SipSessionInboundCallSetupState,
   SipSessionInboundCallAcceptedState,
   SipSessionInboundCallAnswerState,
   SipSessionStartingMediaState,
   SipSessionMediaStreamingState,
   SipSessionConferenceSetupState,
   SipSessionGreetingSetupState,
   SipSessionPlayingGreetingState,
   SipSessionGreetingTeardownState,
   SipSessionPlayingInvalidPinState,
   SipSessionJoiningConferenceState,
   SipSessionInConferenceState,
   SipSessionLeavingConferenceState,
   SipSessionStoppingMediaState,
   SipSessionCallTeardownState,
   SipSessionAddingCallerIdOverlayState,
   SipSessionRemovingCallerIdOverlayState,
   SipSessionRefreshingCallerIdOverlayState
} SipSessionState;

class ASipSessionState;

//*****************************************************************************
// Class: ::CSipSessionStates
// Description: Map of SIP session state objects
//*****************************************************************************
class CSipSessionStates
{
public:
   CSipSessionStates();
   virtual ~CSipSessionStates() {}
   ASipSessionState* SessionState(SipSessionState sipSessionState) {return m_sipSessionStates[sipSessionState];}
private:
   std::map< SipSessionState, ASipSessionState*> m_sipSessionStates;
};

//*****************************************************************************
// Class: ::ASipSessionState
// Description: Abstract SIP Session state
// Notes:      
//    This abstract class declares all functions for all SIP Session states as
//    virtual and implements them as an error.
// 
//    Concrete subclasses implement only the functions that are valid for that
//    state.
// 
//    State classes are responsible only for calling primative functions on the
//    context SIP Session object and changing the state of the context SIP 
//    Session object.
// 
//    As state classes have no non-static member variables, one set of SIP 
//    Session state objects are shared by all SIP Sessions.
//*****************************************************************************
class ASipSessionState
{
public:
   ASipSessionState() {}
   virtual ~ASipSessionState() {}

   virtual const char* Desc() = 0;

   virtual int OnInboundCallOffered(CSipSession& sipSession) {return LogStateError(sipSession, "OnInboundCallOffered");}
   virtual int InboundCallSendProceeding(CSipSession& sipSession) {return LogStateError(sipSession, "InboundCallSendProceeding");}
   virtual int OnInboundCallSendProceedingComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnInboundCallSendProceedingComplete");}
   virtual int ReserveResource(CSipSession& sipSession) {return LogStateError(sipSession, "ReserveResource");}
   virtual int OnReserveResourceComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnReserveResourceComplete");}
   virtual int GetLocalMediaInfo(CSipSession& sipSession) {return LogStateError(sipSession, "GetLocalMediaInfo");}
   virtual int OnGetLocalMediaInfoComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnGetLocalMediaInfoComplete");}
   virtual int AcceptInboundCall(CSipSession& sipSession) {return LogStateError(sipSession, "AcceptInboundCall");}
   virtual int OnAcceptInboundCallComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnAcceptInboundCallComplete");}
   virtual int AnswerInboundCall(CSipSession& sipSession) {return LogStateError(sipSession, "AnswerInboundCall");}
   virtual int OnAnswerInboundCallComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnAnswerInboundCallComplete");}
   virtual int StartMedia(CSipSession& sipSession) {return LogStateError(sipSession, "StartMedia");}
   virtual int OnStartMediaComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnStartMediaComplete");}
   virtual int OpenConferenceParty(CSipSession& sipSession) {return LogStateError(sipSession, "OpenConferenceParty");}
   virtual int OnOpenConferencePartyComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnOpenConferencePartyComplete");}
   virtual int ConnectToConferenceParty(CSipSession& sipSession) {return LogStateError(sipSession, "ConnectToConferenceParty");}
   virtual int OnConnectToConferencePartyComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnConnectToConferencePartyComplete");}
   virtual int JoinConference(CSipSession& sipSession) {return LogStateError(sipSession, "JoinConference");}
   virtual int OnJoinConferenceComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnJoinConferenceComplete");}
   virtual int LeaveConference(CSipSession& sipSession) {return LogStateError(sipSession, "LeaveConference");}
   virtual int OnLeaveConferenceComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnLeaveConferenceComplete");}
   virtual int DisconnectFromConferenceParty(CSipSession& sipSession) {return LogStateError(sipSession, "DisconnectFromConferenceParty");}
   virtual int OnDisconnectFromConferencePartyComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnDisconnectFromConferencePartyComplete");}
   virtual int CloseConferenceParty(CSipSession& sipSession) {return LogStateError(sipSession, "CloseConferenceParty");}
   virtual int OnCloseConferencePartyComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnCloseConferencePartyComplete");}
   virtual int StopMedia(CSipSession& sipSession) {return LogStateError(sipSession, "StopMedia");}
   virtual int OnStopMediaComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnStopMediaComplete");}
   virtual int OnCallDisconnected(CSipSession& sipSession) {return LogStateError(sipSession, "OnCallDisconnected");}
   virtual int ReleaseResource(CSipSession& sipSession) {return LogStateError(sipSession, "ReleaseResource");}
   virtual int OnReleaseResourceComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnReleaseResourceComplete");}
   virtual int ReleaseCall(CSipSession& sipSession) {return LogStateError(sipSession, "ReleaseCall");}
   virtual int OnCallReleased(CSipSession& sipSession) {return LogStateError(sipSession, "OnCallReleased");}

   virtual int OnGetIpmRxPortInfoComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnGetIpmRxPortInfoComplete");}
   virtual int OnGetIpmTxPortInfoComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnGetIpmTxPortInfoComplete");}
   virtual int OnGetCnfRxPortInfoComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnGetCnfRxPortInfoComplete");}
   virtual int OnGetCnfTxPortInfoComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnGetCnfTxPortInfoComplete");}
   virtual int OnGetMmRxPortInfoComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnGetMmRxPortInfoComplete");}
   virtual int OnGetMmTxPortInfoComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnGetMmTxPortInfoComplete");}

   virtual int OnCnfPortConnected(CSipSession& sipSession) {return LogStateError(sipSession, "OnCnfPortConnected");}
   virtual int OnIpmPortConnected(CSipSession& sipSession) {return LogStateError(sipSession, "OnIpmPortConnected");}
   virtual int OnMmPortConnected(CSipSession& sipSession) {return LogStateError(sipSession, "OnMmPortConnected");}
   virtual int OnCnfPortDisconnected(CSipSession& sipSession) {return LogStateError(sipSession, "OnCnfPortDisconnected");}
   virtual int OnIpmPortDisconnected(CSipSession& sipSession) {return LogStateError(sipSession, "OnIpmPortDisconnected");}
   virtual int OnMmPortDisconnected(CSipSession& sipSession) {return LogStateError(sipSession, "OnMmPortDisconnected");}

   virtual int ConnectToMediaPlayer(CSipSession& sipSession) {return LogStateError(sipSession, "ConnectToMediaPlayer");}
   virtual int OnConnectToMediaPlayerComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnConnectToMediaPlayerComplete");}

   virtual int PlayGreeting(CSipSession& sipSession) {return LogStateError(sipSession, "PlayGreeting");}
   virtual int OnPlayComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnPlayComplete");}
   virtual int OnRecordComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnRecordComplete");}

   virtual int OnDtmf(CSipSession& sipSession, int a_dtmf) {return LogStateError(sipSession, "OnDtmf");}

   virtual int OnIFrameRequest(CSipSession& sipSession) {return LogStateError(sipSession, "OnIFrameRequest");}

   virtual int DisconnectFromMediaPlayer(CSipSession& sipSession) {return LogStateError(sipSession, "DisconnectFromMediaPlayer");}
   virtual int OnDisconnectFromMediaPlayerComplete(CSipSession& sipSession) {return LogStateError(sipSession, "OnDisconnectFromMediaPlayerComplete");}

   virtual int AddCallerIdOverlay(CSipSession& sipSession) {return LogStateError(sipSession, "AddCallerIdOverlay");}
   virtual int AddConferenceLayoutOverlay(CSipSession& sipSession,int a_layout) {return LogStateError(sipSession, "AddConferenceLayoutOverlay");}
   virtual int RemoveCallerIdOverlay(CSipSession& sipSession) {return LogStateError(sipSession, "RemoveCallerIdOverlay");}
   virtual int RemoveConferenceLayoutOverlay(CSipSession& sipSession) {return LogStateError(sipSession, "RemoveConferenceLayoutOverlay");}
   virtual int OnAddImageOverlay(CSipSession& sipSession,METAEVENT& a_metaEvent) {return LogStateError(sipSession, "OnAddImageOverlay");}
   virtual int OnRemoveImageOverlay(CSipSession& sipSession,METAEVENT& a_metaEvent) {return LogStateError(sipSession, "OnRemoveImageOverlay");}

   static ASipSessionState* SessionState(SipSessionState sipSessionState) {return m_sipSessionStates.SessionState(sipSessionState);}

private:
   int LogStateError(CSipSession& sipSession, const std::string& msg);
   static CSipSessionStates m_sipSessionStates;
};

//*****************************************************************************
// Class: ::CSipSessionIdleState
// Description: SIP session idle state
//*****************************************************************************
class CSipSessionIdleState : public ASipSessionState
{
public:
   CSipSessionIdleState() {}
   int OnPlayComplete(CSipSession& sipSession);
   int OnRecordComplete(CSipSession& sipSession);
   virtual ~CSipSessionIdleState() {}
   const char* Desc() {return "idle";}
   int OnInboundCallOffered( CSipSession& sipSession);
};

//*****************************************************************************
// Class: ::ASipCallActiveState
// Description: Abstract SIP session active state
//*****************************************************************************
class ASipCallActiveState : public ASipSessionState
{
public:
   ASipCallActiveState() {}
   virtual ~ASipCallActiveState() {}
   int OnCallDisconnected(CSipSession& sipSession);
};

//*****************************************************************************
// Class: ::CSipSessionInboundCallSetupState
// Description: SIP session inbound calkl setup state
//*****************************************************************************
class CSipSessionInboundCallSetupState : public ASipCallActiveState
{
public:
   CSipSessionInboundCallSetupState() {}
   virtual ~CSipSessionInboundCallSetupState() {}
   const char* Desc() {return "inbound-setup";}
   int InboundCallSendProceeding(CSipSession& sipSession);
   int OnInboundCallSendProceedingComplete(CSipSession& sipSession);
   int ReserveResource(CSipSession& sipSession);
   int OnReserveResourceComplete(CSipSession& sipSession);
   int GetLocalMediaInfo(CSipSession& sipSession);
   int OnGetLocalMediaInfoComplete(CSipSession& sipSession);
   int AcceptInboundCall(CSipSession& sipCall);
   int OnAcceptInboundCallComplete(CSipSession& sipCall);
   //int AnswerInboundCall(CSipSession& sipCall);
   //int OnAnswerInboundCallComplete(CSipSession& sipCall);
};

//*****************************************************************************
// Class: ::CSipSessionInboundCallAcceptedState
// Description: SIP session call Accepted state
//*****************************************************************************
class CSipSessionInboundCallAcceptedState : public ASipSessionState
{
public:
   CSipSessionInboundCallAcceptedState() {}
   virtual ~CSipSessionInboundCallAcceptedState() {}
   const char* Desc() {return "accepted";}
   //JM - state changes
   int OpenConferenceParty(CSipSession& sipSession);
   int ConnectToMediaPlayer(CSipSession& sipCall);
   int OnCallDisconnected(CSipSession& sipSession);
   //int StartMedia(CSipSession& sipCall);
};

//*****************************************************************************
// Class: ::CSipSessionInboundCallAnswerState
// Description: SIP session call answer state
//*****************************************************************************
class CSipSessionInboundCallAnswerState : public ASipSessionState
{
public:
   CSipSessionInboundCallAnswerState() {}
   virtual ~CSipSessionInboundCallAnswerState() {}
   const char* Desc() {return "answer";}
   //JM - state change
   int AnswerInboundCall(CSipSession& sipCall);
   int OnAnswerInboundCallComplete(CSipSession& sipCall);
   int OnCallDisconnected(CSipSession& sipSession);
};

//*****************************************************************************
// Class: ::CSipSessionStartingMediaState
// Description: SIP session starting media streaming state
//*****************************************************************************
class CSipSessionStartingMediaState : public ASipCallActiveState
{
public:
   CSipSessionStartingMediaState() {}
   virtual ~CSipSessionStartingMediaState() {}
   const char* Desc() {return "starting media";}
   int OnStartMediaComplete(CSipSession& sipCall);
   int OnCallDisconnected(CSipSession& sipSession);
   int OnStopMediaComplete(CSipSession& sipSession);
};

//*****************************************************************************
// Class: ::CSipSessionMediaStreamingState
// Description: SIP session media streaming state
//*****************************************************************************
class CSipSessionMediaStreamingState : public ASipCallActiveState
{
public:
   CSipSessionMediaStreamingState() {}
   virtual ~CSipSessionMediaStreamingState() {}
   const char* Desc() {return "media streaming";}
   int StopMedia(CSipSession& sipCall);
   //JM - state changes
   int PlayGreeting(CSipSession& sipSession);
   int JoinConference(CSipSession& sipSession);
   int OnCallDisconnected(CSipSession& sipSession);
   //int OpenConferenceParty(CSipSession& sipSession);
   //int ConnectToMediaPlayer(CSipSession& sipCall);
   int OnIFrameRequest(CSipSession& sipCall);
};

//*****************************************************************************
// Class: ::CSipSessionConferenceSetupState
// Description: SIP session conference setup state
//*****************************************************************************
class CSipSessionConferenceSetupState : public ASipCallActiveState
{
public:
   CSipSessionConferenceSetupState() {}
   virtual ~CSipSessionConferenceSetupState() {}
   const char* Desc() {return "conference setup";}
   int OnOpenConferencePartyComplete(CSipSession& sipSession);
   int ConnectToConferenceParty(CSipSession& sipSession);
   int OnGetIpmRxPortInfoComplete(CSipSession& sipSession);
   int OnGetIpmTxPortInfoComplete(CSipSession& sipSession);
   int OnGetCnfRxPortInfoComplete(CSipSession& sipSession);
   int OnGetCnfTxPortInfoComplete(CSipSession& sipSession);
   int OnCnfPortConnected(CSipSession& sipSession);
   int OnIpmPortConnected(CSipSession& sipSession);
   int OnConnectToConferencePartyComplete(CSipSession& sipSession);
   //JM - state change
   int JoinConference(CSipSession& sipSession);
   int StartMedia(CSipSession& sipCall);
   int OnCallDisconnected(CSipSession& sipSession);
};

//*****************************************************************************
// Class: ::CSipSessionGreetingSetupState
// Description: SIP session greeting setup state
//*****************************************************************************
class CSipSessionGreetingSetupState : public ASipCallActiveState
{
public:
   CSipSessionGreetingSetupState() {}
   virtual ~CSipSessionGreetingSetupState() {}
   const char* Desc() {return "greeting setup";}
   int OnGetIpmRxPortInfoComplete(CSipSession& sipSession);
   int OnGetIpmTxPortInfoComplete(CSipSession& sipSession);
   int OnGetMmRxPortInfoComplete(CSipSession& sipSession);
   int OnGetMmTxPortInfoComplete(CSipSession& sipSession);
   int OnIpmPortConnected(CSipSession& a_sipSession);
   int OnMmPortConnected(CSipSession& a_sipSession);
   int OnConnectToMediaPlayerComplete(CSipSession& sipCall);
   //JM - state change
   int StartMedia(CSipSession& sipCall);
   int OnCallDisconnected(CSipSession& sipSession);
   //int PlayGreeting(CSipSession& sipSession);
   //int OnIFrameRequest(CSipSession& sipCall);
};

//*****************************************************************************
// Class: ::CSipSessionPlayingGreetingState
// Description: SIP session playing greeting state
//*****************************************************************************
class CSipSessionPlayingGreetingState : public ASipCallActiveState
{
public:
   CSipSessionPlayingGreetingState() {}
   virtual ~CSipSessionPlayingGreetingState() {}
   const char* Desc() {return "playing greeting";}
   int OnPlayComplete(CSipSession& sipSession);
   int DisconnectFromMediaPlayer(CSipSession& sipSession);
   int OnCallDisconnected(CSipSession& sipSession);
   int OnDtmf(CSipSession& sipSession, int a_dtmf);
   int OnIFrameRequest(CSipSession& sipCall);
};

//*****************************************************************************
// Class: ::CSipSessionGreetingTeardownState
// Description: SIP session greeting teardown state
//*****************************************************************************
class CSipSessionGreetingTeardownState : public ASipCallActiveState
{
public:
   CSipSessionGreetingTeardownState() {}
   virtual ~CSipSessionGreetingTeardownState() {}
   const char* Desc() {return "greeting teardown";}
   int OnMmPortDisconnected(CSipSession& sipSession);
   int OnIpmPortDisconnected(CSipSession& sipSession);
   int OnDisconnectFromMediaPlayerComplete(CSipSession& sipSession);
   int OnIFrameRequest(CSipSession& sipCall);
   int OpenConferenceParty(CSipSession& sipSession);
};

//*****************************************************************************
// Class: ::CSipSessionPlayingInvalidPinState
// Description: SIP session playing greeting state
//*****************************************************************************
class CSipSessionPlayingInvalidPinState : public ASipCallActiveState
{
public:
   CSipSessionPlayingInvalidPinState() {}
   virtual ~CSipSessionPlayingInvalidPinState() {}
   const char* Desc() {return "playing invalidPIN";}
   int OnPlayComplete(CSipSession& sipSession);
   int OnCallDisconnected(CSipSession& sipSession);
   int OnDtmf(CSipSession& sipSession, int a_dtmf);
};


//*****************************************************************************
// Class: ::CSipSessionJoiningConferenceState
// Description: SIP session joining conference state
//*****************************************************************************
class CSipSessionJoiningConferenceState : public ASipCallActiveState
{
public:
   CSipSessionJoiningConferenceState() {}
   virtual ~CSipSessionJoiningConferenceState() {}
   const char* Desc() {return "joining conference";}
   //JM - state changes
   //int OnOpenConferencePartyComplete(CSipSession& sipSession);
   //int ConnectToConferenceParty(CSipSession& sipSession);
   //int OnGetIpmRxPortInfoComplete(CSipSession& sipSession);
   //int OnGetIpmTxPortInfoComplete(CSipSession& sipSession);
   //int OnGetCnfRxPortInfoComplete(CSipSession& sipSession);
   //int OnGetCnfTxPortInfoComplete(CSipSession& sipSession);
   //int OnCnfPortConnected(CSipSession& sipSession);
   //int OnIpmPortConnected(CSipSession& sipSession);
   //int OnConnectToConferencePartyComplete(CSipSession& sipSession);
   //int JoinConference(CSipSession& sipSession);
   int OnJoinConferenceComplete(CSipSession& sipSession);
   int OnIFrameRequest(CSipSession& sipCall);
};

//*****************************************************************************
// Class: ::CSipSessionInConferenceState
// Description: SIP session in confrerence state
//*****************************************************************************
class CSipSessionInConferenceState : public ASipCallActiveState
{
public:
   CSipSessionInConferenceState() {}
   virtual ~CSipSessionInConferenceState() {}
   const char* Desc() {return "in conference";}
   int LeaveConference(CSipSession& sipSession);
   int OnDtmf(CSipSession& sipSession, int a_dtmf);
   int OnIFrameRequest(CSipSession& sipCall);
   int AddCallerIdOverlay(CSipSession& a_sipSession);
};

//*****************************************************************************
// Class: ::CSipSessionAddingCallerIdOverlayState
// Description: Adding the image overlay to display the caller id
//*****************************************************************************
class CSipSessionAddingCallerIdOverlayState : public ASipCallActiveState
{
public:
   CSipSessionAddingCallerIdOverlayState() {}
   virtual ~CSipSessionAddingCallerIdOverlayState() {}
   const char* Desc() {return "adding callerid overlay";}
   int OnAddImageOverlay(CSipSession& sipSession, METAEVENT& a_metaEvent);
   int OnIFrameRequest(CSipSession& sipCall);
   int LeaveConference(CSipSession& sipSession);
};

//*****************************************************************************
// Class: ::CSipSessionRefreshingCallerIdOverlayState
// Description: Removing the image overlay to display the caller id
//*****************************************************************************
class CSipSessionRefreshingCallerIdOverlayState : public ASipCallActiveState
{
public:
   CSipSessionRefreshingCallerIdOverlayState() {}
   virtual ~CSipSessionRefreshingCallerIdOverlayState() {}
   const char* Desc() {return "refreshing callerid overlay";}
   int AddCallerIdOverlay(CSipSession& a_sipSession);
   int OnAddImageOverlay(CSipSession& sipSession, METAEVENT& a_metaEvent);
   int OnRemoveImageOverlay(CSipSession& sipSession, METAEVENT& a_metaEvent);
   int OnIFrameRequest(CSipSession& sipCall);
   int LeaveConference(CSipSession& sipSession);
};

//*****************************************************************************
// Class: ::CSipSessionRemovingCallerIdOverlayState
// Description: Removing the image overlay to display the caller id
//*****************************************************************************
class CSipSessionRemovingCallerIdOverlayState : public ASipCallActiveState
{
public:
   CSipSessionRemovingCallerIdOverlayState() {}
   virtual ~CSipSessionRemovingCallerIdOverlayState() {}
   const char* Desc() {return "removing callerid overlay";}
   int OnRemoveImageOverlay(CSipSession& sipSession, METAEVENT& a_metaEvent);
   int OnIFrameRequest(CSipSession& sipCall);
};

//*****************************************************************************
// Class: ::CSipSessionLeavingConferenceState
// Description: SIP session leaving conference state
//*****************************************************************************
class CSipSessionLeavingConferenceState : public ASipCallActiveState
{
public:
   CSipSessionLeavingConferenceState() {}
   virtual ~CSipSessionLeavingConferenceState() {}
   const char* Desc() {return "leaving conference";}
   int OnLeaveConferenceComplete(CSipSession& sipSession);
   int DisconnectFromConferenceParty(CSipSession& sipSession);
   int OnCnfPortDisconnected(CSipSession& sipSession);
   int OnIpmPortDisconnected(CSipSession& sipSession);
   int OnDisconnectFromConferencePartyComplete(CSipSession& sipSession);
   int CloseConferenceParty(CSipSession& sipSession);
   int OnCloseConferencePartyComplete(CSipSession& sipSession);
   int OnIFrameRequest(CSipSession& sipCall);
};

//*****************************************************************************
// Class: ::CSipSessionStoppingMediaState
// Description: SIP session stopping media stremaing state
//*****************************************************************************
class CSipSessionStoppingMediaState : public ASipCallActiveState
{
public:
   CSipSessionStoppingMediaState() {}
   virtual ~CSipSessionStoppingMediaState() {}
   const char* Desc() {return "stopping media";}
// SKS
   int StopMedia(CSipSession& sipCall);
   int OnMmPortDisconnected(CSipSession& sipSession);
   int OnIpmPortDisconnected(CSipSession& sipSession);
   int DisconnectFromMediaPlayer(CSipSession& sipSession);
   int OnStopMediaComplete(CSipSession& sipCall);
   int OnPlayComplete(CSipSession& sipSession);
   int OnCallDisconnected(CSipSession& sipSession);
};

//*****************************************************************************
// Class: ::CSipSessionCallTeardownState
// Description: SIP session call teardown state
//*****************************************************************************
class CSipSessionCallTeardownState : public ASipSessionState
{
public:
   CSipSessionCallTeardownState() {}
   virtual ~CSipSessionCallTeardownState() {}
   const char* Desc() {return "call teardown";}
   int ReleaseResource(CSipSession& sipSession);
   int OnReleaseResourceComplete(CSipSession& sipSession);
   int ReleaseCall( CSipSession& sipCall);
   int OnCallReleased( CSipSession& sipCall);
   //JM - state changes
   int DisconnectFromMediaPlayer(CSipSession& sipSession);
   int OnMmPortDisconnected(CSipSession& sipSession);
   int OnIpmPortDisconnected(CSipSession& sipSession);
   int OnDisconnectFromMediaPlayerComplete(CSipSession& sipSession);
   int OnPlayComplete(CSipSession& sipSession);
   int OnCallDisconnected(CSipSession& sipSession);
};

#endif

