/**
* @file CSystemConfiguration.cpp
* @brief System Configuration class implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "CSystemConfiguration.h"
#include "LogUtil.h"

#include <algorithm>
#include <fstream>
#include <iostream>
#include <string>
#include <sstream>
#include <deque>

#ifdef WIN32
   #include <conio.h>
   #include <process.h>
#else
   #include <unistd.h>
#endif

//*****************************************************************************
// Function: CSystemConfiguration::CSystemConfiguration()
// Description: Default Constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CSystemConfiguration::CSystemConfiguration()
   : m_configFileName("VideoConfDemo.cfg")
   , m_mediaFilePath("")
   , m_sipPort(5060)
   , m_partyPerConf(MAX_PARTY_PER_CONF)
   , m_confRooms(MAX_CONF_COUNT)
   , m_confDuration(MAX_CONF_DURATION)
   , m_baseConfID("")
   , m_baseConfPIN("")
   , m_numVirtualCallers(0)
   , m_numVirtualCallerClips(0)
   , m_sipRegistrar("0.0.0.0")
   , m_phoneNumber("1000")
   , m_dtmfMode(DTMFMODE_INBAND)
   , m_localIpAddress("127.0.0.1")
   , m_greetingClip("")
   , m_virtualCallerClip1("caller1")
   , m_virtualCallerClip2("caller2")
   , m_virtualCallerClip3("caller3")
   , m_virtualCallerClip4("caller4")
   , m_virtualCallerClip5("caller5")
   , m_virtualCallerClip6("caller6")
   , m_virtualCallerClip7("caller7")
   , m_virtualCallerClip8("caller8")
   , m_initialLayout(1)
   , m_allowedLayoutsCount(0)
   , m_defaultVideoRate(384)
   , m_activeTalkerMode(0)
   , m_featureDelayTime(0)
   , m_demoFeature(0)
   , m_confToneClamping(0)
   , m_confPartyEchoCancelation(0)
   , m_bRecordConf(false)
   , m_sRecordVideoCodec("h263")
   , m_bPinEnabled(false)
{
   m_allowedLayoutsCount = 5;
   m_allowedLayouts[0] = 1;
   m_allowedLayouts[1] = 2;
   m_allowedLayouts[2] = 4;
   m_allowedLayouts[3] = 6;
   m_allowedLayouts[4] = 9;

   for(int i=1; i<MAX_CONF_COUNT+1; i++) {
      m_sConfID[i] = m_baseConfID + AsString(i);
      m_sConfPIN[i] = m_baseConfPIN + AsString(i);
   }
}

//*****************************************************************************
// Function: int CSystemConfiguration::Initialize(const int argc, char *const argv[])
// Description: Initialize the configuration from either the command line or 
//              a configuration file.
// 
//    If there are no command line parameters, use the default configuration 
//    file.
// 
//    If a configuration file is specified on the command line, use that as the 
//    source for configuration
// 
//    Otherwise use the command line as the source for configuration.
// 
//    Default parameters are used for non-specified configurations.
// 
// Return: int - 0 for success, otherwise error 
// Parameters: const int argc 
//             char *const argv[] 
//*****************************************************************************
int CSystemConfiguration::Initialize(const int argc, char* const argv[])
{
   string configFile;

   //JM - First Get License Feature Information
   GetLicenseFeatures();

   if (argc == 0)
   {
      // no arguments, assume config file with default name.
      configFile = m_configFileName;
   }
   else if (argc > 0)
   {
      // get command line arguments
      ConfigurationParameterMap cmdLineParameters;

      if (GetCommandLineConfiguration(argc, argv, cmdLineParameters) == -1)
      {
         Usage();
         return -1;
      }

      ConfigurationParameterMap::iterator parmPos;
      if ((parmPos = cmdLineParameters.find("config")) != cmdLineParameters.end())
      {
         // have config file
         configFile = parmPos->second.c_str();
      }
      else
      {
         if (InitializeConfiguration(cmdLineParameters) == -1)
         {
            return -1;
         }
         return 0;
      }
   }

   // configure from file
   LogMsg("***************************************************************");
   LogMsg("Config file:");
   std::ifstream file(configFile.c_str());
   if (!file)
   {
      LogMsg("Configuration File error: could not open " + configFile);
      return -1;
   }
   else
   {
      ConfigurationParameterMap cfgFileParameters;
      char buf[1024];
      while (file.getline(buf, sizeof(buf)))
      {
         if ((buf[0] >= ' ') && (buf[0] != ';'))
         {
            string parm = buf;
            if (parm.size())
            {
               GetConfigItem(parm, cfgFileParameters);
            }
         }
      }
      LogMsg("***************************************************************");
      if (InitializeConfiguration(cfgFileParameters) == -1)
      {
         return -1;
      }
   }
   LogMsg("***************************************************************");
   return 0;
}

void  CSystemConfiguration::printConfIDandConfPIN()
{
   std::map<std::string,std::string>::iterator it;

   std::cout<<"Conference ID\tConference PIN\n****************************************"<<std::endl;
   for(it=m_mapConferenceIDAndPIN.begin();it!=m_mapConferenceIDAndPIN.end();it++) {
      std::cout<< it->first <<"\t\t"<< it->second <<std::endl;
   }
}

//*****************************************************************************
// Function: void CSystemConfiguration::Usage()
// Description: Output program usage information
// Return: void 
// Parameters: none 
//*****************************************************************************
void CSystemConfiguration::Usage()
{
   LogMsg("VideoConfDemo usage:");
   LogMsg("\tconfig=xxx           use configuration file, one line per item");
   LogMsg("\t                     default=don't use configuration file");
   LogMsg("\t                     VideoConfDemo.cfg used if no command line parameters");
   LogMsg("\tmediafilepath=xxx    path to media files, default is current directory");
   LogMsg("\tport=nnn             SIP port, default = 5060");
   LogMsg("\tmaxcalls=nnn         maximum number of calls, default = 5");
   LogMsg("\tmaxconf=nnn          maximum number of conference, default = 5");
   LogMsg("\tvirtualcallers=nnn   number of virtual callers, default = 0, maximim = 8");
   LogMsg("\tregistrar=n.n.n.n    IP address of SIP Registrar, default = 10.10.10.10");
   LogMsg("\tphonenumber=nnnn     phone number to registrer for, default = 1000");
   LogMsg("\tgreeting=xxxx        base filename for greeting/introduction media .pcm and .vid files, default = no greeting clip");
   LogMsg("\tcaller1=xxxx         base filename for 1st virtual caller media .pcm and .vid files, default = caller1");
   LogMsg("\tcaller2=xxxx         base filename for 2nd virtual caller media .pcm and .vid files, default = caller2");
   LogMsg("\tcaller3=xxxx         base filename for 3rd virtual caller media .pcm and .vid files, default = caller3");
   LogMsg("\tcaller4=xxxx         base filename for 4th virtual caller media .pcm and .vid files, default = caller4");
   LogMsg("\tcaller5=xxxx         base filename for 5th virtual caller media .pcm and .vid files, default = caller4");
   LogMsg("\tcaller6=xxxx         base filename for 6th virtual caller media .pcm and .vid files, default = caller4");
   LogMsg("\tcaller7=xxxx         base filename for 7th virtual caller media .pcm and .vid files, default = caller4");
   LogMsg("\tcaller8=xxxx         base filename for 8th virtual caller media .pcm and .vid files, default = caller4");
   LogMsg("\tlayouts=n,n...       allowed layouts - comma delimited, by number of regions 1,4,6 and/or 9, default = 1,4,6,9");
   LogMsg("\tinitiallayout=n      initial layout by number of regions 1,4,6 or 9, default = 1");
   LogMsg("\tdefaultvideorate=n   default video rate in Kbps, default = 384");
   LogMsg("\tactiveTalkerMode=n   active talker mode, 0 = disabled, 1 = enabled, default = 0");
   LogMsg("\tfeaturedelaytime=n   demo feature delay time in sec, 0 = no timed demo feature, default = 0");
   LogMsg("\tdemofeature=n        demo feature, 0 = cycle conference layouts, 1 = PIP-1, 2 = PIP-2, 3 = PIP-3, default = 0");
   LogMsg("\t                         PIP-1 requires virtualcallers=1 and caller1 defined");
   LogMsg("\t                         PIP-2 requires virtualcallers=4 and caller1 through caller4 defined");
   LogMsg("\t                         PIP-3 requires virtualcallers=1 and caller1 defined defined");
   LogMsg("\tconftoneclamp=n      conference wide tone clamping, 0 = disabled, 1 = enabled, default = 0");
   LogMsg("\tpartyechocancel=n    conference party echo cancelation, 0 = disabled, 1 = enabled, default = 0");
}

//*****************************************************************************
// Function: void CSystemConfiguration::GetConfigItem(string &parm, ConfigurationParameterMap &config)
// Description: Get a configuration item from a string and add it to a map of 
//              configuration parameters
// Return: void 
// Parameters: string &parm 
//             ConfigurationParameterMap &config 
//*****************************************************************************
void CSystemConfiguration::GetConfigItem(string& parm, ConfigurationParameterMap& config)
{
   int delim = parm.find("=");
   string key(parm, 0, delim);

   string value;
   if (delim > 0)
   {
      value = parm.substr(delim+1, parm.size()-delim);
   }

   std::transform(key.begin(), key.end(), key.begin(), tolower);

   LogMsg("\t" + key + ":" + value);
   config[key] = value;
}

//*****************************************************************************
// Function: int CSystemConfiguration::GetCommandLineConfiguration(const int argc, char *const argv[], ConfigurationParameterMap &config)
// Description: Get a set of configuration items from command line parameters 
//              and add them to a map of configuration parameters
// Return: int - 0 for success, otherwise error 
// Parameters: const int argc 
//             char *const argv[] 
//             ConfigurationParameterMap &config 
//*****************************************************************************
int CSystemConfiguration::GetCommandLineConfiguration(const int argc, char* const argv[], ConfigurationParameterMap& config)
{
   LogMsg("Command Line:");
   bool needHelp = false;

   for (int i=0; i<argc; i++)
   {
      string parm(argv[i]);
      if (parm.find("=") == string::npos)
      {
         needHelp = true;
      }

      if (parm.find("-help") != -1)
      {
         needHelp = true;
      }

      if (parm.find("--help") != -1)
      {
         needHelp = true;
      }

      GetConfigItem(parm, config);
   }

   if (needHelp)
      return -1;
   else
      return argc;
}

//*****************************************************************************
// Function: int CSystemConfiguration::InitializeConfiguration(ConfigurationParameterMap &configParameters)
// Description: Initialize the configuration from a map of configuration items
// Return: int - 0 for success, otherwise error 
// Parameters: ConfigurationParameterMap &configParameters 
//*****************************************************************************
int CSystemConfiguration::InitializeConfiguration(ConfigurationParameterMap& configParameters)
{
   LogMsg("Initialize Configuration:");
   ConfigurationParameterMap::iterator parmPos;

   if ((parmPos = configParameters.find("mediafilepath")) != configParameters.end())
   {
      m_mediaFilePath = parmPos->second;
      if (m_mediaFilePath[m_mediaFilePath.size()-1] != '/')
      {
         m_mediaFilePath = m_mediaFilePath+"/";  
      }
   }
   else {
		LogMsg("mediaFilePath must be configured");
		return -1;
	}
   LogMsg("\tmediafilepath:" + m_mediaFilePath);
   
   if ((parmPos = configParameters.find("port")) != configParameters.end())
   {
      m_sipPort = atoi(parmPos->second.c_str());
   }
   LogMsg("\tport:" + AsString(m_sipPort));
   
   // DTMF mode
   if ((parmPos = configParameters.find("dtmfmode")) != configParameters.end())
   {
      m_dtmfMode = atoi(parmPos->second.c_str());
      if (m_dtmfMode < DTMFMODE_INBAND || m_dtmfMode > DTMFMODE_SIPINFO)
      {
        m_dtmfMode = DTMFMODE_INBAND;
      }
   }
   LogMsg("\tdtmfMode:" + AsString(m_dtmfMode));

//SKS
   if ((parmPos = configParameters.find("partyperconf")) != configParameters.end()) {
      m_partyPerConf = atoi(parmPos->second.c_str());
   }
   LogMsg("\tpartyperconf:" + AsString(m_partyPerConf));
   if ((parmPos = configParameters.find("confrooms")) != configParameters.end()) {
      m_confRooms=atoi(parmPos->second.c_str());
      if((m_confRooms+1)>MAX_CONF_COUNT) {
         LogMsg("\tMAX_CONF_COUNT=" + AsString(MAX_CONF_COUNT) + " change value of MAX_CONF_COUNT and recompile");
         m_confRooms=MAX_CONF_COUNT;
      }
   }
   LogMsg("\tconfrooms:" + AsString(m_confRooms));

 	if ((parmPos = configParameters.find("confduration")) != configParameters.end()) {
      m_confDuration= atoi(parmPos->second.c_str());
   }
   LogMsg("\tconfduration:" + AsString(m_confDuration));

  if ((parmPos = configParameters.find("pinenabled")) != configParameters.end()) {
      m_bPinEnabled=(atoi(parmPos->second.c_str())==1?true:false);
   }
   LogMsg("\tpinEnabled:" + AsString(m_bPinEnabled==true?"true":"false"));

   if ((parmPos = configParameters.find("baseconfid")) != configParameters.end()) {
      m_baseConfID= parmPos->second;
      for(int i=1; i<MAX_CONF_COUNT+1; i++) {
         m_sConfID[i] = m_baseConfID + AsString(i);
      }     
   }
   LogMsg("\tbaseconfid:" + AsString(m_baseConfID));
  
   if ((parmPos = configParameters.find("baseconfpin")) != configParameters.end()) {
      m_baseConfPIN= parmPos->second;
      for(int i=1; i<MAX_CONF_COUNT+1; i++) {
         m_sConfPIN[i] = m_baseConfPIN + AsString(i);
      }     
   //} else if (m_bPinEnabled==true) {
   //   LogMsg("pinEnabled is set to true, Must set baseConfPIN in config file");
   //   return -1;
   }
   LogMsg("\tbaseconfpin:" + AsString(m_baseConfPIN));
   
   // JM - ConfIDs = name of conference rooms
   for (int i=1; i<m_confRooms+1; i++) {
      if ((parmPos = configParameters.find("confid" + AsString(i))) != configParameters.end())
      {
         m_sConfID[i] = parmPos->second;
         LogMsg("\tconfid" + AsString(i) + ":" + m_sConfID[i]);
      } else if(i<=m_confRooms) {
         LogMsg("\tConfID" + AsString(i) + ":" + m_sConfID[i] + " (default being used)");
      }
      
      // ConfPIN
      if ((parmPos = configParameters.find("confpin" + AsString(i))) != configParameters.end())
      {
         m_sConfPIN[i] = parmPos->second;
         LogMsg("\tconfpin" + AsString(i) + ":" + m_sConfPIN[i]);
      } else if(i<=m_confRooms) {
         LogMsg("\tConfPIN" + AsString(i) + ":" + m_sConfPIN[i] + " default being used");
      }
      //Set the ConfID, ConfPIN map
      m_mapConferenceIDAndPIN[m_sConfID[i]]=m_sConfPIN[i];
   }

   if ((parmPos = configParameters.find("recordconf")) != configParameters.end()) 
   {
      m_bRecordConf=(atoi(parmPos->second.c_str())==1?true:false);
   }
   LogMsg("\trecordConf:" + AsString(m_bRecordConf==true?"true":"false"));
   
   
   if ((parmPos = configParameters.find("recordvidcodec")) != configParameters.end())
   {
      m_sRecordVideoCodec=parmPos->second;
   }
   LogMsg("\trecordvideocodec:" + m_sRecordVideoCodec);

   if ((parmPos = configParameters.find("virtualcallers")) != configParameters.end())
   {
      m_numVirtualCallers = atoi(parmPos->second.c_str());
      if(m_numVirtualCallers<0 || m_numVirtualCallers>8) {
         LogMsg("\tNumber of Virtual caller ranges is 0 to 8");
         return -1;
      }
   }
   LogMsg("\tvirtualcallers:" + AsString(m_numVirtualCallers));

   if ((parmPos = configParameters.find("registrar")) != configParameters.end())
   {
      	m_sipRegistrar = parmPos->second;
   }
   LogMsg("\tregistrar:" + m_sipRegistrar);

   if ((parmPos = configParameters.find("phonenumber")) != configParameters.end())
   {
      m_phoneNumber = parmPos->second;
   }
   LogMsg("\tphonenumber:" + m_phoneNumber);

   if ((parmPos = configParameters.find("greeting")) != configParameters.end())
   {
      m_greetingClip = parmPos->second;
   } else if(m_bPinEnabled==true) {
      LogMsg("\tgreeting prompt name must be configured, if PinEnabled is true");
      return -1;
   }
   LogMsg("\tgreeting:" + m_greetingClip);

   if ((parmPos = configParameters.find("timeoutclip")) != configParameters.end())
   {
      m_timeoutClip = parmPos->second;
   } else if(m_confDuration!=0) {
      LogMsg("\tTimeOut prompt must be set, if confDuration>0");
      return -1;
   }
   LogMsg("\ttimeOutClip:" + m_timeoutClip);

   if ((parmPos = configParameters.find("invalidpinclip")) != configParameters.end())
   {
      m_invalidPinClip = parmPos->second;
   } else if(m_bPinEnabled==true) {
      LogMsg("\tInvalid PIN prompt must be set, if PinEnabled is true");
      return -1;
   }
   LogMsg("\tinvalidPinClip:" + m_invalidPinClip);

   if ((parmPos = configParameters.find("caller1")) != configParameters.end())
   {
      m_virtualCallerClip1 = parmPos->second;
      m_numVirtualCallerClips++;
   } else if(m_numVirtualCallers>=1) {
      LogMsg("\tcaller1 Must be configured if VirtualCallers>=1");
      return -1;
   }
   LogMsg("\tcaller1:" + m_virtualCallerClip1);

   if ((parmPos = configParameters.find("caller2")) != configParameters.end())
   {
      m_virtualCallerClip2 = parmPos->second;
      m_numVirtualCallerClips++;
   } else if(m_numVirtualCallers>=2) {
      LogMsg("\tcaller2 Must be configured if VirtualCallers>=2");
      return -1;
   }
   LogMsg("\tcaller2:" + m_virtualCallerClip2);

   if ((parmPos = configParameters.find("caller3")) != configParameters.end())
   {
      m_virtualCallerClip3 = parmPos->second;
      m_numVirtualCallerClips++;
   } else if(m_numVirtualCallers>=3) {
      LogMsg("\tcaller3 Must be configured if VirtualCallers>=3");
      return -1;
   }
   LogMsg("\tcaller3:" + m_virtualCallerClip3);

   if ((parmPos = configParameters.find("caller4")) != configParameters.end())
   {
      m_virtualCallerClip4 = parmPos->second;
      m_numVirtualCallerClips++;
   } else if(m_numVirtualCallers>=4) {
      LogMsg("\tcaller4 Must be configured if VirtualCallers>=4");
      return -1;
   }
   LogMsg("\tcaller4:" + m_virtualCallerClip4);

   if ((parmPos = configParameters.find("caller5")) != configParameters.end())
   {
      m_virtualCallerClip5 = parmPos->second;
      m_numVirtualCallerClips++;
   } else if(m_numVirtualCallers>=5) {
      LogMsg("\tcaller5 Must be configured if VirtualCallers>=5");
      return -1;
   }
   LogMsg("\tcaller5:" + m_virtualCallerClip5);

   if ((parmPos = configParameters.find("caller6")) != configParameters.end())
   {
      m_virtualCallerClip6 = parmPos->second;
      m_numVirtualCallerClips++;
   } else if(m_numVirtualCallers>=6) {
      LogMsg("\tcaller6 Must be configured if VirtualCallers>=6");
      return -1;
   }
   LogMsg("\tcaller6:" + m_virtualCallerClip6);

   if ((parmPos = configParameters.find("caller7")) != configParameters.end())
   {
      m_virtualCallerClip7 = parmPos->second;
      m_numVirtualCallerClips++;
   } else if(m_numVirtualCallers>=7) {
      LogMsg("\tcaller7 Must be configured if VirtualCallers>=7");
      return -1;
   }
   LogMsg("\tcaller7:" + m_virtualCallerClip7);

   if ((parmPos = configParameters.find("caller8")) != configParameters.end())
   {
      m_virtualCallerClip8 = parmPos->second;
      m_numVirtualCallerClips++;
   } else if(m_numVirtualCallers==8) {
      LogMsg("\tcaller8 Must be configured if VirtualCallers=8");
      return -1;
   }
   LogMsg("\tcaller8:" + m_virtualCallerClip8);

   if ((parmPos = configParameters.find("layouts")) != configParameters.end())
   {
      std::string l_layouts = parmPos->second;
      m_allowedLayoutsCount = sscanf(   l_layouts.c_str(),
                                        "%d,%d,%d,%d,%d",
                                        &m_allowedLayouts[0],
                                        &m_allowedLayouts[1],
                                        &m_allowedLayouts[2],
                                        &m_allowedLayouts[3],
                                        &m_allowedLayouts[4]);
   }
   {
      std::string l_layouts;
      for (int i=0; i < m_allowedLayoutsCount; i++)
      {
         l_layouts = l_layouts + " " + AsString(m_allowedLayouts[i]);
      }
      LogMsg("\tlayouts:" + l_layouts);
   }

   if ((parmPos = configParameters.find("initiallayout")) != configParameters.end())
   {
      m_initialLayout = atoi(parmPos->second.c_str());
   }
   LogMsg("\tinitiallayout:" + AsString(m_initialLayout));
   
   if ((parmPos = configParameters.find("defaultvideorate")) != configParameters.end())
   {
      m_defaultVideoRate = atoi(parmPos->second.c_str());
   }
   LogMsg("\tdefaultvideorate:" + AsString(m_defaultVideoRate));

   if ((parmPos = configParameters.find("activetalkermode")) != configParameters.end())
   {
      m_activeTalkerMode = atoi(parmPos->second.c_str());
   }
   LogMsg("\tactivetalkermode:" + AsString(m_activeTalkerMode));

   if ((parmPos = configParameters.find("featuredelaytime")) != configParameters.end())
   {
      m_featureDelayTime = atoi(parmPos->second.c_str());
      if (m_featureDelayTime < 0)
      {
         m_featureDelayTime = 0;
      }
   }
   LogMsg("\tfeaturedelaytime:" + AsString(m_featureDelayTime));

   if ((parmPos = configParameters.find("demofeature")) != configParameters.end())
   {
      m_demoFeature = atoi(parmPos->second.c_str());
      if (m_demoFeature < 0)
      {
         m_demoFeature = 0;
      }
   }
   LogMsg("\tdemofeature:" + AsString(m_demoFeature));

   if ((parmPos = configParameters.find("conftoneclamp")) != configParameters.end())
   {
      m_confToneClamping = atoi(parmPos->second.c_str());
      if (m_confToneClamping != 1)
      {
         m_confToneClamping = 0;
      }
   }
   LogMsg("\tconftoneclamp:" + AsString(m_confToneClamping));

   if ((parmPos = configParameters.find("partyechocancel")) != configParameters.end())
   {
      m_confPartyEchoCancelation = atoi(parmPos->second.c_str());
      if (m_confPartyEchoCancelation != 1)
      {
         m_confPartyEchoCancelation = 0;
      }
   }
   LogMsg("\tpartyechocancel:" + AsString(m_confPartyEchoCancelation));

   char l_hostName[256];
   if ((gethostname(l_hostName, 256)) < 0)
   {
      ErrMsg(0, "gethostname failed");
   }
   else
   {
      LogMsg("\thostname:" + AsString(l_hostName));
   }

   char l_hostIpAddress[256];
   struct hostent* l_hostEntry;
   l_hostEntry = gethostbyname(l_hostName);
   struct in_addr l_inAddr;
   char** l_p;
   for (l_p = l_hostEntry->h_addr_list; *l_p != 0; l_p++)
   {
      (void) memcpy(&l_inAddr.s_addr, *l_p, sizeof (l_inAddr.s_addr));
      m_localIpAddress = inet_ntoa(l_inAddr);
      LogMsg("\tlocalipaddr:" + AsString(m_localIpAddress));
      (void) memcpy(&m_localIPinByteOrder, *l_p, sizeof (l_inAddr.s_addr));
   }
   return 0;
}

//*****************************************************************************
// Function: int CSystemConfiguration::GetLicenseFeatures()
// Description: Get and Display the License File Features
// Return: int - 0 for success, otherwise error 
// Parameters: 
//*****************************************************************************
int CSystemConfiguration::GetLicenseFeatures()
{
   LogMsg("***************************************************************");
   LogMsg("License Features:");
  
   unsigned int l_result;
   int l_iptDeviceCount = 0;
   unsigned long lic_msk;
   LIC_FEATURE_BLK lic_data;
   INIT_LICENSED_FEATURES_VERSION(&lic_data);

   if ( (l_result = dev_GetLicFeatureData(&lic_data)) != 0)
   {
       LogMsg( "\tERROR: dev_getLicFeatureData() failed. Could not get licensed feature information");
       return -1;
   }
   else
   {
      for (unsigned int i = 0; i < lic_data.NumFeatures; i++)
      {
         LogMsg( "\tLicense Feature Name:" + AsString(FeatureNameTable[lic_data.lic_data_buf[i].FeatureEnum]) + 
                  ":" + AsString(lic_data.lic_data_buf[i].FeatureCount));
         if( 0 == strcmp(FeatureNameTable[lic_data.lic_data_buf[i].FeatureEnum], LM_FEATURE_NAME_IPCC))
         {
            //LogMsg("\tNumber of 3PCC IPCall Control Licenses:" + AsString(lic_data.lic_data_buf[i].FeatureCount));
            l_iptDeviceCount =  lic_data.lic_data_buf[i].FeatureCount;
         }
      }
   }

   //JM: TODO: Use License counts to gracefully start App.
   //return l_iptDeviceCount;

   LogMsg("***************************************************************");
   return 0;
}



//*****************************************************************************
// Function: eMM_VIDEO_CODING CSystemConfiguration::getRecordVideoCodec()
// Description: Get Record Video Codec
// Return: eMM_VIDEO_CODING 
// Parameters: 
//*****************************************************************************
eMM_VIDEO_CODING CSystemConfiguration::getRecordVideoCodec()
{
   //Get Record Codec
   if(!strcmp(m_sRecordVideoCodec.c_str(),"h264") || !strcmp(m_sRecordVideoCodec.c_str(),"H264")){
      return VIDEO_CODING_H264;
   }
   else {
      return VIDEO_CODING_H263;
   }

}
