/**
* @file CSystemConfiguration.h
* @brief System Configuration class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CSystemConfiguration_h
#define CSystemConfiguration_h

#include <string>
#include <map>

#include "LicensedFeatures.h"
#include "devmgmt.h"
#include "videodefs.h"

#ifdef WIN32
#else
   #include <sys/types.h>
   #include <netdb.h>
   #include <arpa/inet.h>
#endif

#define MAX_ALLOWED_LAYOUT_CONFIG   5
#define MAX_CONF_COUNT   				10
#define MAX_CONF_DURATION				0
#define BASE_CONF_ROOM_ID				3542	// DLGC
#define BASE_CONF_PIN					3542	// DLGC
#define MAX_PARTY_PER_CONF				30
#define MAX_STR_CONF_ROOM_ID		32


#define DTMFMODE_INBAND 0
#define DTMFMODE_RFC2833 1
#define DTMFMODE_SIPINFO 2

typedef std::map<std::string, std::string> ConfigurationParameterMap;

extern const char* CFG_FILENAME;

//*****************************************************************************
// Class: ::CSystemConfiguration
// Description: Class used to obtain, store and access the set of application 
//              configuration parameters
//*****************************************************************************
class CSystemConfiguration
{
public:
   CSystemConfiguration();
   virtual ~CSystemConfiguration() {}

   int 	Initialize(const int argc, char* const argv[]);
   int 	SipPort() {return m_sipPort;}
   int 	DtmfMode() {return m_dtmfMode;}
   int 	getNoOfPartyPerConf() {return m_partyPerConf;}
   int 	getNoOfConfRooms()  {return m_confRooms;}

   const std::string& getBaseConfID() {return m_baseConfID;}
   const std::string& getBaseConfPIN()  {return m_baseConfPIN;}
   const std::string& getConfID(int confID) {return m_sConfID[confID];}
   const std::string& getConfPIN(int confID) {return m_sConfPIN[confID];}
   const std::string& getConfPIN(std::string confID) {return m_mapConferenceIDAndPIN[confID];}
   void SetConfIDandPIN(std::string confID,std::string confPIN){m_mapConferenceIDAndPIN[confID]=confPIN;}
   void printConfIDandConfPIN();
   int 	getConfDuration() {return m_confDuration;}
   int 	getMaxCalls() {return (m_partyPerConf*m_confRooms);}
   bool isRecordConferenceEnabled(){return m_bRecordConf;}
   eMM_VIDEO_CODING getRecordVideoCodec();
   bool	isPinEnabled(){return m_bPinEnabled;}
   const std::string& InvalidPinClip(){return m_invalidPinClip;}
   const std::string& TimeOutClip(){return m_timeoutClip;}

   int 	NumVirtualCallers() {return m_numVirtualCallers;}
   int 	NumVirtualCallerClips() {return m_numVirtualCallerClips;}
   const std::string& SipRegistrar() {return m_sipRegistrar;}
   const std::string& PhoneNumber() {return m_phoneNumber;}
   const std::string& GreetingClip() {return m_greetingClip;}
   const std::string& VirtualCallerClip1() {return m_virtualCallerClip1;}
   const std::string& VirtualCallerClip2() {return m_virtualCallerClip2;}
   const std::string& VirtualCallerClip3() {return m_virtualCallerClip3;}
   const std::string& VirtualCallerClip4() {return m_virtualCallerClip4;}
   const std::string& VirtualCallerClip5() {return m_virtualCallerClip5;}
   const std::string& VirtualCallerClip6() {return m_virtualCallerClip6;}
   const std::string& VirtualCallerClip7() {return m_virtualCallerClip7;}
   const std::string& VirtualCallerClip8() {return m_virtualCallerClip8;}
   const std::string& MediaFilePath() {return m_mediaFilePath;}

   int* AllowedLayouts() {return m_allowedLayouts;}
   int AllowedLayoutsCount() {return m_allowedLayoutsCount;}
   int InitialLayout() {return m_initialLayout;}
   int DefaultVideoRate() {return m_defaultVideoRate;}
   int ConferenceToneClamping() {return m_confToneClamping;}
   int ConferencePartyEchoCancelation() {return m_confPartyEchoCancelation;}
 
#ifdef WIN32
   unsigned long LocalIPinByteOrder() {return m_localIPinByteOrder;}
#else
   in_addr_t LocalIPinByteOrder() {return m_localIPinByteOrder;}
#endif

   const std::string& LocalIpAddress() {return m_localIpAddress;}

   int ActiveTalkerMode() 	{return m_activeTalkerMode;}
   int FeatureDelayTime() 	{return m_featureDelayTime;}
   int DemoFeature() 		{return m_demoFeature;}

private:
   void Usage();
   void GetConfigItem(std::string& parm, ConfigurationParameterMap& config);
   int GetCommandLineConfiguration(const int argc, char* const argv[], ConfigurationParameterMap& config);
   int InitializeConfiguration(ConfigurationParameterMap& configParameters);
   int GetLicenseFeatures();
   
   std::string m_configFileName;
   std::string m_mediaFilePath;
   int m_sipPort;
   int m_partyPerConf;
   int m_confRooms;
   int m_confDuration;
   std::string m_baseConfID;
   std::string m_baseConfPIN;
   std::string m_sConfID[MAX_CONF_COUNT+1];
   std::string m_sConfPIN[MAX_CONF_COUNT+1];
   bool m_bRecordConf;
   std::string m_sRecordVideoCodec;
   bool m_bPinEnabled;

   int m_numVirtualCallers;
   int m_numVirtualCallerClips;

   // SKS
   std::string m_invalidPinClip;
   std::string m_timeoutClip;

   std::string m_sipRegistrar;
   std::string m_phoneNumber;
   std::string m_localIpAddress;
   std::string m_greetingClip;
   std::string m_virtualCallerClip1;
   std::string m_virtualCallerClip2;
   std::string m_virtualCallerClip3;
   std::string m_virtualCallerClip4;
   std::string m_virtualCallerClip5;
   std::string m_virtualCallerClip6;
   std::string m_virtualCallerClip7;
   std::string m_virtualCallerClip8;
   int m_dtmfMode;
   int m_initialLayout;
   int m_allowedLayoutsCount;
   int m_allowedLayouts[MAX_ALLOWED_LAYOUT_CONFIG];
   int m_defaultVideoRate;
   int m_activeTalkerMode;
   int m_featureDelayTime;
   int m_demoFeature;
   int m_confToneClamping;
   int m_confPartyEchoCancelation;
	std::map<std::string, std::string > m_mapConferenceIDAndPIN;

#ifdef WIN32
   unsigned long m_localIPinByteOrder;
#else
   in_addr_t m_localIPinByteOrder;
#endif

};

#endif


