/**
* @file CVirtualCaller.cpp
* @brief Virtual Caller class implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "config.h"

#include "devmgmt.h"

#include "LogUtil.h"
#include "CVirtualCaller.h"
#include "CVirtualCallerState.h"
#include "CMediaDevice.h"
#include "CConference.h"
#include "CConferencePartyDevice.h"
#include "CGcManager.h"

#include "CSystemConfiguration.h"
#include "CIpmDevice.h"

#include "CConnection.h"

int CVirtualCaller::VC_RX_PORT_INFO = 0x0001;
int CVirtualCaller::VC_TX_PORT_INFO = 0x0002;
int CVirtualCaller::CNF_RX_PORT_INFO = 0x0004;
int CVirtualCaller::CNF_TX_PORT_INFO = 0x0008;
int CVirtualCaller::PORT_INFO_COMPLETE = 0x000F;

int CVirtualCaller::VC_TX_CNF_RX_CONNECT = 0x0011;
int CVirtualCaller::CNF_TX_VC_RX_CONNECT = 0x0012;
int CVirtualCaller::PORT_CONNECT_COMPLETE = 0x0013;

int CVirtualCaller::VC_TX_CNF_RX_DISCONNECT = 0x0021;
int CVirtualCaller::CNF_TX_VC_RX_DISCONNECT = 0x0022;
int CVirtualCaller::PORT_DISCONNECT_COMPLETE = 0x0023;


#ifdef USE_RTSP
/*
* Callback for RtspConnector events.
* Forward the event to its target device (virtual caller) via the app event.
*/
void rtspconnector_event_callback( RtspEvent* event )
{
	//printf("rtspconnector_event_callback!\n");

    CVirtualCaller* pVirtCaller = reinterpret_cast<CVirtualCaller*>(event->userdata);
	
    if (pVirtCaller != NULL){
        pVirtCaller->SetRtspEventData(event);

	pVirtCaller->PostEvent(APPEVENT_RTSP_EVENT);
    }
    return;
	
}
#endif

//*****************************************************************************
// Function: CVirtualCaller::CVirtualCaller(const std::string &a_name,std::string confID)
// Description: Initializing constructor
// Return:  
// Parameters: const std::string &a_name 
//*****************************************************************************
CVirtualCaller::CVirtualCaller(const std::string& a_name,std::string confID, int vCallerNum)
   : m_state(AVirtualCallerState::State(VirtualCallerIdleState))
   , m_name(a_name)
   , m_confID(confID)
   , m_conferenceParty(0)
   , m_portInfoSubState(0)
   , m_portConnectSubState(0)
   , m_portDisconnectSubState(0)
   , m_stopPlay(false)
   , m_vCallerNum(vCallerNum)
   , m_mmDevice(0)
   , m_bIsMediaRTSP(false)
   , m_ipmDevice(0)
{
	SetMediaClip();
#ifdef USE_RTSP
	if (m_bIsMediaRTSP)
		OpenRTSPClient();
	else
#endif
	    OpenMediaDevice();
}

void CVirtualCaller::SetMediaClip()
{
      CSystemConfiguration* m_systemConfiguration=CConference::SystemConfiguration();
      switch ( ((m_vCallerNum-1) % m_systemConfiguration->NumVirtualCallerClips())+1)
      {
         case 1:
            m_vCallerMediaClip = m_systemConfiguration->VirtualCallerClip1();
            break;
         case 2:
            m_vCallerMediaClip = m_systemConfiguration->VirtualCallerClip2();
            break;
         case 3:
            m_vCallerMediaClip = m_systemConfiguration->VirtualCallerClip3();
            break;
         case 4:
            m_vCallerMediaClip = m_systemConfiguration->VirtualCallerClip4();
            break;
         case 5:
            m_vCallerMediaClip = m_systemConfiguration->VirtualCallerClip5();
            break;
         case 6:
            m_vCallerMediaClip = m_systemConfiguration->VirtualCallerClip6();
            break;
         case 7:
            m_vCallerMediaClip = m_systemConfiguration->VirtualCallerClip7();
            break;
         case 8:
            m_vCallerMediaClip = m_systemConfiguration->VirtualCallerClip8();
            break;
         default:
            m_vCallerMediaClip = m_systemConfiguration->VirtualCallerClip1();
      }
	  LogMsg(Name(),"m_vCallerNum=" + AsString(m_vCallerNum) + "; numVirtCallerClips=" + AsString(m_systemConfiguration->NumVirtualCallerClips()) );
	  LogMsg(Name(),"m_vCallerMediaClip=" + m_vCallerMediaClip);

	  if (m_vCallerMediaClip.compare(0,7,"rtsp://") == 0)
	  {
#ifdef USE_RTSP
		  m_bIsMediaRTSP = true;
		  LogMsg(Name(),"Media is RTSP Url. using RTSPConnector");
#else
		  m_bIsMediaRTSP = false;
		  LogMsg(Name(),"ERROR: Media is RTSP Url. MUST Compile with USE_RTSP flag to stream RTSP media");
                  
#endif
	  }
	  else
		  m_bIsMediaRTSP = false;
}

void CVirtualCaller::OpenMediaDevice()
{
	CSystemConfiguration* m_systemConfiguration=CConference::SystemConfiguration();

      m_mmDevice = new CMediaDevice(CMediaDevice::GetNextMediaDeviceName());
      CGcManager::RegisterDevice(m_mmDevice);
	  m_mmDevice->Observer(this);
      m_mmDevice->Open();
	  m_mmDevice->SetClip(m_systemConfiguration->MediaFilePath(), m_vCallerMediaClip);
}

#ifdef USE_RTSP
void CVirtualCaller::OpenRTSPClient()
{
   CSystemConfiguration* m_systemConfiguration=CConference::SystemConfiguration();

   //Open the ipm device
   m_ipmDevice = new CIpmDevice(CIpmDevice::GetNextIpmDeviceName());
   m_ipmDevice->SetDtmfMode(DTMFMODE_RFC2833); 
   m_ipmDevice->Observer(this);
   m_ipmDevice->Open();
   CGcManager::RegisterDevice(m_ipmDevice);

}
#endif

//*****************************************************************************
// Function: void CVirtualCaller::SetStateTo(AVirtualCallerState *state)
// Description: Set the virtual caller state
// Return: void 
// Parameters: AVirtualCallerState *state 
//*****************************************************************************
void CVirtualCaller::SetStateTo(AVirtualCallerState* state)
{
   m_state = state;
   LogMsg(Name(), AsString("Virtual Caller State:") + m_state->Desc());
}

//*****************************************************************************
// All session operations and event handling are delegated to the state
// object with the following functions
//*****************************************************************************

int CVirtualCaller::StartVirtualCaller() {return State().StartVirtualCaller(*this);}
int CVirtualCaller::OpenConferenceParty() {return State().OpenConferenceParty(*this);}
int CVirtualCaller::OnOpenConferencePartyComplete() {return State().OnOpenConferencePartyComplete(*this);}
int CVirtualCaller::ConnectToConferenceParty() {return State().ConnectToConferenceParty(*this);}
int CVirtualCaller::OnConnectToConferencePartyComplete() {return State().OnConnectToConferencePartyComplete(*this);}
int CVirtualCaller::JoinConference() {return State().JoinConference(*this);}
int CVirtualCaller::OnJoinConferenceComplete() {return State().OnJoinConferenceComplete(*this);}
int CVirtualCaller::LeaveConference() {return State().LeaveConference(*this);}
int CVirtualCaller::OnLeaveConferenceComplete() {return State().OnLeaveConferenceComplete(*this);}
int CVirtualCaller::OnLeaveConferenceFailed() {return State().OnLeaveConferenceFailed(*this);}
int CVirtualCaller::DisconnectFromConferenceParty() {return State().DisconnectFromConferenceParty(*this);}
int CVirtualCaller::OnDisconnectFromConferencePartyComplete() {return State().OnDisconnectFromConferencePartyComplete(*this);}
int CVirtualCaller::CloseConferenceParty() {return State().CloseConferenceParty(*this);}
int CVirtualCaller::OnCloseConferencePartyComplete() {return State().OnCloseConferencePartyComplete(*this);}
int CVirtualCaller::OnGetVcRxPortInfoComplete() {return State().OnGetVcRxPortInfoComplete(*this);}
int CVirtualCaller::OnGetVcTxPortInfoComplete() {return State().OnGetVcTxPortInfoComplete(*this);}
int CVirtualCaller::OnGetCnfRxPortInfoComplete() {return State().OnGetCnfRxPortInfoComplete(*this);}
int CVirtualCaller::OnGetCnfTxPortInfoComplete() {return State().OnGetCnfTxPortInfoComplete(*this);}
int CVirtualCaller::OnCnfPortConnected() {return State().OnCnfPortConnected(*this);}
int CVirtualCaller::OnVcPortConnected() {return State().OnVcPortConnected(*this);}
int CVirtualCaller::OnCnfPortDisconnected() {return State().OnCnfPortDisconnected(*this);}
int CVirtualCaller::OnVcPortDisconnected() {return State().OnVcPortDisconnected(*this);}
int CVirtualCaller::OnStartMediaComplete() {return State().OnStartMediaComplete(*this);}
int CVirtualCaller::OnStopMediaComplete() {return State().OnStopMediaComplete(*this);}

int CVirtualCaller::Play() {return State().Play(*this);}
int CVirtualCaller::OnPlayComplete() {return State().OnPlayComplete(*this);}
int CVirtualCaller::Stop() {return State().Stop(*this);}

//*****************************************************************************
// Function: int CVirtualCaller::PrimStartVirtualCaller()
// Description: Handle a start virtual caller command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimStartVirtualCaller()
{
   int res(0);

   if (m_bIsMediaRTSP)
   {
	   if(!m_ipmDevice)
	   {
		   res=-1;
	   }
   }
   else
   {
      if (!m_mmDevice)
      {
         res = -1;
      }
   }

   return res;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOpenConferenceParty()
// Description: Handle an open conference party command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOpenConferenceParty()
{
   m_conferenceParty = new CConferencePartyDevice(Name(), CConference::GetConferenceBoard(), this);
   CGcManager::RegisterDevice(m_conferenceParty);
   m_conferenceParty->SetVirtualParty(this);
   return m_conferenceParty->Open();
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnOpenConferencePartyComplete()
// Description: Handle an open conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnOpenConferencePartyComplete()
{
   CGcManager::RegisterDevice(m_conferenceParty->Device(), m_conferenceParty);
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimConnectToConferenceParty()
// Description: Handle a connect to conference party command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimConnectToConferenceParty()
{
   m_portInfoSubState = 0;

   if (m_bIsMediaRTSP)
   {
      m_ipmDevice->GetReceivePortInfo();
      m_ipmDevice->GetTransmitPortInfo();
	  m_ipmDevice->IpmGetLocalMediaInfo();
   }
   else
   {
      m_mmDevice->GetReceivePortInfo();
      m_mmDevice->GetTransmitPortInfo();
   }
   m_conferenceParty->GetReceivePortInfo();
   m_conferenceParty->GetTransmitPortInfo();
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnConnectToConferencePartyComplete()
// Description: Handle a connect to conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnConnectToConferencePartyComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimJoinConference()
// Description: Handle join conference command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimJoinConference()
{
// SKS
   CConference* l_conference = CConference::GetConference(m_confID);
   return l_conference->AddConferenceParty(m_conferenceParty);
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnJoinConferenceComplete()
// Description: Handle join conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnJoinConferenceComplete()
{
   LogMsg(Name(),"PrimOnJoinConferenceComplete");
// SKS
   return CConference::GetConference(m_confID)->OnConferencePartyAdded(m_conferenceParty);
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimLeaveConference()
// Description: Handle a leave conference command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimLeaveConference()
{
   CConference* l_conference = CConference::GetConference(m_confID);
   return l_conference->RemoveConferenceParty(m_conferenceParty);
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnLeaveConferenceComplete()
// Description: Handle a leave conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnLeaveConferenceComplete()
{
   return CConference::GetConference(m_confID)->OnConferencePartyRemoved(m_conferenceParty);
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimDisconnectFromConferenceParty()
// Description: Handle a disconnect from conference party command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimDisconnectFromConferenceParty()
{
   int res=-1;
   m_portDisconnectSubState = 0;
   
   // PortDisconnect Vc --> Cnf
   if (m_bIsMediaRTSP)
   {
	   res = dev_PortDisconnect(m_ipmDevice->Device(), &m_portConnectInfoListVcTxCnfRx, NULL);
	   if (-1 == res)
       {
          ErrMsg( m_ipmDevice->Device(), "dev_PortDisconnect(IPM)");
          return res;
       }
	   LogMsg( Name(), "dev_PortDisconnect(IPM)");
   }
   else
   {
	   res = dev_PortDisconnect(m_mmDevice->Device(), &m_portConnectInfoListVcTxCnfRx, NULL);
	   if (-1 == res)
       {
          ErrMsg( m_mmDevice->Device(), "dev_PortDisconnect(MM)");
          return res;
       }
	   LogMsg( Name(), "dev_PortDisconnect(MM)");
   }

   // PortDisconnect Cnf --> Vc
   res = dev_PortDisconnect(m_conferenceParty->Device(), &m_portConnectInfoListCnfTxVcRx, NULL);
   if (-1 == res)
   {
	   ErrMsg( m_conferenceParty->Device(), "dev_PortDisconnect(CNF)");
	   return res;
   }
	   LogMsg( Name(), "dev_PortDisconnect(CNF)");
   
   return res;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnDisconnectFromConferencePartyComplete()
// Description: Handle a disconnect from conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnDisconnectFromConferencePartyComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimCloseConferenceParty()
// Description: Handle a close conference party command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimCloseConferenceParty()
{
   return m_conferenceParty->Close();
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnCloseConferencePartyComplete()
// Description: Handle a close conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnCloseConferencePartyComplete()
{
// TODO: fix conference party cleanup
//      CGcManager::UnregisterDevice(m_conferenceParty->ID());
//      CGcManager::UnregisterDevice(m_conferenceParty->CnfPartyDevice());
//      delete m_conferenceParty;
//      m_conferenceParty = 0;
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCaller::CreatePortConnectInfoList(DM_PORT_CONNECT_INFO_LIST &a_portConnectionInfoList, const DM_PORT_INFO_LIST &a_txPortInfoList, const DM_PORT_INFO_LIST &a_rxPortInfoList)
// Description: Create a port connedct info list
// Return: int - number of items in the port connection info list
// Parameters: DM_PORT_CONNECT_INFO_LIST &a_portConnectionInfoList 
//             const DM_PORT_INFO_LIST &a_txPortInfoList 
//             const DM_PORT_INFO_LIST &a_rxPortInfoList 
//*****************************************************************************
int CVirtualCaller::CreatePortConnectInfoList(  DM_PORT_CONNECT_INFO_LIST& a_portConnectionInfoList,
                                                const DM_PORT_INFO_LIST& a_txPortInfoList,
                                                const DM_PORT_INFO_LIST& a_rxPortInfoList)
{
   INIT_DM_PORT_CONNECT_INFO_LIST(&a_portConnectionInfoList);

   // Loop through all transmit ports of 1st device
   int k = 0;
   int i;
   for (i = 0; i < a_txPortInfoList.unCount; ++i)
   {
      DM_PORT_MEDIA_TYPE type_tx = a_txPortInfoList.port_info[i].port_media_type;
      if (type_tx == DM_PORT_MEDIA_TYPE_NBUP)
      {
         continue;
      }

      // find appropriate RX port on 2nd device
      bool bFound = false;
      int j;
      for (j = 0; j < a_rxPortInfoList.unCount; ++j)
      {
         DM_PORT_MEDIA_TYPE type_rx = a_rxPortInfoList.port_info[j].port_media_type;
         if (type_tx == type_rx)
         {
            bFound = true;
            break;
         }
      }
      if (!bFound)
      {
         continue;
      }
      // create element of connect list
      DM_PORT_CONNECT_INFO& info = a_portConnectionInfoList.port_connect_info[k];
      INIT_DM_PORT_CONNECT_INFO(&info);

      info.unFlags = DMFL_TRANSCODE_ON;
      info.port_info_tx = a_txPortInfoList.port_info[i];
      info.port_info_rx = a_rxPortInfoList.port_info[j];
      ++k;
   }
   a_portConnectionInfoList.unCount = k;
   return k;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnGetVcRxPortInfoComplete()
// Description: Handle a get RX port info from the VC (mm or ipm) device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnGetVcRxPortInfoComplete()
{
   m_portInfoSubState |= VC_RX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnGetVcTxPortInfoComplete()
// Description: Handle a get TX port info from the VC (mm or ipm) device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnGetVcTxPortInfoComplete()
{
   m_portInfoSubState |= VC_TX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnGetCnfRxPortInfoComplete()
// Description: Handle a get RX port info from the CNF party device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnGetCnfRxPortInfoComplete()
{
   m_portInfoSubState |= CNF_RX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnGetCnfTxPortInfoComplete()
// Description: Handle a get TX port info from the CNF party device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnGetCnfTxPortInfoComplete()
{
   m_portInfoSubState |= CNF_TX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimPortConnect()
// Description: Handle a port connect command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimPortConnect()
{
   int num_matched_ports = 0;
   int res = -1;
   m_portConnectSubState = 0;
   
   // Create Vc --> Cnf PortInfoList
   if (m_bIsMediaRTSP)
   {
	   num_matched_ports = CreatePortConnectInfoList( m_portConnectInfoListVcTxCnfRx,
                                                      m_ipmDevice->TxPortInfoList(),
													  m_conferenceParty->RxPortInfoList());
   }
   else
   {
      num_matched_ports = CreatePortConnectInfoList( m_portConnectInfoListVcTxCnfRx,
                                                      m_mmDevice->TxPortInfoList(),
													  m_conferenceParty->RxPortInfoList());
   }
   if (0 == num_matched_ports)
   {
      LogMsg( Name(), "No matched ports found");
      return -1;
   }

   // PortConnect Vc --> Cnf
   if (m_bIsMediaRTSP)
   {
	  res = dev_PortConnect(m_ipmDevice->Device(), &m_portConnectInfoListVcTxCnfRx, NULL);
	  if (-1 == res)
      {
         ErrMsg( m_ipmDevice->Device(), "dev_PortConnect");
         return res;
      }
   }
   else
   {
      res = dev_PortConnect(m_mmDevice->Device(), &m_portConnectInfoListVcTxCnfRx, NULL);
	  if (-1 == res)
      {
         ErrMsg( m_mmDevice->Device(), "dev_PortConnect");
         return res;
      }
   }
   LogMsg( Name(), "VirtualCallerTx dev_PortConnect");

   // Create Cnf --> Vc PortInfoList
   if (m_bIsMediaRTSP)
   {
	   num_matched_ports = CreatePortConnectInfoList( m_portConnectInfoListCnfTxVcRx,
                                                  m_conferenceParty->TxPortInfoList(),
                                                  m_ipmDevice->RxPortInfoList());
   }
   else
   {
      num_matched_ports = CreatePortConnectInfoList( m_portConnectInfoListCnfTxVcRx,
                                                  m_conferenceParty->TxPortInfoList(),
                                                  m_mmDevice->RxPortInfoList());
   }

   if (0 == num_matched_ports)
   {
      LogMsg( Name(), "No matched ports found");
      return -1;
   }
   
   // PortConnect Cnf --> Vc
   res = dev_PortConnect(m_conferenceParty->Device(), &m_portConnectInfoListCnfTxVcRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_conferenceParty->Device(), "dev_PortConnect");
      return res;
   }
   else
   {
      LogMsg( Name(), "VirtualCallerRx dev_PortConnect");
   }

   return res;
}


//*****************************************************************************
// Function: int CVirtualCaller::PrimOnCnfPortConnected()
// Description: Handle a port connected event from the CNF device
// Return: int - value of port connect substate
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnCnfPortConnected()
{
   m_portConnectSubState |= CNF_TX_VC_RX_CONNECT;
   return m_portConnectSubState;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnVcPortConnected()
// Description: Handle a port connected event from the VC (mm or ipm) device
// Return: int - value of port connect substate 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnVcPortConnected()
{
   m_portConnectSubState |= VC_TX_CNF_RX_CONNECT;
   return m_portConnectSubState;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnCnfPortDisconnected()
// Description: Handle a port disconnected event from the CNF device
// Return: int - value of port disconnect substate 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnCnfPortDisconnected()
{
   m_portDisconnectSubState |= CNF_TX_VC_RX_DISCONNECT;
   return m_portDisconnectSubState;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnVcPortDisconnected()
// Description: Handle a port disconnected event from the VC device
// Return: int - value of port disconnect substate 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnVcPortDisconnected()
{
   m_portDisconnectSubState |= VC_TX_CNF_RX_DISCONNECT;
   return m_portDisconnectSubState;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimPlay()
// Description: Handle a play command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimPlay()
{
   m_stopPlay = false;
   if (m_bIsMediaRTSP)
   {
#ifdef USE_RTSP
      return PlayRTSPFile();
#endif
   }
   else
   {
      return m_mmDevice->MmPlay();
   }
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimOnPlayComplete()
// Description: Handle a play completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimOnPlayComplete()
{
   return(m_stopPlay!=true);
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimStop()
// Description: Handle a stop media operation command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimStop()
{
   m_stopPlay = true;
   if (m_bIsMediaRTSP)
   {
#ifdef USE_RTSP
	   CConference::getRtspConnector().stop(m_rtspIpmInfo.session_id);
	   return m_ipmDevice->IpmStopMedia();
#endif
   }
   else
   {
      return m_mmDevice->MmStop();
   }
}

#ifdef USE_RTSP
//*****************************************************************************
// Function: int CVirtualCaller::PlayRTSPFile()
// Description: Handle a play rtsp command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PlayRTSPFile()
{
   int res = -1;
   
   LogMsg(Name(), "CreateRTSPSession(" + m_vCallerMediaClip + ")");

   /*
   * Extract and strip username and password from the uri string to avoid
   * passing authentication information in clear text to the rtsp server.
   * "rtsp://" username ":" password "@" host ...
   */
   
   std::string l_username;
   std::string l_password;
   std::string l_uri = m_vCallerMediaClip;
   int l_localAudioPort = m_ipmDevice->LocalAudioPortInfo().unPortId;
   int l_localVideoPort = m_ipmDevice->LocalVideoPortInfo().unPortId;

   std::string::size_type at = l_uri.find("@");
   if ( at != std::string::npos )
   {
	   std::string::size_type colon = l_uri.find(":", 7);
	   if ( colon != std::string::npos )
	   {
		   l_username = l_uri.substr(7, colon - 7);
		   l_password = l_uri.substr(colon + 1, at - colon - 1);
	   }
	   l_uri.erase(7, at + 1 - 7);
   }

   // RTSP Connector Open
   LogMsg(Name(), "RTSP url is: " + l_uri);
   LogMsg(Name(), "RTSP username:" + l_username);
   LogMsg(Name(), "RTSP password:" + l_password);
   LogMsg(Name(), "RTSP Local Audio Client port:" + AsString(l_localAudioPort));
   LogMsg(Name(), "RTSP Local Video Client port:" + AsString(l_localVideoPort));

   //Set local variables or rtspIpmInfo struct
   m_rtspIpmInfo.localIpmAudioPort = l_localAudioPort;
   m_rtspIpmInfo.localIpmVideoPort = l_localVideoPort;
   m_rtspIpmInfo.localIpmIPAddr = m_ipmDevice->LocalAudioPortInfo().cIPAddress;
   LogMsg(Name(), "RTSP Local Ipm Address:" + AsString(m_rtspIpmInfo.localIpmIPAddr));
  
   CConference::getRtspConnector().open(l_uri, l_username, l_password, l_localAudioPort,l_localVideoPort, reinterpret_cast<void*>(this) ); 

   return res;
}

//*****************************************************************************
// Function: int CVirtualCaller::PrimRTSPClientPlay()
// Description: Start the RTSP Client Play
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimRTSPClientPlay()
{
   LogMsg(Name(), "PrimRTSPClientPlay(session_id=" +AsString(m_rtspIpmInfo.session_id)+ ")");
   CConference::getRtspConnector().play(m_rtspIpmInfo.session_id);

   return 0;
}

//*****************************************************************************
// Function: int CVirtualCaller::OnRtspEvent()
// Description: Handle an Rtsp Session Event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::OnRtspEvent()
{
   int res = 0;
   
   LogMsg(Name(), "CVirtualCaller:OnRtspEvent()");

   if (m_rtspEvent.type == RTSP_EVT_OPEN_OK)
   {
	   LogMsg(Name(), "OnRtspEvent(RTSP_EVT_OPEN_OK)");

	   m_rtspIpmInfo.session_id = m_rtspEvent.session_id;
	   LogMsg(Name(), "Rtsp session_id=" + AsString(m_rtspIpmInfo.session_id));
	   
	   switch ( m_rtspEvent.audio_codec )
	   {
			case RTSP_CODEC_NONE:
				m_rtspIpmInfo.audioCodecName = "none";
				break;
			case RTSP_CODEC_PCMU:
				m_rtspIpmInfo.audioCodecName = "pcmu";
				break;
			case RTSP_CODEC_PCMA:
				m_rtspIpmInfo.audioCodecName = "pcma";
				break;
			case RTSP_CODEC_AMR:
				m_rtspIpmInfo.audioCodecName = "amr";
				break;
			case RTSP_CODEC_G726_32:
				m_rtspIpmInfo.audioCodecName = "g726-32";
				break;
			default:
				m_rtspIpmInfo.audioCodecName = "unknown";
				break;
	   }
	   LogMsg(Name(), "Rtsp Audio Codec=" + m_rtspIpmInfo.audioCodecName);

	   switch ( m_rtspEvent.video_codec )
	   {
			case RTSP_CODEC_NONE:
				m_rtspIpmInfo.videoCodecName = "none";
				break;
			case RTSP_CODEC_H263:
				m_rtspIpmInfo.videoCodecName = "H263";
				break;
			case RTSP_CODEC_H263_1998:
				m_rtspIpmInfo.videoCodecName = "H263-1998";
				break;
			case RTSP_CODEC_H264:
				m_rtspIpmInfo.videoCodecName = "H264";
				break;
			case RTSP_CODEC_MP4VES:
				m_rtspIpmInfo.videoCodecName = "MP4V";
				break;
			default:
				m_rtspIpmInfo.videoCodecName = "unknown";
				break;
	   }
	   LogMsg(Name(), "Rtsp Video Codec=" + m_rtspIpmInfo.videoCodecName);

	   //Returns Rtsp Server Ports from Open
	   m_rtspIpmInfo.remoteRtspAudioPort = m_rtspEvent.audio_rtsp_port;
	   m_rtspIpmInfo.remoteRtspAudioAddr = m_rtspEvent.audio_rtsp_addr;
	   m_rtspIpmInfo.audioCodecPayloadType = m_rtspEvent.audio_payload_format;
	   LogMsg(Name(), "Rtsp Audio Port=" + AsString(m_rtspIpmInfo.remoteRtspAudioPort));
	   LogMsg(Name(), "Rtsp Audio Addr=" + AsString(m_rtspIpmInfo.remoteRtspAudioAddr));
	   LogMsg(Name(), "Rtsp Audio PT=" + AsString(m_rtspIpmInfo.audioCodecPayloadType));

       m_rtspIpmInfo.remoteRtspVideoPort = m_rtspEvent.video_rtsp_port;	   
       m_rtspIpmInfo.remoteRtspVideoAddr = m_rtspEvent.video_rtsp_addr;
	   m_rtspIpmInfo.videoCodecPayloadType = m_rtspEvent.video_payload_format;
	   m_rtspIpmInfo.videoProfile = m_rtspEvent.profile_level_id;
	   m_rtspIpmInfo.videoSprop.assign(m_rtspEvent.sprop_parameter_sets);
	   LogMsg(Name(), "Rtsp Video Port=" + AsString(m_rtspIpmInfo.remoteRtspVideoPort));
	   LogMsg(Name(), "Rtsp Video Addr=" + AsString(m_rtspIpmInfo.remoteRtspVideoAddr));
	   LogMsg(Name(), "Rtsp Video PT=" + AsString(m_rtspIpmInfo.videoCodecPayloadType));
	   LogMsg(Name(), "Rtsp Video Profile=" + AsHexString(m_rtspIpmInfo.videoProfile));
	   LogMsg(Name(), "Rtsp Video Sprop=" + m_rtspIpmInfo.videoSprop);
	   LogMsg(Name(), "Rtsp Video SpropLen=" + AsString(m_rtspIpmInfo.videoSprop.size()));

	   //Now call StartMedia
	   PrimRTSPStartMedia();

   }
   else if (m_rtspEvent.type == RTSP_EVT_OPEN_FAILED)
   {
	   LogMsg(Name(), "OnRtspEvent(RTSP_EVT_OPEN_FAILED)");
   }
   else if (m_rtspEvent.type == RTSP_EVT_PLAY_OK)
   {
	   LogMsg(Name(), "OnRtspEvent(RTSP_EVT_PLAY_OK)");
   }
   else if (m_rtspEvent.type == RTSP_EVT_PLAY_FAILED)
   {
	   LogMsg(Name(), "OnRtspEvent(RTSP_EVT_PLAY_FAILED)");
   }
   else if (m_rtspEvent.type == RTSP_EVT_PLAY_COMPLETED)
   {
	   LogMsg(Name(), "OnRtspEvent(RTSP_EVT_PLAY_COMPLETED)");
   }
   else if (m_rtspEvent.type == RTSP_EVT_STOP_OK)
   {
	   LogMsg(Name(), "OnRtspEvent(RTSP_EVT_STOP_OK)");
   }
   else if (m_rtspEvent.type == RTSP_EVT_STOP_FAILED)
   {
	   LogMsg(Name(), "OnRtspEvent(RTSP_EVT_STOP_FAILED)");
   }
   else if (m_rtspEvent.type == RTSP_EVT_ERROR)
   {
	   LogMsg(Name(), "ERROR OnRtspEvent(RTSP_EVT_ERROR)");
   }
   else if (m_rtspEvent.type == RTSP_EVT_NONE)
   {
	   LogMsg(Name(), "WARNING OnRtspEvent(RTSP_EVT_NONE)");
   }
   else
   {
	   LogMsg(Name(), "ERROR!! OnRtspEvent(RTSP_EVT_UNKNOWN!!!)");
   }

   return res;

}

//*****************************************************************************
// Function: int CVirtualCaller::SetRtspEventData(RTSPEvent* rtsp_event)
// Description: Handle an Rtsp Session Event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
void CVirtualCaller::SetRtspEventData(RtspEvent* rtsp_event)
{
	m_rtspEvent.type = rtsp_event->type;
	m_rtspEvent.session_id = rtsp_event->session_id;

	m_rtspEvent.audio_codec = rtsp_event->audio_codec;
	m_rtspEvent.audio_payload_format = rtsp_event->audio_payload_format;
	m_rtspEvent.audio_rtsp_port = rtsp_event->audio_rtsp_port;
	m_rtspEvent.audio_rtsp_addr = rtsp_event->audio_rtsp_addr;

	m_rtspEvent.video_codec = rtsp_event->video_codec;
	m_rtspEvent.video_payload_format = rtsp_event->video_payload_format;
	m_rtspEvent.video_rtsp_port = rtsp_event->video_rtsp_port;
	m_rtspEvent.video_rtsp_addr = rtsp_event->video_rtsp_addr;

	m_rtspEvent.height = rtsp_event->height;          
	m_rtspEvent.width = rtsp_event->width;             
	m_rtspEvent.fps = rtsp_event->fps;             
	m_rtspEvent.profile_level_id = rtsp_event->profile_level_id; 

	strcpy(m_rtspEvent.sprop_parameter_sets, rtsp_event->sprop_parameter_sets);

	return;
}



//*****************************************************************************
// Function: int CVirtualCaller::PrimRTSPStartMedia()
// Description: Handle a start RTSP media streaming command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CVirtualCaller::PrimRTSPStartMedia()
{
   // Local Variable Declaration
   int ret=-1;
   int mediaCtr = 0;
   int l_nRTSPAudioRtpPort;
   int l_nRTSPVideoRtpPort;
   char l_cRTSPAudioVideoIp[IP_ADDR_SIZE];
   int l_nLocalAudioRtpPort;
   int l_nLocalVideoRtpPort;

   IPM_VIDEO_CODER_INFO_EX   l_extraVidInfoLocal;
   IPM_VIDEO_CODER_INFO_EX   l_extraVidInfoRemote;

   
   IPM_MEDIA_INFO& l_mediaInfo = m_ipmDevice->RemoteMediaInfo();
   //memset(&l_mediaInfo, 0, sizeof(l_mediaInfo));
   
   LogMsg(Name(), "CVirtualCaller::PrimRTSPStartMedia()");


   LogMsg(Name(), "SetupRTSPRemoteMediaInfo:");

   // local audio connection
   LogMsg(Name(), " --local audio rtp port. mediaCtr=" + AsString(mediaCtr));
   l_mediaInfo.MediaData[mediaCtr].eMediaType = MEDIATYPE_AUDIO_LOCAL_RTP_INFO;
   strcpy(l_mediaInfo.MediaData[mediaCtr].mediaInfo.PortInfo.cIPAddress,  m_rtspIpmInfo.localIpmIPAddr);
   l_mediaInfo.MediaData[mediaCtr].mediaInfo.PortInfo.unPortId = m_rtspIpmInfo.localIpmAudioPort; 
   ++mediaCtr;

   LogMsg(Name(), " --Local Audio IpAddress:" + AsString(m_rtspIpmInfo.localIpmIPAddr) + " Local Audio RTP Port:" + AsString(m_rtspIpmInfo.localIpmAudioPort));

   // local video connection
   LogMsg(Name(), " --local video rtp port. mediaCtr=" + AsString(mediaCtr));
   l_mediaInfo.MediaData[mediaCtr].eMediaType = MEDIATYPE_VIDEO_LOCAL_RTP_INFO;
   strcpy(l_mediaInfo.MediaData[mediaCtr].mediaInfo.PortInfo.cIPAddress,  m_rtspIpmInfo.localIpmIPAddr);
   l_mediaInfo.MediaData[mediaCtr].mediaInfo.PortInfo.unPortId = m_rtspIpmInfo.localIpmVideoPort; 
   ++mediaCtr;
   
   LogMsg(Name(), " --Local Video IpAddress:" + AsString(m_rtspIpmInfo.localIpmIPAddr) + " Local Video RTP Port:" + AsString(m_rtspIpmInfo.localIpmVideoPort));

   // local audio coder
   //TODO: Only AMR and G711u have been tested
   LogMsg(Name(), " --RTSP audio codec = " + AsString(m_rtspIpmInfo.audioCodecName.c_str()));
   if (strncmp(m_rtspIpmInfo.audioCodecName.c_str(), "AMR", 3)==0 || strncmp(m_rtspIpmInfo.audioCodecName.c_str(), "amr", 3)==0)
   {
      LogMsg(Name(), " --local audio AMR. mediaCtr=" + AsString(mediaCtr));
      l_mediaInfo.MediaData[mediaCtr].eMediaType = MEDIATYPE_AUDIO_LOCAL_CODER_INFO;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eCoderType = CODER_TYPE_AMRNB_12_2k;

      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eFrameSize = CODER_FRAMESIZE_20;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unFramesPerPkt = 10;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eVadEnable = CODER_VAD_ENABLE;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unCoderPayloadType = m_rtspIpmInfo.audioCodecPayloadType;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unRedPayloadType = 97;
      ++mediaCtr;

      LogMsg(Name(), " --local audio AMR Options. mediaCtr=" + AsString(mediaCtr));
	  l_mediaInfo.MediaData[mediaCtr].eMediaType = MEDIATYPE_AUDIO_LOCAL_CODER_OPTIONS_INFO;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderOptionsInfo.unVersion =
      IPM_AUDIO_CODER_OPTIONS_INFO_VERSION;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderOptionsInfo.unCoderOptions=
      CODER_OPT_AMR_CMR_TRACK | CODER_OPT_AMR_OCTET; //CODER_OPT_AMR_EFFICIENT;
      ++mediaCtr;
   }
   else if (strncmp(m_rtspIpmInfo.audioCodecName.c_str(), "g722", 4)==0 || strncmp(m_rtspIpmInfo.audioCodecName.c_str(), "G722", 4)==0)
   {
      // G.726
      LogMsg(Name(), " --local audio G722-64k. mediaCtr=" + AsString(mediaCtr));
      l_mediaInfo.MediaData[mediaCtr].eMediaType = MEDIATYPE_AUDIO_LOCAL_CODER_INFO;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eCoderType = CODER_TYPE_G722_64K;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eFrameSize = CODER_FRAMESIZE_20;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unFramesPerPkt = 1;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eVadEnable = CODER_VAD_DISABLE;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unCoderPayloadType  =  9;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unRedPayloadType = 0;
      ++mediaCtr;
   }
   else if (strncmp(m_rtspIpmInfo.audioCodecName.c_str(), "g726-32", 7)==0 || strncmp(m_rtspIpmInfo.audioCodecName.c_str(), "G726-32", 7)==0)
   {
      // G.726
      LogMsg(Name(), " --local audio G726-32k. mediaCtr=" + AsString(mediaCtr));
      l_mediaInfo.MediaData[mediaCtr].eMediaType = MEDIATYPE_AUDIO_LOCAL_CODER_INFO;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eCoderType = CODER_TYPE_G726_32K;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eFrameSize = CODER_FRAMESIZE_20; 
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unFramesPerPkt = 1;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eVadEnable = CODER_VAD_DISABLE;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unCoderPayloadType  =  m_rtspIpmInfo.audioCodecPayloadType;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unRedPayloadType = 0;
      ++mediaCtr;
   }
   else if (strncmp(m_rtspIpmInfo.audioCodecName.c_str(), "PCMA", 4)==0 || strncmp(m_rtspIpmInfo.audioCodecName.c_str(), "pcma", 4)==0)
   {
      // G.711ALaw
      LogMsg(Name(), " --local audio G711a. mediaCtr=" + AsString(mediaCtr));
      l_mediaInfo.MediaData[mediaCtr].eMediaType = MEDIATYPE_AUDIO_LOCAL_CODER_INFO;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eCoderType = CODER_TYPE_G711ALAW64K;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eFrameSize = CODER_FRAMESIZE_20; 
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unFramesPerPkt = 1;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eVadEnable = CODER_VAD_DISABLE;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unCoderPayloadType  = 8;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unRedPayloadType = 0;
      ++mediaCtr;
   }
   else if (strncmp(m_rtspIpmInfo.audioCodecName.c_str(), "PCMU", 4)==0 || strncmp(m_rtspIpmInfo.audioCodecName.c_str(), "pcmu", 4)==0)
   {
      // G.711ULaw
      LogMsg(Name(), " --local audio G711u. mediaCtr=" + AsString(mediaCtr));
      l_mediaInfo.MediaData[mediaCtr].eMediaType = MEDIATYPE_AUDIO_LOCAL_CODER_INFO;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eCoderType = CODER_TYPE_G711ULAW64K;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eFrameSize = CODER_FRAMESIZE_20; 
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unFramesPerPkt = 1;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.eVadEnable = CODER_VAD_DISABLE;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unCoderPayloadType  = 0;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.AudioCoderInfo.unRedPayloadType = 0;
      ++mediaCtr;
   }

   LogMsg(Name(), " --RTSP video codec = " + AsString(m_rtspIpmInfo.videoCodecName.c_str()));
   if (strncmp(m_rtspIpmInfo.videoCodecName.c_str(), "MP4V", 4)==0 || strncmp(m_rtspIpmInfo.videoCodecName.c_str(), "mp4v", 4)==0) {
      
      // local video coder
      LogMsg(Name(), " --local video MP4V. mediaCtr=" + AsString(mediaCtr));
      INIT_IPM_VIDEO_CODER_INFO(&l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo);
      l_mediaInfo.MediaData[mediaCtr].eMediaType = MEDIATYPE_VIDEO_LOCAL_CODER_INFO;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.eCoderType = CODER_TYPE_MP4V_ES;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.unCoderPayloadType = m_rtspIpmInfo.videoCodecPayloadType;

      INIT_IPM_VIDEO_CODER_INFO_EX(&l_extraVidInfoLocal);
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.pExtraCoderInfo = &l_extraVidInfoLocal;
      if (m_rtspIpmInfo.videoProfile == 3) {
         LogMsg(Name(), " --local video MP4V Profile 3");
         l_extraVidInfoLocal.eImageWidth = VIDEO_IMAGE_WIDTH_352;
		 l_extraVidInfoLocal.eImageHeight = VIDEO_IMAGE_HEIGHT_288;
		 l_extraVidInfoLocal.unBitRate = 384000;
		 l_extraVidInfoLocal.eFramesPerSec = VIDEO_FRAMESPERSEC_30;
		 l_extraVidInfoLocal.eSamplingRate = VIDEO_SAMPLING_RATE_DEFAULT;
         l_extraVidInfoLocal.unVisualConfigSize=0;
         l_extraVidInfoLocal.szVisualConfiguration=NULL;
      } else { // assuming QCIF
         LogMsg(Name(), " --local video MP4V Profile NOT 3");
         l_extraVidInfoLocal.eProfile = VIDEO_PROFILE_LEVEL_SP0_MPEG4;
		 l_extraVidInfoLocal.eLevel = VIDEO_LEVEL_10_H263 ;
		 l_extraVidInfoLocal.eImageWidth = VIDEO_IMAGE_WIDTH_176;
		 l_extraVidInfoLocal.eImageHeight = VIDEO_IMAGE_HEIGHT_144;
		 l_extraVidInfoLocal.unBitRate = 40000;
		 l_extraVidInfoLocal.eFramesPerSec = VIDEO_FRAMESPERSEC_15;
		 l_extraVidInfoLocal.eSamplingRate = VIDEO_SAMPLING_RATE_DEFAULT;
         l_extraVidInfoLocal.unVisualConfigSize=0;
         l_extraVidInfoLocal.szVisualConfiguration=NULL;
      }
      ++mediaCtr;

   } else if (strncmp(m_rtspIpmInfo.videoCodecName.c_str(), "H264", 4)==0 || strncmp(m_rtspIpmInfo.videoCodecName.c_str(), "h264", 4)==0) {
      
      // local video coder
      LogMsg(Name(), " --local video H.264. mediaCtr=" + AsString(mediaCtr));
      INIT_IPM_VIDEO_CODER_INFO(&l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo);
      l_mediaInfo.MediaData[mediaCtr].eMediaType = MEDIATYPE_VIDEO_LOCAL_CODER_INFO;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.eCoderType = CODER_TYPE_H264;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.unCoderPayloadType = m_rtspIpmInfo.videoCodecPayloadType;
      INIT_IPM_VIDEO_CODER_INFO_EX(&l_extraVidInfoLocal);
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.pExtraCoderInfo = &l_extraVidInfoLocal;
	  
      LogMsg(Name(), " --local video H.264 Profile = " + AsHexString(m_rtspIpmInfo.videoProfile));
      l_extraVidInfoLocal.eProfile = VIDEO_PROFILE_BASELINE_H264;
      l_extraVidInfoLocal.eLevel = VIDEO_LEVEL_3_0_H264 ;
      l_extraVidInfoLocal.eImageWidth = VIDEO_IMAGE_WIDTH_VGA;
      l_extraVidInfoLocal.eImageHeight = VIDEO_IMAGE_HEIGHT_VGA;
      l_extraVidInfoLocal.unBitRate = VIDEO_BITRATE_768K;
      l_extraVidInfoLocal.eFramesPerSec = VIDEO_FRAMESPERSEC_30;
      l_extraVidInfoLocal.eSamplingRate = VIDEO_SAMPLING_RATE_DEFAULT;
	  //JM - Need to set H264 sprop here for decoder!!!
	l_extraVidInfoLocal.unVisualConfigSize=m_rtspIpmInfo.videoSprop.size();
	l_extraVidInfoLocal.szVisualConfiguration=(unsigned char *)m_rtspIpmInfo.videoSprop.c_str();

#ifdef USE_H264
      l_extraVidInfoLocal.eH264_PacketizationMode = H264_PACKETIZATION_MODE_NON_INTERLEAVED;
      l_extraVidInfoLocal.eH264_AcceptRedundantSlices  = ACCEPT_REDUNDAND_SLICES_DEFAULT; //DEFAULT=false
      l_extraVidInfoLocal.eH264_ProfileIOP             = H264_PROFILE_IOP_DEFAULT; //DEFAULT=0
      l_extraVidInfoLocal.unH264_MaxNalUnitSize      = H264_MAX_NAL_UNIT_SIZE_DEFAULT; //DEFAULT=1400
#endif
      ++mediaCtr;

   } else if (strncmp(m_rtspIpmInfo.videoCodecName.c_str(), "H263-1998", 9)==0 || strncmp(m_rtspIpmInfo.videoCodecName.c_str(), "h263-1998", 9)==0) {
      
      // local video coder
      LogMsg(Name(), " --local video H263-1998. mediaCtr=" + AsString(mediaCtr));
      l_mediaInfo.MediaData[mediaCtr].eMediaType = MEDIATYPE_VIDEO_LOCAL_CODER_INFO;
      INIT_IPM_VIDEO_CODER_INFO(&l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo);
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.eCoderType = CODER_TYPE_H263_1998;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.unCoderPayloadType = m_rtspIpmInfo.videoCodecPayloadType; 
         
	  INIT_IPM_VIDEO_CODER_INFO_EX(&l_extraVidInfoLocal);
	  l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.pExtraCoderInfo = &l_extraVidInfoLocal;
	  l_extraVidInfoLocal.eProfile = VIDEO_PROFILE_0_H263;
          l_extraVidInfoLocal.eLevel = VIDEO_LEVEL_30_H263;
	  l_extraVidInfoLocal.eImageWidth = VIDEO_IMAGE_WIDTH_CIF;
	  l_extraVidInfoLocal.eImageHeight = VIDEO_IMAGE_HEIGHT_CIF;
	  l_extraVidInfoLocal.unBitRate = VIDEO_BITRATE_384K;
	  l_extraVidInfoLocal.eFramesPerSec = VIDEO_FRAMESPERSEC_30;
	  l_extraVidInfoLocal.eSamplingRate = VIDEO_SAMPLING_RATE_DEFAULT;
	  l_extraVidInfoLocal.unVisualConfigSize = 0;         
	  l_extraVidInfoLocal.szVisualConfiguration = NULL;
      ++mediaCtr;

   } else if (strncmp(m_rtspIpmInfo.videoCodecName.c_str(), "H263-2000", 9)==0  ||  strncmp(m_rtspIpmInfo.videoCodecName.c_str(), "h263-2000", 9)==0) {

      // local video coder
      LogMsg(Name(), " --local video H263-2000. mediaCtr=" + AsString(mediaCtr));
      l_mediaInfo.MediaData[mediaCtr].eMediaType = MEDIATYPE_VIDEO_LOCAL_CODER_INFO;
      INIT_IPM_VIDEO_CODER_INFO(&l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo);
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.eCoderType = CODER_TYPE_H263_1998;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.unCoderPayloadType = m_rtspIpmInfo.videoCodecPayloadType; 
	  INIT_IPM_VIDEO_CODER_INFO_EX(&l_extraVidInfoLocal);
	  l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.pExtraCoderInfo = &l_extraVidInfoLocal;
	  l_extraVidInfoLocal.eProfile = VIDEO_PROFILE_0_H263;
          l_extraVidInfoLocal.eLevel = VIDEO_LEVEL_30_H263;
	  l_extraVidInfoLocal.eImageWidth = VIDEO_IMAGE_WIDTH_CIF;
	  l_extraVidInfoLocal.eImageHeight = VIDEO_IMAGE_HEIGHT_CIF;
	  l_extraVidInfoLocal.unBitRate = VIDEO_BITRATE_384K;
	  l_extraVidInfoLocal.eFramesPerSec = VIDEO_FRAMESPERSEC_30;
	  l_extraVidInfoLocal.eSamplingRate = VIDEO_SAMPLING_RATE_DEFAULT;
	  l_extraVidInfoLocal.unVisualConfigSize = 0;
	  l_extraVidInfoLocal.szVisualConfiguration = NULL;
      ++mediaCtr;

   } else if (strncmp(m_rtspIpmInfo.videoCodecName.c_str(), "H263", 4)==0  || strncmp(m_rtspIpmInfo.videoCodecName.c_str(), "h263", 4)==0) {

      // local video coder
      LogMsg(Name(), " --local video H263. mediaCtr=" + AsString(mediaCtr));
      l_mediaInfo.MediaData[mediaCtr].eMediaType= MEDIATYPE_VIDEO_LOCAL_CODER_INFO;
      INIT_IPM_VIDEO_CODER_INFO(&l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo);
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.eCoderType = CODER_TYPE_H263;
      l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.unCoderPayloadType = 34; 
	  INIT_IPM_VIDEO_CODER_INFO_EX(&l_extraVidInfoLocal);
	  l_mediaInfo.MediaData[mediaCtr].mediaInfo.VideoCoderInfo.pExtraCoderInfo = &l_extraVidInfoLocal;
	  l_extraVidInfoLocal.eProfile = VIDEO_PROFILE_0_H263;
          l_extraVidInfoLocal.eLevel = VIDEO_LEVEL_30_H263;
	  l_extraVidInfoLocal.eImageWidth = VIDEO_IMAGE_WIDTH_CIF;
	  l_extraVidInfoLocal.eImageHeight = VIDEO_IMAGE_HEIGHT_CIF;
	  l_extraVidInfoLocal.unBitRate = VIDEO_BITRATE_384K;
	  l_extraVidInfoLocal.eFramesPerSec = VIDEO_FRAMESPERSEC_30;
	  l_extraVidInfoLocal.eSamplingRate = VIDEO_SAMPLING_RATE_DEFAULT;
	  l_extraVidInfoLocal.unVisualConfigSize = 0;
	  l_extraVidInfoLocal.szVisualConfiguration = NULL;
      ++mediaCtr;
   }

   l_mediaInfo.unCount = mediaCtr ;
   LogMsg(Name(), " --l_mediaInfo.unCount =" + AsString(l_mediaInfo.unCount));
   
   m_ipmDevice->PrintRemoteLocalInfo();

   ret = m_ipmDevice->IpmStartMedia(DATA_IP_RECEIVEONLY);
   if (ret==0){
      m_rtspIpmInfo.bRtspMediaStarted=true;
   }
   else {
      m_rtspIpmInfo.bRtspMediaStarted=false;
   }

   return ret;
}

#endif //USE_RTSP

//*****************************************************************************
// Function: void CVirtualCaller::ProcessEvent(METAEVENT &a_metaEvent)
// Description: Process and event
// Return: void 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
void CVirtualCaller::ProcessEvent(METAEVENT& a_metaEvent)
{
   switch (a_metaEvent.evttype)
   {
      case APPEVENT_MM_OPEN_COMPLETE:
		  LogMsg(Name(), "APPEVENT_MM_OPEN_COMPLETE");
// SKS JACK need to move when conference starting
//         StartVirtualCaller();
         break;
	  case APPEVENT_ON_CNF_STARTED:
		 LogMsg(Name(), "APPEVENT_ON_CNF_STARTED");
         StartVirtualCaller();
			break;

      case APPEVENT_ON_OPEN_CNF_PARTY_COMPLETE:
         LogMsg(Name(), "APPEVENT_ON_OPEN_CNF_PARTY_COMPLETE");
		 OnOpenConferencePartyComplete();
         break;

      case APPEVENT_ON_CLOSE_CNF_PARTY_COMPLETE:
         LogMsg(Name(), "APPEVENT_ON_CLOSE_CNF_PARTY_COMPLETE");
		 OnCloseConferencePartyComplete();
         break;

      case APPEVENT_ON_JOIN_CNF_COMPLETE:
         LogMsg(Name(), "APPEVENT_ON_JOIN_CNF_COMPLETE");
		 OnJoinConferenceComplete();
         break;

      case APPEVENT_ON_LEAVE_CNF_COMPLETE:
         LogMsg(Name(), "APPEVENT_ON_LEAVE_CNF_COMPLETE");
		 OnLeaveConferenceComplete();
         break;
      case APPEVENT_ON_LEAVE_CNF_FAILED:
         LogMsg(Name(), "APPEVENT_ON_LEAVE_CNF_FAILED");
		 OnLeaveConferenceFailed();
         break;

      case APPEVENT_CNF_GET_RX_PORT_INFO_COMPLETE:
         LogMsg(Name(), "APPEVENT_CNF_GET_RX_PORT_INFO_COMPLETE");
		 OnGetCnfRxPortInfoComplete();
         break;

      case APPEVENT_CNF_GET_TX_PORT_INFO_COMPLETE:
         LogMsg(Name(), "APPEVENT_CNF_GET_TX_PORT_INFO_COMPLETE");
		 OnGetCnfTxPortInfoComplete();
         break;

      case APPEVENT_CNF_PORT_CONNECTED:
         LogMsg(Name(), "APPEVENT_CNF_PORT_CONNECTED");
		 OnCnfPortConnected();
         break;

      case APPEVENT_CNF_PORT_DISCONNECTED:
         LogMsg(Name(), "APPEVENT_CNF_PORT_DISCONNECTED");
		 OnCnfPortDisconnected();
         break;

      case APPEVENT_MM_GET_RX_PORT_INFO_COMPLETE:
         LogMsg(Name(), "APPEVENT_MM_GET_RX_PORT_INFO_COMPLETE");
		 OnGetVcRxPortInfoComplete();
         break;

      case APPEVENT_MM_GET_TX_PORT_INFO_COMPLETE:
         LogMsg(Name(), "APPEVENT_MM_GET_TX_PORT_INFO_COMPLETE");
		 OnGetVcTxPortInfoComplete();
         break;

      case APPEVENT_MM_PORT_CONNECTED:
         LogMsg(Name(), "APPEVENT_MM_PORT_CONNECTED");
		 OnVcPortConnected();
         break;

      case APPEVENT_MM_PORT_DISCONNECTED:
         LogMsg(Name(), "APPEVENT_MM_PORT_DISCONNECTED");
		 OnVcPortDisconnected();
         break;

      case APPEVENT_IPM_GET_RX_PORT_INFO_COMPLETE:
         LogMsg(Name(), "APPEVENT_IPM_GET_RX_PORT_INFO_COMPLETE");
		 OnGetVcRxPortInfoComplete();
         break;

      case APPEVENT_IPM_GET_TX_PORT_INFO_COMPLETE:
         LogMsg(Name(), "APPEVENT_IPM_GET_TX_PORT_INFO_COMPLETE");
		 OnGetVcTxPortInfoComplete();
         break;

      case APPEVENT_IPM_PORT_CONNECTED:
         LogMsg(Name(), "APPEVENT_IPM_PORT_CONNECTED");
		 OnVcPortConnected();
         break;

      case APPEVENT_IPM_PORT_DISCONNECTED:
         LogMsg(Name(), "APPEVENT_IPM_PORT_DISCONNECTED");
		 OnVcPortDisconnected();
         break;

	  case APPEVENT_GET_LOCAL_MEDIA_INFO_COMPLETE:
         LogMsg(Name(), "APPEVENT_GET_LOCAL_MEDIA_INFO_COMPLETE");
		 break;

	  case APPEVENT_START_MEDIA_COMPLETE:
	     LogMsg(Name(), "APPEVENT_START_MEDIA_COMPLETE");
		 OnStartMediaComplete();
		 break;

      case APPEVENT_STOP_MEDIA_COMPLETE:
	     LogMsg(Name(), "APPEVENT_STOP_MEDIA_COMPLETE");
		 OnStopMediaComplete();
		 break;

      case APPEVENT_PLAY_COMPLETE:
         LogMsg(Name(), "APPEVENT_PLAY_COMPLETE");
		 OnPlayComplete();
         break;

      case APPEVENT_PLAY_FAIL:
         LogMsg(Name(), "APPEVENT_PLAY_FAIL - ignore");
		 break;

	  case APPEVENT_DROPCALL_COMMAND:
		 LogMsg(Name(), "APPEVENT_DROPCALL_COMMAND - ignore");
		 break;

	 case APPEVENT_RTSP_EVENT:
		 LogMsg(Name(), "APPEVENT_RTSP_EVENT");
#ifdef USE_RTSP
		 OnRtspEvent();
#endif
		 break;

      default:
         LogMsg(Name(), "unhandled event:" + AsHexString(a_metaEvent.evttype));
   }
}

