/**
* @file CVirtualCaller.h
* @brief Virtual Caller class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CVirtualCaller_h
#define CVirtualCaller_h

#include <string>

#include "port_connect.h"
#include "ipmlib.h"

#include "config.h"

#include "AEventHandler.h"
#include "CSystemConfiguration.h"

#ifdef USE_RTSP
   #include "CRtspConnector.h"   
#endif

class CIpmDevice;

class CMediaDevice;
class CConferencePartyDevice;
class AVirtualCallerState;

struct RtspIpmInfo
{
	bool bRtspMediaStarted;
	int  session_id;

    int localIpmAudioPort;
	int localIpmVideoPort;
	char* localIpmIPAddr;
	int remoteRtspAudioPort;
	int remoteRtspVideoPort;
	unsigned int remoteRtspAudioAddr;
	unsigned int remoteRtspVideoAddr;
	//char* remoteRtspIPAddr;

	std::string audioCodecName;
	std::string videoCodecName;
	int audioCodecPayloadType;
	int videoCodecPayloadType;
	unsigned int videoProfile;
	std::string videoSprop;

};

//*****************************************************************************
// Class: ::CVirtualCaller
// Description: Class to provide a virtual caller/conference party.  The caller
//    is simulated by playing a multimedia clip to a conference party 
//    connection
//*****************************************************************************
class CVirtualCaller : public AEventHandler
{
   friend class AVirtualCallerState;
   friend class CVirtualCallerIdleState;
   friend class CVirtualCallerSetupState;
   friend class CVirtualCallerAvailableState;
   friend class CVirtualCallerPlayingState;
   friend class CVirtualCallerTeardownState;

public:
   CVirtualCaller(const std::string& a_name,std::string confID, int vCallerNum);
   virtual ~CVirtualCaller() {}

   void ProcessEvent(METAEVENT& a_metaEvent);

   AVirtualCallerState& State() {return *m_state;}
   void SetStateTo(AVirtualCallerState* state);

   const std::string& Name() {return m_name;}
   
   void OpenMediaDevice();
   int StartVirtualCaller();
   int OpenConferenceParty();
   int OnOpenConferencePartyComplete();
   int ConnectToConferenceParty();
   int OnConnectToConferencePartyComplete();
   int JoinConference();
   int OnJoinConferenceComplete();
   int LeaveConference();
   int OnLeaveConferenceComplete();
   int OnLeaveConferenceFailed();
   int DisconnectFromConferenceParty();
   int OnDisconnectFromConferencePartyComplete();
   int CloseConferenceParty();
   int OnCloseConferencePartyComplete();
   void SystemConfiguration(CSystemConfiguration* a_systemConfiguration);

   int Play();
   int OnPlayComplete();
   int Stop();

   int OnGetVcRxPortInfoComplete();
   int OnGetVcTxPortInfoComplete();
   int OnGetCnfRxPortInfoComplete();
   int OnGetCnfTxPortInfoComplete();

   int PortInfoSubState() {return m_portInfoSubState;}

   int OnCnfPortConnected();
   int OnVcPortConnected();

   int PortConnectSubState() {return m_portConnectSubState;}

   int OnCnfPortDisconnected();
   int OnVcPortDisconnected();

   int PortDisconnectSubState() {return m_portDisconnectSubState;}

   int OnStartMediaComplete();
   int OnStopMediaComplete();

#ifdef USE_RTSP
   void OpenRTSPClient();

   void SetRtspEventData(RtspEvent* rtsp_event);
#endif

   static int VC_RX_PORT_INFO;
   static int VC_TX_PORT_INFO;
   static int CNF_RX_PORT_INFO;
   static int CNF_TX_PORT_INFO;
   static int PORT_INFO_COMPLETE;

   static int VC_TX_CNF_RX_CONNECT;
   static int CNF_TX_VC_RX_CONNECT;
   static int PORT_CONNECT_COMPLETE;

   static int VC_TX_CNF_RX_DISCONNECT;
   static int CNF_TX_VC_RX_DISCONNECT;
   static int PORT_DISCONNECT_COMPLETE;

private:

   // force initialization
   CVirtualCaller();

   // make non-copyable
   CVirtualCaller(const CVirtualCaller&);
   const CVirtualCaller& operator=(const CVirtualCaller&);

   AVirtualCallerState* m_state;
   std::string m_name;

   CMediaDevice* m_mmDevice;
   AEventHandler* m_observer;

   CConferencePartyDevice* m_conferenceParty;
   int m_portInfoSubState;
   int m_portConnectSubState;
   int m_portDisconnectSubState;
   bool m_stopPlay;
   
   std::string m_confID;
   int m_vCallerNum;
   std::string m_vCallerMediaClip;
   CIpmDevice* m_ipmDevice;
   bool m_bIsMediaRTSP;

#ifdef USE_RTSP
   int PlayRTSPFile();
   int PrimRTSPStartMedia();
   int PrimRTSPClientPlay();

   RtspIpmInfo m_rtspIpmInfo;
   RtspEvent m_rtspEvent;
   int OnRtspEvent();
#endif

   DM_PORT_CONNECT_INFO_LIST m_portConnectInfoListVcTxCnfRx;
   DM_PORT_CONNECT_INFO_LIST m_portConnectInfoListCnfTxVcRx;

   void SetMediaClip();

   int PrimStartVirtualCaller();
   int PrimOpenConferenceParty();
   int PrimOnOpenConferencePartyComplete();
   int PrimConnectToConferenceParty();
   int PrimOnConnectToConferencePartyComplete();
   int PrimJoinConference();
   int PrimOnJoinConferenceComplete();
   int PrimLeaveConference();
   int PrimOnLeaveConferenceComplete();
   int PrimDisconnectFromConferenceParty();
   int PrimOnDisconnectFromConferencePartyComplete();
   int PrimCloseConferenceParty();
   int PrimOnCloseConferencePartyComplete();
   int PrimOnCallReleased();

   int PrimOnGetVcRxPortInfoComplete();
   int PrimOnGetVcTxPortInfoComplete();
   int PrimOnGetCnfRxPortInfoComplete();
   int PrimOnGetCnfTxPortInfoComplete();
   int PrimPortConnect();

   int PrimOnCnfPortConnected();
   int PrimOnVcPortConnected();

   int PrimOnCnfPortDisconnected();
   int PrimOnVcPortDisconnected();


   int CreatePortConnectInfoList(  DM_PORT_CONNECT_INFO_LIST& a_portConnectionInfoList,
                                   const DM_PORT_INFO_LIST& a_portInfoList1,
                                   const DM_PORT_INFO_LIST& a_portInfoList2);

   int PrimPlay();
   int PrimOnPlayComplete();
   int PrimStop();

};

#endif

