/**
* @file CVirtualCallerState.cpp
* @brief Virtual Caller States - class implementations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#endif

#include "config.h"

#include "LogUtil.h"
#include "CVirtualCallerState.h"

#include "globals.h"

CVirtualCallerStates AVirtualCallerState::m_virtualCallerStates;

//*****************************************************************************
// Function: CVirtualCallerStates::CVirtualCallerStates()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CVirtualCallerStates::CVirtualCallerStates()
{
   m_virtualCallerStates[VirtualCallerIdleState] = new CVirtualCallerIdleState();
   m_virtualCallerStates[VirtualCallerSetupState] = new CVirtualCallerSetupState();
   m_virtualCallerStates[VirtualCallerAvailableState] = new CVirtualCallerAvailableState();
   m_virtualCallerStates[VirtualCallerPlayingState] = new CVirtualCallerPlayingState();
   m_virtualCallerStates[VirtualCallerTeardownState] = new CVirtualCallerTeardownState();
}

//*****************************************************************************
// Function: int AVirtualCallerState::LogStateError(CVirtualCaller &a_virtualCaller, const std::string &msg)
// Description: Log an invalid state for operation/event message
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//             const std::string &msg 
//*****************************************************************************
int AVirtualCallerState::LogStateError(CVirtualCaller& a_virtualCaller, const std::string& msg)
{
   std::cout << "a_virtualCaller"
   << " " << Desc() << " is invalid device state for "
   << msg
   << std::endl;
   return -1;
}


//*****************************************************************************
// Function: int CVirtualCallerIdleState::StartVirtualCaller(CVirtualCaller &a_virtualCaller)
// Description: Start the virtual caller
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerIdleState::StartVirtualCaller( CVirtualCaller& a_virtualCaller)
{
   int res = a_virtualCaller.PrimStartVirtualCaller();
   if (!res)
   {
      a_virtualCaller.SetStateTo(State(VirtualCallerSetupState));
      a_virtualCaller.OpenConferenceParty();
   }
   return res;
}
//*****************************************************************************
// Function: int CVirtualCallerIdleState::OnPlayComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a play completion event
// Return: int - 0 for success, otherwise error
// Parameters: CVirtualCaller &a_virtualCaller
//*****************************************************************************
int CVirtualCallerIdleState::OnPlayComplete(CVirtualCaller &a_virtualCaller)
{
   return 0;
}


//*****************************************************************************
// Function: int CVirtualCallerAvailableState::Play(CVirtualCaller &a_virtualCaller)
// Description: P{lay the virtual caller media clip
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerAvailableState::Play(CVirtualCaller &a_virtualCaller)
{
   a_virtualCaller.SetStateTo(State(VirtualCallerPlayingState));
   return a_virtualCaller.PrimPlay();
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCallerAvailableState::LeaveConference(CVirtualCaller &a_virtualCaller)
// Description: Leave the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerAvailableState::LeaveConference(CVirtualCaller &a_virtualCaller)
{
   a_virtualCaller.SetStateTo(State(VirtualCallerTeardownState));
   return a_virtualCaller.PrimLeaveConference();
}



//*****************************************************************************
// Function: int CVirtualCallerPlayingState::OnPlayComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a play completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerPlayingState::OnPlayComplete(CVirtualCaller &a_virtualCaller)
{
   if ((a_virtualCaller.PrimOnPlayComplete() != 0) && (g_applicationState == ApplicationStateStarted))
   {
      return a_virtualCaller.PrimPlay();
   }
   else
   {
      a_virtualCaller.SetStateTo(State(VirtualCallerAvailableState));
   }
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCallerPlayingState::OnStartMediaComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a play completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerPlayingState::OnStartMediaComplete(CVirtualCaller &a_virtualCaller)
{
#ifdef USE_RTSP
   return a_virtualCaller.PrimRTSPClientPlay();
#else
   return -1;
#endif
}

//*****************************************************************************
// Function: int CVirtualCallerPlayingState::Stop(CVirtualCaller &a_virtualCaller)
// Description: Stop playing the virtual caller media clip
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerPlayingState::Stop(CVirtualCaller &a_virtualCaller)
{
   return a_virtualCaller.PrimStop();
}

//*****************************************************************************
// Function: int CVirtualCallerSetupState::OpenConferenceParty(CVirtualCaller &a_virtualCaller)
// Description: Open the conference party
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerSetupState::OpenConferenceParty(CVirtualCaller &a_virtualCaller)
{
   return a_virtualCaller.PrimOpenConferenceParty();
}

//*****************************************************************************
// Function: int CVirtualCallerSetupState::OnOpenConferencePartyComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle an open conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerSetupState::OnOpenConferencePartyComplete(CVirtualCaller &a_virtualCaller)
{
   int res = a_virtualCaller.PrimOnOpenConferencePartyComplete();
   if (res == 0)
   {
      res = a_virtualCaller.ConnectToConferenceParty();
   }
   return res;
}

//*****************************************************************************
// Function: int CVirtualCallerSetupState::ConnectToConferenceParty(CVirtualCaller &a_virtualCaller)
// Description: Connect to the conference party
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerSetupState::ConnectToConferenceParty(CVirtualCaller &a_virtualCaller)
{
   return a_virtualCaller.PrimConnectToConferenceParty();
}

//*****************************************************************************
// Function: int CVirtualCallerSetupState::OnGetVcRxPortInfoComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a get RX port info completion event from the VC (mm or ipm) device
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerSetupState::OnGetVcRxPortInfoComplete(CVirtualCaller& a_virtualCaller)
{
   if (a_virtualCaller.PrimOnGetVcRxPortInfoComplete() == CVirtualCaller::PORT_INFO_COMPLETE)
   {
      return a_virtualCaller.PrimPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCallerSetupState::OnGetVcTxPortInfoComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a get TX port info completion event from the VC (mm or ipm) device
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerSetupState::OnGetVcTxPortInfoComplete(CVirtualCaller& a_virtualCaller)
{
   if (a_virtualCaller.PrimOnGetVcTxPortInfoComplete() == CVirtualCaller::PORT_INFO_COMPLETE)
   {
      return a_virtualCaller.PrimPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCallerSetupState::OnGetCnfRxPortInfoComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a get RX port info completion event from the CNF party device
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerSetupState::OnGetCnfRxPortInfoComplete(CVirtualCaller& a_virtualCaller)
{
   if (a_virtualCaller.PrimOnGetCnfRxPortInfoComplete() == CVirtualCaller::PORT_INFO_COMPLETE)
   {
      return a_virtualCaller.PrimPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCallerSetupState::OnGetCnfTxPortInfoComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a get TX port info completion event from the CNF party device
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerSetupState::OnGetCnfTxPortInfoComplete(CVirtualCaller& a_virtualCaller)
{
   if (a_virtualCaller.PrimOnGetCnfTxPortInfoComplete() == CVirtualCaller::PORT_INFO_COMPLETE)
   {
      return a_virtualCaller.PrimPortConnect();
   }
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCallerSetupState::OnCnfPortConnected(CVirtualCaller &a_virtualCaller)
// Description: Handle a port connected event from the CNF party device
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerSetupState::OnCnfPortConnected(CVirtualCaller& a_virtualCaller)
{
   if (a_virtualCaller.PrimOnCnfPortConnected() == CVirtualCaller::PORT_CONNECT_COMPLETE)
   {
      return a_virtualCaller.OnConnectToConferencePartyComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCallerSetupState::OnVcPortConnected(CVirtualCaller &a_virtualCaller)
// Description: Handle a port connected event from the VC device
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerSetupState::OnVcPortConnected(CVirtualCaller& a_virtualCaller)
{
   if (a_virtualCaller.PrimOnVcPortConnected() == CVirtualCaller::PORT_CONNECT_COMPLETE)
   {
      return a_virtualCaller.OnConnectToConferencePartyComplete();
   }
   return 0;

}

//*****************************************************************************
// Function: int CVirtualCallerSetupState::OnConnectToConferencePartyComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a connect to conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerSetupState::OnConnectToConferencePartyComplete(CVirtualCaller &a_virtualCaller)
{
   LogMsg(a_virtualCaller.Name(), "OnConnectToConferencePartyComplete");

   int res = a_virtualCaller.PrimOnConnectToConferencePartyComplete();
   if (res == 0)
   {
      res = a_virtualCaller.JoinConference();
   }
   return res;
}

//*****************************************************************************
// Function: int CVirtualCallerSetupState::JoinConference(CVirtualCaller &a_virtualCaller)
// Description: Join the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerSetupState::JoinConference(CVirtualCaller &a_virtualCaller)
{
   return a_virtualCaller.PrimJoinConference();
}

//*****************************************************************************
// Function: int CVirtualCallerSetupState::OnJoinConferenceComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a join conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerSetupState::OnJoinConferenceComplete(CVirtualCaller& a_virtualCaller)
{
   a_virtualCaller.SetStateTo(State(VirtualCallerAvailableState));
   a_virtualCaller.PrimOnJoinConferenceComplete();
   return a_virtualCaller.Play();
}

//*****************************************************************************
// Function: int CVirtualCallerPlayingState::LeaveConference(CVirtualCaller &a_virtualCaller)
// Description: Leave the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerPlayingState::LeaveConference(CVirtualCaller &a_virtualCaller)
{
   a_virtualCaller.Stop();
   a_virtualCaller.SetStateTo(State(VirtualCallerTeardownState));
   return a_virtualCaller.PrimLeaveConference();
}

//*****************************************************************************
// Function: int CVirtualCallerTeardownState::OnLeaveConferenceComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a leave conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerTeardownState::OnLeaveConferenceComplete(CVirtualCaller &a_virtualCaller)
{
   int res = a_virtualCaller.PrimOnLeaveConferenceComplete();
   if (res == 0)
   {
      res = a_virtualCaller.DisconnectFromConferenceParty();
   }
   return res;
}
//*****************************************************************************
// Function: int CVirtualCallerTeardownState::OnLeaveConferenceFailed(CVirtualCaller &a_virtualCaller)
// Description: Handle a leave conference completion event
// Return: int - 0 for success, otherwise error
// Parameters: CVirtualCaller &a_virtualCaller
//*****************************************************************************
int CVirtualCallerTeardownState::OnLeaveConferenceFailed(CVirtualCaller &a_virtualCaller)
{
   int res=0;
// SKS TODO when fail this moment focibly disconnect and close conference
	res = a_virtualCaller.PrimOnLeaveConferenceComplete();
   if (res == 0)
   {
      res = a_virtualCaller.DisconnectFromConferenceParty();
   }
   return res;

}


//*****************************************************************************
// Function: int CVirtualCallerTeardownState::DisconnectFromConferenceParty(CVirtualCaller &a_virtualCaller)
// Description: Disconnect from the conferenmce party
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerTeardownState::DisconnectFromConferenceParty(CVirtualCaller &a_virtualCaller)
{
   return a_virtualCaller.PrimDisconnectFromConferenceParty();
}

//*****************************************************************************
// Function: int CVirtualCallerTeardownState::OnCnfPortDisconnected(CVirtualCaller &a_virtualCaller)
// Description: Handle a port disconnectred event from the CNF party device
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerTeardownState::OnCnfPortDisconnected(CVirtualCaller& a_virtualCaller)
{
   if (a_virtualCaller.PrimOnCnfPortDisconnected() == CVirtualCaller::PORT_DISCONNECT_COMPLETE)
   {
      return a_virtualCaller.OnDisconnectFromConferencePartyComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCallerTeardownState::OnMmPortDisconnected(CVirtualCaller &a_virtualCaller)
// Description: Handle a port disconnectred event from the VC (MM or IPM) device
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerTeardownState::OnVcPortDisconnected(CVirtualCaller& a_virtualCaller)
{
   if (a_virtualCaller.PrimOnVcPortDisconnected() == CVirtualCaller::PORT_DISCONNECT_COMPLETE)
   {
      return a_virtualCaller.OnDisconnectFromConferencePartyComplete();
   }
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCallerTeardownState::OnDisconnectFromConferencePartyComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a disconnect from conferenmce party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerTeardownState::OnDisconnectFromConferencePartyComplete(CVirtualCaller &a_virtualCaller)
{
   int res = a_virtualCaller.PrimOnDisconnectFromConferencePartyComplete();
   if (res == 0)
   {
      res = a_virtualCaller.CloseConferenceParty();
   }
   return res;
}

//*****************************************************************************
// Function: int CVirtualCallerTeardownState::OnPlayComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a play completion event
// Return: int - 0 for success, otherwise error
// Parameters: CVirtualCaller &a_virtualCaller
//*****************************************************************************
int CVirtualCallerTeardownState::OnPlayComplete(CVirtualCaller &a_virtualCaller)
{
   return 0;
}

//*****************************************************************************
// Function: int CVirtualCallerTeardownState::CloseConferenceParty(CVirtualCaller &a_virtualCaller)
// Description: Close the conference party
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int CVirtualCallerTeardownState::CloseConferenceParty(CVirtualCaller &a_virtualCaller)
{
   return a_virtualCaller.PrimCloseConferenceParty();
}

//*****************************************************************************
// Function: int CVirtualCallerTeardownState::OnStopMediaComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a Stop Media Complete
// Return: int - 0 for success, otherwise error
// Parameters: CVirtualCaller &a_virtualCaller
//*****************************************************************************
int CVirtualCallerTeardownState::OnStopMediaComplete(CVirtualCaller &a_virtualCaller)
{
   return 0;
}

//*****************************************************************************
// Function: int AVirtualCallerState::OnCloseConferencePartyComplete(CVirtualCaller &a_virtualCaller)
// Description: Handle a close conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CVirtualCaller &a_virtualCaller 
//*****************************************************************************
int AVirtualCallerState::OnCloseConferencePartyComplete(CVirtualCaller &a_virtualCaller)
{
// SKS
	int res=a_virtualCaller.PrimOnCloseConferencePartyComplete();
	if(res==0)
	{
		a_virtualCaller.SetStateTo(State(VirtualCallerIdleState));
	}
	return res;
}



