/**
* @file CVirtualCallerState.h
* @brief Virtual Caller States - class declarations
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CVirtualCallerState_h
#define CVirtualCallerState_h

#include <map>

#include "CVirtualCaller.h"

typedef enum 
{
   VirtualCallerIdleState = 0,
   VirtualCallerSetupState,
   VirtualCallerAvailableState,
   VirtualCallerPlayingState,
   VirtualCallerTeardownState
} VirtualCallerState;

class AVirtualCallerState;

class CVirtualCallerStates
{
public:
   CVirtualCallerStates();
   virtual ~CVirtualCallerStates() {}
   AVirtualCallerState* State(VirtualCallerState a_virtualCallerState) {return m_virtualCallerStates[a_virtualCallerState];}
private:
   std::map< VirtualCallerState, AVirtualCallerState*> m_virtualCallerStates;
};

//*****************************************************************************
// Class: ::AVirtualCallerState
// Description: Abstract virtual caller state
// Notes:      
//    This abstract class declares all functions for all VirtualCaller states 
//    as virtual and implements them as an error.
// 
//    Concrete subclasses implement only the functions that are valid for that
//    state.
// 
//    State classes are responsible only for calling primative functions on the
//    context virtual caller object and changing the state of the context 
//    virtual caller object.
// 
//    As state classes have no non-static member variables, one set of virtual
//    caller state objects are shared by all virtual caller objects
//*****************************************************************************
class AVirtualCallerState
{
public:
   AVirtualCallerState() {}
   virtual ~AVirtualCallerState() {}

   virtual const char* Desc() = 0;

   virtual int StartVirtualCaller(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "StartVirtualCaller");}
   virtual int OpenConferenceParty(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OpenConferenceParty");}
   virtual int OnOpenConferencePartyComplete(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnOpenConferencePartyComplete");}
   virtual int ConnectToConferenceParty(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "ConnectToConferenceParty");}
   virtual int OnConnectToConferencePartyComplete(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnConnectToConferencePartyComplete");}
   virtual int JoinConference(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "JoinConference");}
   virtual int OnJoinConferenceComplete(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnJoinConferenceComplete");}

   virtual int OnGetVcRxPortInfoComplete(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnGetMmRxPortInfoComplete");}
   virtual int OnGetVcTxPortInfoComplete(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnGetMmTxPortInfoComplete");}
   virtual int OnGetCnfRxPortInfoComplete(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnGetCnfRxPortInfoComplete");}
   virtual int OnGetCnfTxPortInfoComplete(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnGetCnfTxPortInfoComplete");}
 
   virtual int OnCnfPortConnected(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnCnfPortConnected");}
   virtual int OnVcPortConnected(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnMmConnected");}

   virtual int OnStartMediaComplete(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnStartMediaComplete");}
   virtual int OnStopMediaComplete(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnStopMediaComplete");}

   virtual int Play(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "Play");}
   virtual int OnPlayComplete(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnPlayComplete");}
   virtual int Stop(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "Stope");}

   virtual int OnCnfPortDisconnected(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnCnfPortDisconnected");}
   virtual int OnVcPortDisconnected(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnMmPortDisconnected");}

   virtual int LeaveConference(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "LeaveConference");}
   virtual int OnLeaveConferenceComplete(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnLeaveConferenceComplete");}
   virtual int OnLeaveConferenceFailed(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnLeaveConferenceFailed");}
   virtual int DisconnectFromConferenceParty(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "DisconnectFromConferenceParty");}
   virtual int OnDisconnectFromConferencePartyComplete(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "OnDisconnectFromConferencePartyComplete");}
   virtual int CloseConferenceParty(CVirtualCaller& a_virtualCaller) {return LogStateError(a_virtualCaller, "CloseConferenceParty");}
   virtual int OnCloseConferencePartyComplete(CVirtualCaller& a_virtualCaller);

   static AVirtualCallerState* State(VirtualCallerState a_virtualCallerState) {return m_virtualCallerStates.State(a_virtualCallerState);}

private:
   int LogStateError(CVirtualCaller& a_virtualCaller, const std::string& msg);
   static CVirtualCallerStates m_virtualCallerStates;
};

//*****************************************************************************
// Class: ::CVirtualCallerIdleState
// Description: Virtual caller idle state
//*****************************************************************************
class CVirtualCallerIdleState : public AVirtualCallerState
{
public:
   CVirtualCallerIdleState() {}
   int OnPlayComplete(CVirtualCaller& a_virtualCaller);
   virtual ~CVirtualCallerIdleState() {}
   const char* Desc() {return "idle";}
   int StartVirtualCaller(CVirtualCaller& a_virtualCaller);
};

//*****************************************************************************
// Class: ::CVirtualCallerSetupState
// Description: Virtual caller setup state
//*****************************************************************************
class CVirtualCallerSetupState : public AVirtualCallerState
{
public:
   CVirtualCallerSetupState() {}
   virtual ~CVirtualCallerSetupState() {}
   const char* Desc() {return "setup";}
   int OpenConferenceParty(CVirtualCaller& a_virtualCaller);
   int OnOpenConferencePartyComplete(CVirtualCaller& a_virtualCaller);
   int ConnectToConferenceParty(CVirtualCaller& a_virtualCaller);
   int OnConnectToConferencePartyComplete(CVirtualCaller& a_virtualCaller);
   int JoinConference(CVirtualCaller& a_virtualCaller);
   int OnJoinConferenceComplete(CVirtualCaller& a_virtualCaller);
   int OnGetVcRxPortInfoComplete(CVirtualCaller& a_virtualCaller);
   int OnGetVcTxPortInfoComplete(CVirtualCaller& a_virtualCaller);
   int OnGetCnfRxPortInfoComplete(CVirtualCaller& a_virtualCaller);
   int OnGetCnfTxPortInfoComplete(CVirtualCaller& a_virtualCaller);
   int OnCnfPortConnected(CVirtualCaller& a_virtualCaller);
   int OnVcPortConnected(CVirtualCaller& a_virtualCaller);
};

//*****************************************************************************
// Class: ::CVirtualCallerAvailableState
// Description: Virtual caller available state
//*****************************************************************************
class CVirtualCallerAvailableState : public AVirtualCallerState
{
public:
   CVirtualCallerAvailableState() {}
   virtual ~CVirtualCallerAvailableState() {}
   const char* Desc() {return "Available";}
   int Play(CVirtualCaller& a_virtualCaller);
   int LeaveConference(CVirtualCaller& a_virtualCaller);
};

//*****************************************************************************
// Class: ::CVirtualCallerPlayingState
// Description: Virtual caller p[laying state
//*****************************************************************************
class CVirtualCallerPlayingState : public AVirtualCallerState
{
public:
   CVirtualCallerPlayingState() {}
   virtual ~CVirtualCallerPlayingState() {}
   const char* Desc() {return "playing";}
   int OnPlayComplete(CVirtualCaller& a_virtualCaller);
   int Stop(CVirtualCaller& a_virtualCaller);
   int LeaveConference(CVirtualCaller& a_virtualCaller);
   int OnStartMediaComplete(CVirtualCaller& a_virtualCaller);
};

//*****************************************************************************
// Class: ::CVirtualCallerTeardownState
// Description: Virtual caller teardown state
//*****************************************************************************
class CVirtualCallerTeardownState : public AVirtualCallerState
{
public:
   CVirtualCallerTeardownState() {}
   virtual ~CVirtualCallerTeardownState() {}
   const char* Desc() {return "teardown";}
   int OnCnfPortDisconnected(CVirtualCaller& a_virtualCaller);
   int OnVcPortDisconnected(CVirtualCaller& a_virtualCaller);
   int OnLeaveConferenceComplete(CVirtualCaller& a_virtualCaller);
   int OnLeaveConferenceFailed(CVirtualCaller& a_virtualCaller);
   int DisconnectFromConferenceParty(CVirtualCaller& a_virtualCaller);
   int OnDisconnectFromConferencePartyComplete(CVirtualCaller& a_virtualCaller);
   int CloseConferenceParty(CVirtualCaller& a_virtualCaller);
   int OnPlayComplete(CVirtualCaller& a_virtualCaller);
   int OnStopMediaComplete(CVirtualCaller& a_virtualCaller);
};

#endif

