/**
* @file LogUtil.cpp
* @brief Logging Utility implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#include "LogUtil.h"

#ifdef WIN32

#pragma warning(disable:4786)
#include <windows.h>

// Standard Include Files
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <sys/timeb.h>
#include <string>
#include <vector>
#include <queue>
#include <map>
using namespace std;

#else
#include <iostream>
#include <iomanip>
#include <time.h>
#include <sys/timeb.h>
#include <assert.h>
#endif



using std::ostringstream;
using std::stringstream;
using std::cout;
using std::endl;
using std::setw;
using std::setfill;
using std::left;
using std::hex;
using std::dec;

std::ofstream g_logfile;

//*****************************************************************************
// Function: void ::LogMsg(const string &msg)
// Description: Log a string message
// Return: void 
// Parameters: const string &msg 
//*****************************************************************************
void LogMsg(const string& msg)
{
   stringstream logMsg;
   logMsg << msg << endl;
   WriteToLog(logMsg);
}

//*****************************************************************************
// Function: void ::LogMsg(const int dev, const string &msg)
// Description: Log a message including a device name determined from a device handle
// Return: void 
// Parameters: const int dev 
//             const string &msg 
//*****************************************************************************
void LogMsg(const int dev, const string& msg)
{
   stringstream logMsg;
   logMsg << setw(12) << left << ATDV_NAMEP((LINEDEV) dev) << msg << endl;
   WriteToLog(logMsg);
}

//*****************************************************************************
// Function: void ::LogMsg(const string &devName, const string &msg)
// Description: Log a message including a given device name
// Return: void 
// Parameters: const string &devName 
//             const string &msg 
//*****************************************************************************
void LogMsg(const string& devName, const string& msg)
{
   stringstream logMsg;
   logMsg << setw(12) << left << devName << msg << endl;
   WriteToLog(logMsg);
}

//*****************************************************************************
// Function: int ::GcInfoMsg(const GC_INFO &gcInfo, const LINEDEV gcDev, const string &msgType, const string &msg)
// Description: Log a message including the device name determined from a device handle,
//              a message type, and information from a GlobalCall GC_INFO structure
// Return: int - 0 for success, otherwise error 
// Parameters: const GC_INFO &gcInfo 
//             const LINEDEV gcDev 
//             const string &msgType 
//             const string &msg 
//*****************************************************************************
int GcInfoMsg(const GC_INFO& gcInfo, const LINEDEV gcDev, const string& msgType, const string& msg) 
{
   stringstream logMsg;
   logMsg << setw(12) << left << ATDV_NAMEP(gcDev);
   if (msg.size())
   {
      logMsg << " " << msg << "\n\t" << msgType;
   }
   else
   {
      logMsg << "\t" << msgType;
   }
   logMsg 
   << "\n\t\tgcValue  : 0x" << hex << gcInfo.gcValue
   << "\n\t\tgcMsg    : " << gcInfo.gcMsg
   << "\n\t\tccLibId  : " << dec << gcInfo.ccLibId
   << "\n\t\tccLibName: " << gcInfo.ccLibName
   << "\n\t\tccValue  : 0x" << hex << gcInfo.ccValue
   << "\n\t\tccMsg    : " << gcInfo.ccMsg
   << "\n\t\tinfo     : " << gcInfo.additionalInfo
   << endl;
   WriteToLog(logMsg);
   return gcInfo.gcValue;
}

//*****************************************************************************
// Function: void ::GcResultMsg(const LINEDEV gcDev, const METAEVENT &metaevent)
// Description: Log GC result info from a metaevent
// Return: void 
// Parameters: const LINEDEV gcDev 
//             const METAEVENT &metaevent 
//*****************************************************************************
void GcResultMsg(const LINEDEV gcDev, const METAEVENT& metaevent)
{
   GC_INFO gcInfo;
   gc_ResultInfo(const_cast< METAEVENT*> (&metaevent), &gcInfo);
   GcInfoMsg(gcInfo, gcDev, "GC Result:", "");
}

//*****************************************************************************
// Function: int ::GcErrMsg(const LINEDEV gcDev, const string &msg)
// Description: Log a Global Call error message
// Return: int - 0 for success, otherwise error 
// Parameters: const LINEDEV gcDev 
//             const string &msg 
//*****************************************************************************
int GcErrMsg(const LINEDEV gcDev, const string& msg) 
{
   GC_INFO gcInfo;
   if (gc_ErrorInfo(&gcInfo) != GC_SUCCESS)
   {
      stringstream logMsg;
      logMsg << setw(12) << left << ATDV_NAMEP(gcDev) 
      << msg  << " *  Unable to retrieve error info" << endl;
      WriteToLog(logMsg);
      return -1;
   }
   return GcInfoMsg(gcInfo, gcDev, "GC Error:", msg);
}

//*****************************************************************************
// Function: int ::ErrMsg(const int dev, const string &msg)
// Description: Log a message for the last error on a device, including the 
//              device name determined from the device handle
// Return: int - 0 for success, otherwise error 
// Parameters: const int dev 
//             const string &msg 
//*****************************************************************************
int ErrMsg(const int dev, const string& msg)
{
   int err = ATDV_LASTERR(dev);
   stringstream logMsg;
   logMsg << setw(12) << left <<ATDV_NAMEP(dev) 
   << "ERROR: " << msg
   << " error=0x" 
   << hex << err 
   << " " << ATDV_ERRMSGP(dev) 
   << endl;
   WriteToLog(logMsg);
   return err;
}

//*****************************************************************************
// Function: void ::WriteToLog(stringstream &msg)
// Description: Write a log message to console and file
// Return: void 
// Parameters: stringstream &msg 
//*****************************************************************************
void WriteToLog(stringstream& msg)
{
   struct timeb l_timebuffer;
   ftime( &l_timebuffer );
   struct tm* l_tm_data = localtime( &l_timebuffer.time);
   stringstream timeMsg;
   timeMsg 
   << setw(2) << setfill('0') << l_tm_data->tm_mon+1 
   << "/" << setw(2) << setfill('0') << l_tm_data->tm_mday 
   << "/" << setw(2) << setfill('0') << (( l_tm_data->tm_year + 1900) % 100)
   << " " << setw(2) << setfill('0') << l_tm_data->tm_hour 
   << ":" << setw(2) << setfill('0') << l_tm_data->tm_min 
   << ":" << setw(2) << setfill('0') << l_tm_data->tm_sec 
   << "." << setw(3) << setfill('0') << l_timebuffer.millitm;

   cout << timeMsg.str() << " " << msg.str();
   if (g_logfile.is_open())
   {
      g_logfile << timeMsg.str() << " " << msg.str();
   }
}

