/**
* @file MediaCoders.cpp
* @brief Media Coders - class implementations
* @date Mar. 24, 2008
*
* DIALOGIC CONFIDENTIAL
* Copyright  2008 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#pragma warning( push )
#pragma warning (disable:4996)

#include <ipmlib.h>

#include "LogUtil.h"
#include "MediaCoders.h"


//*****************************************************************************
// Function: void AMediaCoder::SetRtpPortAndIpAddress(long &a_rtpPort, const std::string &a_rtpAddress)
// Description: Set the port and IP address for the RTP stream used by the coder
// Return: void 
// Parameters: long &a_rtpPort 
//             const std::string &a_rtpAddress 
//*****************************************************************************
void AMediaCoder::SetRtpPortAndIpAddress(long& a_rtpPort, const std::string& a_rtpAddress)
{
   m_rtpPort = a_rtpPort;
   m_rtpAddress = a_rtpAddress;
}

//*****************************************************************************
// Function: AAudioCoder::AAudioCoder(const char *a_payloadCoderName, unsigned int a_payloadType, const IPM_AUDIO_CODER_INFO &l_audioCoderInfo)
// Description: Initializing constructor
// Return:  
// Parameters: const char *a_payloadCoderName 
//             unsigned int a_payloadType 
//             const IPM_AUDIO_CODER_INFO &l_audioCoderInfo 
//*****************************************************************************
AAudioCoder::AAudioCoder(const char* a_payloadCoderName, unsigned int a_payloadType, const IPM_AUDIO_CODER_INFO& l_audioCoderInfo)
   :AMediaCoder("audio",a_payloadCoderName,a_payloadType, strcmp(a_payloadCoderName, "AMR-WB")?8000:16000)
   ,m_channel(1)
   ,m_matchedVAD(CODER_VAD_DISABLE)
   ,m_matchedPtime(0)
{
   AddVadCapability(l_audioCoderInfo.eVadEnable);
   AddFrameSizeCapability(l_audioCoderInfo.eFrameSize);
   AddFramesPerPacketCapability(l_audioCoderInfo.unFramesPerPkt);
   AddPtimeCapability(l_audioCoderInfo.eFrameSize * l_audioCoderInfo.unFramesPerPkt);
}

//*****************************************************************************
// Function: bool AAudioCoder::AddVadCapability(const eIPM_CODER_VAD a_vadCapability)
// Description: Add a Voice Activity Detection capability to the list of voice 
// 				activity detectiuon capabilities for the coder
// Return: bool 
// Parameters: const eIPM_CODER_VAD a_vadCapability 
//*****************************************************************************
bool AAudioCoder::AddVadCapability(const eIPM_CODER_VAD a_vadCapability)
{
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      if(a_vadCapability == m_vadCapabilities[i])
      {
         return false;
      }
   }
   m_vadCapabilities.push_back(a_vadCapability);
   return true;
}

//*****************************************************************************
// Function: bool AAudioCoder::DeleteVadCapability(const eIPM_CODER_VAD a_vadCapability)
// Description: Delete a Voice Activity Detection capability from the list of 
// 				voice activity detectiuon capabilities for the coder
// Return: bool 
// Parameters: const eIPM_CODER_VAD a_vadCapability 
//*****************************************************************************
bool AAudioCoder::DeleteVadCapability(const eIPM_CODER_VAD a_vadCapability)
{

   std::vector<eIPM_CODER_VAD>::iterator l_iterator;

   for(l_iterator = m_vadCapabilities.begin(); 
       l_iterator != m_vadCapabilities.end(); l_iterator++)
   {
      if (a_vadCapability == *l_iterator)
      {
         m_vadCapabilities.erase(l_iterator);
         return true;
      }
   }
   return false;
}

//*****************************************************************************
// Function: bool AAudioCoder::HasVadCapability(const eIPM_CODER_VAD a_vadCapability)
// Description: Test if the coder has the specified voice avtivity detection 
// 				capability
// Return: bool 
// Parameters: const eIPM_CODER_VAD a_vadCapability 
//*****************************************************************************
bool AAudioCoder::HasVadCapability(const eIPM_CODER_VAD a_vadCapability)
{
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      if(m_vadCapabilities[i] == a_vadCapability)
      {
         return true;
      }
   }
   return false;
}

//*****************************************************************************
// Function: bool AAudioCoder::AddPtimeCapability(const unsigned int a_ptimeCapability)
// Description: Add a packet time capability to the list of packet time 
// 				capabilities for the coder
// Return: bool 
// Parameters: const unsigned int a_ptimeCapability 
//*****************************************************************************
bool AAudioCoder::AddPtimeCapability(const unsigned int a_ptimeCapability)
{
   for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
   {
      if(a_ptimeCapability == m_ptimeCapabilities[i])
      {
         return false;
      }
   }
   m_ptimeCapabilities.push_back(a_ptimeCapability);
   return true;
}

//*****************************************************************************
// Function: bool AAudioCoder::HasPtimeCapability(const unsigned int a_ptimeCapability)
// Description: Test if the coder has the specified packet time capability
// Return: bool 
// Parameters: const unsigned int a_ptimeCapability 
//*****************************************************************************
bool AAudioCoder::HasPtimeCapability(const unsigned int a_ptimeCapability)
{
   for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
   {
      if(m_ptimeCapabilities[i] == a_ptimeCapability)
      {
         return true;
      }
   }
   return false;
}

//*****************************************************************************
// Function: bool AAudioCoder::AddFramesPerPacketCapability(const unsigned int a_framesPerPacketCapability)
// Description: Add a frames per packet capability to the list of frames per 
// 				packet capabilities for the coder.
// Return: bool 
// Parameters: const unsigned int a_framesPerPacketCapability 
//*****************************************************************************
bool AAudioCoder::AddFramesPerPacketCapability(const unsigned int a_framesPerPacketCapability)
{
   for(unsigned int i=0; i < m_framesPerPacketCapabilities.size(); i++)
   {
      if(a_framesPerPacketCapability == m_framesPerPacketCapabilities[i])
      {
         return false;
      }
   }
   m_framesPerPacketCapabilities.push_back(a_framesPerPacketCapability);
   return true;
}

//*****************************************************************************
// Function: bool AAudioCoder::AddFrameSizeCapability(const eIPM_CODER_FRAMESIZE a_frameSizeCapability)
// Description: Add a frame size capability to the list of frame size 
// 				capabilities for the coder.
// Return: bool 
// Parameters: const eIPM_CODER_FRAMESIZE a_frameSizeCapability 
//*****************************************************************************
bool AAudioCoder::AddFrameSizeCapability(const eIPM_CODER_FRAMESIZE  a_frameSizeCapability)
{
   for(unsigned int i=0; i < m_frameSizeCapabilities.size(); i++)
   {
      if(a_frameSizeCapability == m_frameSizeCapabilities[i])
      {
         return false;
      }
   }
   m_frameSizeCapabilities.push_back(a_frameSizeCapability);
   return true;
}

//*****************************************************************************
// Function: void AAudioCoder::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void AAudioCoder::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   char l_buffer[COMMON_STRING_MAX_LENGTH] = {0};

   sdpMedia* l_media = a_mediaDescription->media();
   l_media->setMedia("audio");
   l_media->setPort(a_rtpPort);
   l_media->setTransport("RTP/AVP");
   l_media->setNumPorts(1);
   sprintf(l_buffer,"%d", m_payloadType);
   l_media->addFormat(l_buffer);

   sdpAttribute* l_attribute = a_mediaDescription->attributeList()->addItem();
   l_attribute->setProperty("rtpmap");
   sprintf(l_buffer, "%d %s/%d/%d", m_payloadType, GetPayloadCoderName(), m_clockRate, m_channel);
   l_attribute->setPropertyValue(l_buffer);

   l_attribute = a_mediaDescription->attributeList()->addItem();
   l_attribute->setProperty("sendrecv");

   l_attribute = a_mediaDescription->attributeList()->addItem();
   l_attribute->setProperty("ptime");
   sprintf(l_buffer, "%d", m_matchedPtime);
   l_attribute->setPropertyValue(l_buffer);
}

//*****************************************************************************
// Function: CMediaCoderG711a::CMediaCoderG711a()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CMediaCoderG711a::CMediaCoderG711a()
   :AAudioCoder("PCMA",8)
{
   m_dialogicCoderType = CODER_TYPE_G711ALAW64K;
   AddVadCapability(CODER_VAD_DISABLE);
}

//*****************************************************************************
// Function: CMediaCoderG711a::CMediaCoderG711a(const IPM_AUDIO_CODER_INFO &l_audioCoderInfo)
// Description: Initializing constructor
// Return:  
// Parameters: const IPM_AUDIO_CODER_INFO &l_audioCoderInfo 
//*****************************************************************************
CMediaCoderG711a::CMediaCoderG711a(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo)
   :AAudioCoder("PCMA",8, l_audioCoderInfo)
{
   m_dialogicCoderType = CODER_TYPE_G711ALAW64K;
}

//*****************************************************************************
// Function: void CMediaCoderG711a::ImportSdpFmtp(const std::string &a_attribute)
// Description: Import coder attributes from SDP
// Return: void 
// Parameters: const std::string &a_attribute 
//*****************************************************************************
void CMediaCoderG711a::ImportSdpFmtp(const std::string& a_attribute)
{
//   printf("CMediaCoderG711a::ImportSdpFmtp\n");
   unsigned int l_payloadType, l_ptime, l_tokenCount=0;
   eIPM_CODER_FRAMESIZE l_frameSize;
   char l_separators[] = "= ;";
   char *l_token=NULL;
   char l_subAttribute[COMMON_STRING_MAX_LENGTH];
   char l_subAttributeValue[MEDIA_ATTRIB_SIZE_MAX];
   char l_attribute[MEDIA_ATTRIB_SIZE_MAX];
   char* l_tokenizerBuffer = NULL;

   size_t l_attributeSize = a_attribute.size();
   if(l_attributeSize >= MEDIA_ATTRIB_SIZE_MAX)
   {
      printf("CMediaCoderG711a::ImportSdpFmtp - Attribute.size()= %u is greater than MEDIA_ATTRIB_SIZE_MAX=%u\n",
                         a_attribute.size(),
                         MEDIA_ATTRIB_SIZE_MAX);
      return;
   }

   strcpy(l_attribute, a_attribute.c_str());
   l_attribute[l_attributeSize] = '\0';

   memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
   memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);

   l_token = DLGC_STRTOK(l_attribute, l_separators, &l_tokenizerBuffer);
   while (l_token != NULL)
   {
      switch (l_tokenCount)
      {
      case 0:
         l_payloadType = atoi(l_token);
         if (l_payloadType != m_payloadType)
         {
            printf("CG711a::ImportSdpFmtp - Payload Number Mismatch\n");
            return;
         }
         break;

      case 1:
      case 3:
      case 5:
         strcpy(l_subAttribute, l_token);
         break;

      case 2:
      case 4:
      case 6:
         strcpy(l_subAttributeValue, l_token);
         if (!strcmp(l_subAttribute, "ptime"))
         {
            l_ptime = atoi(l_subAttributeValue);

            //Store the packetization interval
            if(AddPtimeCapability(l_ptime) == true)
            {
               // convert to fpp and frame size

               // We currently support 1 FPP.
               //Insert the frames per packet value. 
               AddFramesPerPacketCapability(G711A_FPP);

               l_frameSize = static_cast<eIPM_CODER_FRAMESIZE>(l_ptime / G711A_FPP);
               AddFrameSizeCapability(l_frameSize);
            }
         }
         memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
         memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);
         break;

      default:
         break;
      }

      l_token = DLGC_STRTOK(NULL, l_separators, &l_tokenizerBuffer);
      l_tokenCount++;
   }
}

//*****************************************************************************
// Function: void CMediaCoderG711a::ListProperties()
// Description: List the coder properties
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderG711a::ListProperties()
{
   printf("\t%s\n", GetDialogicCoderTypeDesc(m_dialogicCoderType));
   printf("\tm=%s %s:%ld plt=%d, peerplt=%d\n", 
      m_mediaType.c_str(), m_rtpAddress.c_str(), m_rtpPort, m_payloadType, m_peerPayloadType);
   printf("\ta=rtpmap:%d %s/%d/%d\n",m_payloadType, m_payloadCoderName.c_str(), m_clockRate, m_channel);
   printf("\ta=ptime:");

   if(m_ptimeCapabilities.size() == 0)
   {
      printf(" any");
   }
   else
   {
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         printf(" %d", m_ptimeCapabilities[i]);
      }
   }
   printf("\n");

   printf("\tvad:");
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      printf(" %s", (m_vadCapabilities[i]==1)?"yes":"no");
   }
   printf("\n");

   if (m_matchedPtime)
   {
      printf("\tMatchedCaps:\n");
      printf("\t\tDialogic Resource Type: %s\n", GetDialogicResourceTypeDesc(m_dialogicResourceType));
      printf("\t\tvad=%d\n", GetMatchedCoderVAD());
      printf("\t\tptime=%d\n", GetMatchedCoderPtime());
      printf("\t\tfpp=%d\n", GetMatchedCoderFramesPerPacket());
      printf("\t\tframe size=%d\n", GetMatchedCoderFrameSize());
   }
}

//*****************************************************************************
// Function: bool CMediaCoderG711a::MatchCoder(AMediaCoder *a_otherMediaCoder)
// Description: Attempt to match the specified coder and return the result.
// Return: bool 
// Parameters: AMediaCoder *a_otherMediaCoder 
//*****************************************************************************
bool CMediaCoderG711a::MatchCoder(AMediaCoder* a_otherMediaCoder)
{
   m_dialogicResourceType = RESOURCE_TYPE_NONE;
   CMediaCoderG711a* l_otherMediaCoder = dynamic_cast<CMediaCoderG711a*>(a_otherMediaCoder);
   if (!l_otherMediaCoder)
   {
      return false;
   }

   if (strcmp(GetPayloadCoderName(), l_otherMediaCoder->GetPayloadCoderName()))
   {
      return false;
   }

   if (GetDialogicCoderType() != l_otherMediaCoder->GetDialogicCoderType())
   {
      return false;
   }

   if (GetClockRate() !=  l_otherMediaCoder->GetClockRate())
   {
      return false;
   }

   if (GetChannel() !=  l_otherMediaCoder->GetChannel())
   {
      return false;
   }

   bool l_haveMatchedPtime(false);
   if(m_ptimeCapabilities.size() == 0)
   {
      // ptime is unspecified, use default from other coder
      m_matchedPtime = l_otherMediaCoder->GetDefaultPtime();
      l_haveMatchedPtime = true;
   }
   else
   {
      // find first match in other coder
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         if (l_otherMediaCoder->HasPtimeCapability(m_ptimeCapabilities[i]))
         {
            m_matchedPtime = m_ptimeCapabilities[i];
            l_haveMatchedPtime = true;
            break;
         }
      }
   }

   bool l_haveMatchedVAD(false);
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      // find first VAD capability match in other coder
      if (l_otherMediaCoder->HasVadCapability(m_vadCapabilities[i]))
      {
         m_matchedVAD = m_vadCapabilities[i];
         l_haveMatchedVAD = true;
         break;
      }
   }

   if (l_haveMatchedPtime && l_haveMatchedVAD)
   {
      switch(m_matchedPtime)
      {
      case 10:
         m_dialogicResourceType = RESOURCE_IPM_G711_10MS;
         break;
      case 20:
         m_dialogicResourceType = RESOURCE_IPM_G711_20MS;
         break;
      case 30:
         m_dialogicResourceType = RESOURCE_IPM_G711_30MS;
         break;
      default:
         m_dialogicResourceType = RESOURCE_IPM_G711_40MS;
         break;
      }
      return true;
   }
   return false;
}

//*****************************************************************************
// Function: void CMediaCoderG711a::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void CMediaCoderG711a::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   AAudioCoder::ExportSdpMediaDescription(a_mediaDescription,a_rtpPort);
}

//*****************************************************************************
// Function: CMediaCoderG711u::CMediaCoderG711u()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CMediaCoderG711u::CMediaCoderG711u()
   :AAudioCoder("PCMU",0)
{
   m_dialogicCoderType = CODER_TYPE_G711ULAW64K;
   AddVadCapability(CODER_VAD_DISABLE);
}

//*****************************************************************************
// Function: CMediaCoderG711u::CMediaCoderG711u(const IPM_AUDIO_CODER_INFO &l_audioCoderInfo)
// Description: Initializing constructor
// Return:  
// Parameters: const IPM_AUDIO_CODER_INFO &l_audioCoderInfo 
//*****************************************************************************
CMediaCoderG711u::CMediaCoderG711u(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo)
   :AAudioCoder("PCMU",0, l_audioCoderInfo)
{
   m_dialogicCoderType = CODER_TYPE_G711ULAW64K;
}

//*****************************************************************************
// Function: void CMediaCoderG711u::ImportSdpFmtp(const std::string &a_attribute)
// Description: Import coder attributes from SDP
// Return: void 
// Parameters: const std::string &a_attribute 
//*****************************************************************************
void CMediaCoderG711u::ImportSdpFmtp(const std::string& a_attribute)
{
//   printf("CMediaCoderG711u::ImportSdpFmtp\n");
   unsigned int l_payloadType, l_ptime, l_tokenCount=0;
   eIPM_CODER_FRAMESIZE l_frameSize;
   char l_separators[] = "= ;";
   char *l_token=NULL;
   char l_subAttribute[COMMON_STRING_MAX_LENGTH];
   char l_subAttributeValue[MEDIA_ATTRIB_SIZE_MAX];
   char l_attribute[MEDIA_ATTRIB_SIZE_MAX];
   char* l_tokenizerBuffer = NULL;

   size_t l_attributeSize = a_attribute.size();
   if(l_attributeSize >= MEDIA_ATTRIB_SIZE_MAX)
   {
      printf("CMediaCoderG711u::ImportSdpFmtp - Attribute.size()= %u is greater than MEDIA_ATTRIB_SIZE_MAX=%u\n",
                         a_attribute.size(),
                         MEDIA_ATTRIB_SIZE_MAX);
      return;
   }

   strcpy(l_attribute, a_attribute.c_str());
   l_attribute[l_attributeSize] = '\0';

   memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
   memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);

   l_token = DLGC_STRTOK(l_attribute, l_separators, &l_tokenizerBuffer);
   while (l_token != NULL)
   {
      switch (l_tokenCount)
      {
      case 0:
         l_payloadType = atoi(l_token);
         if (l_payloadType != m_payloadType)
         {
            printf("CG711u::ImportSdpFmtp - Payload Number Mismatch\n");
            return;
         }
         break;

      case 1:
      case 3:
      case 5:
         strcpy(l_subAttribute, l_token);
         break;

      case 2:
      case 4:
      case 6:
         strcpy(l_subAttributeValue, l_token);
         if (!strcmp(l_subAttribute, "ptime"))
         {
            l_ptime = atoi(l_subAttributeValue);

            //Store the packetization interval; l_uiPTime. 
            if(AddPtimeCapability(l_ptime) == true)
            {
               // convert to fpp and frame size
               // We currently support 1 FPP.
               //Insert the frames per packet value. 
               AddFramesPerPacketCapability(G711U_FPP);
               l_frameSize = static_cast<eIPM_CODER_FRAMESIZE>(l_ptime / G711U_FPP);
               AddFrameSizeCapability(l_frameSize);
            }
         }
         memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
         memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);
         break;

      default:
         break;
      }

      l_token = DLGC_STRTOK(NULL, l_separators, &l_tokenizerBuffer);
      l_tokenCount++;
   }
}

//*****************************************************************************
// Function: void CMediaCoderG711u::ListProperties()
// Description: List the coder properties
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderG711u::ListProperties()
{
   printf("\t%s\n", GetDialogicCoderTypeDesc(m_dialogicCoderType));
   printf("\tm=%s %s:%ld plt=%d, peerplt=%d\n", 
      m_mediaType.c_str(), m_rtpAddress.c_str(), m_rtpPort, m_payloadType, m_peerPayloadType);
   printf("\ta=rtpmap:%d %s/%d/%d\n",m_payloadType, m_payloadCoderName.c_str(), m_clockRate, m_channel);
   printf("\ta=ptime:");

   if(m_ptimeCapabilities.size() == 0)
   {
      printf(" any");
   }
   else
   {
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         printf(" %d", m_ptimeCapabilities[i]);
      }
   }
   printf("\n");

   printf("\tvad:");
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      printf(" %s", (m_vadCapabilities[i]==1)?"yes":"no");
   }
   printf("\n");

   if (m_matchedPtime)
   {
      printf("\tMatchedCaps:\n");
      printf("\t\tDialogic Resource Type: %s\n", GetDialogicResourceTypeDesc(m_dialogicResourceType));
      printf("\t\tvad=%d\n", GetMatchedCoderVAD());
      printf("\t\tptime=%d\n", GetMatchedCoderPtime());
      printf("\t\tfpp=%d\n", GetMatchedCoderFramesPerPacket());
      printf("\t\tframe size=%d\n", GetMatchedCoderFrameSize());
   }
}

//*****************************************************************************
// Function: bool CMediaCoderG711u::MatchCoder(AMediaCoder *a_otherMediaCoder)
// Description: Attempt to match the specified coder and return the result.
// Return: bool 
// Parameters: AMediaCoder *a_otherMediaCoder 
//*****************************************************************************
bool CMediaCoderG711u::MatchCoder(AMediaCoder* a_otherMediaCoder)
{
   m_dialogicResourceType = RESOURCE_TYPE_NONE;
   CMediaCoderG711u* l_otherMediaCoder = dynamic_cast<CMediaCoderG711u*>(a_otherMediaCoder);
   if (!l_otherMediaCoder)
   {
      return false;
   }

   if (strcmp(GetPayloadCoderName(), l_otherMediaCoder->GetPayloadCoderName()))
   {
      return false;
   }

   if (GetDialogicCoderType() != l_otherMediaCoder->GetDialogicCoderType())
   {
      return false;
   }

   if (GetClockRate() !=  l_otherMediaCoder->GetClockRate())
   {
      return false;
   }

   if (GetChannel() !=  l_otherMediaCoder->GetChannel())
   {
      return false;
   }

   bool l_haveMatchedPtime(false);
   if(m_ptimeCapabilities.size() == 0)
   {
      // ptime is unspecified, use default from other coder
      m_matchedPtime = l_otherMediaCoder->GetDefaultPtime();
      l_haveMatchedPtime = true;
   }
   else
   {
      // find first match in other coder
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         if (l_otherMediaCoder->HasPtimeCapability(m_ptimeCapabilities[i]))
         {
            m_matchedPtime = m_ptimeCapabilities[i];
            l_haveMatchedPtime = true;
            break;
         }
      }
   }

   bool l_haveMatchedVAD(false);
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      // find first VAD capability match in other coder
      if (l_otherMediaCoder->HasVadCapability(m_vadCapabilities[i]))
      {
         m_matchedVAD = m_vadCapabilities[i];
         l_haveMatchedVAD = true;
         break;
      }
   }

   if (l_haveMatchedPtime && l_haveMatchedVAD)
   {
      switch(m_matchedPtime)
      {
      case 10:
         m_dialogicResourceType = RESOURCE_IPM_G711_10MS;
         break;
      case 20:
         m_dialogicResourceType = RESOURCE_IPM_G711_20MS;
         break;
      case 30:
         m_dialogicResourceType = RESOURCE_IPM_G711_30MS;
         break;
      default:
         m_dialogicResourceType = RESOURCE_IPM_G711_40MS;
         break;
      }
      return true;
   }
   return false;
}

//*****************************************************************************
// Function: void CMediaCoderG711u::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void CMediaCoderG711u::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   AAudioCoder::ExportSdpMediaDescription(a_mediaDescription,a_rtpPort);
}

//*****************************************************************************
// Function: CMediaCoderG723::CMediaCoderG723()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CMediaCoderG723::CMediaCoderG723()
   :AAudioCoder("G723",4)
{
   m_dialogicCoderType = CODER_TYPE_G7231_6_3K;   // 6.3k default rate
}

//*****************************************************************************
// Function: CMediaCoderG723::CMediaCoderG723(const IPM_AUDIO_CODER_INFO &l_audioCoderInfo)
// Description: Initializing constructor
// Return:  
// Parameters: const IPM_AUDIO_CODER_INFO &l_audioCoderInfo 
//*****************************************************************************
CMediaCoderG723::CMediaCoderG723(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo)
   :AAudioCoder("G723",4, l_audioCoderInfo)
{
   m_dialogicCoderType = l_audioCoderInfo.eCoderType;
}

//*****************************************************************************
// Function: void CMediaCoderG723::ImportSdpFmtp(const std::string &a_attribute)
// Description: Import coder attributes from SDP
// Return: void 
// Parameters: const std::string &a_attribute 
//*****************************************************************************
void CMediaCoderG723::ImportSdpFmtp(const std::string& a_attribute)
{
//   printf("CMediaCoderG723::ImportSdpFmtp\n");
   unsigned int l_payloadType, l_ptime, l_framesPerPacket, l_tokenCount=0;
   double l_bitrate;
   char l_separators[] = "= ;";
   char* l_token = NULL;
   char l_subAttribute[COMMON_STRING_MAX_LENGTH];
   char l_subAttributeValue[MEDIA_ATTRIB_SIZE_MAX];
   char l_attribute[MEDIA_ATTRIB_SIZE_MAX];
   char* l_tokenizerBuffer = NULL;

   size_t l_attributeSize = a_attribute.size();

   if(l_attributeSize >= MEDIA_ATTRIB_SIZE_MAX)
   {
      printf("CMediaCoderG723::ImportSdpFmtp - Attribute.size()= %u is greater than MEDIA_ATTRIB_SIZE_MAX=%u\n",
                         a_attribute.size(),
                         MEDIA_ATTRIB_SIZE_MAX);
      return;
   }

   strcpy(l_attribute, a_attribute.c_str());
   l_attribute[l_attributeSize] = '\0';

   memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
   memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);

   l_token = DLGC_STRTOK(l_attribute, l_separators, &l_tokenizerBuffer);
   while (l_token != NULL)
   {
      switch (l_tokenCount)
      {
      case 0:
         l_payloadType = atoi(l_token);
         if (l_payloadType != m_payloadType)
         {
            printf("CMediaCoderG723::ImportSdpFmtp - Payload Number Mismatch\n");
            return;
         }
         break;

      case 1:
      case 3:
      case 5:
         strcpy(l_subAttribute, l_token);
         break;

      case 2:
      case 4:
      case 6:
         strcpy(l_subAttributeValue, l_token);
         if (!strcmp(l_subAttribute, "bitrate"))
         {
            l_bitrate = atof(l_subAttributeValue);
            if(l_bitrate==G723_5_3 || l_bitrate==G723_6_3)
            {
               AddBitRateCapability(l_bitrate);
            }
            else
            {
               printf("CMediaCoderG723::ImportSdpFmtp - Unsupported bitrate\n");
            }
         }
         else if (!strcmp(l_subAttribute, "annexa"))
         {
            if(!strcmp(l_subAttributeValue, "yes"))
            {
               AddVadCapability(CODER_VAD_ENABLE);
            }
            else if (!strcmp(l_subAttributeValue, "no"))
            {
               DeleteVadCapability(CODER_VAD_ENABLE);
               AddVadCapability(CODER_VAD_DISABLE);
            }

         }
         else if (!strcmp(l_subAttribute, "ptime"))
         {
            l_ptime = atoi(l_subAttributeValue);
            //Store the packetization interval
            if(AddPtimeCapability(l_ptime) == true)
            {
               // fixed Frame Size
               AddFrameSizeCapability(G723_FSIZE);

               l_framesPerPacket = l_ptime / G723_FSIZE;

               //Insert the frames per packet value.
               AddFramesPerPacketCapability(l_framesPerPacket);
            }
         }

         memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
         memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);
         break;

      default:
         break;
      }

      l_token = DLGC_STRTOK(NULL, l_separators, &l_tokenizerBuffer);
      l_tokenCount++;
   }
}

double CMediaCoderG723::G723_ANY = 0;
double CMediaCoderG723::G723_5_3 = 5.3;
double CMediaCoderG723::G723_6_3 = 6.3;

//*****************************************************************************
// Function: bool CMediaCoderG723::AddBitRateCapability(const double &a_bitRate)
// Description: Add a bit rate capability tio the list of bit rate capabilities 
// 				for the coder
// Return: bool 
// Parameters: const double &a_bitRate 
//*****************************************************************************
bool CMediaCoderG723::AddBitRateCapability(const double& a_bitRate)
{
   for(unsigned int i=0; i < m_bitRateCapabilities.size(); i++)
   {
      if(a_bitRate == m_bitRateCapabilities[i])
      {
         return false;
      }
   }
   
   if(G723_5_3 == a_bitRate)
   {
      m_dialogicCoderType = CODER_TYPE_G7231_5_3K;
   }
   if(G723_6_3 == a_bitRate)
   {
      m_dialogicCoderType = CODER_TYPE_G7231_6_3K;
   }

   m_bitRateCapabilities.push_back(a_bitRate);
   return true;
}

//*****************************************************************************
// Function: void CMediaCoderG723::ListProperties()
// Description: List the coder properties
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderG723::ListProperties()
{
   printf("\t%s\n", GetDialogicCoderTypeDesc(m_dialogicCoderType));
   printf("\tm=%s %s:%ld plt=%d, peerplt=%d\n", 
      m_mediaType.c_str(), m_rtpAddress.c_str(), m_rtpPort, m_payloadType, m_peerPayloadType);
   printf("\ta=rtpmap:%d %s/%d/%d\n",m_payloadType, m_payloadCoderName.c_str(), m_clockRate, m_channel);
   printf("\ta=ptime:");
   if(m_ptimeCapabilities.size() == 0)
   {
      printf(" any");
   }
   else
   {
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         printf(" %d", m_ptimeCapabilities[i]);
      }
   }
   printf("\n");

   printf("\ta=bitrate:");
   for(unsigned int i=0; i < m_bitRateCapabilities.size(); i++)
   {
      printf(" %3.1f", m_bitRateCapabilities[i]);
   }
   printf("\n");

   printf("\ta=annexa:");
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      printf(" %s", (m_vadCapabilities[i]==CODER_VAD_ENABLE)?"yes":"no");
   }
   printf("\n");

   if (m_matchedPtime)
   {
      printf("\tMatchedCaps:\n");
      printf("\t\tDialogic Resource Type: %s\n", GetDialogicResourceTypeDesc(m_dialogicResourceType));
      printf("\t\tvad=%d\n", GetMatchedCoderVAD());
      printf("\t\tptime=%d\n", GetMatchedCoderPtime());
      printf("\t\tfpp=%d\n", GetMatchedCoderFramesPerPacket());
      printf("\t\tframe size=%d\n", GetMatchedCoderFrameSize());
   }
}

//*****************************************************************************
// Function: bool CMediaCoderG723::MatchCoder(AMediaCoder *a_otherMediaCoder)
// Description: Attempt to match the specified coder and return the result.
// Return: bool 
// Parameters: AMediaCoder *a_otherMediaCoder 
//*****************************************************************************
bool CMediaCoderG723::MatchCoder(AMediaCoder* a_otherMediaCoder)
{
   m_dialogicResourceType = RESOURCE_TYPE_NONE;
   CMediaCoderG723* l_otherMediaCoder = dynamic_cast<CMediaCoderG723*>(a_otherMediaCoder);
   if (!l_otherMediaCoder)
   {
      return false;
   }

   if (strcmp(GetPayloadCoderName(), l_otherMediaCoder->GetPayloadCoderName()))
   {
      return false;
   }

   if (GetDialogicCoderType() != l_otherMediaCoder->GetDialogicCoderType())
   {
      return false;
   }

   if (GetClockRate() !=  l_otherMediaCoder->GetClockRate())
   {
      return false;
   }

   if (GetChannel() !=  l_otherMediaCoder->GetChannel())
   {
      return false;
   }

   bool l_haveMatchedPtime(false);
   if(m_ptimeCapabilities.size() == 0)
   {
      // ptime is unspecified, use default from other coder
      m_matchedPtime = l_otherMediaCoder->GetDefaultPtime();
      l_haveMatchedPtime = true;
   }
   else
   {
      // find first match in other coder
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         if (l_otherMediaCoder->HasPtimeCapability(m_ptimeCapabilities[i]))
         {
            m_matchedPtime = m_ptimeCapabilities[i];
            l_haveMatchedPtime = true;
            break;
         }
      }
   }

   bool l_haveMatchedVAD(false);
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      // find first VAD capability match in other coder
      if (l_otherMediaCoder->HasVadCapability(m_vadCapabilities[i]))
      {
         m_matchedVAD = m_vadCapabilities[i];
         l_haveMatchedVAD = true;
         break;
      }
   }

   if (l_haveMatchedPtime && l_haveMatchedVAD)
   {
      m_dialogicResourceType = RESOURCE_IPM_G723;
      return true;
   }
   return false;
}

//*****************************************************************************
// Function: void CMediaCoderG723::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void CMediaCoderG723::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   AAudioCoder::ExportSdpMediaDescription(a_mediaDescription,a_rtpPort);

   char l_vadBuffer[COMMON_STRING_MAX_LENGTH] = {0};
   if (m_matchedVAD)
   {
      sprintf(l_vadBuffer, "%d annexa=yes;", m_payloadType);
   }
   else
   {
      sprintf(l_vadBuffer, "%d annexa=no;", m_payloadType);
   }

   char l_bitrateBuffer[COMMON_STRING_MAX_LENGTH] = {0};
   switch(m_dialogicCoderType)
   {
      case CODER_TYPE_G7231_5_3K:
         sprintf(l_bitrateBuffer, " bitrate=5.3", m_payloadType);
         break;
      case CODER_TYPE_G7231_6_3K:
         sprintf(l_bitrateBuffer, " bitrate=5.3", m_payloadType);
         break;
   }
   sdpAttribute* l_attribute = a_mediaDescription->attributeList()->addItem();
   l_attribute ->setProperty("fmtp");
   char l_buffer[COMMON_STRING_MAX_LENGTH] = {0};
   sprintf(l_buffer, "%s %s", l_vadBuffer, l_bitrateBuffer);
   l_attribute->setPropertyValue(l_buffer);
}


//*****************************************************************************
// Function: CMediaCoderG729::CMediaCoderG729()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CMediaCoderG729::CMediaCoderG729()
   :AAudioCoder("G729",18)
{
   m_dialogicCoderType = CODER_TYPE_G729;
   AddVadCapability(CODER_VAD_DISABLE);
}

//*****************************************************************************
// Function: CMediaCoderG729::CMediaCoderG729(const IPM_AUDIO_CODER_INFO &l_audioCoderInfo)
// Description: Initializing construtor
// Return:  
// Parameters: const IPM_AUDIO_CODER_INFO &l_audioCoderInfo 
//*****************************************************************************
CMediaCoderG729::CMediaCoderG729(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo)
   :AAudioCoder("G729",18,l_audioCoderInfo)
{
   m_dialogicCoderType = l_audioCoderInfo.eCoderType;
   switch(m_dialogicCoderType)
   {
   case CODER_TYPE_G729ANNEXAWANNEXB:
      AddVadCapability(CODER_VAD_ENABLE);
      break;
   case CODER_TYPE_G729WANNEXB:
      AddVadCapability(CODER_VAD_ENABLE);
      break;
   case CODER_TYPE_G729ANNEXA:
      AddVadCapability(CODER_VAD_DISABLE);
      break;
   case CODER_TYPE_G729:
      AddVadCapability(CODER_VAD_DISABLE);
      break;
   }
}

//*****************************************************************************
// Function: void CMediaCoderG729::ImportSdpFmtp(const std::string &a_attribute)
// Description: Import coder attributes from SDP
// Return: void 
// Parameters: const std::string &a_attribute 
//*****************************************************************************
void CMediaCoderG729::ImportSdpFmtp(const std::string& a_attribute)
{
//   printf("CMediaCoderG729::ImportSdpFmtp\n");
   unsigned int l_payloadType, l_ptime, l_framesPerPacket, l_tokenCount=0;
   char  l_separators[] = "= ;";
   char* l_token = NULL;
   char  l_subAttribute[COMMON_STRING_MAX_LENGTH];
   char  l_subAttributeValue[MEDIA_ATTRIB_SIZE_MAX];
   char  l_attribute[MEDIA_ATTRIB_SIZE_MAX];
   char* l_tokenizerBuffer = NULL;

   size_t l_attributeSize = a_attribute.size();
   if(l_attributeSize >= MEDIA_ATTRIB_SIZE_MAX)
   {
      printf("CMediaCoderG729::ImportSdpFmtp - Attribute.size()= %u is greater than MEDIA_ATTRIB_SIZE_MAX=%u\n",
                         a_attribute.size(),
                         MEDIA_ATTRIB_SIZE_MAX);
      return;
   }

   strcpy(l_attribute, a_attribute.c_str());
   l_attribute[l_attributeSize] = '\0';

   memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
   memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);

   l_token = DLGC_STRTOK(l_attribute, l_separators, &l_tokenizerBuffer);
   while (l_token != NULL)
   {
      switch (l_tokenCount)
      {
      case 0:
         l_payloadType = atoi(l_token);
         if (l_payloadType != m_payloadType)
         {
            printf("CMediaCoderG729::ImportSdpFmtp - Payload Number Mismatch\n");
            return;
         }
         break;

      case 1:
      case 3:
      case 5:
         strcpy(l_subAttribute, l_token);
         break;

      case 2:
      case 4:
      case 6:
         strcpy(l_subAttributeValue, l_token);
         if (!strcmp(l_subAttribute, "annexb"))
         {
            if(!strcmp(l_subAttributeValue, "yes"))
            {
               DeleteVadCapability(CODER_VAD_DISABLE);
               AddVadCapability(CODER_VAD_ENABLE);
               m_dialogicCoderType = CODER_TYPE_G729ANNEXAWANNEXB;
            }
            else if(!strcmp(l_subAttributeValue, "no"))
            {
               DeleteVadCapability(CODER_VAD_ENABLE);
               AddVadCapability(CODER_VAD_DISABLE);
               m_dialogicCoderType = CODER_TYPE_G729ANNEXA;
            }
         }
         else if (!strcmp(l_subAttribute, "ptime"))
         {
            l_ptime = atoi(l_subAttributeValue);
            //Store the packetization interval
            if(AddPtimeCapability(l_ptime) == true)
            {
               // fixed frame size
               AddFrameSizeCapability(G729_FSIZE);

               l_framesPerPacket = l_ptime / G729_FSIZE;
               //Insert the frames per packet value. 
               AddFramesPerPacketCapability(l_framesPerPacket);
            }
         }

         memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
         memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);
         break;

      default:
         break;
      }

      l_token = DLGC_STRTOK(NULL, l_separators, &l_tokenizerBuffer);
      l_tokenCount++;
   }
}

//*****************************************************************************
// Function: void CMediaCoderG729::ListProperties()
// Description: List the coder properties
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderG729::ListProperties()
{
   printf("\t%s\n", GetDialogicCoderTypeDesc(m_dialogicCoderType));
   printf("\tm=%s %s:%ld plt=%d, peerplt=%d\n", 
      m_mediaType.c_str(), m_rtpAddress.c_str(), m_rtpPort, m_payloadType, m_peerPayloadType);
   printf("\ta=rtpmap:%d %s/%d/%d\n",m_payloadType, m_payloadCoderName.c_str(), m_clockRate, m_channel);
   printf("\ta=ptime:");
   if(m_ptimeCapabilities.size() == 0)
   {
      printf(" any");
   }
   else
   {
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         printf(" %d", m_ptimeCapabilities[i]);
      }
   }
   printf("\n");

   printf("\ta=annexb:");
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      printf(" %s", (m_vadCapabilities[i]==CODER_VAD_ENABLE)?"yes":"no");
   }
   printf("\n");

   if (m_matchedPtime)
   {
      printf("\tMatchedCaps:\n");
      printf("\t\tDialogic Resource Type: %s\n", GetDialogicResourceTypeDesc(m_dialogicResourceType));
      printf("\t\tvad=%d\n", GetMatchedCoderVAD());
      printf("\t\tptime=%d\n", GetMatchedCoderPtime());
      printf("\t\tfpp=%d\n", GetMatchedCoderFramesPerPacket());
      printf("\t\tframe size=%d\n", GetMatchedCoderFrameSize());
   }
}

//*****************************************************************************
// Function: bool CMediaCoderG729::MatchCoder(AMediaCoder *a_otherMediaCoder)
// Description: Attempt to match the specified coder and return the result.
// Return: bool 
// Parameters: AMediaCoder *a_otherMediaCoder 
//*****************************************************************************
bool CMediaCoderG729::MatchCoder(AMediaCoder* a_otherMediaCoder)
{
   m_dialogicResourceType = RESOURCE_TYPE_NONE;
   CMediaCoderG729* l_otherMediaCoder = dynamic_cast<CMediaCoderG729*>(a_otherMediaCoder);
   if (!l_otherMediaCoder)
   {
      return false;
   }

   if (strcmp(GetPayloadCoderName(), l_otherMediaCoder->GetPayloadCoderName()))
   {
      return false;
   }

   if (GetDialogicCoderType() != l_otherMediaCoder->GetDialogicCoderType())
   {
      return false;
   }

   if (GetClockRate() !=  l_otherMediaCoder->GetClockRate())
   {
      return false;
   }

   if (GetChannel() !=  l_otherMediaCoder->GetChannel())
   {
      return false;
   }

   bool l_haveMatchedPtime(false);
   if(m_ptimeCapabilities.size() == 0)
   {
      // ptime is unspecified, use default from other coder
      m_matchedPtime = l_otherMediaCoder->GetDefaultPtime();
      l_haveMatchedPtime = true;
   }
   else
   {
      // find first match in other coder
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         if (l_otherMediaCoder->HasPtimeCapability(m_ptimeCapabilities[i]))
         {
            m_matchedPtime = m_ptimeCapabilities[i];
            l_haveMatchedPtime = true;
            break;
         }
      }
   }

   bool l_haveMatchedVAD(false);
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      // find first VAD capability match in other coder
      if (l_otherMediaCoder->HasVadCapability(m_vadCapabilities[i]))
      {
         m_matchedVAD = m_vadCapabilities[i];
         l_haveMatchedVAD = true;
         break;
      }
   }

   if (l_haveMatchedPtime && l_haveMatchedVAD)
   {
      m_dialogicResourceType = RESOURCE_IPM_G729;
      return true;
   }
   return false;
}

//*****************************************************************************
// Function: void CMediaCoderG729::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void CMediaCoderG729::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   AAudioCoder::ExportSdpMediaDescription(a_mediaDescription,a_rtpPort);

   if (m_matchedVAD)
   {
      sdpAttribute* l_attribute = a_mediaDescription->attributeList()->addItem();
      l_attribute ->setProperty("fmtp");
      char l_buffer[COMMON_STRING_MAX_LENGTH] = {0};
      sprintf(l_buffer, "%d annexb=yes", m_payloadType);
      l_attribute->setPropertyValue(l_buffer);
   }
}

//*****************************************************************************
// Function: CMediaCoderAmrNb::CMediaCoderAmrNb(unsigned a_payloadType)
// Description: Initializing construtor
// Return:  
// Parameters: unsigned a_payloadType 
//*****************************************************************************
CMediaCoderAmrNb::CMediaCoderAmrNb(unsigned a_payloadType)
   :AAudioCoder("AMR",a_payloadType)
   ,m_octetAlign(false)
{
   m_dialogicCoderType = CODER_TYPE_AMRNB_12_2k;
   AddVadCapability(CODER_VAD_ENABLE);
}

//*****************************************************************************
// Function: CMediaCoderAmrNb::CMediaCoderAmrNb(const IPM_AUDIO_CODER_INFO &l_audioCoderInfo)
// Description: Initializing construtor
// Return:  
// Parameters: const IPM_AUDIO_CODER_INFO &l_audioCoderInfo 
//*****************************************************************************
CMediaCoderAmrNb::CMediaCoderAmrNb(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo)
   :AAudioCoder("AMR",96,l_audioCoderInfo)
   ,m_octetAlign(false)
{
   m_dialogicCoderType = l_audioCoderInfo.eCoderType;
   AddVadCapability(CODER_VAD_ENABLE);
   switch(l_audioCoderInfo.eCoderType)
   {
      case CODER_TYPE_AMRNB_4_75k: 
      case CODER_TYPE_AMRNB_5_15k:
      case CODER_TYPE_AMRNB_5_9k:
      case CODER_TYPE_AMRNB_6_7k:
      case CODER_TYPE_AMRNB_7_4k:
      case CODER_TYPE_AMRNB_7_95k:
      case CODER_TYPE_AMRNB_10_2k:
      case CODER_TYPE_AMRNB_12_2k:
         m_modeSet.set(l_audioCoderInfo.eCoderType - CODER_TYPE_AMRNB_4_75k);
         break;
      case CODER_TYPE_AMRNB_NONE:
         EnableAllModeSet();
         break;
      default:
         //should not be here
         break;
   }
}

//*****************************************************************************
// Function: void CMediaCoderAmrNb::ImportSdpFmtp(const std::string &a_attribute)
// Description: Import coder attributes from SDP
// Return: void 
// Parameters: const std::string &a_attribute 
//*****************************************************************************
void CMediaCoderAmrNb::ImportSdpFmtp(const std::string& a_attribute)
{
   unsigned int l_payloadType, l_ptime, l_framesPerPacket, l_tokenCount=0;
   char l_separators[] = "= ;";
   char *l_token;
   char l_subAttribute[COMMON_STRING_MAX_LENGTH];
   char l_subAttributeValue[MEDIA_ATTRIB_SIZE_MAX];
   char l_attribute[MEDIA_ATTRIB_SIZE_MAX];
   char* l_tokenizerBuffer = NULL;

   size_t l_attributeSize = a_attribute.size();
   if(l_attributeSize >= MEDIA_ATTRIB_SIZE_MAX)
   {
      printf("CMediaCoderAmrNb::ImportSdpFmtp - Attribute.size()= %u is greater than MEDIA_ATTRIB_SIZE_MAX=%u\n",
                         a_attribute.size(),
                         MEDIA_ATTRIB_SIZE_MAX);
      return;
   }

   strcpy(l_attribute, a_attribute.c_str());
   l_attribute[l_attributeSize] = '\0';

   memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
   memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);

   l_token = DLGC_STRTOK(l_attribute, l_separators, &l_tokenizerBuffer);
   while (l_token != NULL)
   {
      switch (l_tokenCount)
      {
      case 0:
         l_payloadType = atoi(l_token);
         if (l_payloadType != m_payloadType)
         {
            printf("CMediaCoderAmrNb::ImportSdpFmtp - Payload Number Mismatch\n");
            return;
         }
         break;

      case 1:
      case 3:
      case 5:
         strcpy(l_subAttribute, l_token);
         break;

      case 2:
      case 4:
      case 6:
         strcpy(l_subAttributeValue, l_token);

         if (!strcmp(l_subAttribute, "ptime"))
         {
            l_ptime = atoi(l_subAttributeValue);
            //Store the packetization interval
            if(AddPtimeCapability(l_ptime) == true)
            {
               // fixed frame size
               AddFrameSizeCapability(AMR_NB_FSIZE);

               l_framesPerPacket = l_ptime / AMR_NB_FSIZE;
               //Insert the frames per packet value. 
               AddFramesPerPacketCapability(l_framesPerPacket);
            }
         }
         else if (!strcmp(l_subAttribute, "mode-set"))
         {
            std::string l_modeSet = l_subAttributeValue;
            m_modeSet.reset();
            char l_modeStr[8];
            for(unsigned int i=0;i<8;i++)
            {
               sprintf(l_modeStr,"%d",i);
               if (l_modeSet.find(l_modeStr) != std::string::npos)
               {
                  m_modeSet.set(i);
               }
            }
         }
         else if (!strcmp(l_subAttribute, "octet-align"))
         {
            if (!strcmp(l_subAttributeValue, "1"))
            {
               m_octetAlign = true;
            }
         }

         memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
         memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);
         break;

      default:
         break;
      }

      l_token = DLGC_STRTOK(NULL, l_separators, &l_tokenizerBuffer);
      l_tokenCount++;
   }
}

//*****************************************************************************
// Function: void CMediaCoderAmrNb::EnableAllModeSet()
// Description: Enable tyhe mode set for all modes
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderAmrNb::EnableAllModeSet()
{
   m_modeSet.set(0);
   m_modeSet.set(1);
   m_modeSet.set(2);
   m_modeSet.set(3);
   m_modeSet.set(4);
   m_modeSet.set(5);
   m_modeSet.set(6);
   m_modeSet.set(7);
}

//*****************************************************************************
// Function: void CMediaCoderAmrNb::ListProperties()
// Description: List the coder properties
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderAmrNb::ListProperties()
{
   printf("\t%s\n", GetDialogicCoderTypeDesc(m_dialogicCoderType));
   printf("\tm=%s %s:%ld plt=%d, peerplt=%d\n", 
      m_mediaType.c_str(), m_rtpAddress.c_str(), m_rtpPort, m_payloadType, m_peerPayloadType);
   printf("\ta=rtpmap:%d %s/%d/%d\n",m_payloadType, m_payloadCoderName.c_str(), m_clockRate, m_channel);
   printf("\ta=ptime:");
   if(m_ptimeCapabilities.size() == 0)
   {
      printf(" any");
   }
   else
   {
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         printf(" %d", m_ptimeCapabilities[i]);
      }
   }
   printf("\n");

   char l_modeSetString[20] = {0};
   char* l_char = l_modeSetString;
   for(unsigned i=0; i < 8; i++)
   {
      if (m_modeSet.test(i))
      {
         *l_char++ = (char)('0' + i);
         *l_char++ = ',';
      }
   }
   *--l_char = '\0';
   printf("\ta=mode-set=%s\n", l_modeSetString);

   if (m_octetAlign)
   {
      printf("\ta=octet-align=1\n", l_modeSetString);
   }
}

//*****************************************************************************
// Function: bool CMediaCoderAmrNb::MatchCoder(AMediaCoder *a_otherMediaCoder)
// Description: Attempt to match the specified coder and return the result.
// Return: bool 
// Parameters: AMediaCoder *a_otherMediaCoder 
//*****************************************************************************
bool CMediaCoderAmrNb::MatchCoder(AMediaCoder* a_otherMediaCoder)
{
   m_dialogicResourceType = RESOURCE_TYPE_NONE;
   CMediaCoderAmrNb* l_otherMediaCoder = dynamic_cast<CMediaCoderAmrNb*>(a_otherMediaCoder);
   if (!l_otherMediaCoder)
   {
      return false;
   }

   if (strcmp(GetPayloadCoderName(), l_otherMediaCoder->GetPayloadCoderName()))
   {
      return false;
   }

   if (GetClockRate() !=  l_otherMediaCoder->GetClockRate())
   {
      return false;
   }

   if (GetChannel() !=  l_otherMediaCoder->GetChannel())
   {
      return false;
   }

   bool l_haveMatchedPtime(false);
   if(m_ptimeCapabilities.size() == 0)
   {
      // ptime is unspecified, use default from other coder
      m_matchedPtime = l_otherMediaCoder->GetDefaultPtime();
      l_haveMatchedPtime = true;
   }
   else
   {
      // find first match in other coder
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         if (l_otherMediaCoder->HasPtimeCapability(m_ptimeCapabilities[i]))
         {
            m_matchedPtime = m_ptimeCapabilities[i];
            l_haveMatchedPtime = true;
            break;
         }
      }
   }

   bool l_haveMatchedVAD(false);
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      // find first VAD capability match in other coder
      if (l_otherMediaCoder->HasVadCapability(m_vadCapabilities[i]))
      {
         m_matchedVAD = m_vadCapabilities[i];
         l_haveMatchedVAD = true;
         break;
      }
   }

   bool l_haveMatchedBitRate(false); 
   if (l_haveMatchedPtime && l_haveMatchedVAD)
   {
      for(int i=7;i>=0;i--)
      {
         if(m_modeSet.test(i) && l_otherMediaCoder->IsBitSetModeSet(i))
         {
            m_dialogicCoderType = static_cast<eIPM_CODER_TYPE>(CODER_TYPE_AMRNB_4_75k + i);
            m_dialogicResourceType = RESOURCE_IPM_AMR_NB;
            l_haveMatchedBitRate = true;
            break;
         }
      }
   }
   return l_haveMatchedBitRate;
}

//*****************************************************************************
// Function: void CMediaCoderAmrNb::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void CMediaCoderAmrNb::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   AAudioCoder::ExportSdpMediaDescription(a_mediaDescription,a_rtpPort);

   sdpAttribute* l_attribute = a_mediaDescription->attributeList()->addItem();
   l_attribute ->setProperty("fmtp");
   char l_buffer[COMMON_STRING_MAX_LENGTH * 3] = {0};
   char l_modeSet[20] = {0};
   char* l_currentPosition = l_modeSet;
   for(unsigned int i=0; i < 8; i++)
   {
      if (m_modeSet.test(i))
      {
         *l_currentPosition++ = (char)('0' + i);
         *l_currentPosition++ = ',';
      }
   }
   // add terminator over last comma character
   *--l_currentPosition = '\0';

   if (m_octetAlign)
   {
      sprintf(l_buffer, "%d octet-align=1", m_payloadType);
   }
/*
   if (m_octetAlign)
   {
      sprintf(l_buffer, "%d mode-set=%s octet-align=1", m_payloadType, l_modeSet);
   }
   else
   {
      sprintf(l_buffer, "%d mode-set=%s", m_payloadType, l_modeSet);
   }
*/
   l_attribute->setPropertyValue(l_buffer);
}


//*****************************************************************************
// Function: CMediaCoderAmrWb::CMediaCoderAmrWb(unsigned a_payloadType)
// Description: Initializing construtor
// Return:  
// Parameters: unsigned a_payloadType 
//*****************************************************************************
CMediaCoderAmrWb::CMediaCoderAmrWb(unsigned a_payloadType)
   :AAudioCoder("AMR-WB",a_payloadType)
   ,m_octetAlign(false)
{
   m_dialogicCoderType = CODER_TYPE_AMRWB_23_85K;
   AddVadCapability(CODER_VAD_ENABLE);
}

//*****************************************************************************
// Function: CMediaCoderAmrWb::CMediaCoderAmrWb(const IPM_AUDIO_CODER_INFO &l_audioCoderInfo)
// Description: Initializing construtor
// Return:  
// Parameters: const IPM_AUDIO_CODER_INFO &l_audioCoderInfo 
//*****************************************************************************
CMediaCoderAmrWb::CMediaCoderAmrWb(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo)
   :AAudioCoder("AMR-WB",111,l_audioCoderInfo)
   ,m_octetAlign(false)
{
   m_dialogicCoderType = l_audioCoderInfo.eCoderType;
   AddVadCapability(CODER_VAD_ENABLE);
   switch(l_audioCoderInfo.eCoderType)
   {
      case CODER_TYPE_AMRWB_6_6K: 
      case CODER_TYPE_AMRWB_8_85K:
      case CODER_TYPE_AMRWB_12_65K:
      case CODER_TYPE_AMRWB_14_25K:
      case CODER_TYPE_AMRWB_15_85K:
      case CODER_TYPE_AMRWB_18_25K:
      case CODER_TYPE_AMRWB_19_85K:
      case CODER_TYPE_AMRWB_23_05K:
      case CODER_TYPE_AMRWB_23_85K:
         m_modeSet.set(l_audioCoderInfo.eCoderType - CODER_TYPE_AMRWB_6_6K);
         break;
      case CODER_TYPE_AMRWB_NONE:
         EnableAllModeSet();
         break;
      default:
         //should not be here
         break;
   }
}

//*****************************************************************************
// Function: void CMediaCoderAmrWb::ImportSdpFmtp(const std::string &a_attribute)
// Description: Import coder attributes from SDP
// Return: void 
// Parameters: const std::string &a_attribute 
//*****************************************************************************
void CMediaCoderAmrWb::ImportSdpFmtp(const std::string& a_attribute)
{
   unsigned int l_payloadType, l_ptime, l_framesPerPacket, l_tokenCount=0;
   char l_separators[] = "= ;";
   char *l_token;
   char l_subAttribute[COMMON_STRING_MAX_LENGTH];
   char l_subAttributeValue[MEDIA_ATTRIB_SIZE_MAX];
   char l_attribute[MEDIA_ATTRIB_SIZE_MAX];
   char* l_tokenizerBuffer = NULL;

   size_t l_attributeSize = a_attribute.size();
   if(l_attributeSize >= MEDIA_ATTRIB_SIZE_MAX)
   {
      printf("CMediaCoderAmrWb::ImportSdpFmtp - Attribute.size()= %u is greater than MEDIA_ATTRIB_SIZE_MAX=%u\n",
                         a_attribute.size(),
                         MEDIA_ATTRIB_SIZE_MAX);
      return;
   }

   strcpy(l_attribute, a_attribute.c_str());
   l_attribute[l_attributeSize] = '\0';

   memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
   memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);

   l_token = DLGC_STRTOK(l_attribute, l_separators, &l_tokenizerBuffer);
   while (l_token != NULL)
   {
      switch (l_tokenCount)
      {
      case 0:
         l_payloadType = atoi(l_token);
         if (l_payloadType != m_payloadType)
         {
            printf("CMediaCoderAmrWb::ImportSdpFmtp - Payload Number Mismatch\n");
            return;
         }
         break;

      case 1:
      case 3:
      case 5:
         strcpy(l_subAttribute, l_token);
         break;

      case 2:
      case 4:
      case 6:
         strcpy(l_subAttributeValue, l_token);

         if (!strcmp(l_subAttribute, "ptime"))
         {
            l_ptime = atoi(l_subAttributeValue);
            //Store the packetization interval
            if(AddPtimeCapability(l_ptime) == true)
            {
               // fixed frame size
               AddFrameSizeCapability(AMR_NB_FSIZE);

               l_framesPerPacket = l_ptime / AMR_NB_FSIZE;
               //Insert the frames per packet value. 
               AddFramesPerPacketCapability(l_framesPerPacket);
            }
         }
         else if (!strcmp(l_subAttribute, "mode-set"))
         {
            std::string l_modeSet = l_subAttributeValue;
            m_modeSet.reset();
            char l_modeStr[9];
            for(unsigned int i=0;i<9;i++)
            {
               sprintf(l_modeStr,"%d",i);
               if (l_modeSet.find(l_modeStr) != std::string::npos)
               {
                  m_modeSet.set(i);
               }
            }
         }
         else if (!strcmp(l_subAttribute, "octet-align"))
         {
            if (!strcmp(l_subAttributeValue, "1"))
            {
               m_octetAlign = true;
            }
         }

         memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
         memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);
         break;

      default:
         break;
      }

      l_token = DLGC_STRTOK(NULL, l_separators, &l_tokenizerBuffer);
      l_tokenCount++;
   }
}

//*****************************************************************************
// Function: void CMediaCoderAmrWb::EnableAllModeSet()
// Description: Enable tyhe mode set for all modes
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderAmrWb::EnableAllModeSet()
{
   m_modeSet.set(0);
   m_modeSet.set(1);
   m_modeSet.set(2);
   m_modeSet.set(3);
   m_modeSet.set(4);
   m_modeSet.set(5);
   m_modeSet.set(6);
   m_modeSet.set(7);
   m_modeSet.set(8);
}

//*****************************************************************************
// Function: void CMediaCoderAmrWb::ListProperties()
// Description: List the coder properties
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderAmrWb::ListProperties()
{
   printf("\t%s\n", GetDialogicCoderTypeDesc(m_dialogicCoderType));
   printf("\tm=%s %s:%ld plt=%d, peerplt=%d\n", 
      m_mediaType.c_str(), m_rtpAddress.c_str(), m_rtpPort, m_payloadType, m_peerPayloadType);
   printf("\ta=rtpmap:%d %s/%d/%d\n",m_payloadType, m_payloadCoderName.c_str(), m_clockRate, m_channel);
   printf("\ta=ptime:");
   if(m_ptimeCapabilities.size() == 0)
   {
      printf(" any");
   }
   else
   {
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         printf(" %d", m_ptimeCapabilities[i]);
      }
   }
   printf("\n");

   char l_modeSetString[20] = {0};
   char* l_char = l_modeSetString;
   for(unsigned i=0; i < 9; i++)
   {
      if (m_modeSet.test(i))
      {
         *l_char++ = (char)('0' + i);
         *l_char++ = ',';
      }
   }
   *--l_char = '\0';
   printf("\ta=mode-set=%s\n", l_modeSetString);

   if (m_octetAlign)
   {
      printf("\ta=octet-align=1\n", l_modeSetString);
   }
}

//*****************************************************************************
// Function: bool CMediaCoderAmrWb::MatchCoder(AMediaCoder *a_otherMediaCoder)
// Description: Attempt to match the specified coder and return the result.
// Return: bool 
// Parameters: AMediaCoder *a_otherMediaCoder 
//*****************************************************************************
bool CMediaCoderAmrWb::MatchCoder(AMediaCoder* a_otherMediaCoder)
{
   m_dialogicResourceType = RESOURCE_TYPE_NONE;
   CMediaCoderAmrWb* l_otherMediaCoder = dynamic_cast<CMediaCoderAmrWb*>(a_otherMediaCoder);
   if (!l_otherMediaCoder)
   {
      return false;
   }

   if (strcmp(GetPayloadCoderName(), l_otherMediaCoder->GetPayloadCoderName()))
   {
      return false;
   }

   if (GetClockRate() !=  l_otherMediaCoder->GetClockRate())
   {
      return false;
   }

   if (GetChannel() !=  l_otherMediaCoder->GetChannel())
   {
      return false;
   }

   bool l_haveMatchedPtime(false);
   if(m_ptimeCapabilities.size() == 0)
   {
      // ptime is unspecified, use default from other coder
      m_matchedPtime = l_otherMediaCoder->GetDefaultPtime();
      l_haveMatchedPtime = true;
   }
   else
   {
      // find first match in other coder
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         if (l_otherMediaCoder->HasPtimeCapability(m_ptimeCapabilities[i]))
         {
            m_matchedPtime = m_ptimeCapabilities[i];
            l_haveMatchedPtime = true;
            break;
         }
      }
   }

   bool l_haveMatchedVAD(false);
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      // find first VAD capability match in other coder
      if (l_otherMediaCoder->HasVadCapability(m_vadCapabilities[i]))
      {
         m_matchedVAD = m_vadCapabilities[i];
         l_haveMatchedVAD = true;
         break;
      }
   }

   bool l_haveMatchedBitRate(false); 
   if (l_haveMatchedPtime && l_haveMatchedVAD)
   {
      for(int i=8;i>=0;i--)
      {
         if(m_modeSet.test(i) && l_otherMediaCoder->IsBitSetModeSet(i))
         {
            m_dialogicCoderType = static_cast<eIPM_CODER_TYPE>(CODER_TYPE_AMRWB_6_6K + i);
            m_dialogicResourceType = RESOURCE_IPM_AMR_WB;
            l_haveMatchedBitRate = true;
            break;
         }
      }
   }
   return l_haveMatchedBitRate;
}

//*****************************************************************************
// Function: void CMediaCoderAmrWb::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void CMediaCoderAmrWb::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   AAudioCoder::ExportSdpMediaDescription(a_mediaDescription,a_rtpPort);

   sdpAttribute* l_attribute = a_mediaDescription->attributeList()->addItem();
   l_attribute ->setProperty("fmtp");
   char l_buffer[COMMON_STRING_MAX_LENGTH * 3] = {0};
   char l_modeSet[20] = {0};
   char* l_currentPosition = l_modeSet;
   for(unsigned int i=0; i < 8; i++)
   {
      if (m_modeSet.test(i))
      {
         *l_currentPosition++ = (char)('0' + i);
         *l_currentPosition++ = ',';
      }
   }
   // add terminator over last comma character
   *--l_currentPosition = '\0';

   if (m_octetAlign)
   {
      sprintf(l_buffer, "%d octet-align=1", m_payloadType);
   }
/*
   if (m_octetAlign)
   {
      sprintf(l_buffer, "%d mode-set=%s octet-align=1", m_payloadType, l_modeSet);
   }
   else
   {
      sprintf(l_buffer, "%d mode-set=%s", m_payloadType, l_modeSet);
   }
*/
   l_attribute->setPropertyValue(l_buffer);
}

//*****************************************************************************
// Function: CMediaCoderG722::CMediaCoderG722()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CMediaCoderG722::CMediaCoderG722()
   :AAudioCoder("G722",9)
{
   m_dialogicCoderType = CODER_TYPE_G722_64K;
   AddVadCapability(CODER_VAD_DISABLE);
}

//*****************************************************************************
// Function: CMediaCoderG722::CMediaCoderG722(const IPM_AUDIO_CODER_INFO &l_audioCoderInfo)
// Description: Initializing constructor
// Return:  
// Parameters: const IPM_AUDIO_CODER_INFO &l_audioCoderInfo 
//*****************************************************************************
CMediaCoderG722::CMediaCoderG722(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo)
   :AAudioCoder("G722",9, l_audioCoderInfo)
{
   m_dialogicCoderType = CODER_TYPE_G722_64K;
}

//*****************************************************************************
// Function: void CMediaCoderG722::ImportSdpFmtp(const std::string &a_attribute)
// Description: Import coder attributes from SDP
// Return: void 
// Parameters: const std::string &a_attribute 
//*****************************************************************************
void CMediaCoderG722::ImportSdpFmtp(const std::string& a_attribute)
{
//   printf("CMediaCoderG722::ImportSdpFmtp\n");
   unsigned int l_payloadType, l_ptime, l_tokenCount=0;
   eIPM_CODER_FRAMESIZE l_frameSize;
   char l_separators[] = "= ;";
   char *l_token=NULL;
   char l_subAttribute[COMMON_STRING_MAX_LENGTH];
   char l_subAttributeValue[MEDIA_ATTRIB_SIZE_MAX];
   char l_attribute[MEDIA_ATTRIB_SIZE_MAX];
   char* l_tokenizerBuffer = NULL;

   size_t l_attributeSize = a_attribute.size();
   if(l_attributeSize >= MEDIA_ATTRIB_SIZE_MAX)
   {
      printf("CMediaCoderG722::ImportSdpFmtp - Attribute.size()= %u is greater than MEDIA_ATTRIB_SIZE_MAX=%u\n",
                         a_attribute.size(),
                         MEDIA_ATTRIB_SIZE_MAX);
      return;
   }

   strcpy(l_attribute, a_attribute.c_str());
   l_attribute[l_attributeSize] = '\0';

   memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
   memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);

   l_token = DLGC_STRTOK(l_attribute, l_separators, &l_tokenizerBuffer);
   while (l_token != NULL)
   {
      switch (l_tokenCount)
      {
      case 0:
         l_payloadType = atoi(l_token);
         if (l_payloadType != m_payloadType)
         {
            printf("CG722::ImportSdpFmtp - Payload Number Mismatch\n");
            return;
         }
         break;

      case 1:
      case 3:
      case 5:
         strcpy(l_subAttribute, l_token);
         break;

      case 2:
      case 4:
      case 6:
         strcpy(l_subAttributeValue, l_token);
         if (!strcmp(l_subAttribute, "ptime"))
         {
            l_ptime = atoi(l_subAttributeValue);

            //Store the packetization interval
            if(AddPtimeCapability(l_ptime) == true)
            {
               // convert to fpp and frame size

               // We currently support 1 FPP.
               //Insert the frames per packet value. 
               AddFramesPerPacketCapability(G722_FPP);

               l_frameSize = static_cast<eIPM_CODER_FRAMESIZE>(l_ptime / G722_FPP);
               AddFrameSizeCapability(l_frameSize);
            }
         }
         memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
         memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);
         break;

      default:
         break;
      }

      l_token = DLGC_STRTOK(NULL, l_separators, &l_tokenizerBuffer);
      l_tokenCount++;
   }
}

//*****************************************************************************
// Function: void CMediaCoderG722::ListProperties()
// Description: List the coder properties
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderG722::ListProperties()
{
   printf("\t%s\n", GetDialogicCoderTypeDesc(m_dialogicCoderType));
   printf("\tm=%s %s:%ld plt=%d, peerplt=%d\n", 
      m_mediaType.c_str(), m_rtpAddress.c_str(), m_rtpPort, m_payloadType, m_peerPayloadType);
   printf("\ta=rtpmap:%d %s/%d/%d\n",m_payloadType, m_payloadCoderName.c_str(), m_clockRate, m_channel);
   printf("\ta=ptime:");

   if(m_ptimeCapabilities.size() == 0)
   {
      printf(" any");
   }
   else
   {
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         printf(" %d", m_ptimeCapabilities[i]);
      }
   }
   printf("\n");

   printf("\tvad:");
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      printf(" %s", (m_vadCapabilities[i]==1)?"yes":"no");
   }
   printf("\n");

   if (m_matchedPtime)
   {
      printf("\tMatchedCaps:\n");
      printf("\t\tDialogic Resource Type: %s\n", GetDialogicResourceTypeDesc(m_dialogicResourceType));
      printf("\t\tvad=%d\n", GetMatchedCoderVAD());
      printf("\t\tptime=%d\n", GetMatchedCoderPtime());
      printf("\t\tfpp=%d\n", GetMatchedCoderFramesPerPacket());
      printf("\t\tframe size=%d\n", GetMatchedCoderFrameSize());
   }
}

//*****************************************************************************
// Function: bool CMediaCoderG722::MatchCoder(AMediaCoder *a_otherMediaCoder)
// Description: Attempt to match the specified coder and return the result.
// Return: bool 
// Parameters: AMediaCoder *a_otherMediaCoder 
//*****************************************************************************
bool CMediaCoderG722::MatchCoder(AMediaCoder* a_otherMediaCoder)
{
   m_dialogicResourceType = RESOURCE_TYPE_NONE;
   CMediaCoderG722* l_otherMediaCoder = dynamic_cast<CMediaCoderG722*>(a_otherMediaCoder);
   if (!l_otherMediaCoder)
   {
      return false;
   }

   if (strcmp(GetPayloadCoderName(), l_otherMediaCoder->GetPayloadCoderName()))
   {
      return false;
   }

   if (GetDialogicCoderType() != l_otherMediaCoder->GetDialogicCoderType())
   {
      return false;
   }

   if (GetClockRate() !=  l_otherMediaCoder->GetClockRate())
   {
      return false;
   }

   if (GetChannel() !=  l_otherMediaCoder->GetChannel())
   {
      return false;
   }

   bool l_haveMatchedPtime(false);
   if(m_ptimeCapabilities.size() == 0)
   {
      // ptime is unspecified, use default from other coder
      m_matchedPtime = l_otherMediaCoder->GetDefaultPtime();
      l_haveMatchedPtime = true;
   }
   else
   {
      // find first match in other coder
      for(unsigned int i=0; i < m_ptimeCapabilities.size(); i++)
      {
         if (l_otherMediaCoder->HasPtimeCapability(m_ptimeCapabilities[i]))
         {
            m_matchedPtime = m_ptimeCapabilities[i];
            l_haveMatchedPtime = true;
            break;
         }
      }
   }

   bool l_haveMatchedVAD(false);
   for(unsigned int i=0; i < m_vadCapabilities.size(); i++)
   {
      // find first VAD capability match in other coder
      if (l_otherMediaCoder->HasVadCapability(m_vadCapabilities[i]))
      {
         m_matchedVAD = m_vadCapabilities[i];
         l_haveMatchedVAD = true;
         break;
      }
   }

   if (l_haveMatchedPtime && l_haveMatchedVAD)
   {
      switch(m_matchedPtime)
      {
      default:
         m_dialogicResourceType = RESOURCE_IPM_G722;
         break;
      }
      return true;
   }
   return false;
}

//*****************************************************************************
// Function: void CMediaCoderG722::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void CMediaCoderG722::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   AAudioCoder::ExportSdpMediaDescription(a_mediaDescription,a_rtpPort);
}

//*****************************************************************************
// Function: void CMediaCoderRfc2833::ListProperties()
// Description: List the coder properties
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderRfc2833::ListProperties()
{
   LogMsg("\tm=" +  AsString(m_mediaType.c_str()) + " " + m_rtpAddress.c_str() + ":" 
                 + AsString(m_rtpPort) + " plt=" + AsString(m_payloadType));  
   LogMsg("\ta=rtpmap:" + AsString(m_payloadType) + " " + m_payloadCoderName.c_str() + "/" + AsString(m_clockRate));
   //printf("\tm=%s %s:%ld plt=%d\n", 
   //   m_mediaType.c_str(), m_rtpAddress.c_str(), m_rtpPort, m_payloadType);
   //printf("\ta=rtpmap:%d %s/%d\n",m_payloadType, m_payloadCoderName.c_str(),m_clockRate);
}

//*****************************************************************************
// Function: bool CMediaCoderRfc2833::MatchCoder(AMediaCoder *a_otherMediaCoder)
// Description: Attempt to match the specified coder and return the result.
// Return: bool 
// Parameters: AMediaCoder *a_otherMediaCoder 
//*****************************************************************************
bool CMediaCoderRfc2833::MatchCoder(AMediaCoder* a_otherMediaCoder)
{
   m_dialogicResourceType = RESOURCE_TYPE_NONE;
   CMediaCoderRfc2833* l_otherMediaCoder = dynamic_cast<CMediaCoderRfc2833*>(a_otherMediaCoder);
   if (!l_otherMediaCoder)
   {
      return false;
   }

   if (strcmp(GetPayloadCoderName(), l_otherMediaCoder->GetPayloadCoderName()))
   {
      return false;
   }
   return true;
}

//*****************************************************************************
// Function: void CMediaCoderRfc2833::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void CMediaCoderRfc2833::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   char l_buffer[COMMON_STRING_MAX_LENGTH] = {0};

   sdpMedia* l_media = a_mediaDescription->media();
   sprintf(l_buffer,"%d", m_payloadType);
   l_media->addFormat(l_buffer);

   sdpAttribute* l_attribute = a_mediaDescription->attributeList()->addItem();
   l_attribute->setProperty("rtpmap");
   sprintf(l_buffer, "%d %s/%d", m_payloadType, GetPayloadCoderName(), m_clockRate);
   l_attribute->setPropertyValue(l_buffer);

   l_attribute = a_mediaDescription->attributeList()->addItem();
   l_attribute->setProperty("fmtp");
   sprintf(l_buffer, "%d %s", m_payloadType, REQ_DTMF_EVENTS);
   l_attribute->setPropertyValue(l_buffer);
}

//*****************************************************************************
// Function: void AVideoCoder::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void AVideoCoder::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   char l_buffer[COMMON_STRING_MAX_LENGTH] = {0};

   sdpMedia* l_media = a_mediaDescription->media();
   l_media->setMedia("video");
   l_media->setPort(a_rtpPort);
   l_media->setTransport("RTP/AVP");
   l_media->setNumPorts(1);
   sprintf(l_buffer,"%d", m_payloadType);
   l_media->addFormat(l_buffer);

   sdpAttribute* l_attribute = a_mediaDescription->attributeList()->addItem();
   l_attribute ->setProperty("rtpmap");
   sprintf(l_buffer, "%d %s/%d", m_payloadType, GetPayloadCoderName(), m_clockRate);
   l_attribute ->setPropertyValue(l_buffer);

   l_attribute  = a_mediaDescription->attributeList()->addItem();
   l_attribute ->setProperty("sendrecv");

}

//*****************************************************************************
// Function: bool CMediaCoderH263::AddPictureSizeAndMpiCapability(CPictureSizeAndMpi *a_pictureSizeAndMpiCapability)
// Description: Add a picture size and MPI capability to the list of Picture 
// 				Size and MPI capabilities for the coder
// Return: bool 
// Parameters: CPictureSizeAndMpi *a_pictureSizeAndMpiCapability 
//*****************************************************************************
bool CMediaCoderH263::AddPictureSizeAndMpiCapability(CPictureSizeAndMpi* a_pictureSizeAndMpiCapability)
{
   for(unsigned int i=0; i < m_pictureSizeAndMpiCapabilities.size(); i++)
   {
      if(a_pictureSizeAndMpiCapability == m_pictureSizeAndMpiCapabilities[i])
      {
         return false;
      }
   }
   m_pictureSizeAndMpiCapabilities.push_back(a_pictureSizeAndMpiCapability);
   return true;
}

//*****************************************************************************
// Function: bool CMediaCoderH263::IsPictureSizeAndMpiSupported(CPictureSizeAndMpi *a_pictureAndSizeCapability)
// Description: Test if the picture size and MPI capability pair is supported 
// 				by the coder
// Return: bool 
// Parameters: CPictureSizeAndMpi *a_pictureAndSizeCapability 
//*****************************************************************************
bool CMediaCoderH263::IsPictureSizeAndMpiSupported(CPictureSizeAndMpi* a_pictureAndSizeCapability)
{
   std::vector<CPictureSizeAndMpi*>::iterator l_localPictureSizeAndMpi;
   for (l_localPictureSizeAndMpi=m_pictureSizeAndMpiCapabilities.begin();
        l_localPictureSizeAndMpi!=m_pictureSizeAndMpiCapabilities.end();
        l_localPictureSizeAndMpi++)
   {
      if ( (a_pictureAndSizeCapability->PictureSize() == (*l_localPictureSizeAndMpi)->PictureSize())
         && (a_pictureAndSizeCapability->MinimumPictureInterval() == (*l_localPictureSizeAndMpi)->MinimumPictureInterval()))
      {
         return true;
      }
   }
   return false;
}

//*****************************************************************************
// Function: CMediaCoderH263::CMediaCoderH263()
// Description: Default constructor
// Return:  
// Parameters: none 
//*****************************************************************************
CMediaCoderH263::CMediaCoderH263() 
   :AVideoCoder("H263",34)
   ,m_matchedPictureSizeAndMpiCapability(0)
   ,m_maxBitRate(0)
{
   m_dialogicCoderType = CODER_TYPE_H263;
}

//*****************************************************************************
// Function: CMediaCoderH263::CMediaCoderH263(const IPM_VIDEO_CODER_INFO &l_videoCoderInfo)
// Description: Initializing construtor
// Return:  
// Parameters: const IPM_VIDEO_CODER_INFO &l_videoCoderInfo 
//*****************************************************************************
CMediaCoderH263::CMediaCoderH263(const IPM_VIDEO_CODER_INFO& l_videoCoderInfo)
   :AVideoCoder("H263",34)
   ,m_matchedPictureSizeAndMpiCapability(0)
   ,m_maxBitRate(0)
{
   m_dialogicCoderType = CODER_TYPE_H263;
   CPictureSizeAndMpi* l_pictureCapability;
   l_pictureCapability = new CPictureSizeAndMpi( "CIF", 1);
   AddPictureSizeAndMpiCapability(l_pictureCapability);
   l_pictureCapability = new CPictureSizeAndMpi( "CIF", 2);
   AddPictureSizeAndMpiCapability(l_pictureCapability);
   l_pictureCapability = new CPictureSizeAndMpi( "CIF", 3);
   AddPictureSizeAndMpiCapability(l_pictureCapability);
   l_pictureCapability = new CPictureSizeAndMpi( "CIF", 5);
   AddPictureSizeAndMpiCapability(l_pictureCapability);
   l_pictureCapability = new CPictureSizeAndMpi( "QCIF", 1);
   AddPictureSizeAndMpiCapability(l_pictureCapability);
   l_pictureCapability = new CPictureSizeAndMpi( "QCIF", 2);
   AddPictureSizeAndMpiCapability(l_pictureCapability);
   l_pictureCapability = new CPictureSizeAndMpi( "QCIF", 3);
   AddPictureSizeAndMpiCapability(l_pictureCapability);
   l_pictureCapability = new CPictureSizeAndMpi( "QCIF", 5);
   AddPictureSizeAndMpiCapability(l_pictureCapability);
}

//*****************************************************************************
// Function: void CMediaCoderH263::ImportSdpFmtp(const std::string &a_attribute)
// Description: Import coder attributes from SDP
// Return: void 
// Parameters: const std::string &a_attribute 
//*****************************************************************************
void CMediaCoderH263::ImportSdpFmtp(const std::string& a_attribute)
{
   //printf("CMediaCoderH263::ImportSdpFmtp\n");
   unsigned int l_payloadType, l_tokenCount=0;
   char l_separators[] = "= ;";
   char *l_token;
   char l_subAttribute[COMMON_STRING_MAX_LENGTH];
   char l_subAttributeValue[MEDIA_ATTRIB_SIZE_MAX];
   char l_attribute[MEDIA_ATTRIB_SIZE_MAX];
   char* l_tokenizerBuffer = NULL;

   size_t l_attributeSize = a_attribute.size();
   if(l_attributeSize >= MEDIA_ATTRIB_SIZE_MAX)
   {
      printf("CMediaCoderH263::ImportSdpFmtp - Attribute.size()= %u is greater than MEDIA_ATTRIB_SIZE_MAX=%u\n",
                         a_attribute.size(),
                         MEDIA_ATTRIB_SIZE_MAX);
      return;
   }

   strcpy(l_attribute, a_attribute.c_str());
   l_attribute[l_attributeSize] = '\0';

   memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
   memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);

   l_token = DLGC_STRTOK(l_attribute, l_separators, &l_tokenizerBuffer);
   while (l_token != NULL)
   {
      switch (l_tokenCount)
      {
      case 0:
         l_payloadType = atoi(l_token);
         if (l_payloadType != m_payloadType)
         {
            printf("CMediaCoderH263::ImportSdpFmtp - Payload Number Mismatch\n");
            return;
         }
         break;

      case 1:
      case 3:
      case 5:
         strcpy(l_subAttribute, l_token);
         break;

      case 2:
      case 4:
      case 6:
         strcpy(l_subAttributeValue, l_token);
         if (!strcmp(l_subAttribute, "CIF") || !strcmp(l_subAttribute, "QCIF"))
         {
            CPictureSizeAndMpi* l_pictureCapability = new CPictureSizeAndMpi( l_subAttribute, atoi(l_subAttributeValue));
            if (!AddPictureSizeAndMpiCapability(l_pictureCapability))
            {
               printf("CMediaCoderH263::ImportSdpFmtp - error adding picture size and MPI capability %s:%d\n",
                  l_subAttribute, atoi(l_subAttributeValue));
               return;
            }
         }
         else if (!strcmp(l_subAttribute, "MaxBR"))
         {
            m_maxBitRate = atoi(l_subAttributeValue);
         }

         memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
         memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);
         break;

      default:
         break;
      }

      l_token = DLGC_STRTOK(NULL, l_separators, &l_tokenizerBuffer);
      l_tokenCount++;
   }
}

//*****************************************************************************
// Function: void CMediaCoderH263::ListProperties()
// Description: List the coder properties
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderH263::ListProperties()
{
   //printf("\t%s\n", GetDialogicCoderTypeDesc(m_dialogicCoderType));
   //printf("\tm=%s %s:%ld plt=%d, peerplt=%d\n", 
   //   m_mediaType.c_str(), m_rtpAddress.c_str(), m_rtpPort, m_payloadType, m_peerPayloadType);
   //printf("\ta=rtpmap:%d %s/%d\n",m_payloadType, m_payloadCoderName.c_str(), m_clockRate);
   
   LogMsg("\t" + AsString(GetDialogicCoderTypeDesc(m_dialogicCoderType)));
   LogMsg("\tm=" + AsString(m_mediaType.c_str()) + " " + AsString(m_rtpAddress.c_str()) + ":" + AsString(m_rtpPort));
   LogMsg("\tplt=" + AsString(m_payloadType) + " peerplt=" + AsString(m_peerPayloadType));
   LogMsg("\ta=rtpmap:" + AsString(m_payloadType) + " " + AsString(m_payloadCoderName.c_str()) + "/" + AsString(m_clockRate));

   if (m_pictureSizeAndMpiCapabilities.size() > 0)
   {
      //printf("\ta=rtpmap:");
      LogMsg("\ta=rtpmap:");
      std::vector<CPictureSizeAndMpi*>::iterator l_pictureSizeAndMpi;
      for (l_pictureSizeAndMpi=m_pictureSizeAndMpiCapabilities.begin();
           l_pictureSizeAndMpi!=m_pictureSizeAndMpiCapabilities.end();
           l_pictureSizeAndMpi++)
      {
         //printf(" %s=%d", (*l_pictureSizeAndMpi)->PictureSize().c_str(),(*l_pictureSizeAndMpi)->MinimumPictureInterval());
         LogMsg("\t\t"+AsString((*l_pictureSizeAndMpi)->PictureSize().c_str()) + "=" + AsString((*l_pictureSizeAndMpi)->MinimumPictureInterval()));
      }
      if (m_maxBitRate)
      {
         //printf(" MaxBr=%d", m_maxBitRate);
         LogMsg("MaxBr=" + AsString(m_maxBitRate));
      }
      //printf("\n");
   }
   else if (m_maxBitRate > 0)
   {
      //printf("\ta=rtpmap: MaxBr=%d\n", m_maxBitRate);
      LogMsg("\ta=rtpmap: MaxBr=" + AsString(m_maxBitRate));
   }
}

//*****************************************************************************
// Function: bool CMediaCoderH263::MatchCoder(AMediaCoder *a_otherMediaCoder)
// Description: Attempt to match the specified coder and return the result.
// Return: bool 
// Parameters: AMediaCoder *a_otherMediaCoder 
//*****************************************************************************
bool CMediaCoderH263::MatchCoder(AMediaCoder* a_otherMediaCoder)
{
   m_dialogicResourceType = RESOURCE_TYPE_NONE;
   CMediaCoderH263* l_otherMediaCoder = dynamic_cast<CMediaCoderH263*>(a_otherMediaCoder);
   if (!l_otherMediaCoder)
   {
      return false;
   }

   if (strcmp(GetPayloadCoderName(), l_otherMediaCoder->GetPayloadCoderName()))
   {
      return false;
   }

   if (GetClockRate() !=  l_otherMediaCoder->GetClockRate())
   {
      return false;
   }

   bool l_haveMatchedSizeAndMpi(false);
   std::vector<CPictureSizeAndMpi*>::iterator l_localPictureSizeAndMpi;
   for (l_localPictureSizeAndMpi=m_pictureSizeAndMpiCapabilities.begin();
        l_localPictureSizeAndMpi!=m_pictureSizeAndMpiCapabilities.end();
        l_localPictureSizeAndMpi++)
   {
      if (l_otherMediaCoder->IsPictureSizeAndMpiSupported(*l_localPictureSizeAndMpi))
      {
         if(l_haveMatchedSizeAndMpi==false){   
            m_matchedPictureSizeAndMpiCapability = *l_localPictureSizeAndMpi;
            l_haveMatchedSizeAndMpi = true;
         } 
         // JM - else check for higher resolution match
         else {
            if((m_matchedPictureSizeAndMpiCapability->PictureSize()=="QCIF") &&
               ((*l_localPictureSizeAndMpi)->PictureSize()=="CIF"))
            {
               m_matchedPictureSizeAndMpiCapability = *l_localPictureSizeAndMpi;
               l_haveMatchedSizeAndMpi = true;
            } 
         }   
      }
   }

   // Set Decoder values to max Level
   // Remote = Decoder = Rx Values
   m_matchedFramesPerSec_Local = VIDEO_FRAMESPERSEC_30;
   m_matchedVideoProfile_Local = VIDEO_PROFILE_0_H263;
   m_matchedVideoLevel_Local = VIDEO_LEVEL_30_H263;
   m_matchedImageWidth_Local = VIDEO_IMAGE_WIDTH_352;
   m_matchedImageHeight_Local = VIDEO_IMAGE_HEIGHT_288;
   m_matchedBitRate_Local = 384000;

   if (l_haveMatchedSizeAndMpi)
   {
      m_matchedVideoProfile_Remote = VIDEO_PROFILE_0_H263;
      m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_DEFAULT;
      m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_DEFAULT;
      m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_DEFAULT;
      m_matchedBitRate_Remote = VIDEO_BITRATE_DEFAULT;
      m_matchedVideoLevel_Remote = VIDEO_LEVEL_DEFAULT;
      
      if (m_matchedPictureSizeAndMpiCapability->PictureSize() == "CIF")
      {
         m_matchedImageWidth_Remote = m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_352;
         m_matchedImageHeight_Remote = m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_288;
         switch (m_matchedPictureSizeAndMpiCapability->MinimumPictureInterval())
         {
            case 1:
               m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_30;
               m_matchedVideoLevel_Remote = VIDEO_LEVEL_30_H263;
               m_matchedBitRate_Remote = VIDEO_BITRATE_384K;
               break;
            case 2:
               m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_15;
               m_matchedVideoLevel_Remote = VIDEO_LEVEL_20_H263;
               m_matchedBitRate_Remote = VIDEO_BITRATE_384K;
               break;
            case 3:
               m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_10;
               m_matchedVideoLevel_Remote = VIDEO_LEVEL_10_H263;
               m_matchedBitRate_Remote = VIDEO_BITRATE_128K;
               break;
            case 5:
               m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_6;
               m_matchedVideoLevel_Remote = VIDEO_LEVEL_10_H263;
               m_matchedBitRate_Remote = VIDEO_BITRATE_384K;
               break;

         }
      }
      else if (m_matchedPictureSizeAndMpiCapability->PictureSize() == "QCIF")
      {
         m_matchedImageWidth_Local = m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_176;
         m_matchedImageHeight_Local = m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_144;
         switch (m_matchedPictureSizeAndMpiCapability->MinimumPictureInterval())
         {
            case 1:
               m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_30;
               m_matchedVideoLevel_Remote = VIDEO_LEVEL_20_H263;
               m_matchedBitRate_Remote = VIDEO_BITRATE_384K;
               break;
            case 2:
               m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_15;
               m_matchedVideoLevel_Remote = VIDEO_LEVEL_20_H263;
               m_matchedBitRate_Remote = VIDEO_BITRATE_128K;
               break;
            case 3:
               m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_10;
               m_matchedVideoLevel_Remote = VIDEO_LEVEL_10_H263;
               m_matchedBitRate_Remote = VIDEO_BITRATE_64K;
               break;
            case 5:
               m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_6;
               m_matchedVideoLevel_Remote = VIDEO_LEVEL_10_H263;
               m_matchedBitRate_Remote = VIDEO_BITRATE_64K;
               break;
         }
      }

      if (m_maxBitRate)
      {
         m_matchedBitRate_Remote = m_maxBitRate * 100;
      }
      return true;
   }
   else //(!l_haveMatchedSizeAndMpi) 
   {
      //TODO: Add SQCIF, 4CIF, etc. separately 
      //JM - Add to support case where SIP INVITE does not include fmtp QCIF/CIF value (assume QCIF)
      LogMsg("Warning: SIP INVITE (H263 Coder) did not match ftmp, assume QCIF/15fps/50Kbps/Lev10");  
      CPictureSizeAndMpi* l_pictureCapability;
      l_pictureCapability = new CPictureSizeAndMpi( "QCIF", 2);
      AddPictureSizeAndMpiCapability(l_pictureCapability);
      m_matchedPictureSizeAndMpiCapability = l_pictureCapability;
      m_matchedVideoProfile_Remote = VIDEO_PROFILE_0_H263;
      m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_DEFAULT; //QCIF - 176
      m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_DEFAULT; //QCIF - 144
      m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_DEFAULT; //15FPS
      m_matchedBitRate_Remote = VIDEO_BITRATE_DEFAULT; //50KBPS
      m_matchedVideoLevel_Remote = VIDEO_LEVEL_DEFAULT; //H263 Level10
      return true;
   }
     
   return false;
}

//*****************************************************************************
// Function: bool CMediaCoderH263::UpdateMatchedCoder(unsigned int a_preferredBitRate)
// Description: Update the Matched Coder based on application preference 
// Return: bool 
// Parameters: unsigned int a_preferredBitRate
//*****************************************************************************
bool CMediaCoderH263::UpdateMatchedCoder(unsigned int a_preferredBitRate)
{
   //Only update Video Level if Remote endpoint did not send a maxBitRate
   if(m_maxBitRate == 0)
   {  
      LogMsg("UpdateMatchedCoder with bitrate =" + AsString(a_preferredBitRate));
      if (m_matchedPictureSizeAndMpiCapability->PictureSize() == "CIF")
      {
         if(a_preferredBitRate <= VIDEO_BITRATE_128K)
         {
            m_matchedBitRate_Remote = a_preferredBitRate;
            LogMsg("Update H263 Level to Level 20");
            m_matchedVideoLevel_Remote = VIDEO_LEVEL_20_H263;  
         }
         else
         {
            m_matchedBitRate_Remote = a_preferredBitRate;
            LogMsg("Update H263 Level to Level 30");
            m_matchedVideoLevel_Remote = VIDEO_LEVEL_30_H263;  
         }
      }
      else if (m_matchedPictureSizeAndMpiCapability->PictureSize() == "QCIF")
      {
         if(a_preferredBitRate <= VIDEO_BITRATE_64K)
         {
            m_matchedBitRate_Remote = a_preferredBitRate;
            LogMsg("Update H263 Level to Level 10");
            m_matchedVideoLevel_Remote = VIDEO_LEVEL_10_H263;  
         }
         else if (a_preferredBitRate > VIDEO_BITRATE_64K && a_preferredBitRate <= VIDEO_BITRATE_128K)
         {
            m_matchedBitRate_Remote = a_preferredBitRate;
            LogMsg("Update H263 Level to Level 20");
            m_matchedVideoLevel_Remote = VIDEO_LEVEL_20_H263;  
         }
         else
         {
            m_matchedBitRate_Remote = a_preferredBitRate;
            LogMsg("Update H263 Level to Level 30");
            m_matchedVideoLevel_Remote = VIDEO_LEVEL_30_H263;  
         }
      }
      return true;
    }
    else
    {
      LogMsg("Did NOT UpdateMatchedCoder (H263) with preferred bitrate =" + AsString(a_preferredBitRate));
      return false;
    }
   
}

//*****************************************************************************
// Function: void CMediaCoderH263::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void CMediaCoderH263::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   AVideoCoder::ExportSdpMediaDescription(a_mediaDescription,a_rtpPort);
   if (m_matchedPictureSizeAndMpiCapability || m_maxBitRate)
   {
      sdpAttribute* l_attribute = a_mediaDescription->attributeList()->addItem();
      l_attribute->setProperty("fmtp");

      char l_pictureSizeAndMpiBuffer[COMMON_STRING_MAX_LENGTH] = {0};
      if (m_matchedPictureSizeAndMpiCapability)
      {
         sprintf( l_pictureSizeAndMpiBuffer, "%s=%d", 
                  m_matchedPictureSizeAndMpiCapability->PictureSize().c_str(),
                  m_matchedPictureSizeAndMpiCapability->MinimumPictureInterval());
      }

      char l_maxBitRateBuffer[COMMON_STRING_MAX_LENGTH] = {0};
      if (m_maxBitRate)
      {
         sprintf(l_maxBitRateBuffer, "MaxBr=%d", m_maxBitRate);
      }

      char l_buffer[COMMON_STRING_MAX_LENGTH * 3] = {0};
      sprintf(l_buffer,"%d %s %s",m_payloadType,l_pictureSizeAndMpiBuffer,l_maxBitRateBuffer);
      l_attribute ->setPropertyValue(l_buffer);
   }
}

//*****************************************************************************
// Function: CMediaCoderMp4v::CMediaCoderMp4v(unsigned a_payloadType)
// Description: Initializing constructor
// Return:  
// Parameters: unsigned a_payloadType 
//*****************************************************************************
CMediaCoderMp4v::CMediaCoderMp4v( unsigned a_payloadType)
   :AVideoCoder("MP4V-ES",a_payloadType)
   ,m_matchedProfileLevelIndication(VIDEO_PROFILE_LEVEL_SP0_MPEG4)
{
   m_dialogicCoderType = CODER_TYPE_MP4V_ES;
}

//*****************************************************************************
// Function: CMediaCoderMp4v::CMediaCoderMp4v(const IPM_VIDEO_CODER_INFO &l_videoCoderInfo)
// Description: Initializing construtor
// Return:  
// Parameters: const IPM_VIDEO_CODER_INFO &l_videoCoderInfo 
//*****************************************************************************
CMediaCoderMp4v::CMediaCoderMp4v(const IPM_VIDEO_CODER_INFO& l_videoCoderInfo)
   :AVideoCoder("MP4V-ES",l_videoCoderInfo.unCoderPayloadType)
   ,m_matchedProfileLevelIndication(VIDEO_PROFILE_LEVEL_SP0_MPEG4)
{
   AddProfileLevelIndication(VIDEO_PROFILE_LEVEL_SP0_MPEG4);
   AddProfileLevelIndication(VIDEO_PROFILE_LEVEL_SP1_MPEG4);
   AddProfileLevelIndication(VIDEO_PROFILE_LEVEL_SP2_MPEG4);
   AddProfileLevelIndication(VIDEO_PROFILE_LEVEL_SP3_MPEG4);
}

//*****************************************************************************
// Function: bool CMediaCoderMp4v::AddProfileLevelIndication(unsigned int a_profileLevelIndication)
// Description: Add a profile level indication capability to the list of 
// 				profile level indications
// Return: bool 
// Parameters: unsigned int a_profileLevelIndication 
//*****************************************************************************
bool CMediaCoderMp4v::AddProfileLevelIndication(unsigned int a_profileLevelIndication)
{
   for(unsigned int i=0; i < m_profileLevelIndications.size(); i++)
   {
      if(a_profileLevelIndication == m_profileLevelIndications[i])
      {
         return false;
      }
   }
   m_profileLevelIndications.push_back(a_profileLevelIndication);
   return true;
}

//*****************************************************************************
// Function: bool CMediaCoderMp4v::IsProfileLevelIndicationSupported(unsigned int a_profileLevelIndication)
// Description: Test is a profile level indication is supported by the coder
// Return: bool 
// Parameters: unsigned int a_profileLevelIndication 
//*****************************************************************************
bool CMediaCoderMp4v::IsProfileLevelIndicationSupported(unsigned int a_profileLevelIndication)
{
   for(unsigned int i=0; i < m_profileLevelIndications.size(); i++)
   {
      if(m_profileLevelIndications[i] == a_profileLevelIndication)
      {
         return true;
      }
   }
   return false;
}

//*****************************************************************************
// Function: void CMediaCoderMp4v::ImportSdpFmtp(const std::string &a_attribute)
// Description: Import coder attributes from SDP
// Return: void 
// Parameters: const std::string &a_attribute 
//*****************************************************************************
void CMediaCoderMp4v::ImportSdpFmtp(const std::string& a_attribute)
{
//   printf("CMediaCoderMp4v::ImportSdpFmtp\n");
   unsigned int l_payloadType, l_tokenCount=0;
   char l_separators[] = "= ;";
   char *l_token;
   char l_subAttribute[COMMON_STRING_MAX_LENGTH];
   char l_subAttributeValue[MEDIA_ATTRIB_SIZE_MAX];
   char l_attribute[MEDIA_ATTRIB_SIZE_MAX];
   char* l_tokenizerBuffer = NULL;

   size_t l_attributeSize = a_attribute.size();
   if(l_attributeSize >= MEDIA_ATTRIB_SIZE_MAX)
   {
      printf("CMediaCoderMp4v::ImportSdpFmtp - Attribute.size()= %u is greater than MEDIA_ATTRIB_SIZE_MAX=%u\n",
                         a_attribute.size(),
                         MEDIA_ATTRIB_SIZE_MAX);
      return;
   }

   strcpy(l_attribute, a_attribute.c_str());
   l_attribute[l_attributeSize] = '\0';

   memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
   memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);

   l_token = DLGC_STRTOK(l_attribute, l_separators, &l_tokenizerBuffer);
   while (l_token != NULL)
   {
      switch (l_tokenCount)
      {
      case 0:
         l_payloadType = atoi(l_token);
         if (l_payloadType != m_payloadType)
         {
            LogMsg("CMediaCoderMp4v::ImportSdpFmtp - Payload Number Mismatch");
            return;
         }
         break;

      case 1:
      case 3:
      case 5:
         strcpy(l_subAttribute, l_token);
         break;

      case 2:
      case 4:
      case 6:
         strcpy(l_subAttributeValue, l_token);
         if (!strcmp(l_subAttribute, "profile-level-id"))
         {
            unsigned int l_profileLevelIndication = atoi(l_subAttributeValue);
            AddProfileLevelIndication(l_profileLevelIndication);
         }

         memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
         memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);
         break;

      default:
         break;
      }

      l_token = DLGC_STRTOK(NULL, l_separators, &l_tokenizerBuffer);
      l_tokenCount++;
   }
}

//*****************************************************************************
// Function: void CMediaCoderMp4v::ListProperties()
// Description: List the coder properties
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderMp4v::ListProperties()
{
   
   LogMsg("\t" + AsString(GetDialogicCoderTypeDesc(m_dialogicCoderType)));
   LogMsg("\tm=" + AsString(m_mediaType.c_str()) + " " + AsString(m_rtpAddress.c_str()) + ":" + AsString(m_rtpPort));
   LogMsg("\tplt=" + AsString(m_payloadType) + " peerplt=" + AsString(m_peerPayloadType));
   LogMsg("\ta=rtpmap:" + AsString(m_payloadType) + " " + AsString(m_payloadCoderName.c_str()) + "/" + AsString(m_clockRate));

   if (m_matchedProfileLevelIndication > 0)
   {
      LogMsg("\ta=fmtp:" + AsString(m_payloadType) + " profile-level-id=" + AsString(m_matchedProfileLevelIndication));
   }
}

//*****************************************************************************
// Function: bool CMediaCoderMp4v::MatchCoder(AMediaCoder *a_otherMediaCoder)
// Description: Attempt to match the specified coder and return the result.
// Return: bool 
// Parameters: AMediaCoder *a_otherMediaCoder 
//*****************************************************************************
bool CMediaCoderMp4v::MatchCoder(AMediaCoder* a_otherMediaCoder)
{
   m_matchedProfileLevelIndication = VIDEO_PROFILE_LEVEL_SP0_MPEG4;
   m_dialogicResourceType = RESOURCE_TYPE_NONE;
   CMediaCoderMp4v* l_otherMediaCoder = dynamic_cast<CMediaCoderMp4v*>(a_otherMediaCoder);
   if (!l_otherMediaCoder)
   {
      return false;
   }

   if (strcmp(GetPayloadCoderName(), l_otherMediaCoder->GetPayloadCoderName()))
   {
      return false;
   }

   if (GetClockRate() !=  l_otherMediaCoder->GetClockRate())
   {
      return false;
   }

   std::vector<unsigned int>::iterator l_profileLevelIndication;
   for (l_profileLevelIndication=m_profileLevelIndications.begin();
        l_profileLevelIndication!=m_profileLevelIndications.end();
        l_profileLevelIndication++)
   {
      if (l_otherMediaCoder->IsProfileLevelIndicationSupported(*l_profileLevelIndication))
      {
         m_matchedProfileLevelIndication = *l_profileLevelIndication;
         break;
      }
   }

   m_matchedVideoProfile_Remote = static_cast<eVIDEO_PROFILE>(m_matchedProfileLevelIndication);
   m_matchedVideoLevel_Remote = VIDEO_LEVEL_DEFAULT;
   m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_DEFAULT;
   m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_DEFAULT;
   m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_DEFAULT;
   m_matchedBitRate_Remote = VIDEO_BITRATE_DEFAULT;
   
   // Set Decoder values to max Level
   // Local = Decoder = Rx Values
   m_matchedFramesPerSec_Local = VIDEO_FRAMESPERSEC_30;
   m_matchedVideoProfile_Local = VIDEO_PROFILE_LEVEL_SP3_MPEG4;
   m_matchedVideoLevel_Local = VIDEO_LEVEL_DEFAULT;
   m_matchedImageWidth_Local = VIDEO_IMAGE_WIDTH_352;
   m_matchedImageHeight_Local = VIDEO_IMAGE_HEIGHT_288;
   m_matchedBitRate_Local = 384000;

   switch(m_matchedProfileLevelIndication)
   {
      case VIDEO_PROFILE_LEVEL_SP0_MPEG4:
      case VIDEO_PROFILE_LEVEL_SP1_MPEG4:
         m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_176;
         m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_144;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_15;
         m_matchedBitRate_Remote = 64000;
         break;
      case VIDEO_PROFILE_LEVEL_SP2_MPEG4:
         //m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_176;
         //m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_144;
         m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_352;
         m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_288;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_15;
         m_matchedBitRate_Remote = 128000;
         break;
      case VIDEO_PROFILE_LEVEL_SP3_MPEG4:
         m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_352;
         m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_288;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_30;
         m_matchedBitRate_Remote = 384000;
         break;
   }
   return true;
}


//*****************************************************************************
// Function: bool CMp4v::UpdateMatchedCoder(unsigned int a_preferredBitRate)
// Description: Update the Matched Coder based on application preference 
// Return: bool 
// Parameters: unsigned int a_preferredBitRate
//*****************************************************************************
bool CMediaCoderMp4v::UpdateMatchedCoder(unsigned int a_preferredBitRate)
{
   //Only update Bitrate and Video Level if Remote endpoint did not send a maxBitRate
   if(a_preferredBitRate <= VIDEO_BITRATE_384K)
   {  
      LogMsg("UpdateMatchedCoder with bitrate =" + AsString(a_preferredBitRate));
      switch(m_matchedProfileLevelIndication)
      {
         case VIDEO_PROFILE_LEVEL_SP0_MPEG4:
         case VIDEO_PROFILE_LEVEL_SP1_MPEG4:
            if(a_preferredBitRate <= VIDEO_BITRATE_64K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            else if(VIDEO_BITRATE_64K < a_preferredBitRate && a_preferredBitRate <= VIDEO_BITRATE_128K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
               LogMsg("Update MPEG4 Profile to SimpleProfile2");
               m_matchedProfileLevelIndication = VIDEO_PROFILE_LEVEL_SP2_MPEG4;  
            } 
            else
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
               LogMsg("Update MPEG4 Profile to SimpleProfile3");
               m_matchedProfileLevelIndication = VIDEO_PROFILE_LEVEL_SP3_MPEG4;  
            }
            break; 
         case VIDEO_PROFILE_LEVEL_SP2_MPEG4:
            if(a_preferredBitRate <= VIDEO_BITRATE_128K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            else
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
               LogMsg("Update MPEG4 Profile to SimpleProfile3");
               m_matchedProfileLevelIndication = VIDEO_PROFILE_LEVEL_SP3_MPEG4;  
            }
            break;
         case VIDEO_PROFILE_LEVEL_SP3_MPEG4:
            if(a_preferredBitRate <= VIDEO_BITRATE_384K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            break;
         default:
            LogMsg("Warning: Unknown MPEG4 Profile/Level=" + AsString(m_matchedProfileLevelIndication));
            break;
      }
      return true;
    }
    else
    {
      LogMsg("Did NOT UpdateMatchedCoder (MPEG4) with preferred bitrate =" + AsString(a_preferredBitRate));
      return false;
    }
    return true;   
}

//*****************************************************************************
// Function: void CMediaCoderMp4v::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void CMediaCoderMp4v::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   AVideoCoder::ExportSdpMediaDescription(a_mediaDescription,a_rtpPort);
   if (m_matchedProfileLevelIndication)
   {
      sdpAttribute* l_attribute = a_mediaDescription->attributeList()->addItem();
      l_attribute->setProperty("fmtp");
      char l_buffer[COMMON_STRING_MAX_LENGTH * 3] = {0};
      sprintf(l_buffer,"%d profile-level-id=%d",m_payloadType,m_matchedProfileLevelIndication);
      l_attribute ->setPropertyValue(l_buffer);
   }
}

//*****************************************************************************
// Function: const char* AMediaCoder::GetDialogicCoderTypeDesc(unsigned int a_dialogicCoderType)
// Description: Return the description for a Dialogic coder type enum
// Return: const char* 
// Parameters: unsigned int a_dialogicCoderType 
//*****************************************************************************
const char* AMediaCoder::GetDialogicCoderTypeDesc(unsigned int a_dialogicCoderType)
{
   switch(a_dialogicCoderType)
   {
   case CODER_TYPE_NONSTANDARD:
	   return "CODER_TYPE_NONSTANDARD";
	case CODER_TYPE_G711ALAW64K:
	   return "CODER_TYPE_G711ALAW64K";
	case CODER_TYPE_G711ALAW64K_NATIVE:
	   return "CODER_TYPE_G711ALAW64K_NATIVE";
	case CODER_TYPE_G711ALAW56K:
	   return "CODER_TYPE_G711ALAW56K";
	case CODER_TYPE_G711ALAW56K_NATIVE:
	   return "CODER_TYPE_G711ALAW56K_NATIVE";
	case CODER_TYPE_G711ULAW64K:
	   return "CODER_TYPE_G711ULAW64K";
	case CODER_TYPE_G711ULAW64K_NATIVE:
	   return "CODER_TYPE_G711ULAW64K_NATIVE";
	case CODER_TYPE_G711ULAW56K:
	   return "CODER_TYPE_G711ULAW56K";
	case CODER_TYPE_G711ULAW56K_NATIVE:
	   return "CODER_TYPE_G711ULAW56K_NATIVE";
	case CODER_TYPE_G721ADPCM:
	   return "CODER_TYPE_G721ADPCM";
	case CODER_TYPE_G721ADPCM_NATIVE:
	   return "CODER_TYPE_G721ADPCM_NATIVE";
	case CODER_TYPE_G722_48K:
	   return "CODER_TYPE_G722_48K";
	case CODER_TYPE_G722_48K_NATIVE:
	   return "CODER_TYPE_G722_48K_NATIVE";
	case CODER_TYPE_G722_56K:
	   return "CODER_TYPE_G722_56K";
	case CODER_TYPE_G722_56K_NATIVE:
	   return "CODER_TYPE_G722_56K_NATIVE";
	case CODER_TYPE_G722_64K:
	   return "CODER_TYPE_G722_64K";
	case CODER_TYPE_G722_64K_NATIVE:
	   return "CODER_TYPE_G722_64K_NATIVE";
	case CODER_TYPE_G7231_5_3K:
	   return "CODER_TYPE_G7231_5_3K";
	case CODER_TYPE_G7231_5_3K_NATIVE:
	   return "CODER_TYPE_G7231_5_3K_NATIVE";
	case CODER_TYPE_G7231_6_3K:
	   return "CODER_TYPE_G7231_6_3K";
	case CODER_TYPE_G7231_6_3K_NATIVE:
	   return "CODER_TYPE_G7231_6_3K_NATIVE";
	case CODER_TYPE_G7231ANNEXCCAP:
	   return "CODER_TYPE_G7231ANNEXCCAP";
	case CODER_TYPE_G7231ANNEXCCAP_NATIVE:
	   return "CODER_TYPE_G7231ANNEXCCAP_NATIVE";
	case CODER_TYPE_G726_16K:
	   return "CODER_TYPE_G726_16K";
	case CODER_TYPE_G726_16K_NATIVE:
	   return "CODER_TYPE_G726_16K_NATIVE";
	case CODER_TYPE_G726_24K:
	   return "CODER_TYPE_G726_24K";
	case CODER_TYPE_G726_24K_NATIVE:
	   return "CODER_TYPE_G726_24K_NATIVE";
	case CODER_TYPE_G726_32K:
	   return "CODER_TYPE_G726_32K";
	case CODER_TYPE_G726_32K_NATIVE:
	   return "CODER_TYPE_G726_32K_NATIVE";
	case CODER_TYPE_G726_40K:
	   return "CODER_TYPE_G726_40K";
	case CODER_TYPE_G726_40K_NATIVE:
	   return "CODER_TYPE_G726_40K_NATIVE";
	case CODER_TYPE_G728:
	   return "CODER_TYPE_G728";
	case CODER_TYPE_G728_NATIVE:
	   return "CODER_TYPE_G728_NATIVE";
	case CODER_TYPE_G729:
	   return "CODER_TYPE_G729";
	case CODER_TYPE_G729_NATIVE:
	   return "CODER_TYPE_G729_NATIVE";
	case CODER_TYPE_G729ANNEXA:
	   return "CODER_TYPE_G729ANNEXA";
	case CODER_TYPE_G729ANNEXA_NATIVE:
	   return "CODER_TYPE_G729ANNEXA_NATIVE";
	case CODER_TYPE_G729WANNEXB:
	   return "CODER_TYPE_G729WANNEXB";
	case CODER_TYPE_G729WANNEXB_NATIVE:
	   return "CODER_TYPE_G729WANNEXB_NATIVE";
	case CODER_TYPE_G729ANNEXAWANNEXB:
	   return "CODER_TYPE_G729ANNEXAWANNEXB";
	case CODER_TYPE_G729ANNEXAWANNEXB_NATIVE:
	   return "CODER_TYPE_G729ANNEXAWANNEXB_NATIVE";
	case CODER_TYPE_GSMFULLRATE:
	   return "CODER_TYPE_GSMFULLRATE";
	case CODER_TYPE_GSMFULLRATE_NATIVE:
	   return "CODER_TYPE_GSMFULLRATE_NATIVE";
	case CODER_TYPE_GSMHALFRATE:
	   return "CODER_TYPE_GSMHALFRATE";
	case CODER_TYPE_GSMHALFRATE_NATIVE:
	   return "CODER_TYPE_GSMHALFRATE_NATIVE";
	case CODER_TYPE_GSMEFR:
	   return "CODER_TYPE_GSMEFR";
	case CODER_TYPE_GSMEFR_NATIVE:
	   return "CODER_TYPE_GSMEFR_NATIVE";
	case CODER_TYPE_IS11172:
	   return "CODER_TYPE_IS11172";
	case CODER_TYPE_IS11172_NATIVE:
	   return "CODER_TYPE_IS11172_NATIVE";
	case CODER_TYPE_IS13818:
	   return "CODER_TYPE_IS13818";
	case CODER_TYPE_IS13818_NATIVE:
	   return "CODER_TYPE_IS13818_NATIVE";
	case CODER_TYPE_AMRNB_4_75k:
	   return "CODER_TYPE_AMRNB_4_75k";
	case CODER_TYPE_AMRNB_4_75k_NATIVE:
	   return "CODER_TYPE_AMRNB_4_75k_NATIVE";
	case CODER_TYPE_AMRNB_5_15k:
	   return "CODER_TYPE_AMRNB_5_15k";
	case CODER_TYPE_AMRNB_5_15k_NATIVE:
	   return "CODER_TYPE_AMRNB_5_15k_NATIVE";
	case CODER_TYPE_AMRNB_5_9k:
	   return "CODER_TYPE_AMRNB_5_9k";
	case CODER_TYPE_AMRNB_5_9k_NATIVE:
	   return "CODER_TYPE_AMRNB_5_9k_NATIVE";
	case CODER_TYPE_AMRNB_6_7k:
	   return "CODER_TYPE_AMRNB_6_7k";
	case CODER_TYPE_AMRNB_6_7k_NATIVE:
	   return "CODER_TYPE_AMRNB_6_7k_NATIVE";
	case CODER_TYPE_AMRNB_7_4k:
	   return "CODER_TYPE_AMRNB_7_4k";
	case CODER_TYPE_AMRNB_7_4k_NATIVE:
	   return "CODER_TYPE_AMRNB_7_4k_NATIVE";
	case CODER_TYPE_AMRNB_7_95k:
	   return "CODER_TYPE_AMRNB_7_95k";
	case CODER_TYPE_AMRNB_7_95k_NATIVE:
	   return "CODER_TYPE_AMRNB_7_95k_NATIVE";
	case CODER_TYPE_AMRNB_10_2k:
	   return "CODER_TYPE_AMRNB_10_2k";
	case CODER_TYPE_AMRNB_10_2k_NATIVE:
	   return "CODER_TYPE_AMRNB_10_2k_NATIVE";
	case CODER_TYPE_AMRNB_12_2k:
	   return "CODER_TYPE_AMRNB_12_2k";
	case CODER_TYPE_AMRNB_12_2k_NATIVE:
	   return "CODER_TYPE_AMRNB_12_2k_NATIVE";
	case CODER_TYPE_AMRNB_NONE:
	   return "CODER_TYPE_AMRNB_NONE";
	case CODER_TYPE_EVRC:
	   return "CODER_TYPE_EVRC";
	case CODER_TYPE_EVRC_NATIVE:
	   return "CODER_TYPE_EVRC_NATIVE";
//	case CODER_TYPE_EVRC_1_2k:
//	   return "CODER_TYPE_EVRC_1_2k";
//	case CODER_TYPE_EVRC_1_2k_NATIVE:
//	   return "CODER_TYPE_EVRC_1_2k_NATIVE";
//	case CODER_TYPE_EVRC_4_8k:
//	   return "CODER_TYPE_EVRC_4_8k";
//	case CODER_TYPE_EVRC_4_8k_NATIVE:
//	   return "CODER_TYPE_EVRC_4_8k_NATIVE";
//	case CODER_TYPE_EVRC_9_6k:
//	   return "CODER_TYPE_EVRC_9_6k";
//	case CODER_TYPE_EVRC_9_6k_NATIVE:
//	   return "CODER_TYPE_EVRC_9_6k_NATIVE";
	case CODER_TYPE_G729ANNEXE:
	   return "CODER_TYPE_G729ANNEXE";
	case CODER_TYPE_G729ANNEXE_NATIVE:
	   return "CODER_TYPE_G729ANNEXE_NATIVE";
	case CODER_TYPE_AMRWB_6_6K:
	   return "CODER_TYPE_AMRWB_6_6K";
	case CODER_TYPE_AMRWB_6_6K_NATIVE:
	   return "CODER_TYPE_AMRWB_6_6K_NATIVE";
	case CODER_TYPE_AMRWB_8_85K:
	   return "CODER_TYPE_AMRWB_8_85K";
	case CODER_TYPE_AMRWB_8_85K_NATIVE:
	   return "CODER_TYPE_AMRWB_8_85K_NATIVE";
	case CODER_TYPE_AMRWB_12_65K:
	   return "CODER_TYPE_AMRWB_12_65K";
	case CODER_TYPE_AMRWB_12_65K_NATIVE:
	   return "CODER_TYPE_AMRWB_12_65K_NATIVE";
	case CODER_TYPE_AMRWB_14_25K:
	   return "CODER_TYPE_AMRWB_14_25K";
	case CODER_TYPE_AMRWB_14_25K_NATIVE:
	   return "CODER_TYPE_AMRWB_14_25K_NATIVE";
	case CODER_TYPE_AMRWB_15_85K:
	   return "CODER_TYPE_AMRWB_15_85K";
	case CODER_TYPE_AMRWB_15_85K_NATIVE:
	   return "CODER_TYPE_AMRWB_15_85K_NATIVE";
	case CODER_TYPE_AMRWB_18_25K:
	   return "CODER_TYPE_AMRWB_18_25K";
	case CODER_TYPE_AMRWB_18_25K_NATIVE:
	   return "CODER_TYPE_AMRWB_18_25K_NATIVE";
	case CODER_TYPE_AMRWB_19_85K:
	   return "CODER_TYPE_AMRWB_19_85K";
	case CODER_TYPE_AMRWB_19_85K_NATIVE:
	   return "CODER_TYPE_AMRWB_19_85K_NATIVE";
	case CODER_TYPE_AMRWB_23_05K:
	   return "CODER_TYPE_AMRWB_23_05K";
	case CODER_TYPE_AMRWB_23_05K_NATIVE:
	   return "CODER_TYPE_AMRWB_23_05K_NATIVE";
	case CODER_TYPE_AMRWB_23_85K:
	   return "CODER_TYPE_AMRWB_23_85K";
	case CODER_TYPE_AMRWB_23_85K_NATIVE:
	   return "CODER_TYPE_AMRWB_23_85K_NATIVE";
	case CODER_TYPE_AMRWB_NONE:
	   return "CODER_TYPE_AMRWB_NONE";

	case CODER_TYPE_ILBC_13_33K:
	   return "CODER_TYPE_ILBC_13_33K";
	case CODER_TYPE_ILBC_13_33K_NATIVE:
	   return "CODER_TYPE_ILBC_13_33K_NATIVE";
	case CODER_TYPE_ILBC_15_2K:
	   return "CODER_TYPE_ILBC_15_2K";
	case CODER_TYPE_ILBC_15_2K_NATIVE:
	   return "CODER_TYPE_ILBC_15_2K_NATIVE";
	case CODER_TYPE_QCELP_8K:
      return "CODER_TYPE_QCELP_8K";
	case CODER_TYPE_QCELP_8K_NATIVE:
	   return "CODER_TYPE_QCELP_8K_NATIVE";
	case CODER_TYPE_QCELP_13K:
	   return "CODER_TYPE_QCELP_13K";
	case CODER_TYPE_QCELP_13K_NATIVE:
	   return "CODER_TYPE_QCELP_13K_NATIVE";
	case CODER_TYPE_SMV:
	   return "CODER_TYPE_SMV";
	case CODER_TYPE_SMV_NATIVE:
	   return "CODER_TYPE_SMV_NATIVE";
	case CODER_TYPE_H263:
	   return "CODER_TYPE_H263";
	case CODER_TYPE_H263_1998:
	   return "CODER_TYPE_H263_1998";
	case CODER_TYPE_MP4V_ES:
	   return "CODER_TYPE_MP4V_ES";
	case CODER_TYPE_H264:
	   return "CODER_TYPE_H264";
   default:
      return "unknown coder type";
   }
}

//*****************************************************************************
// Function: const char* AMediaCoder::GetDialogicResourceTypeDesc(eDEV_RESOURCE_TYPE a_dialogicResourceType)
// Description: Return the description for a Dialogic resource type enum
// Return: const char* 
// Parameters: eDEV_RESOURCE_TYPE a_dialogicResourceType 
//*****************************************************************************
const char* AMediaCoder::GetDialogicResourceTypeDesc(eDEV_RESOURCE_TYPE a_dialogicResourceType)
{
   switch(a_dialogicResourceType)
   {
   case RESOURCE_TYPE_NONE:
      return "RESOURCE_TYPE_NONE";
      break;
   case RESOURCE_IPM_LBR:
      return "RESOURCE_IPM_LBR";
      break;
   case RESOURCE_IPM_ALL_AUDIO_CODERS:
      return "RESOURCE_IPM_ALL_AUDIO_CODERS";
      break;
   case RESOURCE_IPM_G711_40MS:
      return "RESOURCE_IPM_G711_40MS";
      break;
   case RESOURCE_IPM_G711_30MS:
      return "RESOURCE_IPM_G711_30MS";
      break;
   case RESOURCE_IPM_G711_20MS:
      return "RESOURCE_IPM_G711_20MS";
      break;
   case RESOURCE_IPM_G711_10MS:
      return "RESOURCE_IPM_G711_10MS";
      break;
   case RESOURCE_IPM_G723:
      return "RESOURCE_IPM_G723";
      break;
   case RESOURCE_IPM_G726:
      return "RESOURCE_IPM_G726";
      break;
   case RESOURCE_IPM_G729:
      return "RESOURCE_IPM_G729";
      break;
   case RESOURCE_IPM_AMR_NB:
      return "RESOURCE_IPM_AMR_NB";
      break;
   case RESOURCE_IPM_EVRC:
      return "RESOURCE_IPM_EVRC";
      break;
   case RESOURCE_IPM_GSM_EFR:
      return "RESOURCE_IPM_GSM_EFR";
      break;
   case RESOURCE_IPM_ILBC:
      return "RESOURCE_IPM_ILBC";
      break;
   case RESOURCE_IPM_QCELP:
      return "RESOURCE_IPM_QCELP";
      break;
   case RESOURCE_IPM_SMV:
      return "RESOURCE_IPM_SMV";
      break;
   case RESOURCE_IPM_AMR_WB:
      return "RESOURCE_IPM_AMR_WB";
      break;
   case RESOURCE_IPM_G729E:
      return "RESOURCE_IPM_G729E";
      break;
   case RESOURCE_MM_VIDEO_NATIVE:
      return "RESOURCE_MM_VIDEO_NATIVE";
      break;
   case RESOURCE_IPM_G722:
      return "RESOURCE_IPM_G722";
      break;
   default:
      return "unknown resource type";
      break;
   }
   return "";
}

#ifdef USE_H264
//*****************************************************************************
// Function: CMediaCoderH264::CMediaCoderH264(unsigned a_payloadType)
// Description: Initializing constructor
// Return:  
// Parameters: unsigned a_payloadType 
//*****************************************************************************
CMediaCoderH264::CMediaCoderH264( unsigned a_payloadType)
   :AVideoCoder("H264",a_payloadType)
   ,m_matchedLevelId(VIDEO_LEVEL_1_H264)
   ,m_matchedProfileId(VIDEO_PROFILE_BASELINE_H264)
   ,m_matchedPacketizationMode(H264_PACKETIZATION_MODE_DEFAULT)
{
   //printf("CMediaCoderH264::CMediaCoderH264(PT)\n");
   m_dialogicCoderType = CODER_TYPE_H264;
}

//*****************************************************************************
// Function: CMediaCoderH264::CMediaCoderH264(const IPM_VIDEO_CODER_INFO &l_videoCoderInfo)
// Description: Initializing construtor
// Return:  
// Parameters: const IPM_VIDEO_CODER_INFO &l_videoCoderInfo 
//*****************************************************************************
CMediaCoderH264::CMediaCoderH264(const IPM_VIDEO_CODER_INFO& l_videoCoderInfo)
   :AVideoCoder("H264",l_videoCoderInfo.unCoderPayloadType)
   ,m_matchedLevelId(VIDEO_LEVEL_1_H264)
   ,m_matchedProfileId(VIDEO_PROFILE_BASELINE_H264)
   ,m_matchedPacketizationMode(H264_PACKETIZATION_MODE_DEFAULT)
{
   //printf("CMediaCoderH264::CMediaCoderH264(IPM_INFO)\n");
   m_dialogicCoderType = CODER_TYPE_H264;
   
   AddProfileId(VIDEO_PROFILE_BASELINE_H264);
   
   AddLevelId(VIDEO_LEVEL_1_H264);
   AddLevelId(VIDEO_LEVEL_1_B_H264);
   AddLevelId(VIDEO_LEVEL_1_1_H264);
   AddLevelId(VIDEO_LEVEL_1_2_H264);
   AddLevelId(VIDEO_LEVEL_1_3_H264);
#ifdef ENABLE_VGA
   AddLevelId(VIDEO_LEVEL_2_0_H264);
   AddLevelId(VIDEO_LEVEL_2_1_H264);
   AddLevelId(VIDEO_LEVEL_2_2_H264);
   AddLevelId(VIDEO_LEVEL_3_0_H264);
   AddLevelId(VIDEO_LEVEL_3_1_H264);
#endif
   
   AddPacketizationMode(H264_PACKETIZATION_MODE_SINGLE_NAL);
   AddPacketizationMode(H264_PACKETIZATION_MODE_NON_INTERLEAVED);
}

//*****************************************************************************
// Function: bool CMediaCoderH264::AddLevelId(unsigned int a_LevelId)
// Description: Add a Level Id capability to the list
// Return: bool 
// Parameters: unsigned int a_LevelId 
//*****************************************************************************
bool CMediaCoderH264::AddLevelId(unsigned int a_LevelId)
{
   for(unsigned int i=0; i < m_LevelId.size(); i++)
   {
      if(a_LevelId == m_LevelId[i])
      {
         return false;
      }
   }
   m_LevelId.push_back(a_LevelId);
   return true;
}

//*****************************************************************************
// Function: bool CMediaCoderH264::IsLevelIdSupported(unsigned int a_LevelId)
// Description: Test is a level Id is supported by the coder
// Return: bool 
// Parameters: unsigned int a_LevelId 
//*****************************************************************************
bool CMediaCoderH264::IsLevelIdSupported(unsigned int a_LevelId)
{
   for(unsigned int i=0; i < m_LevelId.size(); i++)
   {
      if(m_LevelId[i] == a_LevelId)
      {
         return true;
      }
   }
   return false;
}

//*****************************************************************************
// Function: bool CMediaCoderH264::AddProfileId(unsigned int a_ProfileId)
// Description: Add a Level Id capability to the list
// Return: bool 
// Parameters: unsigned int a_ProfileId 
//*****************************************************************************
bool CMediaCoderH264::AddProfileId(unsigned int a_ProfileId)
{
   for(unsigned int i=0; i < m_ProfileId.size(); i++)
   {
      if(a_ProfileId == m_ProfileId[i])
      {
         return false;
      }
   }
   m_ProfileId.push_back(a_ProfileId);
   return true;
}

//*****************************************************************************
// Function: bool CMediaCoderH264::IsProfileIdSupported(unsigned int a_ProfileId)
// Description: Test is a level Id is supported by the coder
// Return: bool 
// Parameters: unsigned int a_ProfileId 
//*****************************************************************************
bool CMediaCoderH264::IsProfileIdSupported(unsigned int a_ProfileId)
{
   for(unsigned int i=0; i < m_ProfileId.size(); i++)
   {
      if(m_ProfileId[i] == a_ProfileId)
      {
         return true;
      }
   }
   return false;
}

//*****************************************************************************
// Function: bool CMediaCoderH264::AddPacketizationMode(unsigned int a_PackMode)
// Description: Add a Packetization Mode capability to the list
// Return: bool 
// Parameters: unsigned int a_PackMode
//*****************************************************************************
bool CMediaCoderH264::AddPacketizationMode(unsigned int a_PackMode)
{
   for(unsigned int i=0; i < m_PackMode.size(); i++)
   {
      if(a_PackMode == m_PackMode[i])
      {
         return false;
      }
   }
   m_PackMode.push_back(a_PackMode);
   return true;
}

//*****************************************************************************
// Function: bool CMediaCoderH264::IsPacketizationModeSupported(unsigned int a_PackMode)
// Description: Test is a Packetization Mode is supported by the coder
// Return: bool 
// Parameters: unsigned int a_PackMode 
//*****************************************************************************
bool CMediaCoderH264::IsPacketizationModeSupported(unsigned int a_PackMode)
{
   for(unsigned int i=0; i < m_PackMode.size(); i++)
   {
      if(m_PackMode[i] == a_PackMode)
      {
         return true;
      }
   }
   return false;
}

//*****************************************************************************
// Function: void CMediaCoderH264::ImportSdpFmtp(const std::string &a_attribute)
// Description: Import coder attributes from SDP
// Return: void 
// Parameters: const std::string &a_attribute 
//*****************************************************************************
void CMediaCoderH264::ImportSdpFmtp(const std::string& a_attribute)
{
   LogMsg("CMediaCoderH264::ImportSdpFmtp");
   unsigned int l_payloadType, l_tokenCount=0;
   char l_separators[] = "= ;";
   char *l_token;
   char l_subAttribute[COMMON_STRING_MAX_LENGTH];
   char l_subAttributeValue[MEDIA_ATTRIB_SIZE_MAX];
   char l_attribute[MEDIA_ATTRIB_SIZE_MAX];
   char* l_tokenizerBuffer = NULL;
   
   unsigned int l_profileLevelIndication = 0;
   unsigned int l_packetizationMode = -1;
   unsigned int l_maxbr = 0;
   unsigned int l_maxmbps = 0;

   size_t l_attributeSize = a_attribute.size();
   if(l_attributeSize >= MEDIA_ATTRIB_SIZE_MAX)
   {
      LogMsg("CMediaCoderH264::ImportSdpFmtp - Attribute.size()=" + AsString(a_attribute.size()) + " is greater than MEDIA_ATTRIB_SIZE_MAX=" + AsString(MEDIA_ATTRIB_SIZE_MAX));
      return;
   }

   strcpy(l_attribute, a_attribute.c_str());
   l_attribute[l_attributeSize] = '\0';

   memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
   memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);

   l_token = DLGC_STRTOK(l_attribute, l_separators, &l_tokenizerBuffer);
   while (l_token != NULL)
   {
      LogMsg("CMediaCoderH264::l_token=" + AsString(l_token));
      switch (l_tokenCount)
      {
      case 0:
         l_payloadType = atoi(l_token);
         if (l_payloadType != m_payloadType)
         {
            LogMsg("CMediaCoderH264::ImportSdpFmtp - Payload Number Mismatch");
            return;
         }
         break;

      case 1:
      case 3:
      case 5:
      case 7:
         strcpy(l_subAttribute, l_token);
         break;

      case 2:
      case 4:
      case 6:
      case 8:
         strcpy(l_subAttributeValue, l_token);
         if (!strcmp(l_subAttribute, "profile-level-id"))
         {
            sscanf(l_subAttributeValue,"%x",&l_profileLevelIndication);
            unsigned int h264Level=(l_profileLevelIndication & 0xFF);
            unsigned int h264Profile=((l_profileLevelIndication>>16) & 0xFF);
            unsigned int h264BConstraint=((l_profileLevelIndication>>12) & 0x01);
            LogMsg("CMediaCoderH264::ImportSdpFmtp:prof=" + AsString(h264Profile) + " level=" + AsString(h264Level));
 
            //Add Profile
            if(h264Profile >= VIDEO_PROFILE_BASELINE_H264)
               AddProfileId(VIDEO_PROFILE_BASELINE_H264);  //only one supported

            //Add Level ID
#ifdef ENABLE_VGA
            if(h264Level >= VIDEO_LEVEL_3_1_H264)
               AddLevelId(VIDEO_LEVEL_3_1_H264);
            if(h264Level == VIDEO_LEVEL_3_0_H264)
               AddLevelId(VIDEO_LEVEL_3_0_H264);
            if(h264Level == VIDEO_LEVEL_2_2_H264)
               AddLevelId(VIDEO_LEVEL_2_2_H264);
            if(h264Level == VIDEO_LEVEL_2_1_H264)
               AddLevelId(VIDEO_LEVEL_2_1_H264);
            if(h264Level == VIDEO_LEVEL_2_0_H264)
               AddLevelId(VIDEO_LEVEL_2_0_H264);
            if(h264Level == VIDEO_LEVEL_1_3_H264)
               AddLevelId(VIDEO_LEVEL_1_3_H264);
#else
            if(h264Level > VIDEO_LEVEL_1_3_H264)
               AddLevelId(VIDEO_LEVEL_1_3_H264);
#endif
            else if((h264Level == VIDEO_LEVEL_1_1_H264) && h264BConstraint)
               AddLevelId(VIDEO_LEVEL_1_B_H264);
            else   
               AddLevelId(h264Level);
         }
         else if(!strcmp(l_subAttribute, "packetization-mode"))
         {
            l_packetizationMode = atoi(l_subAttributeValue);
            m_matchedPacketizationMode  = static_cast<eH264_PACKETIZATION_MODE>(l_packetizationMode);
            LogMsg("\tPeer signals Packetization Mode = " + AsString(l_packetizationMode));
         }
         else if(!strcmp(l_subAttribute, "max-br"))
         {
            l_maxbr = atoi(l_subAttributeValue);
            LogMsg("\tPeer signal Max-br = " + AsString(l_maxbr));
         }
         else if(!strcmp(l_subAttribute, "max-mbps"))
         {
            l_maxmbps = atoi(l_subAttributeValue);
            LogMsg("\tPeer signal Max-mbps = " + AsString(l_maxmbps));
         }



         memset(l_subAttribute, 0, COMMON_STRING_MAX_LENGTH);
         memset(l_subAttributeValue, 0, MEDIA_ATTRIB_SIZE_MAX);
         break;

      default:
         break;
      }

      l_token = DLGC_STRTOK(NULL, l_separators, &l_tokenizerBuffer);
      l_tokenCount++;
   }

   //JM - add default values if no values contained in SDP
   if (l_profileLevelIndication==0){
      AddProfileId(VIDEO_PROFILE_BASELINE_H264);  //only one supported
      //LogMsg("\tPeer signaled no profile-level-id.  Set default to Level1.1 (QCIF/192k)");
      //AddLevelId(VIDEO_LEVEL_1_1_H264);
      LogMsg("\tPeer signaled no profile-level-id.  Set default to Level1.3 (CIF/384k)");
      AddLevelId(VIDEO_LEVEL_1_3_H264);
   }
      
   if (l_packetizationMode==-1){
      l_packetizationMode = 0;
      m_matchedPacketizationMode = static_cast<eH264_PACKETIZATION_MODE>(l_packetizationMode);
      LogMsg("\tSet default Packetization Mode = " + AsString(l_packetizationMode));
   }
     
}

//*****************************************************************************
// Function: void CMediaCoderH264::ListProperties()
// Description: List the coder properties
// Return: void 
// Parameters: none 
//*****************************************************************************
void CMediaCoderH264::ListProperties()
{
   
   LogMsg("\t" + AsString(GetDialogicCoderTypeDesc(m_dialogicCoderType)));
   LogMsg("\tm=" + AsString(m_mediaType.c_str()) + " " + AsString(m_rtpAddress.c_str()) + ":" + AsString(m_rtpPort));
   LogMsg("\tplt=" + AsString(m_payloadType) + " peerplt=" + AsString(m_peerPayloadType));
   LogMsg("\ta=rtpmap:" + AsString(m_payloadType) + " " + AsString(m_payloadCoderName.c_str()) + "/" + AsString(m_clockRate));

   if (m_matchedLevelId > 0)
   {
     // 0x42=Baseline(66), H264BASELINE_IOP_CONSTRAINT is a constant
     unsigned int profileLevelId = (0x42<<16) | (H264BASELINE_IOP_CONSTRAINT<<8) | 0x00;
     if (m_matchedLevelId)
     {
        switch(m_matchedLevelId)
        {
           case VIDEO_LEVEL_1_H264:
              profileLevelId = profileLevelId | 0x0a; //0x42xx0a
              break;
           case VIDEO_LEVEL_1_B_H264:
              profileLevelId = profileLevelId | 0x100b; //0x421x0b
              break;
           case VIDEO_LEVEL_1_1_H264:
              profileLevelId = profileLevelId | 0x0b; //0x42xx0b
              break;
           case VIDEO_LEVEL_1_2_H264:
              //profileLevelId = profileLevelId | 0x0c; //0x42xx0c
              //JM - MWC for iPad
              profileLevelId = profileLevelId | 0x0d; //0x42xx0d
              break;
           case VIDEO_LEVEL_1_3_H264:
              profileLevelId = profileLevelId | 0x0d; //0x42xx0d
              break;
#ifdef ENABLE_VGA
           case VIDEO_LEVEL_2_0_H264:
              profileLevelId = profileLevelId | 0x14; //0x42xx14
              break;
           case VIDEO_LEVEL_2_1_H264:
              //profileLevelId = profileLevelId | 0x15; //0x42xx15
              //JM - workaround. Mirial doesn't send 4CIF unless higher level response.
              profileLevelId = profileLevelId | 0x1f; //0x42xx1f
              break;
           case VIDEO_LEVEL_2_2_H264:
              profileLevelId = profileLevelId | 0x16; //0x42xx16
              break;
           case VIDEO_LEVEL_3_0_H264:
              profileLevelId = profileLevelId | 0x1e; //0x42xx1e
              break;
           case VIDEO_LEVEL_3_1_H264:
              profileLevelId = profileLevelId | 0x1f; //0x42xx1f
              break;
#endif
           default: 
              profileLevelId = profileLevelId | 0x0a;
              LogMsg("default: " + AsHexString(m_matchedLevelId) + " default profile-level-id=" + AsHexString(profileLevelId));
              break;

         }
      }
      LogMsg("\ta=fmtp:" + AsString(m_payloadType) + " profile-level-id=" + AsHexString(profileLevelId));
   }
}

//*****************************************************************************
// Function: bool CMediaCoderH264::MatchCoder(AMediaCoder *a_otherMediaCoder)
// Description: Attempt to match the specified coder and return the result.
// Return: bool 
// Parameters: AMediaCoder *a_otherMediaCoder 
//*****************************************************************************
bool CMediaCoderH264::MatchCoder(AMediaCoder* a_otherMediaCoder)
{
   //LogMsg("H264:MatchCoder");
   
   m_matchedProfileId = VIDEO_PROFILE_BASELINE_H264;
   m_matchedLevelId = VIDEO_LEVEL_1_H264;
   m_dialogicResourceType = RESOURCE_TYPE_NONE;

   m_matchedBitRate_Local = 0;
   m_matchedBitRate_Remote = 0;
   
   CMediaCoderH264* l_otherMediaCoder = dynamic_cast<CMediaCoderH264*>(a_otherMediaCoder);
   if (!l_otherMediaCoder)
   {
      return false;
   }

   if (strcmp(GetPayloadCoderName(), l_otherMediaCoder->GetPayloadCoderName()))
   {
      return false;
   }

   if (GetClockRate() !=  l_otherMediaCoder->GetClockRate())
   {
      return false;
   }

   std::vector<unsigned int>::iterator l_LevelId;
   for (l_LevelId=m_LevelId.begin();
        l_LevelId!=m_LevelId.end();
        l_LevelId++)
   {
      if (l_otherMediaCoder->IsLevelIdSupported(*l_LevelId))
      {
         m_matchedLevelId = *l_LevelId;
         break;
      }
   }

   std::vector<unsigned int>::iterator l_ProfileId;
   for (l_ProfileId=m_ProfileId.begin();
        l_ProfileId!=m_ProfileId.end();
        l_ProfileId++)
   {
      if (l_otherMediaCoder->IsProfileIdSupported(*l_ProfileId))
      {
         m_matchedProfileId = *l_ProfileId;
         break;
      }
   }

   m_matchedVideoProfile_Remote = static_cast<eVIDEO_PROFILE>(m_matchedProfileId);
   m_matchedVideoLevel_Remote = static_cast<eVIDEO_LEVEL>(m_matchedLevelId);
   m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_DEFAULT;
   m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_DEFAULT;
   m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_DEFAULT;
   m_matchedBitRate_Remote = VIDEO_BITRATE_DEFAULT;
   m_matchedPacketizationMode = static_cast<eH264_PACKETIZATION_MODE>(m_matchedPacketizationMode);
   
   // Set Decoder values to max Level
   // Local = Decoder= Rx Values
   if (m_matchedLevelId <= VIDEO_LEVEL_1_3_H264)
   { 
      m_matchedVideoProfile_Local = VIDEO_PROFILE_BASELINE_H264;
      m_matchedVideoLevel_Local = VIDEO_LEVEL_1_3_H264;
      m_matchedImageWidth_Local = VIDEO_IMAGE_WIDTH_352;   
      m_matchedImageHeight_Local = VIDEO_IMAGE_HEIGHT_288;
      m_matchedFramesPerSec_Local = VIDEO_FRAMESPERSEC_30;
      m_matchedBitRate_Local = VIDEO_BITRATE_384K;
   }
#ifdef ENABLE_VGA   
   else if (m_matchedLevelId >= VIDEO_LEVEL_2_0_H264)
   {
      // Set Decoder values to max VGA Level
      m_matchedFramesPerSec_Local = VIDEO_FRAMESPERSEC_30;
      m_matchedVideoProfile_Local = VIDEO_PROFILE_BASELINE_H264;
      m_matchedVideoLevel_Local = VIDEO_LEVEL_3_1_H264;
      m_matchedImageWidth_Local = VIDEO_IMAGE_WIDTH_VGA;  //VIDEO_IMAGE_WIDTH_640
      m_matchedImageHeight_Local = VIDEO_IMAGE_HEIGHT_VGA; //VIDEO_IMAGE_HEIGHT_480
      m_matchedBitRate_Local = VIDEO_BITRATE_768K;
   }
#endif
   switch(m_matchedLevelId)
   {
      case VIDEO_LEVEL_1_H264:
         m_matchedImageWidth_Remote  = VIDEO_IMAGE_WIDTH_176;
         m_matchedImageHeight_Remote  = VIDEO_IMAGE_HEIGHT_144;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_15;
         m_matchedBitRate_Remote = 64000;
         break;
      case VIDEO_LEVEL_1_B_H264:
         m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_176;
         m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_144;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_15;
         m_matchedBitRate_Remote = 128000;
         break;
      case VIDEO_LEVEL_1_1_H264:
         //JM - MWC (for iphone)
         m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_352;
         m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_288;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_15;
         m_matchedBitRate_Remote = 384000;
         //m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_176;
         //m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_144;
         //m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_15;
         //m_matchedBitRate_Remote = 192000;
         break;
      case VIDEO_LEVEL_1_2_H264:
         m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_352;
         m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_288;
         //JM - MWC (for ipad)
         //m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_15;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_30;
         m_matchedBitRate_Remote = 500000;
         break;
      case VIDEO_LEVEL_1_3_H264:
         m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_352;
         m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_288;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_30;
         m_matchedBitRate_Remote = 384000; //can be as high as 768k
         break;
#ifdef ENABLE_VGA
      case VIDEO_LEVEL_2_0_H264:
         m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_352;
         m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_288;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_30;
         m_matchedBitRate_Local = 384000; //can be as high as 2M
         break;
      case VIDEO_LEVEL_2_1_H264:
         m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_352;
         m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_288;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_30;
         m_matchedBitRate_Remote = 384000; //can be as high as 2M
         break;
      case VIDEO_LEVEL_2_2_H264:
         m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_VGA;
         m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_VGA;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_25; // Maybe less
         m_matchedBitRate_Remote =  VIDEO_BITRATE_768K; //can be as high as 4M
         break;
      case VIDEO_LEVEL_3_0_H264:
         m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_VGA;
         m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_VGA;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_25; // Maybe less
         m_matchedBitRate_Remote =  VIDEO_BITRATE_768K; //can be as high as 10M
         break;
      case VIDEO_LEVEL_3_1_H264:
         m_matchedImageWidth_Remote = VIDEO_IMAGE_WIDTH_VGA;
         m_matchedImageHeight_Remote = VIDEO_IMAGE_HEIGHT_VGA;
         m_matchedFramesPerSec_Remote = VIDEO_FRAMESPERSEC_30; // Maybe less
         m_matchedBitRate_Remote =  VIDEO_BITRATE_768K; //can be as high as 14M
         break;
#endif
   }
    
   //LogMsg("H264:MatchCoder_out");
   //LogMsg("m_matchedLevelId=" + AsString(m_matchedLevelId));
   //LogMsg("m_matchedVideoLevel_Local=" + AsString(m_matchedVideoLevel_Local));
   //LogMsg("m_matchedImageWidth_Local=" + AsString(m_matchedImageWidth_Local));
   //LogMsg("m_matchedImageHeight_Local=" + AsString(m_matchedImageHeight_Local));
   //LogMsg("m_matchedBitRate_Local=" + AsString(m_matchedBitRate_Local));
   //LogMsg("m_matchedVideoLevel_Remote=" + AsString(m_matchedVideoLevel_Remote));
   //LogMsg("m_matchedImageWidth_Remot=" + AsString(m_matchedImageWidth_Remote));
   //LogMsg("m_matchedImageHeight_Remote=" + AsString(m_matchedImageHeight_Remote));
   //LogMsg("m_matchedBitRate_Remote=" + AsString(m_matchedBitRate_Remote));
   
   return true;
}

//*****************************************************************************
// Function: bool CMediaCoderH264::UpdateMatchedCoder(unsigned int a_preferredBitRate)
// Description: Update the Matched Coder based on application preference 
// Return: bool 
// Parameters: unsigned int a_preferredBitRate
//*****************************************************************************
bool CMediaCoderH264::UpdateMatchedCoder(unsigned int a_preferredBitRate)
{ 
   //Only update Bitrate and Video Level if Remote endpoint did not send a maxBitRate
//   if(a_preferredBitRate <= VIDEO_BITRATE_768K)
   if(a_preferredBitRate <= 1000000)
   {  
      LogMsg("H264 UpdateMatchedCoder with bitrate =" + AsString(a_preferredBitRate));
      switch(m_matchedLevelId)
      {
         case VIDEO_LEVEL_1_H264:
            if(a_preferredBitRate <= VIDEO_BITRATE_64K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            else if(VIDEO_BITRATE_64K < a_preferredBitRate && a_preferredBitRate <= VIDEO_BITRATE_128K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
               LogMsg("Update H264 Level to Baseline Level 1B");
               m_matchedLevelId = VIDEO_LEVEL_1_B_H264;  
            } 
            else 
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
               LogMsg("Update H264 Level to Baseline Level 1.3");
               m_matchedLevelId = VIDEO_LEVEL_1_3_H264;  
            }
            break; 
         case VIDEO_LEVEL_1_B_H264:
            if(a_preferredBitRate <= VIDEO_BITRATE_128K )
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            else
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
               LogMsg("Update H264 Level to Baseline Level 1.3");
               m_matchedLevelId = VIDEO_LEVEL_1_3_H264;  
            }
            break;
         case VIDEO_LEVEL_1_1_H264:
            if(a_preferredBitRate <= VIDEO_BITRATE_192K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            else
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
               LogMsg("Update H264 Level to Baseline Level 1.3");
               m_matchedLevelId = VIDEO_LEVEL_1_3_H264;  
            }
            break;
         case VIDEO_LEVEL_1_2_H264:
            if(a_preferredBitRate <= VIDEO_BITRATE_384K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            else
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
               LogMsg("Update H264 Level to Baseline Level 1.3");
               m_matchedLevelId = VIDEO_LEVEL_1_3_H264;  
            }
            break;
         case VIDEO_LEVEL_1_3_H264:
            if(a_preferredBitRate <= VIDEO_BITRATE_768K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            break;
#ifdef ENABLE_VGA
         case VIDEO_LEVEL_2_0_H264:
            if(a_preferredBitRate <= VIDEO_BITRATE_768K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            break;
         case VIDEO_LEVEL_2_1_H264:
            if(a_preferredBitRate <= VIDEO_BITRATE_768K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            break;
         case VIDEO_LEVEL_2_2_H264:
            if(a_preferredBitRate <= VIDEO_BITRATE_768K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            break;
         case VIDEO_LEVEL_3_0_H264:
            if(a_preferredBitRate <= VIDEO_BITRATE_768K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            break;
         case VIDEO_LEVEL_3_1_H264:
            if(a_preferredBitRate <= VIDEO_BITRATE_768K)
            {
               m_matchedBitRate_Remote = a_preferredBitRate;
            }
            break;
#endif
         default:
            LogMsg("Warning: Unknown H264 Level Id=" + AsString(m_matchedLevelId));
            break;
       }
       return true;
    }
    else
    {
      LogMsg("Did NOT UpdateMatchedCoder (H264) with preferred bitrate =" + AsString(a_preferredBitRate));
      return false;
    }
    return true;   
}

//*****************************************************************************
// Function: void CMediaCoderH264::ExportSdpMediaDescription(sdpMediaDescription *a_mediaDescription, unsigned int a_rtpPort)
// Description: Export the SDP media description for the coder
// Return: void 
// Parameters: sdpMediaDescription *a_mediaDescription 
//             unsigned int a_rtpPort 
//*****************************************************************************
void CMediaCoderH264::ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort)
{
   AVideoCoder::ExportSdpMediaDescription(a_mediaDescription,a_rtpPort);
   // 0x42=Baseline(66), H264BASELINE_IOP_CONSTRAINT is a constant
   unsigned int profileLevelId = (0x42<<16) | (H264BASELINE_IOP_CONSTRAINT<<8) | 0x00;
   if (m_matchedLevelId)
   {
      switch(m_matchedLevelId)
      {
         case VIDEO_LEVEL_1_H264:
            profileLevelId = profileLevelId | 0x0a; //0x42xx0a
            break;
         case VIDEO_LEVEL_1_B_H264:
            profileLevelId = profileLevelId | 0x100b; //0x421x0b
            break;
         case VIDEO_LEVEL_1_1_H264:
            profileLevelId = profileLevelId | 0x0b; //0x42xx0b
            break;
         case VIDEO_LEVEL_1_2_H264:
            //profileLevelId = profileLevelId | 0x0c; //0x42xx0c
            //JM - MWC for iPad
            profileLevelId = profileLevelId | 0x0d; //0x42xx0d
            break;
         case VIDEO_LEVEL_1_3_H264:
            profileLevelId = profileLevelId | 0x0d; //0x42xx0d
            break;
#ifdef ENABLE_VGA
         case VIDEO_LEVEL_2_0_H264:
            profileLevelId = profileLevelId | 0x14; //0x42xx14
            break;
         case VIDEO_LEVEL_2_1_H264:
            //profileLevelId = profileLevelId | 0x15; //0x42xx15
            //JM - workaround. Mirial doesn't send 4CIF unless higher level response.
            profileLevelId = profileLevelId | 0x1f; //0x42xx1f
            break;
         case VIDEO_LEVEL_2_2_H264:
            profileLevelId = profileLevelId | 0x16; //0x42xx16
            break;
         case VIDEO_LEVEL_3_0_H264:
            profileLevelId = profileLevelId | 0x1e; //0x42xx1e
            break;
         case VIDEO_LEVEL_3_1_H264:
            profileLevelId = profileLevelId | 0x1f; //0x42xx1f
            break;
         default:
            profileLevelId = profileLevelId | 0x0c; //0x42xx0c
            break;
#endif
      }
      
      sdpAttribute* l_attribute = a_mediaDescription->attributeList()->addItem();
      l_attribute->setProperty("fmtp");
      char l_buffer[COMMON_STRING_MAX_LENGTH * 3] = {0};
      //JM - TODO: add max-br 
      //sprintf(l_buffer,"%d profile-level-id=%X; packetization-mode=%d; max-br=%d",m_payloadType,profileLevelId,m_matchedPacketizationMode);
      sprintf(l_buffer,"%d profile-level-id=%X; packetization-mode=%d",m_payloadType,profileLevelId,m_matchedPacketizationMode);
      l_attribute ->setPropertyValue(l_buffer);
   }
}
#endif


#pragma warning( pop )


