/**
* @file MediaCoders.h
* @brief Media Coders - class declarations
* @date Mar. 25, 2008
*
* DIALOGIC CONFIDENTIAL
* Copyright  2008 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef MediaCoders_h
#define MediaCoders_h

#include <string>
#include <bitset>
#include <vector>

#include <devmgmt.h>
#include <sdpAPI.h>
#include "typedefs.h"

//*****************************************************************************
// Class: ::AMediaCoder
// Description: Abstract class for all audio and video coder classes.
//*****************************************************************************
class AMediaCoder
{
public:
   const static unsigned int COMMON_STRING_MAX_LENGTH = 64;
   const static unsigned int MEDIA_ATTRIB_SIZE_MAX = 256;
   virtual ~AMediaCoder() {}

   static const char* GetDialogicCoderTypeDesc(unsigned int a_coderType);
   static const char* GetDialogicResourceTypeDesc(eDEV_RESOURCE_TYPE a_dialogicResourceType);

   eIPM_CODER_TYPE GetDialogicCoderType() {return m_dialogicCoderType;}
   const char* GetPayloadCoderName() {return m_payloadCoderName.c_str();}
   const unsigned int GetPayloadType() {return m_payloadType;}
   //const unsigned int GetRedundencyPayloadType() {return m_payloadType;} //bug - no audio for AMRWB and AMRNB 
   //const unsigned int GetRedundencyPayloadType() {return 127;}  //make 127 for now
   const unsigned int GetRedundencyPayloadType() {return 0;}  //make 0 for now
   const unsigned int GetPeerPayloadType() {return m_peerPayloadType;}
   const std::string& GetMediaType() {return m_mediaType;}
   unsigned int GetClockRate() {return m_clockRate;}
   eDEV_RESOURCE_TYPE GetDialogicResourceType() {return m_dialogicResourceType;}

   long GetRtpPort() {return m_rtpPort;}
   const std::string& GetRtpIpAddress() {return m_rtpAddress;}

   void SetRtpPortAndIpAddress(long& a_rtpPort, const std::string& a_rtpAddress);
   void SetMediaDirection(eIPM_DATA_DIRECTION& a_mediaDirection) {m_mediaDirection=a_mediaDirection;};
   virtual void ImportSdpFmtp(const std::string& a_attribute) = 0;
   virtual void ListProperties() = 0;
   virtual bool MatchCoder(AMediaCoder* a_otherMediaCoder) = 0;
   virtual void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort) = 0;

   virtual unsigned int GetExtendedInformation() {return 0;}

protected:
   AMediaCoder(   const std::string& a_mediaType, 
                  const char * a_payloadCoderName, 
                  unsigned int a_payloadType, 
                  unsigned int a_clockRate) 
      :m_dialogicCoderType((eIPM_CODER_TYPE)0)
      ,m_mediaType(a_mediaType)
      ,m_payloadCoderName(a_payloadCoderName)
      ,m_payloadType(a_payloadType)
      ,m_peerPayloadType(a_payloadType)
      ,m_rtpPort(0)
      ,m_rtpAddress("")
      ,m_mediaDirection(DATA_IP_DIR_NONE)
      ,m_clockRate(a_clockRate)
      ,m_dialogicResourceType(RESOURCE_TYPE_NONE) {}
   
	eIPM_CODER_TYPE m_dialogicCoderType;
   std::string m_mediaType;
   std::string m_payloadCoderName;
  	unsigned int m_payloadType;
  	unsigned int m_peerPayloadType;
   long m_rtpPort;
   std::string m_rtpAddress;
   eIPM_DATA_DIRECTION m_mediaDirection;
   unsigned int m_clockRate;
   eDEV_RESOURCE_TYPE m_dialogicResourceType;

private:
   AMediaCoder();
};

//*****************************************************************************
// Class: ::AAudioCoder
// Description: Abstract class for all audio coder classes
//*****************************************************************************
class AAudioCoder: public AMediaCoder
{
public: 
   virtual ~AAudioCoder() {}

   unsigned int GetChannel() {return m_channel;}
   eIPM_CODER_VAD GetMatchedCoderVAD() {return m_matchedVAD;}     // valid after MatchCoder()
   unsigned int GetMatchedCoderPtime() {return m_matchedPtime;}   // valid after MatchCoder()
   virtual unsigned int GetMatchedCoderFramesPerPacket() = 0;     // valid after MatchCoder()
   virtual eIPM_CODER_FRAMESIZE GetMatchedCoderFrameSize() = 0;   // valid after MatchCoder()
   virtual unsigned int GetDefaultPtime() = 0;

   bool HasPtimeCapability(const unsigned int a_ptimeCapability);
   bool HasVadCapability(const eIPM_CODER_VAD a_vadCapability);

   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);

protected:
   AAudioCoder(const char* a_payloadCoderName, unsigned int a_payloadType) 
      :AMediaCoder("audio",a_payloadCoderName,a_payloadType, strcmp(a_payloadCoderName, "AMR-WB")?8000:16000)
      ,m_channel(1)
      ,m_matchedVAD(CODER_VAD_DISABLE)
      ,m_matchedPtime(0) {}

   AAudioCoder(const char* a_payloadCoderName, unsigned int a_payloadType, const IPM_AUDIO_CODER_INFO& l_audioCoderInfo);

   unsigned int m_channel;
   eIPM_CODER_VAD m_matchedVAD;                 // valid after MatchCoder()
   unsigned int m_matchedPtime;                 // valid after MatchCoder()

   std::vector<unsigned int> m_framesPerPacketCapabilities;
   std::vector<unsigned int> m_ptimeCapabilities;
   std::vector<eIPM_CODER_VAD> m_vadCapabilities;
   std::vector<eIPM_CODER_FRAMESIZE> m_frameSizeCapabilities;

   virtual bool AddVadCapability(const eIPM_CODER_VAD a_vadCapability);
   virtual bool DeleteVadCapability(const eIPM_CODER_VAD a_vadCapability);
   virtual bool AddPtimeCapability(const unsigned int a_pTimeCapability);
   virtual bool AddFrameSizeCapability(const eIPM_CODER_FRAMESIZE a_frameSizeCapability);
   virtual bool AddFramesPerPacketCapability(const unsigned int a_framesPerPacketCapability);

private:
   AAudioCoder();
};

//*****************************************************************************
// Class: ::CMediaCoderG711a
// Description: G.711A Coder
//*****************************************************************************
class CMediaCoderG711a: public AAudioCoder
{
public: 
   CMediaCoderG711a();
   CMediaCoderG711a(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo);
   virtual ~CMediaCoderG711a() {}
   unsigned int GetDefaultPtime() {return 20;}
   unsigned int GetMatchedCoderFramesPerPacket() {return G711A_FPP;} // valid after MatchCoder()
   eIPM_CODER_FRAMESIZE GetMatchedCoderFrameSize() {return static_cast<eIPM_CODER_FRAMESIZE>(m_matchedPtime / G711A_FPP);}

   void ImportSdpFmtp(const std::string& a_attribute);
   void ListProperties();
   bool MatchCoder(AMediaCoder* a_otherMediaCoder);
   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);
private:
   static const unsigned int G711A_FPP = 1;
};

//*****************************************************************************
// Class: ::CMediaCoderG711u
// Description: G711U Coder
//*****************************************************************************
class CMediaCoderG711u: public AAudioCoder
{
public: 
   CMediaCoderG711u();
   CMediaCoderG711u(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo);
   virtual ~CMediaCoderG711u() {}
   unsigned int GetDefaultPtime() {return 20;}
   unsigned int GetMatchedCoderFramesPerPacket() {return G711U_FPP;} // valid after MatchCoder()
   eIPM_CODER_FRAMESIZE GetMatchedCoderFrameSize() {return static_cast<eIPM_CODER_FRAMESIZE>(m_matchedPtime / G711U_FPP);}

   void ImportSdpFmtp(const std::string& a_attribute);
   void ListProperties();
   bool MatchCoder(AMediaCoder* a_otherMediaCoder);
   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);
private:
   static const unsigned int G711U_FPP = 1;
};

//*****************************************************************************
// Class: ::CMediaCoderG723
// Description: G.723 Coder
//*****************************************************************************
class CMediaCoderG723: public AAudioCoder
{
public: 
   CMediaCoderG723();
   CMediaCoderG723(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo);
   virtual ~CMediaCoderG723() {}
   unsigned int GetDefaultPtime() {return 30;}
   unsigned int GetMatchedCoderFramesPerPacket() {return (m_matchedPtime/G723_FSIZE);} // valid after MatchCoder()
   eIPM_CODER_FRAMESIZE GetMatchedCoderFrameSize() {return G723_FSIZE;}                // valid after MatchCoder()
   bool EnableVAD() {return AddVadCapability(CODER_VAD_ENABLE);}

   void ImportSdpFmtp(const std::string& a_attribute);
   void ListProperties();
   bool MatchCoder(AMediaCoder* a_otherMediaCoder);
   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);
protected:

private:
   bool AddBitRateCapability(const double& a_bitRate);

   static const eIPM_CODER_FRAMESIZE G723_FSIZE = CODER_FRAMESIZE_30;
   static double  G723_ANY;      // changeable bitrate, default
   static double  G723_5_3;      // 5.3 kbps
   static double  G723_6_3;      // 6.3 kbps

   std::vector<double> m_bitRateCapabilities;   // supported bit rates
};

//*****************************************************************************
// Class: ::CMediaCoderG729
// Description: G.729 Coder
//*****************************************************************************
class CMediaCoderG729: public AAudioCoder
{
public: 
   CMediaCoderG729();
   CMediaCoderG729(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo);
   virtual ~CMediaCoderG729() {}
   unsigned int GetDefaultPtime() {return 10;}
   unsigned int GetMatchedCoderFramesPerPacket() {return (m_matchedPtime/G729_FSIZE);} // valid after MatchCoder()
   eIPM_CODER_FRAMESIZE GetMatchedCoderFrameSize() {return G729_FSIZE;}                // valid after MatchCoder()
   bool EnableVAD()
   {
      m_dialogicCoderType = CODER_TYPE_G729ANNEXAWANNEXB; 
      return AddVadCapability(CODER_VAD_ENABLE);
   }

   void ImportSdpFmtp(const std::string& a_attribute);
   void ListProperties();
   bool MatchCoder(AMediaCoder* a_otherMediaCoder);
   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);
private:
   static const eIPM_CODER_FRAMESIZE G729_FSIZE = CODER_FRAMESIZE_10;
};

//*****************************************************************************
// Class: ::CMediaCoderAmrNb
// Description: AMR NB Coder
//*****************************************************************************
class CMediaCoderAmrNb: public AAudioCoder
{
public: 
   CMediaCoderAmrNb(unsigned int a_dialogicCoderType);
   CMediaCoderAmrNb(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo);
   virtual ~CMediaCoderAmrNb() {}
   unsigned int GetDefaultPtime() {return 20;}
   unsigned int GetMatchedCoderFramesPerPacket() {return (m_matchedPtime/AMR_NB_FSIZE);}  // valid after MatchCoder()
   eIPM_CODER_FRAMESIZE GetMatchedCoderFrameSize() {return AMR_NB_FSIZE;}                 // valid after MatchCoder()
   bool IsBitSetModeSet(unsigned int a_bit) {return m_modeSet.test(a_bit);};
   void EnableAllModeSet();

   void ImportSdpFmtp(const std::string& a_attribute);
   void ListProperties();
   bool MatchCoder(AMediaCoder* a_otherMediaCoder);
   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);
   unsigned int GetExtendedInformation() {return CODER_OPT_AMR_CMR_TRACK | CODER_OPT_AMR_OCTET;}
private:
   CMediaCoderAmrNb();
   static const eIPM_CODER_FRAMESIZE AMR_NB_FSIZE = CODER_FRAMESIZE_20;
   std::bitset<8> m_modeSet;
   bool m_octetAlign;
};

//*****************************************************************************
// Class: ::CMediaCoderAmrWb
// Description: AMR WB Coder
//*****************************************************************************
class CMediaCoderAmrWb: public AAudioCoder
{
public: 
   CMediaCoderAmrWb(unsigned int a_dialogicCoderType);
   CMediaCoderAmrWb(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo);
   virtual ~CMediaCoderAmrWb() {}
   unsigned int GetDefaultPtime() {return 20;}
   unsigned int GetMatchedCoderFramesPerPacket() {return (m_matchedPtime/AMR_NB_FSIZE);}  // valid after MatchCoder()
   eIPM_CODER_FRAMESIZE GetMatchedCoderFrameSize() {return AMR_NB_FSIZE;}                 // valid after MatchCoder()
   bool IsBitSetModeSet(unsigned int a_bit) {return m_modeSet.test(a_bit);};
   void EnableAllModeSet();

   void ImportSdpFmtp(const std::string& a_attribute);
   void ListProperties();
   bool MatchCoder(AMediaCoder* a_otherMediaCoder);
   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);
   unsigned int GetExtendedInformation() {return CODER_OPT_AMR_CMR_TRACK | CODER_OPT_AMR_OCTET;}
private:
   CMediaCoderAmrWb();
   static const eIPM_CODER_FRAMESIZE AMR_NB_FSIZE = CODER_FRAMESIZE_20;
   std::bitset<9> m_modeSet;
   bool m_octetAlign;
};

//*****************************************************************************
// Class: ::CMediaCoderG722
// Description: G.722 Coder
//*****************************************************************************
class CMediaCoderG722: public AAudioCoder
{
public: 
   CMediaCoderG722();
   CMediaCoderG722(const IPM_AUDIO_CODER_INFO& l_audioCoderInfo);
   virtual ~CMediaCoderG722() {}
   unsigned int GetDefaultPtime() {return 20;}
   unsigned int GetMatchedCoderFramesPerPacket() {return G722_FPP;} // valid after MatchCoder()
   eIPM_CODER_FRAMESIZE GetMatchedCoderFrameSize() {return static_cast<eIPM_CODER_FRAMESIZE>(m_matchedPtime / G722_FPP);}

   void ImportSdpFmtp(const std::string& a_attribute);
   void ListProperties();
   bool MatchCoder(AMediaCoder* a_otherMediaCoder);
   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);
private:
   static const unsigned int G722_FPP = 1;
};

//*****************************************************************************
// Class: ::CMediaCoderRfc2833
// Description: RFC-2833 DTMF 
//*****************************************************************************
class CMediaCoderRfc2833: public AAudioCoder
{
public: 
   CMediaCoderRfc2833( unsigned a_payloadType) :AAudioCoder("telephone-event",a_payloadType) {}
   virtual ~CMediaCoderRfc2833() {}
   unsigned int GetDefaultPtime() {return 0;}
   unsigned int GetMatchedCoderFramesPerPacket() {return 0;}      // valid after MatchCoder()
   eIPM_CODER_FRAMESIZE GetMatchedCoderFrameSize() {return static_cast<eIPM_CODER_FRAMESIZE>(0);}

   void ImportSdpFmtp(const std::string& a_attribute){;} //NOP
   void ListProperties();
   bool MatchCoder(AMediaCoder* a_otherMediaCoder);
   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);
private:
   CMediaCoderRfc2833();
   #define REQ_DTMF_EVENTS    "0-15"
};

//*****************************************************************************
// Class: ::AVideoCoder
// Description: Abstract class for all video coder classes
//*****************************************************************************
class AVideoCoder: public AMediaCoder
{
public: 
   virtual ~AVideoCoder() {}

   eVIDEO_PROFILE GetMatchedCoderRemoteVideoProfile() {return m_matchedVideoProfile_Remote;}        // valid after MatchCoder()
   eVIDEO_LEVEL GetMatchedCoderRemoteVideoLevel() {return m_matchedVideoLevel_Remote;}              // valid after MatchCoder()
   eVIDEO_IMAGE_WIDTH GetMatchedCoderRemoteImageWidth() {return m_matchedImageWidth_Remote;}        // valid after MatchCoder()
   eVIDEO_IMAGE_HEIGHT GetMatchedCoderRemoteImageHeight() {return m_matchedImageHeight_Remote;}     // valid after MatchCoder()
   eVIDEO_FRAMESPERSEC GetMatchedCoderRemoteFramesPerSecond() {return m_matchedFramesPerSec_Remote;}// valid after MatchCoder()
   unsigned int GetMatchedCoderRemoteBitRate() {return m_matchedBitRate_Remote;}                    // valid after MatchCoder()
   
   eVIDEO_PROFILE GetMatchedCoderLocalVideoProfile() {return m_matchedVideoProfile_Local;}        // valid after MatchCoder()
   eVIDEO_LEVEL GetMatchedCoderLocalVideoLevel() {return m_matchedVideoLevel_Local;}              // valid after MatchCoder()
   eVIDEO_IMAGE_WIDTH GetMatchedCoderLocalImageWidth() {return m_matchedImageWidth_Local;}        // valid after MatchCoder()
   eVIDEO_IMAGE_HEIGHT GetMatchedCoderLocalImageHeight() {return m_matchedImageHeight_Local;}     // valid after MatchCoder()
   eVIDEO_FRAMESPERSEC GetMatchedCoderLocalFramesPerSecond() {return m_matchedFramesPerSec_Local;}// valid after MatchCoder()
   unsigned int GetMatchedCoderLocalBitRate() {return m_matchedBitRate_Local;}                    // valid after MatchCoder()

#ifdef USE_H264
   virtual eH264_PACKETIZATION_MODE GetMatchedPacketizationMode() = 0; //
#endif

   virtual bool UpdateMatchedCoder(unsigned int a_preferredBitRate) = 0;  // valid after MatchCoder()

   virtual void ImportSdpFmtp(const std::string& a_attribute) = 0;
   virtual bool MatchCoder(AMediaCoder* a_otherMediaCoder) = 0;
   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);
protected:
   AVideoCoder(const char* a_payloadCoderName, unsigned int a_payloadType) 
      :AMediaCoder("video",a_payloadCoderName,a_payloadType, 90000)
   {}

   // valid after MatchCoder()
   // Remote = Decoder settings 
   eVIDEO_PROFILE m_matchedVideoProfile_Remote;     // Profile ID for H263, Profile and Level ID for MPEG4
   eVIDEO_LEVEL m_matchedVideoLevel_Remote;         // Ignored for MPEG4
   eVIDEO_IMAGE_WIDTH m_matchedImageWidth_Remote;   // Image width
   eVIDEO_IMAGE_HEIGHT m_matchedImageHeight_Remote; // Image height
   eVIDEO_FRAMESPERSEC m_matchedFramesPerSec_Remote;// Frames per second
   unsigned int m_matchedBitRate_Remote;
  
   // valid after MatchCoder()
   // Local = Encoder settings 
   eVIDEO_PROFILE m_matchedVideoProfile_Local;     // Profile ID for H263, Profile and Level ID for MPEG4
   eVIDEO_LEVEL m_matchedVideoLevel_Local;         // Ignored for MPEG4
   eVIDEO_IMAGE_WIDTH m_matchedImageWidth_Local;   // Image width
   eVIDEO_IMAGE_HEIGHT m_matchedImageHeight_Local; // Image height
   eVIDEO_FRAMESPERSEC m_matchedFramesPerSec_Local;// Frames per second
   unsigned int m_matchedBitRate_Local;

private:
   AVideoCoder();
};

//*****************************************************************************
// Class: ::CPictureSizeAndMpi
// Description: Encapsulate picture size and minimum picture interval 
// 				information for H.263 coders
//*****************************************************************************
class CPictureSizeAndMpi
{
public:
   CPictureSizeAndMpi(const char* a_pictureSize, const unsigned int a_minimumPictureInterval)
      :m_pictureSize(a_pictureSize)
      ,m_minimumPictureInterval(a_minimumPictureInterval) {}
   virtual ~CPictureSizeAndMpi() {}
   const std::string& PictureSize() {return m_pictureSize;}
   const int MinimumPictureInterval() {return m_minimumPictureInterval;}
private:
   CPictureSizeAndMpi();  // force initialization
   std::string m_pictureSize;
   unsigned int m_minimumPictureInterval;
};

//*****************************************************************************
// Class: ::CMediaCoderH263
// Description: H.263 video coder
//*****************************************************************************
class CMediaCoderH263: public AVideoCoder
{
public: 
   CMediaCoderH263();
   CMediaCoderH263(const IPM_VIDEO_CODER_INFO& l_videoCoderInfo);
   virtual ~CMediaCoderH263() {}

   CPictureSizeAndMpi* MatchedPictureSizeAndMpiCapability() {return m_matchedPictureSizeAndMpiCapability;}

#ifdef USE_H264
   eH264_PACKETIZATION_MODE GetMatchedPacketizationMode() {return H264_PACKETIZATION_MODE_NON_INTERLEAVED;}// default
#endif

   bool IsPictureSizeAndMpiSupported(CPictureSizeAndMpi* a_pictureAndSizeCapability);
   void ImportSdpFmtp(const std::string& a_attribute);
   void ListProperties();
   bool MatchCoder(AMediaCoder* a_otherMediaCoder);
   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);
   
   bool UpdateMatchedCoder(unsigned int a_preferredBitRate); 
private:
   std::vector<CPictureSizeAndMpi*> m_pictureSizeAndMpiCapabilities;
   CPictureSizeAndMpi* m_matchedPictureSizeAndMpiCapability;
   unsigned int m_maxBitRate;

   bool AddPictureSizeAndMpiCapability(CPictureSizeAndMpi* a_pictureAndSizeCapability);
};

#ifdef USE_H264
//*****************************************************************************
// Class: ::CMediaCoderH264
// Description: H.264 video coder
//*****************************************************************************
class CMediaCoderH264: public AVideoCoder
{
public: 
   // 0x00
   const static unsigned char H264BASELINE_IOP_CONSTRAINT = 0xFF & (H264_PROFILE_IOP_NONE);
   // 0xE0
   //const static unsigned char H264BASELINE_IOP_CONSTRAINT = 0xFF & (H264_PROFILE_IOP_CONTRAINT_SET0 | H264_PROFILE_IOP_CONTRAINT_SET1 | H264_PROFILE_IOP_CONTRAINT_SET2);
   
   CMediaCoderH264( unsigned a_payloadType);
   CMediaCoderH264(const IPM_VIDEO_CODER_INFO& l_videoCoderInfo);
   virtual ~CMediaCoderH264() {}
   
   eH264_PACKETIZATION_MODE GetMatchedPacketizationMode() {return m_matchedPacketizationMode;}// valid after MatchCoder()
   
   unsigned int MatchedLevelId() {return m_matchedLevelId;}
   bool AddLevelId(unsigned int a_LevelId);
   bool IsLevelIdSupported(unsigned int a_LevelId);
   unsigned int MatchedProfileId() {return m_matchedProfileId;}
   bool AddProfileId(unsigned int a_ProfileId);
   bool IsProfileIdSupported(unsigned int a_ProfileId);
   bool AddPacketizationMode(unsigned int a_PackMode);
   bool IsPacketizationModeSupported(unsigned int a_PackMode);

   void ImportSdpFmtp(const std::string& a_attribute);
   void ListProperties();
   bool MatchCoder(AMediaCoder* a_otherMediaCoder);
   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);

   bool UpdateMatchedCoder(unsigned int a_preferredBitRate); 
private:
   CMediaCoderH264();
   std::vector<unsigned int> m_LevelId;
   std::vector<unsigned int> m_ProfileId;
   std::vector<unsigned int> m_PackMode;
   unsigned int m_matchedLevelId;
   unsigned int m_matchedProfileId;
   eH264_PACKETIZATION_MODE m_matchedPacketizationMode;
};
#endif

//*****************************************************************************
// Class: ::CMediaCoderMp4v
// Description: MEG4 video coder
//*****************************************************************************
class CMediaCoderMp4v: public AVideoCoder
{
public: 
   CMediaCoderMp4v( unsigned a_payloadType);
   CMediaCoderMp4v(const IPM_VIDEO_CODER_INFO& l_videoCoderInfo);
   virtual ~CMediaCoderMp4v() {}

#ifdef USE_H264
   eH264_PACKETIZATION_MODE GetMatchedPacketizationMode() {return H264_PACKETIZATION_MODE_NON_INTERLEAVED;}// default
#endif
   
   unsigned int MatchedProfileLevelIndication() {return m_matchedProfileLevelIndication;}
   bool AddProfileLevelIndication(unsigned int a_profileLevelIndication);
   bool IsProfileLevelIndicationSupported(unsigned int a_profileLevelIndication);

   void ImportSdpFmtp(const std::string& a_attribute);
   void ListProperties();
   bool MatchCoder(AMediaCoder* a_otherMediaCoder);
   void ExportSdpMediaDescription(sdpMediaDescription* a_mediaDescription, unsigned int a_rtpPort);
   
   bool UpdateMatchedCoder(unsigned int a_preferredBitRate); 
private:
   CMediaCoderMp4v();
   std::vector<unsigned int> m_profileLevelIndications;
   unsigned int m_matchedProfileLevelIndication;
};

#endif

