/**
* @file VideoConfDemo.cpp
* @brief Video Conferencing Demo application - main
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
   #include <process.h>
#endif

#include <signal.h>

#include "CSystemConfiguration.h"
#include "CGcManager.h"
#include "LogUtil.h"
#include "globals.h"

void SignalHandler(const int signal);
static bool g_appShutdown(false);
static int m_ctrlcCount(0);
int kbhit(void);

//*****************************************************************************
// Function: ::EventHandler(void *a_context)
// Description: Event handler thread function
// Return: unsigned 
// Parameters: void *a_context 
//*****************************************************************************
#ifdef WIN32
unsigned int WINAPI EventHandler(void * a_context)
#else
void *EventHandler(void* a_context)
#endif
{
   CGcManager* l_gcManager = static_cast<CGcManager*>(a_context);
   while (g_applicationState != ApplicationStateShutDownComplete)
   {
      l_gcManager->ProcessEvents();
   }
   LogMsg("EventHandler exiting");
   return 0;
}

//*****************************************************************************
// Function: int ::main(int argc, char *argv[])
// Description: Video Conferencing demp application main function
// Return: int - 0 for success, otherwise error 
// Parameters: int argc 
//             char *argv[] 
//*****************************************************************************
int main(int argc, char* argv[])
{

#ifdef WIN32

   signal(SIGINT, (void (__cdecl*)(int)) SignalHandler);
   signal(SIGTERM, (void (__cdecl*)(int)) SignalHandler);

#else

   signal(SIGHUP,  SignalHandler);
   signal(SIGQUIT, SignalHandler);
   signal(SIGINT, SignalHandler);
   signal(SIGTERM, SignalHandler);

#endif

   g_logfile.open( "logfile.txt");
   if (!g_logfile.is_open())
   {
      exit(-1);
   }


   LogMsg("'Q',or 'q' to quit");

   g_applicationState = ApplicationStateInitializing;

   CSystemConfiguration l_systemConfiguration;
   if (l_systemConfiguration.Initialize(argc-1, &argv[1]) != 0)
   {
      return -1;
   }

   CGcManager l_gcManager(l_systemConfiguration);

   if (l_gcManager.Start() != 0)
   {
      return -1;
   }

 #ifdef WIN32
   // Create and spawn a worker thread for SRL.
   HANDLE hThread = INVALID_HANDLE_VALUE;
   hThread = ((HANDLE) _beginthreadex( (void *)        (NULL), 
                                       (unsigned)      (0), 
                                       EventHandler, 
                                       (void *)        &l_gcManager, 
                                       (unsigned)      (0), 
                                       (unsigned *)    (NULL)));

   if (hThread == INVALID_HANDLE_VALUE)
   {
	  LogMsg("Failed to Create WorkerThread, Exiting ...");	   
      return -1;
   }

#else

   // Creating worker thread.
   pthread_t hThread;
   pthread_create(&hThread, NULL, EventHandler, &l_gcManager);
#endif

   g_applicationState = ApplicationStateStarted;


   // Runtime options.
   char ch;
   while (g_applicationState == ApplicationStateStarted)
   {
      if (kbhit())
      {
         ch = getchar();
         switch (ch)
         {
            case 'q':
            case 'Q':
               LogMsg("Detected request to end application from console");
               g_applicationState = ApplicationStateShuttingDown;
               break;
				case 'p':
				case 'P':
					l_systemConfiguration.printConfIDandConfPIN();
					break;
         }
      }
      else
      {
#ifdef WIN32
		  Sleep(1000);
#else
         sleep(1);
#endif
      }
   }

   l_gcManager.Stop();

   g_applicationState = ApplicationStateShutDownComplete;

#ifdef WIN32
   WaitForSingleObject(hThread, INFINITE);
   CloseHandle(hThread);
#else
   pthread_join(hThread,NULL);
#endif

   g_logfile.close();

   return 0;
}

//*****************************************************************************
// Function: void ::SignalHandler(const int signal)
// Description: Handle Ctrl-C signals by setting flag to end application
// Return: void 
// Parameters: const int signal 
//*****************************************************************************
void SignalHandler(const int signal) 
{
   switch (signal)
   {
      case SIGINT:
         m_ctrlcCount++;
         if (m_ctrlcCount==1)
         {
            printf("Received signal SIGINT # 1 - application shutting down\n", signal);
            g_applicationState = ApplicationStateShuttingDown;
            ungetc('q', stdin);
            ungetc('\n', stdin);
         }
         else if (m_ctrlcCount == 2)
         {           
            printf("Received signal SIGINT # 2 - applicatipon already shutting down\n", signal);
         }
         else if (m_ctrlcCount >= 3)
         {           
            printf("Received signal SIGINT # 3 - killing application, may need to restart ipmedia\n", signal);
            exit(0);
         }
         break;
      case SIGTERM:
         g_applicationState = ApplicationStateShuttingDown;
         printf("Received signal SIGTERM\n", signal);
         break;
      default:
         printf("Received signal 0x%X\n", signal);
   }
}

//*****************************************************************************
// Function: int ::kbhit(void)
// Description: Function to test is there is a character in the keyboard buffer
// Return: int, 0 if keyboard buffer is empty, 1 if not
// Parameters: void 
//*****************************************************************************
int kbhit(void)
{
   struct timeval tv;
   fd_set read_fd;
   
   tv.tv_sec=0;
   tv.tv_usec=0;
   FD_ZERO(&read_fd);
   FD_SET(0,&read_fd);
   
   if(select(1, &read_fd, NULL, NULL, &tv) == -1)
   {
      return 0;
   }
   
   if(FD_ISSET(0,&read_fd))
   {
      return 1;
   }
   
   return 0;
}

